<?php
define( 'activemember360BUILD', '2020-12-01_11-45-10' );
/*
Plugin name: ActiveMember360
Plugin URI: https://activemember360.com
Author: Bob Keen and John Keen, ActiveMember360
Author URI: https://activemember360.com
Copyright: This software is copyright of responseFLOW, Inc., (c) 2016-2018
Description: WordPress membership plugin driven by ActiveCampaign application data, automations and events
Version: 3.2.9
Support: https://activemember360/support/
Text Domain : activemember360
*/
define( 'MBRVERSION',  '3.2.9' );
define( 'MBRNAME',     plugin_basename( __FILE__ ) );
define( 'MBRINCLLIB',  __DIR__ . '/activemember360_lib.php' );
define( 'MBRINCLCLASS',__DIR__ . '/class.activemember360.php' );

IF ( ! defined( 'ABSPATH' ) ) :
	wp_die( 'This file cannot be accessed directly.' );
ENDIF;

IF ( PHP_MAJOR_VERSION < 5 ) :
	return mbr_exit_with_error( 'php_version' );
ELSEIF ( PHP_MAJOR_VERSION == 5 AND PHP_MINOR_VERSION < 4 ) :
	return mbr_exit_with_error( 'php_version' );
ENDIF;

IF ( ! file_exists( MBRINCLLIB ) ) :
	return mbr_exit_with_error( 'lib_not_found' );
ELSEIF ( ( include_once MBRINCLLIB ) <> TRUE ) :
	return mbr_exit_with_error( 'unknown' );
ENDIF;

mbr_sys_load();

IF ( ! file_exists( MBRINCLCLASS ) ) :
	return mbr_exit_with_error( 'class_not_found' );
ELSEIF ( ( include_once MBRINCLCLASS ) <> TRUE ) :
	return mbr_exit_with_error( 'unknown' );
ENDIF;

/* Load ActiveMember360 */
activemember360();

function mbr_exit_with_error( $msg = 'unknown', $needed = '' )
{
	If ( ! is_admin() ) :
		return FALSE;
	ENDIF;

	IF ( $msg == 'php_version' ) :
		$msgs = array( sprintf ('You site is currently running with PHP version %s', PHP_VERSION ), 'This plugin requires PHP version 5.4, 5.5, 5.6 or 7.x.' );

	ELSEIF ( $msg == 'lib_not_found') :
		$msgs = array( sprintf('ActiveMember360 could not find the following required file:<br /><strong style="padding-left:10px">%s</strong>', plugin_basename( MBRINCLLIB ) ), 'Please reinstall the plugin if possible.' );

	ELSEIF ( $msg == 'main_not_found') :
		$msgs = array( sprintf('ActiveMember360 could not find the following required file:<br /><strong style="padding-left:10px">%s</strong>', plugin_basename( MBRINCLCLASS ) ), 'Please reinstall the plugin if possible.' );

	ELSEIF ( $msg == 'redux_not_found') :
		$msgs = array( 'Redux Framework not found.' );

	ELSEIF ( $msg == 'payments_extension_not_found' ) :
		$msgs = array( 'ActiveMember360 Payments Module not found.');

	ELSEIF ( $msg == 'unknown' ) :
		$msgs = array( 'ActiveMember360 could not be loaded due to an error.');

	ELSE :
	$msgs = array( $msg );
	ENDIF;


	$msg_txt = '<p style="font-size:1.1em">' . implode( '</p><p style="font-size:1.1em">', $msgs ) . '</p>';
	$msg_header = "<img src='https://d1toiz2vvvafqj.cloudfront.net/activemember360_logo_small.jpg' /><h1 style='color:darkblue;'>An error occurred while loading ActiveMember360</h1>";
	$return = "<p><a href='" . admin_url( 'plugins.php' ) . "' style='font-size:1.5em;font-weight: bold;'>Click here to go back</a><a href='https://activemember360.com/support/' target='_blank' style='font-size:1.5em;font-weight: bold;float:right;'>Contact support</a></p><p><center><a href='https://activemember360.com/docs/' target='_blank' style='font-size:1.5em;font-weight: bold;'>View the documentation</a></center></p>";

	wp_die($msg_header . "{$msg_txt}" . $return );
}
