<?php
final class mbr_api_settings EXTENDS mbr_redux_base_class
{
	use TRAIT__getInstance;

	public $opt_name  = 'mbr_api_settings';
	public $page_slug = 'activemember360';

	public function _load()
	{
		IF ( is_admin() ) :
			add_action( 'plugins_loaded',                                           [ $this, 'init_admin_panel' ],    99999 );
			add_action( 'plugins_loaded',                                           [ $this, 'init_sections' ],       99999 );
			add_filter( "redux/{$this->opt_name}/panel/template/header.tpl.php",    [ $this, 'alternate_header' ] );
			add_filter( "redux/{$this->opt_name}/panel/template/container.tpl.php", [ $this, 'alternate_container' ] );
			add_action( "redux/options/{$this->opt_name}/validate",                 [ $this, 'validate_save_settings' ], 10, 1 );
			add_action( 'wp_ajax_mbr_re_check_license',                             [ $this, 'ajax_re_check_license' ] );
			add_action( "redux/options/{$this->opt_name}/saved",                    [ $this, 'backup_settings' ],      9999 );
		ENDIF;
	}

	public function ajax_re_check_license()
	{
		apply_filters( 'mbr/license/check', NULL);
		exit;
	}

	public function alternate_container()
	{
		return MBR_DIR . '/templates/container_api_settings.tpl.php';
	}

	public function init_admin_panel()
	{
		global $MBR_REDUX_HEADER_MODULE_NAMES;

		$MBR_REDUX_HEADER_MODULE_NAMES[ $this->opt_name ] = $admin_title = _x( 'API Settings / License', 'wp_admin', 'activemember360' );

		$page_title         = apply_filters( 'mbr/plugin_name', '' ) . ' ' . $admin_title;

		$arr_config         =  [
			'opt_name'        => $this->opt_name,
			'global_variable' => $this->opt_name,
			'page_slug'       => $this->page_slug,
			'page_parent'     => MBR_MENU_PARENT_PAGE,
			'page_title'      => $page_title,
			'display_name'    => $page_title,
			'admin_title'     => $admin_title,
			'menu_title'      => $admin_title,
		];

		$arr_config         = array_merge( $this->arr_default_sub_admin_panel_settings, $arr_config );

		Redux::setArgs( $this->opt_name, $arr_config );
	}

	public function init_sections()
	{
		global $pagenow, $MBR_OPTIONS;

		IF ( $pagenow <> 'options.php' AND $pagenow <> 'admin.php' OR ( $pagenow == 'admin.php' AND ( empty($_GET['page'] ) OR $_GET['page'] <> $this->page_slug ) ) ) :
			return;
		ENDIF;

		$arr_section_api_settings =
		[
			'id'             => 'section_api_settings',
			'title'          => _x( 'API Settings', 'wp_admin', 'activemember360' ),
			'desc'           => _x( 'Connect ActiveMember360 with your ActiveCampaign application.', 'wp_admin', 'activemember360' ),
			'icon'           => 'el el-return-key',
			'class'          => 'mbr_api_section_api_settings'
		];

		$current_error = get_option( 'mbr_api_error_message', FALSE );
		IF ( $current_error ) :
			delete_option( 'mbr_api_error_message' );
			$arr_section_api_settings['fields'][] =
			[
				'id'          => 'mbr_api_info',
				'title'       => _x( 'Warning', 'wp_admin', 'activemember360' ),
				'desc'        => $current_error,
				'type'        => 'info',
				'style'       => 'critical'
			];
		ENDIF;

		$arr_section_api_settings['fields'][] =
		[
			'id'         => 'mbr_api_url',
			'title'      => _x( 'ActiveCampaign API URL', 'wp_admin', 'activemember360' ),
			'desc'       => _x( 'Changing the API URL will result in most settings being reset. Use with care!!!', 'wp_admin', 'activemember360' ),
			'type'       => 'text',
			'validate'   => 'not_empty'
		];
		$arr_section_api_settings['fields'][] =
		[
			'id'         => 'mbr_api_key',
			'title'      => _x( 'ActiveCampaign API KEY', 'wp_admin', 'activemember360' ),
			'desc'       => _x( 'Click inside the input field to toggle view of the API key.', 'wp_admin', 'activemember360' ),
			'type'       => 'password',
			'validate'   => 'not_empty'
		];
		$arr_section_api_settings['fields'][] =
		[
			'id'         => 'mbr_api_settings_notes',
			'title'      => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'full_width' => TRUE,
			'type'       => 'textarea'
		];

		$arr_license = apply_filters( 'mbr/license/status/get', NULL );
		$arr_section_license_status =
		[
			'id'         => 'section_license_status',
			'title'      => _x( 'License Status', 'wp_admin', 'activemember360' ),
			'icon'       => 'el el-check',
			'class'      => 'mbr_api_section_license_status',
		];
		$arr_section_license_status['fields'][] =
		[
			'id'         => 'mbr_license_status',
			'title'      => _x( 'License Status', 'wp_admin', 'activemember360' ),
			'type'       => 'raw',
			'full_width' => FALSE,
			'content'    => $arr_license['status']
		];
		$arr_section_license_status['fields'][] =
		[
			'id'         => 'mbr_license_type',
			'title'      => _x( 'License Type', 'wp_admin', 'activemember360' ),
			'type'       => 'raw',
			'full_width' => FALSE,
			'content'    => $arr_license['level']
		];
		$arr_section_license_status['fields'][] =
		[
			'id'         => 'mbr_re_check_license',
			'type'       => 'raw',
			'full_width' => TRUE,
			'content'    => '<div id="mbr_re_check_license_loading"></div><p><span id="mbr_re_check_license" class="button button-primary">' . _x( 'Check license', 'wp_admin', 'activemember360' ) . '</span></p>',
		];

		
		Redux::setSection( $this->opt_name, $arr_section_api_settings );
		Redux::setSection( $this->opt_name, $arr_section_license_status );
	}

	public function validate_save_settings( &$arr_settings )
	{
		global $MBR_OPTIONS, $wpdb;

		$arr_settings['mbr_api_url']   = trim( $arr_settings['mbr_api_url'] );
		$arr_settings['mbr_api_key']   = trim( $arr_settings['mbr_api_key'] );

		IF ( $arr_settings['mbr_api_url'] == $MBR_OPTIONS['api_url'] AND $arr_settings['mbr_api_key'] == $MBR_OPTIONS['api_key'] ) :
			update_option( 'mbr_api_error_message', _x( 'Your settings were not changed and were not saved.', 'wp_admin', 'activemember360' ) );

			return;
		ENDIF;

		$old_api_url                   = $MBR_OPTIONS['api_url'];
		$old_api_key                   = $MBR_OPTIONS['api_key'];

		$MBR_OPTIONS['api_url']        = $arr_settings['mbr_api_url'];
		$MBR_OPTIONS['api_key']        = $arr_settings['mbr_api_key'];

		$new_api_url                   = $MBR_OPTIONS['api_url'];
		$new_api_key                   = $MBR_OPTIONS['api_key'];

		$check_account                 = apply_filters( 'mbr/api_account/check', NULL );

		IF ( $check_account->result_code == 0 ) :
			
			$MBR_OPTIONS['api_url']      = $old_api_url;
			$MBR_OPTIONS['api_key']      = $old_api_key;
			$arr_settings['mbr_api_url'] = $old_api_url;
			$arr_settings['mbr_api_key'] = $old_api_key;
			update_option( 'mbr_api_error_message', _x( 'Your settings were not saved. The API settings entered could not be verified.', 'wp_admin', 'activemember360' ) );

			return;
		ENDIF;

		IF ( $old_api_url <> $MBR_OPTIONS['api_url'] ) :
			
			$MBR_OPTIONS['api_url']      = $old_api_url;
			$MBR_OPTIONS['api_key']      = $old_api_key;

			apply_filters( 'mbr/api/init', NULL );
			do_action( 'mbr/webhooks/delete/local/all' );

			
			$MBR_OPTIONS['api_url']      = $new_api_url;
			$MBR_OPTIONS['api_key']      = $new_api_key;

			
			delete_option('cron');

			
			$arrTABLES = $wpdb->get_results( "SHOW TABLES LIKE '" . $wpdb->prefix . "mbr%'", ARRAY_N );
			FOREACH ( $arrTABLES AS $k => $table ) :
				$wpdb->query( "TRUNCATE TABLE {$table[0]}" );
			ENDFOREACH;

			



			$license_status_key = apply_filters( 'mbr/options/key/license_status', NULL );
			IF ( ! empty( $license_status_key ) ) :
				delete_transient( $license_status_key );
			ENDIF;

			$license_status_md5_key = apply_filters( 'mbr/options/key/license_status_md5', NULL );
			IF ( ! empty( $license_status_md5_key ) ) :
				delete_transient( $license_status_md5_key );
			ENDIF;

			$ac_account_key = apply_filters( 'mbr/options/key/ac_account', NULL );
			IF ( ! empty( $ac_account_key ) ) :
				delete_transient( $ac_account_key );
			ENDIF;

			$standard_fields_list_key = apply_filters( 'mbr/options/key/standard_fields_list', NULL );
			IF ( ! empty( $standard_fields_list_key ) ) :
				delete_transient( $standard_fields_list_key );
			ENDIF;

			$webhook_fields_list_key = apply_filters( 'mbr/options/key/webhook_fields_list', NULL );
			IF ( ! empty( $webhook_fields_list_key ) ) :
				delete_transient( $webhook_fields_list_key );
			ENDIF;

			$custom_fields_list_key = apply_filters( 'mbr/options/key/custom_fields_list', NULL );
			IF ( ! empty( $custom_fields_list_key ) ) :
				delete_transient( $custom_fields_list_key );
			ENDIF;

			$automations_list_key = apply_filters( 'mbr/options/key/automations_list', NULL );
			IF ( ! empty( $automations_list_key ) ) :
				delete_transient( $automations_list_key );
			ENDIF;

			$tags_list_key = apply_filters( 'mbr/options/key/tags_list', NULL );
			IF ( ! empty( $tags_list_key ) ) :
				delete_transient( $tags_list_key );
			ENDIF;

			return;
		ELSEIF ( $old_api_key <> $MBR_OPTIONS['api_key'] ) :
			$MBR_OPTIONS['api_key'] = $new_api_key;
			$passfield = apply_filters( 'mbr/passfield', NULL );
			
			apply_filters( 'mbr/api', 'list_field_add', [ 'title' => $passfield, 'type' => 1, 'req' => 0, 'perstag' => upper_trim( $passfield ), 'p' => [0] ] );

			return;
		ENDIF;

		return;
	}

	public function backup_settings( $arr_settings = [] )
	{
		global $wpdb;
		set_transient($this->opt_name . '_backup_' . time(), $arr_settings, 45 * 86400 );
		
		$obj_results = $wpdb->get_results( "SELECT * FROM " . $wpdb->prefix . "options WHERE option_name LIKE '_transient_" . $this->opt_name . "_backup_%'", OBJECT );
		FOREACH ($obj_results AS $key => $obj) :
			get_transient( substr( $obj->option_name, 11 ) );
		ENDFOREACH;
	}

}
$GLOBALS['mbr_inst_api_settings'] = mbr_api_settings::getInstance();