<?php
class mbr_badgeos
{
	use TRAIT__getInstance;

	public $badgeos_extension = 'badgeos';

	public function _load()
	{
		global $MBR_OPTIONS;

		mbr_constant( 'MBR_AC_EARNED_BY_TRIGGER',    'ac_earned_by' );
		mbr_constant( 'MBR_AC_STEP_HAS_TAG_TRIGGER', 'ac_contact_has_tag' );
		mbr_constant( 'MBR_AC_STEP_IN_LIST_TRIGGER', 'ac_contact_in_list' );

		IF ( ! empty( $MBR_OPTIONS['extensions']['mbr_ext_ac_event_tracking_enabled'] ) ) :
	  	$this->tracking_activity = TRUE;
		ELSE:
	  	$this->tracking_activity = FALSE;
		ENDIF;
		IF ( $this->tracking_activity ) :
			
			add_filter( 'badgeos_award_achievement',                  [ $this, 'badgeos_award_achievement' ],                10, 2 );
			
			add_filter( 'badgeos_revoke_achievement',                 [ $this, 'badgeos_revoke_achievement' ],               10, 2 );
			
			IF ( is_admin() ) :
				add_filter( 'badgeos_achievement_data_meta_box_fields', [ $this, 'badgeos_achievement_data_meta_box_fields' ], 10, 3 );
			ENDIF;
		ENDIF;

		add_action( 'init', [ $this, '_init' ], 11 );
	}

	public function _init()
	{
		global $MBR_OPTIONS;

		IF ( ! class_exists( 'BadgeOS' ) ) :
			return;
		ENDIF;

		IF ( ! empty( $MBR_OPTIONS['extensions']['mbr_ext_badgeos_woocommerce_enabled'] ) ) :
			$this->woo_activity = TRUE;
		ELSE:
			$this->woo_activity = FALSE;
		ENDIF;

		IF ( $this->woo_activity AND ! class_exists( 'WooCommerce' ) ) :
			$this->woo_activity      = FALSE;
		ENDIF;

		IF ( is_admin() ) :
			add_filter( 'mbr_woo_badgeos_step_exclude_product_types', [ $this, 'woo_badgeos_step_exclude_product_types' ],   10, 1 );
		ENDIF;

		add_action( 'init',                                         [ $this, 'badgeos_load_triggers' ],                    12, 0 );
		add_filter( 'mbr/insert_post/exclude_post_types',           [ $this, 'badgeos_exclude_post_types' ],               11, 1 );

		
		add_action( 'wp_footer',                                    [ $this, 'ac_tag_list_step_trigger' ],                 9999, 0 );
		add_action( 'wp_footer',                                    [ $this, 'ac_tag_list_earned_by_trigger' ],            9999, 0 );

		add_filter( 'badgeos_get_step_requirements',                [ $this, 'badgeos_get_step_requirements' ],            10, 2 );
		add_filter( 'badgeos_activity_triggers',                    [ $this, 'badgeos_activity_triggers' ],                999 );
		add_filter( 'user_deserves_achievement',                    [ $this, 'badgeos_user_deserves_step' ],               15, 6 );
		add_filter( 'user_deserves_achievement',                    [ $this, 'badgeos_user_deserves_achievement_ac_earned_by' ], 16, 6 );

		IF ( is_admin() ) :
			add_filter( 'badgeos_achievement_earned_by',              [ $this, 'badgeos_achievement_earned_by'],            10, 1 );
			add_filter( 'badgeos_achievement_data_meta_box_fields',   [ $this, 'badgeos_achievement_data_meta_box_fields_ac'], 10, 3 );
			add_filter( 'badgeos_save_step',                          [ $this, 'badgeos_save_step'],                        10, 3 );
			add_action( 'admin_footer',                               [ $this, 'badgeos_step_js'],                          10, 0 );
			add_action( 'admin_footer',                               [ $this, 'badgeos_earned_by_js'],                     10, 0 );
			add_action( 'badgeos_steps_ui_html_after_trigger_type',   [ $this, 'badgeos_step_trigger_select'],              10, 2 );
			add_action( 'badgeos_steps_ui_html_after_trigger_type',   [ $this, 'badgeos_step_etc_select'],                  10, 2 );
		ENDIF;

		$this->step_requirements = [
			'mbr_badgeos_step_trigger'      => '_mbr_badgeos_step_trigger',
			'mbr_badgeos_step_requirements' => '_mbr_badgeos_step_requirements'
		];

		$this->mbr_badgeos_activities = [];

		$this->mbr_badgeos_activities['activity']['mbr_badgeos_ac_activity_trigger'] = [
			'label'                   => _x( 'ActiveCampaign Contact', 'wp_admin', 'activemember360' ),
			'step_etc_select'         => 'ac_badgeos_step_etc_select',
			'save_step'               => 'ac_badgeos_save_step',
			'triggers'                => [
			MBR_AC_STEP_HAS_TAG_TRIGGER => [ 'label' => _x( 'has tag',               'wp_admin', 'activemember360' ), 'user_deserves_step' => 'ac_badgeos_user_deserves_has_tag' ],
			MBR_AC_STEP_IN_LIST_TRIGGER => [ 'label' => _x( 'is subscribed to list', 'wp_admin', 'activemember360' ), 'user_deserves_step' => 'ac_badgeos_user_deserves_in_list' ]
			]
		];

		IF ( $this->woo_activity ) :
			$this->mbr_badgeos_activities['activity']['mbr_badgeos_woo_activity_trigger'] = [
			'label'                   => _x( 'WooCommerce Activity', 'wp_admin', 'activemember360' ),
			'step_etc_select'         => 'woo_badgeos_step_etc_select',
			'save_step'               => 'woo_badgeos_save_step',
			'triggers'                => [
				'woocommerce_payment_complete' => [ 'label' => _x( 'made a purchase', 'wp_admin', 'activemember360' ), 'user_deserves_step' => 'woo_badgeos_user_deserves_payment_complete' ]
			]
			];
		ENDIF;

		
		$this->activity_triggers      = wp_list_pluck( $this->mbr_badgeos_activities['activity'], 'label' );
		
		$this->earned_by_triggers     = [ MBR_AC_EARNED_BY_TRIGGER ];
		$this->step_triggers          = wp_list_pluck( $this->mbr_badgeos_activities['activity'], 'triggers' );

		FOREACH ( $this->mbr_badgeos_activities AS $key => $arrACTIVITIES ) :
			FOREACH ( $arrACTIVITIES AS $activity => $arrACTIVITY ) :
			IF ( is_admin() ) :
				add_filter( 'mbr_badgeos_save_step', [ $this, $arrACTIVITY['save_step'] ], 10, 3 );
				IF ( ! empty( $arrACTIVITY['step_etc_select'] ) ) :
				add_filter( 'mbr_badgeos_step_etc_select', [ $this, $arrACTIVITY['step_etc_select'] ], 10, 3 );
				ENDIF;
			ENDIF;
			ENDFOREACH;
		ENDFOREACH;
  }

  public function badgeos_exclude_post_types( $exclude = [] )
  {
		return array_merge( $exclude, [ 'badgeos-log-entry' ] );
  }

  public function badgeos_trigger_get_user_id( $user_id, $trigger, $args )
  {
		global $current_user;
		IF ( empty( $user_id ) ) :
			IF ( ! empty( $current_user->ID ) ) :
			return $current_user->ID;
			ENDIF;
		ENDIF;

		return $user_id;
  }

  


  public function badgeos_award_achievement( $user_id, $achievement_id )
  {
		$log_event = get_post_meta( $achievement_id, '_mbr_badgeos_log_achievement_awarded_event', TRUE );
		IF ( ! empty( $log_event ) AND $log_event == 'on' ) :
			$user = get_user_by( 'id', $user_id );
			IF ( $user->user_email ) :
			$achievement = get_post( $achievement_id );
			do_action( 'mbr/track/log/event', __( 'Achievement awarded', 'activemember360' ), $achievement->post_title, $user->user_email );
			do_action( 'mbr/track/log/event', __( 'Achievement awarded', 'activemember360' ), $achievement_id,          $user->user_email );
			ENDIF;
		ENDIF;
	}

  public function badgeos_revoke_achievement( $user_id, $achievement_id )
  {
		$log_event = get_post_meta( $achievement_id, '_mbr_badgeos_log_achievement_revoked_event', TRUE );
		IF ( ! empty( $log_event ) AND $log_event == 'on' ) :
			$user = get_user_by( 'id', $user_id );
			IF ( $user->user_email ) :
			$achievement = get_post( $achievement_id );
			do_action( 'mbr/track/log/event', __( 'Achievement revoked', 'activemember360' ), $achievement->post_title, $user->user_email );
			do_action( 'mbr/track/log/event', __( 'Achievement revoked', 'activemember360' ), $achievement_id,          $user->user_email );
			ENDIF;
		ENDIF;
  }

  public function badgeos_achievement_data_meta_box_fields( $array, $prefix, $achievement_types )
  {
		$new_array[] = [
			'name'     => _x( 'Log awarded event', 'wp_admin', 'activemember360' ),
			'desc'     => ' ' . _x( 'Log this achievement when awarded as an event in ActiveCampaign.', 'wp_admin', 'activemember360' ),
			'id'       => '_mbr_badgeos_log_achievement_awarded_event',
			'type'     => 'checkbox',
		];
		$new_array[] = [
			'name'     => _x( 'Log revoked event', 'wp_admin', 'activemember360' ),
			'desc'     => ' ' . _x( 'Log this achievement when revoked as an event in ActiveCampaign.', 'wp_admin', 'activemember360' ),
			'id'       => '_mbr_badgeos_log_achievement_revoked_event',
			'type'     => 'checkbox',
		];

		return array_merge( $new_array, $array );
  }

  public function badgeos_achievement_earned_by( $array )
  {
		$new_array[] = [ 'name' => _x( 'ActiveCampaign Contact has specific tag',            'wp_admin', 'activemember360'), 'value' => 'mbr_contact_has_tag' ];
		$new_array[] = [ 'name' => _x( 'ActiveCampaign Contact subscribed to specific list', 'wp_admin', 'activemember360'), 'value' => 'mbr_contact_in_list' ];

		return array_merge( $new_array, $array );
  }

  public function badgeos_achievement_data_meta_box_fields_ac( $array, $prefix, $achievement_types )
  {
		
		$this->badgeos_earned_by_prefix = $prefix;
		$arr_tags        = [];
		FOREACH ( apply_filters( 'mbr/site_tags/get', NULL ) AS $tag_id => $tag_name ) :
			$arr_tags[]    = [ 'name' => $tag_name, 'value' => $tag_id ];
		ENDFOREACH;

		$arr_lists       = [];
		FOREACH ( apply_filters( 'mbr/site_lists/get', NULL ) AS $list_id => $list_name ) :
			$arr_lists[]   = [ 'name' => $list_name, 'value' => $list_id ];
		ENDFOREACH;

		$new_array       = [];
		FOREACH ( $array AS $key => $element ) :
			$new_array[]   = $element;
			IF ( $element['id'] == $prefix . 'earned_by' ) :
			$new_array[] = [
				'name'     => _x( 'Tag ID', 'wp_admin', 'activemember360' ),
				'desc'     => ' ' . _x( 'Required ActiveCampaign Tag for earning this achievement.', 'wp_admin', 'activemember360' ),
				'id'       => '_mbr_badgeos_ac_contact_has_tag',
				'type'     => 'select',
				'options'  => $arr_tags
			];
			$new_array[] = [
				'name'     => _x( 'List ID', 'wp_admin', 'activemember360' ),
				'desc'     => ' ' . _x( 'Required subscribed list for earning this achievement.', 'wp_admin', 'activemember360' ),
				'id'       => '_mbr_badgeos_ac_contact_in_list',
				'type'     => 'select',
				'options'  => $arr_lists
			];
			ENDIF;
		ENDFOREACH;

		return $new_array;
  }

  public function badgeos_earned_by_js()
	{
		IF ( $this->badgeos_earned_by_prefix ) :
			echo "<script>
					function mbr_badgeos_earned_by() {
					if ( jQuery('#{$this->badgeos_earned_by_prefix}earned_by').val() == 'mbr_contact_has_tag' ) {
						jQuery('.cmb_id__mbr_badgeos_ac_contact_has_tag').show();
						jQuery('.cmb_id__mbr_badgeos_ac_contact_in_list').hide();
					} else if ( jQuery('#{$this->badgeos_earned_by_prefix}earned_by').val() == 'mbr_contact_in_list' ) {
						jQuery('.cmb_id__mbr_badgeos_ac_contact_has_tag').hide();
						jQuery('.cmb_id__mbr_badgeos_ac_contact_in_list').show();
					} else {
						jQuery('.cmb_id__mbr_badgeos_ac_contact_has_tag').hide();
						jQuery('.cmb_id__mbr_badgeos_ac_contact_in_list').hide();
					}
					}
					jQuery( document ).ready( function() {
					mbr_badgeos_earned_by();
					jQuery( '#{$this->badgeos_earned_by_prefix}earned_by' ).change(
						function() {
						mbr_badgeos_earned_by();
						}
					)
					} );
				</script>";
		ENDIF;
  }

  


  public function badgeos_activity_triggers( $triggers )
  {
		FOREACH ( $this->activity_triggers AS $activity => $label ) :
			$_triggers[ $activity ]    = $label;
		ENDFOREACH;

		return $_triggers + $triggers;
  }

  public function badgeos_load_triggers()
  {
		FOREACH ( $this->step_triggers AS $activity => $arrTRIGGERS )  :
			FOREACH ( $arrTRIGGERS AS $trigger => $filler ) :
			add_action( $trigger, [ $this, 'badgeos_trigger_event' ], 10, 20 );
			ENDFOREACH;
		ENDFOREACH;
		FOREACH ( $this->earned_by_triggers AS $key => $trigger ) :
			add_action( $trigger, [ $this, 'badgeos_trigger_event' ], 10, 20 );
		ENDFOREACH;
  }

  public function badgeos_get_step_requirements( $requirements, $step_id )
  {
		FOREACH ( $this->step_requirements AS $key => $value ) :
			$requirements[ $key ] = get_post_meta( $step_id, $value, TRUE );
		ENDFOREACH;

		return $requirements;
  }

  public function badgeos_step_trigger_select( $step_id, $post_id )
  {
		FOREACH ( $this->step_triggers AS $activity => $arrTRIGGERS )  :
			echo '<select name="mbr_badgeos_step_trigger" class="select-' . $activity . '">';
			echo '<option value="">' . _x( 'Select a Trigger', 'wp_admin', 'activemember360' ) . '</option>';
			$current_trigger      = get_post_meta( $step_id, '_mbr_badgeos_step_trigger', TRUE );
			FOREACH ( $arrTRIGGERS AS $trigger => $arr_trigger ) :
			echo '<option' . selected( $current_trigger, $trigger, FALSE ) . ' value="' . esc_attr( $trigger ) . '">' . esc_html( $arr_trigger['label'] ) . '</option>';
			ENDFOREACH;
			echo '</select>';
		ENDFOREACH;
  }

  public function badgeos_step_etc_select( $step_id, $post_id )
  {
		$return = '';
		echo apply_filters( 'mbr_badgeos_step_etc_select', $return, $step_id, $post_id );
  }

  public function badgeos_save_step( $title, $step_id, $step_data ) {
		$title = apply_filters( 'mbr_badgeos_save_step', $title, $step_id, $step_data );

		return $title;
  }

  public function badgeos_trigger_event()
  {
		global $current_user, $wpdb;

		IF ( empty( $current_user ) ) :
			return;
		ENDIF;

		$args         = func_get_args();
		$trigger      = current_filter();
		$userID       = $current_user->ID;

		$blog_id      = get_current_blog_id();

		$new_count    = badgeos_update_user_trigger_count( $userID, $trigger, $blog_id );

		





		$triggered_achievements = $wpdb->get_results( $wpdb->prepare( "SELECT post_id FROM {$wpdb->postmeta} WHERE meta_key = '_mbr_badgeos_step_trigger' AND meta_value = %s", $trigger ) );
		FOREACH ( $triggered_achievements AS $achievement ) :
			badgeos_maybe_award_achievement_to_user( $achievement->post_id, $userID, $trigger, $blog_id, $args );
		ENDFOREACH;

		IF ( $trigger == MBR_AC_EARNED_BY_TRIGGER ) :
			$triggered_achievements = $wpdb->get_results( $wpdb->prepare( "SELECT post_id FROM {$wpdb->postmeta} WHERE (meta_key = '_badgeos_earned_by' AND meta_value = %s) OR (meta_key = '_badgeos_earned_by' AND meta_value = %s)", 'mbr_contact_has_tag', 'mbr_contact_in_list' ) );
			FOREACH ( $triggered_achievements AS $achievement ) :
			badgeos_maybe_award_achievement_to_user( $achievement->post_id, $userID, $trigger, $blog_id, $args );
			ENDFOREACH;
		ENDIF;
  }

  public function badgeos_user_deserves_achievement_ac_earned_by( $return, $user_id, $achievement_id, $trigger = '', $site_id = 1, $args = [] )
  {
		IF ($trigger <> MBR_AC_EARNED_BY_TRIGGER) :
			return $return;
		ENDIF;

		$contact_id = apply_filters( 'mbr/contact_id',    NULL );
		$arr_tags   = apply_filters( 'mbr/contact/tags',  NULL );
		$arr_lists  = apply_filters( 'mbr/contact/lists', NULL );

		IF ( 'mbr_contact_has_tag' == get_post_meta( $achievement_id, '_badgeos_earned_by', TRUE ) ) :
			$tag_id = get_post_meta( $achievement_id, '_mbr_badgeos_ac_contact_has_tag', TRUE );
			IF ( ! empty( $tag_id ) AND ! empty( $arr_tags ) ) :
			IF ( $contact_id ) :
				IF ( array_key_exists( $tag_id, $arr_tags ) ) :
				return TRUE;
				ENDIF;
			ENDIF;
			ENDIF;
			return FALSE;
		ENDIF;

		IF ( 'mbr_contact_in_list' == get_post_meta( $achievement_id, '_badgeos_earned_by', TRUE ) ) :
			$list_id = get_post_meta( $achievement_id, '_mbr_badgeos_ac_contact_in_list', TRUE );
			IF ( ! empty( $list_id ) AND ! empty( $arr_lists ) ) :
			IF ( $contact_id ) :
				IF ( array_key_exists( $list_id, $arr_lists ) ) :
				return TRUE;
				ENDIF;
			ENDIF;
			ENDIF;
			return FALSE;
		ENDIF;

		return $return;
  }

  public function badgeos_user_deserves_step( $return, $user_id, $achievement_id, $trigger = '', $site_id = 1, $args = [] )
  {
		IF ( get_post_type( $achievement_id ) <> 'step' ) :
			return $return;
		ENDIF;

		$requirements = badgeos_get_step_requirements( $achievement_id );

		$callback     = $this->mbr_badgeos_activities['activity'][ $requirements['trigger_type'] ]['triggers'][ $trigger ]['user_deserves_step'];

		IF ( is_callable( [ $this, $callback ] ) ) :
			$return = call_user_func( [ $this, $callback ], $return, $requirements, $user_id, $trigger, $site_id, $args );
		ENDIF;

		return $return;
		}

		public function badgeos_step_js()
		{
		?>
		<script>
			jQuery( document ).ready( function( $ ) {
			$( document ).on( 'change', '.select-trigger-type', function () {
				var trigger_type = $( this );

				if ( 'mbr_badgeos_ac_activity_trigger' == trigger_type.val() ) {
					trigger_type.siblings( '.select-mbr_badgeos_ac_activity_trigger' ).show().change();
					if ( trigger_type.siblings( '.select-mbr_badgeos_ac_activity_trigger' ).val() != '' ) {
						if ( trigger_type.siblings( '.select-mbr_badgeos_ac_activity_trigger' ).val() == '<?php echo MBR_AC_STEP_HAS_TAG_TRIGGER;?>' ) {
							trigger_type.siblings('.select-mbr-badgeos-ac-has-tag' ).show();
						} else if ( trigger_type.siblings('.select-mbr_badgeos_ac_activity_trigger' ).val() == '<?php echo MBR_AC_STEP_IN_LIST_TRIGGER;?>' ) {
							trigger_type.siblings('.select-mbr-badgeos-ac-in-list' ).show();
						} else {
							trigger_type.siblings( '.select-mbr-badgeos-ac-has-tag' ).hide();
							trigger_type.siblings( '.select-mbr-badgeos-ac-in-list' ).hide();
						}
					}
				}

				<?php
				IF ( $this->woo_activity ) :
				?>
				if ( 'mbr_badgeos_woo_activity_trigger' == trigger_type.val() ) {
					trigger_type.siblings( '.select-mbr_badgeos_woo_activity_trigger' ).show().change();
					if ( trigger_type.siblings('.select-mbr_badgeos_woo_activity_trigger' ).val() != '' ) {
					if ( trigger_type.siblings( '.select-mbr_badgeos_woo_activity_trigger' ).val() == 'woocommerce_payment_complete' ) {
						trigger_type.siblings( '.select-mbr-badgeos-woo-made-purchase' ).show();
					} else {
						trigger_type.siblings( '.select-mbr-badgeos-woo-made-purchase' ).hide();
					}
					}
				}
				<?php
				ENDIF;
				?>
				mbr_badgeos_hide_activity_steps( trigger_type );
			});

			$( document ).on( 'change', '.select-mbr_badgeos_ac_activity_trigger,' + '.select-mbr-badgeos-ac-has-tag', function () {
				mbr_ac_badgeos_step_change( $( this ) );
			} );
			$( document ).on( 'change', '.select-mbr_badgeos_ac_activity_trigger,' + '.select-mbr-badgeos-ac-in-list', function () {
				mbr_ac_badgeos_step_change( $( this ) );
			} );

			<?php
			IF ( $this->woo_activity ) :
				?>
				
				$( document ).on('change', '.select-mbr_badgeos_woo_activity_trigger,' + '.select-mbr-badgeos-woo-made-purchase', function () {
				mbr_woo_badgeos_step_change( $( this ) );
				} );
				<?php
			ENDIF;
			?>

			$( '.select-trigger-type' ).change();

			$( document ).on( 'update_step_data', function ( event, step_details, step ) {

				if ( step_details.trigger_type == 'mbr_badgeos_ac_activity_trigger' ) {
				step_details.mbr_badgeos_step_trigger       = $( '.select-mbr_badgeos_ac_activity_trigger', step ).val();
				step_details.mbr_badgeos_step_trigger_label = $( '.select-mbr_badgeos_ac_activity_trigger option', step ).filter( ':selected' ).text();
				step_details.mbr_badgeos_ac_has_tag         = $( '.select-mbr-badgeos-ac-has-tag', step ).val();
				step_details.mbr_badgeos_ac_in_list         = $( '.select-mbr-badgeos-ac-in-list', step ).val();
				}

				<?php
				IF ( $this->woo_activity ) :
				?>
				if ( step_details.trigger_type == 'mbr_badgeos_woo_activity_trigger' ) {
					step_details.mbr_badgeos_step_trigger       = $( '.select-mbr_badgeos_woo_activity_trigger', step ).val();
					step_details.mbr_badgeos_step_trigger_label = $( '.select-mbr_badgeos_woo_activity_trigger option', step ).filter( ':selected' ).text();
					step_details.mbr_badgeos_woo_made_purchase  = $( '.select-mbr-badgeos-woo-made-purchase', step ).val();
				}
				<?php
				ENDIF;
				?>
			});

			});

			function mbr_ac_badgeos_step_change( $this ) {
				var trigger_parent = $this.parent(), trigger_value = trigger_parent.find( '.select-mbr_badgeos_ac_activity_trigger' ).val();
				trigger_parent.find( '.select-mbr-badgeos-ac-has-tag' ).toggle( '<?php echo MBR_AC_STEP_HAS_TAG_TRIGGER;?>' == trigger_value );
				trigger_parent.find( '.select-mbr-badgeos-ac-in-list' ).toggle( '<?php echo MBR_AC_STEP_IN_LIST_TRIGGER;?>' == trigger_value );
			}

			<?php
			IF ( $this->woo_activity ) :
			?>
			function mbr_woo_badgeos_step_change( $this ) {
				var trigger_parent = $this.parent(), trigger_value = trigger_parent.find( '.select-mbr_badgeos_woo_activity_trigger' ).val();
				trigger_parent.find( '.select-mbr-badgeos-woo-made-purchase' ).toggle( 'woocommerce_payment_complete' == trigger_value );
			}
			<?php
			ENDIF;
			?>

			function mbr_badgeos_hide_activity_steps( trigger_type ) {
			if ( trigger_type.val() != 'mbr_badgeos_ac_activity_trigger' ) {
					trigger_type.siblings( '.select-mbr_badgeos_ac_activity_trigger' ).hide().change();
					trigger_type.siblings( '.select-mbr-badgeos-ac-has-tag' ).hide();
					trigger_type.siblings( '.select-mbr-badgeos-ac-in-list' ).hide();
			}
			<?php
			IF ( $this->woo_activity ) :
				?>
				if ( trigger_type.val() != 'mbr_badgeos_woo_activity_trigger' ) {
					trigger_type.siblings( '.select-mbr_badgeos_woo_activity_trigger' ).hide().change();
					trigger_type.siblings( '.select-mbr-badgeos-woo-made-purchase' ).hide();
				}
				<?php
			ENDIF;
			?>
			}

		</script>
		<?php
  }

  


  




  public function ac_tag_list_step_trigger()
  {
		IF ( ! is_admin() AND ! is_administrator() ) :
			do_action( MBR_AC_STEP_HAS_TAG_TRIGGER );
			do_action( MBR_AC_STEP_IN_LIST_TRIGGER );
		ENDIF;
  }

  public function ac_tag_list_earned_by_trigger()
  {
		IF ( ! is_admin() AND ! is_administrator() ) :
			do_action( MBR_AC_EARNED_BY_TRIGGER );
		ENDIF;
  }

  public function ac_badgeos_step_etc_select( $return, $step_id, $post_id )
  {
		
		$current_trigger      = get_post_meta( $step_id, '_mbr_badgeos_step_trigger', TRUE );

		$current_requirements = json_decode( get_post_meta($step_id, '_mbr_badgeos_step_requirements', TRUE ), TRUE );

		$return   .= '<select name="mbr_badgeos_ac_has_tag" class="select-mbr-badgeos-ac-has-tag">';
		FOREACH ( apply_filters( 'mbr/site_tags/get', NULL ) AS $tag_id => $tag_name ) :
			$selected_tag = $current_requirements['has_tag'] == $tag_id ? 'selected="selected"' : '';
			$return .= '<option value="' . $tag_id . '" ' . $selected_tag . '>' . $tag_name . '</option>';
		ENDFOREACH;
		$return   .= '</select>';

		$return   .= '<select name="mbr_badgeos_ac_in_list" class="select-mbr-badgeos-ac-in-list">';
		FOREACH ( apply_filters( 'mbr/site_lists/get' ) AS $list_id => $list_name ) :
			$selected_list = $current_requirements['in_list'] == $list_id ? 'selected="selected"' : '';
			$return .= '<option value="' . $list_id . '" ' . $selected_list . '>' . $list_name . '</option>';
		ENDFOREACH;
		$return   .= '</select>';

		return $return;
  }

  public function ac_badgeos_save_step( $title, $step_id, $step_data )
  {
		IF ( $step_data['trigger_type'] == 'mbr_badgeos_ac_activity_trigger' ) :

			$arr_tags                     = apply_filters( 'mbr/site_tags/get',  NULL );
			$arr_lists                    = apply_filters( 'mbr/site_lists/get', NULL );

			update_post_meta( $step_id, '_mbr_badgeos_step_trigger', $step_data['mbr_badgeos_step_trigger'] );
			$title                        = $step_data['mbr_badgeos_step_trigger_label'];
			IF ( $step_data['mbr_badgeos_step_trigger'] == MBR_AC_STEP_HAS_TAG_TRIGGER ) :
			$tag_id                     = $step_data['mbr_badgeos_ac_has_tag'];
			$tmp_title                  = ' ' . sprintf( _x( 'AM360 Contact has tag: %s', 'wp_admin', 'activemember360' ), $arr_tags[ $tag_id ] );
			$ac_requirements['has_tag'] = $tag_id;
			update_post_meta( $step_id, '_mbr_badgeos_step_requirements', json_encode( $ac_requirements ) );
			$title                      = $tmp_title;
			ELSEIF ( $step_data['mbr_badgeos_step_trigger'] == MBR_AC_STEP_IN_LIST_TRIGGER ) :
			$list_id                    = $step_data['mbr_badgeos_ac_in_list'];
			$tmp_title                  = ' ' . sprintf( _x('AM360 Contact is subscribed to list: %s', 'wp_admin', 'activemember360' ), $arr_lists[ $list_id ] );
			$ac_requirements['in_list'] = $list_id;
			update_post_meta( $step_id, '_mbr_badgeos_step_requirements', json_encode( $ac_requirements ) );
			$title                      = $tmp_title;
			ENDIF;
		ENDIF;

		return $title;
  }

  public function ac_badgeos_user_deserves_has_tag( $return, $requirements, $user_id, $trigger, $site_id, $args )
  {
		IF ( empty( $this->step_triggers[ $requirements['trigger_type'] ][ $trigger ] ) ) :
			return $return;
		ENDIF;

		$ac_trigger           = $requirements['mbr_badgeos_step_trigger'];
		IF ( $ac_trigger == MBR_AC_STEP_HAS_TAG_TRIGGER ) :
			$return             = FALSE;
			$step_completed     = FALSE;

			$step_requirement   = json_decode( $requirements['mbr_badgeos_step_requirements'], TRUE );
			$tag_id             = $step_requirement['has_tag'];

			$arr_tags           = apply_filters( 'mbr/contact/tags', NULL );
			$contact_id         = apply_filters( 'mbr/contact_id',   NULL );

			IF ( ! empty( $tag_id ) AND ! empty( $arr_tags ) ) :
			IF ( $contact_id ) :
				IF ( array_key_exists( $tag_id, $arr_tags ) ) :
				$step_completed = TRUE;
				ENDIF;
			ENDIF;
			ENDIF;

			IF ( $step_completed ) :
			IF ( $trigger_count = badgeos_get_user_trigger_count( $user_id, $trigger, $site_id ) ) :
				IF ( $requirements['count'] <= $trigger_count ) :
				$return = TRUE;
				ENDIF;
			ENDIF;
			ENDIF;
		ENDIF;

		return $return;
  }

  public function ac_badgeos_user_deserves_in_list( $return, $requirements, $user_id, $trigger, $site_id, $args )
  {
		IF ( empty( $this->step_triggers[ $requirements['trigger_type'] ][ $trigger ] ) ) :
			return $return;
		ENDIF;

		$ac_trigger           = $requirements['mbr_badgeos_step_trigger'];
		IF ( $ac_trigger == MBR_AC_STEP_IN_LIST_TRIGGER ) :
			$return              = FALSE;
			$step_completed      = FALSE;

			$step_requirement    = json_decode( $requirements['mbr_badgeos_step_requirements'], TRUE );
			$list_id             = $step_requirement['in_list'];

			$arr_lists           = apply_filters( 'mbr/contact/lists', NULL );
			$contact_id          = apply_filters( 'mbr/contact_id',    NULL );

			IF ( ! empty( $list_id ) AND ! empty( $arr_lists ) ) :
			IF ( $contact_id ) :
				IF ( array_key_exists( $list_id, $arr_lists ) ) :
				$step_completed = TRUE;
				ENDIF;
			ENDIF;
			ENDIF;

			IF ( $step_completed ) :
			IF ( $trigger_count = badgeos_get_user_trigger_count( $user_id, $trigger, $site_id ) ) :
				IF ( $requirements['count'] <= $trigger_count ) :
				$return = TRUE;
				ENDIF;
			ENDIF;
			ENDIF;
		ENDIF;

		return $return;
  }

  


  public function woo_badgeos_step_exclude_product_types( $array )
  {
		unset( $array['external'] );
		unset( $array['grouped'] );

		return $array;
  }

  public function woo_badgeos_step_etc_select( $return, $step_id, $post_id )
  {
		$current_trigger      = get_post_meta( $step_id, '_mbr_badgeos_step_trigger', TRUE );
		$current_requirements = json_decode( get_post_meta( $step_id, '_mbr_badgeos_step_requirements', TRUE), TRUE );

		$return          .= '<select name="mbr_badgeos_woo_made_purchase" class="select-mbr-badgeos-woo-made-purchase">';

		$arrPRODUCT_TYPES = apply_filters( 'mbr_woo_badgeos_step_exclude_product_types', wc_get_product_types() );
		$objPRODUCTS      = get_posts( [ 'post_type' => 'product', 'post_status' => 'publish', 'posts_per_page' => -1 ] );
		$arrPRODUCTS      = [];

		IF ( ! empty( $objPRODUCTS ) ) :
			FOREACH ( $objPRODUCTS AS $object ) :
				$product    = wc_get_product( $object->ID );
				$arrPRODUCTS[ $product->product_type ][ $object->ID ] = esc_html( get_the_title( $object->ID ) . ' (' . $object->ID. ')' );
			ENDFOREACH;
		ENDIF;

		$return        .= '<option value="any" ' . $selected = selected( $current_requirements['product_type'], 'any', FALSE ) . '>' . _x( 'Anything', 'wp_admin','activemember360' ) . '</option>';

		FOREACH ( $arrPRODUCT_TYPES AS $product_type => $label ) :
			IF ( ! empty( $arrPRODUCTS[ $product_type ] ) ) :
			$selected   = '';
			$return    .= '<optgroup label="' . esc_html( $label ) . '">';

			IF ( empty( $current_requirements['product_id'] ) AND $current_requirements['product_type'] == $product_type ) :
				$selected = selected( TRUE, TRUE, FALSE );
			ENDIF;
			$return    .= '<option' . $selected . ' value="any_' . $product_type . '">' . sprintf( _x( 'Any %s', 'wp_admin', 'activemember360' ), $label ) . '</option>';

			FOREACH ( $arrPRODUCTS[ $product_type ] AS $product_id => $product_name ) :
				$selected = selected( $current_requirements['product_id'], $product_id, FALSE );
				$return  .= '<option' . $selected . ' value="' .$product_id . '">' . $product_name . '</option>';
			ENDFOREACH;
			$return    .= '</optgroup>';
			ENDIF;
		ENDFOREACH;
		$return .= '</select>';

		return $return;
  }

  public function woo_badgeos_save_step( $title, $step_id, $step_data )
  {
		IF ( $step_data['trigger_type'] == 'mbr_badgeos_woo_activity_trigger' ) :
			update_post_meta( $step_id, '_mbr_badgeos_step_trigger', $step_data['mbr_badgeos_step_trigger'] );
			$title              = $step_data['mbr_badgeos_step_trigger_label'];

			IF ( $step_data['mbr_badgeos_step_trigger'] == 'woocommerce_payment_complete' ) :
				$arrPRODUCT_TYPES = wc_get_product_types();

				$product_id       = $step_data['mbr_badgeos_woo_made_purchase'];
				$tmp_title        = ' ' . _x( 'AM360 woo made purchase: ', 'wp_admin', 'activemember360' );

				IF ( $product_id == 'any' ) :
					$product_type   = 'any';
					$product_id     = 0;
					$tmp_title     .= _x( 'anything', 'wp_admin', 'activemember360' );
				ELSEIF ( stripos( $product_id, 'any_' ) !== FALSE ) :
					$tmp_product_type = explode( 'any_', $product_id );
					$product_type   = $tmp_product_type[1];
					$product_id     = 0;
					$tmp_title     .= sprintf( _x( 'Any %s', 'wp_admin', 'activemember360'), $arrPRODUCT_TYPES[ $product_type ] );
				ELSE:
					$product        = wc_get_product( $product_id );
					$product_type   = $product->product_type;
					$tmp_title     .= sprintf( '%s "%s" (%s)', $arrPRODUCT_TYPES[$product_type], get_the_title( $product_id ), $product_id );
				ENDIF;

				$woo_requirements['product_type'] = $product_type;
				$woo_requirements['product_id']   = $product_id;

				update_post_meta( $step_id, '_mbr_badgeos_step_requirements', json_encode( $woo_requirements ) );
				$title           = $tmp_title;
			ENDIF;
		ENDIF;

		return $title;
  }

  public function woo_badgeos_user_deserves_payment_complete( $return, $requirements, $user_id, $trigger, $site_id, $args )
  {
		IF ( empty( $this->step_triggers[ $requirements['trigger_type'] ][ $trigger ] ) ) :
			return $return;
		ENDIF;

		$woo_trigger           = $requirements['mbr_badgeos_step_trigger'];
		IF ( $woo_trigger == 'woocommerce_payment_complete' ) :
			$return              = FALSE;
			$step_completed      = FALSE;

			$step_requirement    = json_decode( $requirements['mbr_badgeos_step_requirements'], TRUE );
			$product_type        = $step_requirement['product_type'];
			$product_id          = $step_requirement['product_id'];
			$order_id            = $args[0];

			$order               = new WC_Order( $order_id );
			$order_items         = $order->get_items();

			FOREACH ( $order_items AS $k => $order_item ) :

				IF ( $order_item['type'] <> 'line_item' ) :
					continue;
				ENDIF;

				
				IF ( ! empty( $product_id ) AND $order_item['product_id'] <> $product_id ) :
					continue;
				ENDIF;

				$product   = wc_get_product( $order_item['product_id'] );

				IF ( $product_type == 'any' ) :
					$step_completed = TRUE;
					break;
				ELSE:
					IF ( $product_type == $product->product_type ) :
						IF ( empty( $product_id ) OR $product_id == $order_item['product_id'] ) :
							$step_completed = TRUE;
							break;
						ENDIF;
					ENDIF;
				ENDIF;

			ENDFOREACH;

			IF ( $step_completed ) :
				IF ( $trigger_count = badgeos_get_user_trigger_count( $user_id, $trigger, $site_id ) ) :
					IF ( $requirements['count'] <= $trigger_count ) :
						$return = TRUE;
					ENDIF;
				ENDIF;
			ENDIF;
		ENDIF;

		return $return;
  }

}
$GLOBALS['mbr_badgeos'] = mbr_badgeos::getInstance();
