<?php
add_action( 'bbp_after_setup_actions', 'MBRBBP', 9999 );

function MBRBBP()
{
	IF( empty( $GLOBALS['mbr_bbpress'] ) ) :
		$GLOBALS['mbr_bbpress'] = mbr_bbpress::getInstance();
	ENDIF;

	return $GLOBALS['mbr_bbpress'];
}

class mbr_bbpress
{
	use TRAIT__getInstance;

	public function _load()
	{
		IF ( ! class_exists( 'bbPress' ) ) :
			return;
		ENDIF;

		$this->sc_prefix = MBR()->scPREFIX;

		$this->bbforum = apply_filters( 'bbp_forum_post_type', 'forum' );
		$this->bbtopic = apply_filters( 'bbp_topic_post_type', 'topic' );
		$this->bbreply = apply_filters( 'bbp_reply_post_type', 'reply' );

		global $pagenow;
		IF ( is_admin() ) :
			
			IF ( in_array( $pagenow, [ 'post.php', 'post-new.php' ] ) ) :
				add_action( "load-{$pagenow}",            [ $this, 'init_metaboxes' ] );
			ENDIF;
			add_action( 'mbr/quick_edit/protection/box', [ $this, 'init_quick_edit' ], 0 );
		ENDIF;

		
		add_action( 'profile_update',               [ $this, 'profile_update' ],             10, 2 ); 
		add_action( 'user_profile_update_errors',   [ $this, 'user_profile_update_errors' ], 10, 3 );

		add_filter( 'mbr/protectable_post_types/built_in', [ $this, 'add_protectable_post_types' ],      10, 1 );

		
		add_filter( 'mbr/template_redirect/escape', [ $this, 'bbp_is_single_user' ],         10, 0 );

		IF ( ! is_admin() ) :
			add_filter( 'bbp_get_topic_content',        [ $this, 'bbp_get_content' ], -99, 2 );
			add_filter( 'bbp_get_reply_content',        [ $this, 'bbp_get_content' ], -99, 2 );
			add_filter( 'the_title',                    [ $this, 'bbp_get_content' ], -99, 2 );
			add_filter( 'document_title_parts',         [ $this, 'bbp_title' ],        9 );
		ENDIF;
	}

	public function init_metaboxes( $_post_type = FALSE, $_post = FALSE )
	{
		global $pagenow, $typenow;

		IF ( in_array( $typenow, [ $this->bbforum, $this->bbtopic, $this->bbreply ] ) ) :
			IF ( "load-{$pagenow}" == current_filter() ) :
				add_action( 'add_meta_boxes', [ $this, __FUNCTION__ ], 10, 2 );
			ELSEIF ( 'add_meta_boxes' == current_filter() ) :
				do_action( 'mbr/add_meta_box', $_post, [ $this, 'render_meta_boxes_protection' ], NULL );
			ENDIF;
		ENDIF;
	}

	public function init_quick_edit( $_post )
	{
		IF ( $this->bbforum == $_post->post_type ) :
			$this->render_meta_boxes_protection( $_post, 'quick-protection' );
		ENDIF;
	}

	public function render_meta_boxes_protection($_post, $box)
	{
		$settings = apply_filters( 'mbr/get_meta_box_protection_settings', $_post );

		do_action( 'mbr/render_metabox_css_helper', $box);

		IF ( in_array( $_post->post_type, [$this->bbtopic, $this->bbreply] ) OR ( $_post->post_type == $this->bbforum AND !empty( $_post->post_parent ) ) ) :
			echo "<p style='color:" . apply_filters( 'mbr/plugin_color', NULL ) . ";font-size:10px;'>" . sprintf( _x( 'Permissions for this %s are automatically inherited from its parent %s.', 'wp_admin', 'activemember360' ), $_post->post_type, $this->bbforum ) . "</p>";
			return;
		ELSEIF ( empty( $settings['membership_levels'] ) ) :
			echo "<p style='color:" . apply_filters( 'mbr/plugin_color', NULL ) . ";font-size:10px;'>" . sprintf( _x( 'You have not yet defined which memberships you want to use on this site. Please do so in <a href="' . admin_url( "admin.php?page=ActiveMember360&mbrtab=memberships" ) . '">%s.</a>', 'wp_admin', 'activemember360' ), "'Memberships'" ) . "</p>";
			return;
		ENDIF;

		do_action( 'mbr/render_meta_boxes_protection_generic',     $_post, $settings);
		do_action( 'mbr/render_meta_boxes_protection_memberships', $_post, $settings);
		do_action( 'mbr/render_meta_boxes_protection_filters',     $_post, $settings);
		do_action( 'mbr/render_meta_boxes_protection_redirect',    $_post, $settings);
	}

	public function add_protectable_post_types( $arr_protectable_post_types = [] )
	{
		array_push( $arr_protectable_post_types, $this->bbforum );
		return $arr_protectable_post_types;
	}

	public function bbp_is_single_user()
	{
		IF ( function_exists('bbp_is_single_user' ) ) :
			return bbp_is_single_user();
		ENDIF;

		return FALSE;
	}

	public function user_profile_update_errors($errors, $update, $user)
	{
		IF ( !class_exists( 'bbpress' ) OR empty( $_POST['bbp_user_edit_submit'] ) ) :
			return;
		ENDIF;

		IF ( ! $contact_id = apply_filters( 'mbr/contact_id', FALSE ) ) :
			return;
		ENDIF;

		$SESSION = [];
		$SESSION = apply_filters( 'mbr/session/get', $SESSION );

		$_POST['email'] = lower_trim( $_POST['email'] );
		IF ( !empty($_POST['email']) AND $_POST['email'] <> lower_trim( $SESSION['email'] ) ) :
			IF ( $objCONTACT = apply_filters( 'mbr/api', "contact_view_email?email={$_POST['email']}" ) AND (int)$objCONTACT->result_code ) :
				$errors->add( 'ac_dupe_mail', __('The email address entered already exists. Please use a different one.', 'activemember360'), '' );
			ENDIF;
		ENDIF;
	}

	public function profile_update( $user_id, $user_data )
	{
		IF ( !class_exists( 'bbpress' ) OR empty( $_POST['bbp_user_edit_submit'] ) ) :
			return;
		ENDIF;

		IF ( ! $contact_id = apply_filters( 'mbr/contact_id', FALSE ) ) :
			return;
		ENDIF;

		$SESSION = [];
		$SESSION = apply_filters( 'mbr/session/get', $SESSION );

		$_POST['first_name'] = trim( $_POST['first_name'] );
		IF ( !empty( $_POST['first_name'] ) AND $_POST['first_name'] <> trim( $SESSION['first_name'] ) ) :
			$contact['first_name'] = $_POST['first_name'];
			$SESSION['first_name'] = $_POST['first_name'];
		ENDIF;

		$_POST['last_name'] = trim( $_POST['last_name'] );
		IF ( !empty( $_POST['last_name'] ) AND $_POST['last_name'] <> trim( $SESSION['last_name'] ) ) :
			$contact['last_name'] = $_POST['last_name'];
			$SESSION['last_name'] = $_POST['last_name'];
		ENDIF;

		$perstag        = apply_filters( 'mbr/password_perstag', FALSE );
		$passfield      = apply_filters( 'mbr/passfield', FALSE );
		$_POST['pass1'] = trim( $_POST['pass1'] );

		IF ( !empty( $_POST['pass1'] ) AND $_POST['pass1'] <> trim( $SESSION[$passfield] ) ) :
			$contact[$perstag]   = $_POST['pass1'];
			$SESSION[$passfield] = $_POST['pass1'];
		ENDIF;

		IF ( !empty( $contact ) ) :
			$contact['id'] = $contact_id;
			do_action( 'mbr/session/update' );
			apply_filters( 'mbr/save_contact_fields', [ 'id' => $contact_id, 'email' => $_POST['email'] ] );
		ENDIF;

		$_POST['email'] = lower_trim( $_POST['email'] );
		IF ( !empty( $_POST['email'] ) AND $_POST['email'] <> lower_trim( $SESSION['email'] ) ) :
			$user = get_user_by( 'ID', $user_id );
			IF ( apply_filters( 'mbr/save_contact_fields', [ 'id' => $contact_id, 'email' => $_POST['email'] ] ) ) :
				
				do_action( 'mbr/fetch_remote_contact/by_id', $contact_id );
				do_action( 'mbr/update_cookie', $user_id, $_POST['email'] . '|' . date( 'YmdHis' ) );
				do_action( 'mbr/delete_cookie', MBR_COOKIE_NAME );
				do_action( 'mbr/create_user_cookie', $SESSION['hash'] );
				IF ( is_email( $user->user_login ) ) :
					$GLOBALS['wpdb']->update( $GLOBALS['wpdb']->users, [ 'user_login' => $_POST['email'] ], [ 'ID' => $user_id ] );
					$GLOBALS['current_user']->user_login = $_POST['email'];
					wp_cache_delete( $user_id, 'users' );
					wp_clear_auth_cookie();
					wp_set_current_user( $user_id );
					wp_set_auth_cookie( $user_id, 'forever', is_ssl() );
				ENDIF;
			ENDIF;
		ENDIF;
	}

	public function bbp_get_content( $content = '', $post_id = 0 , $filler = FALSE )
	{
		$arr_search  = [ '[' . $this->sc_prefix . '_', '[/' . $this->sc_prefix . '_', '[ELSE_' ];
		$arr_replace = [ '{' . $this->sc_prefix . '_', '{/' . $this->sc_prefix . '_', '{ELSE_' ];

		IF ( bbp_is_topic( $post_id ) OR bbp_is_reply( $post_id ) ) :
			$content = str_ireplace( $arr_search, $arr_replace,  $content);
		ENDIF;

		return $content;
	}

	public function bbp_title( $arrTITLE )
	{
		global $post;

		$arr_search  = [ '[' . $this->sc_prefix . '_', '[/' . $this->sc_prefix . '_', '[ELSE_' ];
		$arr_replace = [ '{' . $this->sc_prefix . '_', '{/' . $this->sc_prefix . '_', '{ELSE_' ];

		IF ( bbp_is_topic( $post->ID ) OR bbp_is_reply( $post->ID ) ) :
			$arrTITLE['title'] = str_ireplace( $arr_search, $arr_replace,  $arrTITLE['title'] );
		ENDIF;

		return $arrTITLE;
	}

}
//$GLOBALS['mbr_bbpress'] = mbr_bbpress::getInstance();
