<?php
class mbr_buddypress
{
	use TRAIT__getInstance;

	public function _load()
	{
		add_action( 'init', [ $this, '_init' ] );
	}

	public function _init()
	{
		IF ( ! class_exists( 'BuddyPress' ) ) :
			return;
		ENDIF;

		global $blog_id;

		
		mbr_constant( 'MBR_BUDDYPRESS_GROUP_META_KEY',            '_mbr_access_' . $blog_id );

		
		add_action( 'mbr_authenticated_remote_login',             [ $this, 'bp_groups_join_leave_group' ], 1, 2 );

		IF ( ! is_admin() ) :
			add_filter( 'mbr/template_redirect/escape',             [ $this, 'buddypress_template_redirect_escape' ],   10, 1 );
			add_action( 'init',                                     [ $this, 'bp_groups_join_leave_group' ],            11, 1 );
			add_filter( 'bp_get_group_join_button',                 [ $this, 'bp_get_group_join_button' ],              10, 2 );
			add_filter( 'bp_group_status_message',                  [ $this, 'bp_group_status_message' ],               10, 2 );
			add_filter( 'bp_get_options_nav_request-membership',    [ $this, 'bp_get_options_nav_request_membership' ], 10, 3 );
			add_filter( 'bp_get_options_nav_general',               '__return_null',                                    10, 3 );
			add_filter( 'bp_get_options_nav_delete-account',        '__return_null',                                    10, 3 );
			add_filter( 'bp_current_action',                        [ $this, 'bp_current_action' ] );
		ELSE:
			add_action( 'bp_groups_admin_meta_boxes',               [ $this, 'buddypress_groups_admin_meta_boxes' ],     0, 1 );
			add_action( 'bp_group_admin_edit_after',                [ $this, 'buddypress_groups_update_group' ],       999, 1 );
		ENDIF;

		add_filter( 'mbr/get_meta_box_group_protection_settings', [ $this, 'get_meta_box_protection_settings' ],       10, 1 );

		
		add_action( 'bp_template_redirect',                       [ $this, 'bp_template_redirect_permissions' ],       0, 0 );
	}

	
	public function bp_template_redirect_permissions()
	{
		global $MBR_OPTIONS, $current_user;

		IF ( bp_is_group() ) :
			$group_id      = bp_get_current_group_id();
			$current_group = groups_get_group( [ 'group_id' => $group_id ] );
			IF ( $current_group->status <> 'private' ) :
				return;
			ENDIF;

			IF ( groups_is_user_member( $current_user->ID, $group_id ) ) :
				return;
			ENDIF;

			$meta = groups_get_groupmeta( $group_id, MBR_BUDDYPRESS_GROUP_META_KEY );
			IF ( (int) $meta['redirect_no_access'] > 0 ) :
				$redirect_url = get_permalink( $meta['redirect_no_access'] );
			ELSEIF ( (int) $meta['redirect_no_access'] == -1 ) :
				$redirect_url = $meta['redirect_no_access_url'];
			ELSEIF ( (int) $meta['redirect_no_access'] == -2 ) :
				$redirect_url = home_url();
			ELSE:
				$redirect_url = $MBR_OPTIONS['extensions']['mbr_ext_buddypress_redirect'];
			ENDIF;
			IF ( ! empty( $redirect_url ) ) :
				mbr_redirect( $redirect_url );
			ENDIF;
		ENDIF;

	}

	public function bp_current_action( $current_action )
	{
		IF ( in_array( $current_action, [ 'general', 'delete-account' ] ) ) :
			return 'notifications';
		ENDIF;

		return $current_action;
	}

	public function bp_get_options_nav_notifications( $link_html, $subnav_item, $selected_item )
	{
		IF ( $selected_item == 'general' ) :
			$list_type = bp_is_group() ? 'groups' : 'personal';
			return '<li id="' . esc_attr( $subnav_item->css_id . '-' . $list_type . '-li' ) . '" class="current selected"><a id="' . esc_attr( $subnav_item->css_id ) . '" href="' . esc_url( $subnav_item->link ) . '">' . $subnav_item->name . '</a></li>';
		ENDIF;

		return $link_html;
	}

	public function bp_get_options_nav_request_membership( $link_html, $subnav_item, $selected_item )
	{
		$group_id      = bp_get_current_group_id();
		$current_group = groups_get_group( [ 'group_id' => $group_id ] );

		$meta          = groups_get_groupmeta( $current_group->id, MBR_BUDDYPRESS_GROUP_META_KEY );
		$group_access  = array_filter( explode( ',', $meta['group_access'] ) );
		$tag_pos_list  = array_filter( explode( ',', $meta['tag_pos_list'] ) );
		$tag_neg_list  = array_filter( explode( ',', $meta['tag_neg_list'] ) );

		IF ( empty( $tag_neg_list ) AND empty( $tag_pos_list ) AND empty( $group_access ) ) :
			return $link_html;
		ENDIF;

		IF ( in_array( $current_group->status, [ 'private', 'hidden' ] ) ) :
			return;
		ENDIF;

		return $link_html;
	}

	public function bp_get_group_join_button( $button, $group )
	{
		$meta         = groups_get_groupmeta( $group->id, MBR_BUDDYPRESS_GROUP_META_KEY );
		$group_access = array_filter( explode( ',', $meta['group_access'] ) );
		$tag_pos_list = array_filter( explode( ',', $meta['tag_pos_list'] ) );
		$tag_neg_list = array_filter( explode( ',', $meta['tag_neg_list'] ) );

		IF ( empty( $tag_neg_list ) AND empty( $tag_pos_list ) AND empty( $group_access ) ) :
			return $button;
		ENDIF;

		IF ( $group->status == 'private' OR $group->status == 'hidden' ) :
			return;
		ENDIF;

		return $button;
	}

	public function bp_group_status_message( $message, $group )
	{
		IF ( empty( $group ) ) :
			return $message;
		ENDIF;

		$meta         = groups_get_groupmeta( $group->id, MBR_BUDDYPRESS_GROUP_META_KEY );
		$group_access = array_filter( explode( ',', $meta['group_access'] ) );
		$tag_pos_list = array_filter( explode( ',', $meta['tag_pos_list'] ) );
		$tag_neg_list = array_filter( explode( ',', $meta['tag_neg_list'] ) );

		IF ( empty( $tag_neg_list ) AND empty( $tag_pos_list ) AND empty( $group_access ) ) :
			return $message;
		ENDIF;

		return __( 'This is a private group and only registered site members can access it.', 'activemember360' );
	}

	public function bp_groups_join_leave_group( $user = FALSE, $contact = NULL )
	{
		global $current_user;

		IF ( !class_exists( 'BP_Groups_Group' ) ) :
			return;
		ENDIF;

		IF ( MBRADMIN ) :
			return;
		ENDIF;

		IF ( empty( $user ) ) :
			IF ( empty( $current_user->ID ) ) :
				return;
			ENDIF;
			$user = $current_user;
		ENDIF;

		IF ( empty( $contact['tag_ids'] ) ) :
			$contact = mbr_session();
			IF ( empty( $contact['tag_ids'] ) ) :
				return;
			ENDIF;
		ENDIF;

		$arr_contact_tags = array_map( 'trim', explode( ',', $contact['tag_ids'] ) );
		IF ( empty( $arr_contact_tags ) ) :
			return;
		ENDIF;

		$arr_bp_groups = BP_Groups_Group::get( [ 'show_hidden' => 1, 'type' => 'alphabetical', 'orderby' => 'name', 'order' => 'ASC' ] );
		IF ( empty( $arr_bp_groups['groups'] ) ) :
			return;
		ENDIF;

		FOREACH ( $arr_bp_groups['groups'] AS $group ) :

			$meta         = groups_get_groupmeta( $group->id, MBR_BUDDYPRESS_GROUP_META_KEY );
			$group_access = array_filter( explode( ',', $meta['group_access'] ) );
			$tag_pos_list = array_filter( explode( ',', $meta['tag_pos_list'] ) );
			$tag_neg_list = array_filter( explode( ',', $meta['tag_neg_list'] ) );

			IF ( empty( $tag_neg_list ) AND empty( $tag_pos_list ) AND empty( $group_access ) ) :
				continue;
			ENDIF;

			$join_group   = FALSE;

			IF ( empty( $tag_neg_list ) OR ! array_intersect( $tag_neg_list, $arr_contact_tags ) ) :
				IF ( empty( $tag_pos_list ) OR array_intersect( $tag_pos_list, $arr_contact_tags ) ) :
					IF ( empty( $group_access ) OR in_array( '-1', $group_access ) OR array_intersect( $group_access, $arr_contact_tags ) ) :
						$join_group  = TRUE;
					ENDIF;
				ENDIF;
			ENDIF;

			IF ( $join_group ) :
				groups_join_group( $group->id, $user->ID );
			ELSE:
				groups_leave_group( $group->id, $user->ID );
			ENDIF;

		ENDFOREACH;
	}

	public function buddypress_template_redirect_escape( $return = FALSE )
	{
		IF ( $return === TRUE OR bp_current_component() ) :
			return TRUE;
		ENDIF;

		return FALSE;
	}

	


	public function buddypress_groups_admin_meta_boxes()
	{
		



		do_action( 'mbr/meta_boxes_tag_list', NULL );

		add_meta_box(
			'bp_group_access_perms',
			apply_filters( 'mbr/meta_box_header', _x( 'BP Groups', 'wp_admin', 'activemember360' ) ),
			[ $this, 'render_meta_boxes_protection' ],
			get_current_screen()->id,
			'side',
			'high'
		);
	}

	public function render_meta_boxes_protection( $group, $box = NULL )
	{
		$settings = $this->get_meta_box_protection_settings( $group );

		do_action( 'mbr/render_metabox_css_helper', $box );

		IF ( empty( $settings['membership_levels'] ) ) :
			echo "<p style='color:" . apply_filters( 'mbr/plugin_color', NULL ) . ";font-size:10px;'>" . sprintf( _x( 'You have not yet defined which memberships you want to use on this site. Please do so in <a href="' . admin_url( "admin.php?page=ActiveMember360&mbrtab=memberships" ) . '">%s.</a>', 'wp_admin', 'activemember360' ), "'Memberships'" ) . "</p>";
			return;
		ENDIF;

		do_action( 'mbr/render_meta_boxes_protection_generic',      $group, $settings );
		do_action( 'mbr/render_meta_boxes_protection_memberships',  $group, $settings);
		do_action( 'mbr/render_meta_boxes_protection_addtl_tags',   $group, $settings);
		do_action( 'mbr/render_meta_boxes_protection_redirect',     $group, $settings);
	}

	public function get_meta_box_protection_settings( $group = '' )
	{
		
		IF ( empty( $group->id ) ) :
			return;
		ENDIF;

		
		$update_flag = FALSE;

		
		$settings                           = [];
		$settings['setGROUPS']              = [];
		$settings['tag_pos_list']           = '';
		$settings['tag_neg_list']           = '';
		$settings['redirect_no_access']     = 0;
		$settings['redirect_no_access_url'] = '';

		
		$meta = groups_get_groupmeta( $group->id, MBR_BUDDYPRESS_GROUP_META_KEY );

		IF ( isset( $meta['group_access'] ) ) :
			$settings['setGROUPS'] = array_filter( explode( ',', lower_trim( $meta['group_access'] ) ) );
		ELSE :
			$meta['group_access']  = '';
			$update_flag           = TRUE;
		ENDIF;

		IF ( isset( $meta['tag_pos_list'] ) ) :
			$settings['tag_pos_list'] = array_filter( explode( ',', $meta['tag_pos_list'] ) );
		ELSE :
			$meta['tag_pos_list']     = '';
			$update_flag              = TRUE;
		ENDIF;

		IF ( isset( $meta['tag_neg_list'] ) ) :
			$settings['tag_neg_list'] = array_filter( explode( ',', $meta['tag_neg_list'] ) );
		ELSE :
			$meta['tag_neg_list']     = '';
			$update_flag              = TRUE;
		ENDIF;

		$settings['redirect_no_access']     = (int) $meta['redirect_no_access'];
		$settings['redirect_no_access_url'] = $meta['redirect_no_access_url'];

		IF ( $settings['redirect_no_access'] == -1 AND empty( $settings['redirect_no_access_url'] ) ) :
			$settings['redirect_no_access']   = $meta['redirect_no_access'] = 0;
			$update_flag                      = TRUE;
		ENDIF;

		IF ( $settings['redirect_no_access'] >= 0 ) :
			$settings['redirect_no_access_url'] = $meta['redirect_no_access_url'] = '';
			$update_flag                        = TRUE;
		ENDIF;

		IF ( $update_flag ) :
			groups_update_groupmeta( $group->id, MBR_BUDDYPRESS_GROUP_META_KEY, $meta );
		ENDIF;

		IF ( ! empty( $settings['setGROUPS'] ) OR ! empty( $settings['tag_pos_list'] ) OR ! empty( $settings['tag_neg_list'] ) ) :
			$current_group = groups_get_group( [ 'group_id' => $group->id ] );
			IF ( $current_group->status == 'public' ) :
				groups_create_group( [ 'group_id' => $group->id, 'status' => 'private' ] );
			ENDIF;
		ENDIF;

		$settings['plugcolor']         = apply_filters( 'mbr/plugin_color',               NULL );
		$settings['membership_levels'] = apply_filters( 'mbr/site_membership_levels/get', NULL );
		$settings['site_tags']         = apply_filters( 'mbr/site_tags/get',              NULL );
		$settings['site_pages']        = apply_filters( 'mbr/pages/get',                  NULL );

		return $settings;
	}

	public function buddypress_groups_update_group( $group_id )
	{
		$set_group_to_private     = FALSE;

		$meta['group_access']     = '';
		IF ( ! empty( $_POST['group_access'] ) ) :
			IF ( in_array(-1, $_POST['group_access'] ) ) :
				$meta['group_access'] = '-1';
			ELSE:
				$meta['group_access'] = implode( ',', $_POST['group_access'] );
			ENDIF;
		ENDIF;

		IF ( ! empty( $meta['group_access'] ) ) :
			$set_group_to_private = TRUE;
		ENDIF;

		$tag_pos_list         = [];
		$tag_neg_list         = [];
		IF ( ! empty( $_POST['tag_list'] ) ) :
			FOREACH ( $_POST['tag_list'] AS $key => $tag_id ) :
				IF ( substr( $tag_id,0,1 ) == '-' ) :
					$tag_neg_list[] = absint( $tag_id );
				ELSE:
					$tag_pos_list[] = $tag_id;
				ENDIF;
			ENDFOREACH;
		ENDIF;

		$meta['tag_pos_list']   = '';
		IF ( ! empty( $tag_pos_list ) ) :
			$meta['tag_pos_list'] = implode( ',', $tag_pos_list );
			$set_group_to_private = TRUE;
		ENDIF;

		$meta['tag_neg_list']   = '';
		IF ( ! empty( $tag_neg_list ) ) :
			$meta['tag_neg_list'] = implode( ',', $tag_neg_list );
			$set_group_to_private = TRUE;
		ENDIF;

		$meta['redirect_no_access']     = (int) $_POST['forbidden_redirect'];
		$meta['redirect_no_access_url'] = $_POST['forbidden_redirect_url'];

		IF ( $meta['redirect_no_access'] == -1 AND empty( $meta['redirect_no_access_url'] ) ) :
			$meta['redirect_no_access']   = 0;
		ENDIF;

		IF ( $meta['redirect_no_access'] >= 0 ) :
			$meta['redirect_no_access_url'] = '';
		ENDIF;

		IF ( $set_group_to_private ) :
			$current_group = groups_get_group( [ 'group_id' => $group_id ] );
			IF ($current_group->status == 'public') :
				groups_create_group( [ 'group_id' => $group_id, 'status' => 'private' ] );
			ENDIF;
		ENDIF;

		groups_update_groupmeta( $group_id, MBR_BUDDYPRESS_GROUP_META_KEY, $meta );
	}

}
mbr_buddypress::getInstance();
