<?php
final class mbr_bulk_editor EXTENDS mbr_redux_base_class
{
	use TRAIT__getInstance;

	public $opt_name  = 'mbr_bulk_editor';
	public $page_slug = 'mbr360-bulk-editor';

	public function _load()
	{
		IF ( is_admin() ) :
			add_action( 'plugins_loaded',                                           [ $this, 'init_admin_panel' ], 99999 );
			add_action( 'plugins_loaded',                                           [ $this, 'init_sections' ],    99999 );
			add_filter( "redux/{$this->opt_name}/panel/template/header.tpl.php",    [ $this, 'alternate_header' ] );
			add_filter( "redux/{$this->opt_name}/panel/template/container.tpl.php", [ $this, 'alternate_container' ] );
			add_action( "redux/options/{$this->opt_name}/validate",                 [ $this, 'validate_save_settings' ] );
			add_action( 'wp_ajax_mbr_be_init_bulk_editor',                          [ $this, 'ajax_init_bulk_editor' ] );
			add_action( 'wp_ajax_mbr_be_bulk_edit_posts',                           [ $this, 'ajax_bulk_edit_posts' ] );

			IF ( $arrTYPES = get_option( 'site_bulk_edit_types', [] ) AND is_array( $arrTYPES ) ) :
				$this->arr_enabled_bulk_edit_post_types = $arrTYPES;
			ELSE :
				$this->arr_enabled_bulk_edit_post_types = [ 'page', 'post', 'attachment' ];
				update_option('site_bulk_edit_types', $this->arr_enabled_bulk_edit_post_types);
			ENDIF;
		ENDIF;
	}

	public function alternate_container()
	{
		return MBR_DIR . '/templates/container_bulk_editor.tpl.php';
	}

	public function init_admin_panel()
	{
		global $MBR_REDUX_HEADER_MODULE_NAMES;

		$MBR_REDUX_HEADER_MODULE_NAMES[ $this->opt_name ] = $admin_title = _x( 'Bulk Editor', 'wp_admin', 'activemember360' );

		$page_title         = apply_filters( 'mbr/plugin_name', '' ) . ' ' . $admin_title;

		$arr_config         =  [
			'opt_name'        => $this->opt_name,
			'global_variable' => $this->opt_name,
			'page_slug'       => $this->page_slug,
			'page_parent'     => MBR_MENU_PARENT_PAGE,
			'page_title'      => $page_title,
			'display_name'    => $page_title,
			'admin_title'     => $admin_title,
			'menu_title'      => $admin_title,
		];

		$arr_config         = array_merge( $this->arr_default_sub_admin_panel_settings, $arr_config );

		Redux::setArgs( $this->opt_name, $arr_config );
	}

	public function init_sections()
	{
		global $pagenow;

		IF ( $pagenow <> 'options.php' AND $pagenow <> 'admin.php' OR ( $pagenow == 'admin.php' AND ( empty($_GET['page'] ) OR $_GET['page'] <> $this->page_slug ) ) ) :
			return;
		ENDIF;

		$arr_section_be_post_types =
		[
			'id'             => 'section_bulk_editor',
			'title'          => _x( 'Bulk Editor Post Types', 'wp_admin', 'activemember360' ),
			'desc'           => _x( 'To enable additional post types to be "Bulk Edited", please "drag and drop" the desired post types into the "Enabled" box, as needed.', 'wp_admin', 'activemember360' ),
			'icon'           => 'el el-cogs',
			'class'          => 'mbr_be_section_bulk_enable_post_type',
			'fields'         => [
				[
					'id'         => 'mbr_be_post_types_enabled',
					'title'      => _x( 'Enable Post Types', 'wp_admin', 'activemember360' ),
					'type'       => 'sorter',
					'data'       => 'callback',
					'args'       => [ 'mbr_be_post_type_support' ]
				]
			]
		];

		
		Redux::setSection( $this->opt_name, $arr_section_be_post_types );

		$arr_protectable_post_types        = apply_filters( 'mbr/protectable_post_types/all', [] );

		IF ( ! empty( $arr_protectable_post_types ) ):
			FOREACH ( $arr_protectable_post_types AS $key => $post_type ) :

				
				IF ( ! in_array( $post_type, $this->arr_enabled_bulk_edit_post_types ) ) :
					continue;
				ENDIF;

				$script = "<script>
				jQuery( 'li.mbr_" . $post_type . " a'  ).click( function () {
					jQuery( '.mbr_be_post_type_edit_container' ).html('');
					mbr_send_be_ajax( 'mbr_be_init_bulk_editor', '" . $post_type . "', '#mbr_be_post_type_edit_container_" . $post_type . "', false, false );
					return false;
				} );
				</script>";

				$arr_section_be_edit_post_type =
				[
					'id'              => 'section_bulk_edit_post_type_' . $post_type,
					'title'           => sprintf( _x( 'Edit %s', 'wp_admin', 'activemember360' ), ucfirst( $post_type ) ),
					'desc'            => _x( 'To apply changes in bulk to your lessons, please select the desired membership from the pull-down selection as well as the desired action. When you click on "Apply", the chosen action will be applied to all the lessons selected below (leftmost checkboxes, "Sel" column). The remaining checkboxes in the individual lesson rows will also be processed. Processing priority will be given to individual settings following by the bulk settings.', 'wp_admin', 'activemember360' ),
					'icon'            => 'el el-cog',
					'class'           => 'mbr_be_section_bulk_edit_post_type mbr_' . $post_type,
					'fields'          => [
						[
							'id'          => 'mbr_be_post_type_edit_' . $post_type,
							'type'        => 'raw',
							'content'     => $script . '<div id="mbr_be_post_type_edit_container_' . $post_type . '" class="mbr_be_post_type_edit_container"></div>'
						]
					]
				];

				Redux::setSection( $this->opt_name, $arr_section_be_edit_post_type );
			ENDFOREACH;
		ENDIF;
	}

	public function ajax_bulk_edit_posts()
	{
		global $wpdb;
		//echo  mbr_admin_msg( _x( "Permissions saved successfully.", 'wp_admin', 'activemember360' ), FALSE, TRUE);
		IF ( ! empty( $_POST['mbr_be_edit_data'] ) ) :
			$arr_bulk_update = json_decode( stripslashes( $_POST['mbr_be_edit_data'] ), TRUE );

			IF ( empty( $arr_bulk_update ) OR ! is_array( $arr_bulk_update ) ) :
				echo  mbr_admin_msg( _x( 'Corrupted data (1). No updated attempt was tried.', 'wp_admin', 'activemember360' ), TRUE, TRUE);
				exit;
			ENDIF;

			$arr_posts = $arr_bulk_update['posts'];

			IF ( empty( $arr_posts ) OR ! is_array( $arr_posts ) ) :
				echo  mbr_admin_msg( _x( 'Corrupted data (2). No updated attempt was tried.', 'wp_admin', 'activemember360' ), TRUE, TRUE);
				exit;
			ENDIF;

			$arr_bulk_posts = [];
			$postmeta_permissions_key = apply_filters( 'mbr/permissions/postmeta/key', NULL );

			FOREACH ( $arr_posts AS $this_post_id => $arr_permissions ) :

				
				$ie = $arr_permissions['mbr_cb_ie'];
				
				$hm = $arr_permissions['mbr_cb_hm'];
				
				$po = $arr_permissions['mbr_cb_po'];

				
				$tp = '';
				$tn = '';
				IF ( ! empty( $arr_permissions['mbr_chg_tags'] ) AND is_array( $arr_permissions['mbr_chg_tags'] ) ) :
					$arr_tag_post_list       = [];
					$arr_tag_neg_list        = [];
					FOREACH ( $arr_permissions['mbr_chg_tags'] AS $key => $tag_id ) :
						$tag_id = trim( $tag_id );
						IF ( substr( $tag_id,0,1 ) == '-' ) :
							$tag_id              = substr( $tag_id,1 );
							$arr_tag_neg_list[]  = $tag_id;
						ELSE:
							$arr_tag_post_list[] = $tag_id;
						ENDIF;
					ENDFOREACH;
					IF ( count( $arr_tag_post_list ) ) :
						$tmp = implode( ',', $arr_tag_post_list );
						$tp  = str_ireplace( ' ', '', trim( $tmp ) );
					ENDIF;
					IF ( count( $arr_tag_neg_list ) ) :
						$tmp = implode( ',', $arr_tag_neg_list );
						$tn  = str_ireplace( ' ', '', trim( $tmp ) );
					ENDIF;
				ENDIF;

				$au = $arr_permissions['mbr_chg_automation'];

				IF ( array_key_exists( -3, $arr_permissions['mbr_cb_grp'] ) AND ! empty( $arr_permissions['mbr_cb_grp'][-3] ) ) :
					$grp = '-3';
					$ie  = 0;
					$po  = 0;
					$tp  = '';
					$tn  = '';
				ELSEIF ( array_key_exists( -2, $arr_permissions['mbr_cb_grp'] ) AND ! empty( $arr_permissions['mbr_cb_grp'][-2] ) ) :
					$grp = '-2';
					$ie  = 0;
					$po  = 0;
					$tp  = '';
					$tn  = '';
				ELSEIF ( array_key_exists( -1, $arr_permissions['mbr_cb_grp'] ) AND ! empty( $arr_permissions['mbr_cb_grp'][-1] ) ) :
					$grp = '-1';
					$po  = 0;
				ELSEIF ( ! empty( $arr_permissions['mbr_cb_hc'] ) ) :
					$grp = 0;
					$ie  = 0;
					$po  = 0;
					$tp  = '';
					$tn  = '';
					$au  = '';
				ELSE :
					$arr_groups = [];
					FOREACH ( $arr_permissions['mbr_cb_grp'] AS $group => $group_status ) :
						IF ( ! empty( $group_status ) ) :
							$arr_groups[] = $group;
						ENDIF;
					ENDFOREACH;
					$grp = implode( ',', $arr_groups );
					$grp = str_ireplace( ' ', '', $grp );
				ENDIF;

				$arr_current                             = get_post_meta( $this_post_id, $postmeta_permissions_key, TRUE );
				$arr_current                             = json_decode( $arr_current, TRUE );
				$arr_update_post_meta                    = [];
				$arr_update_post_meta['more']            = $arr_current['more'] ? $arr_current['more'] : 0;
				$arr_update_post_meta['hide_menu']       = $hm;
				$arr_update_post_meta['ignore_excerpt']  = $ie;
				$arr_update_post_meta['public_only']     = $po;
				$arr_update_post_meta['group_access']    = $grp;
				$arr_update_post_meta['tag_pos_list']    = $tp;
				$arr_update_post_meta['tag_neg_list']    = $tn;
				$arr_update_post_meta['automation_list'] = $au;
				update_post_meta( $this_post_id, $postmeta_permissions_key, json_encode( $arr_update_post_meta ) );

				IF ( ! empty( $arr_permissions['mbr_be_bulk_action'] ) ) :
					$arr_bulk_posts[] = $this_post_id;
				ENDIF;
			ENDFOREACH;

			IF ( $arr_bulk_update['bulk_actions']['memberships'] <> '0' AND $arr_bulk_update['bulk_actions']['action'] <> '0') :
				IF ( ! empty( $arr_bulk_posts ) ) :
					$grp_id                 = $arr_bulk_update['bulk_actions']['memberships'];
					$post_list              = implode( ',', $arr_bulk_posts );

					$arr_posts_to_bulk_edit = $wpdb->get_results( "SELECT post_id, meta_value FROM {$wpdb->postmeta} WHERE FIND_IN_SET(post_id, '{$post_list}') AND meta_key = '{$postmeta_permissions_key}'" );

					IF ( $arr_bulk_update['bulk_actions']['action'] == 'add' ) :
						FOREACH ( $arr_posts_to_bulk_edit AS $key =>  $obj ) :
							$permissions  = json_decode( $obj->meta_value);
							IF ( trim( $permissions->group_access ) == '' ) :
								$group_access     = $grp_id;
							ELSE :
								IF ( $arr_tmp = array_filter( explode(',', $permissions->group_access ) ) ) :
									IF ( ! in_array( $grp_id, $arr_tmp ) ) :
										$arr_tmp[]    = $grp_id;
									ENDIF;
									$group_access   = implode( ',', $arr_tmp );
								ELSE :
									$group_access   = '';
								ENDIF;
							ENDIF;
							$arr_update_post_meta = (array) $permissions;
							$arr_update_post_meta['group_access'] = $group_access;
							update_post_meta( $obj->post_id, $postmeta_permissions_key, json_encode( $arr_update_post_meta ) );
						ENDFOREACH;
					ELSEIF ( $arr_bulk_update['bulk_actions']['action'] == 'rem' ) :
						FOREACH ( $arr_posts_to_bulk_edit AS $key => $obj ) :
							$permissions  = json_decode( $obj->meta_value);
							IF ( trim( $permissions->group_access ) == '' ) :
								$group_access = '';
							ELSE :
								$group_access = $permissions->group_access;
								IF ($arr_tmp = array_filter( explode( ',', $permissions->group_access ) ) ) :
									IF ( in_array( $grp_id, $arr_tmp ) ) :
										FOREACH ( $arr_tmp AS $tmp_key => $tmp_value ) :
											IF ( $tmp_value == $grp_id ) :
												IF ( isset( $arr_tmp[ $tmp_key ] ) ) :
													unset( $arr_tmp[ $tmp_key ] );
												ENDIF;
											ENDIF;
										ENDFOREACH;
										$group_access = implode( ',', $arr_tmp );
									ENDIF;
								ENDIF;
							ENDIF;
							$arr_update_post_meta = (array) $permissions;
							$arr_update_post_meta['group_access'] = $group_access;
							update_post_meta( $obj->post_id, $postmeta_permissions_key, json_encode( $arr_update_post_meta ) );
						ENDFOREACH;
					ENDIF;
				ENDIF;
			ENDIF;

			//delete_transient( $postmeta_permissions_key );
			do_action( 'mbr/permissions/load' );

			echo  mbr_admin_msg( _x( 'Permissions saved successfully.', 'wp_admin', 'activemember360' ), FALSE, TRUE );

		ENDIF;
		exit;
	}

	public function ajax_init_bulk_editor()
	{
		global $MBR_OPTIONS, $wpdb;

		IF ( empty( $_POST['mbr_be_post_type'] ) ) :
			echo _x( 'Error: No post type to bulk editor transferred.', 'wp_admin', 'activemember360' );
			exit;
		ENDIF;

		$post_type = $_POST['mbr_be_post_type'];

		$bbforum   = apply_filters( 'bbp_forum_post_type', 'forum' );

		IF ($post_type == $bbforum) :
			$arr_posts = $wpdb->get_results( "SELECT ID, post_type, post_title FROM {$wpdb->posts} WHERE post_type='{$post_type}' AND post_parent='0' AND post_status='publish' ORDER BY post_title, menu_order" );
		ELSE :
			$arr_posts = $wpdb->get_results( "SELECT ID, post_type, post_title FROM {$wpdb->posts} WHERE post_type='{$post_type}' AND post_status='publish' ORDER BY post_title, menu_order" );
		ENDIF;

		IF ( empty( $arr_posts ) ) :
			echo _x( 'Nothing found.', 'wp_admin', 'activemember360' );
			exit;
		ENDIF;

		do_action( 'mbr/site_tags/fetch' );
		$arr_tags          = apply_filters( 'mbr/site_tags/get', NULL );

		$supports_comments = post_type_supports( $post_type, 'comments' );

		$out          = '<script>';

		$out         .= "function mbr_send_be_edit_ajax( action, data, response_container ) {
											 jQuery( response_container ).html( \"<div style='width:100%;height:41px;padding-top:5px;padding-bottom:5px;background-color:#f3f3f3;border:solid 1px #ccc;text-align:center;margin-bottom:10px;'><img src='" . plugin_dir_url( __FILE__ ) . 'img/ajax-loader.gif' . "' style='margin-top:5px' /></div>\" );
											 jQuery.post(
												 ajaxurl, {
													 'action'           : action,
													 'mbr_be_edit_data' : JSON.stringify( data )
												 },
												 function( response ){
													 mbr_send_be_ajax( 'mbr_be_init_bulk_editor', '" . $post_type . "', '#mbr_be_post_type_edit_container_" . $post_type . "', response_container, response );
													 jQuery( '#mbr_be_post_type_edit_inner_container_" . $post_type . "' ).show();
												 }
											 );
										 }";

		$out         .= "jQuery( document ).ready( function() {
											 jQuery( '.mbr_be_save_permissions'  ).click( function () {
												 jQuery( '#mbr_be_post_type_edit_inner_container_" . $post_type . "' ).hide();
												 arr_mbr_be_inputs = {};
												 arr_mbr_be_inputs[ 'bulk_actions' ] = {};
												 arr_mbr_be_inputs[ 'bulk_actions' ][ 'memberships' ] = jQuery( '#mbr_be_bulk_memberships' ).val();
												 arr_mbr_be_inputs[ 'bulk_actions' ][ 'action' ]      = jQuery( '#mbr_be_bulk_memberships_action' ).val();
												 
												 arr_mbr_be_inputs[ 'posts' ] = {};
												 jQuery( '.mbr_be_post_id' ).each( function() {
													 arr_mbr_be_inputs[ 'posts' ][ jQuery( this ).val() ] = {};
													 arr_mbr_be_inputs[ 'posts' ][ jQuery( this ).val() ][ 'mbr_cb_grp' ]  = {};
													 arr_mbr_be_inputs[ 'posts' ][ jQuery( this ).val() ][ 'mbr_chg_tag' ] = {};
												 } );
												 
												 jQuery( '.mbr_be_checkbox' ).each( function() {
													 this_post_id    = jQuery( this ).data( 'post-id' );
													 this_field_name = jQuery( this ).attr( 'name' );
													 
													 if ( this_field_name == 'mbr_cb_grp' ) {
														 this_tag_id   = jQuery( this ).data( 'tag-id' );
														 arr_mbr_be_inputs[ 'posts' ][ this_post_id ][ this_field_name ][ this_tag_id  ] = jQuery( this ).is( ':checked' );
													 } else {
														 arr_mbr_be_inputs[ 'posts' ][ this_post_id ][ this_field_name ]                 = jQuery( this ).is( ':checked' );
													 }  
												 } );
												 
												 jQuery( 'select.mbr_be_input' ).each( function() {
													 this_post_id    = jQuery( this ).data( 'post-id' );
													 this_field_name = jQuery( this ).attr( 'name' );
													 arr_mbr_be_inputs[ 'posts' ][ this_post_id ][ this_field_name ] = jQuery( this ).val();
												 } );
												 
												 mbr_send_be_edit_ajax( 'mbr_be_bulk_edit_posts', arr_mbr_be_inputs, '#mbr_be_bulk_edit_save_container_" . $post_type . "' );
											 } ); 
										 } );";

		$out         .=  "jQuery( function() {
												jQuery( '.mbr_be_select_all').click( function () { 
													jQuery( '.mbr_be_case' ).attr( 'checked', this.checked );
													jQuery( '.mbr_be_select_all' ).attr( 'checked', this.checked );
												} );
												
												jQuery( '.mbr_be_case' ).click( function() { 
													if ( jQuery('.mbr_be_case' ).length == jQuery( '.mbr_be_case:checked' ).length ) {
														jQuery( '.mbr_be_select_all' ).attr( 'checked', 'checked' );
													} else {
														jQuery( '.mbr_be_select_all' ).removeAttr( 'checked' );
													}
												} );
											} );";
		$out         .= '</script>';

		$out         .= '<div id="mbr_be_bulk_edit_save_container_' . $post_type . '"></div>';

		$out         .= '<div id="mbr_be_post_type_edit_inner_container_' . $post_type . '">';
		$out         .= '<p><span class="mbr_be_save_permissions button button-primary">' . _x( 'Save Permissions', 'wp_admin', 'activemember360' ) . '</span></p><br />';


		$out         .= '<select id="mbr_be_bulk_memberships" name="mbr_be_bulk_memberships">';
		$out         .= "<option value='0'>" . _x( 'Select Membership', 'wp_admin', 'activemember360' ) . "</option>";
		IF ( is_post_type_hierarchical( $post_type ) ) :
			$out       .= "<option value='-3'>" . _x( 'Inherit from parent', 'wp_admin', 'activemember360' ) . "</option>";
		ENDIF;
		$out         .= "<option value='-2'>" . _x( 'Anyone at any time', 'wp_admin', 'activemember360' ) . "</option>";
		$out         .= "<option value='-1'>" . _x( 'Any logged in user', 'wp_admin', 'activemember360' ) . "</option>";
		FOREACH ( $MBR_OPTIONS['membership_levels'] AS $grp_id => $this_data ) :
			IF ( trim( $MBR_OPTIONS['membership_levels'][ $grp_id ]['name'] ) == '' ) :
				$this_group_name = $arr_tags[ $grp_id ];
			ELSE :
				$this_group_name = trim( $MBR_OPTIONS['membership_levels'][ $grp_id ]['name'] );
			ENDIF;
			$out       .= "<option value='$grp_id'>" . $this_group_name . "&nbsp;($grp_id)</option>";
		ENDFOREACH;
		$out         .= '</select>';

		$out         .= '<select id="mbr_be_bulk_memberships_action" name="mbr_be_bulk_memberships_action">';
		$out         .= "<option value='0'>"   . _x( 'Bulk actions',         'wp_admin', 'activemember360' ) . "</option>";
		$out         .= "<option value='add'>" . _x( 'Add to selected',      'wp_admin', 'activemember360' ) . "</option>";
		$out         .= "<option value='rem'>" . _x( 'Remove from selected', 'wp_admin', 'activemember360' ) . "</option>";
		$out         .= '</select>';

		$out         .= '<table class="wp-list-table widefat striped mbr-redux-listing">';
		$theadfoot    = '<tr>';
		$theadfoot   .= '<td><input type="checkbox" class="mbr_be_select_all" style="margin:0;" /></td>';
		$theadfoot   .= '<td>' . _x( 'Page Title (ID)',     'wp_admin', 'activemember360' ) . '</td>';
		$theadfoot   .= '<td>' . _x( 'Memberships (ID)',    'wp_admin', 'activemember360' ) . '</td>';
		$theadfoot   .= '<td>' . _x( 'Only/Not these Tags', 'wp_admin', 'activemember360' ) . '</td>';

		IF ( $post_type <> 'sfwd-lessons' AND 'attachment' <> $post_type AND $supports_comments ) :
			$theadfoot .= '<td>' . _x( 'Run automation on comments', 'wp_admin', 'activemember360' ) . '</td>';
		ENDIF;

		IF ( $post_type <> $bbforum ) :
			$theadfoot .= '<td>' . _x( 'Hide Compl.',         'wp_admin', 'activemember360' ) . '</td>';
			$theadfoot .= '<td>' . _x( 'Hide Menu',           'wp_admin', 'activemember360' ) . '</td>';
		ENDIF;

		$theadfoot   .= '<td>' . _x( 'Visitors Only',       'wp_admin', 'activemember360' ) . '</td>';
		$theadfoot   .= '<td>' . _x( 'Excerpt w/ Perm.',    'wp_admin', 'activemember360' ) . '</td>';

		IF ( $post_type <> $bbforum ) :
			$theadfoot .= '<td>' . _x( 'Has Excerpt',         'wp_admin', 'activemember360' ) . '</td>';
			$theadfoot .= '<td>' . _x( 'Uses "more"',         'wp_admin', 'activemember360' ) . '</td>';
		ENDIF;

		$theadfoot   .= '</tr>';

		$out         .= '<thead>' . $theadfoot . '</thead>';
		$out         .= '<tfoot>' . $theadfoot . '</tfoot>';
		$out         .= '<tbody id="the-list">';

		IF ( ! empty( $arr_posts ) ) :
			do_action( 'mbr/site_automations/fetch' );
			$arr_automations        = apply_filters( 'mbr/site_automations/get', NULL );
			$arr_automations_status = apply_filters( 'mbr/site_automations/get/status', NULL );

			FOREACH ( $arr_posts AS $tmp_id => $this_post ) :
				$perms = apply_filters( 'mbr/permissions/get', NULL, $this_post->ID );
				$out .= '<input type="hidden" class="mbr_be_post_id" value="' . $this_post->ID . '" />';
				IF ( trim( $perms->group_access ) <> '' ) :
					$arr_group_access = explode( ',', $perms->group_access );
				ELSE :
					$arr_group_access = [];
				ENDIF;

				IF ( isset( $perms->tag_pos_list ) ) :
					$tag_pos_list = explode( ',', $perms->tag_pos_list );
				ELSE :
					$tag_pos_list ='';
				ENDIF;

				IF ( isset( $perms->tag_neg_list ) ) :
					$tag_neg_list = explode( ',', $perms->tag_neg_list );
				ELSE :
					$tag_neg_list = '';
				ENDIF;

				$out          .= '<tr><td><input class="mbr_be_checkbox mbr_be_case" data-post-id="' . $this_post->ID . '" type="checkbox" name="mbr_be_bulk_action" /></td>';
				$out          .= '<td>' . $this_post->post_title . ' (' . $this_post->ID. ')</td>';
				$out          .= '<td>';

				$checked       = in_array( -3, $arr_group_access ) ? "checked='checked'" : '';
				IF ( is_post_type_hierarchical( $post_type ) ) :
					$out        .= "<input class='mbr_be_checkbox' data-post-id='" . $this_post->ID. "' data-tag-id='-3' type='checkbox' name='mbr_cb_grp' $checked />" . _x( 'Inherit from parent', 'wp_admin', 'activemember360' ) . "<br />";
				ENDIF;

				$checked       = in_array( -2, $arr_group_access ) ? "checked='checked'" : '';
				$out          .= "<input class='mbr_be_checkbox' data-post-id='" . $this_post->ID. "' data-tag-id='-2' type='checkbox' name='mbr_cb_grp' $checked />" . _x( 'Anyone at any time', 'wp_admin', 'activemember360' );

				$checked       = in_array( -1, $arr_group_access ) ? "checked='checked'" : '';
				$out          .= "<br /><input class='mbr_be_checkbox' data-post-id='" . $this_post->ID. "' data-tag-id='-1' type='checkbox' name='mbr_cb_grp' $checked />" . _x( 'Any logged in user', 'wp_admin', 'activemember360' );

				IF ( ! empty( $MBR_OPTIONS['membership_levels'] ) ) :
					FOREACH ( $MBR_OPTIONS['membership_levels'] AS $this_tag_id => $this_level ) :
						IF ( in_array( $this_tag_id, $arr_group_access ) ) :
							$checked = "checked='checked'";
						ELSE :
							$checked = '';
						ENDIF;
						IF ( trim( $this_level['name'] ) == '' ) :
							$this_level['name'] = $arr_tags[ $this_tag_id ];
						ENDIF;
							$out      .= "<br /><input class='mbr_be_checkbox' data-post-id='" . $this_post->ID. "' data-tag-id='" . $this_tag_id . "' type='checkbox' name='mbr_cb_grp' $checked />" . $this_level['name'] . " ({$this_tag_id})";
					ENDFOREACH;
				ENDIF;
				$out          .= '</td>';

				$out          .= '<td><select multiple="multiple" data-post-id="' . $this_post->ID . '" name="mbr_chg_tags" class="mbr-select2 mbr_be_input">';
				FOREACH ( $arr_tags AS $tid => $tname ) :
					$selected    = ( in_array( $tid, $tag_pos_list ) ) ? 'selected="selected"' : '';
					$out        .= "<option value='{$tid}' $selected>$tid - $tname</option>";
				ENDFOREACH;

				FOREACH ( $arr_tags AS $tid => $tname ) :
					$selected    = ( in_array( $tid, $tag_neg_list ) ) ? 'selected="selected"' : '';
					$out        .= "<option value='-{$tid}' $selected>not $tid - $tname</option>";
				ENDFOREACH;
				$out          .= '</select></td>';

				IF ( $post_type <> 'sfwd-lessons' AND 'attachment' <> $post_type AND $supports_comments ) :
					IF ( 'open' == $this_post->comment_status ) :
						$out      .= "<td><select data-post-id='" . $this_post->ID . "' class='mbr_be_input' name='mbr_chg_automation' style='width:100%;'>";
						$out      .= '<option value="">' . _x( 'Do not run an automation', 'wp_admin', 'activemember360' ) . '</option>';
						FOREACH ( $arr_automations AS $automationid => $automationname ) :
							$status  = $arr_automations_status[ $automationid ] <> 1 ? 'disabled="disabled"' : '';
							$out    .= "<option value='$automationid' " . selected( $this_post->automation_list, $automationid, FALSE ) . " " . $status . ">$automationname</option>";
						ENDFOREACH;
						$out      .= "</select></td>";
					ELSE:
						$out      .= "<td>" . sprintf( _x( 'Disabled for this %s', 'wp_admin', 'activemember360'), $post_type ) . "</td>";
					ENDIF;
				ENDIF;

				IF ( $post_type <> $bbforum ) :
					$out        .= '<td><input class="mbr_be_checkbox" data-post-id="' . $this_post->ID . '" type="checkbox" name="mbr_cb_hc" ' . ( in_array( '0', $arr_group_access ) ? "checked='checked'" : '' ) . ' /></td>';
					$out        .= '<td><input class="mbr_be_checkbox" data-post-id="' . $this_post->ID . '" type="checkbox" name="mbr_cb_hm" ' . ( $perms->hide_menu == 1               ? "checked='checked'" : '' ) . ' /></td>';
				ENDIF;

				$out          .= '<td><input class="mbr_be_checkbox" data-post-id="' . $this_post->ID . '" type="checkbox" name="mbr_cb_po" ' . ( $perms->public_only    == 1 ? "checked='checked'" : '' ) . ' /></td>';
				$out          .= '<td><input class="mbr_be_checkbox" data-post-id="' . $this_post->ID . '" type="checkbox" name="mbr_cb_ie" ' . ( $perms->ignore_excerpt == 1 ? "checked='checked'" : '' ) . ' /></td>';

				IF ( $post_type <> $bbforum ) :
					$out        .= '<td>' . ( $perms->post_excerpt ? "<strong><img src='" . MBRWPURL . "/wp-admin/images/yes.png' border='0' /></strong>" : '' ) . '</td>';
					$out        .= '<td>' . ( $perms->more         ? "<strong><img src='" . MBRWPURL . "/wp-admin/images/yes.png' border='0' /></strong>" : '' ) . '</td>';
				ENDIF;

				$out          .= '</tr>';
			ENDFOREACH;
		ELSE:
			$out            .= '<tr><td colspan="11">' . _x( 'Nothing found.', 'wp_admin', 'activemember360' ) . '</td></tr>';
		ENDIF;

		$out              .= '</tbody>';
		$out              .= '</table>';
		$out              .= "<script type='text/javascript'>jQuery( '.mbr-select2' ).select2( { width: '100%' } );</script>";

		$out              .= '<br /><p><span class="mbr_be_save_permissions button button-primary">' . _x( 'Save Permissions', 'wp_admin', 'activemember360' ) . '</span></p>';
		$out              .= '</div>';

		echo $out;
		exit;
	}

	public function validate_save_settings( &$arr_settings )
	{
		IF ( ! empty( $arr_settings ) ) :
			$arr_enabled_post_types     = [];
			IF ( ! empty( $arr_settings['mbr_be_post_types_enabled']['Enabled'] ) ) :
				FOREACH ( $arr_settings['mbr_be_post_types_enabled']['Enabled'] AS $post_type => $label ) :
					IF ( $post_type == 'placebo' ) :
						continue;
					ENDIF;
					$arr_enabled_post_types[] = $post_type;
				ENDFOREACH;
			ENDIF;

			$arr_default_post_types = apply_filters( 'mbr/post_types/default', [] );
			$arr_settings['mbr_be_post_types_enabled']['Enabled'] = array_unique( array_merge( [ 'placebo' ], $arr_default_post_types,  $arr_enabled_post_types ) );

			$arr_disabled_post_types = [];
			IF ( ! empty( $arr_settings['mbr_be_post_types_enabled']['Disabled'] ) ) :
				FOREACH ( $arr_settings['mbr_be_post_types_enabled']['Disabled'] AS $post_type => $label ) :
					IF ( $post_type == 'placebo' OR in_array( $post_type, $arr_enabled_post_types ) ) :
						continue;
					ENDIF;
					$arr_disabled_post_types[] = $post_type;
				ENDFOREACH;
			ENDIF;

			$arr_settings['mbr_be_post_types_enabled']['Disabled'] = array_merge( [ 'placebo' ], $arr_disabled_post_types );

			update_option( 'site_bulk_edit_types', array_unique( array_merge( $arr_default_post_types, $arr_enabled_post_types ) ) );
		ENDIF;
	}

}

function mbr_be_post_type_support()
{
	global $wp_post_types;

	$arr_protectable_post_types        = apply_filters( 'mbr/protectable_post_types/all', [] );
	$arr_enabled                       = array_intersect( $GLOBALS['mbr_inst_bulk_editor']->arr_enabled_bulk_edit_post_types, $arr_protectable_post_types );
	$arr_disabled                      = array_diff( $arr_protectable_post_types, $GLOBALS['mbr_inst_bulk_editor']->arr_enabled_bulk_edit_post_types );

	$arr_enabled_post_types  = [];
	FOREACH ( $arr_enabled AS $key => $post_type ) :
		$arr_enabled_post_types[ $post_type ]  = ucfirst( $wp_post_types[ $post_type ]->labels->singular_name ) . ' (' . $post_type . ')';
	ENDFOREACH;

	$arr_disabled_post_types = [];
	FOREACH ( $arr_disabled AS $key => $post_type ) :
		$arr_disabled_post_types[ $post_type ] = ucfirst( $wp_post_types[ $post_type ]->labels->singular_name ) . ' (' . $post_type . ')';
	ENDFOREACH;

	
	return [ 'Enabled' => $arr_enabled_post_types, 'Disabled' => $arr_disabled_post_types ];
}

$GLOBALS['mbr_inst_bulk_editor'] = mbr_bulk_editor::getInstance();