<?php
class mbr_contactform7
{
	use TRAIT__getInstance;

	public $mbr_crm_identifier              = 'mbr';
	public $panel_title                     = 'ActiveMember360';
	public $integration_name                = 'ActiveCampaign';
	public $form_tag_generator_button_title = 'ActiveCampaign Contact Fields';
	public $allowed_standard_contact_fields = [ 'first_name' => 'First Name', 'last_name' => 'Last Name', 'email' => 'Email', 'phone' => 'Phone' ];
	public $arrFORMKEYS                     = [
			'form_enable_integration_key'      => '_mbr_cf7_enable_integration',
			'form_assign_tags_key'             => '_mbr_cf7_assign_tags',
			'form_remove_tags_key'             => '_mbr_cf7_remove_tags',
			'form_assign_automations_key'      => '_mbr_cf7_assign_automations',
			'form_remove_automations_key'      => '_mbr_cf7_remove_automations',
			'form_subscribe_lists_key'         => '_mbr_cf7_subscribe_lists',
			'form_unsubscribe_lists_key'       => '_mbr_cf7_unsubscribe_lists',
			'form_redirect_pid_key'            => '_mbr_cf7_redirect_pid',
			'form_skip_mail_key'               => '_mbr_cf7_skip_mail',
			'form_log_event_key'               => '_mbr_cf7_log_event',
			'form_autologin_key'               => '_mbr_cf7_autologin'
	];

	public function _load()
	{

		function MBRCF7()
		{
			IF ( empty( $GLOBALS['mbrcf7'] ) ) :
			$GLOBALS['mbrcf7'] = mbr_contactform7::getInstance();
			ENDIF;

			return $GLOBALS['mbrcf7'];
		}

		add_action( 'wpcf7_init', [ $this, '_init' ], 11 );
	}

	public function _init()
	{
		FOREACH ( $this->arrFORMKEYS AS $form_key_name => $form_key_value ) :
			$this->$form_key_name = $form_key_value;
		ENDFOREACH;

		IF ( is_admin() ) :
			$this->contact_fields = [];

			FOREACH ( $this->allowed_standard_contact_fields AS $field => $name ) :
				$this->contact_fields[ lower_trim( $field ) ] = $name;
			ENDFOREACH;

			FOREACH ( apply_filters( 'mbr/site_custom_fields/get', NULL ) AS $k => $field ) :
				IF ( ! empty( $field['perstag'] ) ) :
					$this->contact_fields[ lower_trim( $field['perstag'] ) ] = $field['title'];
				ENDIF;
			ENDFOREACH;
		ENDIF;

		add_action( 'wp_ajax_mbr_ajax_cf7_redirect_url',        [ $this, 'ajax_mbr_cf7_redirect' ],                   10, 0 );
		add_action( 'wp_ajax_nopriv_mbr_ajax_cf7_redirect_url', [ $this, 'ajax_mbr_cf7_redirect' ],                   10, 0 );

		IF ( is_admin() ) :
			add_action( 'admin_footer',                           [ $this, 'mbr_cf7_scripts' ],                         10, 0 );
			add_action( 'admin_init',                             [ $this, 'mbr_cf7_add_tag_generator' ],               99, 0 );
			add_action( 'wpcf7_editor_panels',                    [ $this, 'mbr_cf7_tag_page_panels' ],                 10, 1 );
			add_action( 'wpcf7_after_save',                       [ $this, 'mbr_cf7_save_contact_form' ],               10, 1 );
			add_filter( 'wpcf7_contact_form_properties',          [ $this, 'mbr_cf7_contact_form_properties' ],         10, 2 );
		ELSE:
			add_action( 'wp_head',                                [ $this, 'mbr_cf7_redirect_after_form_submission'], 9999, 0 );
			add_filter( 'wpcf7_form_tag',                         [ $this, 'mbr_cf7_form_tag' ],                        19, 2 );
			add_action( 'wpcf7_mail_sent',                        [ $this, 'mbr_cf7_signup_form_submitted' ],           10, 1 );
			add_filter( 'wpcf7_skip_mail',                        [ $this, 'mbr_cf7_skip_mail' ],                     9999, 2 );
		ENDIF;
	}

	public function mbr_cf7_tag_generator( $contact_form, $args = '' )
	{
		$args = wp_parse_args( $args, [] );
		?>
		<div class="control-box">
			<fieldset>
			<table class="form-table">
				<tbody>
				<tr>
				<th scope="row"><label for="<?php echo $this->mbr_crm_identifier; ?>-field-name"><?php _ex( 'Field Name', 'wp_admin', 'activemember360' ); ?></label></th>
				<td>
					<select name="<?php echo $this->mbr_crm_identifier; ?>-field-name" class="tg-name oneline" id="<?php echo $this->mbr_crm_identifier; ?>-field-name">
					<option value="0">--</option>
					<?php
					FOREACH ( $this->contact_fields AS $value => $name ) :
						echo "<option value='{$value}'>{$name}</option>";
					ENDFOREACH;
					?>
					</select>
				</td>
				</tr>
				<tr id="mbr_cf7_field_type">
				<th scope="row"><label for="<?php echo $this->mbr_crm_identifier; ?>-tag-code-type"><?php _ex( 'AC Field Type', 'wp_admin', 'activemember360' ); ?></label></th>
				<td>
					<select name="<?php echo $this->mbr_crm_identifier; ?>-field-tag-code-type" class="tg-name oneline" id="<?php echo $this->mbr_crm_identifier; ?>-field-tag-code-type">
					<?php
					FOREACH ( ['text' => 'Text', 'email' => 'Email', 'url' => 'URL', 'tel' => 'Phone', 'number' => 'Number (Spinbox)', 'range' => 'Number (Slider)', 'date' => 'Date', 'textarea' => 'Textarea', 'checkbox' => 'Checkbox', 'radio' => 'Radio', 'select' => 'Drop-Down' ] AS $key => $value) :
						echo "<option value='{$key}'>{$value}</option>";
					ENDFOREACH;
					?>
					</select>
				</td>
				</tr>
				<tr id="mbr_cf7_option_default_value">
				<th scope="row"><label for="<?php echo esc_attr( $args['content'] . '-values' ); ?>"><?php echo esc_html( __('Default value', 'contact-form-7' ) ); ?></label></th>
				<td><input type="text" name="values" class="oneline" id="<?php echo esc_attr( $args['content'] . '-values' ); ?>" /><br />
					<label><input type="checkbox" name="placeholder" class="option" /> <?php echo esc_html( __( 'Use this text as the placeholder of the field', 'contact-form-7' ) ); ?></label></td>
				</tr>
				<tr id="mbr_cf7_option_checkbox_radio" style="display:none">
				<th scope="row"><?php echo esc_html( __( 'Options', 'contact-form-7' ) ); ?></th>
				<td>
					<fieldset>
					<legend class="screen-reader-text"><?php echo esc_html( __( 'Options', 'contact-form-7' ) ); ?></legend>
					<textarea name="values" class="values" id="<?php echo esc_attr( $args['content'] . '-values' ); ?>"></textarea>
					<label for="<?php echo esc_attr( $args['content'] . '-values' ); ?>"><span class="description"><?php echo esc_html( __( 'One option per line.', 'contact-form-7' ) ); ?></span></label>
					<label id="mbr_cf7_checkbox_label_first_label" style="display:block;"><input id="mbr_cf7_checkbox_label_first" type="checkbox" name="label_first" class="option" /> <?php echo esc_html( __( 'Put a label first, a checkbox last', 'contact-form-7' ) ); ?></label>
					<label id="mbr_cf7_checkbox_use_label_element_label" style="display:block;"><input id="mbr_cf7_checkbox_use_label_element" type="checkbox" name="use_label_element" class="option" /> <?php echo esc_html( __( 'Wrap each item with label element', 'contact-form-7' ) ); ?></label>
					<label id="mbr_cf7_checkbox_exclusive_label" style="display:block;"><input id="mbr_cf7_checkbox_exclusive" type="checkbox" name="exclusive" class="option" /> <?php echo esc_html( __( 'Make checkboxes exclusive', 'contact-form-7' ) ); ?></label>
					<label id="mbr_cf7_checkbox_multiple_label" style="display:block;"><input id="mbr_cf7_checkbox_multiple" type="checkbox" name="multiple" class="option" /> <?php echo esc_html( __( 'Allow multiple selections', 'contact-form-7' ) ); ?></label>
					<label id="mbr_cf7_checkbox_include_blank_label" style="display:block;"><input id="mbr_cf7_checkbox_include_blank" type="checkbox" name="include_blank" class="option" /> <?php echo esc_html( __( 'Insert a blank item as the first option', 'contact-form-7' ) ); ?></label>
					</fieldset>
				</td>
				</tr>
				<tr id="mbr_cf7_option_akismet" style="display:none">
				<th scope="row"><?php echo esc_html( __( 'Akismet', 'contact-form-7' ) ); ?></th>
				<td>
					<fieldset>
					<legend class="screen-reader-text"><?php echo esc_html( __( 'Akismet', 'contact-form-7' ) ); ?></legend>
					<label id="mbr_cf7_author_name" style="display:none"><input type="checkbox" id="mbr_cf7_akismet_author_name" name="akismet:author" class="option" /> <?php echo esc_html( __( "This field requires author's name", 'contact-form-7' ) ); ?></label>
					<label id="mbr_cf7_author_email" style="display:none"><input type="checkbox" id="mbr_cf7_akismet_author_email" name="akismet:author_email" class="option" /> <?php echo esc_html( __( "This field requires author's email address", 'contact-form-7' ) ); ?></label>
					<label id="mbr_cf7_author_url" style="display:none"><input type="checkbox" id="mbr_cf7_akismet_author_url" name="akismet:author_url" class="option" /> <?php echo esc_html( __( "This field requires author's URL", 'contact-form-7' ) ); ?></label>
					</fieldset>
				</td>
				</tr>
				<tr id="mbr_cf7_option_date_range" style="display:none">
				<th scope="row"><?php echo esc_html( __( 'Range', 'contact-form-7' ) ); ?></th>
				<td>
					<fieldset>
					<legend class="screen-reader-text"><?php echo esc_html( __( 'Range', 'contact-form-7' ) ); ?></legend>
					<label>
						<?php echo esc_html( __( 'Min', 'contact-form-7' ) ); ?>
						<input type="date" id="mbr_cf7_date_range_min" name="min" class="date option" />
					</label>
					&ndash;
					<label>
						<?php echo esc_html( __( 'Max', 'contact-form-7' ) ); ?>
						<input type="date" id="mbr_cf7_date_range_max" name="max" class="date option" />
					</label>
					</fieldset>
				</td>
				</tr>
				<tr id="mbr_cf7_option_number_range" style="display:none">
				<th scope="row"><?php echo esc_html( __( 'Range', 'contact-form-7' ) ); ?></th>
				<td>
					<fieldset>
					<legend class="screen-reader-text"><?php echo esc_html( __( 'Range', 'contact-form-7' ) ); ?></legend>
					<label>
						<?php echo esc_html( __( 'Min', 'contact-form-7' ) ); ?>
						<input type="number" id="mbr_cf7_number_range_min" name="min" class="numeric option" />
					</label>
					&ndash;
					<label>
						<?php echo esc_html( __( 'Max', 'contact-form-7' ) ); ?>
						<input type="number" id="mbr_cf7_number_range_max" name="max" class="numeric option" />
					</label>
					</fieldset>
				</td>
				</tr>
				<tr>
				<th scope="row"><label for="<?php echo esc_attr( $args['content'] . '-id' ); ?>"><?php echo esc_html( __( 'Id attribute', 'contact-form-7' ) ); ?></label></th>
				<td><input type="text" name="id" class="idvalue oneline option" id="<?php echo esc_attr( $args['content'] . '-id'); ?>" /></td>
				</tr>
				<tr>
				<th scope="row"><label for="<?php echo esc_attr( $args['content'] . '-class' ); ?>"><?php echo esc_html( __( 'Class attribute', 'contact-form-7' ) ); ?></label></th>
				<td><input type="text" name="class" class="classvalue oneline option" id="<?php echo esc_attr( $args['content'] . '-class' ); ?>" /></td>
				</tr>
				<tr id="mbr_cf7_option_required_field">
				<th scope="row"><?php echo esc_html( __( 'Required field', 'contact-form-7' ) ); ?></th>
				<td>
					<fieldset>
					<legend class="screen-reader-text"><?php echo esc_html( __( 'Required field', 'contact-form-7' ) ); ?></legend>
					<label><input id="mbr_cf7_checkbox_required_field" type="checkbox" name="required" /> <?php echo esc_html( __( 'Required field', 'contact-form-7' ) ); ?></label>
					</fieldset>
				</td>
				</tr>
				</tbody>
			</table>
			</fieldset>
		</div>
		<div class="insert-box">
			<input type="text" name="text" id="mbr_tag_code_type" class="tag code" readonly="readonly" onfocus="this.select()" />
			<div class="submitbox">
			<input type="button" class="button button-primary insert-tag" value="<?php echo esc_attr( __( 'Insert Tag', 'contact-form-7' ) ); ?>" />
			</div>
			<br class="clear" />
			<p class="description mail-tag">
			<label for="<?php echo esc_attr( $args['content'] . '-mailtag' ); ?>">
				<?php echo sprintf( esc_html( __('To use the value input through this field in a mail field, you need to insert the corresponding mail-tag (%s) into the field on the Mail tab.', 'contact-form-7' ) ), '<strong><span class="mail-tag"></span></strong>' ); ?>
				<input type="text" class="mail-tag code hidden" readonly="readonly" id="<?php echo esc_attr( $args['content'] . '-mailtag' ); ?>" />
			</label>
			</p>
		</div>
		<script>
			var mbr_cf7_default_value_element;
			var mbr_cf7_checkbox_radio_element;
			jQuery( document ).ready( function() {
				mbr_cf7_display_related_field_type_options();
			});

			jQuery( "#<?php echo $this->mbr_crm_identifier; ?>-field-tag-code-type" ).change( function() {
				mbr_cf7_display_related_field_type_options();
			});

			function mbr_cf7_display_related_field_type_options() {
				mbr_cf7_tag_type = jQuery( "#<?php echo $this->mbr_crm_identifier; ?>-field-tag-code-type" ).val();
				jQuery( "#mbr_tag_code_type" ).attr( 'name', mbr_cf7_tag_type );

				mbr_cf7_init_all_options( mbr_cf7_tag_type );

				if ( mbr_cf7_tag_type == 'text' || mbr_cf7_tag_type == 'email' || mbr_cf7_tag_type == 'url' ) {
					mbr_cf7_show_akismet();
					if ( mbr_cf7_tag_type == 'text' ) {
						jQuery( "#mbr_cf7_author_name" ).show();
						jQuery( "#mbr_cf7_author_email" ).hide();
						jQuery( "#mbr_cf7_author_url" ).hide();
						jQuery( "#mbr_cf7_akismet_author_email" ).prop( "checked", false );
						jQuery( "#mbr_cf7_akismet_author_url" ).prop( "checked", false );
					} else if ( mbr_cf7_tag_type == 'email' ) {
						jQuery( "#mbr_cf7_author_name" ).hide();
						jQuery( "#mbr_cf7_author_email" ).show();
						jQuery( "#mbr_cf7_author_url" ).hide();
						jQuery( "#mbr_cf7_akismet_author_name" ).prop( "checked", false );
						jQuery( "#mbr_cf7_akismet_author_url" ).prop( "checked", false );
					} else if ( mbr_cf7_tag_type == 'url' ) {
						jQuery( "#mbr_cf7_author_name" ).hide();
						jQuery( "#mbr_cf7_author_email" ).hide();
						jQuery( "#mbr_cf7_author_url" ).show();
						jQuery( "#mbr_cf7_akismet_author_name" ).prop( "checked", false );
						jQuery( "#mbr_cf7_akismet_author_email" ).prop( "checked", false );
					}
				} else if ( mbr_cf7_tag_type == 'date' ) {
					mbr_cf7_show_date_range();
				} else if ( mbr_cf7_tag_type == 'number' || mbr_cf7_tag_type == 'range' ) {
					mbr_cf7_show_number_range();
				} else if ( mbr_cf7_tag_type == 'checkbox' || mbr_cf7_tag_type == 'radio' || mbr_cf7_tag_type == 'select' ) {
					mbr_cf7_show_checkbox_radio( mbr_cf7_tag_type );
				}
			}

			function mbr_cf7_init_all_options( tag_type ) {
				mbr_cf7_hide_akismet();
				mbr_cf7_hide_date_range();
				mbr_cf7_hide_number_range();
				mbr_cf7_hide_checkbox_radio();
				mbr_cf7_show_default_value();
				mbr_cf7_show_required_field();
			}

			function mbr_cf7_show_akismet() {
				jQuery( "#mbr_cf7_option_akismet" ).show();
			}

			function mbr_cf7_hide_akismet() {
				jQuery( "#mbr_cf7_option_akismet" ).hide();
				jQuery( "#mbr_cf7_akismet_author_url" ).prop( "checked", false );
				jQuery( "#mbr_cf7_akismet_author_name" ).prop( "checked", false );
				jQuery( "#mbr_cf7_akismet_author_email" ).prop( "checked", false );
			}

			function mbr_cf7_show_date_range() {
				jQuery( "#mbr_cf7_option_date_range" ).show();
			}

			function mbr_cf7_hide_date_range() {
				jQuery( "#mbr_cf7_option_date_range" ).hide();
				jQuery( "#mbr_cf7_date_range_min" ).val('');
				jQuery( "#mbr_cf7_date_range_max" ).val('');
			}

			function mbr_cf7_show_number_range() {
				jQuery( "#mbr_cf7_option_number_range" ).show();
			}

			function mbr_cf7_hide_number_range() {
				jQuery( "#mbr_cf7_option_number_range" ).hide();
				jQuery( "#mbr_cf7_option_number_range_display" ).hide();
				jQuery( "#mbr_cf7_number_range_min" ).val('');
				jQuery( "#mbr_cf7_number_range_max" ).val('');
			}

			function mbr_cf7_show_checkbox_radio( tag_type ) {
				mbr_cf7_hide_default_value();
				jQuery( "#mbr_cf7_option_checkbox_radio" ).show();
				if (mbr_cf7_checkbox_radio_element) {
					mbr_cf7_checkbox_radio_element.insertAfter( "#mbr_cf7_field_type" );
					mbr_cf7_checkbox_radio_element = null;
				}
				if ( tag_type == 'checkbox' ) {
					jQuery( "#mbr_cf7_checkbox_multiple_label" ).hide();
					jQuery( "#mbr_cf7_checkbox_include_blank_label" ).hide();
					jQuery( "#mbr_cf7_checkbox_exclusive_label" ).show();
					jQuery( "#mbr_cf7_checkbox_label_first_label" ).show();
					jQuery( "#mbr_cf7_checkbox_use_label_element_label" ).show();
				} else if ( tag_type == 'radio' ) {
					mbr_cf7_hide_required_field( true );
					jQuery( "#mbr_cf7_checkbox_exclusive_label" ).hide();
					jQuery( "#mbr_cf7_checkbox_multiple_label" ).hide();
					jQuery( "#mbr_cf7_checkbox_include_blank_label" ).hide();
					jQuery( "#mbr_cf7_checkbox_label_first_label" ).show();
					jQuery( "#mbr_cf7_checkbox_use_label_element_label" ).show();
				} else {
					jQuery( "#mbr_cf7_checkbox_exclusive_label" ).hide();
					jQuery( "#mbr_cf7_checkbox_label_first_label" ).hide();
					jQuery( "#mbr_cf7_checkbox_use_label_element_label" ).hide();
					jQuery( "#mbr_cf7_checkbox_multiple_label" ).show();
					jQuery( "#mbr_cf7_checkbox_include_blank_label" ).show();
				}
			}

			function mbr_cf7_hide_checkbox_radio() {
				jQuery( "#mbr_cf7_option_checkbox_radio" ).hide();
				jQuery( "#mbr_cf7_checkbox_exclusive_label" ).hide();
				jQuery( "#mbr_cf7_checkbox_label_first_label" ).hide();
				jQuery( "#mbr_cf7_checkbox_use_label_element_label" ).hide();
				jQuery( "#mbr_cf7_checkbox_label_first" ).prop( "checked", false );
				jQuery( "#mbr_cf7_checkbox_use_label_element" ).prop( "checked", false );
				jQuery( "#mbr_cf7_checkbox_exclusive" ).prop( "checked", false );
				jQuery( "#mbr_cf7_checkbox_multiple" ).prop( "checked", false );
				jQuery( "#mbr_cf7_checkbox_include_blank" ).prop( "checked", false);
				mbr_cf7_checkbox_radio_element = jQuery( "#mbr_cf7_checkbox_radio" ).detach();
			}

			function mbr_cf7_hide_default_value() {
				mbr_cf7_default_value_element = jQuery( "#mbr_cf7_option_default_value" ).detach();
			}

			function mbr_cf7_show_default_value() {
				jQuery( "#mbr_cf7_option_default_value" ).show();
				if ( mbr_cf7_default_value_element ) {
					mbr_cf7_default_value_element.insertAfter( "#mbr_cf7_field_type" );
					mbr_cf7_default_value_element = null;
				}
			}

			function mbr_cf7_hide_required_field( uncheck_box ) {
				jQuery( "#mbr_cf7_option_required_field" ).hide();
				if ( uncheck_box ) {
					jQuery( "#mbr_cf7_checkbox_required_field" ).prop( "checked", false );
				}
			}

			function mbr_cf7_show_required_field() {
				jQuery("#mbr_cf7_option_required_field").show();
			}
		</script>
		<?php
	}

	public function mbr_cf7_scripts()
	{
		?>
		<script>
			( function( $ ){
				'use strict';
				$( document ).ready( function() {
					var $MBRFieldNames = $( '#<?php echo $this->mbr_crm_identifier; ?>-field-name' );
					var $form = $MBRFieldNames.closest( 'form' );
					var $tagGenerator = $form.find( '.insert-box > input:not(.hidden)' );
					$form.change( function() {
						var tag = $tagGenerator.val();
						var selectedField = $MBRFieldNames.val();
						
						if ( selectedField != '0' ) {
							
							if ( tag.match( /(\s+<?php echo $this->mbr_crm_identifier; ?>[a-z-]+)/ ) ) {
							$tagGenerator.val( tag.replace( /(\s+<?php echo $this->mbr_crm_identifier; ?>[a-z-]+)/, selectedField ) );
							} else {
							$tagGenerator.val( tag.replace( /(^\[\w+\*?)/, '$1 <?php echo $this->mbr_crm_identifier; ?>-' + selectedField ) );
							}
							
							$form.find( '#tag-generator-panel-<?php echo $this->mbr_crm_identifier; ?>-mailtag' ).val( '[<?php echo $this->mbr_crm_identifier; ?>-' + selectedField + ']' );
							$form.find( 'span.mail-tag').text( '[<?php echo $this->mbr_crm_identifier; ?>-' + selectedField + ']' );
						} else {
							$tagGenerator.val( tag.replace( /(\s+<?php echo $this->mbr_crm_identifier; ?>[a-z-]+)/, '' ) );
							$form.find( '#tag-generator-panel-<?php echo $this->mbr_crm_identifier; ?>-mailtag' ).val('[]' );
							$form.find( 'span.mail-tag').text( '[]' );
						}
					});
				});
			} )( jQuery );
		</script>
		<?php
	}

	


	public function mbr_cf7_add_tag_generator()
	{
		wpcf7_add_tag_generator( 'mbr', $this->form_tag_generator_button_title, 'wpcf7-tg-panel-mbr', [ $this, 'mbr_cf7_tag_generator' ] );
	}

	



	public function mbr_cf7_tag_page_panels( $panels )
	{
		$panels = [ 'mbr-panel' => [ 'title' => $this->panel_title, 'callback' => [ $this, 'mbr_cf7_panel_meta' ] ] ] + $panels;

		return $panels;
	}

	
	public function mbr_cf7_panel_meta( $post )
	{
		global $MBR_OPTIONS;

		wp_nonce_field( 'mbr_cf7_form_settings', 'mbr_cf7_form_settings_nonce' );

		
		$integrate          = get_post_meta( $post->id, $this->form_enable_integration_key, TRUE );
		$skip_mail          = get_post_meta( $post->id, $this->form_skip_mail_key,          TRUE );
		$log_event          = get_post_meta( $post->id, $this->form_log_event_key,          TRUE );
		$autologin          = get_post_meta( $post->id, $this->form_autologin_key,          TRUE );
		$redirect_pid       = get_post_meta( $post->id, $this->form_redirect_pid_key,       TRUE );
		$assign_tags        = json_decode( get_post_meta($post->id, $this->form_assign_tags_key,        TRUE ), TRUE );
		$remove_tags        = json_decode( get_post_meta($post->id, $this->form_remove_tags_key,        TRUE ), TRUE );
		$assign_automations = json_decode( get_post_meta($post->id, $this->form_assign_automations_key, TRUE ), TRUE );
		$remove_automations = json_decode( get_post_meta($post->id, $this->form_remove_automations_key, TRUE ), TRUE );
		$subscribe_lists    = json_decode( get_post_meta($post->id, $this->form_subscribe_lists_key,    TRUE ), TRUE );
		$unsubscribe_lists  = json_decode( get_post_meta($post->id, $this->form_unsubscribe_lists_key,  TRUE ), TRUE );

		$arr_pages    = [];
		$arr_pages[0] = _x( 'Default redirect', 'wp_admin', 'activemember360' );
		FOREACH ( apply_filters('mbr/pages/get', NULL ) AS $page ) :
			$arr_pages[ $page->ID ] = $page->post_title;
		ENDFOREACH;

		
		$str_settings          =  '<h3>' . _x( 'Integrate with %s', 'wp_admin', 'activemember360' ) . '</h3><fieldset><input type="checkbox" id="mbr_cf7_enable_integration" name="mbr_cf7_enable_integration" value="1" %s /> ' . _x( 'Integrate with %s', 'wp_admin', 'activemember360' ) . '.</fieldset>
									 <div id="mbr_cf7_form_settings" style="display:none;">
									 <h3>' . _x( 'Do not send an email', 'wp_admin', 'activemember360' ) . '</h3><fieldset><input type="checkbox" id="mbr_cf7_skip_mail" name="mbr_cf7_skip_mail" value="1" %s /> ' . _x( 'Do NOT send an email after form submission.', 'wp_admin', 'activemember360' ) . '</fieldset>';
		$event_tracking_enabled = $MBR_OPTIONS['extensions']['mbr_ext_ac_event_tracking_enabled'];
		IF ( ! empty( $event_tracking_enabled ) ) :
			$checked_log_event   =  checked( esc_attr( $log_event ), 'yes', FALSE );
			$str_settings       .= '<h3>' . _x( 'Log form submission', 'wp_admin', 'activemember360' ) . '</h3><fieldset><input type="checkbox" id="mbr_cf7_log_event" name="mbr_cf7_log_event" value="1" %s /> ' . _x( 'Log an event in ActiveCampaign.', 'wp_admin', 'activemember360' ) . '</fieldset>';
		ELSE:
			$checked_log_event   = '';
			$str_settings       .= '<h3>' . _x( 'Log form submission', 'wp_admin', 'activemember360' ) . '</h3><fieldset><em>' . _x( 'Event Tracking has been disabled in "Extensions => Site &amp; Event Tracking".', 'wp_admin', 'activemember360' ) . '</em></fieldset>%s';
		ENDIF;

		IF ( ! empty( $MBR_OPTIONS['autologin_module'] ) ) :
			$checked_autologin   = checked( esc_attr( $autologin ), 'yes', FALSE );
			$str_settings       .= '<h3>' . _x( 'Autologin', 'wp_admin', 'activemember360' ) . '</h3><fieldset><input type="checkbox" id="mbr_cf7_autologin" name="mbr_cf7_autologin" value="1" %s /> Autologin on successful form submission (use of the AC email field is required. Autologin is NOT recommended).</fieldset>';
		ELSE:
			$checked_autologin   = '';
			$str_settings       .= '<h3>' . _x( 'Autologin', 'wp_admin', 'activemember360' ) . '</h3><fieldset><em>Autologin has been disabled in "Plugin Options".</em></fieldset>%s';
		ENDIF;
		$str_settings         .= '<h3>' . _x( 'Redirect Page after successful form submissions', 'wp_admin', 'activemember360' ) . '</h3><fieldset><legend>' . _x( 'Choose a page where you want to send your contacts after a successful form submission.', 'wp_admin', 'activemember360' ) . '</legend>%s</fieldset>
									<h3>' . _x( 'Assign Tags', '         wp_admin', 'activemember360' ) . '</h3><fieldset><legend>' . _x( 'Choose Tags to assign after a successful form submission.',        'wp_admin', 'activemember360' ) . '</legend>%s</fieldset>
									<h3>' . _x( 'Remove Tags',          'wp_admin', 'activemember360' ) . '</h3><fieldset><legend>' . _x( 'Choose Tags to remove after a successful form submission.',        'wp_admin', 'activemember360' ) . '</legend>%s</fieldset>
									<h3>' . _x( 'Assign Automations',   'wp_admin', 'activemember360' ) . '</h3><fieldset><legend>' . _x( 'Choose Automations to assign after a successful form submission.', 'wp_admin', 'activemember360' ) . '</legend>%s</fieldset>
									<h3>' . _x( 'Remove Automations',   'wp_admin', 'activemember360' ) . '</h3><fieldset><legend>' . _x( 'Choose Automations to remove after a successful form submission.', 'wp_admin', 'activemember360' ) . '</legend>%s</fieldset>
									<h3>' . _x( 'Subscribe to Lists',   'wp_admin', 'activemember360' ) . '</h3><fieldset><legend>' . _x( 'Choose lists to subscribe after a successful form submission.',    'wp_admin', 'activemember360' ) . '</legend>%s</fieldset>
									<h3>' . _x( 'Unsubscribe to Lists', 'wp_admin', 'activemember360' ) . '</h3><fieldset><legend>' . _x( 'Choose lists to unsubscribe after a successful form submission.',  'wp_admin', 'activemember360' ) . '</legend>%s</fieldset>
									</div>
									<script>
									jQuery( document ).ready( function() {
										mbr_cf7_display_form_settings();
									} );
									jQuery( "#mbr_cf7_enable_integration" ).click( function() {
										mbr_cf7_display_form_settings();
									} );
									function mbr_cf7_display_form_settings() {
										if ( jQuery( "#mbr_cf7_enable_integration" ).is( ":checked" ) ) {
											jQuery( "#mbr_cf7_form_settings" ).show();
										} else {
											jQuery( "#mbr_cf7_form_settings" ).hide();
										}
									}
									</script>';

		printf( $str_settings,
			$this->integration_name,
			checked( esc_attr( $integrate ), 'yes', FALSE),
			$this->integration_name,
			checked( esc_attr( $skip_mail ), 'yes', FALSE),
			$checked_log_event,
			$checked_autologin,
			apply_filters( 'mbr/select_box_helper', $arr_pages,                                        $redirect_pid,       'mbr_cf7_redirect_pid',       MBR_SELECTBOX_DO_NOT_DISPLAY_ID, MBR_SELECTBOX_DO_NOT_MULTIPLE, MBR_SELECTBOX_DO_NOT_SELECT2 ),
			apply_filters( 'mbr/select_box_helper', apply_filters( 'mbr/site_tags/get' , NULL ),       $assign_tags,        'mbr_cf7_assign_tags',        MBR_SELECTBOX_DO_DISPLAY_ID,     MBR_SELECTBOX_DO_MULTIPLE,     MBR_SELECTBOX_DO_SELECT2 ),
			apply_filters( 'mbr/select_box_helper', apply_filters( 'mbr/site_tags/get' , NULL ),       $remove_tags,        'mbr_cf7_remove_tags',        MBR_SELECTBOX_DO_DISPLAY_ID,     MBR_SELECTBOX_DO_MULTIPLE,     MBR_SELECTBOX_DO_SELECT2 ),
			apply_filters( 'mbr/select_box_helper', apply_filters( 'mbr/site_automations/get', NULL ), $assign_automations, 'mbr_cf7_assign_automations', MBR_SELECTBOX_DO_DISPLAY_ID,     MBR_SELECTBOX_DO_MULTIPLE,     MBR_SELECTBOX_DO_SELECT2 ),
			apply_filters( 'mbr/select_box_helper', apply_filters( 'mbr/site_automations/get', NULL ), $remove_automations, 'mbr_cf7_remove_automations', MBR_SELECTBOX_DO_DISPLAY_ID,     MBR_SELECTBOX_DO_MULTIPLE,     MBR_SELECTBOX_DO_SELECT2 ),
			apply_filters( 'mbr/select_box_helper', apply_filters( 'mbr/site_lists/get', NULL ),       $subscribe_lists,    'mbr_cf7_subscribe_lists',    MBR_SELECTBOX_DO_DISPLAY_ID,     MBR_SELECTBOX_DO_MULTIPLE,     MBR_SELECTBOX_DO_SELECT2 ),
			apply_filters( 'mbr/select_box_helper', apply_filters( 'mbr/site_lists/get', NULL ),       $unsubscribe_lists,  'mbr_cf7_unsubscribe_lists',  MBR_SELECTBOX_DO_DISPLAY_ID,     MBR_SELECTBOX_DO_MULTIPLE,     MBR_SELECTBOX_DO_SELECT2 )
		);
	}

	
	public function mbr_cf7_save_contact_form( $contact_form )
	{
		$contact_form_id     = $contact_form->id();

		IF ( empty( $_POST ) OR empty( $_POST['mbr_cf7_form_settings_nonce'] ) ) :
			return;
		ENDIF;

		IF ( ! wp_verify_nonce( $_POST['mbr_cf7_form_settings_nonce'], 'mbr_cf7_form_settings' ) ) :
			return;
		ENDIF;

		$integrate          = ( ! empty( $_POST['mbr_cf7_enable_integration'] )    ? 'yes' : 'no' );
		update_post_meta( $contact_form_id, $this->form_enable_integration_key, $integrate );

		$skip_mail          = ( ! empty( $_POST['mbr_cf7_skip_mail'] )    ? 'yes' : 'no' );
		update_post_meta( $contact_form_id, $this->form_skip_mail_key, $skip_mail );

		$log_event          = ( ! empty ( $_POST['mbr_cf7_log_event'] )    ? 'yes' : 'no' );
		update_post_meta( $contact_form_id, $this->form_log_event_key, $log_event );

		$autologin          = ( ! empty( $_POST['mbr_cf7_autologin'] )    ? 'yes' : 'no' );
		update_post_meta( $contact_form_id, $this->form_autologin_key, $autologin );

		$redirect_pid       = ( ! empty( $_POST['mbr_cf7_redirect_pid'] ) ? $_POST['mbr_cf7_redirect_pid'] : '' );
		update_post_meta( $contact_form_id, $this->form_redirect_pid_key, $redirect_pid );

		$assign_tags        = ( ! empty( $_POST['mbr_cf7_assign_tags'] )  ? $_POST['mbr_cf7_assign_tags'] : '' );
		update_post_meta( $contact_form_id, $this->form_assign_tags_key, json_encode( $assign_tags ) );

		$remove_tags        = ( ! empty( $_POST['mbr_cf7_remove_tags'] )  ? $_POST['mbr_cf7_remove_tags'] : '' );
		update_post_meta( $contact_form_id, $this->form_remove_tags_key, json_encode( $remove_tags ) );

		$assign_automations = ( ! empty( $_POST['mbr_cf7_assign_automations'] )  ? $_POST['mbr_cf7_assign_automations'] : '' );
		update_post_meta( $contact_form_id, $this->form_assign_automations_key, json_encode( $assign_automations ) );

		$remove_automations = ( ! empty( $_POST['mbr_cf7_remove_automations'] )  ? $_POST['mbr_cf7_remove_automations'] : '' );
		update_post_meta( $contact_form_id, $this->form_remove_automations_key, json_encode( $remove_automations ) );

		$subscribe_lists    = ( ! empty( $_POST['mbr_cf7_subscribe_lists'] )     ? $_POST['mbr_cf7_subscribe_lists'] : '' );
		update_post_meta( $contact_form_id, $this->form_subscribe_lists_key, json_encode($subscribe_lists));

		$unsubscribe_lists  = ( ! empty( $_POST['mbr_cf7_unsubscribe_lists'] )   ? $_POST['mbr_cf7_unsubscribe_lists'] : '' );
		update_post_meta( $contact_form_id, $this->form_unsubscribe_lists_key, json_encode( $unsubscribe_lists ) );
	}

	public function mbr_cf7_signup_form_submitted( $contact_form )
	{
		$integrate = get_post_meta( $contact_form->id(), $this->form_enable_integration_key, TRUE );
		IF ( $integrate <> 'yes' ) :
			return;
		ENDIF;
		$submission      = WPCF7_Submission::get_instance();
		$posted_data     = $submission->get_posted_data();
		
		IF ( empty( $posted_data[ $this->mbr_crm_identifier . '-email' ] ) AND ! apply_filters( 'mbr/contact_id', NULL ) ) :
			return;
		ENDIF;
		
		$this->mbr_cf7_add_contact( $contact_form->id(), $posted_data );
	}

	public function mbr_cf7_add_contact( $contact_form_id, $posted_data )
	{
		global $MBR_OPTIONS;

		$contact_id           = apply_filters( 'mbr/contact_id', NULL );

		
		$strlen_ac_identifier = strlen( $this->mbr_crm_identifier );
		$contact_data         = [];

		FOREACH ( $posted_data AS $field => $value ) :
			IF ( substr($field, 0, $strlen_ac_identifier ) == $this->mbr_crm_identifier ) :

				$arrFIELD = explode( $this->mbr_crm_identifier . '-', $field );
				list( $filler, $ac_field ) = $arrFIELD;

				IF ( $contact_id AND trim( strtolower( $ac_field ) ) == 'email' ) :
					
					
					continue;
				ENDIF;

				IF ( in_array( $ac_field, apply_filters( 'mbr/site_contact_std_fields/get', NULL ) ) ) :
					$contact_data[ lower_trim($ac_field) ] = $value;
				ELSE:
					FOREACH ( apply_filters( 'mbr/site_custom_fields/get', NULL ) AS $cust_id => $cust_def) :
						IF ( $ac_field == strtolower( $cust_def['perstag'] ) ) :
							IF ( $cust_def['type'] == 'listbox' OR $cust_def['type'] == 'checkbox' ) :
								IF ( ! is_array( $value) ) :
									$value = [ $value ];
								ENDIF;
							ELSE:
								IF ( is_array( $value ) ) :
									$value = $value[0];
								ENDIF;
							ENDIF;
							$contact_data[ 'field[%' . strtoupper( $ac_field ) . '%]' ] = $value;
							break;
						ENDIF;
					ENDFOREACH;
				ENDIF;
			ENDIF;
		ENDFOREACH;

		$subscribe_lists    = json_decode( get_post_meta( $contact_form_id, $this->form_subscribe_lists_key,    TRUE ), TRUE );
		$unsubscribe_lists  = json_decode( get_post_meta( $contact_form_id, $this->form_unsubscribe_lists_key,  TRUE ), TRUE );
		$assign_tags        = json_decode( get_post_meta( $contact_form_id, $this->form_assign_tags_key,        TRUE ), TRUE );
		$remove_tags        = json_decode( get_post_meta( $contact_form_id, $this->form_remove_tags_key,        TRUE ), TRUE );
		$assign_automations = json_decode( get_post_meta( $contact_form_id, $this->form_assign_automations_key, TRUE ), TRUE );
		$remove_automations = json_decode( get_post_meta( $contact_form_id, $this->form_remove_automations_key, TRUE ), TRUE );

		IF ( ! empty( $contact_data['email'] ) AND is_email( $contact_data['email'] ) ) :
			$obj_contact         = apply_filters( 'mbr/api', "contact_view_email?email={$contact_data['email']}" );
			IF ( empty( $obj_contact->result_code ) ) :
				update_option( 'mbr_cf7_contact_created_' . $contact_data['email'], 'yes' );
			ENDIF;
		ENDIF;

		apply_filters( 'mbr/api/sync_ctla',
			[
			'contact_data'       => $contact_data,
			'assign_automations' => $assign_automations,
			'remove_automations' => $remove_automations,
			'assign_tags'        => $assign_tags,
			'remove_tags'        => $remove_tags,
			'subscribe_lists'    => $subscribe_lists,
			'unsubscribe_lists'  => $unsubscribe_lists
			]
		);

		$event_tracking_enabled = $MBR_OPTIONS['extensions']['mbr_ext_ac_event_tracking_enabled'];
		IF ( ! empty( $event_tracking_enabled ) ) :
			$log_event = get_post_meta( $contact_form_id, $this->form_log_event_key, TRUE );
			IF ( ! empty( $log_event ) AND $log_event == 'yes' ) :
				IF ( ! empty( $contact_data['email'] ) ) :
					$user_email = $contact_data['email'];
				ELSEIF ( $contact_id ) :
					$tmp_session = apply_filters( 'mbr/session/get', NULL );
					IF ( ! empty( $tmp_session['email'] ) ) :
						$user_email = $tmp_session['email'];
					ENDIF;
				ENDIF;
				IF ( ! empty( $user_email ) ) :
					$contact_form_title = get_the_title( $contact_form_id );
					do_action( 'mbr/track/log/event', $contact_form_title, MBRWPURL, $user_email );
				ENDIF;
			ENDIF;
		ENDIF;
	}

	public function mbr_cf7_redirect_after_form_submission()
	{
		?>
		<script>
			document.addEventListener( 'wpcf7mailsent', function( event ) {
				var inputs    = event.detail.inputs;
				var mbr_email = false;

				for ( var i = 0; i < inputs.length; i++ ) {
					if ( '<?php echo $this->mbr_crm_identifier; ?>-email' == inputs[i].name ) {
						mbr_email = inputs[i].value;
						break;
					}
				}

				jQuery.post( {
					url         : ajaxurl,
					dataType    : 'json',
					data        : {
						action    : 'mbr_ajax_cf7_redirect_url',
						form_id   : event.detail.contactFormId,
						mbr_email : mbr_email
					},
					success     : function ( response ) {
						if ( typeof response.redirect_url !== 'undefined' && response.redirect_url != '' ) {
							location = response.redirect_url;
						}
					}
				} );

			}, false );
		</script>
		<?php
	}
	public function allow_autologin( $email = FALSE )
	{
		$allow_autologin         = FALSE;
		IF ( ! empty( $email ) AND is_email( $email ) ) :

			IF ( ! get_user_by( 'email', $email ) ) :
				$obj_contact         = apply_filters( 'mbr/api', "contact_view_email?email={$email}" );
				IF ( empty( $obj_contact->result_code ) ) :
					$allow_autologin   = TRUE;
				ELSE:
					$contact_created   = get_option( 'mbr_cf7_contact_created_' . $email, 'no' );

					IF ( $contact_created == 'yes' ) :
						$allow_autologin = TRUE;
					ENDIF;

				ENDIF;
			ENDIF;

			delete_option( 'mbr_cf7_contact_created_' . $email );
		ENDIF;

		return $allow_autologin;
	}

	public function ajax_mbr_cf7_redirect()
	{
		global $MBR_OPTIONS;

		$contact_form_id        = $_POST['form_id'];
		$email                  = $_POST['mbr_email'];

		IF ( empty( $contact_form_id ) ) :
			return;
		ENDIF;

		$integrate              = get_post_meta( $contact_form_id, $this->form_enable_integration_key, TRUE );
		IF ( $integrate <> 'yes' ) :
			return;
		ENDIF;

		$found_autologin        = FALSE;
		IF ( ! empty( $MBR_OPTIONS['autologin_module'] ) AND $this->allow_autologin( $email ) ) :
			IF ( 'yes' == get_post_meta( $contact_form_id, $this->form_autologin_key, TRUE ) ) :
				$found_autologin    = TRUE;
			ENDIF;
		ENDIF;

		$found_redirect         = FALSE;
		IF ( $redirect_pid      = get_post_meta( $contact_form_id, $this->form_redirect_pid_key, TRUE ) ) :
			IF ( $redirect_url = get_permalink( $redirect_pid ) ) :
				$found_redirect     = TRUE;
			ENDIF;
		ENDIF;

		IF ( $found_redirect ) :
			IF ( $found_autologin ) :
				$arr_security_codes = unserialize( $MBR_OPTIONS['security_codes'] );
				IF ( ! empty( $arr_security_codes[0] ) AND ! empty( $email ) ) :
					$email            = urlencode( $email );
					$redirect_url     = home_url() . "?mbr_autologin={$arr_security_codes[0]}&email={$email}&redir={$redirect_url}";
				ENDIF;
			ENDIF;

			die( json_encode( [ 'redirect_url' => $redirect_url ] ) );
		ENDIF;

		IF ( $found_autologin ) :
			$arr_security_codes   = unserialize( $MBR_OPTIONS['security_codes'] );
			IF ( ! empty( $arr_security_codes[0] ) AND ! empty( $email ) ) :
				$email              = urlencode( $email );
				$redirect_url       = home_url() . "?mbr_autologin={$arr_security_codes[0]}&email={$email}";

				die( json_encode( [ 'redirect_url' => $redirect_url ] ) );
			ENDIF;
		ENDIF;

		exit;
	}

	public function mbr_cf7_skip_mail( $skip_mail, $contact_form )
	{
		$integrate = get_post_meta( $contact_form->id(), $this->form_enable_integration_key, TRUE );
		IF ( $integrate == 'no' ) :
			return $skip_mail;
		ENDIF;
		$skip_mail = get_post_meta( $contact_form->id(), $this->form_skip_mail_key, TRUE );

		return ( ( ! empty( $skip_mail ) AND $skip_mail == 'yes' ) ? TRUE : FALSE );
	}

	
	
	public function mbr_cf7_contact_form_properties( $properties, $thisOBJ )
	{
		IF ( $thisOBJ->id ) :
			FOREACH ( $this->arrFORMKEYS AS $key => $field ) :
			$properties[ substr( $field,1 ) ] = get_post_meta( $thisOBJ->id, $field, TRUE );
			ENDFOREACH;
		ENDIF;

		return $properties;
	}

	
	public function mbr_cf7_form_tag( $scanned_tag, $exec )
	{
		$contact_form_id = wpcf7_get_current_contact_form()->id;
		IF ( empty( $scanned_tag['name'] ) OR ! apply_filters( 'mbr/contact_id', NULL ) OR 'yes' <> get_post_meta( $contact_form_id, $this->form_enable_integration_key, TRUE ) ) :
			return $scanned_tag;
		ENDIF;

		IF ( stripos($scanned_tag['name'], $this->mbr_crm_identifier . '-' ) === FALSE ) :
			return $scanned_tag;
		ENDIF;

		$strlen_ac_identifier = strlen( $this->mbr_crm_identifier ) + 1;
		$ac_field_name        = substr( $scanned_tag['name'], $strlen_ac_identifier );

		IF ( in_array( $scanned_tag['basetype'], ['checkbox', 'select', 'radio'] ) ) :
			$arr_defaults     = $scanned_tag['values'];
			$arr_default_keys = [];

			
			
			FOREACH ( $arr_defaults AS $k => $option ) :
				$arr_ac_values = apply_filters( 'mbr/get_contact_field', $ac_field_name );
				$arr_ac_values = trim( $arr_ac_values, '||' );
				$arr_ac_values = explode( '||', $arr_ac_values );
				IF ( in_array( $option, $arr_ac_values ) ) :
					$arr_default_keys[] = $k+1;
				ENDIF;
			ENDFOREACH;

			
			
			IF ( ! empty( $scanned_tag['options'] ) ) :
				FOREACH ( $scanned_tag['options'] AS $key => $option ) :
					IF ( stripos( $option, 'default:' ) !== FALSE ) :
						unset( $scanned_tag['options'][ $key ] );
					ENDIF;
				ENDFOREACH;
			ENDIF;
			
			$scanned_tag['options'][] = 'default:' . implode( '_', $arr_default_keys );
			
			$scanned_tag['options']   = array_values( $scanned_tag['options'] );

		ELSEIF ( $scanned_tag['basetype'] == 'email' ) :
			
			IF ( ! empty( $scanned_tag['options'] ) ) :
				FOREACH ( $scanned_tag['options'] AS $key => $option ) :
					IF ( strtolower( $option ) == 'placeholder' OR strtolower( $option ) == 'readonly' ) :
						unset( $scanned_tag['options'][ $key ] );
					ENDIF;
				ENDFOREACH;
			ENDIF;
			$scanned_tag['options'][]     = 'readonly';
			$scanned_tag['options']       = array_values( $scanned_tag['options'] );
			$scanned_tag['raw_values'][0] = $scanned_tag['values'][0] = $scanned_tag['labels'][0] = apply_filters( 'mbr/get_contact_field', 'email' );
		ELSE:
			$scanned_tag['raw_values'][0] = $scanned_tag['values'][0] = $scanned_tag['labels'][0] = apply_filters( 'mbr/get_contact_field', $ac_field_name );
		ENDIF;

		return $scanned_tag;
	}

}
$GLOBALS['mbrcf7'] = mbr_contactform7::getInstance();