<?php
final class mbr_contacts EXTENDS mbr_redux_base_class
{
	use TRAIT__getInstance;

	public $opt_name  = 'mbr_contacts';
	public $page_slug = 'mbr360-contacts';

	public function _load()
	{
		IF ( is_admin() ) :
			add_action( 'plugins_loaded',                                                    [ $this, 'init_admin_panel' ], 99999 );
			add_action( 'plugins_loaded',                                                    [ $this, 'init_sections' ],    99999 );
			add_filter( "redux/{$this->opt_name}/panel/template/header.tpl.php",             [ $this, 'alternate_header' ] );
			add_filter( "redux/{$this->opt_name}/panel/template/container.tpl.php",          [ $this, 'alternate_container' ] );
			add_action( 'wp_ajax_mbr_con_get_contact_info_results',                          [ $this, 'ajax_get_contact_info_results' ] );
			add_action( 'wp_ajax_mbr_con_get_contact_email_change_results',                  [ $this, 'ajax_get_contact_email_change_results' ] );
			add_action( 'wp_ajax_mbr_con_get_contact_password_change_results',               [ $this, 'ajax_get_contact_password_change_results' ] );
		ENDIF;
	}

	public function alternate_container()
	{
		return MBR_DIR . '/templates/container_contacts.tpl.php';
	}

	public function init_admin_panel()
	{
		global $MBR_REDUX_HEADER_MODULE_NAMES;

		$MBR_REDUX_HEADER_MODULE_NAMES[ $this->opt_name ] = $admin_title = _x( 'Contacts', 'wp_admin', 'activemember360' );

		$page_title         = apply_filters( 'mbr/plugin_name', '' ) . ' ' . $admin_title;

		$arr_config         =  [
			'opt_name'        => $this->opt_name,
			'global_variable' => $this->opt_name,
			'page_slug'       => $this->page_slug,
			'page_parent'     => MBR_MENU_PARENT_PAGE,
			'page_title'      => $page_title,
			'display_name'    => $page_title,
			'admin_title'     => $admin_title,
			'menu_title'      => $admin_title,
			'settings_api'    => FALSE,
			'hide_save'       => TRUE,
		];

		$arr_config         = array_merge( $this->arr_default_sub_admin_panel_settings, $arr_config );

		Redux::setArgs( $this->opt_name, $arr_config );
	}

	public function init_sections()
	{
		global $pagenow;

		IF ( $pagenow <> 'options.php' AND $pagenow <> 'admin.php' OR ( $pagenow == 'admin.php' AND ( empty( $_GET['page'] ) OR $_GET['page'] <> $this->page_slug ) ) ) :
			return;
		ENDIF;

		


		$content_contact_info       = "
		<script>
			jQuery( document ).ready( function() {
				jQuery( '#mbr_con_get_contact_info_results' ).click( function () {
					jQuery( '#mbr_con_contact_info_results_container' ).html('');
					mbr_send_con_ajax( 'mbr_con_get_contact_info_results', '#mbr_con_contact_info_results_container', jQuery( '#mbr_con_contact_id').val(), jQuery( '#mbr_con_contact_email' ).val() );
					return false;
				} );
			} );
		</script>";
		$content_contact_info      .= '<p><span id="mbr_con_get_contact_info_results" class="button button-primary">' . _x( 'Search Contact', 'wp_admin', 'activemember360' ) . '</span></p>';

		$arr_section_contact_info   =
		[
			'id'                      => 'section_contact_info',
			'title'                   => _x( 'Show Contact Info', 'wp_admin', 'activemember360' ),
			'desc'                    => _x( 'Look up a contact record in ActiveCampaign by contact ID or email address', 'wp_admin', 'activemember360' ),
			'icon'                    => 'el el-user',
			'fields'                  => [
				[
					'id'                  => 'mbr_con_contact_id',
					'title'               => _x( 'Contact ID', 'wp_admin', 'activemember360' ),
					'type'                => 'text'
				],
				[
					'id'                  => 'mbr_con_contact_email',
					'title'               => _x( 'Email Address', 'wp_admin', 'activemember360' ),
					'type'                => 'text'
				],
				[
					'id'                  => 'mbr_con_contact_email_change',
					'full_width'          => TRUE,
					'type'                => 'raw',
					'content'             => $content_contact_info . '<br /><div id="mbr_con_contact_info_results_container"></div>'
				]
			]
		];

		


		$content_contact_email_change     = "
		<script>
			jQuery( document ).ready( function() {
				jQuery( '#mbr_con_get_contact_email_change_results' ).click( function () {
					jQuery( '#mbr_con_contact_email_change_results_container' ).html('');
					mbr_send_con_ajax( 'mbr_con_get_contact_email_change_results', '#mbr_con_contact_email_change_results_container', jQuery( '#mbr_con_contact_email_current').val(), jQuery( '#mbr_con_contact_email_new_1' ).val(), jQuery( '#mbr_con_contact_email_new_2' ).val() );
					return false;
				} );
			} );
		</script>";
		$content_contact_email_change    .= '<p><span id="mbr_con_get_contact_email_change_results" class="button button-primary">' . _x( 'Change Email Address', 'wp_admin', 'activemember360' ) . '</span></p>';

		$arr_section_contact_email_change =
		[
			'id'                      => 'section_contact_email_change',
			'title'                   => _x( 'Change Contact Email', 'wp_admin', 'activemember360' ),
			'desc'                    => _x( 'Before changing a user\'s email, please consider whether they are double-optin users or not.', 'wp_admin', 'activemember360' ),
			'icon'                    => 'el el-envelope',
			'fields'                  => [
				[
					'id'                  => 'mbr_con_contact_email_change_1',
					'full_width'          => TRUE,
					'type'                => 'raw',
					'content'             => '<div id="mbr_con_contact_email_change_results_container"></div>'
				],
				[
					'id'                  => 'mbr_con_contact_email_current',
					'title'               => _x( 'Current Email Address', 'wp_admin', 'activemember360' ),
					'type'                => 'text'
				],
				[
					'id'                  => 'mbr_con_contact_email_new_1',
					'title'               => _x( 'New Email Address', 'wp_admin', 'activemember360' ),
					'type'                => 'text'
				],
				[
					'id'                  => 'mbr_con_contact_email_new_2',
					'title'               => _x( 'Repeat New Email Address', 'wp_admin', 'activemember360' ),
					'type'                => 'text'
				],
				[
					'id'                  => 'mbr_con_contact_email_change_2',
					'full_width'          => TRUE,
					'type'                => 'raw',
					'content'             => $content_contact_email_change
				]
			]
		];

		 


		$content_contact_password_change     = "
		<script>
			jQuery( document ).ready( function() {
				jQuery( '#mbr_con_get_contact_password_change_results' ).click( function () {
					jQuery( '#mbr_con_contact_password_change_results_container' ).html('');
					mbr_send_con_ajax( 'mbr_con_get_contact_password_change_results', '#mbr_con_contact_password_change_results_container', jQuery( '#mbr_con_contact_password_email_current').val(), jQuery( '#mbr_con_contact_password_new_1' ).val(), jQuery( '#mbr_con_contact_password_new_2' ).val() );
					return false;
				} );
			} );
		</script>";
		$content_contact_password_change    .= '<p><span id="mbr_con_get_contact_password_change_results" class="button button-primary">' . _x( 'Change Password', 'wp_admin', 'activemember360' ) . '</span></p>';

		$arr_section_contact_password_change =
		[
			'id'                      => 'section_contact_password_change',
			'title'                   => _x( 'Change Contact Password', 'wp_admin', 'activemember360' ),
			'icon'                    => 'el el-lock',
			'fields'                  => [
				[
					'id'                  => 'mbr_con_contact_password_change_1',
					'full_width'          => TRUE,
					'type'                => 'raw',
					'content'             => '<div id="mbr_con_contact_password_change_results_container"></div>'
				],
				[
					'id'                  => 'mbr_con_contact_password_email_current',
					'title'               => _x( 'Current Email Address', 'wp_admin', 'activemember360' ),
					'type'                => 'text'
				],
				[
					'id'                  => 'mbr_con_contact_password_new_1',
					'title'               => _x( 'New Password', 'wp_admin', 'activemember360' ),
					'type'                => 'text'
				],
				[
					'id'                  => 'mbr_con_contact_password_new_2',
					'title'               => _x( 'Repeat New Password', 'wp_admin', 'activemember360' ),
					'type'                => 'text'
				],
				[
					'id'                  => 'mbr_con_contact_password_change_2',
					'full_width'          => TRUE,
					'type'                => 'raw',
					'content'             => $content_contact_password_change
				]
			]
		];

		
		Redux::setSection( $this->opt_name, $arr_section_contact_info );
		Redux::setSection( $this->opt_name, $arr_section_contact_email_change );
		Redux::setSection( $this->opt_name, $arr_section_contact_password_change );
	}

	public function ajax_get_contact_info_results()
	{
		$contact_id     = ! empty( $_POST['parm1'] ) ? trim( $_POST['parm1'] ) : FALSE;
		$email_address  = ! empty( $_POST['parm2'] ) ? trim( $_POST['parm2'] ) : FALSE;

		IF ( $contact_id ) : 
			$query        = [ 'local' => FALSE, 'id' => $contact_id ];
			$search_field = 'contact ID';
			$search_val   = $_POST['parm1'];
		ELSEIF ( $email_address ) : 
			$query        = [ 'local' => FALSE, 'email' => $email_address ];
			$search_field = 'email address';
			$search_val   = $_POST['parm2'];
		ELSE :
			$query        = FALSE;
		ENDIF;

		$out            = '';
		IF ( $query ) :
			$out         .= '<h2>' . sprintf( _x( 'Found contact with %s: %s', 'wp_admin', 'activemember360' ), $search_field, $search_val ) . '</h2>';
			$out         .= '<table class="wp-list-table widefat striped mbr-redux-listing mbr-col-300">';
			$out         .= '<thead><tr><td>' . _x( 'Field', 'wp_admin', 'activemember360' ). '</td><td>' . _x( 'Value', 'wp_admin', 'activemember360' ) . '</td></tr></thead>';
			$out         .= '<tfoot><tr><td>' . _x( 'Field', 'wp_admin', 'activemember360' ). '</td><td>' . _x( 'Value', 'wp_admin', 'activemember360' ) . '</td></tr></tfoot>';
			$out         .= '<tbody id="the-list">';
			IF ( $arr_contact = apply_filters( 'mbr/fetch_contact', $query, FALSE ) ) :
				FOREACH ( $arr_contact AS $name => $value ) :
					$out     .= '<tr><td><strong>' . $name . '</strong></td><td>' . $value . '</td></tr>';
				ENDFOREACH;
			ELSE:
				$out       .= '<tr><td colspan="2">' . _x( 'No contact found in ActiveCampaign', 'wp_admin', 'activemember360' ) . '</td></tr>';
			ENDIF;
			$out         .= '</tbody>';
			$out         .= '</table>';
		ENDIF;

		echo $out;
		exit;
	}

	public function ajax_get_contact_email_change_results()
	{
		global $wpdb;

		$email_current = ! empty( $_POST['parm1'] ) ? trim( $_POST['parm1'] ) : FALSE;
		$email_new_1   = ! empty( $_POST['parm2'] ) ? trim( $_POST['parm2'] ) : FALSE;
		$email_new_2   = ! empty( $_POST['parm3'] ) ? trim( $_POST['parm3'] ) : FALSE;

		IF ( empty( $email_current ) ) :
			echo mbr_admin_msg( _x( "Please enter a 'current' email address.", 'wp_admin', 'activemember360' ), TRUE, TRUE );
			exit;
		ENDIF;

		IF ( $email_new_1 <> $email_new_2 ) :
			echo mbr_admin_msg( _x( "Please ensure that the new email address is identical in both fields.", 'wp_admin', 'activemember360' ), TRUE, TRUE );
			exit;
		ENDIF;

		IF ( empty( $email_new_1 ) ) :
			echo mbr_admin_msg( _x( "Please enter a 'new' email address.", 'wp_admin', 'activemember360' ), TRUE, TRUE );
			exit;
		ENDIF;

		IF ( $arr_contact = apply_filters( 'mbr/fetch_contact', [ 'email' => $email_current ], FALSE ) ) :
			$cid  = $arr_contact['id'];
			IF ( ! $arr_check_contact = apply_filters( 'mbr/fetch_contact', [ 'email' => $email_new_1 ], FALSE ) ) :
				IF ( apply_filters( 'mbr/save_contact_fields', [ 'id' => $cid, "email" => $email_new_1 ] ) ) :
					$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->tbl_contact} SET email=%s WHERE contact_id=%d", $email_new_1, $cid ) );
					IF ( FALSE == $wpdb->update( $wpdb->users, [ 'user_login' => $email_new_1, 'user_email' => $email_new_1 ], ['user_login' => $email_current ] ) ) :
						IF ( FALSE == $wpdb->update( $wpdb->users, [ 'user_email' => $email_new_1 ], ['user_email' => $email_current ] ) ) :
							echo mbr_admin_msg( _x( 'Your Wordpress data could not be updated.', 'wp_admin', 'activemember360' ), TRUE, TRUE );
							exit;
						ENDIF;
					ENDIF;
				ELSE:
					echo mbr_admin_msg( _x( 'Contact in ActiveCampaign could not be updated.', 'wp_admin', 'activemember360' ), TRUE, TRUE );
					exit;
				ENDIF;
			ELSE:
				echo mbr_admin_msg( _x( 'This email address is defined multiple times in ActiveCampaign.', 'wp_admin', 'activemember360' ), TRUE, TRUE );
				exit;
			ENDIF;
		ELSE:
			echo mbr_admin_msg( _x( 'This email address was not found in ActiveCampaign.', 'wp_admin', 'activemember360' ), TRUE, TRUE );
			exit;
		ENDIF;

		echo  mbr_admin_msg( _x( "The contact's email address was successfully changed in ActiveCampaign and Wordpress.", 'wp_admin', 'activemember360' ), FALSE, TRUE);
		exit;
	}

	public function ajax_get_contact_password_change_results()
	{
		global $wpdb;

		$email_current  = ! empty( $_POST['parm1'] ) ? trim( $_POST['parm1'] ) : FALSE;
		$password_new_1 = ! empty( $_POST['parm2'] ) ? trim( $_POST['parm2'] ) : FALSE;
		$password_new_2 = ! empty( $_POST['parm3'] ) ? trim( $_POST['parm3'] ) : FALSE;

		IF ( empty( $email_current ) ) :
			echo mbr_admin_msg( _x( "Please enter a 'current' email address.", 'wp_admin', 'activemember360' ), TRUE, TRUE );
			exit;
		ENDIF;

		IF ( $password_new_1 <> $password_new_2 ) :
			echo mbr_admin_msg( _x( "Please ensure that the new password is identical in both fields.", 'wp_admin', 'activemember360' ), TRUE, TRUE );
			exit;
		ENDIF;

		IF ( empty( $password_new_1 ) ) :
			echo mbr_admin_msg( _x( "Please enter a 'new' password.", 'wp_admin', 'activemember360' ), TRUE, TRUE );
			exit;
		ENDIF;

		IF ( $arr_contact = apply_filters( 'mbr/fetch_contact', [ 'email' => $email_current ], FALSE ) ) :
			$cid              = $arr_contact['id'];
			$password         = apply_filters( 'mbr/password/hash', $password_new_1 );
			$password_perstag = apply_filters( 'mbr/password_perstag', NULL );
			IF ( apply_filters( 'mbr/save_contact_fields', [ 'id' => $cid, $password_perstag => $password ] ) ) :
				IF ( FALSE === $wpdb->update( $wpdb->users, [ 'user_pass' => wp_hash_password( $password_new_1 ), 'user_activation_key' => '' ], [ 'user_login' => $email_current ] ) ) :
					echo mbr_admin_msg( _x( 'Your Wordpress data could not be updated.', 'wp_admin', 'activemember360' ), TRUE, TRUE );
					exit;
				ENDIF;
			ELSE:
				echo mbr_admin_msg( _x( 'Contact in ActiveCampaign could not be updated.', 'wp_admin', 'activemember360' ), TRUE, TRUE );
				exit;
			ENDIF;

		ELSE:
			echo mbr_admin_msg( _x( 'This email address was not found in ActiveCampaign.', 'wp_admin', 'activemember360' ), TRUE, TRUE );
			exit;
		ENDIF;

		echo  mbr_admin_msg( _x( "The contact's password was successfully changed in ActiveCampaign and Wordpress.", 'wp_admin', 'activemember360' ), FALSE, TRUE);
		exit;
	}

}
$GLOBALS['mbr_inst_contacts'] = mbr_contacts::getInstance();
