<?php
add_action('mbr/classes/init', 'mbr_endpoints_instance', 20);

function mbr_endpoints_instance()
{
	IF (class_exists('mbr_endpoints')) :
		$GLOBALS['mbrendpoints'] = mbr_endpoints::getinstance();
		function MBRENDPOINTS()
		{
			return $GLOBALS['mbrendpoints'];
		}
	ENDIF;
}

class mbr_endpoints
{
	use TRAIT__getInstance;

	
	const MBR_ROUTE_PREFIX = 'mbr';
	const MBR_HOOK_PREFIX  = 'mbr/endpoints/';

	function _load()
	{
		add_action( 'init',          [ $this, 'rewrite_endpoints' ] );
		add_action( 'init',          [ $this, 'register_route_prefixes' ], 10, 0 );
		add_action( 'wp_loaded',     [ $this, 'flush_rewrite_rules' ] );
		add_action( 'rest_api_init', [ $this, 'register_routes_process' ] );
	}

	function flush_rewrite_rules()
	{
		static $flushed = null;
		IF ( is_null( $flushed ) ) :
			$GLOBALS['wp_rewrite']->flush_rules( FALSE );
			$flushed = true;
		ENDIF;
	}

	function rewrite_endpoints()
	{
		$rewrite_endpoints = apply_filters( 'mbr/endpoints/rewrite', [] );
		FOREACH ($rewrite_endpoints AS $endpoint => $places) :
			add_rewrite_endpoint( $endpoint, $places );
		ENDFOREACH;
	}

	function register_route_prefixes()
	{
		global $wp_rewrite;
		$route_prefixes = apply_filters( 'mbr/route/prefix', [ self::MBR_ROUTE_PREFIX ] );
		FOREACH ($route_prefixes AS $k => $route_prefix) :
			IF ( !empty( $route_prefix ) ) :

				add_rewrite_rule( '^' . $route_prefix . '/?$', 'index.php?rest_route=/', 'top' );
				add_rewrite_rule( '^' . $route_prefix . '/(.*)?', 'index.php?rest_route=/$matches[1]', 'top' );

				add_rewrite_rule( '^' . $wp_rewrite->index . '/' . $route_prefix . '/?$', 'index.php?rest_route=/', 'top' );
				add_rewrite_rule( '^' . $wp_rewrite->index . '/' . $route_prefix . '/(.*)?', 'index.php?rest_route=/$matches[1]', 'top' );
			ENDIF;
		ENDFOREACH;
	}

	function register_routes_process()
	{
		














		$route_args = [];
		$route_args['password']['operations']['create'] = [$this, 'password_create'];
		$route_args['password']['operations']['send'] = [$this, 'password_send'];

		$math_operations = ['add', 'subtract', 'divide', 'multiply', 'power', 'modulus', 'squareroot', 'max', 'min'];
		FOREACH ($math_operations AS $operation) :
			$route_args['math']['operations'][$operation] = [$this, 'math_operations'];
		ENDFOREACH;

		
		$route_args = apply_filters( 'mbr/routes/register', $route_args );
		IF ( is_admin() ) :
			
			$route_args = apply_filters( 'mbr/routes/register/admin', $route_args );
		ELSE :
			
			$route_args = apply_filters( 'mbr/routes/register/nonadmin', $route_args );
		ENDIF;

		FOREACH ($route_args AS $namespace => $arg) :
			IF ( empty( $arg['methods'] ) ) :
				$arg['methods'] = ['POST', 'GET'];
			ENDIF;
			FOREACH ((array)$arg['methods'] AS $method) :
				FOREACH ($arg['operations'] AS $operation => $callback) :
					register_rest_route( $namespace . '/' . $operation . '/', '(?P<seccode>[\S]+)', ['methods' => $method, 'callback' => $callback] );
				ENDFOREACH;
			ENDFOREACH;
		ENDFOREACH;
	}

	function password_send( $request )
	{
	}

	function password_create( $request )
	{
		$thisID = $this->getContactIdFromPOST();
		IF ( $thisID === FALSE ) :
			mbr_error412( 'ID Precondition Failed' );
		ENDIF;

		$password_existed = FALSE;
		$passfield        = mbr_passfield();
		$password_perstag = mbr_password_perstag();
		$thisPASS         = '';
		$arrCONTACT       = mbr_get_contact_by_id( $thisID, FALSE ); 

		IF ( empty( $arrCONTACT[$passfield] ) ) :
			IF ( !empty( $_REQUEST['default_pass'] ) ) :
				$thisPASS = trim( $_REQUEST['default_pass'] );
			ELSE :
				$thisPASS = mbr_generate_password();
			ENDIF;
			mbr_save_contact_fields( ['id' => $thisID, $password_perstag => $thisPASS] );
		ELSE :
			$password_existed = TRUE;
			$thisPASS = $arrCONTACT[$passfield];
		ENDIF;

		IF ( !empty( $_REQUEST['autoregister'] ) AND strtolower( $_REQUEST['autoregister'] ) == 'n' ) :
			header( $_SERVER["SERVER_PROTOCOL"] . ' 200 OK' );
			die( 'password/generate => ' . ($password_existed ? 'password existed' : 'password generated') );
		ENDIF;

		$default_role = blog_get_option( 'default_role' );
		$alternate_role = false;

		IF ( has_filter( 'mbr_alternate_role' ) ) :
			$alternate_role = apply_filters( 'mbr_alternate_role', $default_role, explode( ',', $arrCONTACT['tags'] ) );
		ENDIF;

		IF ( isset( $arrCONTACT['email'] ) ) :
			$checkNAME = username_exists( $arrCONTACT['email'] );
			$checkMAIL = email_exists( $arrCONTACT['email'] );
			
			IF ( ($checkNAME AND $checkMAIL AND $checkNAME <> $checkMAIL) OR (!$checkNAME AND $checkMAIL) ) :
				exit();
			ENDIF;
			$pUSERNAME = $arrCONTACT['email'];
			$pPASSWORD = $arrCONTACT[$passfield];

			
			add_filter( 'send_password_change_email', '__return_false' );

			IF ( !$checkNAME ) :

				$display_name = $arrCONTACT['first_name'] . ' ' . $arrCONTACT['last_name'];
				$user_nicename = str_ireplace( ' ', '-', strtolower( $display_name ) );
				$nickname = $arrCONTACT['first_name'];
				$user_nicename = apply_filters( 'mbr_nicename', $user_nicename, $arrCONTACT['first_name'], $arrCONTACT['last_name'], $arrCONTACT );
				$display_name = apply_filters( 'mbr_display_name', $display_name, $arrCONTACT['first_name'], $arrCONTACT['last_name'], $arrCONTACT );
				$nickname = apply_filters( 'mbr_nickname', $nickname, $arrCONTACT['first_name'], $arrCONTACT['last_name'], $arrCONTACT );

				$arrUSER = [
					'user_login' => $pUSERNAME,
					'user_pass' => $thisPASS,
					'user_email' => $pUSERNAME,
					'first_name' => $arrCONTACT['first_name'],
					'last_name' => $arrCONTACT['last_name'],
					'user_nicename' => $user_nicename,
					'display_name' => $display_name,
					'nickname' => $nickname
				];

				IF ( $alternate_role ) :
					$arrUSER['role'] = $alternate_role;
				ENDIF;

				$thisUID = wp_insert_user( $arrUSER );
				wp_cache_delete( $thisUID, 'users' );

				IF ( function_exists( 'WPCW_actions_users_newUserCreated' ) ) :
					WPCW_actions_users_newUserCreated( $thisUID );
				ENDIF;

			ELSE :

				$thisUID = $checkNAME;

				IF ( $alternate_role ) :
					wp_cache_delete( $thisUID, 'users' );
					wp_update_user( ['ID' => $thisUID, 'role' => $alternate_role] );
				ENDIF;

			ENDIF;

			update_user_meta( $thisUID, 'mbr_user_id', $arrCONTACT['id'] );

		ENDIF;

		IF ( isset( $_REQUEST['tid'] ) AND trim( $_REQUEST['tid'] ) <> '' ) :
			$arrTID = explode( ',', $_REQUEST['tid'] );
			$arrTAGStoADD = [];
			$arrTAGS = explode( ',', $arrCONTACT['tag_ids'] );
			FOREACH ($arrTID as $ktid => $tid) :
				IF ( !in_array( $tid, $arrTAGS ) ) :
					$arrTAGStoADD[] = mbr_get_site_tag( $tid );
				ENDIF;
			ENDFOREACH;
			IF ( $arrTAGStoADD ) :
				mbr_api( 'contact_tag_add', ['id' => $arrCONTACT['id'], 'tags' => $arrTAGStoADD] );
			ENDIF;
		ENDIF;

		IF ( isset( $_REQUEST['automation'] ) AND $_REQUEST['automation'] ) :
			mbr_api( "automation_contact_add", ['contact_id' => $arrCONTACT['id'], 'automation' => $_REQUEST['automation']] );
		ENDIF;

		mbr_clear_ob();
		header( $_SERVER["SERVER_PROTOCOL"] . ' 200 OK' );

		die( 'password/generate => ' . ($password_existed ? 'password existed' : 'password created') );
	}

	function math_operations( $request )
	{
		IF ( !$thisID = $this->getContactIdFromPOST() ) :
			mbr_error412( 'ID Precondition Failed' );
		ENDIF;

		$arrCONTACT = mbr_get_contact_by_id( $thisID, false );
		IF ( empty( $arrCONTACT['email'] ) ) :
			exit;
		ENDIF;

		list( , $namespace, $operation, $code ) = explode( '/', $route = get_protected_object_property( $request, 'route' ) );
		$operation = strtolower( $operation );

		$req = [];
		FOREACH ($_REQUEST as $field => $value) :
			$lcfield = strtolower( $field );
			IF ( in_array( $lcfield, ['field', 'value', 'round', 'decimals', 'target'] ) ) :
				$req[$lcfield] = $value;
			ENDIF;
		ENDFOREACH;

		IF ( empty( $req['field'] ) ) :
			mbr_error412( 'FLD Precondition Failed' );
		ENDIF;

		
		IF ( !isset( $req['value'] ) AND $operation <> 'squareroot' ) :
			mbr_error412( 'VAL Precondition Failed' );
		ENDIF;

		IF ( empty( $req['round'] ) ) :
			$req['round'] = 0;
		ENDIF;

		IF ( empty( $req['decimals'] ) ) :
			$req['round'] = 0;
		ENDIF;

		IF ( empty( $req['target'] ) ) :
			$req['target'] = $req['field'];
		ENDIF;

		$field = $req['field'];
		$value = $req['value'];
		$round = $req['round'];
		$target = $req['target'];

		IF ( empty( $operation ) OR !in_array( $operation, $this->math_operations ) ) :
			mbr_error412( 'OPN Precondition Failed' );
		ENDIF;

		IF ( in_array( $operation, ['divide', 'modulus'] ) AND $value == 0 ) :
			mbr_error412( 'DV0 Precondition Failed' );
		ENDIF;

		$op1 = 0;
		IF ( isset( $_REQUEST[$field] ) ) :
			$op1 = (float)$_REQUEST[$field];
		ELSEIF ( $getVALUE = $arrCONTACT[$field] ) :
			$op1 = (float)$getVALUE;
		ELSE :
			$op1 = 0;
		ENDIF;

		$op2 = $value;
		IF ( is_string( $value ) ) :
			IF ( isset( $_REQUEST[$value] ) ) :
				$op2 = (float)$_REQUEST[$value];
			ELSEIF ( $getVALUE = $arrCONTACT[$value] ) :
				$op2 = (float)$getVALUE;
			ENDIF;
		ENDIF;
		$op2 = (float)$op2;

		$new_value = 0;
		IF ( $operation == 'add' ) :
			$new_value = $op1 + $op2;
		ELSEIF ( $operation == 'subtract' ) :
			$new_value = $op1 - $op2;
		ELSEIF ( $operation == 'divide' ) :
			$new_value = ($op1 == 0 OR $op2 == 0) ? 0 : $op1 / $op2;
		ELSEIF ( $operation == 'multiply' ) :
			$new_value = $op1 * $op2;
		ELSEIF ( $operation == 'power' ) :
			$new_value = pow( $op1, $op2 );
		ELSEIF ( $operation == 'modulus' ) :
			$new_value = ($op1 == 0 OR $op2 == 0) ? 0 : $op1 % $op2;
		ELSEIF ( $operation == 'squareroot' ) :
			$new_value = sqrt( $op1 );
		ELSEIF ( $operation == 'max' ) :
			$new_value = max( $op1, $op2 );
		ELSEIF ( $operation == 'min' ) :
			$new_value = min( $op1, $op2 );
		ENDIF;

		$round = (int)$round;
		$new_value = round( $new_value, $round );
		IF ( $decimals == 0 ) :
			$new_value = (int)$new_value;
		ENDIF;

		IF ( in_array( $target, mbr_contact_std_fields() ) ) :
			$arrUPD[trim( strtolower( $target ) )] = $value;
		ELSE:
			FOREACH (mbr_site_cust_fields() AS $cust_id => $cust_def) :
				IF ( $target == strtolower( $cust_def['perstag'] ) ) :
					$arrUPD['field[%' . strtoupper( $target ) . '%]'] = $new_value;
					break;
				ENDIF;
			ENDFOREACH;
		ENDIF;

		IF ( !empty( $arrUPD ) ) :
			$arrUPD['email'] = $arrCONTACT['email'];
			$contact_sync = mbr_api( 'contact_sync', $arrUPD );
			IF ( $contact_sync->result_code == 1 ) :
				mbr_http200( $target . ' was updated with the value ' . $new_value . ' for contact ID ' . $thisID );
			ENDIF;
		ENDIF;

		mbr_http200( 'An error may have occurred while updating ' . $field . ' with the value ' . $new_value . ' for contact ID ' . $thisID );
	}

	function getContactIdFromPost()
	{
		
		IF ( !empty( $_REQUEST['contact']['id'] ) ) :
			IF ( is_numeric( $_REQUEST['contact']['id'] ) ) :
				
				$id = absint( $_REQUEST['contact']['id'] );
			ENDIF;
		ENDIF;
		
		IF ( !empty( $id ) ) :
			return $id;
		ENDIF;
		

		return FALSE;
	}

}
