<?php
final class mbr_extensions EXTENDS mbr_redux_base_class
{
	use TRAIT__getInstance;

	public $opt_name  = 'mbr_extensions';
	public $page_slug = 'mbr360-extensions';

	public function _load()
	{
		IF ( is_admin() ) :
			add_action( 'plugins_loaded',                                                   [ $this, 'init_admin_panel' ],                 99999 );
			add_action( 'plugins_loaded',                                                   [ $this, 'init_sections' ],                    99999 );
			add_filter( "redux/{$this->opt_name}/panel/template/header.tpl.php",            [ $this, 'alternate_header' ] );
			add_action( "redux/options/{$this->opt_name}/saved",                            [ $this, 'badgeos_specific_extensions_saved'], 9999, 2 );
			add_action( "redux/options/{$this->opt_name}/saved",                            [ $this, 'mailer_specific_extensions_saved'],  9999, 2 );
			add_action( "redux/options/{$this->opt_name}/saved",                            [ $this, 'backup_settings' ],  9999 );

			add_action( 'mbr/extensions/badgeos/steps/activate',                            [ $this, 'badgeos_completion_steps_actions' ], 10, 2 );
			add_action( 'mbr/extensions/badgeos/steps/deactivate',                          [ $this, 'badgeos_completion_steps_actions' ], 10, 2 );
			add_action( 'mbr/extensions/badgeos/steps/delete',                              [ $this, 'badgeos_completion_steps_actions' ], 10, 2 );
			add_action( 'mbr/extensions/tracking/events/remove/all',                        [ $this, 'tracking_remove_events'] );
		ENDIF;

		add_action( 'mbr_plugin_deactivation',                                            [ $this, 'tracking_remove_events'] );
	}

	public function init_admin_panel()
	{
		global $MBR_REDUX_HEADER_MODULE_NAMES;

		$MBR_REDUX_HEADER_MODULE_NAMES[ $this->opt_name ] = $admin_title = _x( 'Extensions', 'wp_admin', 'activemember360' );

		$page_title         = apply_filters( 'mbr/plugin_name', '' ) . ' ' . $admin_title;

		$arr_config         =  [
			'opt_name'        => $this->opt_name,
			'global_variable' => $this->opt_name,
			'page_slug'       => $this->page_slug,
			'page_parent'     => MBR_MENU_PARENT_PAGE,
			'page_title'      => $page_title,
			'display_name'    => $page_title,
			'admin_title'     => $admin_title,
			'menu_title'      => $admin_title,
		];

		$arr_config         = array_merge( $this->arr_default_sub_admin_panel_settings, $arr_config );

		Redux::setArgs( $this->opt_name, $arr_config );
	}

	public function init_sections()
	{
		global $pagenow;

		IF ( $pagenow <> 'options.php' AND $pagenow <> 'admin.php' OR ( $pagenow == 'admin.php' AND ( empty( $_GET['page'] ) OR $_GET['page'] <> $this->page_slug ) ) ) :
			return;
		ENDIF;

		IF ( ! function_exists( 'is_plugin_active' ) ) :
			include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		ENDIF;

		


		$arr_section_alternate_member_menu =
		[
			'id'           => 'section_alternate_member_menu',
			'title'        => _x( 'Alt. Membership Menus', 'wp_admin', 'activemember360' ),
			'heading'      => _x( 'Alternate Membership Menus', 'wp_admin', 'activemember360' ),
			'desc'         => _x( 'This option activates ActiveMember360\'s alternate menus for various levels of memberships.', 'wp_admin', 'activemember360' ),
			'icon'         => 'el el-th-list',
			'fields'       => [
				[
					'id'       => 'mbr_ext_alternate_member_menu_enabled',
					'title'    => _x( 'Enable Alternate Membership Menus extension', 'wp_admin', 'activemember360' ),
					'type'     => 'switch'
				]
			]
		];

		$arr_section_alternate_member_menu['fields'][] =
		[
			'id'           => 'mbr_ext_alternate_member_menu_notes',
			'title'        => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'type'         => 'textarea'
		];

		


		$arr_section_badgeos =
		[
			'id'         => 'section_badgeos',
			'title'      => _x( 'BadgeOS', 'wp_admin', 'activemember360' ),
			'desc'       => _x( 'This option activates ActiveMember360\'s BadgeOS extension and provides additional achievement completion steps and event logging.<br />The BadgeOS plugin must have previously been installed and activated on your site in order to be able to activate the ActiveMember360 BadgeOS extension.', 'wp_admin', 'activemember360' ),
			'icon'       => 'el el-star'
		];
		IF ( is_plugin_active('badgeos/badgeos.php' ) ) :
			$arr_section_badgeos['fields'][] =
			[
				'id'       => 'mbr_ext_badgeos_enabled',
				'title'    => _x( 'Enable BadgeOS extension', 'wp_admin', 'activemember360' ),
				'type'     => 'switch'
			];
			$arr_section_badgeos['fields'][] =
			[
				'id'       => 'mbr_ext_badgeos_woocommerce_enabled',
				'title'    => _x( 'Enable BadgeOS extension for WooCommerce activities', 'wp_admin', 'activemember360' ),
				'desc'     => _x( 'Once enabled, additional completion triggers are available.', 'wp_admin', 'activemember360' ),
				'type'     => 'switch',
				'required' => [ 'mbr_ext_badgeos_enabled', 'equals', '1' ]
			];
			$arr_section_badgeos['fields'][] =
			[
				'id'       => 'mbr_ext_badgeos_delete_completion_steps_info',
				'type'     => 'info',
				'style'    => 'critical',
				'icon'     => 'el-icon-info-sign',
				'title'    => _x( 'Delete ActiveMember360\'s BadgeOS completion steps', 'wp_admin', 'activemember360' ),
				'desc'     => _x( 'Delete your current ActiveMember360\'s BadgeOS completion steps IRREVERSIBLE!', 'wp_admin', 'activemember360' ),
				'required' => [ 'mbr_ext_badgeos_enabled', 'equals', '0' ]
			];
			$arr_section_badgeos['fields'][] =
			[
				'id'       => 'mbr_ext_badgeos_delete_completion_steps',
				'title'    => _x( 'Delete ActiveMember360\'s BadgeOS completion steps', 'wp_admin', 'activemember360' ),
				'desc'     => _x( 'To delete the ActiveMember360\'s BadgeOS completion steps type in DELETE.', 'wp_admin', 'activemember360' ),
				'type'     => 'text',
				'validate_callback' => [ $this, 'badgeos_delete_completion_steps_validate_callback' ],
				'required' => [ 'mbr_ext_badgeos_enabled', 'equals', '0' ],
			];
		ELSE:
			$arr_section_badgeos['fields'][] =
			[
				'id'       => 'mbr_ext_badgeos_not_active_info',
				'type'     => 'info',
				'style'    => 'info',
				'icon'     => 'el-icon-info-sign',
				'desc'     => _x( 'BadgeOS is not installed/activated.', 'wp_admin', 'activemember360' ),
			];
		ENDIF;

		$arr_section_badgeos['fields'][] =
		[
			'id'         => 'mbr_ext_badgeos_notes',
			'title'      => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'type'       => 'textarea'
		];

		


		$arr_section_beaver_builder =
			[
				'id'         => 'section_beaver_builder',
				'title'      => _x( 'Beaver Builder', 'wp_admin', 'activemember360' ),
				'desc'       => _x( 'This option activates ActiveMember360\'s Beaver Builder extension and provides additional tag based and custom field conditions.<br />The Beaver Builder plugin must have previously been installed and activated on your site in order to be able to activate the ActiveMember360 Beaver Builder extension.', 'wp_admin', 'activemember360' ),
				'icon'       => 'el el-file-edit'
			];
		IF ( defined( 'FL_BUILDER_VERSION' ) ) :
			$arr_section_beaver_builder['fields'][] =
				[
					'id'       => 'mbr_ext_beaver_builder_enabled',
					'title'    => _x( 'Enable Beaver Builder extension', 'wp_admin', 'activemember360' ),
					'type'     => 'switch'
				];
		ELSE:
			$arr_section_beaver_builder['fields'][] =
				[
					'id'       => 'mbr_ext_beaver_builder_not_active_info',
					'type'     => 'info',
					'style'    => 'info',
					'icon'     => 'el-icon-info-sign',
					'desc'     => _x( 'Beaver Builder is not installed/activated.', 'wp_admin', 'activemember360' ),
				];
		ENDIF;

		$arr_section_beaver_builder['fields'][] =
			[
				'id'         => 'mbr_ext_beaver_builder_notes',
				'title'      => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
				'type'       => 'textarea'
			];

		


		$arr_section_buddypress =
		[
			'id'             => 'section_buddypress',
			'title'          => _x( 'BuddyPress', 'wp_admin', 'activemember360' ),
			'desc'           => _x( 'This option activates ActiveMember360\'s BuddyPress extension and gives you the possibility to auto-enroll contacts based by ActiveCampaign tags to groups.<br />The BuddyPress plugin must have previously been installed and activated on your site in order to be able to activate the ActiveMember360 BuddyPress extension.', 'wp_admin', 'activemember360' ),
			'icon'           => 'el el-group'
		];
		IF ( is_plugin_active('buddypress/bp-loader.php' ) || is_plugin_active('buddyboss-platform/bp-loader.php' ) ) :
			$arr_section_buddypress['fields'][] =
			[
				'id'           => 'mbr_ext_buddypress_enabled',
				'title'        => _x( 'Enable BuddyPress extension', 'wp_admin', 'activemember360' ),
				'type'         => 'switch'
			];
			$arr_section_buddypress['fields'][] =
			[
				'id'           => 'mbr_ext_buddypress_redirect',
				'title'        => _x( 'BuddyPress redirect page', 'wp_admin', 'activemember360' ),
				'type'         => 'select',
				'placeholder'  => _x( 'Default redirect', 'wp_admin', 'activemember360' ),
				'data'         => 'posts',
				'args'         => [ 'post_type' => 'any', 'posts_per_page' => -1 ],
				'required'     => [ 'mbr_ext_buddypress_enabled', 'equals', '1' ]
			];
		ELSE:
			$arr_section_buddypress['fields'][] =
				[
					'id'         => 'mbr_ext_buddypress_not_active_info',
					'type'       => 'info',
					'style'      => 'info',
					'icon'       => 'el-icon-info-sign',
					'desc'       => _x( 'BuddyPress is not installed/activated.', 'wp_admin', 'activemember360' ),
				];
		ENDIF;

		$arr_section_buddypress['fields'][] =
		[
			'id'            => 'mbr_ext_buddypress_notes',
			'title'         => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'type'          => 'textarea'
		];

		


		$arr_section_contactform_7 =
		[
			'id'         => 'section_contactform_7',
			'title'      => _x( 'Contact Form 7', 'wp_admin', 'activemember360' ),
			'desc'       => _x( 'This option activates ActiveMember360\'s Contact Form 7 extension and provides integration to ActiveCampaign upon form submission.<br />The Contact Form 7 plugin must have previously been installed and activated on your site in order to be able to activate the ActiveMember360 Contact Form 7 extension.', 'wp_admin', 'activemember360' ),
			'icon'       => 'el el-edit'
		];
		IF ( is_plugin_active('contact-form-7/wp-contact-form-7.php' ) ) :
			$arr_section_contactform_7['fields'][] =
			[
				'id'       => 'mbr_ext_contactform7_enabled',
				'title'    => _x( 'Enable Contact Form 7 extension', 'wp_admin', 'activemember360' ),
				'type'     => 'switch'
			];
		ELSE:
			$arr_section_contactform_7['fields'][] =
			[
				'id'       => 'mbr_ext_contactform_7_not_active_info',
				'type'     => 'info',
				'style'    => 'info',
				'icon'     => 'el-icon-info-sign',
				'desc'     => _x( 'Contact Form 7 is not installed/activated.', 'wp_admin', 'activemember360' ),
			];
		ENDIF;

		$arr_section_contactform_7['fields'][] =
		[
			'id'         => 'mbr_ext_contactform_7_notes',
			'title'      => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'type'       => 'textarea'
		];

		


		$arr_section_custom_post_types =
		[
			'id'         => 'section_custom_post_types',
			'title'      => _x( 'Custom Post Types', 'wp_admin', 'activemember360' ),
			'desc'       => _x( 'Enable/disable ActiveMember360 integration for the following custom post types:', 'wp_admin', 'activemember360' ),
			'icon'       => 'el el-edit'
		];

		$arr_section_custom_post_types['fields'][] =
		[
			'id'         => 'mbr_ext_custom_post_types_enabled',
			'title'      => _x( 'Enable Custom Post Types for ActiveMember360', 'wp_admin', 'activemember360' ),
			'type'       => 'select',
			'select2'    => TRUE,
			'multi'      => TRUE,
			'data'       => 'callback',
			'args'       => [ 'mbr_ext_get_integration_custom_post_types' ] // This is the callback function. Can't be a method nor a direct call to a filter due to limitations in Redux | Must be a key 0 array
		];

		$arr_section_custom_post_types['fields'][] =
		[
			'id'         => 'mbr_ext_custom_post_types_notes',
			'title'      => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'type'       => 'textarea'
		];

		


		$arr_section_elementor =
			[
				'id'         => 'section_elementor',
				'title'      => _x( 'Elementor', 'wp_admin', 'activemember360' ),
				'desc'       => _x( 'This option activates ActiveMember360\'s Elementor extension and provides additional tag based and custom field conditions.<br />The Elementor plugin must have previously been installed and activated on your site in order to be able to activate the ActiveMember360 Elementor extension.', 'wp_admin', 'activemember360' ),
				'icon'       => 'el el-file-edit'
			];
		IF ( defined( 'ELEMENTOR_VERSION' ) ) :
			$arr_section_elementor['fields'][] =
				[
					'id'       => 'mbr_ext_elementor_enabled',
					'title'    => _x( 'Enable Elementor extension', 'wp_admin', 'activemember360' ),
					'type'     => 'switch'
				];
		ELSE:
			$arr_section_elementor['fields'][] =
				[
					'id'       => 'mbr_ext_elementor_not_active_info',
					'type'     => 'info',
					'style'    => 'info',
					'icon'     => 'el-icon-info-sign',
					'desc'     => _x( 'Elementor is not installed/activated.', 'wp_admin', 'activemember360' ),
				];
		ENDIF;

		$arr_section_elementor['fields'][] =
			[
				'id'         => 'mbr_ext_elementor_notes',
				'title'      => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
				'type'       => 'textarea'
			];

		


		$arr_section_gamipress =
			[
				'id'         => 'section_gamipress',
				'title'      => _x( 'GamiPress', 'wp_admin', 'activemember360' ),
				'desc'       => _x( 'This option activates ActiveMember360\'s GamiPress extension and provides ActiveCampaign tag based and contact field conditions to award points and set requirements for achievements and ranks.<br />It also allows for the addition and removal of tags and the setting of a contact field when earning an achievement and rank, and the storing of point totals in a contact field.<br />The GamiPress plugin must have previously been installed and activated on your site in order to be able to activate the ActiveMember360 GamiPress extension.', 'wp_admin', 'activemember360' ),
				'icon'       => 'el el-star'
			];
		IF ( defined( 'GAMIPRESS_VER' ) ) :
			$arr_section_gamipress['fields'][] =
				[
					'id'       => 'mbr_ext_gamipress_enabled',
					'title'    => _x( 'Enable GamiPress extension', 'wp_admin', 'activemember360' ),
					'type'     => 'switch'
				];
		ELSE:
			$arr_section_gamipress['fields'][] =
				[
					'id'       => 'mbr_ext_gamipress_not_active_info',
					'type'     => 'info',
					'style'    => 'info',
					'icon'     => 'el-icon-info-sign',
					'desc'     => _x( 'GamiPress is not installed/activated.', 'wp_admin', 'activemember360' ),
				];
		ENDIF;

		$arr_section_gamipress['fields'][] =
			[
				'id'         => 'mbr_ext_gamipress_notes',
				'title'      => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
				'type'       => 'textarea'
			];

		


		$arr_section_gutenberg =
			[
				'id'         => 'section_gutenberg',
				'title'      => _x( 'Gutenberg', 'wp_admin', 'activemember360' ),
				'desc'       => _x( 'This option activates ActiveMember360\'s Gutenberg extension and provides additional tag based and custom field conditions.<br />The Gutenberg plugin must have previously been installed and activated on your site in order to be able to activate the ActiveMember360 Gutenberg extension.', 'wp_admin', 'activemember360' ),
				'icon'       => 'el el-file-edit'
			];
		$arr_section_gutenberg['fields'][] =
			[
				'id'       => 'mbr_ext_gutenberg_enabled',
				'title'    => _x( 'Enable Gutenberg extension', 'wp_admin', 'activemember360' ),
				'type'     => 'switch'
			];

		$arr_section_gutenberg['fields'][] =
			[
				'id'         => 'mbr_ext_Gutenberg_notes',
				'title'      => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
				'type'       => 'textarea'
			];

		





































		


		$arr_section_learndash =
			[
				'id'         => 'section_learndash',
				'title'      => _x( 'Learndash', 'wp_admin', 'activemember360' ),
				'desc'       => _x( 'This option activates ActiveMember360\'s Learndash extension and provides tag-based access control to your courses, lessons, etc.<br />The Learndash plugin must have previously been installed and activated on your site in order to be able to activate the ActiveMember360 Learndash extension.', 'wp_admin', 'activemember360' ),
				'icon'       => 'el el-book'
			];
		IF ( is_plugin_active('sfwd-lms/sfwd_lms.php' ) ) :
			$arr_section_learndash['fields'][] =
				[
					'id'       => 'mbr_ext_learndash_enabled',
					'title'    => _x( 'Enable Learndash extension', 'wp_admin', 'activemember360' ),
					'type'     => 'switch'
				];
		ELSE:
			$arr_section_learndash['fields'][] =
				[
					'id'       => 'mbr_ext_learndash_not_active_info',
					'type'     => 'info',
					'style'    => 'info',
					'icon'     => 'el-icon-info-sign',
					'desc'     => _x( 'Learndash is not installed/activated.', 'wp_admin', 'activemember360' ),
				];
		ENDIF;

		$arr_section_learndash['fields'][] =
		[
			'id'           => 'mbr_ext_learndash_notes',
			'title'        => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'type'         => 'textarea'
		];

		


		$arr_section_lifterlms =
		[
			'id'         => 'section_lifterlms',
			'title'      => _x( 'LifterLMS', 'wp_admin', 'activemember360' ),
			'desc'       => _x( 'This option activates ActiveMember360\'s integration for LifterLMS.', 'wp_admin', 'activemember360' ),
			'icon'       => 'el el-book'
		];
		IF ( is_plugin_active('lifterlms/lifterlms.php' ) ) :
			$arr_section_lifterlms['fields'][] =
			[
				'id'       => 'mbr_ext_lifterlms_enabled',
				'title'    => _x( 'Enable LifterLMS extension', 'wp_admin', 'activemember360' ),
				'type'     => 'switch'
			];
		ELSE:
			$arr_section_lifterlms['fields'][] =
			[
				'id'       => 'mbr_ext_lifterlms_not_active_info',
				'type'     => 'info',
				'style'    => 'info',
				'icon'     => 'el-icon-info-sign',
				'desc'     => _x( 'LifterLMS is not installed/activated.', 'wp_admin', 'activemember360' ),
			];
		ENDIF;

		$arr_section_lifterlms['fields'][] =
		[
			'id'         => 'mbr_ext_lifterlms_notes',
			'title'      => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'type'       => 'textarea'
		];

		


		$arr_section_remoteauth =
		[
			'id'           => 'section_remoteauth',
			'title'        => _x( 'Remote Auth', 'wp_admin', 'activemember360' ),
			'desc'         => _x( 'This option activates ActiveMember360\'s Remote Auth extension.', 'wp_admin', 'activemember360' ),
			'icon'         => 'el el-retweet',
			'fields'       => [
				[
					'id'       => 'mbr_ext_remoteauth_enabled',
					'title'    => _x( 'Enable Remote Auth extension', 'wp_admin', 'activemember360' ),
					'type'     => 'switch'
				]
			]
		];

		$arr_section_remoteauth['fields'][] =
		[
			'id'           => 'mbr_ext_remoteauth_notes',
			'title'        => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'type'         => 'textarea'
		];

		


		$arr_section_ticket_system =
		[
			'id'           => 'section_ticket_system',
			'title'        => _x( 'Ticket System', 'wp_admin', 'activemember360' ),
			'heading'      => _x( 'Ticket System', 'wp_admin', 'activemember360' ),
			'desc'         => _x( 'This option activates ActiveMember360\'s ticket system.', 'wp_admin', 'activemember360' ),
			'icon'         => 'el el-th-list',
			'fields'       => [
				[
					'id'       => 'mbr_ext_ticket_system_enabled',
					'title'    => _x( 'Enable Ticket System extension', 'wp_admin', 'activemember360' ),
					'type'     => 'switch'
				]
			]
		];

		$arr_section_ticket_system['fields'][] =
		[
			'id'           => 'mbr_ext_ticket_system_notes',
			'title'        => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'type'         => 'textarea'
		];

		


		$arr_section_themeswitcher =
		[
			'id'           => 'section_themeswitcher',
			'title'        => _x( 'Theme Switcher', 'wp_admin', 'activemember360' ),
			'desc'         => _x( 'Once enabled, you will be able to switch themes based on a user\'s membership level.', 'wp_admin', 'activemember360' ),
			'icon'         => 'el el-website',
			'fields'       => [
				[
					'id'       => 'mbr_ext_themeswitcher_enabled',
					'title'    => _x( 'Enable Theme Switcher extension', 'wp_admin', 'activemember360' ),
					'type'     => 'switch'
				]
			]
		];

		$arr_section_themeswitcher['fields'][] =
		[
			'id'           => 'mbr_ext_themeswitcher_notes',
			'title'        => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'type'         => 'textarea'
		];

		


		$arr_section_oxygenbuilder =
			[
				'id'         => 'section_oxygenbuilder',
				'title'      => _x( 'Oxygen Builder', 'wp_admin', 'activemember360' ),
				'desc'       => _x( 'This option activates ActiveMember360\'s Oxygen Builder extension and provides additional display conditions.<br />The Oxygen Builder plugin must have previously been installed and activated on your site in order to be able to activate the ActiveMember360 Oxygen Builder extension.', 'wp_admin', 'activemember360' ),
				'icon'       => 'el el-file-edit'
			];
		IF ( defined( 'CT_VERSION' ) ) :
			$arr_section_oxygenbuilder['fields'][] =
				[
					'id'       => 'mbr_ext_oxygenbuilder_enabled',
					'title'    => _x( 'Enable Oxygen Builder extension', 'wp_admin', 'activemember360' ),
					'type'     => 'switch'
				];
		ELSE:
			$arr_section_oxygenbuilder['fields'][] =
				[
					'id'       => 'mbr_ext_oxygenbuilder_not_active_info',
					'type'     => 'info',
					'style'    => 'info',
					'icon'     => 'el-icon-info-sign',
					'desc'     => _x( 'Oxygen Builder is not installed/activated.', 'wp_admin', 'activemember360' ),
				];
		ENDIF;


		


		$arr_section_payments =
		[
			'id'           => 'section_payments',
			'title'        => _x( 'Payments', 'wp_admin', 'activemember360' ),
			'desc'         => _x( 'This extensions gives you the possibility create one-time payments, one click upsells and subscriptions processed with Stripe.', 'wp_admin', 'activemember360' ),
			'icon'         => 'el el-usd',
			'fields'       => [
				[
					'id'       => 'mbr_ext_payments_enabled',
					'title'    => _x( 'Enable Payments extension', 'wp_admin', 'activemember360' ),
					'type'     => 'switch'
				]
			]
		];

		$arr_section_payments['fields'][] =
		[
			'id'           => 'mbr_ext_payments_notes',
			'title'        => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'type'         => 'textarea'
		];

		


		$arr_section_popup_maker =
		[
			'id'         => 'section_popup_maker',
			'title'      => _x( 'Popup Maker', 'wp_admin', 'activemember360' ),
			'desc'       => _x( 'This option activates ActiveMember360\'s Popup Maker extension and provides additional tag based conditions.<br />The Popup Maker plugin must have previously been installed and activated on your site in order to be able to activate the ActiveMember360 Popup Maker extension.', 'wp_admin', 'activemember360' ),
			'icon'       => 'el el-view-mode'
		];
		IF ( is_plugin_active('popup-maker/popup-maker.php' ) ) :
			$arr_section_popup_maker['fields'][] =
			[
				'id'       => 'mbr_ext_popup_maker_enabled',
				'title'    => _x( 'Enable Popup Maker extension', 'wp_admin', 'activemember360' ),
				'type'     => 'switch'
			];
		ELSE:
			$arr_section_popup_maker['fields'][] =
			[
				'id'       => 'mbr_ext_popup_maker_not_active_info',
				'type'     => 'info',
				'style'    => 'info',
				'icon'     => 'el-icon-info-sign',
				'desc'     => _x( 'Popup Maker is not installed/activated.', 'wp_admin', 'activemember360' ),
			];
		ENDIF;

		$arr_section_popup_maker['fields'][] =
		[
			'id'         => 'mbr_ext_popup_maker_notes',
			'title'      => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'type'       => 'textarea'
		];

		


		$arr_section_tracking =
		[
			'id'             => 'section_tracking',
			'title'          => _x( 'Site/Event Tracking', 'wp_admin', 'activemember360' ),
			'desc'           => _x( 'This option activates ActiveMember360\'s ActiveCampaign Tracking extension.', 'wp_admin', 'activemember360' ),
			'icon'           => 'el el-fork',
			'fields'         => [
				[
					'id'         => 'mbr_ext_ac_tracking_enabled',
					'title'      => _x( 'Enable the tracking extension', 'wp_admin', 'activemember360' ),
					'desc'       => _x( 'Set to "On" to enable tracking for this site and to get more configuration options.', 'wp_admin', 'activemember360' ),
					'validate_callback' => [ $this, 'tracking_enable_tracking' ],
					'type'       => 'switch'
				],
				[
					'id'         => 'mbr_ext_ac_site_tracking_enabled',
					'title'      => _x( 'Enable Site Tracking', 'wp_admin', 'activemember360' ),
					'desc'       => _x( 'Set to "On" to enable site tracking for this site.', 'wp_admin', 'activemember360' ),
					'type'       => 'switch',
					'validate_callback' => [ $this, 'tracking_enable_site_tracking' ],
					'required'   => [ 'mbr_ext_ac_tracking_enabled', 'equals', '1' ]
				],
				[
					'id'         => 'mbr_ext_ac_event_tracking_enabled',
					'title'      => _x( 'Enable Event Tracking', 'wp_admin', 'activemember360' ),
					'desc'       => _x( 'Set to "On" to enable event tracking for this site.', 'wp_admin', 'activemember360' ),
					'type'       => 'switch',
					'validate_callback' => [ $this, 'tracking_enable_event_tracking' ],
					'required'   => [ 'mbr_ext_ac_tracking_enabled', 'equals', '1' ]
				],
				[
					'id'         => 'mbr_ext_ac_event_tracking_standard_events',
					'type'       => 'raw',
					'content'    => '<h2>' . _x( 'Standard Events', 'wp_admin', 'activemember360' ) . '</h2>',
					'full_width' => TRUE,
					'required'   => [ 'mbr_ext_ac_event_tracking_enabled', 'equals', '1' ]
				],
				[
					'id'         => 'mbr_ext_ac_event_login',
					'title'      => _x( 'Login Event', 'wp_admin', 'activemember360' ),
					'desc'       => _x( 'Fill in any name that makes sense to you to track logins of your contacts. This event will provide the domain of the successful login.', 'wp_admin', 'activemember360' ),
					'type'       => 'text',
					'validate_callback' => [ $this, 'tracking_event_cleanup' ],
					'required'   => [ 'mbr_ext_ac_event_tracking_enabled', 'equals', '1' ]
				],
				[
					'id'         => 'mbr_ext_ac_event_logout',
					'title'      => _x( 'Logout Event', 'wp_admin', 'activemember360' ),
					'desc'       => _x( 'Fill in any name that makes sense to you to track logouts of your contacts. This event will provide the domain of the successful logout.', 'wp_admin', 'activemember360' ),
					'type'       => 'text',
					'validate_callback' => [ $this, 'tracking_event_cleanup' ],
					'required'   => [ 'mbr_ext_ac_event_tracking_enabled', 'equals', '1' ]
				],
				[
					'id'         => 'mbr_ext_ac_event_user_registration',
					'title'      => _x( 'User registration Event', 'wp_admin', 'activemember360' ),
					'desc'       => _x( 'Fill in any name that makes sense to you to track new user registrations. This event will provide the domain of the successful registration.', 'wp_admin', 'activemember360' ),
					'type'       => 'text',
					'validate_callback' => [ $this, 'tracking_event_cleanup' ],
					'required'   => [ 'mbr_ext_ac_event_tracking_enabled', 'equals', '1' ]
				],
				[
					'id'         => 'mbr_ext_ac_event_tracking_woocommerce_events',
					'type'       => 'raw',
					'content'    => '<h2>' . _x( 'WooCommerce Events', 'wp_admin', 'activemember360' ) . '</h2>',
					'full_width' => TRUE,
					'required'   => [ 'mbr_ext_ac_event_tracking_enabled', 'equals', '1' ]
				]
			]
		];

		IF ( class_exists('WooCommerce') ) :
			$arr_section_tracking['fields'][] =
			[
				'id'         => 'mbr_ext_ac_event_woocommerce_payment_completed',
				'title'      => _x( 'WooCommerce Payment Completed Event', 'wp_admin', 'activemember360' ),
				'desc'       => _x( 'Fill in any name that makes sense to you to track when a WooCommerce Payment is completed. This event will provide the Order Id.', 'wp_admin', 'activemember360' ),
				'type'       => 'text',
				'validate_callback' => [ $this, 'tracking_event_cleanup' ],
				'required'   => [ [ 'mbr_ext_ac_event_tracking_enabled', 'equals', '1' ], [ 'mbr_ext_woocommerce_enabled', 'equals', '1'] ]
			];
			$arr_section_tracking['fields'][] =
			[
				'id'         => 'mbr_ext_ac_event_woocommerce_purchased_product',
				'title'      => _x( 'WooCommerce Purchased Product Event', 'wp_admin', 'activemember360' ),
				'desc'       => _x( 'Fill in any name that makes sense to you to track when a WooCommerce Product is purchased.<br />This event will be logged in ActiveCampaign twice, one event provides the Product Id and one the Product Name.', 'wp_admin', 'activemember360' ),
				'type'       => 'text',
				'validate_callback' => [ $this, 'tracking_event_cleanup' ],
				'required'   => [ [ 'mbr_ext_ac_event_tracking_enabled', 'equals', '1' ], [ 'mbr_ext_woocommerce_enabled', 'equals', '1'] ]
			];
			FOREACH ( wc_get_order_statuses() AS $order_status => $order_status_label ) :
				$arr_section_tracking['fields'][] =
				[
					'id'         => 'mbr_ext_ac_event_woocommerce_' . str_replace( '-', '_', $order_status ),
					'title'      => sprintf( _x( 'WooCommerce %s Event', 'wp_admin', 'activemember360' ), $order_status_label ),
					'desc'       => sprintf( _x( 'Fill in any name that makes sense to you to track the WooCommerce Order Status "%s". This event will provide the Order Id.', 'wp_admin', 'activemember360' ), $order_status_label ),
					'type'       => 'text',
					'validate_callback' => [ $this, 'tracking_event_cleanup' ],
					'required'   => [ [ 'mbr_ext_ac_event_tracking_enabled', 'equals', '1' ], [ 'mbr_ext_woocommerce_enabled', 'equals', '1'] ]
				];
			ENDFOREACH;
			$arr_section_tracking['fields'][] =
			[
				'id'           => 'mbr_ext_ac_event_woocommerce_not_active_info',
				'type'         => 'info',
				'style'        => 'info',
				'icon'         => 'el-icon-info-sign',
				'desc'         => _x( 'Enable the WooCommerce Extension for more event tracking options.', 'wp_admin', 'activemember360' ),
				'required'     => [ [ 'mbr_ext_ac_event_tracking_enabled', 'equals', '1' ], [ 'mbr_ext_woocommerce_enabled', 'not', '1'] ]
			];
		ELSE:
			$arr_section_tracking['fields'][] =
			[
				'id'           => 'mbr_ext_ac_event_woocommerce_not_active_info',
				'type'         => 'info',
				'style'        => 'info',
				'icon'         => 'el-icon-info-sign',
				'desc'         => _x( 'Install/activate WooCommerce for more event tracking options.', 'wp_admin', 'activemember360' ),
			];
		ENDIF;

		$arr_section_tracking['fields'][] =
		[
			'id'         => 'mbr_ext_ac_tracking_notes',
			'title'      => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'type'       => 'textarea'
		];

		


		$arr_section_woocommerce =
		[
			'id'         => 'section_woocommerce',
			'title'      => _x( 'WooCommerce', 'wp_admin', 'activemember360' ),
			'desc'       => _x( 'This option activates ActiveMember360\'s WooCommerce extension, providing deep integration between WooCommerce, ActiveCampaign and ActiveMember360.<br />In order to activate this extension, the WooCommerce plugin must have been previously installed and activated on your site.', 'wp_admin', 'activemember360' ),
			'icon'       => 'el el-shopping-cart'
		];

		IF ( is_plugin_active('woocommerce/woocommerce.php' ) ) :
			$arr_section_woocommerce['fields'][] =
			[
				'id'       => 'mbr_ext_woocommerce_enabled',
				'title'    => _x( 'Enable WooCommerce extension', 'wp_admin', 'activemember360' ),
				'type'     => 'switch'
			];
		ELSE:
			$arr_section_woocommerce['fields'][] =
			[
				'id'       => 'mbr_ext_woocommerce_not_active_info',
				'type'     => 'info',
				'style'    => 'info',
				'icon'     => 'el-icon-info-sign',
				'desc'     => _x( 'WooCommerce is not installed/activated.', 'wp_admin', 'activemember360' ),
			];
		ENDIF;

		$arr_section_woocommerce['fields'][] =
		[
			'id'         => 'mbr_ext_woocommerce_notes',
			'title'      => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'type'       => 'textarea'
		];

		$arr_section_mailer =
		[
			'id'           => 'section_mailer',
			'title'        => _x( 'Mailer', 'wp_admin', 'activemember360' ),
			'icon'         => 'el el-envelope',
			'fields'       => [
				[
					'id'       => 'mbr_ext_mailer_enabled',
					'title'    => _x( 'Enable mailer', 'wp_admin', 'activemember360' ),
					'type'     => 'switch'
				]
			]
		];

		$arr_section_mailer['fields'][] =
		[
			'id'           => 'mbr_ext_mailer_notes',
			'title'        => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'type'         => 'textarea'
		];

		
		Redux::setSection( $this->opt_name, $arr_section_alternate_member_menu );
		Redux::setSection( $this->opt_name, $arr_section_badgeos );
		Redux::setSection( $this->opt_name, $arr_section_beaver_builder );
		Redux::setSection( $this->opt_name, $arr_section_buddypress );
		Redux::setSection( $this->opt_name, $arr_section_contactform_7 );
		Redux::setSection( $this->opt_name, $arr_section_custom_post_types );



		Redux::setSection( $this->opt_name, $arr_section_elementor );
		Redux::setSection( $this->opt_name, $arr_section_gamipress );
		Redux::setSection( $this->opt_name, $arr_section_gutenberg );
		Redux::setSection( $this->opt_name, $arr_section_learndash );
		Redux::setSection( $this->opt_name, $arr_section_lifterlms );
		Redux::setSection( $this->opt_name, $arr_section_mailer );
		Redux::setSection( $this->opt_name, $arr_section_remoteauth );
		Redux::setSection( $this->opt_name, $arr_section_ticket_system );
		Redux::setSection( $this->opt_name, $arr_section_themeswitcher );
		Redux::setSection( $this->opt_name, $arr_section_oxygenbuilder );
		Redux::setSection( $this->opt_name, $arr_section_payments );
		Redux::setSection( $this->opt_name, $arr_section_popup_maker );
		Redux::setSection( $this->opt_name, $arr_section_tracking );
		Redux::setSection( $this->opt_name, $arr_section_woocommerce );
	}

	






	



	public function badgeos_delete_completion_steps_validate_callback( $field, $value, $existing_value )
	{

		$this->mbr_ext_badgeos_delete_completion_steps   = FALSE;
		IF ( $value == 'DELETE' ) :
			$this->mbr_ext_badgeos_delete_completion_steps = TRUE;
		ENDIF;

		
		$return['value'] = '';

		return $return;
	}

	


	public function badgeos_specific_extensions_saved( $arr_settings = [], $arr_prev_settings = [] )
	{
		
		IF ( $arr_settings['mbr_ext_badgeos_enabled'] == 0 AND ! empty( $this->mbr_ext_badgeos_delete_completion_steps ) ) :
			do_action( 'mbr/extensions/badgeos/steps/delete', FALSE, FALSE );

			return;
		ENDIF;

		
		$badgeos_deactivate_ac_steps      = FALSE;
		$badgeos_deactivate_woo_steps     = FALSE;

		IF ( $arr_settings['mbr_ext_badgeos_enabled'] == 0 AND $arr_prev_settings['mbr_ext_badgeos_enabled'] == 1  ) :
			$badgeos_deactivate_ac_steps    = TRUE;
			$badgeos_deactivate_woo_steps   = TRUE;
		ELSEIF ( $arr_settings['mbr_ext_badgeos_woocommerce_enabled'] == 0 AND $arr_prev_settings['mbr_ext_badgeos_woocommerce_enabled'] == 1 ) :
			$badgeos_deactivate_woo_steps   = TRUE;
		ENDIF;

		IF ( $badgeos_deactivate_ac_steps OR $badgeos_deactivate_woo_steps ) :
			do_action( 'mbr/extensions/badgeos/steps/deactivate', $badgeos_deactivate_ac_steps, $badgeos_deactivate_woo_steps );

			return;
		ENDIF;

		do_action( 'mbr/track/create/event', __( 'Achievement awarded', 'activemember360' ), FALSE, FALSE );
		do_action( 'mbr/track/create/event', __( 'Achievement revoked', 'activemember360' ), FALSE, FALSE );

		
		$badgeos_activate_ac_steps       = FALSE;
		$badgeos_activate_woo_steps      = FALSE;

		IF ( $arr_settings['mbr_ext_badgeos_enabled'] == 1 AND ( isset( $arr_prev_settings['mbr_ext_badgeos_enabled'] ) AND  $arr_prev_settings['mbr_ext_badgeos_enabled'] == 0 )  ) : // DO NOT USE EMPTY !!!!
			$badgeos_activate_ac_steps     = TRUE;
			IF ( $arr_settings['mbr_ext_badgeos_woocommerce_enabled'] == 1 AND $arr_prev_settings['mbr_ext_badgeos_woocommerce_enabled'] == 0 ) :
				$badgeos_activate_woo_steps  = TRUE;
			ENDIF;
		ELSEIF ( $arr_settings['mbr_ext_badgeos_enabled'] == 1 AND $arr_settings['mbr_ext_badgeos_woocommerce_enabled'] == 1 AND $arr_prev_settings['mbr_ext_badgeos_woocommerce_enabled'] == 0 ) :
			$badgeos_activate_woo_steps    = TRUE;
		ENDIF;

		IF ( $badgeos_activate_ac_steps OR $badgeos_activate_woo_steps ) :
			do_action( 'mbr/extensions/badgeos/steps/activate', $badgeos_activate_ac_steps, $badgeos_activate_woo_steps );

			return;
		ENDIF;
	}

	


	public function badgeos_completion_steps_actions( $ac_steps = FALSE, $woo_steps = FALSE )
	{
		global $wpdb;

		$current_action = current_filter();
		$delete         = FALSE;

		IF ( $current_action == 'mbr/extensions/badgeos/steps/deactivate' ) :
			$post_type    = 'MBR_archive_step';
		ELSEIF ( $current_action == 'mbr/extensions/badgeos/steps/activate' ) :
			$post_type    = 'step';
		ELSEIF ( $current_action == 'mbr/extensions/badgeos/steps/delete' ):
			$delete       = TRUE;
		ELSE:
			return;
		ENDIF;

		IF ( $delete ) :

			$arrSTEPS = $wpdb->get_results( $wpdb->prepare( "SELECT post_id FROM {$wpdb->postmeta} WHERE (meta_key=%s AND meta_value=%s) OR (meta_key=%s AND meta_value=%s)", '_badgeos_trigger_type', 'mbr_badgeos_ac_activity_trigger', '_badgeos_trigger_type', 'mbr_badgeos_woo_activity_trigger' ) );
			IF ( ! empty( $arrSTEPS ) ) :
				FOREACH ( $arrSTEPS AS $step ) :
					wp_delete_post( $step->post_id );
				ENDFOREACH;
			ENDIF;

			return;
		ENDIF;

		IF ( $ac_steps AND $woo_steps ) :
			$arrSTEPS = $wpdb->get_results( $wpdb->prepare( "SELECT post_id FROM {$wpdb->postmeta} WHERE (meta_key=%s AND meta_value=%s) OR (meta_key=%s AND meta_value=%s)", '_badgeos_trigger_type', 'mbr_badgeos_ac_activity_trigger', '_badgeos_trigger_type', 'mbr_badgeos_woo_activity_trigger' ) );
		ELSEIF ( $ac_steps ) :
			$arrSTEPS = $wpdb->get_results( $wpdb->prepare( "SELECT post_id FROM {$wpdb->postmeta} WHERE meta_key=%s AND meta_value=%s", '_badgeos_trigger_type', 'mbr_badgeos_ac_activity_trigger' ) );
		ELSEIF ( $woo_steps ) :
			$arrSTEPS = $wpdb->get_results( $wpdb->prepare( "SELECT post_id FROM {$wpdb->postmeta} WHERE meta_key=%s AND meta_value=%s", '_badgeos_trigger_type', 'mbr_badgeos_woo_activity_trigger' ) );
		ELSE:
			return;
		ENDIF;

		IF ( ! empty( $arrSTEPS ) ) :
			FOREACH ( $arrSTEPS AS $step ) :
				$wpdb->update( $wpdb->posts, [ 'post_type' => $post_type ], [ 'ID' => $step->post_id ] );
			ENDFOREACH;
		ENDIF;
	}

	




	public function tracking_enable_tracking( $field, $value, $existing_value )
	{
		IF ( empty( $value ) OR $value == '0' ) :
			do_action( 'mbr/extensions/tracking/events/remove/all' );
		ENDIF;

		return $return['value'] = $value;
	}

	




	public function tracking_enable_site_tracking( $field, $value, $existing_value )
	{
		$return                     = [];
		IF ( empty( $value ) OR $value == '0' ) :

			return $return['value']   = $value;
		ENDIF;

		IF ( empty( $this->fetch_account_info ) ) :
			do_action( 'mbr/ac_account/fetch' );
			$this->fetch_account_info = TRUE;
		ENDIF;

		






		$site_tracking_enabled      = TRUE;
		$domain_whitelisted         = TRUE;

		$ac_tracking_info           = apply_filters( 'mbr/api', 'tracking/site/list', [] );
		IF ( empty( $ac_tracking_info->enabled ) ) :
			$tracking_status          = apply_filters( 'mbr/api', 'tracking/site/status', [ 'status' => 'enable' ] );
			IF ( empty( $tracking_status->result_code ) ) :
				$site_tracking_enabled  = FALSE;
				$field['msg']           = _x( 'Site tracking could not be enabled.', 'wp_admin',  'activemember360' );
			ENDIF;
		ENDIF;

		





		$domain                     = apply_filters( 'mbr/site/domain/get', NULL );
		IF ( $site_tracking_enabled AND ( empty( $ac_tracking_info->whitelist ) OR ! in_array( $domain, $ac_tracking_info->whitelist ) ) ) :
			$whitelist                = apply_filters( 'mbr/api', 'tracking/whitelist', [ 'domain' => $domain ] );
			IF ( empty( $whitelist->result_code ) ) :
				$domain_whitelisted     = FALSE;
				$field['msg']           = _x( 'Domain for site tracking could not be whitelisted.', 'wp_admin', 'activemember360' );
			ENDIF;
		ENDIF;

		IF ( ! $site_tracking_enabled OR ! $domain_whitelisted ) :
			$return['value']          = FALSE;
			$return['error']          = $field;
		ELSE:
			$return['value']          = $value;
		ENDIF;

		return $return;
	}

	





	public function tracking_enable_event_tracking( $field, $value, $existing_value )
	{
		$return                     = [];
		IF ( empty( $value ) OR $value == '0' ) :
			do_action( 'mbr/extensions/tracking/events/remove/all' );

			return $return['value']   = $value;
		ENDIF;

		IF ( empty( $this->fetch_account_info ) ) :
			do_action( 'mbr/ac_account/fetch' );
			$this->fetch_account_info = TRUE;
		ENDIF;

		$event_tracking_enabled     = TRUE;
		$ac_event_info              = apply_filters( 'mbr/api', 'tracking/event/list', [] );

		IF ( empty( $ac_event_info->enabled ) ) :
			$event_status = apply_filters( 'mbr/api', 'tracking/event/status', [ 'status' => 'enable' ] );
			IF ( empty( $event_status->result_code ) ) :
				$event_tracking_enabled = FALSE;
			ENDIF;
		ENDIF;

		IF ( ! $event_tracking_enabled ) :
			$return['value']          = FALSE;
			$field['msg']             = _x( 'Event tracking could not be enabled.', 'wp_admin', 'activemember360' );
			$return['error']          = $field;
		ELSE:
			$return['value']          = $value;
		ENDIF;

		return $return;
	}

	



	public function tracking_event_cleanup( $field, $value, $existing_value )
	{
		
		IF ( empty( $this->tracking_event_tracking_disabled ) ) :
			IF ( ! empty( $existing_value ) AND $existing_value <> $value ) :
				apply_filters( 'mbr/api', 'tracking/event/remove', [ 'event' => $existing_value ] );
			ENDIF;
			IF ( ! empty( $value ) ) :
				do_action( 'mbr/track/create/event', $value, FALSE, FALSE );
			ENDIF;
		ENDIF;

		$return['value'] = $value;

		return $return;
	}

	



	public function tracking_remove_events()
	{
		$this->tracking_event_tracking_disabled = TRUE;

		static $has_run = FALSE;
		IF ( empty( $has_run ) ) :
			$has_run = TRUE;
		ELSE:
			return;
		ENDIF;

		$arr_extension_settings = get_option( $this->opt_name, FALSE );

		IF ( ! empty( $arr_extension_settings['mbr_ext_ac_event_login'] ) ) :
			apply_filters( 'mbr/api', 'tracking/event/remove',     [ 'event' => $arr_extension_settings['mbr_ext_ac_event_login'] ] );
		ENDIF;
		IF ( ! empty( $arr_extension_settings['mbr_ext_ac_event_logout'] ) ) :
			apply_filters( 'mbr/api', 'tracking/event/remove',     ['event' => $arr_extension_settings['mbr_ext_ac_event_logout'] ] );
		ENDIF;
		IF ( ! empty( $arr_extension_settings['mbr_ext_ac_event_user_registration'] ) ) :
			apply_filters( 'mbr/api', 'tracking/event/remove',     ['event' => $arr_extension_settings['mbr_ext_ac_event_user_registration'] ] );
		ENDIF;

		IF ( class_exists('WooCommerce') ) :
			IF ( ! empty( $arr_extension_settings['mbr_ext_ac_event_woocommerce_payment_completed'] ) ) :
				apply_filters( 'mbr/api', 'tracking/event/remove',   ['event' => $arr_extension_settings['mbr_ext_ac_event_woocommerce_payment_completed'] ] );
			ENDIF;
			IF ( ! empty( $arr_extension_settings['mbr_ext_ac_event_woocommerce_purchased_product'] ) ) :
				apply_filters( 'mbr/api', 'tracking/event/remove',   ['event' => $arr_extension_settings['mbr_ext_ac_event_woocommerce_purchased_product'] ] );
			ENDIF;
			IF ( function_exists( 'wc_get_order_statuses' ) ) :
				FOREACH ( wc_get_order_statuses() AS $order_status => $order_status_label ) :
					IF ( ! empty( $arr_extension_settings['mbr_ext_ac_event_woocommerce_' . str_replace( '-', '_', $order_status ) ] ) ) :
						apply_filters( 'mbr/api', 'tracking/event/remove', [ 'event' => $arr_extension_settings['mbr_ext_ac_event_woocommerce_' . str_replace( '-', '_', $order_status ) ] ] );
					ENDIF;
				ENDFOREACH;
			ENDIF;
		ENDIF;
	}

	public function mailer_specific_extensions_saved( $arr_settings = [], $arr_prev_settings = [] )
	{
		IF ( $arr_settings['mbr_ext_mailer_enabled'] == 0 ) :
			wp_clear_scheduled_hook( 'mbr/mailer/templates/refresh' );
		ENDIF;
	}

	public function backup_settings( $arr_settings = [] )
	{
		global $wpdb;
		set_transient($this->opt_name . '_backup_' . time(), $arr_settings, 45 * 86400 );
		
		$obj_results = $wpdb->get_results( "SELECT * FROM " . $wpdb->prefix . "options WHERE option_name LIKE '_transient_" . $this->opt_name . "_backup_%'", OBJECT );
		FOREACH ($obj_results AS $key => $obj) :
			get_transient( substr( $obj->option_name, 11 ) );
		ENDFOREACH;
	}

}

function mbr_ext_get_integration_custom_post_types() {
	$arr_integration_cpts       = apply_filters( 'mbr/cpt/integration/get', NULL );
	$arr_valid_integration_cpts = [];

	FOREACH ( $arr_integration_cpts AS $key => $cpt_slug ) :
		IF (in_array( $cpt_slug, [ MBRCUSTPRODUCTS, MBRCUSTSUBSCRIPTIONS ] ) ) :
			continue;
		ENDIF;
		$arr_valid_integration_cpts[ $cpt_slug ] = $cpt_slug;
	ENDFOREACH;

	return $arr_valid_integration_cpts;
}

$GLOBALS['mbr_inst_extensions'] = mbr_extensions::getInstance();
