<?php
IF ( ! defined('ABSPATH' ) ) :
  wp_die( 'Sorry, but you should not access this page directly.' );
ENDIF;

class mbr_gravity_forms {

  use TRAIT__getInstance;

  













  public function _load()
  {
	IF ( ! class_exists( 'GFForms' ) ) :
	  return;
	ENDIF;

	function MBRGF()
	{
	  IF ( empty($GLOBALS['mbr_gravityforms'] ) ) :
		$GLOBALS['mbr_gravityforms'] = mbr_gravity_forms::getInstance();
	  ENDIF;

	  return $GLOBALS['mbr_gravityforms'];
	}

	
	add_action( 'init',                          [ $this, '_init' ], 11 );
	add_action( 'gform_field_advanced_settings', [ $this, 'add_advanced_setting_mbr_sync' ], 10, 2 );
	add_filter( 'gform_tooltips',                [ $this, 'add_tooltips' ] );
	add_action( 'gform_editor_js',               [ $this, 'register_gf_custom_fields' ] );
	add_action( 'gform_after_submission',        [ $this, 'after_form_submission' ], 10, 2 );
	add_filter( 'gform_field_value',             [ $this, 'field_router_generic' ], 999, 3 );
  }

  












  public function _init()
  {
	$this->arr_regular_fields                = apply_filters( 'mbr/site_regular_fields/get', NULL );
	$this->arr_custom_fields                 = apply_filters( 'mbr/site_custom_fields/get',  NULL );

	$this->arr_fields                        = [];
	$this->arr_field_types                   = [];

	$this->remote_contact_id                 = FALSE;
	IF ( ! is_admin() ) :
	  IF ( $this->remote_contact_id          = apply_filters( 'mbr/contact_id', NULL ) ) :
		$this->contact_session               = apply_filters( 'mbr/session/get', NULL );
	  ENDIF;
	ENDIF;

	IF ( ! empty( $this->arr_regular_fields ) ) :
	  FOREACH ( $this->arr_regular_fields AS $perstag => $title ) :
		IF ( $perstag = lower_trim( $perstag ) ) :
		  $this->arr_fields[ $perstag ]      = str_replace( '_', ' ', ucwords( strtolower( $title ) ) );
		  $this->arr_field_types[ $perstag ] = 'text';
		ENDIF;
	  ENDFOREACH;
	ENDIF;

	IF ( ! empty( $this->arr_custom_fields ) ) :
	  FOREACH ( $this->arr_custom_fields AS $key => $arr_cust_field ) :
		IF ( $perstag = lower_trim( $arr_cust_field['perstag'] ) ) :
		  $this->arr_fields[ $perstag ]      = str_replace( '_', ' ', ucwords( strtolower( $arr_cust_field['title'] ) ) );
		  $this->arr_field_types[ $perstag ] = $arr_cust_field['type'];
		ENDIF;
	  ENDFOREACH;
	ENDIF;
  }

  















  public function field_router_generic( $value, $field, $parameter_name )
  {
	
	$default_value  = $field->defaultValue;

	






	$shortcode_value = do_shortcode( $parameter_name );
	IF ( ! empty( $shortcode_value ) ) :

	  IF ( stripos( $parameter_name, '[mbr_') !== FALSE AND stripos( $shortcode_value, '||' ) !== FALSE ) :
		$tmp_field_value = $shortcode_value;
		$return_value    = array_filter( explode( '||', $tmp_field_value ) );
	  ELSE:
		$return_value    = $shortcode_value;
	  ENDIF;

	ELSEIF ( ! empty( $value ) ) :
	  $return_value = $value;
	ELSEIF ( ! empty( $default_value ) ):
	  $return_value = $default_value;
	ENDIF;

	return $return_value ? $return_value : $value;
  }

  














  public function add_advanced_setting_mbr_sync( $position, $form_id )
  {
	IF ( $position !== 50 ) :
	  return;
	ENDIF;
	?>

	<li class="mbr_sync_setting field_setting">
	  <ul>
		<li>
		  <label for="field_mbr_sync" class="section_label">
			<?php _ex( 'ActiveMember360: Sync field to ActiveCampaign', 'gravity-forms', 'activemember360' ); ?>
			<?php gform_tooltip( 'form_field_mbr_sync' ); ?>
		  </label>
		  <select id="field_mbr_sync" type="text" onchange="SetFieldProperty('mbr_sync', this.value)">
			<?php
			FOREACH ( $this->arr_fields AS $perstag => $title ) :
			  echo '<option value="' . $perstag . '">' . $title . '</option>';
			ENDFOREACH;
			?>
		  </select>
		</li>
	  </ul>
	</li>

  <?php
  }

  














  public function add_tooltips( $tooltips )
  {
	


	$tooltips['form_field_mbr_sync'] = sprintf( '<h6>%s</h6>%s', _x( 'ActiveMember360: Sync field to ActiveCampaign', 'gravity-forms', 'activemember360' ), _x( 'Choose your ActiveCampaign field in which it should be synced', 'gravity-forms', 'activemember360' ) );
	return $tooltips;
  }

  













  public function register_gf_custom_fields() {
	?>
	<script type="text/javascript">
		
		jQuery.map(fieldSettings, function (el, i) {
			fieldSettings[i] += ', .mbr_sync_setting';
		});
		
		jQuery(document).on('gform_load_field_settings', function(ev, field) {
			jQuery('#field_mbr_sync').val(field.mbr_sync || '');
		});
	</script>
	<?php
  }

  















  












  public function after_form_submission( $entry, $form )
  {
	IF ( $this->remote_contact_id ) :

	  $arr_sync        = [];

	  FOREACH ( $form['fields'] AS $key => $field ) :
		IF ( ! empty( $field['mbr_sync'] ) ) :
		  $mbr_field   = $field['mbr_sync'];
		  $field_value = $entry[ $field['id'] ]; // @TODO if is checkbox or multi select the vales are in 4.1, 4.2 etc

		  IF ( array_key_exists( $mbr_field, $this->arr_regular_fields ) ) :
			$arr_sync[ $mbr_field ] = $field_value;
		  ELSEIF ( array_key_exists( $mbr_field, $this->arr_custom_fields ) ) :
			IF ( is_array( $field_value ) ) :
			  $field_value = implode( '||', $field_value );
			ENDIF;
			$arr_sync['field[%' . $mbr_field . '%]'] = $field_value;
		  ENDIF;
		ENDIF;
	  ENDFOREACH;

	  IF ( ! empty( $arr_sync ) ) :
		$arr_sync['id'] = $this->remote_contact_id;
		apply_filters( 'mbr/save_contact_fields', $arr_sync );
	  ENDIF;

	ENDIF;
  }

}
$GLOBALS['mbr_gravityforms'] = mbr_gravity_forms::getInstance();

// add a custom menu item to the Form Settings page menu
add_filter( 'gform_form_settings_menu', 'my_custom_form_settings_menu_item' );
function my_custom_form_settings_menu_item( $menu_items ) {

  $menu_items[] = array(
	'name' => 'my_custom_form_settings_page',
	'label' => __( 'My Custom Form Settings Page' )
  );

  return $menu_items;
}