<?php
class mbr_learndash
{
	use TRAIT__getInstance;

	public function _load()
	{
		IF ( ! is_plugin_active('sfwd-lms/sfwd_lms.php' ) ) :
			return null;
		ENDIF;

		function MBRLD()
		{
			IF ( empty( $GLOBALS['mbrlearndash'] ) ) :
				$GLOBALS['mbrlearndash'] = mbr_learndash::getInstance();
			ENDIF;

			return $GLOBALS['mbrlearndash'];
		}

		global $pagenow;

		$this->do_not_enroll_key = 'sfwd-courses_course_completed_mbr_noenroll';
		$this->visible_key       = 'sfwd-courses_course_completed_mbr_visible';
		$this->price_type_key    = 'sfwd-courses_course_price_type';
		$this->access_list_key   = 'sfwd-courses_course_access_list';
		$this->short_desc_key    = 'sfwd-courses_course_short_description';

		add_post_type_support( 'sfwd-courses', 'excerpt' );

		IF ( is_admin() ) :
			
			IF ( in_array( $pagenow, [ 'post.php', 'post-new.php' ] ) ) :
				add_action( "load-{$pagenow}",                    [ $this, 'init_metaboxes' ] );
			ENDIF;
			add_action( 'mbr/quick_edit/protection/box',        [ $this, 'init_quick_edit' ], 0 );
		ENDIF;

		
		add_filter( 'mbr/protectable_post_types/built_in',    [ $this, 'add_protectable_post_types' ],      10, 1 );

		
		add_filter( 'learndash_post_args',                    [ $this, 'learndash_post_args' ] );

		add_action( 'learndash_quiz_completed',               [ $this, 'learndash_quiz_completed' ],   2 );
		add_action( 'learndash_topic_completed',              [ $this, 'learndash_topic_completed' ],  3 );
		add_action( 'learndash_lesson_completed',             [ $this, 'learndash_lesson_completed' ], 4 );
		add_action( 'learndash_course_completed',             [ $this, 'learndash_course_completed' ], 5 );
		add_filter( 'learndash_course_completion_url',        [ $this, 'learndash_course_completion_url' ], 5, 2 );

		IF ( ! is_admin() ) :
			add_filter( 'mbr/visibility/check/exception',       [ $this, 'add_learndash_post_types_visibility'], 10, 2 );
			add_filter( 'the_posts',                            [ $this, 'filter_the_posts' ], 8, 2 );

			add_filter( 'learndash_template',                   [ $this, 'learndash_template' ], 99, 5 );

			add_filter( 'mbr_excerpt',                          [ $this, 'learndash_mbr_excerpt' ], 0, 2 );

			add_filter( 'template_redirect',                    [ $this, 'learndash_template_redirect' ],  0 );
			add_filter( 'learndash_payment_closed_button',      [ $this, 'learndash_payment_button' ], 10, 2 );
			add_filter( 'learndash_payment_button',             [ $this, 'learndash_payment_button' ], 10, 2 );
		ENDIF;
	}

	public function learndash_template( $filepath, $name, $args, $echo, $return_file_path )
	{
		IF ( $name == 'profile' ) :
			$this->profile_template_filepath = $filepath;
			$filepath = __DIR__ . '/class.learndashprofile.php';
		ENDIF;

		return $filepath;
	}

	public function get_course_attributes( $post_id )
	{
		$array_course_attributes_1 = get_post_meta( $post_id, '_sfwd-courses', TRUE );
		$array_course_attributes_2 = get_post_meta( $post_id, '_learndash_course_grid_short_description', TRUE );
		IF ( ! empty( $array_course_attributes_2 ) ) :
			return array_merge( $array_course_attributes_1, [ $this->short_desc_key => $array_course_attributes_2 ] );
		ELSE:
			return $array_course_attributes_1;
		ENDIF;
	}

	public function learndash_mbr_excerpt( $excerpt = NULL, $_post )
	{
		IF ( $_post->post_type <> 'sfwd-courses' ) :
			return $excerpt;
		ENDIF;

		
		$course_attributes = $this->get_course_attributes( $_post->ID );

		
		IF (  !empty( $course_attributes[ $this->visible_key ] ) AND $course_attributes[ $this->visible_key ] == 'on' ) :
			IF ( ! empty( $_post->post_excerpt ) ) :
				return $_post->post_excerpt;
			ELSEIF ( ! empty( $course_attributes[ $this->short_desc_key ] ) ) :
				return $course_attributes[ $this->short_desc_key ];
			ELSE :
				return __('There is currently no short description for this course.', 'activemember360');
			ENDIF;
		ENDIF;

		IF ( ! empty( $_post->post_content ) ) :
			return apply_filters( 'mbr/excerpt/trim', $_post->post_content );
		ENDIF;

		return $excerpt;
	}

	public function add_protectable_post_types( $arr_protectable_post_types = [] )
	{
		array_push( $arr_protectable_post_types, 'sfwd-courses', 'sfwd-lessons', 'sfwd-topic', 'sfwd-quiz' );
		return $arr_protectable_post_types;
	}

	public function init_metaboxes( $_post_type = FALSE, $_post = FALSE )
	{
		global $pagenow, $typenow;

		IF ( in_array( $typenow,  [ 'sfwd-courses', 'sfwd-lessons', 'sfwd-topic', 'sfwd-quiz' ] ) ) :

			$current_filter = current_filter();

			IF ( "load-{$pagenow}" == $current_filter ) :

				add_action( 'add_meta_boxes', [ $this, __FUNCTION__ ], 10, 2 );

			ELSEIF ( 'add_meta_boxes' == $current_filter ) :

				IF ( 'sfwd-courses' == $_post_type ) :
					do_action( 'mbr/add_meta_box',          $_post, [ $this, 'render_meta_boxes_protection_sfwd_courses' ], NULL );
					do_action( 'mbr/meta_boxes_excerpt',    $_post );
				ELSEIF ( 'sfwd-lessons' == $_post_type ) :
					do_action( 'mbr/add_meta_box',          $_post, [ $this, 'render_meta_boxes_protection_sfwd_lessons' ], NULL );
				ELSEIF ( 'sfwd-topic'   == $_post_type ) :
					do_action( 'mbr/add_meta_box',          $_post, [ $this, 'render_meta_boxes_protection_sfwd_topic' ], NULL );
				ELSEIF ( 'sfwd-quiz'    == $_post_type ) :
					do_action( 'mbr/add_meta_box',          $_post, [ $this, 'render_meta_boxes_protection_sfwd_quiz' ], NULL );
				ENDIF;

			ENDIF;
		ENDIF;
	}

	public function init_quick_edit( $_post )
	{
		IF ( 'sfwd-courses' == $_post->post_type ) :
			$this->render_meta_boxes_protection_sfwd_courses( $_post, 'quick-protection' );
		ELSEIF ( 'sfwd-lessons' == $_post->post_type ) :
			$this->render_meta_boxes_protection_sfwd_lessons( $_post, 'quick-protection' );
		ELSEIF ( 'sfwd-topic' == $_post->post_type ) :
			$this->render_meta_boxes_protection_sfwd_topic( $_post, 'quick-protection' );
		ELSEIF ( 'sfwd-quiz' == $_post->post_type ) :
			$this->render_meta_boxes_protection_sfwd_quiz( $_post, 'quick-protection' );
		ENDIF;
	}

	public function render_meta_boxes_protection_sfwd_courses( $_post, $box )
	{
		$settings = apply_filters( 'mbr/get_meta_box_protection_settings', $_post );

		do_action( 'mbr/render_metabox_css_helper', $box );

		IF ( empty( $settings['membership_levels'] ) ) :
			echo "<p style='color:" . apply_filters( 'mbr/plugin_color', NULL ) . ";font-size:10px;'>" . sprintf( _x( 'You have not yet defined which memberships you want to use on this site. Please do so in <a href="' . admin_url( "admin.php?page=ActiveMember360&mbrtab=memberships" ) . '">%s.</a>',  'wp_admin', 'activemember360' ), "'Memberships'" ) . "</p>";
			return;
		ENDIF;

		do_action( 'mbr/render_meta_boxes_protection_generic',             $_post, $settings );
		do_action( 'mbr/render_meta_boxes_protection_memberships',         $_post, $settings );
		do_action( 'mbr/render_meta_boxes_protection_filters',             $_post, $settings );
		do_action( 'mbr/render_meta_boxes_protection_addtl_tags',          $_post, $settings );
		do_action( 'mbr/render_meta_boxes_protection_comment_automations', $_post, $settings );
		do_action( 'mbr/render_meta_boxes_protection_redirect',            $_post, $settings );
	}

	public function render_meta_boxes_protection_sfwd_lessons( $_post, $box )
	{
		$settings = apply_filters( 'mbr/get_meta_box_protection_settings', $_post );

		do_action( 'mbr/render_metabox_css_helper', $box );

		IF ( empty( $settings['membership_levels'] ) ) :
			echo "<p style='color:" . apply_filters( 'mbr/plugin_color', NULL ) . ";font-size:10px;'>" . sprintf( _x( 'You have not yet defined which memberships you want to use on this site. Please do so in <a href="' . admin_url( "admin.php?page=ActiveMember360&mbrtab=memberships" ) . '">%s.</a>', 'wp_admin', 'activemember360' ), "'Memberships'" ) . "</p>";
			return;
		ENDIF;

		do_action( 'mbr/render_meta_boxes_protection_addtl_tags', $_post, $settings );
	}

	public function render_meta_boxes_protection_sfwd_topic( $_post, $box )
	{
		$settings = apply_filters( 'mbr/get_meta_box_protection_settings', $_post );

		do_action( 'mbr/render_metabox_css_helper', $box );

		IF ( empty( $settings['membership_levels'] ) ) :
			echo "<p style='color:" . apply_filters( 'mbr/plugin_color', NULL ) . ";font-size:10px;'>" . sprintf( _x( 'You have not yet defined which memberships you want to use on this site. Please do so in <a href="' . admin_url( "admin.php?page=ActiveMember360&mbrtab=memberships" ) . '">%s.</a>', 'wp_admin', 'activemember360' ), "'Memberships'" ) . "</p>";
			return;
		ENDIF;

		do_action( 'mbr/render_meta_boxes_protection_addtl_tags', $_post, $settings );
	}

	public function render_meta_boxes_protection_sfwd_quiz( $_post, $box )
	{
		$settings = apply_filters( 'mbr/get_meta_box_protection_settings', $_post );

		do_action( 'mbr/render_metabox_css_helper', $box );

		IF ( empty( $settings['membership_levels'] ) ) :
			echo "<p style='color:" . apply_filters( 'mbr/plugin_color', NULL ) . ";font-size:10px;'>" . sprintf( _x( 'You have not yet defined which memberships you want to use on this site. Please do so in <a href="' . admin_url( "admin.php?page=ActiveMember360&mbrtab=memberships" ) . '">%s.</a>', 'wp_admin', 'activemember360' ), "'Memberships'" ) . "</p>";
			return;
		ENDIF;

		do_action( 'mbr/render_meta_boxes_protection_addtl_tags', $_post, $settings );
	}

	public function add_learndash_post_types_visibility( $return, $_post )
	{
		IF ( in_array( $_post->post_type, [ 'sfwd-courses', 'sfwd-lessons', 'sfwd-topic', 'sfwd-quiz' ] ) ) :
			return TRUE;
		ENDIF;

		return $return;
	}

	public function filter_the_posts( $posts, $r = '' )
	{
		IF ( empty( $posts ) OR current_user_can( 'publish_pages' ) ) :
			return $posts;
		ENDIF;

		$filtered_posts = [];

		FOREACH ( $posts AS $k => $_post ) :
			IF ( in_array( $_post->post_type, [ 'sfwd-courses', 'sfwd-lessons', 'sfwd-topic', 'sfwd-quiz' ] ) ) :
				IF ( FALSE === $this->match_user_content( $_post ) ) :
					continue;
				ENDIF;
			ENDIF;
			$filtered_posts[] = $_post;
		ENDFOREACH;

		return $filtered_posts;
	}

	public function match_user_content( $_post )
	{
		global $current_user;

		static $is_logged_in = NULL;
		IF ( is_null( $is_logged_in ) ) :
			 $is_logged_in = is_user_logged_in();
		ENDIF;

		$user_course_access = learndash_user_get_enrolled_courses( $current_user->ID );

		IF ( $_post->post_type == 'sfwd-courses' ) :

			
			$course_attributes = $this->get_course_attributes( $_post->ID );

			IF ( ! empty( $course_attributes[ $this->price_type_key ] ) ) :
				$price_type = strtolower( $course_attributes[ $this->price_type_key ] );
			ELSE :
				$price_type = FALSE;
			ENDIF;

			IF ( ! empty( $course_attributes[ $this->do_not_enroll_key ] ) AND $course_attributes[ $this->do_not_enroll_key ] == 'on' ) :
				$auto_enroll = FALSE;
			ELSE :
				$auto_enroll = TRUE;
			ENDIF;

			IF ( ! empty( $course_attributes[ $this->visible_key ] ) AND $course_attributes[ $this->visible_key ] == 'on' ) :
				$visible_course = TRUE;
			ELSE :
				$visible_course = FALSE;
			ENDIF;

			IF ( ! $is_logged_in AND ( $price_type == 'paynow' OR $price_type == 'subscribe' ) ) :
				return FALSE;
			ENDIF;

			IF ( $is_logged_in AND in_array( $_post->ID, $user_course_access ) ) :
				$has_access = TRUE;
			ELSE :
				$has_access = FALSE;
			ENDIF;

			
			IF ( $price_type == 'free' ) :

				

				IF ( FALSE === $this->course_is_protected( $_post ) ) :
					
					IF ( $is_logged_in AND ! $has_access AND $auto_enroll ) :
						ld_update_course_access( $current_user->ID, $_post->ID, FALSE );
					ENDIF;
					return TRUE;
				ENDIF;

				
				IF ( apply_filters( 'mbr/match_user_content', $_post, MBR_IGNORE_EXCERPT ) ) :
					
					IF ( $is_logged_in AND ! $has_access AND $auto_enroll ) :
						ld_update_course_access( $current_user->ID, $_post->ID, FALSE );
					ENDIF;
					return TRUE;
				ELSE:
					IF ( $is_logged_in AND $has_access AND $auto_enroll ) :
						ld_update_course_access( $current_user->ID, $_post->ID, TRUE );
					ENDIF;
				ENDIF;

				IF ( $is_logged_in AND ! MBR()->PAGE_IS_PAYF ) :
					
					ld_update_course_access( $current_user->ID, $_post->ID, TRUE );
				ENDIF;

				
				IF ( $visible_course OR ! empty( $_post->post_excerpt ) ) :
					return TRUE;
				ENDIF;

				return FALSE;

			ENDIF;

			
			IF ( $price_type == 'closed' ) :

				IF ( FALSE === $this->course_is_protected( $_post ) ) :
					
					IF ( $is_logged_in AND ! $has_access AND $auto_enroll ) :
						ld_update_course_access( $current_user->ID, $_post->ID, FALSE );
					ENDIF;
					return TRUE;
				ENDIF;

				
				IF ( apply_filters( 'mbr/match_user_content', $_post, MBR_IGNORE_EXCERPT ) ) :
					
					IF ( $is_logged_in AND ! $has_access AND $auto_enroll ) :
						ld_update_course_access( $current_user->ID, $_post->ID, FALSE );
					ENDIF;
					return TRUE;
				ELSE:
					IF ( $is_logged_in AND $has_access AND $auto_enroll ) :
						ld_update_course_access( $current_user->ID, $_post->ID, TRUE );
					ENDIF;
				ENDIF;

				IF ( $is_logged_in AND ! MBR()->PAGE_IS_PAYF ) :
					
					ld_update_course_access( $current_user->ID, $_post->ID, TRUE );
				ENDIF;

				
				IF ( $visible_course OR ! empty( $_post->post_excerpt ) ) :
					return TRUE;
				ENDIF;

				return FALSE;

			ENDIF;

			
			IF ( $price_type == 'open' ) :

				
				IF ( FALSE === $this->course_is_protected( $_post ) ) :
					
					
					IF ( $is_logged_in AND ! $has_access ) :
						ld_update_course_access( $current_user->ID, $_post->ID, FALSE );
					ENDIF;

					return TRUE;
				ENDIF;

				
				IF ( apply_filters( 'mbr/match_user_content', $_post, MBR_IGNORE_EXCERPT ) ) :
					




					IF ( $is_logged_in AND ! $has_access ) :
						ld_update_course_access( $current_user->ID, $_post->ID, FALSE );
					ENDIF;

					return TRUE;
				ENDIF;

				



				





				
				IF ( $visible_course OR ! empty( $_post->post_excerpt ) ) :
					return TRUE;
				ENDIF;

				return FALSE;

			ENDIF;

			
			IF ( in_array( $price_type, [ 'paynow', 'subscribe' ] ) ) :

				IF ( $is_logged_in AND $has_access ) :
					return TRUE;
				ENDIF;

				return FALSE;

			ENDIF;

			return FALSE;

		ELSEIF ( $_post->post_type == 'sfwd-lessons' ) :
			IF ( apply_filters( 'mbr/match_user_content', $_post->ID, MBR_IGNORE_EXCERPT ) ) :
				IF ( $course = get_post( learndash_get_course_id( $_post->ID ) ) ) :
					IF ( apply_filters( 'mbr/match_user_content', $course, MBR_IGNORE_EXCERPT ) ) :
						IF ( $is_logged_in AND in_array( $course->ID, $user_course_access ) ) :
							return TRUE;
						ELSEIF ( $is_logged_in AND FALSE === $this->course_is_protected( $_post ) ) :
							return TRUE;
						ELSEIF ( ! $is_logged_in ) :
							return TRUE;
						ENDIF;
					ENDIF;
				ENDIF;
			ENDIF;

			return FALSE;

		ELSEIF ( $_post->post_type == 'sfwd-topic' ) :
			IF ( apply_filters( 'mbr/match_user_content', $_post->ID, MBR_IGNORE_EXCERPT ) ) :
				IF ( $lesson = get_post( learndash_get_lesson_id( $_post->ID ) ) ) :
					IF ( apply_filters( 'mbr/match_user_content', $lesson, MBR_IGNORE_EXCERPT ) ) :
						IF ( $course = get_post( learndash_get_course_id( $lesson->ID ) ) ) :
							IF ( apply_filters( 'mbr/match_user_content', $course, MBR_IGNORE_EXCERPT ) ) :
								return TRUE;
							ENDIF;
						ENDIF;
					ENDIF;
				ENDIF;
			ENDIF;

			return FALSE;
		ELSEIF ( $_post->post_type == 'sfwd-quiz' ) :
			






			IF ( apply_filters( 'mbr/match_user_content', $_post->ID, MBR_IGNORE_EXCERPT ) ) :

				IF ( ! $lesson_or_topic_or_course = get_post( learndash_get_lesson_id( $_post->ID ) ) ) :
					$lesson_or_topic_or_course = get_post( learndash_get_course_id( $_post->ID ) );
				ENDIF;

				IF ( $lesson_or_topic_or_course ) :
					IF ( $lesson_or_topic_or_course->post_type == 'sfwd-courses' ) :
						IF ( apply_filters( 'mbr/match_user_content', $lesson_or_topic_or_course, MBR_IGNORE_EXCERPT ) ) :
							IF ( $is_logged_in AND in_array( $lesson_or_topic_or_course->ID, $user_course_access ) ) :
								return TRUE;
							ELSE:
								return FALSE;
							ENDIF;
						ENDIF;
					ELSEIF ( $lesson_or_topic_or_course->post_type == 'sfwd-lessons' ) :
						IF ( apply_filters( 'mbr/match_user_content', $lesson_or_topic_or_course, MBR_IGNORE_EXCERPT ) ) :
							IF ( $course = get_post( learndash_get_course_id( $lesson_or_topic_or_course->ID ) ) ) :
								IF ( apply_filters( 'mbr/match_user_content', $course, MBR_IGNORE_EXCERPT ) ) :
									return TRUE;
								ENDIF;
							ENDIF;
						ENDIF;
						return FALSE;
					ELSEIF ( $lesson_or_topic_or_course->post_type == 'sfwd-topic' ) :
						IF ( apply_filters( 'mbr/match_user_content', $lesson_or_topic_or_course, MBR_IGNORE_EXCERPT ) ) :
							IF ( $lesson = get_post( learndash_get_lesson_id( $lesson_or_topic_or_course->ID ) ) ) :
								IF ( apply_filters( 'mbr/match_user_content', $lesson, MBR_IGNORE_EXCERPT ) ) :
									IF ( $course = get_post( learndash_get_course_id( $lesson_or_topic_or_course->ID ) ) ) :
										IF ( apply_filters( 'mbr/match_user_content', $course, MBR_IGNORE_EXCERPT ) ) :
											return TRUE;
										ENDIF;
									ENDIF;
								ENDIF;
							ENDIF;
						ENDIF;
						return FALSE;
					ENDIF;
				ENDIF;

			ENDIF;

			return FALSE;
		ENDIF;
	}

	public function course_is_protected( $_post )
	{
		global $current_user;

		
		$protected = TRUE;
		$perms     = apply_filters( 'mbr/permissions/get', NULL, $_post->ID );

		IF ( empty( $perms->tag_pos_list ) AND empty( $perms->tag_neg_list ) ) :
			
			IF ( empty( $perms->group_access ) ) :
				$protected = FALSE;
			
			ELSEIF ( $perms->group_access == -2 ) :
				$protected = FALSE;
			
			ELSEIF ( $current_user->ID AND $perms->group_access == '-1' ) :
				$protected = FALSE;
			ENDIF;
		ENDIF;

		return $protected;
	}

	public function learndash_post_args( $args )
	{
		$automations[0] = _x( '-- Select an automation --', 'wp_admin', 'activemember360' );

		$_automations   = apply_filters( 'mbr/site_automations/get/active', [] );

		FOREACH ( $_automations  AS $k => $v) :
			$automations[$k] = $v;
		ENDFOREACH;

		$label        = LearnDash_Custom_Label::get_label( 'course' );
		$key          = 'sfwd-courses';

		$args[ $key ]['fields']['course_completed_mbr_visible'] = [
			'name'      => $this->learndash_ext_hilite( sprintf( _x( '%s Visibility:<br/>Make visible if protected', 'wp_admin', 'activemember360' ), ucfirst( $label ) ) ),
			'type'      => 'checkbox',
			'default'   => '',
			'help_text' => sprintf( _x( 'Allow the %s to be visible even if the %s is protected.', 'wp_admin', 'activemember360' ), strtolower( $label ), strtolower( $label ) )
		];

		$args[ $key ]['fields']['course_completed_mbr_noenroll'] = [
			'name'      => $this->learndash_ext_hilite( _x( 'NO Auto-Enrollment:<br/>Don\'t autoenroll users', 'wp_admin', 'activemember360' ) ),
			'type'      => 'checkbox',
			'default'   => '',
			'help_text' => sprintf( _x( 'Only allow manual enrollment; do not automatically enroll users from this %s.', 'wp_admin', 'activemember360' ), strtolower( $label ) )
		];

		$args[ $key ]['fields']['course_completed_mbr_tagid'] = [
			'name'      => $this->learndash_ext_hilite( sprintf( _x( '%s Completed:<br/>Assign tag ID(s)', 'wp_admin', 'activemember360' ), ucfirst( $label ) ) ),
			'type'      => 'text',
			'default'   => '',
			'help_text' => sprintf( _x( 'Enter one or more tag ID\'s or names to be assigned when the %s has been completed. We highly recommend the use of tag ID\'s over tag names.', 'wp_admin', 'activemember360' ), strtolower( $label ) )
		];

		$args[ $key ]['fields']['course_completed_mbr_automation'] = [
			'name'      => $this->learndash_ext_hilite( sprintf( _x( '%s Completed:<br/>Assign to an automation', 'wp_admin', 'activemember360' ), ucfirst( $label ) ) ),
			'type'      => 'select',
			'initial_options' => $automations,
			'default'   => '',
			'help_text' => sprintf( _x( 'Select an automation to which to assign the user when the %s has been completed', 'wp_admin', 'activemember360' ), strtolower( $label ) )
		];

		$args[ $key ]['fields']['course_completed_mbr_redir'] = [
			'name'      => $this->learndash_ext_hilite( sprintf( _x( '%s Completed:<br/>Redirect user to URL', 'wp_admin', 'activemember360' ), ucfirst( $label ) ) ),
			'type'      => 'text',
			'default'   => '',
			'help_text' => sprintf( _x( 'Enter the URL to which the user should be redirected when the %s has been completed', 'wp_admin', 'activemember360' ), strtolower( $label ) )
		];

		$label        = LearnDash_Custom_Label::get_label( 'lesson' );
		$key          = 'sfwd-lessons';

		$args[ $key ]['fields']['lesson_completed_mbr_tagid'] = [
			'name'      => $this->learndash_ext_hilite( sprintf( _x('%s Completed<br/>Assign tag ID(s)', 'wp_admin', 'activemember360' ), ucfirst( $label ) ) ),
			'type'      => 'text',
			'default'   => '',
			'help_text' => sprintf( _x( 'Enter one or more tag ID\'s or names to be assigned when the %s has been completed. We highly recommend the use of tag ID\'s over tag names.', 'wp_admin', 'activemember360' ), strtolower( $label ) )
		];

		$args[ $key ]['fields']['lesson_completed_mbr_automation'] = [
			'name'      => $this->learndash_ext_hilite( sprintf( _x( '%s Completed<br/>Assign to an automation', 'wp_admin', 'activemember360' ), ucfirst( $label ) ) ),
			'type'      => 'select',
			'initial_options' => $automations,
			'default'   => '',
			'help_text' => sprintf( _x( 'Select an automation to which to assign the user when the %s has been completed', 'wp_admin', 'activemember360' ), strtolower( $label ) )
		];

		$label        = LearnDash_Custom_Label::get_label( 'quiz' );
		$key          = 'sfwd-quiz';

		$args[ $key ]['fields']['quiz_completed_mbr_tagid'] = [
			'name'      => $this->learndash_ext_hilite( sprintf( _x( '%s Passed<br/>Assign tag ID(s)', 'wp_admin', 'activemember360' ), ucfirst( $label ) ) ),
			'type'      => 'text',
			'default'   => '',
			'help_text' => sprintf( _x( 'Enter one or more tag ID\'s or names to be assigned when the %s has been completed. We highly recommend the use of tag ID\'s over tag names.', 'wp_admin', 'activemember360' ), strtolower( $label ) )
		];

		$args[ $key ]['fields']['quiz_completed_mbr_automation'] = [
			'name'      => $this->learndash_ext_hilite( sprintf( _x( '%s Passed<br/>Assign to an automation', 'wp_admin', 'activemember360' ), ucfirst( $label ) ) ),
			'type'      => 'select',
			'initial_options' => $automations,
			'default'   => '',
			'help_text' => sprintf( _x( 'Select an automation to which to assign the user when the %s has been completed', 'wp_admin', 'activemember360' ), strtolower( $label ) )
		];

		$args[ $key ]['fields']['quiz_failed_mbr_tagid'] = [
			'name'      => $this->learndash_ext_hilite( sprintf( _x( '%s Failed<br/>Assign tag ID(s)', 'wp_admin', 'activemember360' ), ucfirst( $label ) ) ),
			'type'      => 'text',
			'default'   => '',
			'help_text' => sprintf( _x( 'Enter one or more tag ID\'s or names to be assigned when the %s has been completed. We highly recommend the use of tag ID\'s over tag names.', 'wp_admin', 'activemember360' ), strtolower( $label ) )
		];

		$args[ $key ]['fields']['quiz_failed_mbr_automation'] = [
			'name'      => $this->learndash_ext_hilite( sprintf( __( '%s Failed<br/>Assign to an automation', 'wp_admin', 'activemember360' ), ucfirst( $label ) ) ),
			'type'      => 'select',
			'initial_options' => $automations,
			'default'   => '',
			'help_text' => sprintf( _x( 'Select an automation to which to assign the user when the %s has been completed', 'wp_admin', 'activemember360' ), strtolower( $label ) )
		];

		$label        = LearnDash_Custom_Label::get_label( 'topic' );
		$key          = 'sfwd-topic';

		$args[ $key ]['fields']['topic_completed_mbr_tagid'] = [
			'name'      => $this->learndash_ext_hilite( sprintf( _x( '%s Completed<br/>Assign tag ID(s)', 'wp_admin', 'activemember360' ), ucfirst( $label ) ) ),
			'type'      => 'text',
			'default'   => '',
			'help_text' => sprintf( _x( 'Enter one or more tag ID\'s or names to be assigned when the %s has been completed. We highly recommend the use of tag ID\'s over tag names.', 'wp_admin', 'activemember360'), strtolower( $label ) )
		];

		$args[ $key ]['fields']['topic_completed_mbr_automation'] = [
			'name'      => $this->learndash_ext_hilite( sprintf( _x( '%s Completed<br/>Assign to an automation', 'wp_admin', 'activemember360' ), ucfirst( $label ) ) ),
			'type'      => 'select',
			'initial_options' => $automations,
			'default'   => '',
			'help_text' => sprintf( _x( 'Select an automation to which to assign the user when the %s has been completed', 'wp_admin', 'activemember360' ), strtolower( $label ) )
		];

		return $args;
	}

	public function learndash_process_tags( $contact_id, $arrTAGS = [] )
	{
		$site_tags = apply_filters( 'mbr/site_tags/get', NULL );
		$tags      = array_map('strtolower', $site_tags);

		IF ( $arrTAGS ) :
			$arrTAGStoADD = [];
			FOREACH ( $arrTAGS AS $tag ) :
				IF ( is_numeric( $tag ) ) :
					$tag_id = (int) $tag;
					IF ( ! apply_filters( 'mbr/contact/has_tag', $tag_id ) AND isset( $site_tags[ $tag_id ] ) ) :
						$arrTAGStoADD[] = $site_tags[ $tag_id ];
					ENDIF;
				ELSE :
					IF ( in_array( lower_trim( $tag ), $tags ) ) :
						$arrTAGStoADD[] = trim( $tag );
					ENDIF;
				ENDIF;
			ENDFOREACH;
			IF ( $arrTAGStoADD ) :
				apply_filters( 'mbr/api', 'contact_tag_add', [ 'id' => $contact_id, 'tags' => $arrTAGStoADD ] );
			ENDIF;
		ENDIF;
	}

	public function learndash_course_completed( $completion_args )
	{
		$contact_id = apply_filters( 'mbr/contact_id', NULL );
		IF ( empty( $contact_id ) ) :
			return NULL;
		ENDIF;

		$sfwd_obj = get_post_meta( (int) $completion_args['course']->ID,'_sfwd-courses',TRUE );

		IF ( ! empty( $sfwd_obj['sfwd-courses_course_completed_mbr_tagid'] ) ) :
			$this->learndash_process_tags( $contact_id, array_filter( explode( ',', $sfwd_obj['sfwd-courses_course_completed_mbr_tagid'] ) ) );
		ENDIF;

		IF ( ! empty( $sfwd_obj['sfwd-courses_course_completed_mbr_automation'] ) ) :
			apply_filters( 'mbr/api', 'automation_contact_add', [ 'contact_id' => $contact_id, 'automation' => (int) $sfwd_obj['sfwd-courses_course_completed_mbr_automation'] ] );
		ENDIF;
	}

	public function learndash_lesson_completed( $completion_args )
	{
		$contact_id = apply_filters( 'mbr/contact_id', NULL );
		IF ( empty( $contact_id ) ) :
			return NULL;
		ENDIF;

		$sfwd_obj = get_post_meta( (int) $completion_args['lesson']->ID,'_sfwd-lessons',TRUE );

		IF ( ! empty( $sfwd_obj['sfwd-lessons_lesson_completed_mbr_tagid'] ) ) :
			$this->learndash_process_tags( $contact_id, array_filter( explode( ',', $sfwd_obj['sfwd-lessons_lesson_completed_mbr_tagid'] ) ) );
		ENDIF;

		IF ( ! empty( $sfwd_obj['sfwd-lessons_lesson_completed_mbr_automation'] ) ) :
			apply_filters( 'mbr/api', 'automation_contact_add', [ 'contact_id' => $contact_id, 'automation' => (int) $sfwd_obj['sfwd-lessons_lesson_completed_mbr_automation'] ] );
		ENDIF;
	}

	public function learndash_topic_completed( $completion_args )
	{
		$contact_id = apply_filters( 'mbr/contact_id', NULL );
		IF ( empty( $contact_id ) ) :
			return NULL;
		ENDIF;

		$sfwd_obj = get_post_meta( (int) $completion_args['topic']->ID,'_sfwd-topic',TRUE );

		IF ( ! empty( $sfwd_obj['sfwd-topic_topic_completed_mbr_tagid'] ) ) :
			$this->learndash_process_tags( $contact_id, array_filter( explode( ',', $sfwd_obj['sfwd-topic_topic_completed_mbr_tagid'] ) ) );
		ENDIF;

		IF ( ! empty( $sfwd_obj['sfwd-topic_topic_completed_mbr_automation'] ) ) :
			apply_filters( 'mbr/api', 'automation_contact_add', [ 'contact_id' => $contact_id, 'automation' => (int) $sfwd_obj['sfwd-topic_topic_completed_mbr_automation'] ] );
		ENDIF;
	}

	public function learndash_quiz_completed( $completion_args )
	{
		$contact_id = apply_filters( 'mbr/contact_id', NULL );
		IF ( empty( $contact_id ) ) :
			return NULL;
		ENDIF;

		global $current_user;

		$arr_quiz = array_pop( get_user_meta( (int) $current_user->ID, '_sfwd-quizzes',TRUE ) );
		$sfwd_obj = get_post_meta( (int) $completion_args['quiz']->ID, '_sfwd-quiz',TRUE );

		IF ( $arr_quiz['pass'] ) :
			IF ( ! empty( $sfwd_obj['sfwd-quiz_quiz_completed_mbr_tagid'] ) ) :
				$this->learndash_process_tags( $contact_id, array_filter( explode( ',', $sfwd_obj['sfwd-quiz_quiz_completed_mbr_tagid'] ) ) );
			ENDIF;
			IF ( ! empty($sfwd_obj['sfwd-quiz_quiz_completed_mbr_automation'] ) ) :
				apply_filters( 'mbr/api', 'automation_contact_add', [ 'contact_id' => $contact_id, 'automation' => (int) $sfwd_obj['sfwd-quiz_quiz_completed_mbr_automation'] ] );
			ENDIF;
		ELSE :
			IF ( ! empty( $sfwd_obj['sfwd-quiz_quiz_failed_mbr_tagid'] ) ) :
				$this->learndash_process_tags( $contact_id, array_filter( explode( ',', $sfwd_obj['sfwd-quiz_quiz_failed_mbr_tagid'] ) ) );
			ENDIF;
			IF ( ! empty($sfwd_obj['sfwd-quiz_quiz_failed_mbr_automation'] ) ) :
				apply_filters( 'mbr/api', 'automation_contact_add', [ 'contact_id' => $contact_id, 'automation' => (int) $sfwd_obj['sfwd-quiz_quiz_failed_mbr_automation'] ] );
			ENDIF;
		ENDIF;
	}

	public function learndash_course_completion_url( $url, $course_id )
	{
		$post_type = get_post_type( $course_id );
		IF ( $post_type == 'sfwd-courses' ) :
			IF ( $sfwd_obj = $this->get_course_attributes( $course_id ) ) :
				IF ( ! empty( $sfwd_obj['sfwd-courses_course_completed_mbr_redir'] ) ) :
					return $sfwd_obj['sfwd-courses_course_completed_mbr_redir'];
				ENDIF;
			ENDIF;
		ENDIF;

		return $url;
	}

	

	



	public function learndash_payment_button( $button_html, $params )
	{
		
		IF ( $course_attributes = $this->get_course_attributes( $params['post']->ID ) ) :
			IF ( 'free' == strtolower( $course_attributes['sfwd-courses_course_price_type'] ) AND 'on' !== strtolower( $course_attributes['sfwd-courses_course_completed_mbr_noenroll'] )) :				IF ( ! apply_filters( 'mbr/match_user_content', $params['post'], MBR_IGNORE_EXCERPT ) ) :
					return NULL;
				ENDIF;
			ENDIF;
		ENDIF;

		return $button_html;
	}

	public function learndash_template_redirect()
	{
		IF ( defined( 'MBR_NO_TEMPLATE_REDIRECT' ) OR defined( 'MBR_NO_TEMPLATE_REDIRECT_404' ) ) :
			return;
		ENDIF;

		
		$url = apply_filters( 'mbr/url', NULL );
		mbr_constant( 'MBR_URL_TO_POSTID', apply_filters( 'mbr/url_to_post_id', $url ) );

		IF ( ! MBR_URL_TO_POSTID ) :
			return;
		ENDIF;

		
		$post_type = get_post_type( MBR_URL_TO_POSTID );

		IF ( ! in_array( $post_type, [ 'sfwd-courses','sfwd-lessons','sfwd-quiz','sfwd-topic','sfwd-certificates','sfwd-assignment','sfwd-transactions','sfwd-groups','sfwd-essays' ] ) ) :
			return;
		ENDIF;

		IF ( ! is_404() ) :
			IF ( $post_type == 'sfwd-courses' ) :
				IF ( ! apply_filters( 'mbr/match_user_content', MBR_URL_TO_POSTID, TRUE ) ) :
					$redirect_no_access = get_post_meta( MBR_URL_TO_POSTID, $this->postmeta_no_access_redirect_key, TRUE );
					IF ( ! empty( $redirect_no_access ) ):
						do_action( 'mbr/redirect/no_access', MBR_URL_TO_POSTID );
						return;
					ENDIF;
				ENDIF;
			ENDIF;

			defined( 'MBR_NO_TEMPLATE_REDIRECT_404') OR define( 'MBR_NO_TEMPLATE_REDIRECT_404', 'MBR_NO_TEMPLATE_REDIRECT_404' );
			return;
		ENDIF;

		
		IF ( $post_type == 'sfwd-courses' ) :
			
			do_action( 'mbr/redirect/no_access', MBR_URL_TO_POSTID );
			
			do_action( 'mbr/redirect', site_url( '/' . strtolower( LearnDash_Custom_Label::get_label( 'courses' ) ) ) );
			
			do_action( 'mbr/redirect/op_sn', MBR_URL_TO_POSTID );

			return;
		ENDIF;

		IF ( in_array($post_type, [ 'sfwd-certificates','sfwd-assignment','sfwd-transactions','sfwd-groups','sfwd-essays' ] ) ) :
			
			IF ( $course_id = learndash_get_course_id( MBR_URL_TO_POSTID ) ) :
				
				IF ( apply_filters( 'mbr/match_user_content', $course_id ) ) :
					do_action( 'mbr/redirect', get_permalink( $course_id ) );
				ENDIF;
				
				do_action( 'mbr/redirect/no_access', $course_id );
				
				do_action( 'mbr/redirect', site_url( '/' . strtolower( LearnDash_Custom_Label::get_label( 'courses' ) ) ) );
				
				do_action( 'mbr/redirect/op_sn', $course_id );
			ENDIF;

			return;
		ENDIF;

		
		IF ( in_array( $post_type, [ 'sfwd-lessons','sfwd-quiz','sfwd-topic' ] ) ) :
			IF ( $post_type == 'sfwd-quiz' ) :
				IF ( $lesson_or_topic_id = learndash_get_lesson_id( MBR_URL_TO_POSTID ) ) :
					
					IF ( apply_filters( 'mbr/match_user_content', $lesson_or_topic_id ) ) :
						do_action( 'mbr/redirect', get_permalink( $lesson_or_topic_id ) );
					ENDIF;
					IF ( $lesson_id = learndash_get_lesson_id( $lesson_or_topic_id ) ) :
						
						IF ( apply_filters( 'mbr/match_user_content', $lesson_id ) ) :
							do_action( 'mbr/redirect', get_permalink( $lesson_id ) );
						ENDIF;
					ENDIF;
				ENDIF;
			ENDIF;

			IF ( $post_type == 'sfwd-topic' ) :
				IF ( $lesson_id = learndash_get_lesson_id( MBR_URL_TO_POSTID ) ) :
					
					IF ( apply_filters( 'mbr/match_user_content', $lesson_id ) ) :
						do_action( 'mbr/redirect', get_permalink( $lesson_id ) );
					ENDIF;
				ENDIF;
			ENDIF;

			
			IF ( $course_id = learndash_get_course_id( MBR_URL_TO_POSTID ) ) :
				
				IF ( apply_filters( 'mbr/match_user_content', $course_id ) ) :
					do_action( 'mbr/redirect', get_permalink( $course_id ) );
				ENDIF;
				
				do_action( 'mbr/redirect/no_access', $course_id );
				
				do_action( 'mbr/redirect', site_url( '/' . strtolower( LearnDash_Custom_Label::get_label( 'courses' ) ) ) );
				
				do_action( 'mbr/redirect/op_sn', $course_id );
			ENDIF;
			do_action( 'mbr/redirect', site_url('/' . strtolower( LearnDash_Custom_Label::get_label( 'courses' ) ) ) );
		ENDIF;
	}

	public function learndash_ext_hilite( $str = '' )
	{
		$retVAL = "<span style='color:gray;'><i style='color:darkblue;font-size:.9em'>AM360</i>: " . __( $str, 'activemember360' );

		return str_ireplace( '<br/>', '</span><br/><span style=\'color:darkblue;font-size:.8em;float:left;\'>', $retVAL );
	}

}
$GLOBALS['mbrlearndash'] = mbr_learndash::getInstance();
