<?php
final class mbr_mailer EXTENDS mbr_redux_base_class
{
	use TRAIT__getInstance;

	const MBR_ROUTE_PREFIX           = 'mbr-mailer';

	public $opt_name_settings_parent = 'mbr_mailer_settings_parent';
	public $opt_name_settings        = 'mbr_mailer_settings';
	public $page_slug_parent         = 'mbr360-mailer-parent';
	public $page_slug                = 'mbr360-mailer';
	public $rewrite_endpoint         = 'mbr-mailer-optout';

	public $openssl_iv;
	public $openssl_method           = 'aes-128-cbc';
	public $openssl_key              = 'activemember360';

	public function plugins_loaded()
	{
		$this->openssl_iv              = substr( md5( $this->openssl_method ), 0, 16 );

		add_action( 'mbr/route/prefix',                                               [ $this, 'register_route_prefix' ],        10, 1 );
		add_filter( 'mbr/routes/register/nonadmin',                                   [ $this, 'register_routes' ],              10, 1 );
		add_filter( 'mbr/endpoints/rewrite',                                          [ $this, 'rewrite_endpoints' ],            10, 1 );
		add_action( 'template_redirect',                                              [ $this, 'template_redirect_optout' ],         0 );

		add_action( 'plugins_loaded',                                                 [ $this, 'init_admin_panel' ],             99998 );
		add_action( 'plugins_loaded',                                                 [ $this, 'init_sections' ],                99999 );
		add_filter( "redux/{$this->opt_name_settings}/panel/template/header.tpl.php", [ $this, 'alternate_header' ]                    );
		add_action( "redux/options/{$this->opt_name_settings}/validate",              [ $this, 'validate_save_settings' ] );
		add_action( "redux/options/{$this->opt_name_settings}/saved",                 [ $this, 'settings_saved' ],             9999, 2 );

		add_filter( 'cron_schedules',                                                 [ $this, 'cron_schedules' ],                   0 );
		add_action( 'init',                                                           [ $this, 'setup_schedules'],                   0 );
		add_filter( 'custom_menu_order',                                              [ $this, 'submenu_order' ],                10, 1 );
		add_action( 'mbr/mailer/templates/refresh',                                   [ $this, 'refresh_templates' ] );
	}

	public function submenu_order( $menu_order )
	{
		global $submenu;

		$main_item          = FALSE;
		$order              = [];

		FOREACH ( $submenu[ $this->page_slug_parent ] AS $key => $submenu_item ) :
			IF ( $submenu_item[0] == __( 'Options Object', 'redux-framework' ) ) :
				unset( $submenu[ $this->page_slug_parent ][ $key ]);
			ELSEIF ( $submenu_item[0] == _x( 'Settings', 'wp_admin', 'activemember360' ) ):
				$main_item      = $submenu_item;
			ELSE:
				IF ( $submenu_item[2] <> $this->page_slug_parent ) :
					$order[]      = $submenu_item;
				ENDIF;
			ENDIF;
		ENDFOREACH;

		IF ( ! empty( $main_item ) ) :
			array_unshift( $order, $main_item );
		ENDIF;

		$submenu[ $this->page_slug_parent ] = $order;

		
		return $menu_order;
	}

	public function init_admin_panel()
	{
		global $MBR_REDUX_HEADER_MODULE_NAMES;

		$admin_title        = _x( 'Mailer', 'wp_admin', 'activemember360' );

		$arr_config         =  [
			'opt_name'        => $this->opt_name_settings_parent,
			'page_slug'       => $this->page_slug_parent,
			'admin_title'     => $admin_title,
			'menu_title'      => $admin_title,
			'menu_icon'       => apply_filters( 'mbr/plugin_favicon', NULL ),
		];

		$arr_config         = array_merge( $this->arr_default_main_admin_panel_settings, $arr_config );

		Redux::setArgs( $this->opt_name_settings_parent, $arr_config );

		$MBR_REDUX_HEADER_MODULE_NAMES[ $this->opt_name_settings ] = 'Mailer Module';

		Redux::setExtensions( $this->opt_name_settings, MBR_REDUX_EXTENSIONS_DIR );

		$admin_title        = $page_title = _x( 'Settings', 'wp_admin', 'activemember360' );
		$page_title         = apply_filters( 'mbr/plugin_name', NULL ) . ' ' . $page_title;

		$arr_config         =  [
			'opt_name'        => $this->opt_name_settings,
			'global_variable' => $this->opt_name_settings,
			'page_slug'       => $this->page_slug,
			'page_parent'     => $this->page_slug_parent,
			'page_title'      => $page_title,
			'display_name'    => $page_title,
			'admin_title'     => $admin_title,
			'menu_title'      => $admin_title,
		];

		$arr_config         = array_merge( $this->arr_default_sub_admin_panel_settings, $arr_config );

		Redux::setArgs( $this->opt_name_settings, $arr_config );
	}

	public function init_sections()
	{
		global $pagenow;

		IF ( $pagenow <> 'options.php' AND $pagenow <> 'admin.php' OR ( $pagenow == 'admin.php' AND ( empty( $_GET['page'] ) OR $_GET['page'] <> $this->page_slug ) ) ) :
			return;
		ENDIF;

		$arr_section_settings     =
			[
				'id'                    => 'section_mailer_settings',
				'title'                 => _x( 'Settings', 'wp_admin', 'activemember360' ),
				'desc'                  => _x( 'Mailer Settings of ActiveMember360\'s integrated mailing extension.', 'wp_admin', 'activemember360' ),
				'icon'                  => 'el el-cog',
				'fields'                => [
					[
						'id'                => 'mbr_mailer_unsubscribe_page',
						'type'              => 'select',
						'title'             => _x( 'Unsubscribe Page', 'wp_admin', 'activemember360' ),
						'desc'              => _x( 'Choose the page to which the unsubscribe link in your mail templates will point to and will automatically unsubscribe your contact.', 'wp_admin', 'activemember360' ),
						'data'              => 'pages',
						'placeholder'       => _x( 'Select unsubscribe page', 'wp_admin', 'activemember360' ),
						'select2'           => [ 'allowClear' => TRUE ],
						'validate'          => 'not_empty'
					],
					[
						'id'                => 'mbr_mailer_ssl_verify',
						'title'             => _x( 'SSL Verify', 'wp_admin', 'activemember360' ),
						'desc'              => _x( 'Verify SSL peer when sending emails and do not allow self signed certificates.', 'wp_admin', 'activemember360' ) . '<br />' . _x( 'If your SSL certificate is not set up correctly or do not match with the domain you\'re trying to send through or an self signed certificate is involved, PHPMailer per default rejects the attempt to send the email. We recommend to fix your certificates, if this is not possible, which can be the case when sending through a 3rd party mailer set this option to off.', 'wp_admin', 'activemember360' ),
						'type'              => 'switch',
						'default'           => 1,
					],
					[
						'id'                => 'mbr_mailer_debug_mode',
						'title'             => _x( 'Debug Mode', 'wp_admin', 'activemember360' ),
						'desc'              => _x( 'Enable debug mode.', 'wp_admin', 'activemember360' ) . '<br />' . _x( 'When enabled, PHPMailer error codes will be display on screen. Only for use when testing through your browser.', 'wp_admin', 'activemember360' ),
						'type'              => 'switch',
						'default'           => 0
					],
					[
						'id'                => 'mbr_mailer_cache_refresh',
						'title'             => _x( 'Cache refresh in minutes', 'wp_admin', 'activemember360' ),
						'desc'              => _x( 'Fill in the number of minutes after the email template cache should be refreshed automatically.', 'wp_admin', 'activemember360' ),
						'type'              => 'text',
						'default'           => 60,
						'validate'          => 'numeric_not_empty',
					],
					[
						'id'                => 'mbr_mailer_settings_notes',
						'title'             => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
						'type'              => 'textarea'
					]
				]
			];

		$arr_section_gateways     =
			[
				'id'                    => 'section_mailer_gateways',
				'title'                 => _x( 'Gateways', 'wp_admin', 'activemember360' ),
				'desc'                  => _x( 'Mailer Gateways of ActiveMember360\'s integrated mailing extension.', 'wp_admin', 'activemember360' ),
				'icon'                  => 'el el-cog',
				'fields'                => [
					[
						'id'                => 'mbr_mailer_smtp_gateways',
						'title'             => _x( 'SMTP Gateways', 'wp_admin', 'activemember360' ),
						'subtitle'          => _x( 'Drag and drop your gateways as you wish. Your default gateway will be the one at the very top.<p><strong>All fields require input.</strong></p>', 'wp_admin', 'activemember360' ),
						'type'              => 'repeater',
						'item_name'         => _x( 'SMTP Gateway', 'wp_admin', 'activemember360' ),
						'bind_title'        => 'gateway_id',
						'group_values'      => TRUE,
						'sortable'          => TRUE,
						'limit'             => 9999,
						'fields'            => [
							[
								'id'            => 'gateway_status',
								'type'          => 'select',
								'title'         => _x( 'Gateway Status', 'wp_admin', 'activemember360' ),
								'options'       => [ 'enabled' => _x( 'Enabled', 'wp_admin', 'activemember360' ), 'disabled' => _x( 'Disabled', 'wp_admin', 'activemember360' ), 'pending_correction' => _x( 'Disabled Pending Correction', 'wp_admin', 'activemember360') ],
								'default'       => 'enabled',
								'select2'       => [ 'allowClear' => FALSE ]
							],
							[
								'id'            => 'gateway_id',
								'title'         => _x( 'Gateway ID', 'wp_admin', 'activemember360' ),
								'type'          => 'text'
							],
							[
								'id'            => 'sender_email',
								'title'         => _x( 'Sender Email', 'wp_admin', 'activemember360' ),
								'type'          => 'text'
							],
							[
								'id'            => 'sender_name',
								'title'         => _x( 'Sender Name', 'wp_admin', 'activemember360' ),
								'type'          => 'text'
							],
							[
								'id'            => 'secure_mode',
								'title'         => _x( 'Secure Mode', 'wp_admin', 'activemember360' ),
								'type'          => 'select',
								'options'       => [ 'none' => _x( 'None', 'wp_admin', 'activemember360' ), 'ssl' => _x( 'SSL', 'wp_admin', 'activemember360' ), 'tls' => _x( 'TLS', 'wp_admin', 'activemember360' ) ],
								'placeholder'   => _x( 'Select secure mode', 'wp_admin', 'activemember360' ),
								'default'       => 'none',
								'select2'       => [ 'allowClear' => FALSE ]
							],
							[
								'id'            => 'host_url',
								'title'         => _x( 'Host URL', 'wp_admin', 'activemember360' ),
								'type'          => 'text'
							],
							[
								'id'            => 'port',
								'title'         => _x( 'Port', 'wp_admin', 'activemember360' ),
								'type'          => 'text'
							],
							[
								'id'            => 'username',
								'title'         => _x( 'Username', 'wp_admin', 'activemember360' ),
								'type'          => 'text'
							],
							[
								'id'            => 'password',
								'title'         => _x( 'Password', 'wp_admin', 'activemember360' ),
								'type'          => 'password'
							],
						]
					]
				]
			];

		Redux::setSection( $this->opt_name_settings, $arr_section_settings );
		Redux::setSection( $this->opt_name_settings, $arr_section_gateways );
	}

	public function validate_save_settings( &$arr_settings )
	{
		IF ( ! empty( $arr_settings['mbr_mailer_smtp_gateways']['gateway_id'] ) ) :
			FOREACH ( $arr_settings['mbr_mailer_smtp_gateways']['gateway_id'] AS $key => $gateway_id ) :
				$pending_correction     = FALSE;
				IF ( $arr_settings['mbr_mailer_smtp_gateways']['gateway_status'][ $key ] <> 'disabled' ) :
					IF ( empty( $gateway_id ) ) :
						$pending_correction = TRUE;
					ELSEIF ( empty( $arr_settings['mbr_mailer_smtp_gateways']['username'][ $key ] ) ) :
						$pending_correction = TRUE;
					ELSEIF ( empty( $arr_settings['mbr_mailer_smtp_gateways']['password'][ $key ] ) ) :
						$pending_correction = TRUE;
					ELSEIF ( empty( $arr_settings['mbr_mailer_smtp_gateways']['host_url'][ $key ] ) ) :
						$pending_correction = TRUE;
					ELSEIF ( empty( $arr_settings['mbr_mailer_smtp_gateways']['port'][ $key ] ) OR ! is_numeric( $arr_settings['mbr_mailer_smtp_gateways']['port'][ $key ] )) :
						$pending_correction = TRUE;
					ELSEIF ( empty( $arr_settings['mbr_mailer_smtp_gateways']['sender_email'][ $key ] ) OR ! is_email( $arr_settings['mbr_mailer_smtp_gateways']['sender_email'][ $key ] ) ) :
						$pending_correction = TRUE;
					ELSEIF ( empty( $arr_settings['mbr_mailer_smtp_gateways']['sender_name'][ $key ] ) ) :
						$pending_correction = TRUE;
					ENDIF;

					IF ( $pending_correction ) :
						$arr_settings['mbr_mailer_smtp_gateways']['gateway_status'][ $key ] = 'pending_correction';
					ELSEIF ( $arr_settings['mbr_mailer_smtp_gateways']['gateway_status'][ $key ] == 'pending_correction' ) :
						$arr_settings['mbr_mailer_smtp_gateways']['gateway_status'][ $key ] = 'enabled';
					ENDIF;
				ENDIF;
			ENDFOREACH;
		ENDIF;
	}

	public function settings_saved( $arr_settings = [] )
	{
		$arr_gateways      = [];
		IF ( ! empty( $arr_settings['mbr_mailer_smtp_gateways']['gateway_id'] ) ) :
			FOREACH ( $arr_settings['mbr_mailer_smtp_gateways']['gateway_id'] AS $key => $gateway_id ) :
				$arr_gateways[ $gateway_id ] = [
					'gateway_status' => $arr_settings['mbr_mailer_smtp_gateways']['gateway_status'][ $key ],
					'sender_email'   => $arr_settings['mbr_mailer_smtp_gateways']['sender_email'][ $key ],
					'sender_name'    => $arr_settings['mbr_mailer_smtp_gateways']['sender_name'][ $key ],
					'secure_mode'    => $arr_settings['mbr_mailer_smtp_gateways']['secure_mode'][ $key ],
					'host_url'       => $arr_settings['mbr_mailer_smtp_gateways']['host_url'][ $key ],
					'port'           => $arr_settings['mbr_mailer_smtp_gateways']['port'][ $key ],
					'username'       => $arr_settings['mbr_mailer_smtp_gateways']['username'][ $key ],
					'password'       => $arr_settings['mbr_mailer_smtp_gateways']['password'][ $key ]
				];
			ENDFOREACH;
		ENDIF;

		wp_clear_scheduled_hook( 'mbr/mailer/templates/refresh' );
		update_option( 'mbr_mailer_all_gateways', $arr_gateways );
	}

	public function cron_schedules( $schedules )
	{
		IF ( ! array_key_exists( 'mbr_cron_mailer_templates_refresh_schedule', $schedules ) ) :
			$arr_settings  = get_option( 'mbr_mailer_settings', FALSE );
			IF ( empty( $arr_settings ) OR empty( $arr_settings['mbr_mailer_cache_refresh'] ) ) :
				$seconds     = 86400;
			ELSE:
				$seconds     = $arr_settings['mbr_mailer_cache_refresh'] * 60;
			ENDIF;
			$schedules['mbr_cron_mailer_templates_refresh_schedule'] = [
				'interval'   => $seconds,
				'display'    => sprintf( _x( 'Once every %s hours', 'wp_admin', 'activemember360' ), round( ( $seconds / 3600 ), 2 ) )
			];
		ENDIF;

		return $schedules;
	}

	public function setup_schedules()
	{
		IF ( ! wp_next_scheduled( 'mbr/mailer/templates/refresh' ) ) :
			wp_schedule_event( time(), 'mbr_cron_mailer_templates_refresh_schedule', 'mbr/mailer/templates/refresh' );
		ENDIF;
	}

	public function refresh_templates()
	{
		apply_filters( 'mbr/site_messages/templates/fetch', [] );
	}

	public function register_route_prefix( $route_prefixes = [] )
	{
		$route_prefixes[] = self::MBR_ROUTE_PREFIX;

		return array_merge( $route_prefixes, [ self::MBR_ROUTE_PREFIX ] );
	}

	public function register_routes( $route_args = [] )
	{
		$arr_request_types                        = [ 'POST' ];

		$arr_mailer_settings                      = get_option( 'mbr_mailer_settings', FALSE );
		IF ( ! empty( $arr_mailer_settings['mbr_mailer_debug_mode'] ) ) :
			$arr_request_types[]                    = 'GET';
		ENDIF;

		$route_args['mailer']['methods']          = $arr_request_types;
		$route_args['mailer']['operations']['v1'] = [ $this, 'send_mail' ];

		return $route_args;
	}

	public function rewrite_endpoints( $rewrite_endpoints = [] )
	{
		$rewrite_endpoints[ $this->rewrite_endpoint ] = EP_ROOT;

		return $rewrite_endpoints;
	}

	public function template_redirect_optout()
	{
		global $wp_query, $wp_rewrite;

		$mailer_endpoints        = $this->rewrite_endpoints();

		$endpoint_confirmed      = FALSE;
		FOREACH ( $wp_rewrite->endpoints AS $key => $arr_endpoint ) :
			IF ( array_key_exists( $arr_endpoint[1], $mailer_endpoints ) ) :
				$endpoint_confirmed  = TRUE;
				break;
			ENDIF;
		ENDFOREACH;

		IF ( ! $endpoint_confirmed ) :
			return;
		ENDIF;

		IF ( ! empty( $wp_query->query_vars[ $this->rewrite_endpoint ] ) ) :
			$status                = 'error';

			$tmp_data              = explode( '/', $wp_query->query_vars[ $this->rewrite_endpoint ] );

			$segment_1             = base64_decode( $tmp_data[0] );
			$segment_2             = base64_decode( $tmp_data[1] );
			$decrypted_contact     = openssl_decrypt( $segment_1, $this->openssl_method, md5( $this->openssl_key ), 0, $this->openssl_iv );
			$decrypted_lists       = openssl_decrypt( $segment_2, $this->openssl_method, md5( $this->openssl_key ), 0, $this->openssl_iv );
			$unsubscribe_lists     = $decrypted_lists;

			$tmp_contact           = explode( '||', $decrypted_contact );
			$email                 = $tmp_contact[0];
			$contact_id            = $tmp_contact[1];

			IF ( empty( $email ) OR ! is_email( $email) OR empty( $contact_id ) ) :
				GOTO REDIRECT_POINT;
			ENDIF;

			$remote_contact        = apply_filters( 'mbr/fetch_remote_contact/by_id', $contact_id );

			IF ( empty( $remote_contact ) OR $remote_contact['email'] <> $email ) :
				GOTO REDIRECT_POINT;
			ENDIF;

			$arr_site_lists        = apply_filters( 'mbr/site_lists/get', [] );

			$optout                = [];
			IF ( empty( $unsubscribe_lists ) OR $unsubscribe_lists == 'all' ) :
				IF ( ! empty( $arr_site_lists ) ) :
					FOREACH ( $arr_site_lists AS $list_id => $filler ) :
						$optout[]        = $list_id;
					ENDFOREACH;
				ENDIF;
			ELSE:
				$tmp_optout          = explode(',', $unsubscribe_lists );
				IF ( ! empty( $tmp_optout) ) :
					FOREACH ( $tmp_optout AS $key => $list_id ) :
						$optout[]        = trim( $list_id );
					ENDFOREACH;
					$optout            = array_filter( $optout );
					$optout            = array_unique( $optout );
				ENDIF;
			ENDIF;

			IF ( ! empty( $optout ) ) :
				$arr_update          = [];
				$arr_update['email'] = $email;

				FOREACH ( $optout AS $key => $list_id ) :
					IF ( array_key_exists( $list_id, $arr_site_lists ) ) :
						$arr_update['p[' . $list_id. ']']      = $list_id;
						$arr_update['status[' . $list_id. ']'] = 2;
					ENDIF;
				ENDFOREACH;

				IF ( apply_filters( 'mbr/api', 'contact_sync', $arr_update ) ) :
					$status            = 'success';
				ENDIF;
			ENDIF;

			REDIRECT_POINT:
			$arr_mailer_settings   = get_option( 'mbr_mailer_settings', FALSE );
			$redirect              = get_permalink( $arr_mailer_settings['mbr_mailer_unsubscribe_page'] );
			do_action( 'mbr/redirect', add_query_arg( [ 'status' => $status ], $redirect ) );
		ENDIF;
	}

	public function php_mail( $arr_settings )
	{
		global $phpmailer, $wp_version;

		IF ( version_compare( $wp_version, '5.5', '<' ) ) :
			IF ( ! ( $phpmailer instanceof PHPMailer ) ) :
				require_once ABSPATH . WPINC . '/class-phpmailer.php';
				require_once ABSPATH . WPINC . '/class-smtp.php';
				$phpmailer = new PHPMailer( TRUE );
			ENDIF;
		ELSE:
			IF ( ! ( $phpmailer instanceof \PHPMailer\PHPMailer ) ) :
				require_once ABSPATH . WPINC . '/PHPMailer/PHPMailer.php';
				require_once ABSPATH . WPINC . '/PHPMailer/SMTP.php';
				$phpmailer = new \PHPMailer\PHPMailer\PHPMailer( TRUE );
			ENDIF;
		ENDIF;

		$phpmailer->IsSMTP();
		$phpmailer->SMTPAuth      = 1;

		IF ( empty( $arr_settings['ssl_verify'] ) ) :
			$phpmailer->SMTPOptions = [
				'ssl' => [
					'verify_peer'       => FALSE,
					'verify_peer_name'  => FALSE,
					'allow_self_signed' => TRUE
				]
			];
		ENDIF;

		IF ( ! empty( $arr_settings['debug_mode'] ) ) :
			$phpmailer->SMTPDebug   = 1;
		ENDIF;

		// Empty out the values that may be set
		$phpmailer->ClearAddresses();
		$phpmailer->ClearAllRecipients();
		$phpmailer->ClearAttachments();
		$phpmailer->ClearBCCs();
		$phpmailer->ClearCCs();
		$phpmailer->ClearCustomHeaders();
		$phpmailer->ClearReplyTos();
		$phpmailer->IsHTML( TRUE );

		// Set values
		$phpmailer->AddAddress( utf8_decode( urldecode( $arr_settings['recipient'] ) ) );
		$phpmailer->Subject       = $arr_settings['template']['subject'];
		$phpmailer->Body          = $arr_settings['template']['content'];
		$phpmailer->From          = $arr_settings['gateway']['sender_email'];
		$phpmailer->FromName      = $arr_settings['gateway']['sender_name'];
		$phpmailer->Sender        = $phpmailer->From;
		$phpmailer->AddReplyTo( $phpmailer->From, $phpmailer->FromName );
		$phpmailer->Host          = $arr_settings['gateway']['host_url'];
		$phpmailer->SMTPSecure    = $arr_settings['gateway']['secure_mode'];
		$phpmailer->Port          = $arr_settings['gateway']['port'];
		$phpmailer->Username      = $arr_settings['gateway']['username'];
		$phpmailer->Password      = $arr_settings['gateway']['password'];

		TRY {
			IF ( $phpmailer->Send() ) :
				return TRUE;
			ENDIF;
		} CATCH ( phpmailerException $e ) {
			return FALSE;
		}
	}

	public function send_mail($data)
	{
		IF (  empty( $data['seccode'] ) OR ! apply_filters( 'mbr/secodes/validate', urldecode( $data['seccode'] ) ) ) :
			header( $_SERVER['SERVER_PROTOCOL'] . ' 412 Incorrect Security Code' );
			exit;
		ENDIF;

		IF ( ! isset( $_REQUEST['force'] ) ) :
			IF ( ! $this->check_opt_status( $_REQUEST['contact']['id'] ) ) :
				exit;
			ENDIF;
		ENDIF;

		IF ( empty( $_REQUEST['template'] ) ) :
			exit;
		ENDIF;

		IF ( empty( $_REQUEST['contact']['email'] ) AND is_email( $_REQUEST['contact']['email'] ) ) :
			exit;
		ENDIF;

		$arr_templates                      = apply_filters( 'mbr/site_messages/templates/fetch', NULL );
		IF ( ! isset( $arr_templates[ $_REQUEST['template'] ] ) ) :
			exit;
		ENDIF;

		$arr_gateways                       = get_option( 'mbr_mailer_all_gateways', [] );
		IF ( empty( $arr_gateways ) OR ! is_array( $arr_gateways ) ) :
			exit;
		ENDIF;

		$optout                             = $_REQUEST['optout'];
		IF ( empty( $_REQUEST['optout'] ) OR $_REQUEST['optout'] == 'all' ) :
			$optout                           = 'all';
		ENDIF;

		$arr_mailer_settings                = get_option( 'mbr_mailer_settings', FALSE );

		$arr_php_mailer['recipient']        = $_REQUEST['contact']['email'];
		$arr_php_mailer['ssl_verify']       = $arr_mailer_settings['mbr_mailer_ssl_verify'];
		$arr_php_mailer['debug_mode']       = $arr_mailer_settings['mbr_mailer_debug_mode'];
		$arr_php_mailer['unsubscribe_page'] = $arr_mailer_settings['mbr_mailer_unsubscribe_page'];
		$arr_php_mailer['unsubscribe_list'] = $optout;

		IF ( ! empty( $_REQUEST['gateway'] ) ) :
			IF ( $_REQUEST['gateway'] <> 'random' ) :
				IF ( ! empty( $arr_gateways[ $_REQUEST['gateway'] ] ) ) :
					$arr_php_mailer['gateway']    = $arr_gateways[ $_REQUEST['gateway'] ];
				ENDIF;
			ELSE:
				$gateway_id                     = array_rand( $arr_gateways, 1 );
				IF ( ! empty( $arr_gateways[ $gateway_id ] ) ) :
					$arr_php_mailer['gateway']    = $arr_gateways[ $gateway_id ];
				ENDIF;
			ENDIF;
		ENDIF;

		IF ( empty( $arr_php_mailer['gateway'] ) ) :
			$arr_php_mailer['gateway']        = array_values( $arr_gateways )[0];
		ENDIF;

		IF ( empty( $arr_php_mailer['gateway'] ) OR ! is_array( $arr_php_mailer['gateway'] ) ) :
			exit;
		ENDIF;

		IF ( $arr_php_mailer['gateway']['gateway_status'] == 'enabled' ) :
			$template                           = $arr_templates[ $_REQUEST['template'] ];
			$arr_php_mailer['template']         = [ 'subject' => trim( $template->name ), 'content' => $this->merge_content( $template->content, $_REQUEST['contact'], $arr_php_mailer['unsubscribe_page'], $arr_php_mailer['unsubscribe_list'] ) ];
			IF ( $this->php_mail( $arr_php_mailer ) ) :
				$send_status                      = 'Email sent.';
			ELSE:
				$send_status                      = 'Email NOT sent.';
			ENDIF;
		ELSE:
			$send_status                        = 'Email NOT sent: ' . $arr_php_mailer['gateway']['gateway_status'];
		ENDIF;
		IF ( ! empty( $arr_php_mailer['debug_mode'] ) ) :
			echo '==> ' . $send_status;
		ENDIF;

		http_response_code( 200 );
		exit;
	}

	public function check_opt_status( $contact_id )
	{
		IF ( empty( $contact_id ) ) :
			return FALSE;
		ENDIF;

		IF ( $contact = apply_filters( 'mbr/fetch_remote_contact/by_id', $contact_id ) ) :
			IF ( empty( $contact['lists'] ) ) :
				return TRUE;
			ELSE:
				FOREACH ( $contact['lists'] AS $key => $arr_list ) :
					IF ( $arr_list['status'] == 1 ) :
						return TRUE;
					ENDIF;
				ENDFOREACH;
			ENDIF;
		ENDIF;

		return FALSE;
	}

	public function merge_content( $content, $contact, $unsubscribe_page_id, $unsubscribe_list_id )
	{
		$content                = trim( $content );

		$unsubscribe_page_url   = '';
		IF ( ! empty( $unsubscribe_page_id ) ) :
			$encrypted_contact    = openssl_encrypt( $contact['email'] . '||' . $contact['id'], $this->openssl_method, md5( $this->openssl_key ), 0, $this->openssl_iv );
			$encrypted_lists      = openssl_encrypt( $unsubscribe_list_id, $this->openssl_method, md5( $this->openssl_key ), 0, $this->openssl_iv );
			$unsubscribe_page_url = home_url( $this->rewrite_endpoint ) . '/' . base64_encode( $encrypted_contact ) . '/' . base64_encode( $encrypted_lists );
		ENDIF;

		


		$user_found        = FALSE;
		$wp_contact        = get_user_by( 'email', $contact['email'] );

		IF ( ! is_wp_error( $wp_contact ) AND ! empty( $wp_contact->ID ) ) :
			$user_found      = TRUE;
			$wp_contact_meta = get_user_meta( $wp_contact->ID );
		ENDIF;

		preg_match_all( '/%%WP.([\S]*)%%/', $content, $matches_wp_placeholders );

		FOREACH ( $matches_wp_placeholders[0] AS $key => $placeholder ) :
			$replace     = '';
			$user_field  = $matches_wp_placeholders[1][ $key ];
			IF ( $user_found AND ! empty( $user_field ) ) :

				IF ( ! empty( $wp_contact->data->$user_field ) ) :
					$replace = $wp_contact->data->$user_field;
				ELSEIF ( ! empty( $wp_contact_meta[ $user_field ][0] ) ):
					$replace = $wp_contact_meta[ $user_field ][0];
				ENDIF;
				IF ( empty( $replace ) ) :// fallback in case of NULL, FALSE to have a valid str_ireplace
					$replace = '';
				ENDIF;
			ENDIF;
			$content     = str_ireplace( $placeholder, $replace, $content );
		ENDFOREACH;

		
		preg_match_all( '/%TODAY([\S]*)%/', $content, $matches_todays );

		FOREACH ( $matches_todays[0] AS $key => $placeholder ) :
			IF ( ! empty( $matches_todays[1][ $key ] ) ) :
				IF ( $matches_todays[1][ $key ] == '+0' OR $matches_todays[1][ $key ] == '-0' ) :
					$replace = date( get_option( 'date_format' ) );
				ELSEIF ( $matches_todays[1][ $key ] == '+1' OR $matches_todays[1][ $key ] == '-1') :
					$replace = date( get_option( 'date_format' ), strtotime( $matches_todays[1][ $key ] . ' day' ) );
				ELSE:
					$replace = date( get_option( 'date_format' ), strtotime( $matches_todays[1][ $key ] . ' days' ) );
				ENDIF;
			ELSE:
				$replace   = date( get_option( 'date_format' ) );
			ENDIF;
			$content     = str_ireplace( $placeholder, $replace, $content );
		ENDFOREACH;

		
		
		$SLASHED_TAGS  = [];
		FOREACH ( $contact AS $field => $value ) :

			IF ( $field == 'tags' ) :
				$arr_slashed_tags   = explode(',', $value );
				IF ( ! empty( $arr_slashed_tags ) ) :
					FOREACH ( $arr_slashed_tags AS $key => $tag_name ) :
						$SLASHED_TAGS[] = trim( $tag_name );
					ENDFOREACH;
				ENDIF;
				continue;
			ENDIF;

			IF ( $field == 'first_name' ) :
				$FIRSTNAME = $value;
				continue;
			ENDIF;

			IF ( $field == 'last_name' ) :
				$LASTNAME = $value;
				continue;
			ENDIF;

			IF ( $field == 'full_name' ) :
				$FULLNAME = $value;
				continue;
			ENDIF;

			IF ( $field == 'fields' ) :
				IF ( ! empty( $value ) ) :
					FOREACH ( $value AS $custom_field => $custom_value ) :
						$custom_field   = strtoupper( str_replace( ['-', ' ' ], '_', $custom_field ) );
						// to avoid $60 vars which causing fatal errors in eval
						IF ( ctype_digit( substr( $custom_field, 0, 1 ) ) ) :
							$custom_field = '_' . $custom_field;
						ENDIF;
						$$custom_field  = $custom_value;
					ENDFOREACH;
				ENDIF;
				continue;
			ENDIF;

			$standard_field  = strtoupper( $field );
			$$standard_field = $value; // creates $EMAIL, and other standard fields etc.
		ENDFOREACH;

		
		preg_match_all( '#<!-- %IF (.*)% -->[\t\r\n\f]*(.*)[\t\r\n\f]*<!-- %/IF% -->#smU', $content, $matches_conditions );

		$arr_conditional_blocks     = [];
		FOREACH ( $matches_conditions[0] AS $key => $arr_full_matches ) :

			$current_condition = $matches_conditions[1][ $key ];
			preg_match_all( '#(\$\d+.*)#smU', $current_condition, $matches_digit_vars );

			FOREACH ( $matches_digit_vars[0] AS $k => $v ) :
				$current_condition      = str_replace( $v, '$_' . substr( $v, 1 ), $current_condition );
			ENDFOREACH;

			$arr_conditional_blocks[] = [
				'condition'             => str_replace( [ '&gt;', '&lt;' ], [ '>', '<' ], $current_condition ),
				'old_content'           => $matches_conditions[0][ $key ],
				'new_content'           => $matches_conditions[2][ $key ]
			];
		ENDFOREACH;

		
		FOREACH ( $arr_conditional_blocks AS $key => $arr_block ) :
			IF ( ! empty( $arr_block['condition'] ) ) :
				$conditional_result = eval( "return {$arr_block['condition']};" ) ? $arr_block['new_content'] : '';
				$content            = str_replace( $arr_block['old_content'], $conditional_result, $content );
			ENDIF;
		ENDFOREACH;

		
		$arr_search             = [
			'%UNSUBSCRIBELINK%&amp;ALL',
			'%UNSUBSCRIBELINK%',
			'%FIRSTNAME%',
			'%LASTNAME%',
			'%FULLNAME%'
		];

		$arr_replace            = [
			$unsubscribe_page_url,
			$unsubscribe_page_url,
			$contact['first_name'],
			$contact['last_name'],
			$contact['first_name'] . ' ' . $contact['last_name']
		];

		$content                = str_ireplace( $arr_search, $arr_replace, $content );

		
		FOREACH ( $contact AS $field => $value ) :
			IF ( in_array( $field, [ 'fields', 'tags' ] ) ) :
				continue;
			ENDIF;
			$field_value          = ! empty( $value ) ? $value : '';
			$content              = str_ireplace( '%' . $field . '%', $field_value, $content );
		ENDFOREACH;

		
		FOREACH ( $contact['fields'] AS $field => $value ) :
			$field_value          = ! empty( $value ) ? $value : '';
			$field_value          = trim( $field_value, '||' );
			$field_value          = explode( '||', $field_value );
			$field_value          = implode( ', ', $field_value );
			$content              = str_ireplace( '%' . $field . '%', $field_value, $content );
		ENDFOREACH;

		
















		$content                = utf8_decode( $content );

		return $content;
	}

}
$GLOBALS['mbr_mailer']      = mbr_mailer::getInstance();
