<?php
final class mbr_membership_levels EXTENDS mbr_redux_base_class
{
	use TRAIT__getInstance;

	public $opt_name  = 'mbr_membership_levels';
	public $page_slug = 'mbr360-membership-levels';

	public function _load()
	{
		IF ( is_admin() ) :
			add_action( 'plugins_loaded',                                        [ $this, 'init_admin_panel' ], 99999 );
			add_action( 'plugins_loaded',                                        [ $this, 'init_sections' ],    99999 );
			add_filter( "redux/{$this->opt_name}/panel/template/header.tpl.php", [ $this, 'alternate_header' ] );
			add_action( "redux/options/{$this->opt_name}/validate",              [ $this, 'validate_save_settings' ] );
			add_action( "redux/options/{$this->opt_name}/saved",                 [ $this, 'backup_settings' ],  9999 );
		ENDIF;

		add_action( 'mbr/membership_levels/save',                              [ $this, 'save_membership_levels' ],                10, 1 );
	}

	public function init_admin_panel()
	{
		global $MBR_REDUX_HEADER_MODULE_NAMES;

		$MBR_REDUX_HEADER_MODULE_NAMES[ $this->opt_name ] = $admin_title = _x( 'Membership Levels', 'wp_admin', 'activemember360' );

		Redux::setExtensions( $this->opt_name, MBR_REDUX_EXTENSIONS_DIR );

		$page_title         = apply_filters( 'mbr/plugin_name', '' ) . ' ' . $admin_title;

		$arr_config         =  [
			'opt_name'        => $this->opt_name,
			'global_variable' => $this->opt_name,
			'page_slug'       => $this->page_slug,
			'page_parent'     => MBR_MENU_PARENT_PAGE,
			'page_title'      => $page_title,
			'display_name'    => $page_title,
			'admin_title'     => $admin_title,
			'menu_title'      => $admin_title,
		];

		$arr_config         = array_merge( $this->arr_default_sub_admin_panel_settings, $arr_config );

		Redux::setArgs( $this->opt_name, $arr_config );
	}

	public function init_sections()
	{
		global $MBR_OPTIONS, $pagenow;

		IF ( $pagenow <> 'options.php' AND $pagenow <> 'admin.php' OR ( $pagenow == 'admin.php' AND ( empty($_GET['page'] ) OR $_GET['page'] <> $this->page_slug ) ) ) :
			return;
		ENDIF;

		do_action( 'mbr/site_tags/fetch' );
		$arr_tags           = apply_filters( 'mbr/site_tags/get', NULL );
		FOREACH ( $arr_tags AS $tag_id => $tag_name ) :
			$arr_tags[ $tag_id ] = $tag_name . ' (' . $tag_id . ')';
		ENDFOREACH;

		$arr_pages          = [];
		$obj_pages          = apply_filters( 'mbr/pages/get', NULL );
		FOREACH ( $obj_pages AS $page ) :
			$arr_pages[ $page->ID ] = ! empty( $page->post_title ) ? $page->post_title : _x( 'No page title', 'wp_admin', 'activemember360' );
		ENDFOREACH;

		$sub_fields        = [];
		$sub_fields[]      =
		[
			'id'             => 'membership_name',
			'title'          => _x( 'Membership Name', 'wp_admin', 'activemember360' ),
			'type'           => 'text',
		];

		$sub_fields[]      =
		[
			'id'             => 'access_tag',
			'title'          => _x( 'Tag Name (ID)', 'wp_admin', 'activemember360' ),
			'type'           => 'select',
			'options'        => $arr_tags
		];

		$sub_fields[]      =
		[
			'id'             => 'deny_tag',
			'title'          => _x( 'Failed Payment Tag Name (ID)', 'wp_admin', 'activemember360' ),
			'type'           => 'select',
			'options'        => $arr_tags
		];

		$sub_fields[]      =
		[
			'id'             => 'login_redirect',
			'title'          => _x( 'Redirect on login', 'wp_admin', 'activemember360' ),
			'type'           => 'select',
			'options'        => $arr_pages
		];

		$sub_fields[]      =
		[
			'id'             => 'logout_redirect',
			'title'          => _x( 'Redirect on logout', 'wp_admin', 'activemember360' ),
			'type'           => 'select',
			'options'        => $arr_pages
		];

		IF ( $MBR_OPTIONS['extensions']['mbr_ext_themeswitcher_enabled'] ) :
			$sub_fields[]    =
			[
				'id'           => 'theme',
				'title'        => _x( 'Select theme for this membership', 'wp_admin', 'activemember360' ),
				'type'         => 'select',
				'options'      => apply_filters( 'mbr/theme_switch/add_theme_column/redux', NULL ),
				'placeholder'  => _x( 'Default Site Theme', 'wp_admin', 'activemember360' )
			];
		ENDIF;

		$field_membership_levels =
		[
			'id'             => 'mbr_membership_levels',
			'type'           => 'repeater',
			'item_name'      => _x( 'Membership Level', 'wp_admin', 'activemember360' ),
			'full_width'     => TRUE,
			'group_values'   => TRUE,
			'limit'          => 9999,
			'fields'         => $sub_fields
		];

		$field_panel_notes  =
		[
			'id'             => 'mbr_membership_levels_notes',
			'title'          => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'full_width'     => TRUE,
			'type'           => 'textarea'
		];

		$arr_section_membership_levels =
		[
			'id'             => 'section_membership_levels',
			'title'          => _x( 'Membership Levels', 'wp_admin', 'activemember360' ),
			'desc'           => _x( 'You can change the login and logout "redirect" priorities by dragging the individual memberships into the desired position, with the highest priority at the top and the least at the bottom. A user with multiple memberships will be matched against the login/logout priorities defined below and will be redirected accordingly.', 'wp_admin', 'activemember360' ),
			'icon'           => 'el el-group',
			'fields'         => [ $field_membership_levels, $field_panel_notes ]
		];

		Redux::setSection( $this->opt_name, $arr_section_membership_levels );
	}

	public function validate_save_settings( &$arr_settings )
	{
		IF ( ! empty( $arr_settings['mbr_membership_levels']['access_tag'] ) ) :
			$rebase      = FALSE;
			$arr_tags    = apply_filters( 'mbr/site_tags/get', NULL );
			FOREACH ( $arr_settings['mbr_membership_levels']['access_tag'] AS $key => $tag_id ) :
				IF ( empty( $tag_id ) ) :
					unset( $arr_settings['mbr_membership_levels']['redux_repeater_data'][ $key ] );
					unset( $arr_settings['mbr_membership_levels']['membership_name'][ $key ] );
					unset( $arr_settings['mbr_membership_levels']['access_tag'][ $key ] );
					unset( $arr_settings['mbr_membership_levels']['deny_tag'][ $key ] );
					unset( $arr_settings['mbr_membership_levels']['login_redirect'][ $key ] );
					unset( $arr_settings['mbr_membership_levels']['logout_redirect'][ $key ] );
					unset( $arr_settings['mbr_membership_levels']['theme'][ $key ] );
					$rebase = TRUE;
					continue;
				ENDIF;

				IF ( $arr_settings['mbr_membership_levels']['access_tag'][ $key ] == $arr_settings['mbr_membership_levels']['deny_tag'][ $key ] ) :
					$arr_settings['mbr_membership_levels']['deny_tag'][ $key ] = '';
				ENDIF;
				IF ( empty( $arr_settings['mbr_membership_levels']['membership_name'][ $key ] ) ) :
					$arr_settings['mbr_membership_levels']['membership_name'][ $key ] = $arr_tags[ $tag_id ] . ' (' . $tag_id . ')';
				ENDIF;
			ENDFOREACH;
		ENDIF;

		IF ( $rebase ) :
			$arr_settings['mbr_membership_levels']['redux_repeater_data'] = array_values( $arr_settings['mbr_membership_levels']['redux_repeater_data'] );
			$arr_settings['mbr_membership_levels']['membership_name']     = array_values( $arr_settings['mbr_membership_levels']['membership_name'] );
			$arr_settings['mbr_membership_levels']['access_tag']          = array_values( $arr_settings['mbr_membership_levels']['access_tag'] );
			$arr_settings['mbr_membership_levels']['deny_tag']            = array_values( $arr_settings['mbr_membership_levels']['deny_tag'] );
			$arr_settings['mbr_membership_levels']['login_redirect']      = array_values( $arr_settings['mbr_membership_levels']['login_redirect'] );
			$arr_settings['mbr_membership_levels']['logout_redirect']     = array_values( $arr_settings['mbr_membership_levels']['logout_redirect'] );
			$arr_settings['mbr_membership_levels']['theme']               = array_values( $arr_settings['mbr_membership_levels']['theme'] );
		ENDIF;
	}

	public function save_membership_levels()
	{
		global $MBR_OPTIONS;
		IF ( empty( $MBR_OPTIONS ) ) :
			return;
		ENDIF;

		$arr_settings                                                             = get_option( 'mbr_membership_levels', [] );

		IF ( ! empty( $MBR_OPTIONS['membership_levels'] ) AND is_array( $MBR_OPTIONS['membership_levels'] ) ) :
			$cnt = 0;
			FOREACH ( $MBR_OPTIONS['membership_levels'] AS $access_tag_id => $arr_membership_level ) :
				$arr_settings['mbr_membership_levels']['redux_repeater_data'][ $cnt ] = [ 'title' => '' ];
				$arr_settings['mbr_membership_levels']['membership_name'][ $cnt ]     = $arr_membership_level['name'];
				$arr_settings['mbr_membership_levels']['access_tag'][ $cnt ]          = $access_tag_id;
				$arr_settings['mbr_membership_levels']['deny_tag'][ $cnt ]            = $arr_membership_level['payf'];
				$arr_settings['mbr_membership_levels']['login_redirect'][ $cnt ]      = $arr_membership_level['redr'];
				$arr_settings['mbr_membership_levels']['logout_redirect'][ $cnt ]     = $arr_membership_level['lout'];
				$arr_settings['mbr_membership_levels']['theme'][ $cnt ]               = $arr_membership_level['theme'];
				$cnt++;
			ENDFOREACH;
		ENDIF;

		update_option( 'mbr_membership_levels', $arr_settings );
	}

	public function backup_settings( $arr_settings = [] )
	{
		global $wpdb;
		set_transient($this->opt_name . '_backup_' . time(), $arr_settings, 45 * 86400 );
		
		$obj_results = $wpdb->get_results( "SELECT * FROM " . $wpdb->prefix . "options WHERE option_name LIKE '_transient_" . $this->opt_name . "_backup_%'", OBJECT );
		FOREACH ($obj_results AS $key => $obj) :
			get_transient( substr( $obj->option_name, 11 ) );
		ENDFOREACH;
	}

}
$GLOBALS['mbr_inst_membership_levels'] = mbr_membership_levels::getInstance();