<?php
class mbr_passwords
{
	use TRAIT__getInstance;

	public $site_password_reminder_email_template = 'site_password_reminder_email_template';

	public function _load()
	{
		add_action( 'init', [ $this, '_init' ] );
	}

	public function _init()
	{

		function MBRPASS()
		{
			IF ( empty( $GLOBALS['mbr_passwords'] ) ) :
				$GLOBALS['mbr_passwords'] = mbr_passwords::getInstance();
			ENDIF;

			return $GLOBALS['mbr_passwords'];
		}

		global $MBR_OPTIONS;

		

		add_filter( 'random_password',               [ $this, 'random_password' ], 9999 );
//		add_filter( 'lostpassword_redirect',         [ $this, 'login_url' ], 1, 1 );
		add_filter( 'mbr/password/hash',             [ $this, 'hash_password' ] );
		add_action( 'mbr/password/send',             [ $this, 'send_password' ], 10, 1 );

		





 		IF ( $MBR_OPTIONS['mbr_encrypted_passwords'] ) :
			add_action( 'password_reset',              [ $this, 'password_reset' ], 10, 2 );
			add_action( 'profile_update',              [ $this, 'profile_update' ], 9999, 2 );
		ELSE:
			add_action( 'lostpassword_post',           [ $this, 'lostpassword_post_form_process' ], 99 );
			add_action( 'lostpassword_form',           [ $this, 'lostpassword' ],    99 );
			add_action( 'lost_password',               [ $this, 'lostpassword' ],    99 );
//			add_filter( 'mbr/login/message',           [ $this, 'login_message' ] );
//			add_filter( 'mbr/login/footer',            [ $this, 'login_footer' ] );
		ENDIF;

		IF ( MBR_MULTISITE ) :
			add_filter( 'lostpassword_url',            [ $this, 'lostpassword_url_multisite' ], 10, 2 );
			add_filter( 'lostpassword_redirect',       [ $this, 'lostpassword_redirect_multisite' ], 10, 2 );
			add_filter( 'network_site_url',            [ $this, 'network_site_url_multisite' ], 10, 3 );
			add_filter( 'retrieve_password_message',   [ $this, 'retrieve_password_message_multisite' ], 10, 2 );
			add_filter( 'retrieve_password_title',     [ $this, 'retrieve_password_title_multisite' ] );
		ENDIF;
	}

	public function hash_password( $password )
	{
		global $MBR_OPTIONS;

		IF ( $MBR_OPTIONS['mbr_encrypted_passwords'] ) :
			return wp_hash_password( $password );
		ENDIF;

		return $password;
	}

	public function site_password_encryption( $password )
	{
		global $MBR_OPTIONS;

		IF ( ! $MBR_OPTIONS['mbr_encrypted_passwords'] ) :
			IF ( substr( $password, 0, 4 ) == '$P$B' ) :
				do_action( 'mbr/options/load' );
				$MBR_OPTIONS['mbr_encrypted_passwords'] = 1;
				do_action( 'mbr/options/save' );
			ENDIF;
		ENDIF;
	}

	public function lostpassword_redirect_multisite( $lostpassword_url = '', $redirect = '' )
	{
		return site_url( 'wp-login.php?checkemail=confirm', 'login' );
	}

	public function lostpassword_url_multisite( $url, $redirect = '' )
	{
		IF ( defined( 'WOOCOMMERCE_VERSION' ) ) :
			IF ( basename( $url ) == get_option( 'woocommerce_myaccount_lost_password_endpoint', 'lost-password' ) ) :
				return $url;
			ENDIF;
		ENDIF;

		$args = [ 'action' => 'lostpassword' ];
		IF ( ! empty( $redirect ) ) :
			$args['redirect_to'] = $redirect;
		ENDIF;

		return add_query_arg( $args, site_url( 'wp-login.php' ) );
	}

	public function network_site_url_multisite( $url = '', $path = '', $scheme = '' )
	{
		IF ( stripos( $url, 'action=lostpassword') !== FALSE ) :
			return site_url( 'wp-login.php?action=lostpassword', $scheme );
		ENDIF;
		IF ( stripos( $url, 'action=resetpass') !== FALSE ) :
			return site_url( 'wp-login.php?action=resetpass', $scheme );
		ENDIF;

		return $url;
	}

	public function retrieve_password_message_multisite( $message = '', $key = '' )
	{
		return str_replace( get_site_url( 1 ), get_site_url(), $message );
	}

	public function retrieve_password_title_multisite( $title = '' )
	{
		return "[" . wp_specialchars_decode( get_option( 'blogname' ), ENT_QUOTES ) . "] Password Reset";
	}

	public function password_reset( $user, $new_pass = '' )
	{
		$contact = apply_filters( 'mbr/fetch_contact', ['email' => $user->data->user_email, 'local' => FALSE, 'sync' => FALSE ], FALSE );

		




		$hashed_password = wp_hash_password($new_pass);

		IF (!empty($hashed_password)) :
			apply_filters( 'mbr/save_contact_fields', ['id' => $contact['id'], apply_filters( 'mbr/password_perstag', NULL ) => $hashed_password ] );
		ENDIF;
	}

	public function profile_update( $user_id, $old_user_data )
	{
		




		$user    = get_userdata($user_id);
		$contact = apply_filters( 'mbr/fetch_contact', ['email' => $user->data->user_email, 'local' => FALSE, 'sync' => FALSE ], FALSE );
		apply_filters( 'mbr/save_contact_fields', ['id' => $contact['id'], apply_filters( 'mbr/password_perstag', NULL ) => $user->data->user_pass ] );
	}

	public function login_url()
	{
		global $MBR_OPTIONS;

		IF ( $MBR_OPTIONS['login_page'] == 0 OR $GLOBALS['pagenow'] == 'wp-login.php' ) :
			return site_url( 'wp-login.php' );
		ENDIF;

		return get_permalink( $MBR_OPTIONS['login_page'] );
	}

	public function retrieve_password( $user_login )
	{
		global $MBR_OPTIONS;

		$errors      = new WP_Error();

		$user_data   = FALSE;

		IF ( empty( $user_login ) ) :
			$errors->add( 'empty_username', __( '<strong>ERROR</strong>: Enter a username or email address.' ) );
		ELSEIF ( strpos( $user_login, '@' ) ) :
			$user_data = get_user_by( 'email', trim( wp_unslash( $user_login ) ) );
		ELSE:
			$login     = trim( $user_login );
			$user_data = get_user_by( 'login', $login );
		ENDIF;

		IF ( empty( $user_data ) AND is_email( $user_login ) ) :
			IF ( $contact = apply_filters( 'mbr/fetch_contact', ['email' => $user_login, 'local' => FALSE, 'sync' => FALSE ], FALSE ) ) :

				$arr_usertags  = array_flip( explode( ',', $contact['DBSHORT']['tag_ids'] ) );
				IF ( $MBR_OPTIONS['siteban_tag'] AND array_key_exists( $MBR_OPTIONS['siteban_tag'], $arr_usertags ) ) :
					GOTO SKIPPED_USER_CREATION;
				ENDIF;

				IF ( empty( $MBR_OPTIONS['allow_login_wo_ms'] ) AND ! array_intersect_key( $arr_usertags, $MBR_OPTIONS['membership_levels'] ) ) :
					GOTO SKIPPED_USER_CREATION;
				ENDIF;

				$arrUSER     = [ 'user_login' => $contact['email'], 'user_pass' => wp_generate_password(), 'user_email' => $contact['email'], 'first_name' => $contact['first_name'], 'last_name' => $contact['last_name'] ];
				IF ( wp_insert_user( $arrUSER ) ) :
					$user_data = get_user_by( 'email', trim( wp_unslash( $user_login ) ) );
				ENDIF;

			ENDIF;
		ENDIF;

		SKIPPED_USER_CREATION:

		IF ( empty( $user_data ) ) :
			$errors->add( 'invalidcombo', __( '<strong>ERROR</strong>: Invalid username or email.' ) );
		ENDIF;

		do_action( 'lostpassword_post', $errors );

		IF ( $errors->get_error_code() ) :
			return $errors;
		ENDIF;

		$user_login = $user_data->user_login;
		$user_email = $user_data->user_email;
		$key        = get_password_reset_key( $user_data );

		IF ( is_wp_error( $key ) ) :
			return $key;
		ENDIF;

		$message  = __( 'Someone has requested a password reset for the following account:' ) . "\r\n\r\n";
		$message .= network_home_url( '/' ) . "\r\n\r\n";
		$message .= sprintf( __( 'Username: %s' ), $user_login ) . "\r\n\r\n";
		$message .= __( 'If this was a mistake, just ignore this email and nothing will happen.' ) . "\r\n\r\n";
		$message .= __( 'To reset your password, visit the following address:' ) . "\r\n\r\n";
		$message .= '<' . network_site_url( "wp-login.php?action=rp&key=$key&login=" . rawurlencode( $user_login ), 'login' ) . ">\r\n";

		IF ( is_multisite() ) :
			$blogname = get_network()->site_name;
		ELSE:
			$blogname = wp_specialchars_decode( get_option('blogname'), ENT_QUOTES );
		ENDIF;

		$title   = sprintf( __( '[%s] Password Reset' ), $blogname );
		$title   = apply_filters( 'retrieve_password_title', $title, $user_login, $user_data );

		$message = apply_filters( 'retrieve_password_message', $message, $key, $user_login, $user_data );

		IF ( $message AND ! wp_mail( $user_email, wp_specialchars_decode( $title ), $message ) ) :
			mbr_die( __( 'The email could not be sent.' ) . "<br />\n" . __( 'Possible reason: your host may have disabled the mail() function.' ) );
		ENDIF;

		return true;
	}

	public function generate_site_password_reminder_email_template()
	{
		global $MBR_OPTIONS;

		$template['mbr_tpl_password_reminder_from_name']  = $MBR_OPTIONS['mail_fromname'];
		$template['mbr_tpl_password_reminder_from_email'] = $MBR_OPTIONS['mail_fromaddr'];

		$template['mbr_tpl_password_reminder_subject']    = sprintf( __('[%s] Your username and password info' ), get_option( 'blogname' ) );
		$template['mbr_tpl_password_reminder_message']    = sprintf( __('Login at: %s', 'activemember360' ), '%login_url%' )     . "\n\r\n\r";
		$template['mbr_tpl_password_reminder_message']   .= sprintf( __('Username: %s', 'activemember360' ), '%user_login%' )    . "\n\r";
		$template['mbr_tpl_password_reminder_message']   .= sprintf( __('Password: %s', 'activemember360' ), '%user_password%' );

		return $template;
	}

	public function send_password( $parms = [] )
	{
		global $MBR_OPTIONS, $wp_version;

		$parms           = apply_filters( 'mbr/shortcodes/atts', [ 'user_login' => '', 'fromaddr' => $MBR_OPTIONS['mail_fromaddr'], 'fromname' => $MBR_OPTIONS['mail_fromname'], 'automation_id' => 0 ], $parms );

		$user_login      = trim( $parms['user_login'] );
		$automation_id   = (int) $parms['automation_id'];

		IF  ($user_login == '' ) :
			return [ 0, __( 'Please enter your login name (email).', 'activemember360' ) ];
		ENDIF;

		$user_login = stripcslashes( $user_login );

		IF ( $MBR_OPTIONS['mbr_encrypted_passwords'] ) :
			$result = $this->retrieve_password( $user_login );
			IF ( is_wp_error($result ) ) :
				return [ 0, $result->get_error_message() ];
			ELSE:
				return [ 1, __('An email with your password reset link has been sent to you.', 'activemember360' ) ];
			ENDIF;
		ENDIF;

		$contact = apply_filters( 'mbr/fetch_contact', ['email' => $user_login, 'local' => FALSE, 'sync' => FALSE ], FALSE );

		IF ( ! empty( $contact ) ) :
			$arr_usertags  = array_flip( array_map( 'trim', explode( ',', $contact['tag_ids'] ) ) );

			IF ( $MBR_OPTIONS['siteban_tag'] AND array_key_exists( $MBR_OPTIONS['siteban_tag'], $arr_usertags ) ) :
				return [ 0, __( 'Sorry, you do not have login privileges on this site.', 'activemember360' ) ];
			ENDIF;

			IF ( empty( $MBR_OPTIONS['allow_login_wo_ms'] ) AND ! array_intersect_key( $arr_usertags, $MBR_OPTIONS['membership_levels'] ) ) :
				return [ 0, __( 'Sorry, you do not have login privileges on this site.', 'activemember360' ) ];
			ENDIF;

			$password_field = apply_filters( 'mbr/passfield', NULL );
			$remotePASS     = $contact[ $password_field ];
			$this->site_password_encryption( $remotePASS );

			IF ( empty( $remotePASS ) OR stripos( strtolower( $remotePASS ), '#$mbr$capture$#') !== FALSE ) :
				$password = $this->generate_password();
				$savePASS = apply_filters( 'mbr/password/hash', $password );
				$contact[ $password_field ] = $savePASS;
				apply_filters( 'mbr/save_contact_fields', ['id' => $contact['id'], apply_filters( 'mbr/password_perstag', NULL ) => $savePASS ] );
			ENDIF;

			IF ( has_filter( 'mbr_password_run_automation' ) ) :
				$automation_id = apply_filters( 'mbr_password_run_automation', $automation_id );
			ENDIF;

			IF ( $automation_id ) :
				$response = apply_filters( 'mbr/api', 'automation_contact_add', [ 'contact_email' => $user_login, 'automation' => $automation_id ] );
				IF ( $response->result_code == '1' ) :
					return [ 1, __( 'Your password was sent to your email address. Please check your e-mail.', 'activemember360' ) ];
				ELSE :
					return [ 0, __( 'An error occurred while mailing your password to you. Please try again.', 'activemember360' ) ];
				ENDIF;
			ENDIF;

			$thisPASS = trim( $contact[ $password_field ] );
			IF ( $thisPASS <> '' ) :
				$arr_templates                  = $MBR_OPTIONS['templates'];
				$arr_password_reminder_template = [];
				IF ( ! empty( $arr_templates['mbr_tpl_password_reminder_from_name'] ) ) :
					$arr_password_reminder_template['mbr_tpl_password_reminder_from_name']  = $arr_templates['mbr_tpl_password_reminder_from_name'];
				ENDIF;
				IF ( !empty( $arr_templates['mbr_tpl_password_reminder_from_email'] ) ) :
					$arr_password_reminder_template['mbr_tpl_password_reminder_from_email'] = $arr_templates['mbr_tpl_password_reminder_from_email'];
				ENDIF;
				IF ( ! empty( $arr_templates['mbr_tpl_password_reminder_subject'] ) ) :
					$arr_password_reminder_template['mbr_tpl_password_reminder_subject']    = $arr_templates['mbr_tpl_password_reminder_subject'];
				ENDIF;
				IF ( ! empty( $arr_templates['mbr_tpl_password_reminder_message'] ) ) :
					$arr_password_reminder_template['mbr_tpl_password_reminder_message']    = $arr_templates['mbr_tpl_password_reminder_message'];
				ENDIF;

				IF ( empty( $arr_password_reminder_template ) ) :
					$arr_password_reminder_template = $this->generate_site_password_reminder_email_template();
				ENDIF;

				IF ( empty( $arr_password_reminder_template['mbr_tpl_password_reminder_from_name'] ) ) :
					$arr_password_reminder_template['mbr_tpl_password_reminder_from_name']  = $MBR_OPTIONS['mail_fromname'];
					$template_changed = TRUE;
				ENDIF;

				IF ( empty( $arr_password_reminder_template['mbr_tpl_password_reminder_from_email'] ) ) :
					$arr_password_reminder_template['mbr_tpl_password_reminder_from_email'] = $MBR_OPTIONS['mail_fromaddr'];
					$template_changed = TRUE;
				ENDIF;

				IF ( empty( $arr_password_reminder_template['mbr_tpl_password_reminder_subject'] ) ) :
					$arr_password_reminder_template['mbr_tpl_password_reminder_subject']    = sprintf( __( '[%s] Your username and password info' ), get_option( 'blogname' ) );
					$template_changed = TRUE;
				ENDIF;

				IF ( empty( $arr_password_reminder_template['mbr_tpl_password_reminder_message'] ) ) :
					$arr_password_reminder_template['mbr_tpl_password_reminder_message']  = sprintf( __( 'Login at: %s', 'activemember360' ), '%login_url%' )  . "\n\r\n\r";
					$arr_password_reminder_template['mbr_tpl_password_reminder_message'] .= sprintf( __( 'Username: %s', 'activemember360' ), '%user_login%' ) . "\n\r";
					$arr_password_reminder_template['mbr_tpl_password_reminder_message'] .= sprintf( __( 'Password: %s', 'activemember360' ), '%user_password%' );
					$template_changed = TRUE;
				ENDIF;

				IF ( $template_changed ) :
					$arr_templates = array_merge( $arr_templates, $arr_password_reminder_template);
					update_option( 'mbr_templates', $arr_templates );
				ENDIF;

				// BEGIN: This section is used to manipulate certain phpmailer variables
				global $phpmailer;
				// (Re)create it, if it's gone missing
				IF ( version_compare( $wp_version, '5.5', '<' ) ) :
					IF ( ! ( $phpmailer instanceof PHPMailer ) ) :
						require_once ABSPATH . WPINC . '/class-phpmailer.php';
						require_once ABSPATH . WPINC . '/class-smtp.php';
						$phpmailer = new PHPMailer( TRUE );
					ENDIF;
				ELSE:
					IF ( ! ( $phpmailer instanceof \PHPMailer\PHPMailer ) ) :
						require_once ABSPATH . WPINC . '/PHPMailer/PHPMailer.php';
						require_once ABSPATH . WPINC . '/PHPMailer/SMTP.php';
						$phpmailer = new \PHPMailer\PHPMailer\PHPMailer( TRUE );
					ENDIF;
				ENDIF;
	
				$phpmailer->XMailer    = 'ActiveMember360 (https://activemember360.com)';
				$phpmailer->Priority   = 1;
				// END: This section is used to manipulate certain phpmailer variables

				IF ( ! empty( $GLOBALS['mbr_generated_password'] ) ) :
					$password_for_email  = $GLOBALS['mbr_generated_password'];
				ELSEIF ( substr( $thisPASS,0,4 ) <> '$P$B' ) :
					$password_for_email  = $thisPASS;
				ELSE:
					$password_for_email  = __( 'Use your existing password to login.', 'activemember360' ) . '<br />';
					$password_for_email .= sprintf( __( 'If you can\'t remember your password, you can reset it here: %s', 'activemember360' ), wp_lostpassword_url() );
				ENDIF;

				$login_url = '<a href="' . wp_login_url() . '">' . wp_login_url() . '</a>';
				$message   = str_ireplace( [ '%user_login%', '%user_password%', '%login_url%' ], [ $user_login, $password_for_email, $login_url ], nl2br( stripcslashes( $arr_password_reminder_template['mbr_tpl_password_reminder_message'] ) ) );
				$headers   = [
					"MIME-Version: 1.0"
					,"Content-Type: text/html; charset='" . get_option('blog_charset') . "'"
					,"From: {$arr_password_reminder_template['mbr_tpl_password_reminder_from_name']} <{$arr_password_reminder_template['mbr_tpl_password_reminder_from_email']}>"
				];

				IF ( @wp_mail( $user_login, $arr_password_reminder_template['mbr_tpl_password_reminder_subject'], $message, $headers ) ) :
					return [ 1, __( 'Your password was sent to your email address. Please check your e-mail.', 'activemember360' ) ];
				ELSE :
					return [ 0, __( 'An error occurred while mailing your password to you. Please try again.', 'activemember360' ) ];
				ENDIF;

			ELSE :
				$reg_page = apply_filters( 'mbr/sign_up/link', __( 'Please click here to register', 'activemember360' ) );
				IF ( trim( $reg_page) <> '' ) :
					return [ 0, "<p>" . __( 'This login name is not registered.', 'activemember360') . "</p><p>$reg_page</p>" ];
				ELSE :
					return [ 0, "<p>" . __( 'This login name is not registered.', 'activemember360') . "</p><p>" . __('Please register.', 'activemember360') . "</p>" ];
				ENDIF;
			ENDIF;
		ELSE :
			return [ 0, __( 'This login name is not registered.', 'activemember360' ) ];
		ENDIF;
	}

	public function lostpassword()
	{
		IF ( stripos( $_SERVER['QUERY_STRING'], 'action=lostpassword') === FALSE AND stripos( $_SERVER['QUERY_STRING'], 'checkemail=confirm') === FALSE) :
			do_action( 'mbr/redirect', wp_login_url() );
		ENDIF;
	}

	public function lostpassword_post_form_process( $user_login )
	{
		$url = apply_filters( 'mbr/url', NULL );

		
		IF ( ! defined( 'WOOCOMMERCE_VERSION' ) OR basename( $url ) <> get_option( 'woocommerce_myaccount_lost_password_endpoint', 'lost-password' ) ) :
			IF ( stripos( $_SERVER['QUERY_STRING'], 'action=lostpassword') === FALSE AND stripos( $_SERVER['QUERY_STRING'], 'checkemail=confirm') === FALSE ) :
				do_action( 'mbr/redirect', wp_login_url() );
			ENDIF;
		ENDIF;

		IF ( empty( $user_login ) OR ! is_email( $user_login ) ) :
			IF ( ! empty( $_POST['send_user_login'] ) ) :
				$user_login = $_POST['send_user_login'];
			ELSE :
				$user_login = $_POST['user_login'];
			ENDIF;
		ENDIF;

		IF ( ! empty( $user_login ) AND is_email( $user_login ) ) :

			$parms = [ 'user_login' => $user_login, 'fromaddr' => '', 'fromname' => '', 'use_template_id' => 0 ];
			$result = $this->send_password( $parms );

			do_action( 'mbr/clear_ob' );

			IF ( $result[0] ) :
				IF ( defined( 'WOOCOMMERCE_VERSION' ) AND basename( $url ) == get_option( 'woocommerce_myaccount_lost_password_endpoint', 'lost-password' ) ) :
					do_action( 'mbr/redirect', add_query_arg( 'reset-link-sent', 'true', $url ) );
				ELSE:
					do_action( 'mbr/redirect', add_query_arg( 'checkemail', 'sentpass', wp_login_url() ) );
				ENDIF;
			ENDIF;
		ENDIF;
	}

	public function generate_password( $p = FALSE )
	{
		global $MBR_OPTIONS;

		$password      = '';

		$password_base = apply_filters( 'mbr/password/char_base', NULL );

		IF ( empty( $password_base ) ) :
			$strlen_p    = strlen( $p );
			IF ( $strlen_p >= $MBR_OPTIONS['min_passlen'] AND $strlen_p <= $MBR_OPTIONS['max_passlen'] ) :
				$password  = $p;
			ENDIF;
		ENDIF;

		IF ( empty( $password ) ) :
			$strlen_base = strlen( $password_base ) - 1;
			$pass_length = mt_rand( $MBR_OPTIONS['min_passlen'], $MBR_OPTIONS['max_passlen'] );
			FOR ( $i = 0; $i < $pass_length; $i++ ) :
				$password .= substr( $password_base, mt_rand( 0, $strlen_base ), 1 );
			ENDFOR;
		ENDIF;
		$GLOBALS['mbr_generated_password'] = $password;

		return $password;
	}

	public function random_password( $p = NULL )
	{
		IF ( strlen( $p ) > 12 ) :
			return $p;
		ENDIF;

		return $this->generate_password( $p );
	}

}
$GLOBALS['mbr_passwords'] = mbr_passwords::getInstance();
