<?php
class mbr_payment {
	use
		TRAIT__getInstance
	 ,TRAIT__credit_card_validation
	 ;

	const MBR_ROUTE_PREFIX                         = 'mbr-payment';

	public $stripe_plans                           = NULL;

	public $site_stripe_customer_id_field          = 'STRIPE_CUSTOMER_ID';
	public $stripe_payment_fields                  = [ 'STRIPE_CUSTOMER_ID', 'billing_email', 'billing_first_name', 'billing_last_name', 'billing_company', 'billing_address_1', 'billing_address_2', 'billing_city', 'billing_state', 'billing_postcode', 'billing_country', 'billing_phone' ];

	// post_meta keys, pm_key => post_meta key
	public $stripe_charges_pm_key                  = '_mbr_stripe_charges';
	public $stripe_charge_id_pm_key                = '_mbr_stripe_charge_id';
	public $stripe_subscription_id_pm_key          = '_mbr_stripe_subscription_id';
	public $stripe_customer_id_pm_key              = '_mbr_stripe_customer_id';
	public $stripe_refunds_pm_key                  = '_mbr_stripe_refunds';

	public $order_post_status_pm_key               = '_mbr_order_post_status';
	public $order_customer_email_pm_key            = '_mbr_order_customer_email';
	public $order_payment_method_pm_key            = '_mbr_order_payment_method';
	public $order_activecampaign_contact_id_pm_key = '_mbr_order_activecampaign_contact_id';
	public $order_billing_details_pm_key           = '_mbr_order_billing_details';
	public $order_items_pm_key                     = '_mbr_order_items';
	public $order_amount_pm_key                    = '_mbr_order_amount';
	public $order_currency_pm_key                  = '_mbr_order_currency';
	public $order_country_tax_pm_key               = '_mbr_order_country_tax';
	public $order_state_tax_pm_key                 = '_mbr_order_state_tax';
	public $order_invoice_id_pm_key                = '_mbr_order_invoice_id';

	public function _load()
	{

		function MBRPAY()
		{
			IF ( empty( $GLOBALS['mbrpayment'] ) ) :
				$GLOBALS['mbrpayment'] = mbrpayment::getInstance();
			ENDIF;

			return $GLOBALS['mbrpayment'];
		}

		add_filter( 'mbr/cust_fields/required',               [ $this, 'required_fields_payment' ], 10, 1 );
		add_filter( 'mbr/insert_post/exclude_post_types',     [ $this, 'exclude_post_types' ],      11, 1 );
		add_action( 'mbr/route/prefix',                       [ $this, 'register_route_prefix' ], 10, 1 );
		add_filter( 'mbr/routes/register/nonadmin',           [ $this, 'register_routes_payment' ],  10, 1 );
		add_filter( 'mbr/endpoints/rewrite',                  [ $this, 'rewrite_endpoints_payment' ], 10, 1 );
		add_filter( 'mbr/payment/invoice/get_id',             [ $this, 'get_next_invoice_id' ], 0 );
		add_filter( 'mbr/payment/invoice/prefix',             [ $this, 'get_invoice_prefix' ], 0 );
		add_filter( 'mbr/shortcodes/extend',                  [ $this, 'extend_payments_shortcodes' ], 0, 1 );

		add_action( 'init',                                   [ $this, '_init' ], 20 );

		add_action( 'template_redirect',                      [ $this, 'template_redirect' ] );

		
		add_action( 'wp_ajax_mbr_payment_checkout',           [ $this, 'checkout_calculation' ] );

		
		add_action( 'wp_ajax_nopriv_mbr_payment_checkout',    [ $this, 'checkout_calculation' ] );

		add_filter( 'single_template',                        [ $this, 'checkout_template' ] );
		add_action( 'admin_menu',                             [ $this, 'remove_redux_menu' ], 12 );
		add_action( 'init',                                   [ $this, 'register_post_types' ], 21 );
		add_action( 'init',                                   [ $this, 'order_custom_post_status' ], 21 );

		add_action( 'wp_enqueue_scripts',                     [ $this, 'stripe_process_payment' ], 999 );

		//add_action('save_post_' . MBRCUSTSUBSCRIPTIONS,    [ $this, 'get_stripe_plans' ] );// only used when using transients

		// to extend the search range for order list search
		add_filter( 'posts_join',                             [ $this, 'order_list_posts_join' ] );
		add_filter( 'posts_where',                            [ $this, 'order_list_posts_where' ] );
		add_filter( 'posts_distinct',                         [ $this, 'order_list_posts_distinct' ] );

		// add new order list columns and make them sortable
		add_action( 'admin_head-edit.php',                    [ $this, 'order_change_title_in_list' ] );
		add_filter( 'manage_' .      MBRCUSTORDERS . '_posts_columns',       [ $this, 'order_add_columns' ] );
		add_action( 'manage_' .      MBRCUSTORDERS . '_posts_custom_column', [ $this, 'order_render_columns' ], 10, 2 );
		add_filter( 'manage_edit-' . MBRCUSTORDERS . '_sortable_columns',    [ $this, 'order_sortable_columns' ] );
		add_filter( 'request',                                [ $this, 'order_sortable_columns_orderby' ] );

		// add metaboxes to orders
		add_action( 'add_meta_boxes_' . MBRCUSTORDERS,        [ $this, 'order_add_metaboxes' ] );
		add_action( 'admin_menu',                             [ $this, 'order_remove_metaboxes' ] );
		add_filter( 'post_row_actions',                       [ $this, 'order_remove_quick_edit' ], 99999 );

		// do_action of this hook already ran before we do the add_action
		//add_action('mbr/init/hooks',      [$this, 'init_hooks']);
		//$this->init_hooks();

		$this->payment = $this->get_payment_settings();
		$this->setup_constants();

		// just for dynamic purposes
		__( 'month', 'activemember360' );
		__( 'year', 'activemember360' );
	}

	public function _init()
	{
		// load the stripe libraries
		IF ( ! class_exists( 'Stripe' ) ) :
			require_once MBR_REDUX_STRIPE_INIT;
		ENDIF;

		// must be after requiring the Stripe class
		//$this->get_stripe_plans();// only used when using transients
		



		$this->order_payment_statuses = [
			'mbr_payment_pending'   => [ 'label_overview' => _x( 'Pending',                 'wp_admin', 'activemember360' ), 'label_regular' => _x( 'Pending',                 'wp_admin', 'activemember360' ) ],
			'mbr_payment_failed'    => [ 'label_overview' => _x( 'Failed',                  'wp_admin', 'activemember360' ), 'label_regular' => _x( 'Failed',                  'wp_admin', 'activemember360' ) ],
			'mbr_payment_ref_full'  => [ 'label_overview' => _x( 'Refunded',                'wp_admin', 'activemember360' ), 'label_regular' => _x( 'Refunded',                'wp_admin', 'activemember360' ) ],
			'mbr_payment_ref_part'  => [ 'label_overview' => _x( 'Part. refunded',          'wp_admin', 'activemember360' ), 'label_regular' => _x( 'Part. refunded',          'wp_admin', 'activemember360' ) ],
			'mbr_payment_success'   => [ 'label_overview' => _x( 'Paid',                    'wp_admin', 'activemember360' ), 'label_regular' => _x( 'Paid',                    'wp_admin', 'activemember360' ) ],
			'mbr_payment_sub_actv'  => [ 'label_overview' => _x( 'Active',                  'wp_admin', 'activemember360' ), 'label_regular' => _x( 'Active',                  'wp_admin', 'activemember360' ) ],
			'mbr_payment_sub_paus'  => [ 'label_overview' => _x( 'Paused',                  'wp_admin', 'activemember360' ), 'label_regular' => _x( 'Paused',                  'wp_admin', 'activemember360' ) ],
			'mbr_payment_sub_canc'  => [ 'label_overview' => _x( 'Cancelled',               'wp_admin', 'activemember360' ), 'label_regular' => _x( 'Cancelled',               'wp_admin', 'activemember360' ) ],
			'mbr_payment_sub_cape'  => [ 'label_overview' => _x( 'Cancelled at period end', 'wp_admin', 'activemember360' ), 'label_regular' => _x( 'Cancelled at period end', 'wp_admin', 'activemember360' ) ],
			'mbr_payment_sub_tril'  => [ 'label_overview' => _x( 'Trialing',                'wp_admin', 'activemember360' ), 'label_regular' => _x( 'Trialing',                'wp_admin', 'activemember360' ) ]
		];
	}

	public function extend_payments_shortcodes( $SHORT = [] )
	{
		$SHORT['stripe_one_click_upsell']             = [ [$this, 'sc_stripe_one_click_upsell' ], 0 ];
		$SHORT['product_name']                        = [ [$this, 'sc_product_details' ],         0 ];
		$SHORT['product_regular_price']               = [ [$this, 'sc_product_details' ],         0 ];
		$SHORT['product_sale_price']                  = [ [$this, 'sc_product_details' ],         0 ];
		$SHORT['product_full_desc']                   = [ [$this, 'sc_product_details' ],         0 ];
		$SHORT['product_short_desc']                  = [ [$this, 'sc_product_details' ],         0 ];
		$SHORT['product_image']                       = [ [$this, 'sc_product_details' ],         0 ];
		$SHORT['product_regular_plan_interval']       = [ [$this, 'sc_product_details' ],         0 ];
		$SHORT['product_sale_plan_interval']          = [ [$this, 'sc_product_details' ],         0 ];
		$SHORT['product_regular_plan_interval_count'] = [ [$this, 'sc_product_details' ],         0 ];
		$SHORT['product_sale_plan_interval_count']    = [ [$this, 'sc_product_details' ],         0 ];
		$SHORT['product_regular_plan_trial_period']   = [ [$this, 'sc_product_details' ],         0 ];
		$SHORT['product_sale_plan_trial_period']      = [ [$this, 'sc_product_details' ],         0 ];
		$SHORT['product_regular_plan_name']           = [ [$this, 'sc_product_details' ],         0 ];
		$SHORT['product_sale_plan_name']              = [ [$this, 'sc_product_details' ],         0 ];
		$SHORT['my_payments']                         = [ [$this, 'sc_my_payments' ],             0 ];

		return $SHORT;
	}

	public function exclude_post_types( $exclude = [] )
	{
		return array_merge( $exclude, [ MBRCUSTPRODUCTS, MBRCUSTSUBSCRIPTIONS, MBRCUSTORDERS ] );
	}

	public function get_invoice_prefix( $prefix = NULL )
	{
		IF ( ! empty( $this->payment['invoice_number_prefix'] ) ) :
			$prefix = $this->payment['invoice_number_prefix'];
		ENDIF;

		return $prefix;
	}

	public function get_next_invoice_id( $next_invoice_id = NULL )
	{
		$last_invoice_id = get_option( 'mbr_payment_invoice_id', 0 );
		$next_invoice_id = $last_invoice_id + 1;
		update_option( 'mbr_payment_invoice_id', $next_invoice_id );

		return $next_invoice_id;
	}

	public function get_invoice_id( $order_id, $charge_id )
	{
		$arr_invoice_ids = $this->get_invoices( $order_id );

		IF ( ! empty( $arr_invoice_ids[ $charge_id ] ) ) :
			$invoice_id    = $arr_invoice_ids[ $charge_id ];
		ELSE:
			$invoice_id    = __( 'Unknown', 'activemember360' );
		ENDIF;

		return $invoice_id;
	}

	public function get_invoices( $order_id = FALSE )
	{
		IF ( empty( $order_id ) ) :
			return [];
		ENDIF;

		$arr_invoice_ids      = [];
		$tmp_arr_invoice_ids  = get_post_meta( $order_id, $this->order_invoice_id_pm_key, FALSE );
		FOREACH ( $tmp_arr_invoice_ids AS $key => $arr_this_charge ) :
			FOREACH ( $arr_this_charge AS $ch_id => $arr_inv ) :
				$arr_invoice_ids[ $ch_id ] = $arr_inv['invoice_id'];
			ENDFOREACH;
		ENDFOREACH;

		return $arr_invoice_ids;
	}

	public function remove_redux_menu()
	{
		remove_submenu_page( 'tools.php', 'redux-about' );
	}

	public function register_route_prefix( $route_prefixes = [] )
	{
		$route_prefixes[] = self::MBR_ROUTE_PREFIX;

		return array_merge( $route_prefixes, [ self::MBR_ROUTE_PREFIX ] );
	}

	public function checkout_template( $single_template )
	{
		global $post, $wp_query;

		IF ( $post->post_type == MBRCUSTPRODUCTS ) :
			$settings_key            = MBR_PRODUCT_SETTINGS;
		ELSEIF ( $post->post_type == MBRCUSTSUBSCRIPTIONS) :
			$settings_key            = MBR_SUBSCRIPTION_SETTINGS;
		ELSE:
			return $single_template;
		ENDIF;

		$pmeta                     = redux_post_meta( $settings_key, $post->ID );

		$checkout_tpl_id_org       = $pmeta['_mbr_oitem_checkout_template'];
		$checkout_tpl_id_alt       = ! empty( $wp_query->query_vars['page'] ) ? $wp_query->query_vars['page'] : '';

		IF ( ! empty( $checkout_tpl_id_alt ) ) :
			$checkout_tpl_alt        = get_post( $checkout_tpl_id_alt );
			IF ( ! empty( $checkout_tpl_alt->ID ) AND $checkout_tpl_alt->post_type == MBRCUSTCHECKOUTTPL ) :
				$cmeta_alt             = redux_post_meta( MBR_CHECKOUTTPL_SETTINGS, $checkout_tpl_id_alt );
				$checkout_tpl_alt_path = dirname( __FILE__ ) . '/templates/' . $cmeta_alt['_mbr_checkout_template'] . '.php';
				IF ( file_exists( $checkout_tpl_alt_path ) ) :
					return $checkout_tpl_alt_path;
				ENDIF;
			ENDIF;
		ENDIF;

		$checkout_tpl_org          = get_post( $checkout_tpl_id_org );
		IF ( ! empty( $checkout_tpl_org->ID ) AND $checkout_tpl_org->post_type == MBRCUSTCHECKOUTTPL ) :
			$cmeta_org               = redux_post_meta( MBR_CHECKOUTTPL_SETTINGS, $checkout_tpl_id_org );
			$checkout_tpl_org_path   = dirname( __FILE__ ) . '/templates/' . $cmeta_org['_mbr_checkout_template'] . '.php';
			IF ( file_exists( $checkout_tpl_org_path ) ) :
				return $checkout_tpl_org_path;
			ENDIF;
		ENDIF;

		return $single_template;
	}

	public function checkout_calculation()
	{
		IF ( empty( $_POST['mbr_checkout_product_id'] ) ) :
			echo __( 'No product ID provided', 'activemember360' );
			exit;
		ENDIF;

		$product = get_post( $_POST['mbr_checkout_product_id'] );
		$country = $_POST['mbr_checkout_country'];
		$state   = $_POST['mbr_checkout_state'];

		IF ( empty( $product ) OR $product->post_type <> MBRCUSTPRODUCTS AND $product->post_type <> MBRCUSTSUBSCRIPTIONS ) :
			echo __( 'Product does not exist.', 'activemember360' );
			exit;
		ENDIF;

		IF ( $product->post_type == MBRCUSTSUBSCRIPTIONS ) :
			$this->get_stripe_plans();
			$settings_key = MBR_SUBSCRIPTION_SETTINGS;
		ELSEIF ( $product->post_type == MBRCUSTPRODUCTS ) :
			$settings_key = MBR_PRODUCT_SETTINGS;
		ELSE:
			echo __( 'No product ID provided', 'activemember360' );
			exit;
		ENDIF;

		



		$product_meta = redux_post_meta( $settings_key, $product->ID );

		IF ( empty( $product_meta['_mbr_oitem_status'] ) ) :
			echo __( 'Product is inactive', 'activemember360' );
			exit;
		ENDIF;

		$is_subscription     = FALSE;

		IF ( $product_meta['_mbr_oitem_status'] == 1 ) :
			IF ( $product->post_type == MBRCUSTSUBSCRIPTIONS ) :
				$is_subscription = TRUE;
				$sale_plan       = $product_meta['_mbr_oitem_live_sale_plan'];
				$regular_plan    = $product_meta['_mbr_oitem_live_regular_plan'];
				$sale_price      = $this->stripe_plans['live'][ $sale_plan ]['amount'];
				$regular_price   = $this->stripe_plans['live'][ $regular_plan ]['amount'];
				$trial_days      = $this->stripe_plans['live'][ $regular_plan ]['trial_period_days'];
				$interval        = $this->stripe_plans['live'][ $regular_plan ]['interval'];
				$interval_count  = $this->stripe_plans['live'][ $regular_plan ]['interval_count'];
			ELSE:
				$sale_price      = $product_meta['_mbr_oitem_live_sale_price'];
				$regular_price   = $product_meta['_mbr_oitem_live_regular_price'];
			ENDIF;
		ELSEIF ( $product_meta['_mbr_oitem_status'] == 2 ) :
			IF ( $product->post_type == MBRCUSTSUBSCRIPTIONS ) :
				$is_subscription = TRUE;
				$sale_plan       = $product_meta['_mbr_oitem_test_sale_plan'];
				$regular_plan    = $product_meta['_mbr_oitem_test_regular_plan'];
				$sale_price      = $this->stripe_plans['test'][ $sale_plan ]['amount'];
				$regular_price   = $this->stripe_plans['test'][ $regular_plan ]['amount'];
				$trial_days      = $this->stripe_plans['test'][ $regular_plan ]['trial_period_days'];
				$interval        = $this->stripe_plans['test'][ $regular_plan ]['interval'];
				$interval_count  = $this->stripe_plans['test'][ $regular_plan ]['interval_count'];
			ELSE:
				$sale_price      = $product_meta['_mbr_oitem_test_sale_price'];
				$regular_price   = $product_meta['_mbr_oitem_test_regular_price'];
			ENDIF;
		ENDIF;

		$amount   = $regular_price;
		IF ( ! empty( $sale_price ) ) :
			$amount = $sale_price;
		ENDIF;

		$net_price   = $net_price_org = $amount;

		IF ( $product->post_type == MBRCUSTPRODUCTS ) :
			$net_price = $net_price_org = apply_filters( 'mbr/payments/stripe/convert_amount', $net_price );
			$amount    = apply_filters( 'mbr/payments/stripe/convert_amount', $amount );
		ENDIF;

		$price       = $this->calculate_tax( $amount, $country, $state );
		$price       = $this->stripe_price_format( MBR_PAYMENT_CURRENCY, $price );
		$net_price   = $this->stripe_price_format( MBR_PAYMENT_CURRENCY, $net_price );

		$country_tax        = FALSE;
		$state_tax          = FALSE;
		$country_tax_amount = 0;
		$state_tax_amount   = 0;

		IF ( ! empty( $country ) AND $country == $this->payment['company_country'] ) :
			IF ( ! empty( $this->payment['country_tax_percentage'] ) ) :
				$country_tax        = $this->payment['country_tax_percentage'];
				$country_tax_amount = ( $net_price_org / 100 ) * $this->payment['country_tax_percentage'];
			ENDIF;
			IF ( ! empty( $this->payment['state_tax_percentage'] ) ) :
				IF ( $state == $this->payment['company_state'] ) :
					$state_tax        = $this->payment['state_tax_percentage'];
					$state_tax_amount = ( $net_price_org / 100 ) * $this->payment['state_tax_percentage'];
				ENDIF;
			ENDIF;
		ENDIF;

		$sales_tax_percentage = $country_tax + $state_tax;
		$sales_tax_amount     = $country_tax_amount + $state_tax_amount;

		echo '<div class="row">';

		echo '<section class="col col-12"><table class="col-12">';
		echo '<tr><td class="col-70">' . __( 'Net Price:', 'activemember360' ) . '</td><td>' . $net_price . '</td></tr>';
		echo '<tr><td>' . sprintf( __( 'Sales Tax %s&#37;:', 'activemember360' ), $sales_tax_percentage ) . '</td><td>'   . $this->stripe_price_format( MBR_PAYMENT_CURRENCY, $sales_tax_amount ) . '</td></tr>';

		IF ( $is_subscription ) :
			$trial   = FALSE;
			IF ( ! empty( $trial_days ) ) :
				$trial = $trial_days;
			ENDIF;
			echo '<tr><td>' . __( 'Recurring Totals:', 'activemember360' ) . '</td><td>' . $price . '</td></tr>';
			echo '<tr><td colspan="2" class="recurring-pad-top">' . __( 'Billing Cycle:', 'activemember360' ) . '</td></tr>';
			echo '<tr><td colspan="2" class="recurring-pad-bottom">'   . $this->text_subscription_interval( $interval_count, $interval, $trial ) . '</td></tr>';
		ENDIF;

		echo '<tr><td>' . __( 'Total Price:', 'activemember360' ) . '</td><td>' . $price . '</td></tr>';
		echo '</table></section>';
		echo '</div>';
		exit;
	}

	public function subscription_cycles( $interval = FALSE, $interval_count = FALSE )
	{
		IF ( $interval_count == 1 ) :
			IF ( $interval == 'year' ) :
				return __( 'Yearly',  'activemember360' );
			ELSEIF ( $interval == 'month' ) :
				return __( 'Monthly', 'activemember360' );
			ELSEIF ( $interval == 'week' ) :
				return __( 'Weekly',  'activemember360' );
			ELSEIF ( $interval == 'day' ) :
				return __( 'Daily',   'activemember360' );
			ENDIF;
		ELSE :
			IF ( $interval == 'year' ) :
				return sprintf( __( 'Every %s years',  'activemember360' ), $interval_count );
			ELSEIF ( $interval == 'month' ) :
				return sprintf( __( 'Every %s months', 'activemember360' ), $interval_count );
			ELSEIF ( $interval == 'week' ) :
				return sprintf( __( 'Every %s weeks',  'activemember360' ), $interval_count );
			ELSEIF ( $interval == 'day' ) :
				return sprintf( __( 'Every %s days',   'activemember360' ), $interval_count );
			ENDIF;
		ENDIF;

		return __( 'Unknown', 'activemember360' );
	}

	public function template_redirect()
	{
		global $wp_query, $wp_rewrite;
		// if this is not a request for json or a singular object then bail

		$remote_contact_id = apply_filters( 'mbr/contact_id', NULL );

		IF ( ! MBRADMIN AND ! $remote_contact_id ) :
			return;
		ENDIF;

		$payment_endpoints  = $this->rewrite_endpoints_payment();

		$endpoint_confirmed = FALSE;
		FOREACH ( $wp_rewrite->endpoints AS $key => $arr_endpoint ) :
			IF ( array_key_exists( $arr_endpoint[1], $payment_endpoints ) ) :
				$endpoint_confirmed = TRUE;
				break;
			ENDIF;
		ENDFOREACH;

		IF ( ! $endpoint_confirmed ) :
			return;
		ENDIF;

		
		IF ( ! empty( $wp_query->query_vars[ MBR_INVOICE ] ) ) :
			$tmp_invoice_data = explode('/', $wp_query->query_vars[ MBR_INVOICE ] );
			$order_id         = $tmp_invoice_data[0];

			IF ( empty( $order_id ) OR empty( $_POST['mbr-charge-id'] ) ) :
				return;
			ENDIF;

			$charge_id        = $_POST['mbr-charge-id'];

			IF ( ! MBRADMIN ) :
				$customer_id    = get_post_meta( $order_id, $this->order_activecampaign_contact_id_pm_key, TRUE );
				IF ($customer_id <> $remote_contact_id ) :
					return;
				ENDIF;
			ENDIF;
			$arr_invoice = $this->print_invoice($order_id, $charge_id);
			echo $arr_invoice['html'];
			exit;
		ENDIF;

		
		IF ( ! empty( $wp_query->query_vars[ MBR_REFUND ] ) ) :
			$tmp_refund_data  = explode( '/', $wp_query->query_vars[ MBR_REFUND ] );
			$order_id         = $tmp_refund_data[0];
			$charge_id        = $_POST['mbr-charge-id'];

			IF ( empty( $order_id ) OR empty( $charge_id ) ) :
				return;
			ENDIF;

			IF ( ! MBRADMIN ) :
				$customer_id    = get_post_meta( $order_id, $this->order_activecampaign_contact_id_pm_key, TRUE );
				IF ( $customer_id <> $remote_contact_id ) :
					return;
				ENDIF;
			ENDIF;
			$arr_refund = $this->print_refund( $order_id, $charge_id );
			echo $arr_refund['html'];
			exit;
		ENDIF;

		return;
	}

	public function text_subscription_interval( $interval_count, $interval, $trial_days = FALSE )
	{
		$text_interval = $this->subscription_cycles( $interval, $interval_count );
		IF ( ! empty( $trial_days ) ) :
			$trial_text = sprintf( __( ', first charge after %s day(s)', 'activemember360' ), $trial_days );
			return "{$text_interval}{$trial_text}";
		ELSE:
			return "{$text_interval}";
		ENDIF;
	}

	public function print_invoice( $order_id, $charge_id )
	{
		$invoice_id = $this->get_invoice_id( $order_id, $charge_id );
		return $this->populate_invoice_refund_template( $order_id, $charge_id, $invoice_id, $this->payment['invoice_template'], FALSE );
	}

	function print_refund($order_id, $charge_id)
	{
		$invoice_id = $this->get_invoice_id($order_id, $charge_id);
		return $this->populate_invoice_refund_template($order_id, $charge_id, $invoice_id, $this->payment['refund_template'], TRUE);
	}

	public function populate_invoice_refund_template( $order_id, $charge_id, $invoice_id, $html, $include_refund = FALSE )
	{
		$arr_charges  = get_post_meta( $order_id, $this->stripe_charges_pm_key, TRUE );
		IF ( ! empty( $arr_charges ) AND is_array( $arr_charges ) ) :
			IF ( ! empty( $arr_charges[ $charge_id ] ) ) :
				$refund_history            = '';
				$total_amount_refunded     = 0;
				IF ( $include_refund ) :
					$arr_refunds             = get_post_meta( $order_id, $this->stripe_refunds_pm_key, TRUE );
					IF ( empty( $arr_refunds ) OR empty( $arr_refunds[ $charge_id ] ) ) :
						return;
					ENDIF;
					$refund_history          = '<table>';
					FOREACH ( $arr_refunds[ $charge_id ] AS $key => $arr_refund ) :
						$total_amount_refunded = $total_amount_refunded + $arr_refund['amount'];
						$refund_history       .= '<tr><td><span>' . apply_filters( 'mbr/datetime/convert', $arr_refund['created'] ) . '</span></td><td><span>' . $this->stripe_price_format( $arr_refund['currency'], $arr_refund['amount'] ) . '</span></td></tr>';
					ENDFOREACH;
					$refund_history         .= '</table>';
					$subject              = str_replace( '%%invoice_id%%', $invoice_id, $this->payment['refund_email_subject'] );
				ELSE:
					$subject              = str_replace( '%%invoice_id%%', $invoice_id, $this->payment['invoice_email_subject'] );
				ENDIF;

				$country_tax            = get_post_meta( $order_id, $this->order_country_tax_pm_key, TRUE );
				$state_tax              = get_post_meta( $order_id, $this->order_state_tax_pm_key,   TRUE );
				$total_tax              = $country_tax + $state_tax;

				$net_price_wo_cur       = $arr_charges[ $charge_id ]['amount'] - ( ( $arr_charges[ $charge_id ]['amount'] / ( 100 + $total_tax ) ) * ( $total_tax ) );
				$net_price              = $this->stripe_price_format( $arr_charges[ $charge_id ]['currency'], $net_price_wo_cur );

				$order_items            = get_post_meta( $order_id, $this->order_items_pm_key, TRUE );
				$item_id                = $order_items[0]['id'];
				$item_type              = $order_items[0]['type'];

				$item                   = get_post( $item_id );
				$item_name              = $item->post_title;

				IF ( $item_type == 'subscription' ) :
					$item_name           .= sprintf( __( ' (Billing Cycle: %s)', 'activemember360' ), $this->text_subscription_interval( $order_items[0]['interval_count'], $order_items[0]['interval'] ) );
				ENDIF;

				$billing_details        = get_post_meta( $order_id, $this->order_billing_details_pm_key, TRUE );

				$countries              = apply_filters( 'mbr/payments/countries/get', NULL );
				$us_states              = apply_filters( 'mbr/payments/states/us/get', NULL );

				$billing_address        = [];
				IF ( ! empty( $billing_details['billing_company'] ) ) :
					$billing_address[]    = $billing_details['billing_company'];
				ENDIF;
				IF ( ! empty( $billing_details['first_name'] ) OR ! empty( $billing_details['last_name'] ) ) :
					$billing_address[]    = ( ! empty( $billing_details['first_name'] ) ? $billing_details['first_name'] : '') . ' ' . ( ! empty( $billing_details['last_name'] ) ? $billing_details['last_name'] : '' ) ;
				ENDIF;
				IF ( ! empty( $billing_details['billing_address_1'] ) ) :
					$billing_address[]    = $billing_details['billing_address_1'];
				ENDIF;
				IF ( ! empty( $billing_details['billing_address_2'] ) ) :
					$billing_address[]    = $billing_details['billing_address_2'];
				ENDIF;
				IF ( ! empty( $billing_details['billing_postcode']) OR ! empty( $billing_details['billing_city'] ) ) :
					$billing_address[]    = ( ! empty( $billing_details['billing_postcode'] ) ? $billing_details['billing_postcode'] : '' ) . ' ' . ( ! empty( $billing_details['billing_city'] ) ? $billing_details['billing_city'] : '' ) ;
				ENDIF;
				IF ( ! empty( $billing_details['billing_country'] ) ) :
					IF ( ! empty( $billing_details['billing_state'] ) AND $billing_details['billing_country'] == 'US' ) :
						$billing_address[]    = $us_states[ $billing_details['billing_state'] ];
					ENDIF;
					$billing_address[]    = $countries[ $billing_details['billing_country'] ];
				ENDIF;
				$billing_address        = implode( '<br />', $billing_address );

				$company_address        = [];
				IF ( ! empty( $this->payment['company_name'] ) ) :
					$company_address[]    = $this->payment['company_name'];
				ENDIF;
				IF ( ! empty( $this->payment['company_street_address_1'] ) ) :
					$company_address[]    = $this->payment['company_street_address_1'];
				ENDIF;
				IF ( ! empty( $this->payment['company_street_address_2'] ) ) :
					$company_address[]    = $this->payment['company_street_address_2'];
				ENDIF;
				IF ( ! empty( $this->payment['company_postal_code']) OR ! empty( $this->payment['company_city'] ) ) :
					$company_address[]    = ( ! empty( $this->payment['company_postal_code'] ) ? $this->payment['company_postal_code'] : '') . ' ' . ( ! empty( $this->payment['company_city'] ) ? $this->payment['company_city'] : '' ) ;
				ENDIF;
				IF ( ! empty( $this->payment['company_country'] ) ) :
					IF ( ! empty( $this->payment['company_state'] ) ) :
						$company_address[]  = $us_states[ $this->payment['company_state'] ];
					ENDIF;
					$company_address[]    = $countries[ $this->payment['company_country'] ];
				ENDIF;
				$company_address        = implode( '<br />', $company_address );

				$arr_replace            = [
					'%%company_logo%%'    => $this->payment['company_logo']['url'],
					'%%company_vat_id%%'  => $this->payment['company_vat_id'],
					'%%order_id%%'        => $order_id,
					'%%invoice_id%%'      => $invoice_id,
					'%%order_date%%'      => apply_filters( 'mbr/datetime/convert', $arr_charges[ $charge_id ]['created'] ),
					'%%due_date%%'        => apply_filters( 'mbr/datetime/convert', $arr_charges[ $charge_id ]['created'] ),
					'%%billing_address%%' => $billing_address,
					'%%company_address%%' => $company_address,
					'%%item_name%%'       => $item_name,
					'%%item_price%%'      => $net_price,
					'%%order_total%%'     => $this->stripe_price_format( $arr_charges[ $charge_id ]['currency'], $arr_charges[ $charge_id ]['amount'] ),
					'%%sales_tax%%'       => '(' . ( $country_tax + $state_tax ) . '%) ' . $this->stripe_price_format( $arr_charges[$charge_id]['currency'], ( ( $net_price_wo_cur / 100 ) * ( $country_tax + $state_tax ) ) ),
					'%%net_total%%'       => $net_price,
					'%%order_refunds%%'   => $refund_history,
					'%%refund_total%%'    => $this->stripe_price_format( $arr_charges[ $charge_id ]['currency'], $total_amount_refunded )
				];
				$html                   = str_replace( array_keys( $arr_replace ), array_values( $arr_replace ), $html );
				return [ 'subject' => $subject, 'html' => $html ];
			ENDIF;
		ENDIF;

		return FALSE;
	}

	public function rewrite_endpoints_payment( $rewrite_endpoints = [] )
	{
		$rewrite_endpoints[ MBR_INVOICE ] = EP_ROOT;
		$rewrite_endpoints[ MBR_REFUND ]  = EP_ROOT;

		return $rewrite_endpoints;
	}

	public function register_routes_payment( $route_args = [] )
	{
		
		
		
		
		
		//$route_args['password']['operations']['send']   = [$this, 'password_send'];

		$route_args['stripe']['methods'] = ['POST'];
		$route_args['stripe']['operations']['v1']  = [ $this, 'stripe_receive_webhook' ];

		return $route_args;
	}

	public function setup_constants()
	{
		IF ( empty( $this->payment['currency'] ) ) :
			$currency = 'usd';
		ELSEIF ( $this->payment['currency'] == 'custom_currency' ) :
			$currency = $this->payment['custom_currency'];
		ELSE:
			$currency = $this->payment['currency'];
		ENDIF;

		mbr_constant( 'MBR_PAYMENT_CURRENCY', strtolower( $currency ) );

		mbr_constant( 'STRIPE_BASE_URL',         plugin_dir_url( __FILE__ ) );
		mbr_constant( 'MBR_INVOICE',             'mbr-invoice');
		mbr_constant( 'MBR_REFUND',              'mbr-refund');
		mbr_constant( 'MBR_STRIPE_CUST_ID_FIELD', lower_trim( $this->site_stripe_customer_id_field ) );
	}

	public function get_payment_settings()
	{
		IF ( empty( $GLOBALS['mbr_payment_settings'] ) ) :
			$payment_settings = get_option( 'mbr_payment_settings', FALSE );
		ELSE :
			$payment_settings = $GLOBALS['mbr_payment_settings'];
		ENDIF;

		IF ( ! is_array( $payment_settings ) OR empty( $payment_settings ) ) :
			$payment_settings = FALSE;
		ELSE :
			IF ( empty( $GLOBALS['mbr_payment_settings'] ) ) :
				$GLOBALS['mbr_payment_settings'] = $payment_settings;
			ENDIF;
		ENDIF;

		return $payment_settings;
	}

	public function required_fields_payment( $cust_fields = [] )
	{
		FOREACH ( $this->stripe_payment_fields AS $key => $field_name ) :
			$cust_fields[] = $field_name;
		ENDFOREACH;

		return $cust_fields;
	}

	public function get_stripe_plans() {
		IF ( ! is_null( $this->stripe_plans ) ) :
			return;
		ENDIF;

		$test_secret_key    = trim( $this->payment['stripe_test_secret'] );
		$live_secret_key    = trim( $this->payment['stripe_live_secret'] );

		$this->stripe_plans = [];
		IF ( $live_secret_key ) :
			$this->stripe_set_api_key( $live_secret_key );
			$plans = $this->stripe_get_all_plans();
			FOREACH ( $plans['data'] AS $plan ) :
				$this->stripe_plans['live'][ $plan['id'] ] = [ 'name' => $plan['name'], 'amount' => $plan['amount'], 'interval' => $plan['interval'], 'interval_count' => $plan['interval_count'], 'trial_period_days' => $plan['trial_period_days'] ];
			ENDFOREACH;
		ENDIF;

		IF ( $test_secret_key ) :
			$this->stripe_set_api_key( $test_secret_key );
			$plans = $this->stripe_get_all_plans();
			FOREACH ( $plans['data'] AS $plan ) :
				$this->stripe_plans['test'][ $plan['id'] ] = [ 'name' => $plan['name'], 'amount' => $plan['amount'], 'interval' => $plan['interval'], 'interval_count' => $plan['interval_count'], 'trial_period_days' => $plan['trial_period_days'] ];
			ENDFOREACH;
		ENDIF;
	}

	public function stripe_webhook_verify_order_status( $local_order_id, $type = 'unknown' )
	{
		IF ( get_post_status($local_order_id) == 'trash' ) :
			mbr_log( $type . ' => Error order #' . $local_order_id . ' : Order is locally trashed.' );
			http_response_code( 200 );
			exit;
		ENDIF;
	}

	public function stripe_webhook_verify_order_id( $local_order_id = FALSE, $type = 'unknown' )
	{
		IF ( empty( $local_order_id ) ) :
			http_response_code( 200 );
			exit;
		ENDIF;

		$local_post = get_post( $local_order_id );
		IF ( empty( $local_post->ID ) ) :
			mbr_log( $type . ' => Error order #' . $local_order_id . ' : Order id does not exists.' );
			http_response_code( 200 );
			exit;
		ENDIF;
	}

	public function stripe_webhook_verify_customer( $local_customer_id = FALSE, $stripe_customer_id = FALSE, $local_order_id = 'unknown', $type = 'unknown' )
	{
		IF ( ( empty( $local_customer_id ) AND empty( $stripe_customer_id ) ) OR $local_customer_id <> $stripe_customer_id ) :
			mbr_log( $type . ' => Error order #' . $local_order_id . ' : Customer id could not be verified.' );
			http_response_code( 200 );
			exit;
		ENDIF;
	}

	public function stripe_webhook_verify_charge_id( $local_charge_id = FALSE, $stripe_charge_id = FALSE, $local_order_id = 'unknown', $type = 'unknown' )
	{
		IF ( ( empty( $local_charge_id ) AND empty( $stripe_charge_id ) ) OR $local_charge_id <> $stripe_charge_id ) :
			mbr_log( $type . ' => Error order #' . $local_order_id . ' : Charge id could not be verified.' );
			http_response_code( 200 );
			exit;
		ENDIF;
	}

	public function stripe_webhook_check_for_charge_or_subscription_id( $local_charge_id = FALSE, $local_subscription_id = FALSE, $local_order_id = 'unknown', $type = 'unknown' )
	{
		IF ( empty( $local_charge_id ) AND empty( $local_subscription_id ) ) :
			mbr_log( $type . ' => Error order #' . $local_order_id . ' : No local charge or subscription id found.' );
			http_response_code( 200 );
			exit;
		ENDIF;
	}

	public function stripe_webhook_check_remote_subscription_id( $stripe_subscription_id = FALSE, $local_order_id = 'unknown', $type = 'unknown' )
	{
		IF ( empty( $stripe_subscription_id ) ) :
			mbr_log( $type . ' => Error order #' . $local_order_id . ' : Stripe subscription id is missing.' );
			http_response_code( 200 );
			exit;
		ENDIF;
	}

	public function stripe_webhook_check_local_subscription_id( $local_subscription_id = FALSE, $local_order_id = 'unknown', $type = 'unknown' )
	{
		IF ( empty( $local_subscription_id ) ) :
			mbr_log( $type . ' => Error order #' . $local_order_id . ' : Local subscription id is missing.' );
			http_response_code( 200 );
			exit;
		ENDIF;
	}

	public function stripe_wehook_verify_subscription_id( $local_subscription_id = FALSE, $stripe_subscription_id = FALSE, $local_order_id = 'unknown', $type = 'unknown' )
	{
		IF ( ( empty( $local_subscription_id ) AND empty( $stripe_subscription_id ) ) OR $local_subscription_id <> $stripe_subscription_id ) :
			mbr_log( $type . ' => Error order #' . $local_order_id . ' : Subscription could not be verified.' );
			http_response_code( 200 );
			exit;
		ENDIF;
	}

	public function stripe_webhook_verify_shop_url( $stripe_home_url = FALSE, $local_order_id = 'unknown', $type = 'unknown' )
	{
		$home_url = home_url();
		$home_url = explode( '://', $home_url );
		$home_url = $home_url[1];

		$stripe_home_url = explode( '://', $stripe_home_url );
		$stripe_home_url = $stripe_home_url[1];

		IF ( $home_url <> $stripe_home_url ) :
			mbr_log( $type . ' => Error order #' . $local_order_id . ' : Shop URL could not be verified.' );
			http_response_code( 200 );
			exit;
		ENDIF;
	}

	public function stripe_webhook_get_local_order_details( $local_order_id )
	{
		$local_charge_id           = get_post_meta( $local_order_id, $this->stripe_charge_id_pm_key,       TRUE );
		$local_subscription_id     = get_post_meta( $local_order_id, $this->stripe_subscription_id_pm_key, TRUE );
		$local_customer_id         = get_post_meta( $local_order_id, $this->stripe_customer_id_pm_key,     TRUE );
		$local_customer_email      = get_post_meta( $local_order_id, $this->order_customer_email_pm_key,   TRUE );
		$charges                   = get_post_meta( $local_order_id, $this->stripe_charges_pm_key,         TRUE );
		IF ( empty( $charges ) OR ! is_array( $charges ) ) :
			$charges = [];
		ENDIF;

		return [ 'local_charge_id' => $local_charge_id, 'local_subscription_id' => $local_subscription_id, 'local_customer_id' => $local_customer_id, 'local_customer_email' => $local_customer_email, 'charges' => $charges ];
	}

	public function stripe_webhook_trigger_actions( $local_order_id = FALSE, $type = FALSE )
	{
		IF ( empty( $local_order_id ) OR empty( $type ) ) :
			return;
		ENDIF;

		$activecampaign_contact_id = get_post_meta( $local_order_id, $this->order_activecampaign_contact_id_pm_key, TRUE );
		$arr_sync                  = [];

		IF ( $activecampaign_contact_id ) :
			$arr_local_order_items   = get_post_meta( $local_order_id, $this->order_items_pm_key, TRUE );

			$contact_data = apply_filters( 'mbr/fetch_remote_contact/by_id', $activecampaign_contact_id );
			IF ( ! empty( $contact_data['email'] ) ) :
				$arr_sync['contact_data']['email'] = $contact_data['email'];
			ENDIF;

			FOREACH ( $arr_local_order_items AS $key => $arr_order_item ) :
				IF ( ! empty( $arr_order_item['id'] ) AND get_post( $arr_order_item['id'] ) ) :
					IF ( $arr_order_item['type'] == 'product' ) :
						$pmeta   = redux_post_meta( MBR_PRODUCT_SETTINGS, $arr_order_item['id'] );
					ELSEIF ( $arr_order_item['type'] == 'subscription' ) :
						$pmeta   = redux_post_meta( MBR_SUBSCRIPTION_SETTINGS, $arr_order_item['id'] );
					ELSE:
						break;
					ENDIF;

					IF ( ! empty( $pmeta['_mbr_oitem_' . $type . '_add_automations'] ) ) :
						$arr_sync['assign_automations'] = ( empty( $arr_sync['assign_automations'] ) ? $pmeta['_mbr_oitem_' . $type . '_add_automations']    : array_merge( $arr_sync['assign_automations'], $pmeta['_mbr_oitem_' . $type . '_add_automations'] ) );
					ENDIF;
					IF ( ! empty($pmeta['_mbr_oitem_' . $type . '_remove_automations'] ) ) :
						$arr_sync['remove_automations'] = ( empty( $arr_sync['remove_automations'] ) ? $pmeta['_mbr_oitem_' . $type . '_remove_automations'] : array_merge( $arr_sync['remove_automations'], $pmeta['_mbr_oitem_' . $type . '_remove_automations'] ) );
					ENDIF;
					IF ( ! empty( $pmeta['_mbr_oitem_' . $type . '_assign_tags'] ) ) :
						$arr_sync['assign_tags']        = ( empty( $arr_sync['assign_tags'] )        ? $pmeta['_mbr_oitem_' . $type . '_assign_tags']        : array_merge( $arr_sync['assign_tags'],        $pmeta['_mbr_oitem_' . $type . '_assign_tags'] ) );
					ENDIF;
					IF ( ! empty( $pmeta['_mbr_oitem_' . $type . '_remove_tags'] ) ) :
						$arr_sync['remove_tags']        = ( empty( $arr_sync['remove_tags'] )        ? $pmeta['_mbr_oitem_' . $type . '_remove_tags']        : array_merge( $arr_sync['remove_tags'],        $pmeta['_mbr_oitem_' . $type . '_remove_tags'] ) );
					ENDIF;
					IF ( ! empty( $pmeta['_mbr_oitem_' . $type . '_subscribe_lists'] ) ) :
						$arr_sync['subscribe_lists']    = ( empty( $arr_sync['subscribe_lists'] )    ? $pmeta['_mbr_oitem_' . $type . '_subscribe_lists']    : array_merge( $arr_sync['subscribe_lists'],    $pmeta['_mbr_oitem_' . $type . '_subscribe_lists'] ) );
					ENDIF;
					IF ( ! empty( $pmeta['_mbr_oitem_' . $type . '_unsubscribe_lists'] ) ) :
						$arr_sync['unsubscribe_lists']  = ( empty( $arr_sync['unsubscribe_lists'] )  ? $pmeta['_mbr_oitem_' . $type . '_unsubscribe_lists']  : array_merge( $arr_sync['unsubscribe_lists'],  $pmeta['_mbr_oitem_' . $type . '_unsubscribe_lists'] ) );
					ENDIF;
				ENDIF;
			ENDFOREACH;
		ENDIF;

		IF ( ! empty( $arr_sync ) AND ! empty( $arr_sync['contact_data']['email'] ) AND count( $arr_sync ) > 1 ) :
			apply_filters( 'mbr/api/sync_ctla', $arr_sync );
		ENDIF;
	}

	public function stripe_receive_webhook( $data )
	{
		IF (  empty( $data['seccode'] ) OR ! apply_filters( 'mbr/secodes/validate', urldecode( $data['seccode'] ) ) ) :
			header( $_SERVER['SERVER_PROTOCOL'] . ' 412 Incorrect Security Code' );
			exit;
		ENDIF;

		
		IF ( stripos( $_SERVER['HTTP_USER_AGENT'], 'Stripe/' ) === FALSE ) :
			exit;
		ENDIF;

		IF ( stripos( $_SERVER['REQUEST_URI'], 'mbr-payment' ) === FALSE ) :
			header( $_SERVER['SERVER_PROTOCOL'] . ' 412 Incorrect Stripe webhook parameters' );
			exit;
		ENDIF;

		$payload      = @file_get_contents( 'php://input' );
		$event_json   = json_decode( $payload );

		IF ( ! $event_json->livemode ):
			$secret_key      = trim( $this->payment['stripe_test_secret'] );
			$endpoint_secret = trim( $this->payment['stripe_signing_secret_test'] );
		ELSE:
			$endpoint_secret = trim( $this->payment['stripe_signing_secret'] );
			$secret_key      = trim( $this->payment['stripe_live_secret'] );
		ENDIF;

		
		$this->stripe_set_api_key( $secret_key );

		$sig_header        = $_SERVER['HTTP_STRIPE_SIGNATURE'];

		$event = NULL;

		TRY {
			$event = \Stripe\Webhook::constructEvent( $payload, $sig_header, $endpoint_secret );
		} CATCH ( \UnexpectedValueException $e ) {
			// Invalid payload
			http_response_code(400); // PHP 5.4 or greater
			exit;
		} CATCH ( \Stripe\Error\SignatureVerification $e ) {
			// Invalid signature
			http_response_code( 400 ); // PHP 5.4 or greater
			exit;
		}
		$event = $event->__toArray( TRUE );

		IF ( is_array( $event ) AND $event['id'] == $event_json->id) :

			
			$stripe_customer_id           = $event['data']['object']['customer'];

			IF ( $event['type'] == 'customer.subscription.deleted' ) :
				$local_order_id             = $event['data']['object']['metadata']['local_order_id'];
				$stripe_home_url            = $event['data']['object']['metadata']['shop'];
				$stripe_subscription_id     = $event['data']['object']['id'];

				
				$this->stripe_webhook_verify_order_status( $local_order_id, 'Subscription cancelled' );

				
				$this->stripe_webhook_verify_order_id( $local_order_id, 'Subscription cancelled' );

				$arr_local_order_details    = $this->stripe_webhook_get_local_order_details( $local_order_id );
				$local_customer_id          = $arr_local_order_details['local_customer_id'];
				$local_subscription_id      = $arr_local_order_details['local_subscription_id'];

				
				$this->stripe_webhook_verify_customer( $local_customer_id, $stripe_customer_id, $local_order_id, 'Subscription cancelled' );

				
				$this->stripe_webhook_check_remote_subscription_id( $stripe_subscription_id, $local_order_id, 'Subscription cancelled' );
				$this->stripe_webhook_check_local_subscription_id( $local_subscription_id, $local_order_id, 'Subscription cancelled' );
				$this->stripe_wehook_verify_subscription_id( $local_subscription_id, $stripe_subscription_id, $local_order_id, 'Subscription cancelled' );

				
				$this->stripe_webhook_verify_shop_url( $stripe_home_url, $local_order_id, 'Subscription cancelled' );

				wp_update_post( [ 'ID' => $local_order_id, 'post_status' => 'mbr_payment_sub_canc' ] );
				update_post_meta( $local_order_id, $this->order_post_status_pm_key, $this->order_payment_statuses['mbr_payment_sub_canc']['label_regular'] );
				$this->stripe_webhook_trigger_actions( $local_order_id, 'cancel' );
			ELSEIF ( $event['type'] == 'charge.refunded' ) :
				
				
				
				$stripe_charge_id           = $event['data']['object']['id'];
				
				$stripe_invoice_id          = ( ! empty( $event['data']['object']['invoice']) ? $event['data']['object']['invoice'] : FALSE );

				IF ( empty( $stripe_invoice_id ) ) :
					
					$local_order_id           = $event['data']['object']['metadata']['local_order_id'];
					$stripe_home_url          = $event['data']['object']['metadata']['shop'];
				ELSE:
					
					TRY {
						$tmp_invoice            = \Stripe\Invoice::retrieve( $stripe_invoice_id );
						$tmp2_invoice           = $tmp_invoice->__toArray( TRUE );
						$invoice                = $tmp2_invoice['lines']['data'];

						$stripe_home_url        = $invoice[0]['metadata']['shop'];
						$local_order_id         = $invoice[0]['metadata']['local_order_id'];
						$stripe_subscription_id = $invoice[0]['id'];
					} CATCH ( Exception $e ) {
						$error                  = new stdClass();
						$error->message         = $e->getMessage();
						$error->code            = $e->getCode();
						mbr_log( 'Refunded' . ' => Error invoice #' . $stripe_invoice_id . ' : Could not retrieve invoice id ' . $stripe_invoice_id . '.' );
						http_response_code( 200 );
						exit;
					}
				ENDIF;

				
				$this->stripe_webhook_verify_order_status( $local_order_id, 'Refund' );

				
				$this->stripe_webhook_verify_order_id( $local_order_id, 'Refund' );

				
				$this->stripe_webhook_verify_shop_url( $stripe_home_url, $local_order_id, 'Refund' );

				$arr_local_order_details    = $this->stripe_webhook_get_local_order_details( $local_order_id );
				$local_charge_id            = $arr_local_order_details['local_charge_id'];
				$local_customer_id          = $arr_local_order_details['local_customer_id'];
				$local_customer_email       = $arr_local_order_details['local_customer_email'];
				$local_subscription_id      = $arr_local_order_details['local_subscription_id'];

				
				$this->stripe_webhook_verify_customer( $local_customer_id, $stripe_customer_id, $local_order_id, 'Refund' );

				$all_refunds                = $event['data']['object']['refunds']['data'];
				$fully_refunded             = $event['data']['object']['refunded'];

				
				$this->stripe_webhook_check_for_charge_or_subscription_id( $local_charge_id, $local_subscription_id, $local_order_id, 'Refund' );

				
				IF ( ! empty( $local_subscription_id ) ) :
					$this->stripe_wehook_verify_subscription_id( $local_subscription_id, $stripe_subscription_id, $local_order_id, 'Refund' );
				ENDIF;

				
				IF ( empty( $stripe_invoice_id ) ) :
					IF ( $fully_refunded ) :
						$order_status = 'mbr_payment_ref_full';
					ELSE:
						$order_status = 'mbr_payment_ref_part';
					ENDIF;

					wp_update_post( [ 'ID' => $local_order_id, 'post_status' => $order_status ] );
					update_post_meta( $local_order_id, $this->order_post_status_pm_key, $this->order_payment_statuses[$order_status]['label_regular'] );
				ENDIF;
				$refund_history                    = get_post_meta( $local_order_id, $this->stripe_refunds_pm_key, TRUE );
				IF ( empty( $refund_history ) OR ! is_array( $refund_history ) ) :
					$refund_history = [];
				ENDIF;

				$refund_history[ $stripe_charge_id ] = $all_refunds;
				update_post_meta( $local_order_id, $this->stripe_refunds_pm_key, $refund_history );

				IF ( $fully_refunded ) :
					$this->stripe_webhook_trigger_actions( $local_order_id, 'refund' );
				ENDIF;
				$this->email_refund( $local_customer_email, $local_order_id, $stripe_charge_id );
			ELSEIF ( $event['type'] == 'invoice.payment_succeeded' ) :

				$stripe_home_url            = $event['data']['object']['lines']['data'][0]['metadata']['shop'];
				$local_order_id             = $event['data']['object']['lines']['data'][0]['metadata']['local_order_id'];
				$stripe_subscription_id     = $event['data']['object']['subscription'];
				$stripe_charge_id           = $event['data']['object']['charge'];

				
				$this->stripe_webhook_verify_order_status( $local_order_id, 'Payment succeeded' );

				
				$this->stripe_webhook_verify_order_id( $local_order_id, 'Payment succeeded' );

				
				$this->stripe_webhook_verify_shop_url( $stripe_home_url, $local_order_id, 'Payment succeeded' );

				$arr_local_order_details    = $this->stripe_webhook_get_local_order_details( $local_order_id );

				$local_charge_id            = $arr_local_order_details['local_charge_id'];
				$local_customer_id          = $arr_local_order_details['local_customer_id'];
				$local_customer_email       = $arr_local_order_details['local_customer_email'];
				$local_subscription_id      = $arr_local_order_details['local_subscription_id'];
				$charges                    = $arr_local_order_details['charges'];

				
				$this->stripe_webhook_verify_customer( $local_customer_id, $stripe_customer_id, $local_order_id, 'Payment succeeded' );

				
				$this->stripe_webhook_check_remote_subscription_id( $stripe_subscription_id, $local_order_id, 'Payment succeeded' );
				$this->stripe_webhook_check_local_subscription_id( $local_subscription_id, $local_order_id, 'Payment succeeded' );
				$this->stripe_wehook_verify_subscription_id( $local_subscription_id, $stripe_subscription_id, $local_order_id, 'Payment succeeded' );

				$current_post_status = get_post_status( $local_order_id );

				$order_status   = 'mbr_payment_sub_actv';

				$run_triggers   = TRUE;
				$all_paid_up    = TRUE;

				
				IF ( ! empty( $stripe_charge_id ) ) :
					TRY {
						$charge                             = \Stripe\Charge::retrieve( $stripe_charge_id );
						$arr_result                         = $charge->__toArray( TRUE );
						$charge_created                     = $arr_result['created'];
						$local_charge_id                    = $stripe_charge_id;
						update_post_meta( $local_order_id, $this->stripe_charge_id_pm_key, $stripe_charge_id );
						$charges[ $stripe_charge_id ]       = $arr_result;

						$next_invoice_id                    = apply_filters( 'mbr/payment/invoice/get_id', NULL );
						$prefix                             = apply_filters( 'mbr/payment/invoice/prefix', NULL );
						$invoice_id                         = $prefix . $next_invoice_id;

						$charge->metadata = [ 'local_invoice_id' => $invoice_id, 'local_order_id' => $local_order_id ];
						$charge->save();

						$this->add_charge_to_order( $local_order_id, $stripe_charge_id, $invoice_id, $charge_created );

						update_post_meta( $local_order_id, $this->stripe_charges_pm_key, $charges );

						







						IF ( $all_paid_up ) :
							wp_update_post( [ 'ID' => $local_order_id, 'post_status' => $order_status ] );
							update_post_meta( $local_order_id, $this->order_post_status_pm_key, $this->order_payment_statuses[$order_status]['label_regular'] );
						ENDIF;
					} CATCH ( Exception $e ) {
						mbr_log( 'Payment succeeded' . ' => Error order #' . $local_order_id . ' : Could not retrieve charge id ' . $stripe_charge_id . '.' );
					}
				ENDIF;

				
				IF ( $local_charge_id == 'initial' ) :
					$run_triggers = FALSE;
				ENDIF;

				IF ( $local_charge_id <> 'initial' AND $run_triggers AND $current_post_status == 'mbr_payment_sub_tril' ) :
					$this->stripe_webhook_trigger_actions( $local_order_id, 'purchase' );
				ELSEIF ( $run_triggers AND $current_post_status == 'mbr_payment_sub_paus' ) :
					$this->stripe_webhook_trigger_actions( $local_order_id, 'reactivation' );
				ENDIF;
				$this->email_invoice( $local_customer_email, $local_order_id, $stripe_charge_id );
			ELSEIF ( $event['type'] == 'invoice.payment_failed' ) :
				$stripe_home_url            = $event['data']['object']['lines']['data'][0]['metadata']['shop'];
				$local_order_id             = $event['data']['object']['lines']['data'][0]['metadata']['local_order_id'];
				$stripe_subscription_id     = $event['data']['object']['subscription'];
				$stripe_charge_id           = $event['data']['object']['charge'];

				
				$this->stripe_webhook_verify_order_status( $local_order_id, 'Payment failed' );

				
				$this->stripe_webhook_verify_order_id( $local_order_id, 'Payment failed' );

				
				$this->stripe_webhook_verify_shop_url( $stripe_home_url, $local_order_id, 'Payment failed' );

				$arr_local_order_details    = $this->stripe_webhook_get_local_order_details( $local_order_id );

				$local_customer_id          = $arr_local_order_details['local_customer_id'];
				$local_customer_email       = $arr_local_order_details['local_customer_email'];
				$local_subscription_id      = $arr_local_order_details['local_subscription_id'];
				$charges                    = $arr_local_order_details['charges'];

				
				$this->stripe_webhook_verify_customer( $local_customer_id, $stripe_customer_id, $local_order_id, 'Payment failed' );

				
				$this->stripe_webhook_check_remote_subscription_id( $stripe_subscription_id, $local_order_id, 'Payment failed' );
				$this->stripe_webhook_check_local_subscription_id( $local_subscription_id, $local_order_id, 'Payment failed' );
				$this->stripe_wehook_verify_subscription_id( $local_subscription_id, $stripe_subscription_id, $local_order_id, 'Payment failed' );

				
				IF ( ! empty( $stripe_charge_id ) ) :
					TRY {
						$charge               = \Stripe\Charge::retrieve( $stripe_charge_id );
						$arr_result           = $charge->__toArray( TRUE );

						update_post_meta( $local_order_id, $this->stripe_charge_id_pm_key, $stripe_charge_id );
						$charges[ $stripe_charge_id ] = $arr_result;

						update_post_meta( $local_order_id, $this->stripe_charges_pm_key, $charges );
					} CATCH ( Exception $e ) {
						mbr_log( 'Payment failed' . ' => Error order #' . $local_order_id . ' : Could not retrieve charge id ' . $stripe_charge_id . '.' );
					}
				ENDIF;

				
				wp_update_post( [ 'ID' => $local_order_id, 'post_status' => 'mbr_payment_sub_paus' ] );
				update_post_meta( $local_order_id, $this->order_post_status_pm_key, $this->order_payment_statuses['mbr_payment_sub_paus']['label_regular'] );

				
				$this->stripe_webhook_trigger_actions( $local_order_id, 'failed' );
				$this->email_invoice( $local_customer_email, $local_order_id, $stripe_charge_id );
			ENDIF;

		ENDIF;

		http_response_code( 200 );
		exit;
	}

	public function add_charge_to_order( $order_id, $charge_id, $invoice_id, $created )
	{
		add_post_meta( $order_id, $this->order_invoice_id_pm_key, [ $charge_id => [ 'invoice_id' => $invoice_id, 'created' => $created ] ] );
	}

	public function email_invoice( $email, $order_id, $charge_id )
	{
		IF ( empty( $email ) OR ! is_email( $email ) ) :
			return;
		ENDIF;

		$arr_email = $this->print_invoice( $order_id, $charge_id );
		IF ( empty( $arr_email['html'] ) ) :
			return;
		ENDIF;

		$header  = "MIME-Version: 1.0\r\n";
		$header .= "Content-type: text/html; charset=utf-8\r\n";
		$header .= "From: {$this->payment['company_name']} <{$this->payment['company_email']}>\r\n";
		$header .= "X-Mailer: PHP ". phpversion();

		@wp_mail( $email, $arr_email['subject'], $arr_email['html'], $header );
	}

	public function email_refund( $email, $order_id, $charge_id )
	{
		IF ( empty( $email ) OR ! is_email( $email ) ) :
			return;
		ENDIF;

		$arr_email = $this->print_refund( $order_id, $charge_id );
		IF ( empty( $arr_email['html'] ) ) :
			return;
		ENDIF;

		$header  = "MIME-Version: 1.0\r\n";
		$header .= "Content-type: text/html; charset=utf-8\r\n";
		$header .= "From: {$this->payment['company_name']} <{$this->payment['company_email']}>\r\n";
		$header .= "X-Mailer: PHP ". phpversion();

		@wp_mail( $email, $arr_email['subject'], $arr_email['html'], $header );
	}

	public function stripe_process_payment()
	{
		IF ( ! is_admin() ) :
			$this->stripe_process_payment_one_click_upsell();
			$this->stripe_process_payment_regular();
		ENDIF;
	}

	public function stripe_process_payment_one_click_upsell()
	{
		IF ( empty( $_POST['action'] ) OR $_POST['action'] <> 'stripe' ) :
			return;
		ENDIF;

		IF ( empty( $_POST['oc_upsell'] ) OR $_POST['oc_upsell'] <> 'yes' ) :
			return;
		ENDIF;

		TRY {
			IF ( ! wp_verify_nonce( $_POST['stripe_nonce'], 'stripe-nonce' ) ) :
				throw new Exception( 'COULD-NOT-VERIFY-NONCE' );
			ENDIF;

			
			$arr_product_data  = $this->stripe_get_product_data( $_POST['product_id'] );
			$product_id        = $arr_product_data['product_id'];
			$mode              = $arr_product_data['mode'];
			$secret_key        = $arr_product_data['secret_key'];
			$amount            = $arr_product_data['amount'];
			$plan_id           = $arr_product_data['plan_id'];
			$pmeta             = $arr_product_data['pmeta'];

			$this->stripe_set_api_key( $secret_key );

			$error             = FALSE;
			$remote_contact_id = apply_filters( 'mbr/contact_id', NULL );
			IF ( ! $remote_contact_id ) :
				$error = TRUE;
			ENDIF;

			$tmp_contact = apply_filters( 'mbr/session/get', NULL );
			IF ( empty( $tmp_contact ) ) :
				$error = TRUE;
			ENDIF;

			IF ( $error ) :
				IF ( ! empty( $_POST['fallback_url'] ) ) :
					$redirect = add_query_arg( 'payment_error', __( 'Your payment could not be processed, please try again.', 'activemember360' ), $_POST['fallback_url'] );
					do_action( 'mbr/redirect', $redirect );
				ENDIF;

				$_REQUEST['payment_error'] = __( 'Your payment could not be processed, please try again.', 'activemember360' );
				return;
			ENDIF;

			$arr_account = $this->stripe_get_account_details();
			$account_id  = $arr_account['id'];

			
			$arr_customer_ids = json_decode( stripslashes( $tmp_contact[ MBR_STRIPE_CUST_ID_FIELD ] ), TRUE );
			IF ( ! empty( $arr_customer_ids[ $account_id ][ $mode ] ) ) :
				$customer_id = end( $arr_customer_ids[ $account_id ][ $mode ] );
			ENDIF;

			$arr_validation = $this->stripe_one_click_upsell_validate_fields( $tmp_contact );
			$arrPOST        = $arr_validation['sanitized'];

			IF ( ! $arr_validation['passed'] ) :
				IF ( ! empty( $_POST['fallback_url'] ) ) :
					$redirect = add_query_arg( 'payment_error', $arr_validation['error'], $_POST['fallback_url'] );
					do_action( 'mbr/redirect', $redirect );
				ENDIF;

				$_REQUEST['payment_error'] = $arr_validation['error'];
				return;
			ENDIF;

			IF ( ! empty( $customer_id ) ) :
				$new_customer         = FALSE;
				$card_id              = FALSE;

				IF ( ! empty( $amount ) AND $pmeta['_mbr_oitem_type'] == 'product' ) :
					
					$arr_charge           = $this->stripe_charge( $amount, $customer_id, $new_customer, $card_id, $product_id, $mode, $arrPOST );
					$charged_successfully = $arr_charge['result'];
					$trialing             = FALSE;
					$local_order_id       = $arr_charge['local_order_id'];
				ELSEIF ( ! empty( $plan_id ) AND $pmeta['_mbr_oitem_type'] == 'subscription' ) :
					$arr_charge           = $this->stripe_charge_subscription( $plan_id, $customer_id, $new_customer, $card_id, $product_id, $mode, $arrPOST );
					$charged_successfully = $arr_charge['result'];
					$trialing             = $arr_charge['trialing'];
					$local_order_id       = $arr_charge['local_order_id'];
				ELSE:
					throw new Exception( 'STRIPE-PRODUCT-HAS-PLAN-AND-AMOUNT' );
				ENDIF;

				$run_triggers   = FALSE;
				$trigger_type   = FALSE;
				IF ( $charged_successfully AND ! $trialing ) :
					$run_triggers = TRUE;
					$trigger_type = 'purchase';
				ELSEIF ( $charged_successfully AND $trialing ) :
					$run_triggers = TRUE;
					$trigger_type = 'trial';
				ENDIF;

				IF ( $run_triggers AND ! empty( $trigger_type ) ) :
					$arr_sync = [];
					$arr_sync['contact_data']       = [
						'email'                       => $tmp_contact['email'],
					];
					IF ( ! empty( $pmeta['_mbr_oitem_' . $trigger_type . '_add_automations'] ) ) :
						$arr_sync['assign_automations'] = $pmeta['_mbr_oitem_' . $trigger_type . '_add_automations'];
					ENDIF;
					IF ( ! empty( $pmeta['_mbr_oitem_' . $trigger_type . '_remove_automations'] ) ) :
						$arr_sync['remove_automations'] = $pmeta['_mbr_oitem_' . $trigger_type . '_remove_automations'];
					ENDIF;
					IF ( ! empty( $pmeta['_mbr_oitem_' . $trigger_type . '_assign_tags'] ) ) :
						$arr_sync['assign_tags']        = $pmeta['_mbr_oitem_' . $trigger_type . '_assign_tags'];
					ENDIF;
					IF ( ! empty( $pmeta['_mbr_oitem_' . $trigger_type . '_remove_tags'] ) ) :
						$arr_sync['remove_tags']        = $pmeta['_mbr_oitem_' . $trigger_type . '_remove_tags'];
					ENDIF;
					IF ( ! empty( $pmeta['_mbr_oitem_' . $trigger_type . '_subscribe_lists'] ) ) :
						$arr_sync['subscribe_lists']    = $pmeta['_mbr_oitem_' . $trigger_type . '_subscribe_lists'];
					ENDIF;
					IF ( ! empty( $pmeta['_mbr_oitem_' . $trigger_type . '_unsubscribe_lists'] ) ) :
						$arr_sync['unsubscribe_lists']  = $pmeta['_mbr_oitem_' . $trigger_type . '_unsubscribe_lists'];
					ENDIF;
					apply_filters( 'mbr/api/sync_ctla', $arr_sync );
				ENDIF;

				update_post_meta( $local_order_id, $this->order_activecampaign_contact_id_pm_key, $remote_contact_id );

				IF ( $charged_successfully ) :
					IF ( ! empty( $pmeta['_mbr_oitem_purchase_redirect_page'] ) ) :
						IF ( $pmeta['_mbr_oitem_purchase_redirect_page'] <> 'no_redirect' ) :
							IF ( $pmeta['_mbr_oitem_purchase_redirect_page'] == 'static_url' ) :
								IF ( ! empty( $pmeta['_mbr_oitem_purchase_redirect_url'] ) ) :
									do_action( 'mbr/redirect', $pmeta['_mbr_oitem_purchase_redirect_url'] );
								ENDIF;
							ELSE:
								do_action( 'mbr/redirect', get_permalink( $pmeta['_mbr_oitem_purchase_redirect_page'] ) );
							ENDIF;
						ENDIF;
					ENDIF;
					IF ( $charged_successfully AND ! $trialing ) :
						$redirect = add_query_arg( [ 'payment' => 'paid', 'order_id' => $local_order_id ], $_POST['redirect'] );
						do_action( 'mbr/redirect', $redirect );
					ELSEIF ( $charged_successfully AND $trialing ) :
						$redirect = add_query_arg( [ 'payment' => 'trialing', 'order_id' => $local_order_id ], $_POST['redirect'] );
						do_action( 'mbr/redirect', $redirect );
					ENDIF;
				ELSE:
					IF ( ! empty( $_POST['fallback_url'] ) ) :
						$redirect = add_query_arg( 'payment_error', __( 'Your payment could not be processed, please try again.', 'activemember360' ), $_POST['fallback_url'] );
						do_action( 'mbr/redirect', $redirect );
					ENDIF;
					$redirect = add_query_arg( 'payment_error', __( 'Your payment could not be processed, please try again.', 'activemember360' ), $_POST['redirect'] );
					do_action( 'mbr/redirect', $redirect );
				ENDIF;
			ELSE:
				
				IF ( ! empty( $_POST['fallback_url'] ) ) :
					do_action( 'mbr/redirect', $_POST['fallback_url'] );
				ENDIF;
				$redirect = add_query_arg( 'payment_error', __( 'Your payment could not be processed, please our contact support staff.', 'activemember360' ), $_POST['redirect'] );
				do_action( 'mbr/redirect', $redirect );
			ENDIF;
		} CATCH ( Exception $e ) {
			$error = mbrErrHdlr::create_error_std_object( $e->getMessage(), __LINE__, __FILE__ );
			mbrErrHdlr::mbr_error_exit( $error );
		}
	}

	public function stripe_process_payment_regular( $arr_to_validate = [] )
	{
		IF ( empty( $_POST['action'] ) OR $_POST['action'] <> 'stripe' ) :
			return;
		ENDIF;
		IF ( ! empty( $_POST['oc_upsell'] ) ) :
			return;
		ENDIF;

		TRY {
			IF ( ! wp_verify_nonce( $_POST['stripe_nonce'], 'stripe-nonce' ) ) :
				throw new Exception( 'COULD-NOT-VERIFY-NONCE' );
			ENDIF;

			
			$arr_product_data = $this->stripe_get_product_data( $_POST['product_id'] );
			$product_id       = $arr_product_data['product_id'];
			$mode             = $arr_product_data['mode'];
			$secret_key       = $arr_product_data['secret_key'];
			$amount           = $arr_product_data['amount'];
			$plan_id          = $arr_product_data['plan_id'];
			$pmeta            = $arr_product_data['pmeta'];

			$arr_validation   = $this->stripe_validate_fields( $_POST, $arr_to_validate );
			$arrPOST          = $arr_validation['sanitized'];
			IF ( ! $arr_validation['passed'] ) :
				$_REQUEST['payment_error'] = $arr_validation['error'];
				return;
			ENDIF;

			
			$this->stripe_set_api_key( $secret_key );

			$card_token      = $_POST['mbr_stripe_token'];

			IF ( $pmeta['_mbr_oitem_type'] == 'product' ) :
				$arr_customer  = $this->stripe_sync_customer_onetime( $arrPOST['email'], $card_token );
			ELSE:
				$arr_customer  = $this->stripe_sync_customer_subscriptions( $arrPOST['email'], $card_token );
			ENDIF;
			$customer_id     = $arr_customer['cus_id'];
			$card_id         = $arr_customer['card_id'];
			$new_customer    = $arr_customer['new_cus'];
			$arr_cus_ids     = $arr_customer['arr_cus_ids'];

			IF ( $customer_id ) :
				IF ( ! empty( $amount ) AND $pmeta['_mbr_oitem_type'] == 'product' ) :
					
					$arr_charge           = $this->stripe_charge( $amount, $customer_id, $new_customer, $card_id, $product_id, $mode, $arrPOST );
					$charged_successfully = $arr_charge['result'];
					$trialing             = FALSE;
					$local_order_id       = $arr_charge['local_order_id'];
				ELSEIF ( ! empty( $plan_id ) AND $pmeta['_mbr_oitem_type'] == 'subscription' ) :
					$arr_charge           = $this->stripe_charge_subscription( $plan_id, $customer_id, $new_customer, $card_id, $product_id, $mode, $arrPOST );
					$charged_successfully = $arr_charge['result'];
					$trialing             = $arr_charge['trialing'];
					$local_order_id       = $arr_charge['local_order_id'];
				ELSE:
					throw new Exception( 'STRIPE-PRODUCT-HAS-PLAN-AND-AMOUNT' );
				ENDIF;

				IF ( $arrPOST['billing_country'] <> 'US' ) :
					unset($arrPOST['billing_state']);
				ENDIF;

				$arr_sync = [];
				$arr_sync['contact_data']       = [
					'email'                       => $arrPOST['email'],
					'first_name'                  => $arrPOST['first_name'],
					'last_name'                   => $arrPOST['last_name'],
					'phone'                       => $arrPOST['phone'],
					'field[%' . $this->site_stripe_customer_id_field . '%]' => json_encode( $arr_cus_ids ),
					'field[%billing_first_name%]' => $arrPOST['first_name'],
					'field[%billing_last_name%]'  => $arrPOST['last_name'],
					'field[%billing_company%]'    => $arrPOST['billing_company'],
					'field[%billing_email%]'      => $arrPOST['email'],
					'field[%billing_address_1%]'  => $arrPOST['billing_address_1'],
					'field[%billing_address_2%]'  => $arrPOST['billing_address_2'],
					'field[%billing_city%]'       => $arrPOST['billing_city'],
					'field[%billing_state%]'      => $arrPOST['billing_state'],
					'field[%billing_postcode%]'   => $arrPOST['billing_postcode'],
					'field[%billing_country%]'    => $arrPOST['billing_country'],
					'field[%billing_phone%]'      => $arrPOST['phone']
				];

				$remote_contact_id = apply_filters( 'mbr/contact_id', NULL );

				IF ( ! $remote_contact_id ) :
					$generated_password = apply_filters( 'mbr/password/generate', NULL );
					$savePASS           = apply_filters( 'mbr/password/hash', $generated_password );
					$arr_sync['contact_data'][ apply_filters( 'mbr/password_perstag', NULL ) ] = $savePASS;
				ENDIF;

				$run_triggers   = FALSE;
				$trigger_type   = FALSE;
				IF ( $charged_successfully AND ! $trialing ) :
					$run_triggers = TRUE;
					$trigger_type = 'purchase';
				ELSEIF ( $charged_successfully AND $trialing ) :
					$run_triggers = TRUE;
					$trigger_type = 'trial';
				ENDIF;

				IF ( $run_triggers AND ! empty( $trigger_type ) ) :
					IF ( ! empty( $pmeta['_mbr_oitem_' . $trigger_type . '_add_automations'] ) ) :
						$arr_sync['assign_automations'] = $pmeta['_mbr_oitem_' . $trigger_type . '_add_automations'];
					ENDIF;
					IF ( ! empty( $pmeta['_mbr_oitem_' . $trigger_type . '_remove_automations'] ) ) :
						$arr_sync['remove_automations'] = $pmeta['_mbr_oitem_' . $trigger_type . '_remove_automations'];
					ENDIF;
					IF ( ! empty( $pmeta['_mbr_oitem_' . $trigger_type . '_assign_tags'] ) ) :
						$arr_sync['assign_tags']        = $pmeta['_mbr_oitem_' . $trigger_type . '_assign_tags'];
					ENDIF;
					IF ( ! empty( $pmeta['_mbr_oitem_' . $trigger_type . '_remove_tags'] ) ) :
						$arr_sync['remove_tags']        = $pmeta['_mbr_oitem_' . $trigger_type . '_remove_tags'];
					ENDIF;
					IF ( ! empty($pmeta['_mbr_oitem_' . $trigger_type . '_subscribe_lists'] ) ) :
						$arr_sync['subscribe_lists']    = $pmeta['_mbr_oitem_' . $trigger_type . '_subscribe_lists'];
					ENDIF;
					IF ( ! empty( $pmeta['_mbr_oitem_' . $trigger_type . '_unsubscribe_lists'] ) ) :
						$arr_sync['unsubscribe_lists']  = $pmeta['_mbr_oitem_' . $trigger_type . '_unsubscribe_lists'];
					ENDIF;
				ENDIF;

				$activecampaign_contact_id = apply_filters( 'mbr/api/sync_ctla', $arr_sync );

				update_post_meta( $local_order_id, $this->order_activecampaign_contact_id_pm_key, $activecampaign_contact_id );

				IF ( $charged_successfully ) :
					IF ( ! $remote_contact_id AND $arr_validation['wp_signon'] ) :
						do_action( 'mbr/templates/email/login_credentials/send', $arrPOST['email'], $generated_password );
						wp_signon( [ 'user_login' => $arrPOST['email'], 'user_password' => $generated_password ] );
					ENDIF;

					IF ( $arr_validation['wp_signon'] ) :
						IF ( ! empty( $pmeta['_mbr_oitem_purchase_redirect_page'] ) ) :
							IF ( $pmeta['_mbr_oitem_purchase_redirect_page'] <> 'no_redirect' ) :
								IF ( $pmeta['_mbr_oitem_purchase_redirect_page'] == 'static_url' ) :
									IF ( ! empty( $pmeta['_mbr_oitem_purchase_redirect_url'] ) ) :
										do_action( 'mbr/redirect', $pmeta['_mbr_oitem_purchase_redirect_url'] );
									ENDIF;
								ELSE:
									do_action( 'mbr/redirect', get_permalink( $pmeta['_mbr_oitem_purchase_redirect_page'] ) );
								ENDIF;
							ENDIF;
						ENDIF;
					ENDIF;;
				ENDIF;
			ELSE :
				
				throw new Exception( 'STRIPE-COULD-NOT-CREATE-OR-FIND-CUSTOMER' );
			ENDIF;

			// redirect on successful payment
			IF ( $charged_successfully AND ! $trialing ) :
				$redirect = add_query_arg( [ 'payment' => 'paid', 'order_id' => $local_order_id ], $_POST['redirect'] );
				do_action( 'mbr/redirect', $redirect);
			ELSEIF ($charged_successfully AND $trialing) :
				$redirect = add_query_arg( [ 'payment' => 'trialing', 'order_id' => $local_order_id ], $_POST['redirect'] );
				do_action( 'mbr/redirect', $redirect);
			ELSE:
				$redirect = add_query_arg( 'payment_error', __('Your payment could not be processed, please try again.', 'activemember360' ), $_POST['redirect'] );
				do_action( 'mbr/redirect', $redirect);
			ENDIF;
		} CATCH ( Exception $e ) {
			$error = mbrErrHdlr::create_error_std_object( $e->getMessage(), __LINE__, __FILE__ );
			mbrErrHdlr::mbr_error_exit( $error );
		}
	}

	public function sc_my_payments($parms, $content = NULL, $scname = '')
	{
		global $wpdb;

		IF ( ! is_user_logged_in() ) :
			return;
		ENDIF;

		$parms           = apply_filters( 'mbr/shortcodes/atts', [ 'mode'=> 'test', 'status' => 'all' ], $parms );
		$payment_status  = $parms['status'];

		IF ( $parms['mode'] == 'live' ) :
			$secret_key                  = trim( $this->payment['stripe_live_secret'] );
			$publishable_key             = trim( $this->payment['stripe_live_publishable'] );
		ELSE:
			$secret_key                  = trim( $this->payment['stripe_test_secret'] );
			$publishable_key             = trim( $this->payment['stripe_test_publishable'] );
		ENDIF;

		$this->stripe_set_api_key( $secret_key );
		$out                         = '';

		
		static $has_run_subscription_action = FALSE;

		IF ( ! $has_run_subscription_action ) :

			$has_run_subscription_action = TRUE;

			IF ( isset( $_POST['mbr_cancel_subscription'] ) ) :
				TRY {
					$sub = \Stripe\Subscription::retrieve( $_POST['mbr_subscription_to_cancel'] );
					$sub->cancel( [ 'at_period_end' => TRUE ] );

					IF ( ! empty( $_POST['mbr_order_id'] ) ) :
						wp_update_post( [ 'ID' => $_POST['mbr_order_id'], 'post_status' => 'mbr_payment_sub_cape' ] );
						update_post_meta( $_POST['mbr_order_id'], $this->order_post_status_pm_key, $this->order_payment_statuses['mbr_payment_sub_cape']['label_regular'] );
					ENDIF;

					$out .= '<p class="mbr-payment-card-update-success">' . __( 'Subscription cancelled at period end.', 'activemember360' ) . '</p>';
				} CATCH ( Exception $e ) {
					$out .= '<p class="mbr-payment-card-update-error">' . __( 'Could not cancel subscription. Please contact support staff.', 'activemember360' ) . '</p>';
				}
			ELSEIF ( isset( $_POST['mbr_reactivate_subscription'] ) ) :
				TRY {
					$sub          = \Stripe\Subscription::retrieve( $_POST['mbr_subscription_to_reactivate'] );
					$arr_sub_plan = $sub->plan->__toArray( TRUE );
					$sub->plan    = $arr_sub_plan['id'];
					$sub->save();

					IF ( ! empty( $_POST['mbr_order_id'] ) ) :
						wp_update_post( [ 'ID' => $_POST['mbr_order_id'], 'post_status' => 'mbr_payment_sub_actv' ] );
						update_post_meta( $_POST['mbr_order_id'], $this->order_post_status_pm_key, $this->order_payment_statuses['mbr_payment_sub_actv']['label_regular'] );
					ENDIF;

					$out .= '<p class="mbr-payment-card-update-success">' . __( 'Subscription reactivated.', 'activemember360' ) . '</p>';
				} CATCH ( Exception $e ) {
					$out .= '<p class="mbr-payment-card-update-error">' . __( 'Could not reactivate subscription. Please contact support staff.', 'activemember360' ) . '</p>';
				}
			ELSEIF ( isset( $_POST['mbr_change_card_subscription'] ) ) :
				IF ( ! empty( $publishable_key ) ) :
					$customer           = \Stripe\Customer::retrieve( $_POST['mbr_subscription_customer_id'] );
					$customer_sources   = $customer->sources->__toArray( TRUE );
					$last4_default_card = '**** **** **** **** ' . $customer_sources['data'][0]['last4'] . ' - ' . $customer_sources['data'][0]['exp_month'] . '/' . $customer_sources['data'][0]['exp_year'];
					$out .= '<script type="text/javascript" src="https://js.stripe.com/v1/"></script>';
					$out .= "<script>Stripe.setPublishableKey('{$publishable_key}');</script>";
					$out .= '<script src="' . plugins_url( '/templates/assets/js/token.js', MBRNAME ) . '"></script>';
					$out .= '<script src="' . plugins_url( '/templates/assets/js/jquery.validate.min.js', MBRNAME ) . '"></script>';
					$out .= '<div id="mbr-payment-update-credit-card-container" style="display:none;"><p>';
					$out .= '<form method="post" action="" id="mbr-payment-form">';
					$out .= '<input type="hidden" name="mbr_action" value="add_new_card" />';
					$out .= '<input type="hidden" name="mbr_customer_id" value="' . $_POST['mbr_subscription_customer_id'] . '" />';
					$out .= '<p>' . sprintf( __( 'Current Card: %s', 'activemember360' ), ( ! empty( $last4_default_card ) ? $last4_default_card : __( 'none', 'activemember360' ) ) ) . '</p>';
					$out .= '<table id="mbr-payment-update-credit-card" class="mbr-full-width">';
					$out .= '<tr><td><input class="mbr-td-one-third" type="text" id="mbr_card_number" placeholder="' . __( 'Card Number', 'activemember360' ) . '" maxlength="16" data-validation="number length" data-validation-length="min13" /></td></tr>';
					$out .= '<tr><td><input class="mbr-td-one-third" type="text" id="mbr_card_cvc" placeholder="' . __( 'CVC', 'activemember360' ) . '" maxlength="4" data-validation="number length" data-validation-length="min3" /></td></tr>';
					$out .= '<tr><td>' . __( 'Expiration:', 'activemember360' ) . '</td></tr>';
					$out .= '<tr><td><select id="mbr_card_exp_month" class="mbr-td-one-third" data-validation="required">';
					$out .= '<option value="">'   . __( 'Month', 'activemember360' )     . '</option>';
					$out .= '<option value="01">' . __( 'January', 'activemember360' )   . '</option>';
					$out .= '<option value="02">' . __( 'February', 'activemember360' )  . '</option>';
					$out .= '<option value="03">' . __( 'March', 'activemember360' )     . '</option>';
					$out .= '<option value="04">' . __( 'April', 'activemember360' )     . '</option>';
					$out .= '<option value="05">' . __( 'May', 'activemember360' )       . '</option>';
					$out .= '<option value="06">' . __( 'June', 'activemember360' )      . '</option>';
					$out .= '<option value="07">' . __( 'July', 'activemember360' )      . '</option>';
					$out .= '<option value="08">' . __( 'August', 'activemember360' )    . '</option>';
					$out .= '<option value="09">' . __( 'September', 'activemember360' ) . '</option>';
					$out .= '<option value="10">' . __( 'October', 'activemember360' )   . '</option>';
					$out .= '<option value="11">' . __( 'November', 'activemember360' )  . '</option>';
					$out .= '<option value="12">' . __( 'December', 'activemember360' )  . '</option>';
					$out .= '</select></td></tr>';
					$out .= '<tr><td><select id="mbr_card_exp_year" class="mbr-td-one-third" data-validation="required">';
					$out .= '<option value="">' . __( 'Year', 'activemember360' ) . '</option>';
					FOR ( $x = date( 'Y' ); $x <= date( 'Y' ) + 25; $x++ ) :
						$out .= '<option value="' . $x . '">' . $x . '</option>';
					ENDFOR;
					$out .= '</select></td></tr>';
					$out .= '</table>';
					$out .= '<p><input type="submit" id="mbr-payment-submit" name="" value="' . __( 'Update', 'activemember360' ) . '" /></p>';
					$out .= '</form>';
					$out .= '</p></div>';
					$out .= '<script>
										 jQuery( window ).load( function() {
											 tb_show( "' . sprintf( __( 'Update Credit Card for Order# %s', 'activemember360' ), $_POST['mbr_order_id'] ) . '", "#TB_inline?width=600&height=350&inlineId=mbr-payment-update-credit-card-container" );
										 } );
										 jQuery.validate( { form:\'#mbr-payment-form\' } );
									 </script>';
				ELSE:
					$out .= '<p class="mbr-payment-card-update-error">' . __( 'An error occured. Please contact support staff.', 'activemember360' ) . '</p>';
				ENDIF;
			ELSEIF ( ! empty( $_POST['mbr_action'] ) AND $_POST['mbr_action'] == 'add_new_card' ) :
				TRY {
					$customer = \Stripe\Customer::retrieve( $_POST['mbr_customer_id'] );
					$customer->source = $_POST['mbr_stripe_token'];
					$customer->save();
					$out .= '<p class="mbr-payment-card-update-success">' . __( 'Credit Card updated.',   'activemember360' ) . '</p>';
				} CATCH (Exception $e) {
					$out .= '<p class="mbr-payment-card-update-error">' . __( 'Could not create card.', 'activemember360' ) . '</p>';
				}
			ENDIF;
		ENDIF;

		$arr_account                   = $this->stripe_get_account_details();
		$account_id                    = $arr_account['id'];

		
		$remote_contact_id = apply_filters( 'mbr/contact_id', NULL );
		IF ( $remote_contact_id ) :
			$tmp_contact                 = apply_filters( 'mbr/session/get', NULL );
			$arr_customer_ids            = json_decode( stripslashes( $tmp_contact[ MBR_STRIPE_CUST_ID_FIELD ] ), TRUE );
			IF (!empty($arr_customer_ids[ $account_id][ $parms['mode'] ] ) ) :
				$arr_order_ids             = $wpdb->get_results( 'SELECT post_id FROM ' . $wpdb->postmeta . ' WHERE meta_key = "' . $this->order_activecampaign_contact_id_pm_key . '" AND meta_value = "' . $remote_contact_id . '" ORDER BY post_id DESC', ARRAY_N );
				$arr_order_ids             = call_user_func_array( 'array_merge', $arr_order_ids );

				IF ( ! empty( $arr_order_ids ) ) :
					$out                    .= '<table class="mbr_my_payments_tbl">';
					$theadfoot               = '<tr>
																				<th>' . __( 'Order No.', 'activemember360' ) . '</th>
																				<th>' . __( 'Date',      'activemember360' ) . '</th>
																				<th>' . __( 'Amount',    'activemember360' ) . '</th>
																				<th>' . __( 'Status',    'activemember360' ) . '</th>
																				<th>' . __( 'Details',   'activemember360' ) . '</th>
																			</tr>';
					$out                    .= "<thead>{$theadfoot}</thead>";
					$out                    .= "<tfoot>{$theadfoot}</tfoot>";
					$out                    .= "<tbody>";
					FOREACH ( $arr_order_ids AS $key => $order_id ) :
						$order                 = get_post( $order_id );
						$arr_payment_mode      = get_post_meta( $order_id, $this->order_payment_method_pm_key, TRUE );
						IF ( $arr_payment_mode['mode'] <> $parms['mode'] ) :
							continue;
						ENDIF;

						IF ( $payment_status <> 'all' ) :
							IF ( $payment_status == 'paid' ) :
								IF ( $order->post_status <> 'mbr_payment_success' ) :
									continue;
								ENDIF;
							ELSEIF ( $payment_status == 'failed' ) :
								IF ( $order->post_status <> 'mbr_payment_failed' ) :
									continue;
								ENDIF;
							ELSEIF ( $payment_status == 'pending' ) :
								IF ( $order->post_status <> 'mbr_payment_pending' ) :
									continue;
								ENDIF;
							ELSEIF ( $payment_status == 'active' ) :
								IF ( $order->post_status <> 'mbr_payment_sub_actv' ) :
									continue;
								ENDIF;
							ELSEIF ( $payment_status == 'paused' ) :
								IF ( $order->post_status <> 'mbr_payment_sub_paus' ) :
									continue;
								ENDIF;
							ELSEIF ( $payment_status == 'trial' ) :
								IF ( $order->post_status <> 'mbr_payment_sub_tril' ) :
									continue;
								ENDIF;
							ELSEIF ( $payment_status == 'cancelled' ) :
								IF ( $order->post_status <> 'mbr_payment_sub_canc' ) :
									continue;
								ENDIF;
							ELSEIF ( $payment_status == 'cancelled_at_period_end' ) :
								IF ( $order->post_status <> 'mbr_payment_sub_cape' ) :
									continue;
								ENDIF;
							ELSEIF ( $payment_status == 'refund' ) :
								IF ( $order->post_status <> 'mbr_payment_ref_full' AND $order->post_status <> 'mbr_payment_ref_part' ) :
									continue;
								ENDIF;
							ELSEIF ( $payment_status == 'refund_full' ) :
								IF ( $order->post_status <> 'mbr_payment_ref_full' ) :
									continue;
								ENDIF;
							ELSEIF ( $payment_status == 'refund_partial' ) :
								IF ( $order->post_status <> 'mbr_payment_ref_part' ) :
									continue;
								ENDIF;
							ENDIF;
						ENDIF;

						$date                  = apply_filters( 'mbr/datetime/convert', substr( $order->post_date_gmt,0,10 ) );
						$status                = $this->order_payment_statuses[ $order->post_status ]['label_regular'];
						$order_customer_email  = get_post_meta( $order_id, $this->order_customer_email_pm_key,   TRUE );
						$order_customer_id     = get_post_meta( $order_id, $this->stripe_customer_id_pm_key,     TRUE );

						$order_amount          = get_post_meta( $order_id, $this->order_amount_pm_key,           TRUE );
						$order_currency        = get_post_meta( $order_id, $this->order_currency_pm_key,         TRUE );
						$billing_details       = get_post_meta( $order_id, $this->order_billing_details_pm_key,  TRUE );
						$subscription_id       = get_post_meta( $order_id, $this->stripe_subscription_id_pm_key, TRUE );
						$arr_refunds           = get_post_meta( $order_id, $this->stripe_refunds_pm_key,         TRUE );
						IF ( empty( $arr_refunds) OR ! is_array( $arr_refunds ) ) :
							$arr_refunds         = [];
						ENDIF;
						$arr_charges           = get_post_meta( $order_id, $this->stripe_charges_pm_key,         TRUE );
						IF ( empty( $charges ) OR ! is_array( $charges ) ) :
							$charges = [];
						ENDIF;
						$price                 = $this->stripe_price_format( $order_currency, $order_amount );

						$details               = "<div id='mbr_payment_details_{$order_id}' style='display:none;'>";
						$details              .= '<strong>' . __( 'Order Details', 'activemember360' ) . '</strong>';
						$details              .= '<table class="mbr_payment_details">';
						$details              .= '<tr><td>' . __( 'Order No.',     'activemember360' ) . ':</td><td>' . $order_id . '</td></tr>';
						$details              .= '<tr><td>' . __( 'Status',        'activemember360' ) . ':</td><td>' . $status . '</td></tr>';
						$details              .= '<tr><td>' . __( 'Date',          'activemember360' ) . ':</td><td>' . apply_filters( 'mbr/datetime/convert', $order->post_date_gmt, TRUE ) . '</td></tr>';
						$details              .= '<tr><td>' . __( 'Amount',        'activemember360' ) . ':</td><td>' . $price . '</td></tr>';
						$details              .= '</table>';
						$details              .= '<strong>' . __( 'Order Items',   'activemember360' ) . '</strong>';
						$details              .= '<table class="mbr_payment_details">';

						$arr_order_items       = get_post_meta( $order_id, $this->order_items_pm_key, TRUE );
						FOREACH ( $arr_order_items AS $key => $arr_order_item ) :
							$product_name          = $this->get_product_name_by_id( $arr_order_item['id'], FALSE, FALSE );
							$product_single_price  = $this->stripe_price_format( $order_currency, $arr_order_item['amount'] );
							$details              .= '<tr><td># ' . $arr_order_item['id'] . '</td><td>' . $arr_order_item['qty'] . ' x ' . $product_name . ' (' . $product_single_price . ')</td></tr>';
						ENDFOREACH;

						$us_states             = apply_filters( 'mbr/payments/states/us/get', NULL );
						$countries             = apply_filters( 'mbr/payments/countries/get', NULL );

						$details              .= '</table>';
						$details              .= '<strong>' . __( 'Billing Details', 'activemember360' ) . '</strong>';
						$details              .= '<table class="mbr_payment_details">';
						$details              .= '<tr><td>' . __( 'Email',       'activemember360' ) . ':</td><td>' . $order_customer_email . '</td></tr>';
						$details              .= '<tr><td>' . __( 'First Name',  'activemember360' ) . ':</td><td>' . $billing_details['first_name']        . '</td></tr>';
						$details              .= '<tr><td>' . __( 'Last Name',   'activemember360' ) . ':</td><td>' . $billing_details['last_name']         . '</td></tr>';
						$details              .= '<tr><td>' . __( 'Company',     'activemember360' ) . ':</td><td>' . $billing_details['billing_company']   . '</td></tr>';
						$details              .= '<tr><td>' . __( 'Address 1',   'activemember360' ) . ':</td><td>' . $billing_details['billing_address_1'] . '</td></tr>';
						$details              .= '<tr><td>' . __( 'Address 2',   'activemember360' ) . ':</td><td>' . $billing_details['billing_address_2'] . '</td></tr>';
						$details              .= '<tr><td>' . __( 'City',        'activemember360' ) . ':</td><td>' . $billing_details['billing_city']      . '</td></tr>';
						$details              .= '<tr><td>' . __( 'Postal Code', 'activemember360' ) . ':</td><td>' . $billing_details['billing_postcode']  . '</td></tr>';
						$order_us_state        = '';
						IF ( ! empty( $billing_details['billing_state'] ) AND ! empty( $us_states[ $billing_details['billing_state'] ] ) ) :
							$order_us_state      = $us_states[ $billing_details['billing_state'] ];
						ENDIF;
						$details              .= '<tr><td>' . __( 'State',       'activemember360' ) . ':</td><td>' . $order_us_state   . '</td></tr>';
						$order_country         = '';
						IF ( ! empty( $billing_details['billing_country'] ) AND ! empty( $countries[ $billing_details['billing_country'] ] ) ) :
							$order_country       = $countries[ $billing_details['billing_country'] ];
						ENDIF;
						$details              .= '<tr><td>' . __( 'Country',     'activemember360' ) . ':</td><td>' .$order_country . '</td></tr>';
						$details              .= '</table>';
						$details              .= '<strong>' . __( 'Refund History', 'activemember360') . '</strong>';

						$arr_invoice_ids       = $this->get_invoices($order_id);

						IF ( ! empty( $arr_refunds ) ) :
							$total_amount_refunded   = 0;
							$details            .= '<table class="mbr_payment_details">';
							FOREACH ( $arr_refunds AS $charge_id => $arr_refund ) :
								FOREACH ( $arr_refund AS $key => $refund ) :
									$total_amount_refunded = $total_amount_refunded + $refund['amount'];
									$invoice_id      = ( ! empty( $arr_invoice_ids[ $charge_id ] ) ? $arr_invoice_ids[ $charge_id ] : __( 'Unknown', 'activemember360' ) );
									$details        .= '<tr><td>' . apply_filters( 'mbr/datetime/convert', $refund['created'], TRUE ) . ':</td><td>' . sprintf( __( 'Invoice #%s', 'activemember360' ), $invoice_id ) . ' ' . $this->stripe_price_format( $refund['currency'], $refund['amount'] ) . '</td></tr>';
								ENDFOREACH;
							ENDFOREACH;
							$details            .= '</table>';
							$details            .= '<table class="mbr_payment_details">';
							$details            .= '<tr><td>' . __( 'Total refund:', 'activemember360' ) . '</td><td>' . $this->stripe_price_format( $order_currency, $total_amount_refunded ) . '</td></tr>';
							$details            .= '</table>';
						ELSE:
							$details            .= '<table class="mbr_payment_details"><tr><td>' .  __( 'No refunds has been issued yet.', 'activemember360' ) . '</td></tr></table>';
						ENDIF;
						$details              .= '<strong>' . __( 'Charge History', 'activemember360' ) . '</strong>';
						IF ( ! empty( $arr_charges ) ) :
							$details            .= '<table class="mbr_payment_details">';
							FOREACH ( $arr_charges AS $charge_id => $arr_charge ) :
								$invoice           = $this->jquery_append_body_form( home_url( '/mbr-invoice/' . $order_id ), $charge_id, __( 'Invoice', 'activemember360' ) );
								$refund_receipt    = '';
								IF ( ! empty( $arr_refunds[ $charge_id ] ) ) :
									$refund_receipt  = ' | ' . $this->jquery_append_body_form( home_url( '/mbr-refund/' . $order_id ), $charge_id, __( 'Refund Receipt', 'activemember360' ) );
								ENDIF;
								IF ( ! empty( $arr_invoice_ids[ $charge_id ] ) ) :
									$invoice_id      = $arr_invoice_ids[ $charge_id ];
									$details        .= '<tr><td>' . apply_filters( 'mbr/datetime/convert', $arr_charge['created'], TRUE ) . ':</td><td>' . sprintf( __( 'Invoice #%s', 'activemember360' ), $invoice_id ) . ' ' . $this->stripe_price_format( $arr_charge['currency'], $arr_charge['amount'] ) . '</td><td><strong>' . ( $arr_charge['paid'] == 1 ?  __( 'Paid', 'activemember360' )  :  __( 'Pending', 'activemember360' ) ) . '</strong></td><td>' . $invoice . $refund_receipt . '</td></tr>';
								ELSE:
									$details        .= '<tr><td>' . apply_filters( 'mbr/datetime/convert', $arr_charge['created'], TRUE ) . ':</td><td colspan="2">' . $this->stripe_price_format( $arr_charge['currency'], $arr_charge['amount'] ) . '</td><td><strong>' . ( $arr_charge['paid'] == 1 ?  __( 'Paid', 'activemember360' )  :  __( 'Pending', 'activemember360' ) ) . '</strong></td></tr>';
								ENDIF;
							ENDFOREACH;
							$details            .= '</table>';
						ELSE:
							$details            .= '<table class="mbr_payment_details"><tr><td>' .  __( 'No charges has been made yet.', 'activemember360' ) . '</td></tr></table>';
						ENDIF;
						$details              .= "</div>";

						$button_cancel         = '';
						$button_change         = '';
						IF ( in_array( $order->post_status, [ 'mbr_payment_sub_tril', 'mbr_payment_sub_paus', 'mbr_payment_sub_actv' ] ) AND ! empty( $subscription_id ) ) :
							$button_change      .= '<form method="post" action="" class="mbr-forms mbr-label-left mbr-payment-button-form">';
							$button_change      .= '<input type="hidden" name="mbr_order_id" value="' . $order_id . '" />';
							$button_change      .= '<input type="hidden" name="mbr_subscription_customer_id" value="' . $order_customer_id . '" />';
							$button_change      .= '<input id="mbr-submit" type="submit" name="mbr_change_card_subscription" value="' . __( 'Change Credit Card', 'activemember360' ) . '" />';
							$button_change      .= '</form>';
							$button_cancel      .= '<form method="post" action="" class="mbr-forms mbr-label-left mbr-payment-button-form">';
							$button_cancel      .= '<input type="hidden" name="mbr_order_id" value="' . $order_id . '" />';
							$button_cancel      .= '<input type="hidden" name="mbr_subscription_to_cancel" value="' . $subscription_id . '" />';
							$button_cancel      .= '<input id="mbr-submit" type="submit" name="mbr_cancel_subscription" value="' . __( 'Cancel', 'activemember360' ) . '" />';
							$button_cancel      .= '</form>';
						ENDIF;

						$button_reactivate     = '';
						IF ( $order->post_status == 'mbr_payment_sub_cape' AND ! empty( $subscription_id ) ) :
							$button_reactivate  .= '<form method="post" action="" class="mbr-forms mbr-label-left mbr-payment-button-form">';
							$button_reactivate  .= '<input type="hidden" name="mbr_order_id" value="' . $order_id . '" />';
							$button_reactivate  .= '<input type="hidden" name="mbr_subscription_to_reactivate" value="' . $subscription_id . '" />';
							$button_reactivate  .= '<input id="mbr-submit" type="submit" name="mbr_reactivate_subscription" value="' . __( 'Reactivate', 'activemember360' ) . '" />';
							$button_reactivate  .= '</form>';
						ENDIF;
						$out                  .= "<tr>
																				<td>{$order_id}</td>
																				<td>{$date}</td>
																				<td>{$price}</td>
																				<td>{$status}</td>
																				<td>
																					<form method='post' action='' class=\"mbr-forms mbr-label-left mbr-payment-button-form\" onsubmit='return false;'><input id='mbr-submit' type='submit' onclick='tb_show(\"" . sprintf( __( 'Order %s', 'activemember360' ), $order_id ) . "\", \"#TB_inline?width=750&height=680&inlineId=mbr_payment_details_" . $order_id . "\")' value='" . __( 'More', 'activemember360' ) . "'/></form>
																					{$button_change}{$button_cancel}{$button_reactivate}{$details}
																				</td>
																			</tr>";
					ENDFOREACH;
					$out                    .= "</tbody>";
					$out                    .= '</table>';
				ELSE:
					$out                    .= __( 'No payments found related to your account.', 'activemember360' );
				ENDIF;
			ELSE:
				$out                      .= __( 'No payments found related to your account.', 'activemember360' );
			ENDIF;

			return $out;
		ENDIF;

		return;
	}

	public function jquery_append_body_form( $url, $charge_id, $link_text )
	{
		$uniqid = uniqid() . $charge_id;
		$ret    =   <<<EOT
			<a href='#' id='link_{$uniqid}'>$link_text</a>
			<script>
				jQuery( document ).ready( function() {
					jQuery( '<form method="post" id="form_{$uniqid}" action="$url" target="_blank"><input type="hidden" name="mbr-charge-id" value="$charge_id" /></form>' ).appendTo( 'body' );
				} );
				jQuery( "#link_$uniqid" ).click( function() {
					jQuery( "#form_$uniqid" ).submit();
				} );
			</script>
EOT;
		return $ret;
	}

	public function sc_product_details( $parms, $content = NULL, $scname )
	{

		$parms = apply_filters( 'mbr/shortcodes/atts', [ 'product_id'=> FALSE, 'size' => '', 'width' => '', 'height' => '' ], $parms );

		IF ( empty( $parms['product_id'] ) ) :
			return __( 'No product ID provided', 'activemember360' );
		ENDIF;

		$product = get_post( $parms['product_id'] );

		IF ( empty( $product ) OR $product->post_type <> MBRCUSTPRODUCTS AND $product->post_type <> MBRCUSTSUBSCRIPTIONS ) :
			return __( 'Product does not exist.', 'activemember360' );
		ENDIF;

		IF ( $product->post_type == MBRCUSTPRODUCTS ) :
			$settings_meta_key = MBR_PRODUCT_SETTINGS;
		ELSE:
			$this->get_stripe_plans();
			$settings_meta_key = MBR_SUBSCRIPTION_SETTINGS;
		ENDIF;

		$pmeta   = redux_post_meta( $settings_meta_key, $product->ID );

		IF ( empty( $pmeta['_mbr_oitem_status'] ) ) :
			return __( 'Product is inactive', 'activemember360' );
		ENDIF;

		IF ( $pmeta['_mbr_oitem_type'] <> 'product' AND $pmeta['_mbr_oitem_type'] <> 'subscription' ) :
			return __( 'Item is not product type "Product" or "Subscription"', 'activemember360' );
		ENDIF;

		IF ( $scname == 'mbr_product_name' ) :
			return $product->post_title;
		ENDIF;

		IF ( $scname == 'mbr_product_full_desc' ) :
			return $product->post_content;
		ENDIF;

		IF ( $scname == 'mbr_product_short_desc' ) :
			return $product->post_excerpt;
		ENDIF;

		IF ( $scname == 'mbr_product_image' ) :
			IF ( empty( $parms['size'] ) ) :
				$style   = 'style="';
				$style  .= ! empty( $parms['width'] )  ? 'width:'  . $parms['width']  . ';' : '';
				$style  .= ! empty( $parms['height'] ) ? 'height:' . $parms['height'] . ';' : '';
				$style  .= '"';
			ENDIF;
			IF ( $image_url = get_the_post_thumbnail_url( $parms['product_id'], $parms['size'] ) ) :
				return "<img src='" . $image_url . "' $style alt='" . get_the_title( $parms['product_id'] ) . "' />";
			ENDIF;
		ENDIF;

		IF ( $product->post_type == MBRCUSTPRODUCTS ) :
			IF ( $scname == 'mbr_product_regular_price' ) :
				IF ( $pmeta['_mbr_oitem_status'] == 1 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_live_regular_price'] ) ) :
						$amount =  $pmeta['_mbr_oitem_live_regular_price'];
					ENDIF;
				ELSEIF ( $pmeta['_mbr_oitem_status'] == 2 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_test_regular_price'] ) ) :
						$amount = $pmeta['_mbr_oitem_test_regular_price'];
					ENDIF;
				ENDIF;
				$amount = apply_filters( 'mbr/payments/stripe/convert_amount', $amount );
				return $this->stripe_price_format( MBR_PAYMENT_CURRENCY, $amount );
			ENDIF;

			IF ( $scname == 'mbr_product_sale_price' ) :
				IF ( $pmeta['_mbr_oitem_status'] == 1 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_live_sale_price'] ) ) :
						$amount = $pmeta['_mbr_oitem_live_sale_price'];
					ENDIF;
				ELSEIF ($pmeta['_mbr_oitem_status'] == 2) :
					IF ( ! empty( $pmeta['_mbr_oitem_test_sale_price'] ) ) :
						$amount = $pmeta['_mbr_oitem_test_sale_price'];
					ENDIF;
				ENDIF;
				IF ( ! empty( $amount ) ) :
					$amount = apply_filters( 'mbr/payments/stripe/convert_amount', $amount );
					return $this->stripe_price_format( MBR_PAYMENT_CURRENCY, $amount );
				ENDIF;
			ENDIF;
		ELSE:
			IF ( $scname == 'mbr_product_regular_plan_interval' ) :
				IF ( $pmeta['_mbr_oitem_status'] == 1 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_live_regular_plan'] ) ) :
						return __( $this->stripe_plans['live'][ $pmeta['_mbr_oitem_live_regular_plan'] ]['interval'], 'activemember360' );
					ENDIF;
				ELSEIF ( $pmeta['_mbr_oitem_status'] == 2 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_test_regular_plan'] ) ) :
						return __( $this->stripe_plans['test'][ $pmeta['_mbr_oitem_test_regular_plan'] ]['interval'], 'activemember360' );
					ENDIF;
				ENDIF;
			ENDIF;

			IF ( $scname == 'mbr_product_sale_plan_interval' ) :
				IF ( $pmeta['_mbr_oitem_status'] == 1 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_live_sale_plan'] ) ) :
						return __( $this->stripe_plans['live'][ $pmeta['_mbr_oitem_live_sale_plan'] ]['interval'], 'activemember360' );
					ENDIF;
				ELSEIF ( $pmeta['_mbr_oitem_status'] == 2 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_test_sale_plan'] ) ) :
						return __( $this->stripe_plans['test'][ $pmeta['_mbr_oitem_test_sale_plan'] ]['interval'], 'activemember360' );
					ENDIF;
				ENDIF;
			ENDIF;

			IF ( $scname == 'mbr_product_regular_plan_interval_count' ) :
				IF ( $pmeta['_mbr_oitem_status'] == 1 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_live_regular_plan'] ) ) :
						return $this->stripe_plans['live'][ $pmeta['_mbr_oitem_live_regular_plan'] ]['interval_count'];
					ENDIF;
				ELSEIF ( $pmeta['_mbr_oitem_status'] == 2 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_test_regular_plan'] ) ) :
						return $this->stripe_plans['test'][ $pmeta['_mbr_oitem_test_regular_plan'] ]['interval_count'];
					ENDIF;
				ENDIF;
			ENDIF;

			IF ( $scname == 'mbr_product_sale_plan_interval_count' ) :
				IF ( $pmeta['_mbr_oitem_status'] == 1 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_live_sale_plan'] ) ) :
						return $this->stripe_plans['live'][ $pmeta['_mbr_oitem_live_sale_plan'] ]['interval_count'];
					ENDIF;
				ELSEIF ($pmeta['_mbr_oitem_status'] == 2) :
					IF ( ! empty( $pmeta['_mbr_oitem_test_sale_plan'] ) ) :
						return $this->stripe_plans['test'][ $pmeta['_mbr_oitem_test_sale_plan'] ]['interval_count'];
					ENDIF;
				ENDIF;
			ENDIF;

			IF ( $scname == 'mbr_product_regular_plan_trial_period' ) :
				IF ( $pmeta['_mbr_oitem_status'] == 1 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_live_regular_plan'] ) ) :
						return ! empty( $this->stripe_plans['live'][ $pmeta['_mbr_oitem_live_regular_plan'] ]['trial_period_days'] ) ? $this->stripe_plans['test'][ $pmeta['_mbr_oitem_live_regular_plan'] ]['trial_period_days'] : 0;
					ENDIF;
				ELSEIF ( $pmeta['_mbr_oitem_status'] == 2 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_test_regular_plan'] ) ) :
						return ! empty( $this->stripe_plans['test'][ $pmeta['_mbr_oitem_test_regular_plan'] ]['trial_period_days'] ) ? $this->stripe_plans['test'][ $pmeta['_mbr_oitem_test_regular_plan'] ]['trial_period_days'] : 0;
					ENDIF;
				ENDIF;
			ENDIF;

			IF ( $scname == 'mbr_product_sale_plan_trial_period' ) :
				IF ( $pmeta['_mbr_oitem_status'] == 1 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_live_sale_plan'] ) ) :
						return ! empty( $this->stripe_plans['live'][ $pmeta['_mbr_oitem_live_sale_plan'] ]['trial_period_days'] ) ? $this->stripe_plans['test'][ $pmeta['_mbr_oitem_live_sale_plan'] ]['trial_period_days'] : 0;
					ENDIF;
				ELSEIF ( $pmeta['_mbr_oitem_status'] == 2 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_test_sale_plan'] ) ) :
						return ! empty( $this->stripe_plans['test'][ $pmeta['_mbr_oitem_test_sale_plan'] ]['trial_period_days'] ) ? $this->stripe_plans['test'][ $pmeta['_mbr_oitem_test_sale_plan'] ]['trial_period_days'] : 0;
					ENDIF;
				ENDIF;
			ENDIF;

			IF ( $scname == 'mbr_product_regular_plan_name' ) :
				IF ( $pmeta['_mbr_oitem_status'] == 1 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_live_regular_plan'] ) ) :
						return $this->stripe_plans['live'][ $pmeta['_mbr_oitem_live_regular_plan'] ]['name'];
					ENDIF;
				ELSEIF ( $pmeta['_mbr_oitem_status'] == 2 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_test_regular_plan'] ) ) :
						return $this->stripe_plans['test'][ $pmeta['_mbr_oitem_test_regular_plan'] ]['name'];
					ENDIF;
				ENDIF;
			ENDIF;

			IF ( $scname == 'mbr_product_sale_plan_name' ) :
				IF ( $pmeta['_mbr_oitem_status'] == 1 ) :
					IF ( ! empty($pmeta['_mbr_oitem_live_sale_plan'] ) ) :
						return $this->stripe_plans['live'][ $pmeta['_mbr_oitem_live_sale_plan'] ]['name'];
					ENDIF;
				ELSEIF ( $pmeta['_mbr_oitem_status'] == 2 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_test_sale_plan'] ) ) :
						return $this->stripe_plans['test'][ $pmeta['_mbr_oitem_test_sale_plan'] ]['name'];
					ENDIF;
				ENDIF;
			ENDIF;

			IF ( $scname == 'mbr_product_regular_price' ) :
				IF ( $pmeta['_mbr_oitem_status'] == 1 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_live_regular_plan'] ) ) :
						$amount =  $this->stripe_plans['live'][ $pmeta['_mbr_oitem_live_regular_plan'] ]['amount'];
					ENDIF;
				ELSEIF ( $pmeta['_mbr_oitem_status'] == 2 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_test_regular_plan'] ) ) :
						$amount =  $this->stripe_plans['test'][ $pmeta['_mbr_oitem_test_regular_plan'] ]['amount'];
					ENDIF;
				ENDIF;
				return $this->stripe_price_format( MBR_PAYMENT_CURRENCY, $amount );
			ENDIF;

			IF ( $scname == 'mbr_product_sale_price' ) :
				IF ( $pmeta['_mbr_oitem_status'] == 1 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_live_sale_plan'] ) ) :
						$amount =  $this->stripe_plans['live'][ $pmeta['_mbr_oitem_live_sale_plan'] ]['amount'];
					ENDIF;
				ELSEIF ( $pmeta['_mbr_oitem_status'] == 2 ) :
					IF ( ! empty( $pmeta['_mbr_oitem_test_sale_plan'] ) ) :
						$amount =  $this->stripe_plans['test'][ $pmeta['_mbr_oitem_test_sale_plan'] ]['amount'];
					ENDIF;
				ENDIF;
				IF ( ! empty( $amount ) ) :
					return $this->stripe_price_format( MBR_PAYMENT_CURRENCY, $amount );
				ENDIF;
			ENDIF;
		ENDIF;
	}

	public function sc_stripe_one_click_upsell( $parms, $content = NULL )
	{
		IF ( ! $this->payment['stripe_status'] ) :
			return __( 'Stripe Payment is not active', 'activemember360' );
		ENDIF;

		IF ( ! apply_filters( 'mbr/contact_id', NULL ) ) :
			return;
		ENDIF;

		$parms = apply_filters( 'mbr/shortcodes/atts',
			[
				'product_id'     => FALSE,
				'fallback_url'   => FALSE,
				'button_text'    => __( 'Purchase now!', 'activemember360' ),
				'button_image'   => '',
				'button_style'   => '',
			], $parms );

		IF ( empty( $parms['product_id'] ) ) :
			return __( 'No product ID provided', 'activemember360' );
		ENDIF;

		$product = get_post($parms['product_id']);

		IF ( empty( $product ) OR ( $product->post_type <> MBRCUSTPRODUCTS AND $product->post_type <> MBRCUSTSUBSCRIPTIONS ) ) :
			return __( 'Product does not exist.', 'activemember360' );
		ENDIF;

		IF ( empty( $parms['fallback_url'] ) ) :
			$fallback_url = get_permalink( $parms['product_id'] );
		ELSE:
			$fallback_url = $parms['fallback_url'];
		ENDIF;

		IF ( $product->post_type == MBRCUSTPRODUCTS ) :
			$settings_key = MBR_PRODUCT_SETTINGS;
		ELSE:
			$settings_key = MBR_SUBSCRIPTION_SETTINGS;
		ENDIF;

		$pmeta   = redux_post_meta( $settings_key, $product->ID );

		IF ( empty( $pmeta['_mbr_oitem_status'] ) ) :
			return __( 'Product is inactive', 'activemember360' );
		ENDIF;

		IF ( ! empty( $_GET['payment'] ) AND $_GET['payment'] == 'paid' ) :
			$out    = "<div class='mbr-payment-success'>";
			$out   .= "<h2>" . __( 'Order received', 'activemember360' ) . "</h2>";
			$out   .= "<p>"  . __( 'Thank you, your order has been received.', 'activemember360' ) . "</p>";
			IF (!empty($_GET['order_id'])) :
				$out .= $this->order_details_template( $_GET['order_id'] );
			ENDIF;
			$out   .= "</div>";
			return $out;
		ENDIF;

		IF ( ! empty( $_GET['payment']) AND $_GET['payment'] == 'trialing' ) :
			$out  = "<div class='mbr-payment-success'>";
			$out .= "<h2>" . __( 'Order received', 'activemember360' ) . "</h2>";
			$out .= "<p>"  . __( 'Thank you, your order has been received and you will be charged for the first time after your trial period.', 'activemember360' ) . "</p>";
			IF ( ! empty( $_GET['order_id'] ) ) :
				$out .= $this->order_details_template( $_GET['order_id'] );
			ENDIF;
			$out .= "</div>";
			return $out;
		ENDIF;

		$out           = '';
		IF ( ! empty( $_REQUEST['payment_error'] ) ) :
			IF ( is_array( $_REQUEST['payment_error'] ) ) :
				$arr_error = implode( '<br />', $_REQUEST['payment_error'] );
				$error     = $arr_error;
			ELSE:
				$error     = urldecode( $_REQUEST['payment_error'] );
			ENDIF;
			$out        .= "<div class='mbr-payment-error'><p>{$error}</p></div>";
		ENDIF;

		$out .= '<form method="post" action=""">';
		$out .= '<input type="hidden" name="action"        value="stripe" />';
		$out .= '<input type="hidden" name="product_id"    value="' . $product->ID . '" />';
		$out .= '<input type="hidden" name="redirect"      value="' . get_permalink() . '" />';
		$out .= '<input type="hidden" name="stripe_nonce"  value="' . wp_create_nonce('stripe-nonce') . '" />';
		$out .= '<input type="hidden" name="oc_upsell"     value="yes" />';
		$out .= '<input type="hidden" name="fallback_url"  value="' . $fallback_url . '" />';

		$b_style   = '';
		IF ( ! empty( $parms['button_style'] ) ) :
			$b_style = 'style="' . $parms['button_style'] . '"';
		ENDIF;
		IF ( empty( $parms['button_image'] ) ) :
			$out .= '<input type="submit" name="mbr_purchase" id="mbr-submit" ' . $b_style . ' value="' . $parms['button_text'] . '" />';
		ELSE:
			$out .= '<input type="image" name="mbr_purchase" src="' . $parms['button_image'] . '" ' . $b_style . ' alt="' . $parms['button_text'] . '" />';
		ENDIF;
		$out .= '</form>';

		return $out;
	}

	public function order_details_template( $order_id = FALSE )
	{
		IF ( empty( $order_id ) ) :
			return '';
		ENDIF;

		$arr_order_details = $this->get_order_details( $order_id );
		$out           = '';
		IF ( ! empty( $arr_order_details ) ) :
			$out        .= "<h2>" . __( 'Order details', 'activemember360' ) . '</h2>';
			$out        .= "<table>";
			$out        .= "<tr><th><strong>" . __( 'Product', 'activemember360' ) . "</strong></th><th><strong>" . __( 'Total', 'activemember360' ) . "</strong></th></tr>";
			$total       = 0;
			FOREACH ( $arr_order_details['order_items'] AS $key => $order_item ) :
				$post_item = get_post( $order_item['id'] );
				$out      .= "<tr><td>" . $post_item->post_title. " x " . $order_item['qty'] . "</td><td>" . $this->stripe_price_format( $arr_order_details['currency'], $order_item['amount'] ) . "</td></tr>";
				$total     = $total + $order_item['amount'];
			ENDFOREACH;
			$out        .= "<tr><td>" . __( 'Total:', 'activemember360' ) . "</td><td>" . $this->stripe_price_format( $arr_order_details['currency'], $total ) . "</td></tr>";
			$out        .= "</table>";
			IF ( is_user_logged_in() ) :
				$out      .= "<h2>"     . __( 'Customer details', 'activemember360' ) . '</h2>';
				$out      .= "<table>";
				$out      .= "<tr><td>" . __( 'Email:', 'activemember360' ) . "</td><td>" . $arr_order_details['billing_details']['email'] . "</td></tr>";
				$out      .= "<tr><td>" . __( 'Phone:', 'activemember360' ) . "</td><td>" . $arr_order_details['billing_details']['phone'] . "</td></tr>";
				$out      .= "</table>";

				$arr_us_states = apply_filters( 'mbr/payments/states/us/get', NULL );
				$arr_countries = apply_filters( 'mbr/payments/countries/get', NULL );

				$out     .= "<h2>" . __( 'Billing address', 'activemember360' ) . '</h2>';
				$out     .= $arr_order_details['billing_details']['first_name'] . ' ' . $arr_order_details['billing_details']['last_name'];
				IF ( ! empty($arr_order_details['billing_details']['billing_company'] ) ) :
					$out   .= '<br />' . $arr_order_details['billing_details']['billing_company'];
				ENDIF;
				$out     .= '<br />' . $arr_order_details['billing_details']['billing_address_1'];
				IF ( ! empty( $arr_order_details['billing_details']['billing_address_2'] ) ) :
					$out   .= '<br />' . $arr_order_details['billing_details']['billing_address_2'];
				ENDIF;
				$out     .= '<br />' . $arr_order_details['billing_details']['billing_postcode'] . ' ' . $arr_order_details['billing_details']['billing_city'];
				IF ( ! empty( $arr_order_details['billing_details']['billing_state']) AND $arr_order_details['billing_details']['billing_country'] == 'US' ) :
					$out   .= '<br />' . $arr_us_states[ $arr_order_details['billing_details']['billing_state'] ];
				ENDIF;
				$out     .= '<br />' . $arr_countries[ $arr_order_details['billing_details']['billing_country'] ];
			ENDIF;
		ENDIF;

		return $out;
	}

	public function get_order_details( $order_id = FALSE )
	{
		IF ( empty( $order_id ) ) :
			return FALSE;
		ENDIF;

		$arr_order_details['billing_details'] = get_post_meta( $order_id, $this->order_billing_details_pm_key, TRUE );
		$arr_order_details['order_items']     = get_post_meta( $order_id, $this->order_items_pm_key,           TRUE );
		$arr_order_details['amount']          = get_post_meta( $order_id, $this->order_amount_pm_key ,         TRUE );
		$arr_order_details['currency']        = get_post_meta( $order_id, $this->order_currency_pm_key ,       TRUE );
		$arr_order_details['country_tax']     = get_post_meta( $order_id, $this->order_country_tax_pm_key ,    TRUE );
		$arr_order_details['state_tax']       = get_post_meta( $order_id, $this->order_state_tax_pm_key ,      TRUE );

		return $arr_order_details;
	}

	public function register_post_types()
	{
		$this->register_post_type_product();
		$this->register_post_type_subscription();
		$this->register_post_type_order();
		$this->register_post_type_checkout_template();
		flush_rewrite_rules();
	}

	public function register_post_type_product()
	{
		$product_permalink = empty( $permalinks['mbr_product_base'] ) ? _x( 'mbr-product', 'slug', 'activemember360' ) : $permalinks['mbr_product_base'];

		$args = [
			'labels'  => [
				'name'                   => 'Products'
				,'singular_name'         => 'Product'
				,'add_new'               => 'Add Product'
				,'add_new_item'          => 'Add a new product'
				,'edit_item'             => 'Edit Product'
				,'new_item'              => 'New Product'
				,'view'                  => FALSE
				,'view_item'             => 'View Product'
				,'search_items'          => 'Search Product'
				,'not_found'             => 'No matching products found'
				,'not_found_in_trash'    => 'No products found in trash'
				,'featured_image'        => 'Product image'
				,'set_featured_image'    => 'Set product image'
				,'remove_featured_image' => 'Remove product image'
				,'use_featured_image'    => 'Use as product image'
			]
			,'rewrite'             => $product_permalink ? [ 'slug' => untrailingslashit( $product_permalink ), 'with_front' => TRUE, 'feeds' => TRUE ] : FALSE
			,'description'         => 'Products for ' . apply_filters( 'mbr/plugin_name', NULL )
			,'public'              => TRUE
			,'exclude_from_search' => FALSE
			,'publicly_queryable'  => TRUE
			,'show_ui'             => current_user_can( 'activate_plugins' )
			,'show_in_nav_menus'   => TRUE
			,'show_in_menu'        => MBR_MENU_PAYMENT_PAGE
			,'show_in_rest'        => TRUE
			,'menu_position'       => 10
			,'show_in_admin_bar'   => current_user_can( 'activate_plugins' )
			,'capability_type'     => 'post'
			,'can_export'          => TRUE
			,'supports'            => [ 'revisions', 'title', 'editor', 'excerpt', 'thumbnail' ]
		];

		register_post_type( MBRCUSTPRODUCTS, $args );
	}

	public function register_post_type_subscription()
	{
		$product_permalink = empty( $permalinks['mbr_subscription_base'] ) ? _x( 'mbr-subscription', 'slug', 'activemember360' ) : $permalinks['mbr_subscription_base'];

		$args = [
			'labels'  => [
				'name'                   => 'Subscriptions'
				,'singular_name'         => 'Subscription'
				,'add_new'               => 'Add Subscription'
				,'add_new_item'          => 'Add a new subscription'
				,'edit_item'             => 'Edit Subscription'
				,'new_item'              => 'New Subscription'
				,'view'                  => FALSE
				,'view_item'             => 'View Subscription'
				,'search_items'          => 'Search Subscription'
				,'not_found'             => 'No matching subscriptions found'
				,'not_found_in_trash'    => 'No subscriptions found in trash'
				,'featured_image'        => 'Subscription image'
				,'set_featured_image'    => 'Set subscription image'
				,'remove_featured_image' => 'Remove subscription image'
				,'use_featured_image'    => 'Use as subscription image'
			]
			,'rewrite'             => $product_permalink ? [ 'slug' => untrailingslashit( $product_permalink ), 'with_front' => FALSE, 'feeds' => TRUE ] : FALSE
			,'description'         => 'Subscriptions for ' . apply_filters( 'mbr/plugin_name', NULL )
			,'public'              => TRUE
			,'exclude_from_search' => FALSE
			,'publicly_queryable'  => TRUE
			,'show_ui'             => current_user_can( 'activate_plugins' )
			,'show_in_nav_menus'   => TRUE
			,'show_in_menu'        => MBR_MENU_PAYMENT_PAGE
			,'show_in_rest'        => TRUE
			,'menu_position'       => 10
			,'show_in_admin_bar'   => current_user_can( 'activate_plugins ')
			,'capability_type'     => 'post'
			,'can_export'          => TRUE
			,'supports'            => [ 'revisions', 'title', 'editor', 'excerpt', 'thumbnail' ]
		];

		register_post_type( MBRCUSTSUBSCRIPTIONS, $args );
	}

	public function register_post_type_checkout_template()
	{
		$args = [
			'labels'  => [
				'name'                   => 'Checkout Templates'
				,'singular_name'         => 'Checkout Template'
				,'add_new'               => 'Add Checkout Template'
				,'add_new_item'          => 'Add a new checkout template'
				,'edit_item'             => 'View checkout template'
				,'new_item'              => 'New checkout template'
				,'view'                  => FALSE
				,'view_item'             => 'View Checkout Template'
				,'search_items'          => 'Search Checkout Template'
				,'not_found'             => 'No matching checkout templates found'
				,'not_found_in_trash'    => 'No checkout templates found in trash'
			]
			,'rewrite'             => [ 'slug' => FALSE, 'with_front' => FALSE ]
			,'description'         => 'Checkout templates for ' . apply_filters( 'mbr/plugin_name', NULL )
			,'public'              => FALSE
			,'exclude_from_search' => TRUE
			,'publicly_queryable'  => FALSE
			,'show_ui'             => current_user_can( 'activate_plugins' )
			,'show_in_nav_menus'   => FALSE
			,'show_in_menu'        => MBR_MENU_PAYMENT_PAGE
			,'show_in_rest'        => TRUE
			,'menu_position'       => 10
			,'show_in_admin_bar'   => current_user_can( 'activate_plugins' )
			,'capability_type'     => 'post'
			,'can_export'          => TRUE
			,'supports'            => ['title']
		];
		register_post_type( MBRCUSTCHECKOUTTPL, $args );
	}

	public function register_post_type_order()
	{
		$args = [
			'labels'  => [
				'name'                   => 'Orders'
				,'singular_name'         => 'Order'
				,'add_new'               => 'Add Order'
				,'add_new_item'          => 'Add a new order'
				,'edit_item'             => 'View Order'
				,'new_item'              => 'New Order'
				,'view'                  => FALSE
				,'view_item'             => 'View Order'
				,'search_items'          => 'Search Order'
				,'not_found'             => 'No matching orders found'
				,'not_found_in_trash'    => 'No orders found in trash'
			]
			,'rewrite'             => [ 'slug' => FALSE, 'with_front' => FALSE ]
			,'description'         => 'Orders for ' . apply_filters( 'mbr/plugin_name', NULL )
			,'public'              => FALSE
			,'exclude_from_search' => TRUE
			,'publicly_queryable'  => FALSE

			,'show_ui'             => current_user_can( 'activate_plugins' )
			,'show_in_nav_menus'   => FALSE
			,'show_in_menu'        => MBR_MENU_PAYMENT_PAGE
			,'show_in_rest'        => TRUE
			,'menu_position'       => 10
			,'show_in_admin_bar'   => current_user_can( 'activate_plugins' )
			,'capability_type'     => 'post'
			,'capabilities'        => [ 'create_posts' => 'do_not_allow' ] // removes the "add order" button
			,'map_meta_cap'        => TRUE // needed to be able to view the orders without the "add order" button
			,'can_export'          => TRUE
			,'supports'            => FALSE
		];

		register_post_type( MBRCUSTORDERS, $args );
	}

	public function order_remove_metaboxes()
	{
		remove_meta_box( 'submitdiv', MBRCUSTORDERS, 'side' );
		remove_meta_box( 'submitdiv', MBRCUSTORDERS, 'normal' );
		remove_meta_box( 'submitdiv', MBRCUSTORDERS, 'advanced' );
		remove_meta_box( 'slugdiv',   MBRCUSTORDERS, 'side' );
		remove_meta_box( 'slugdiv',   MBRCUSTORDERS, 'normal' );
		remove_meta_box( 'slugdiv',   MBRCUSTORDERS, 'advanced' );
	}

	public function order_add_metaboxes()
	{
		global $post;

		$payment_method  = get_post_meta( $post->ID, $this->order_payment_method_pm_key, TRUE );
		$payment_mode    = $payment_method['mode'];
		$subscription_id = get_post_meta( $post->ID, $this->stripe_subscription_id_pm_key, TRUE );

		IF ( ! $subscription_id ) :
			$ref_id        = get_post_meta( $post->ID, $this->stripe_charge_id_pm_key, TRUE );
			add_meta_box( 'mbr_order_details', 'Order <a href="https://dashboard.stripe.com/' . $payment_mode . '/payments/'      . $ref_id . '" target="_blank">' . sprintf( _x( '#%s', 'wp_admin', 'activemember360' ), $post->ID ) . '</a>', [ $this, 'order_render_metabox_order_details' ],  MBRCUSTORDERS, 'normal', 'default' );
		ELSE:
			$ref_id        = get_post_meta( $post->ID, $this->stripe_subscription_id_pm_key, TRUE );
			add_meta_box( 'mbr_order_details', 'Order <a href="https://dashboard.stripe.com/' . $payment_mode . '/subscriptions/' . $ref_id . '" target="_blank">' . sprintf( _x( '#%s', 'wp_admin', 'activemember360' ), $post->ID ) . '</a>', [ $this, 'order_render_metabox_order_details' ],  MBRCUSTORDERS, 'normal', 'default' );
		ENDIF;

		add_meta_box( 'mbr_order_payment_status', _x( 'Payment Status', 'wp_admin', 'activemember360'), [ $this, 'order_render_metabox_payment_status' ], MBRCUSTORDERS, 'side',   'default' );
		add_meta_box( 'mbr_order_order_items',    _x( 'Order Items',    'wp_admin', 'activemember360'), [ $this, 'order_render_metabox_order_items' ],    MBRCUSTORDERS, 'normal', 'default' );
		add_meta_box( 'mbr_order_charge_history', _x( 'Charge History', 'wp_admin', 'activemember360'), [ $this, 'order_render_metabox_charge_history' ], MBRCUSTORDERS, 'normal', 'default' );
		add_meta_box( 'mbr_order_refund_history', _x( 'Refund History', 'wp_admin', 'activemember360'), [ $this, 'order_render_metabox_refund_history' ], MBRCUSTORDERS, 'normal', 'default' );
	}

	public function order_render_metabox_order_details()
	{
		global $post;

		$payment_method        = get_post_meta( $post->ID, $this->order_payment_method_pm_key,  TRUE );
		$payment_type          = $payment_method['type'];
		$payment_mode          = $payment_method['mode'];
		$stripe_customer_id    = get_post_meta( $post->ID, $this->stripe_customer_id_pm_key,    TRUE );
		$order_customer_email  = get_post_meta( $post->ID, $this->order_customer_email_pm_key,  TRUE );
		$order_amount          = get_post_meta( $post->ID, $this->order_amount_pm_key,          TRUE );
		$order_currency        = get_post_meta( $post->ID, $this->order_currency_pm_key,        TRUE );
		$billing_details       = get_post_meta( $post->ID, $this->order_billing_details_pm_key, TRUE );

		echo '<div style="float:left;width:45%">';
		echo '<strong>' . _x( 'Order Details', 'wp_admin', 'activemember360' ) . '</strong>';
		echo '<table>';
		echo '<tr><td style="width:200px;">' . _x( 'Price', 'wp_admin', 'activemember360') . ':</td><td>' . $this->stripe_price_format( $order_currency, $order_amount )        . '</td></tr>';
		echo '<tr><td>' . _x( 'Date',                  'wp_admin', 'activemember360' )     . ':</td><td>' . apply_filters( 'mbr/datetime/convert', $post->post_date_gmt, TRUE ) . '</td></tr>';
		echo '<tr><td>' . _x( 'Payment Type',          'wp_admin', 'activemember360' )     . ':</td><td>' . $payment_type         . '</td></tr>';
		echo '<tr><td>' . _x( 'Payment Mode',          'wp_admin', 'activemember360' )     . ':</td><td>' . $payment_mode         . '</td></tr>';
		echo '<tr><td>' . _x( 'Stripe Customer',       'wp_admin', 'activemember360' )     . ':</td><td><a href="https://dashboard.stripe.com/' . $payment_mode . '/customers/' . $stripe_customer_id . '" target="_blank">' . $billing_details['first_name'] . ' ' . $billing_details['last_name'] . '</td></tr>';
		echo '<tr><td>' . _x( 'Stripe Customer Email', 'wp_admin', 'activemember360' )     . ':</td><td>' . $order_customer_email . '</td></tr>';
		echo '</table>';
		echo '</div>';

		$us_states = apply_filters( 'mbr/payments/states/us/get', NULL );
		$countries = apply_filters( 'mbr/payments/countries/get', NULL );

		echo '<div style="float:right;width:45%">';
		echo '<strong>' . _x( 'Billing Details', 'wp_admin', 'activemember360') . '</strong>';
		echo '<table>';
		echo '<tr><td style="width:200px;">' . _x( 'First Name',  'wp_admin', 'activemember360') . ':</td><td>' . $billing_details['first_name'] . '</td></tr>';
		echo '<tr><td>' . _x( 'Last Name',   'wp_admin', 'activemember360' ) . ':</td><td>' . $billing_details['last_name']         . '</td></tr>';
		echo '<tr><td>' . _x( 'Company',     'wp_admin', 'activemember360' ) . ':</td><td>' . $billing_details['billing_company']   . '</td></tr>';
		echo '<tr><td>' . _x( 'Address 1',   'wp_admin', 'activemember360' ) . ':</td><td>' . $billing_details['billing_address_1'] . '</td></tr>';
		echo '<tr><td>' . _x( 'Address 2',   'wp_admin', 'activemember360' ) . ':</td><td>' . $billing_details['billing_address_2'] . '</td></tr>';
		echo '<tr><td>' . _x( 'City',        'wp_admin', 'activemember360' ) . ':</td><td>' . $billing_details['billing_city']      . '</td></tr>';
		echo '<tr><td>' . _x( 'Postal Code', 'wp_admin', 'activemember360' ) . ':</td><td>' . $billing_details['billing_postcode']  . '</td></tr>';
		echo '<tr><td>' . _x( 'State',       'wp_admin', 'activemember360' ) . ':</td><td>' . $us_states[ $billing_details['billing_state'] ]     . '</td></tr>';
		echo '<tr><td>' . _x( 'Country',     'wp_admin', 'activemember360' ) . ':</td><td>' . $countries[ $billing_details['billing_country'] ]   . '</td></tr>';
		echo '</table>';
		echo '</div>';

		echo '<div style="clear:both;"><!-- clear --></div>';
	}

	public function order_render_metabox_order_items()
	{
		global $post;

		$order_currency        = get_post_meta( $post->ID, $this->order_currency_pm_key, TRUE );
		$arr_order_items       = get_post_meta( $post->ID, $this->order_items_pm_key,    TRUE );

		IF ( ! empty( $arr_order_items ) ) :
			$cnt_order_items = count( $arr_order_items );
			$counter         = 0;

			FOREACH ( $arr_order_items AS $key => $arr_order_item ) :
				$counter++;
				echo '<table>';
				echo '<tr><td style="width:200px;">' . _x( 'Product ID', 'wp_admin', 'activemember360' ) . ':</td><td>' . $arr_order_item['id'] . '</td></tr>';
				echo '<tr><td>'   . _x( 'Product Name', 'wp_admin', 'activemember360' ) . ':</td><td>' . $this->get_product_name_by_id( $arr_order_item['id'], TRUE )  . '</td></tr>';
				echo '<tr><td>'   . _x( 'Single Price', 'wp_admin', 'activemember360' ) . ':</td><td>' . $this->stripe_price_format( $order_currency, $arr_order_item['amount'] ) . '</td></tr>';
				echo '<tr><td>'   . _x( 'Quantity',     'wp_admin', 'activemember360' ) . ':</td><td>' . $arr_order_item['qty']  . '</td></tr>';
				echo '<tr><td>'   . _x( 'Product Type', 'wp_admin', 'activemember360' ) . ':</td><td>' . $arr_order_item['type'] . '</td></tr>';
				IF ( $arr_order_item['type'] == 'subscription' ) :
					echo '<tr><td>' . _x( 'Interval',  'wp_admin', 'activemember360' )    . ':</td><td>'  . $this->text_subscription_interval( $arr_order_item['interval_count'], $arr_order_item['interval'] ) . '</td></tr>';
				ENDIF;
				echo '</table>';

				IF ( $counter <> $cnt_order_items ) :
					echo '<hr />';
				ENDIF;

			ENDFOREACH;
		ELSE:
			echo _x( 'No order items has been found.', 'wp_admin', 'activemember360' );
		ENDIF;
	}

	public function order_render_metabox_payment_status()
	{
		global $post;

		echo sprintf( _x( 'Payment Status: %s' , 'activemember360' ), $this->order_payment_statuses[ $post->post_status ]['label_regular'] );
	}

	public function order_render_metabox_charge_history()
	{
		global $post;

		$arr_charges              = get_post_meta( $post->ID, $this->stripe_charges_pm_key, TRUE );
		IF ( ! empty( $arr_charges) AND is_array( $arr_charges ) ) :
			$arr_invoice_ids = $this->get_invoices( $post->ID );
			echo '<table>';
			FOREACH ( $arr_charges AS $charge_id => $arr_charge ) :
				$invoice          = $this->jquery_append_body_form( home_url( '/mbr-invoice/' . $post->ID ), $charge_id, _x( 'Invoice', 'wp_admin', 'activemember360' ) );
				$refund_receipt   = '';
				$arr_refunds      = get_post_meta( $post->ID, $this->stripe_refunds_pm_key, TRUE );
				IF ( ! empty( $arr_refunds[ $charge_id ] ) ) :
					$refund_receipt = ' | ' . $this->jquery_append_body_form( home_url( '/mbr-refund/' . $post->ID ), $charge_id, _x( 'Refund Receipt', 'wp_admin', 'activemember360' ) );
				ENDIF;
				IF ( ! empty( $arr_invoice_ids[ $charge_id ] ) ) :
					$invoice_id = $arr_invoice_ids[ $charge_id ];
					echo '<tr><td style="width:200px;">' . apply_filters( 'mbr/datetime/convert', $arr_charge['created'], TRUE ) . ':</td><td>' . sprintf( _x( 'Invoice #%s', 'activemember360' ), $invoice_id ) . ' ' . $this->stripe_price_format( $arr_charge['currency'], $arr_charge['amount'] ) . '</td><td><strong>' . ( $arr_charge['paid'] == 1 ? '<span style="color:green;">' . _x( 'Paid', 'wp_admin', 'activemember360' ) . '</span>' : '<span style="color:red;">' . _x( 'Not paid', 'wp_admin', 'activemember360' ) . '</span>' ) . '</strong></td><td>' . $invoice . $refund_receipt . '</td></tr>';
				ELSE:
					echo '<tr><td style="width:200px;">' . apply_filters( 'mbr/datetime/convert', $arr_charge['created'], TRUE ) . ':</td><td>' . $this->stripe_price_format( $arr_charge['currency'], $arr_charge['amount'] ) . '</td><td colspan="2"><strong>' . ( $arr_charge['paid'] == 1 ? '<span style="color:green;">' . _x( 'Paid', 'wp_admin', 'activemember360' ) . '</span>' : '<span style="color:red;">' . _x( 'Not paid', 'wp_admin', 'activemember360' ) . '</span>') . '</strong></td></tr>';
				ENDIF;
			ENDFOREACH;
			echo '</table>';
		ELSE:
			echo _x( 'No charges has been made yet.', 'wp_admin', 'activemember360' );
		ENDIF;
	}

	public function order_render_metabox_refund_history()
	{
		global $post;

		$arr_refunds              = get_post_meta( $post->ID, $this->stripe_refunds_pm_key, TRUE );
		$order_currency           = get_post_meta( $post->ID, $this->order_currency_pm_key, TRUE );

		IF ( ! empty( $arr_refunds) AND is_array( $arr_refunds ) ) :
			$total_amount_refunded   = 0;
			echo '<table>';
			FOREACH ( $arr_refunds AS $charge_id => $arr_refund ) :
				FOREACH ( $arr_refund AS $key => $refund ) :
					$total_amount_refunded = $total_amount_refunded + $refund['amount'];
					echo '<tr><td style="width:200px;">' . apply_filters( 'mbr/datetime/convert', $refund['created'], TRUE ) . ':</td><td>' . $this->stripe_price_format( $refund['currency'], $refund['amount'] ) . '</td></tr>';
				ENDFOREACH;
			ENDFOREACH;
			echo '</table>';
			echo '<hr />';
			echo '<table>';
			echo '<tr><td style="width:200px;">' . _x( 'Total refund:', 'wp_admin', 'activemember360' ) . '</td><td>' . $this->stripe_price_format( $order_currency, $total_amount_refunded ) . '</td></tr>';
			echo '</table>';
		ELSE:
			echo _x( 'No refunds has been issued yet.', 'wp_admin', 'activemember360' );
		ENDIF;
	}

	public function order_custom_post_status()
	{
		FOREACH ( $this->order_payment_statuses AS $post_status => $arr_post_status ) :
			register_post_status( $post_status,
				[
					'label'                     => $arr_post_status['label_overview'],
					'public'                    => TRUE,
					'exclude_from_search'       => FALSE,
					'show_in_admin_status_list' => TRUE,
					'label_count'               => _n_noop( $arr_post_status['label_overview'] . ' <span class="count">(%s)</span>', $arr_post_status['label_overview'] . ' <span class="count">(%s)</span>' ),
				]
			);
		ENDFOREACH;
	}

	public function order_add_columns( $columns )
	{
		$old_cb                    = $columns['cb'];
		$columns                   = [];
		$columns['cb']             = $old_cb;
		$columns['title']          = _x( 'Order ID',    'wp_admin', 'activemember360' );
		$columns['customer_email'] = _x( 'Email',       'wp_admin', 'activemember360' );
		$columns['post_date']      = _x( 'Order Date',  'wp_admin', 'activemember360' );
		$columns['post_modified']  = _x( 'Last Change', 'wp_admin', 'activemember360' );
		$columns['payment_type']   = _x( 'Type',        'wp_admin', 'activemember360' );
		$columns['total']          = _x( 'Total',       'wp_admin', 'activemember360' );
		$columns['status']         = _x( 'Status',      'wp_admin', 'activemember360' );

		return $columns;
	}

	public function order_render_columns( $column, $post_id )
	{
		SWITCH ( $column ) :
			CASE 'customer_email':
				echo get_post_meta( $post_id, $this->order_customer_email_pm_key, TRUE );
				break;
			CASE 'post_date':
				echo apply_filters( 'mbr/datetime/convert', get_post_field( 'post_date_gmt',     $post_id ), FALSE );
				break;
			CASE 'post_modified':
				echo apply_filters( 'mbr/datetime/convert', get_post_field( 'post_modified_gmt', $post_id ), FALSE );
				break;
			CASE 'product':
				$product_id     = get_post_meta( $post_id, $this->order_product_id_pm_key, TRUE );
				$product_name   = $this->get_product_name_by_id( $product_id, TRUE );
				echo $product_name;
				break;
			CASE 'payment_type':
				$payment_method = get_post_meta( $post_id, $this->order_payment_method_pm_key, TRUE );
				echo $payment_method['type'] . ' (' . $payment_method['mode'] . ')';
				break;
			CASE 'total':
				$order_amount   = get_post_meta( $post_id, $this->order_amount_pm_key,   TRUE );
				$order_currency = get_post_meta( $post_id, $this->order_currency_pm_key, TRUE );
				echo $this->stripe_price_format( $order_currency, $order_amount );
				break;
			CASE 'status':
				$post_status    = get_post_field( 'post_status', $post_id );
				echo $this->order_payment_statuses[ $post_status ]['label_regular'];
				break;
		ENDSWITCH;
	}

	public function order_sortable_columns_orderby( $vars )
	{
		IF ( empty( $vars['orderby'] ) ) :
			return $vars;
		ENDIF;

		IF ( 'total' == $vars['orderby'] ) :
			$vars = array_merge( $vars,
				[
					'meta_key' => $this->order_amount_pm_key,
					'orderby'  => 'meta_value_num'
				]
			);
		ELSEIF ( 'customer_email' == $vars['orderby'] ) :
			$vars = array_merge( $vars,
				[
					'meta_key' => $this->order_customer_email_pm_key,
					'orderby'  => 'meta_value'
				]
			);
		ELSEIF ( 'status' == $vars['orderby'] ) :
			$vars = array_merge( $vars,
				[
					'meta_key' => $this->order_post_status_pm_key,
					'orderby'  => 'meta_value'
				]
			);
		ENDIF;

		return $vars;
	}

	public function order_sortable_columns( $columns )
	{
		$columns['customer_email'] = 'customer_email';
		$columns['post_date']      = 'post_date';
		$columns['post_modified']  = 'post_modified';
		$columns['total']          = 'total';
		$columns['status']         = 'status';

		return $columns;
	}

	public function order_remove_quick_edit( $actions )
	{
		$screen = get_current_screen();
		IF ( $screen->post_type == MBRCUSTORDERS ) :
			unset( $actions['inline hide-if-no-js'] );
		ENDIF;

		return $actions;
	}

	public function order_change_title_in_list()
	{
		$screen = get_current_screen();
		IF ( $screen->post_type == MBRCUSTORDERS ) :
			add_filter( 'the_title', [ $this, 'order_filter_title_in_list' ], 100, 2 );
		ENDIF;
	}

	public function order_filter_title_in_list( $title, $order_id )
	{
		IF ( ! empty( $order_id ) ) :
			//$stripe_customer_id    = get_post_meta($order_id, $this->stripe_customer_id_pm_key,    TRUE);
			$order_customer_email = get_post_meta( $order_id, $this->order_customer_email_pm_key, TRUE );
			// return sprintf(__('#%s by %s (%s)', 'activemember360'), $order_id, $order_customer_email, $stripe_customer_id);
			return  $order_id;
		ENDIF;

		return $title;
	}

	public function order_list_posts_join( $join )
	{
		global $pagenow, $wpdb;

		// I want the filter only when performing a search on edit page of Custom Post Type named MBRCUSTORDERS
		IF ( is_admin() AND $pagenow == 'edit.php' AND isset( $_GET['post_type'] ) AND $_GET['post_type'] == MBRCUSTORDERS AND ! empty( $_GET['s'] ) ) :
			$join .= 'LEFT JOIN ' . $wpdb->postmeta . ' ON '. $wpdb->posts . '.ID = ' . $wpdb->postmeta . '.post_id ';
		ENDIF;

		return $join;
	}

	public function order_list_posts_where( $where )
	{
		global $pagenow, $wpdb;

		// I want the filter only when performing a search on edit page of Custom Post Type named MBRCUSTORDERS
		IF ( is_admin() AND $pagenow == 'edit.php' AND isset( $_GET['post_type'] ) AND $_GET['post_type'] == MBRCUSTORDERS AND ! empty( $_GET['s'] ) ) :
			$where = preg_replace( "/\(\s*" . $wpdb->posts . ".post_title\s+LIKE\s*(\'[^\']+\')\s*\)/", "(" . $wpdb->posts . ".post_title LIKE $1) OR (" . $wpdb->posts . ".ID LIKE $1) OR (" . $wpdb->posts . ".post_modified LIKE $1) OR (" . $wpdb->postmeta . ".meta_value LIKE $1)", $where );
		ENDIF;

		return $where;
	}

	public function order_list_posts_distinct( $where )
	{
		global $pagenow;

		IF ( is_admin() AND $pagenow == 'edit.php' AND isset( $_GET['post_type'] ) AND $_GET['post_type'] == MBRCUSTORDERS AND ! empty( $_GET['s'] ) ) :
			return "DISTINCT";
		ENDIF;

		return $where;
	}

	public function stripe_get_account_details()
	{
		TRY {
			$account     = Stripe\Account::retrieve();
			$arr_account = $account->__toArray( TRUE );
		} CATCH ( Exception $e ) {
			$error = mbrErrHdlr::create_error_std_object( 'STRIPE-COULD-NOT-GET-ACCOUNT-DETAILS', __LINE__, __FILE__ );
			mbrErrHdlr::mbr_error_exit( $error );
		}

		return $arr_account;
	}

	public function stripe_price_format( $currency, $amount )
	{
		$currency_position = $this->payment['currency_position'];
		$currency_symbol   = apply_filters( 'mbr/payments/currency/symbol/get', $currency );
		IF ( 'left_with_space' == $currency_position ) :
			return $currency_symbol . ' ' . $this->stripe_money_format( $amount );
		ELSEIF ( 'right_with_space' == $currency_position ) :
			return $this->stripe_money_format( $amount )   . ' ' . $currency_symbol;
		ELSEIF ( 'right' == $currency_position ) :
			return $this->stripe_money_format( $amount )   . $currency_symbol;
		ELSE:
			return $currency_symbol . $this->stripe_money_format( $amount );
		ENDIF;
	}

	public function stripe_money_format( $amount )
	{
		$tho_sep  = $this->payment['currency_thousand_sep'];
		$dec_sep  = $this->payment['currency_decimal_sep'];
		$decimals = $this->payment['currency_decimals'];
		$amount   = number_format( ( $amount/100 ), $decimals, $dec_sep, $tho_sep );

		return $amount;
	}

	public function stripe_get_product_data( $product_id = FALSE )
	{
		IF ( empty( $product_id ) ) :
			throw new Exception( 'NO-PRODUCT-ID' );
		ENDIF;
		$product     = get_post( $product_id );
		IF ( $product->post_type == MBRCUSTPRODUCTS ) :
			$settings_key = MBR_PRODUCT_SETTINGS;
		ELSE:
			$settings_key = MBR_SUBSCRIPTION_SETTINGS;
		ENDIF;
		$pmeta       = redux_post_meta( $settings_key, $product->ID );

		IF ( empty( $pmeta['_mbr_oitem_status'] ) ) :
			throw new Exception( 'INACTIVE-PRODUCT' );
		ENDIF;

		IF ( $pmeta['_mbr_oitem_status'] == 1 ) :
			$mode       = 'live';
			$secret_key = trim( $this->payment['stripe_live_secret'] );
		ELSE:
			$mode       = 'test';
			$secret_key = trim( $this->payment['stripe_test_secret'] );
		ENDIF;

		$amount  = 0;
		$plan_id = 0;
		IF ( $product->post_type == MBRCUSTPRODUCTS ) :
			IF ( $mode == 'live' ) :
				IF ( ! empty( $pmeta['_mbr_oitem_live_sale_price'] ) ) :
					$amount   = $pmeta['_mbr_oitem_live_sale_price'];
				ELSEIF ( ! empty( $pmeta['_mbr_oitem_live_regular_price'] ) ) :
					$amount   = $pmeta['_mbr_oitem_live_regular_price'];
				ENDIF;
			ELSE:
				IF ( ! empty( $pmeta['_mbr_oitem_test_sale_price'] ) ) :
					$amount   = $pmeta['_mbr_oitem_test_sale_price'];
				ELSEIF ( ! empty( $pmeta['_mbr_oitem_test_regular_price'] ) ) :
					$amount   = $pmeta['_mbr_oitem_test_regular_price'];
				ENDIF;
			ENDIF;
			$amount = apply_filters( 'mbr/payments/stripe/convert_amount', $amount );
			IF ( empty( $amount ) ) :
				throw new Exception( 'NO-AMOUNT-SPECIFIED' );
			ENDIF;
		ELSE :
			IF ( $mode == 'live' ) :
				IF ( ! empty( $pmeta['_mbr_oitem_live_sale_plan'] ) ) :
					$plan_id   = $pmeta['_mbr_oitem_live_sale_plan'];
				ELSEIF ( ! empty( $pmeta['_mbr_oitem_live_regular_plan'] ) ) :
					$plan_id   = $pmeta['_mbr_oitem_live_regular_plan'];
				ENDIF;
			ELSE:
				IF ( ! empty( $pmeta['_mbr_oitem_test_sale_plan'] ) ) :
					$plan_id   = $pmeta['_mbr_oitem_test_sale_plan'];
				ELSEIF ( ! empty( $pmeta['_mbr_oitem_test_regular_plan'] ) ) :
					$plan_id   = $pmeta['_mbr_oitem_test_regular_plan'];
				ENDIF;
			ENDIF;
		ENDIF;

		IF ( empty( $secret_key ) ) :
			throw new Exception( 'STRIPE-API-SECRET-KEY-MISSING' );
		ENDIF;

		return [ 'product_id'=> $product->ID, 'pmeta' => $pmeta, 'amount' => $amount, 'plan_id' => $plan_id, 'mode' => $mode, 'secret_key' => $secret_key ];
	}

	public function stripe_set_api_key( $key )
	{
		//$key = null;  // Used to trigger an error
		TRY {
			IF ( empty( $key ) ) :
				throw new Exception( 'STRIPE-API-SECRET-KEY-MISSING' );
			ENDIF;
			IF ( FALSE === stripos( $key, 'sk_live' ) AND FALSE === stripos( $key, 'sk_test' ) ) :
				throw new Exception( 'STRIPE-API-SECRET-KEY-WRONG-FORMAT' );
			ENDIF;
			IF ( FALSE !== stripos( $key, 'sk_live' ) ) :
				$this->stripe_operating_mode = 'live';
			ELSE:
				$this->stripe_operating_mode = 'test';
			ENDIF;
			\Stripe\Stripe::setApiKey( $key );
			\Stripe\Stripe::setApiVersion( '2017-08-15' );
		} CATCH ( Exception $e ) {
			$error = mbrErrHdlr::create_error_std_object( $e->getMessage(), __LINE__, __FILE__ );
			mbrErrHdlr::mbr_error_exit( $error );
		}

		return;
	}

	public function stripe_sync_customer_onetime( $cus_email, $card_token )
	{
		$new_customer    = FALSE;
		$customer_id     = FALSE;
		$new_card_id     = FALSE;

		$arr_account     = $this->stripe_get_account_details();
		$account_id      = $arr_account['id'];

		// get stored stripe customer id from AC
		IF ( apply_filters( 'mbr/contact_id', NULL ) ) :
			$tmp_contact        = apply_filters( 'mbr/session/get', NULL );
			$email              = $tmp_contact['email'];
			$arr_customer_ids   = json_decode( stripslashes( $tmp_contact[ MBR_STRIPE_CUST_ID_FIELD ] ), TRUE );
		ELSE:
			$email              = $cus_email;
			$tmp_contact        = apply_filters( 'mbr/get_remote_contact/by_email', [ 'email' => $email, 'local' => FALSE, 'sync' => FALSE ], FALSE );
			IF ( ! empty( $tmp_contact[ MBR_STRIPE_CUST_ID_FIELD ] ) ) :
				$arr_customer_ids = json_decode( stripslashes( $tmp_contact[ MBR_STRIPE_CUST_ID_FIELD ] ), TRUE );
			ENDIF;
		ENDIF;

		IF ( empty($arr_customer_ids ) ) :
			$arr_customer_ids = [];
		ENDIF;
		IF ( empty( $arr_customer_ids[ $account_id ] ) ) :
			$arr_customer_ids[ $account_id ] = [];
		ENDIF;
		IF ( empty( $arr_customer_ids[ $account_id ][ $this->stripe_operating_mode ] ) ) :
			$arr_customer_ids[ $account_id ][ $this->stripe_operating_mode ] = [];
		ENDIF;

		TRY {
			$customer     = \Stripe\Customer::create( [ 'card' => $card_token, 'email' => strip_tags( trim( $email ) ) ] );
			$arr_customer = $customer->__toArray( TRUE );
			$new_card_id  = $arr_customer['default_source'];
			$customer_id  = $customer->id;
			$arr_customer_ids[ $account_id ][ $this->stripe_operating_mode ][] = $customer_id;
			$new_customer = TRUE;
		} CATCH ( Exception $e ) {
			$error = mbrErrHdlr::create_error_std_object( 'STRIPE-COULD-NOT-CREATE-CUSTOMER', __LINE__, __FILE__ );
			mbrErrHdlr::mbr_error_exit( $error );
		}

		return [ 'cus_id' => $customer_id, 'arr_cus_ids' =>  $arr_customer_ids, 'card_id' => $new_card_id, 'new_cus' => $new_customer ];
	}

	public function stripe_sync_customer_subscriptions( $cus_email, $card_token )
	{
		$new_customer    = FALSE;
		$customer_id     = FALSE;
		$new_card_id     = FALSE;

		$arr_account     = $this->stripe_get_account_details();
		$account_id      = $arr_account['id'];

		// get stored stripe customer id from AC
		IF ( apply_filters( 'mbr/contact_id', NULL ) ) :
			$tmp_contact        = apply_filters( 'mbr/session/get', NULL );
			$email              = $tmp_contact['email'];
			$arr_customer_ids   = json_decode( stripslashes( $tmp_contact[ MBR_STRIPE_CUST_ID_FIELD ] ), TRUE );
		ELSE:
			$email              = $cus_email;
			$tmp_contact        = apply_filters( 'mbr/get_remote_contact/by_email', [ 'email' => $email, 'local' => FALSE, 'sync' => FALSE ], FALSE );
			IF ( ! empty( $tmp_contact[ MBR_STRIPE_CUST_ID_FIELD ] ) ) :
				$arr_customer_ids = json_decode( stripslashes( $tmp_contact[ MBR_STRIPE_CUST_ID_FIELD ] ), TRUE );
			ENDIF;
		ENDIF;

		IF ( empty( $arr_customer_ids ) ) :
			$arr_customer_ids = [];
		ENDIF;
		IF ( empty( $arr_customer_ids[$account_id] ) ) :
			$arr_customer_ids[ $account_id ] = [];
		ENDIF;
		IF ( empty( $arr_customer_ids[ $account_id ][ $this->stripe_operating_mode ] ) ) :
			$arr_customer_ids[ $account_id ][ $this->stripe_operating_mode ] = [];
		ENDIF;

		TRY {
			$customer     = \Stripe\Customer::create( [ 'card' => $card_token, 'email' => strip_tags( trim( $email ) ) ] );
			$arr_customer = $customer->__toArray( TRUE );
			$new_card_id  = $arr_customer['default_source'];
			$customer_id  = $customer->id;
			$arr_customer_ids[ $account_id ][ $this->stripe_operating_mode ][] = $customer_id;
			$new_customer = TRUE;
		} CATCH ( Exception $e ) {
			$error = mbrErrHdlr::create_error_std_object( 'STRIPE-COULD-NOT-CREATE-CUSTOMER', __LINE__, __FILE__ );
			mbrErrHdlr::mbr_error_exit($error);
		}

		return [ 'cus_id' => $customer_id, 'arr_cus_ids' =>  $arr_customer_ids, 'card_id' => $new_card_id, 'new_cus' => $new_customer ];
	}

	public function get_tax_percentage( $country = FALSE, $state = FALSE )
	{
		$tax_number   = 0;
		$tax_number   = $tax_number + $this->get_order_country_tax( $country );
		$tax_number   = $tax_number + $this->get_order_state_tax( $country, $state );

		IF ($tax_number > 100) :
			$tax_number = 100;
		ENDIF;

		return $tax_number;
	}

	public function get_order_country_tax( $country )
	{
		IF ( ! empty( $country ) AND $country == $this->payment['company_country'] ) :
			IF ( ! empty( $this->payment['country_tax_percentage'] ) ) :
				return $this->payment['country_tax_percentage'];
			ENDIF;
		ENDIF;

		return 0;
	}

	public function get_order_state_tax( $country, $state )
	{
		IF ( ! empty( $country ) AND $country == $this->payment['company_country'] ) :
			IF ( ! empty( $state ) AND $state == $this->payment['company_state'] ) :
				IF ( ! empty( $this->payment['state_tax_percentage'] ) ) :
					return $this->payment['state_tax_percentage'];
				ENDIF;
			ENDIF;
		ENDIF;

		return 0;
	}

	public function calculate_tax( $amount, $country = FALSE, $state = FALSE )
	{
		$tax_number   = $this->get_tax_percentage( $country, $state );
		$tax_number   = $tax_number / 100;
		$tax          = round( $amount * $tax_number );
		$amount       = $amount + $tax;

		return $amount;
	}

	public function stripe_charge( $amount = FALSE, $customer_id = FALSE, $new_customer = FALSE, $card_id = FALSE, $product_id = FALSE, $mode, $arrPOST, $arr_meta = [] )
	{
		$amount          = $this->calculate_tax( $amount, $arrPOST['billing_country'], $arrPOST['billing_state'] );
		$arr_order_items = [ [ 'id' => $product_id, 'type' => 'one-time', 'amount' => $amount, 'qty' => 1 ] ];

		TRY {
			$order_id = wp_insert_post( [ 'post_title' => sprintf(__('Order &ndash; %s'), date('F d, Y @ h:i A')), 'post_type' => MBRCUSTORDERS, 'post_status' => 'mbr_payment_pending' ] );
			update_post_meta( $order_id, $this->order_post_status_pm_key, $this->order_payment_statuses['mbr_payment_pending']['label_regular'] );

			IF ( empty( $order_id ) ) :
				throw new Exception( 'COULD-NOT-CREATE-ORDER' );
			ENDIF;

			$arr_billing_details   = [
				'email'              => $arrPOST['email'],
				'first_name'         => $arrPOST['first_name'],
				'last_name'          => $arrPOST['last_name'],
				'phone'              => $arrPOST['phone'],
				'billing_company'    => $arrPOST['billing_company'],
				'billing_address_1'  => $arrPOST['billing_address_1'],
				'billing_address_2'  => $arrPOST['billing_address_2'],
				'billing_city'       => $arrPOST['billing_city'],
				'billing_state'      => $arrPOST['billing_state'],
				'billing_postcode'   => $arrPOST['billing_postcode'],
				'billing_country'    => $arrPOST['billing_country'],
			];

			IF ( $arr_billing_details['billing_country'] <> 'US' ) :
				unset( $arr_billing_details['billing_state'] );
			ENDIF;

			update_post_meta( $order_id, $this->order_billing_details_pm_key, $arr_billing_details );
			update_post_meta( $order_id, $this->order_items_pm_key, $arr_order_items );
			update_post_meta( $order_id, $this->order_payment_method_pm_key, [ 'type' => 'stripe', 'mode' => $mode ] );
		} CATCH ( Exception $e ) {
			$error = mbrErrHdlr::create_error_std_object( $e->getMessage(), __LINE__, __FILE__ );
			mbrErrHdlr::mbr_error_exit( $error );
		}

		$arr_meta['shop']             = home_url();
		$arr_meta['order_items']      = json_encode( $arr_order_items );
		$arr_meta['local_order_id']   = $order_id;

		$arr_charge             = [];
		$arr_charge['amount']   = $amount;
		$arr_charge['currency'] = MBR_PAYMENT_CURRENCY;
		$arr_charge['customer'] = $customer_id;
		$arr_charge['metadata'] = $arr_meta;

		IF ( ! $new_customer AND ! empty( $card_id ) ) :
			$arr_charge['source'] = $card_id;
		ENDIF;

		update_post_meta( $order_id, $this->stripe_customer_id_pm_key,   $customer_id );
		update_post_meta( $order_id, $this->order_customer_email_pm_key, $arrPOST['email'] );
		update_post_meta( $order_id, $this->order_amount_pm_key,         $amount );
		update_post_meta( $order_id, $this->order_currency_pm_key,       MBR_PAYMENT_CURRENCY );
		update_post_meta( $order_id, $this->order_country_tax_pm_key,    $this->get_order_country_tax( $arrPOST['billing_country'] ) );
		update_post_meta( $order_id, $this->order_state_tax_pm_key,      $this->get_order_state_tax( $arrPOST['billing_country'], $arrPOST['billing_state'] ) );

		TRY {
			$charge                     = \Stripe\Charge::create( $arr_charge );
			$arr_result                 = $charge->__toArray( TRUE );
			$charges                    = get_post_meta( $order_id, $this->stripe_charges_pm_key, TRUE );
			IF ( empty( $charges ) OR ! is_array( $charges ) ) :
				$charges                  = [];
			ENDIF;
			$charge_id                  = $arr_result['id'];
			$charge_created             = $arr_result['created'];
			$charges[$charge_id]        = $arr_result;

			$next_invoice_id            = apply_filters( 'mbr/payment/invoice/get_id', NULL );
			$prefix                     = apply_filters( 'mbr/payment/invoice/prefix', NULL );
			$next_invoice_id            = $prefix . $next_invoice_id;

			$this_charge = \Stripe\Charge::retrieve( $charge_id );
			$this_charge->metadata['local_invoice_id'] = $next_invoice_id;
			$this_charge->save();

			update_post_meta( $order_id, $this->stripe_charges_pm_key,   $charges );
			update_post_meta( $order_id, $this->stripe_charge_id_pm_key, $charge_id );

			wp_update_post( [ 'ID' => $order_id, 'post_status' => 'mbr_payment_success' ] );
			update_post_meta( $order_id, $this->order_post_status_pm_key, $this->order_payment_statuses['mbr_payment_success']['label_regular'] );

			$this->add_charge_to_order( $order_id, $charge_id, $next_invoice_id, $charge_created );

			$this->email_invoice( $arrPOST['email'], $order_id, $charge_id );
			$charged         = TRUE;
		} CATCH ( Exception $e ) {
			$error           = $e->getJsonBody();
			$charge_id       = $error['error']['charge'];
			$charge_created  = time();

			update_post_meta( $order_id, $this->stripe_charges_pm_key,   [ $charge_id => [ 'id' => $charge_id, 'amount' => $amount, 'currency' => MBR_PAYMENT_CURRENCY, 'created' => $charge_created ] ] );
			update_post_meta( $order_id, $this->stripe_charge_id_pm_key, $charge_id );

			wp_update_post( [ 'ID' => $order_id, 'post_status' => 'mbr_payment_failed' ] );
			update_post_meta( $order_id, $this->order_post_status_pm_key, $this->order_payment_statuses['mbr_payment_failed']['label_regular'] );

			$charged         = FALSE;
		}

		return [ 'result' => $charged, 'local_order_id' => $order_id ];
	}

	public function stripe_charge_subscription( $plan_id = FALSE, $customer_id = FALSE, $new_customer = FALSE, $card_id = FALSE, $product_id = FALSE, $mode, $arrPOST, $arr_meta = [] )
	{
		// get plan details live
		TRY {
			$plan             = \Stripe\Plan::retrieve( $plan_id );
			$arr_plan         = $plan->__toArray( TRUE );
		} CATCH ( Exception $e ) {
			throw new Exception( 'COULD-NOT-RETRIEVE-PLAN' );
		}

		$amount             = $this->calculate_tax( $arr_plan['amount'], $arrPOST['billing_country'], $arrPOST['billing_state'] );

		$arr_order_items    = [ [ 'id' => $product_id, 'type' => 'subscription', 'plan_id' => $plan_id, 'interval' => $arr_plan['interval'], 'interval_count' => $arr_plan['interval_count'], 'amount' => $amount, 'qty' => 1 ] ];

		TRY {
			$order_id = wp_insert_post( [ 'post_title' => sprintf( __( 'Order &ndash; %s'), date( 'F d, Y @ h:i A' ) ), 'post_type' => MBRCUSTORDERS, 'post_status' => 'mbr_payment_pending' ] );
			update_post_meta( $order_id, $this->order_post_status_pm_key, $this->order_payment_statuses['mbr_payment_pending']['label_regular'] );

			IF ( empty( $order_id ) ) :
				throw new Exception( 'COULD-NOT-CREATE-ORDER' );
			ENDIF;
			$arr_billing_details   = [
				'email'              => $arrPOST['email'],
				'first_name'         => $arrPOST['first_name'],
				'last_name'          => $arrPOST['last_name'],
				'phone'              => $arrPOST['phone'],
				'billing_company'    => $arrPOST['billing_company'],
				'billing_address_1'  => $arrPOST['billing_address_1'],
				'billing_address_2'  => $arrPOST['billing_address_2'],
				'billing_city'       => $arrPOST['billing_city'],
				'billing_state'      => $arrPOST['billing_state'],
				'billing_postcode'   => $arrPOST['billing_postcode'],
				'billing_country'    => $arrPOST['billing_country'],
			];

			IF ( $arr_billing_details['billing_country'] <> 'US' ) :
				unset( $arr_billing_details['billing_state'] );
			ENDIF;

			update_post_meta( $order_id, $this->order_billing_details_pm_key, $arr_billing_details );
			update_post_meta( $order_id, $this->order_items_pm_key, $arr_order_items );
			update_post_meta( $order_id, $this->order_payment_method_pm_key, [ 'type' => 'stripe', 'mode' => $mode ] );
		} CATCH ( Exception $e ) {
			$error = mbrErrHdlr::create_error_std_object( $e->getMessage(), __LINE__, __FILE__ );
			mbrErrHdlr::mbr_error_exit( $error );
		}

		$arr_meta['shop']           = home_url();
		$arr_meta['order_items']    = json_encode( $arr_order_items );
		$arr_meta['local_order_id'] = $order_id;

		$arr_charge             = [];
		$arr_charge['plan']     = $plan_id;
		$arr_charge['customer'] = $customer_id;
		$arr_charge['metadata'] = $arr_meta;

		$tax_percentage = $this->get_tax_percentage( $arrPOST['billing_country'], $arrPOST['billing_state'] );
		IF ( ! empty( $tax_percentage ) ) :
			$arr_charge['tax_percent'] = $tax_percentage;
		ENDIF;

		update_post_meta( $order_id, $this->stripe_customer_id_pm_key,    $customer_id );
		update_post_meta( $order_id, $this->stripe_charge_id_pm_key,     'initial' );
		update_post_meta( $order_id, $this->order_customer_email_pm_key,  $arrPOST['email'] );
		update_post_meta( $order_id, $this->order_amount_pm_key,          $amount );
		update_post_meta( $order_id, $this->order_currency_pm_key,        MBR_PAYMENT_CURRENCY );
		update_post_meta( $order_id, $this->order_country_tax_pm_key,     $this->get_order_country_tax( $arrPOST['billing_country'] ) );
		update_post_meta( $order_id, $this->order_state_tax_pm_key,       $this->get_order_state_tax( $arrPOST['billing_country'], $arrPOST['billing_state'] ) );

		TRY {
			$charge               = \Stripe\Subscription::create( $arr_charge );
			$arr_result           = $charge->__toArray( TRUE );
			$subscription_id      = $arr_result['id'];
			update_post_meta( $order_id, $this->stripe_subscription_id_pm_key, $subscription_id );
			IF ( $arr_result['status'] == 'trialing' ) :
				$order_status       = 'mbr_payment_sub_tril';
				update_post_meta( $order_id, $this->order_post_status_pm_key, $this->order_payment_statuses['mbr_payment_sub_tril']['label_regular'] );
				$trialing           = TRUE;
			ELSE:
				$order_status       = 'mbr_payment_sub_actv';
				update_post_meta( $order_id, $this->order_post_status_pm_key, $this->order_payment_statuses['mbr_payment_sub_actv']['label_regular'] );
				$trialing           = FALSE;
			ENDIF;

			wp_update_post( [ 'ID' => $order_id, 'post_status' => $order_status ] );

			return [ 'result' => $arr_result, 'trialing' => $trialing,'local_order_id' => $order_id ];
		} CATCH ( Exception $e ) {
			$error           = $e->getJsonBody();
			$subscription_id = $error['error']['charge'];
			update_post_meta( $order_id, $this->stripe_subscription_id_pm_key, $subscription_id );
			wp_update_post( [ 'ID' => $order_id, 'post_status' => 'mbr_payment_sub_paus' ] );
			update_post_meta( $order_id, $this->order_post_status_pm_key, $this->order_payment_statuses['mbr_payment_sub_paus']['label_regular'] );

			return [ 'result' => FALSE, 'trialing' => FALSE,'local_order_id' => $order_id ];
		}
	}

	public function stripe_validate_fields( $array, $arr_to_validate = [] )
	{
		$array                                  = array_map( 'trim', $array );
		$array                                  = array_map( 'stripslashes', $array );
		$wp_signon                              = true;
		$passed                                 = TRUE;
		$arr_error                              = [];
		IF (empty($array['email'])) :
			$passed                               = FALSE;
			$arr_error['email']                   = __( 'Please enter an email address.', 'activemember360' );
		ELSEIF ( ! is_email( $array['email'] ) ) :
			$passed                               = FALSE;
			$arr_error['email']                   = __( 'Please enter a valid email address.', 'activemember360' );
		ELSE:
			IF ( ! apply_filters( 'mbr/contact_id', NULL ) ) :
				$tmp_contact                          = apply_filters( 'mbr/get_remote_contact/by_email', [ 'email' => $array['email'], 'local' => FALSE, 'sync' => FALSE ], FALSE );
				IF ( ! empty( $tmp_contact['id'] ) ) :
					$wp_signon                          = false;
					//$passed                             = FALSE;
					//$arr_error['email']                 = sprintf( __( 'A user with this email address already exists. Please login %s.', 'activemember360' ), '<a href="' . wp_login_url() . '">' . __( 'here', 'activemember360' ) . '</a>' );
				ENDIF;
			ENDIF;
		ENDIF;
		IF ( empty( $array['first_name'] ) ) :
			$passed                                 = FALSE;
			$arr_error['first_name']                = __( 'Please enter your first name.', 'activemember360' );
		ENDIF;
		IF ( empty( $array['last_name'] ) ) :
			$passed                                 = FALSE;
			$arr_error['last_name']                 = __( 'Please enter your last name.', 'activemember360' );
		ENDIF;
		IF ( ! empty( $arr_to_validate['phone'] ) ) :
			IF ( empty( $array['phone'] ) ) :
				$passed                               = FALSE;
				$arr_error['phone']                   = __( 'Please enter your phone number.', 'activemember360' );
			ENDIF;
		ENDIF;
		IF ( ! empty( $arr_to_validate['billing'] ) ) :
			IF ( empty( $array['billing_address_1'] ) ) :
				$passed                               = FALSE;
				$arr_error['billing_address_1']       = __( 'Please enter your billing address 1.', 'activemember360' );
			ENDIF;
			IF ( empty( $array['billing_postcode'] ) ) :
				$passed                               = FALSE;
				$arr_error['billing_postcode']        = __( 'Please enter your postal code.', 'activemember360' );
			ENDIF;
			IF ( empty( $array['billing_city'] ) ) :
				$passed                               = FALSE;
				$arr_error['billing_city']            = __( 'Please enter your city.', 'activemember360' );
			ENDIF;
			IF ( ! empty($array['billing_country']) AND $array['billing_country'] == 'US' ) :
				IF ( empty( $array['billing_state'] ) ) :
					$passed                             = FALSE;
					$arr_error['billing_state']         = __( 'Please enter your state.', 'activemember360' );
				ENDIF;
			ENDIF;
			IF ( empty( $array['billing_country'] ) ) :
				$passed                               = FALSE;
				$arr_error['billing_country']         = __( 'Please enter your country.', 'activemember360' );
			ENDIF;
		ENDIF;

		return [ 'passed' => $passed, 'error' => $arr_error, 'sanitized' => $array, 'wp_signon' => $wp_signon ];
	}

	public function stripe_one_click_upsell_validate_fields( $array )
	{
		$array                                  = array_map( 'trim', $array );
		$array                                  = array_map( 'stripslashes', $array );
		$passed                                 = TRUE;
		$arr_error                              = [];
		IF ( empty( $array['email'] ) ) :
			$passed                               = FALSE;
			$arr_error['email']                   = __( 'Email address is missing.', 'activemember360' );
		ELSEIF ( ! is_email( $array['email'] ) ) :
			$passed                               = FALSE;
			$arr_error['email']                   = __( 'Your email address on file has not a valid format.', 'activemember360' );
		ENDIF;
		IF ( empty( $array['first_name'] ) ) :
			$passed                               = FALSE;
			$arr_error['first_name']              = __( 'First Name is missing.', 'activemember360' );
		ENDIF;
		IF ( empty( $array['last_name'] ) ) :
			$passed                               = FALSE;
			$arr_error['last_name']               = __( 'Last Name is missing.', 'activemember360' );
		ENDIF;
		IF ( empty( $array['billing_address_1'] ) ) :
			$passed                               = FALSE;
			$arr_error['billing_address_1']       = __( 'Billing Address 1 is missing.', 'activemember360' );
		ENDIF;
		IF ( empty( $array['billing_postcode'] ) ) :
			$passed                               = FALSE;
			$arr_error['billing_postcode']        = __( 'Postal Code is missing.', 'activemember360' );
		ENDIF;
		IF ( empty( $array['billing_city'] ) ) :
			$passed                               = FALSE;
			$arr_error['billing_city']            = __( 'City is missing.', 'activemember360' );
		ENDIF;
		IF ( ! empty( $array['billing_country'] ) AND $array['billing_country'] == 'US' ) :
			IF ( empty( $array['billing_state'] ) ) :
				$passed                             = FALSE;
				$arr_error['billing_state']         = __( 'Please enter your state.', 'activemember360' );
			ENDIF;
		ENDIF;
		IF ( empty( $array['billing_country'] ) ) :
			$passed                               = FALSE;
			$arr_error['billing_country']         = __( 'Country is missing.', 'activemember360' );
		ENDIF;

		return [ 'passed' => $passed, 'error' => $arr_error, 'sanitized' => $array ];
	}

	public function stripe_get_all_plans()
	{
		// Used to triger errors
		// Stripe::setApiKey('sk_live_xxxxxxxxxxxxxxxxxxxx');
		// Stripe::setApiKey('sk_test_xxxxxxxxxxxxxxxxxxxx');
		TRY {
			$plans = \Stripe\Plan::all( [ 'limit' => '100' ] );
		} CATCH ( \Stripe\Error\Authentication $e ) {
			$message = $this->stripe_authentication_error_message( $e );
			$error   = mbrErrHdlr::create_error_std_object( $message, __LINE__, __FILE__ );
			mbrErrHdlr::mbr_error_exit( $error );
		} CATCH ( Exception $e ) {
			$error = mbrErrHdlr::create_error_std_object( $e->getMessage(), __LINE__, __FILE__ );
			mbrErrHdlr::mbr_error_exit( $error );
		}

		return $plans;
	}

	public function stripe_authentication_error_message( $e )
	{
		$message     = $e->getMessage();
		IF ( FALSE !== stripos( $message, 'Invalid API Key' ) ) :
			$mode      = FALSE !== stripos($message, 'sk_live') ? 'live' : 'test';
			IF ( $mode == 'live' ) :
				$message = 'STRIPE-API-SECRET-KEY-LIVE-CONNECTION-ERROR';
			ELSEIF ( $mode == 'test' ) :
				$message = 'STRIPE-API-SECRET-KEY-TEST-CONNECTION-ERROR';
			ELSE:
				$message = 'STRIPE-API-SECRET-KEY-GENERAL-CONNECTION-ERROR';
			ENDIF;
		ELSE :
			$message   = 'STRIPE-API-SECRET-KEY-GENERAL-CONNECTION-ERROR';
		ENDIF;

		return $message;
	}

	public function get_product_name_by_id( $product_id, $admin_link = FALSE, $pid = TRUE )
	{
		$product               = FALSE;
		IF ( ! empty( $product_id ) ) :
			$product             = get_post( $product_id, FALSE );
		ENDIF;

		IF ( ! empty( $product ) ) :
			$suffix   = '';
			IF ( $pid ) :
				$suffix =  " (#{$product->ID}";
			ENDIF;
			IF ( $admin_link ) :
				$product_link = admin_url( "post.php?post={$product->ID}&action=edit" );
				$product_name = "<a href='{$product_link}' target='_blank'>{$product->post_title}{$suffix})</a>";
			ELSE:
				$product_name = $product->post_title . $suffix;
			ENDIF;
		ELSEIF ( ! empty( $product_id ) ):
			$product_name = '#' . $product_id . ' (' . __('Product does not exist anymore', 'activemember360') . ')';
		ELSE:
			$product_name = __( 'Unknown', 'activemember360' );
		ENDIF;

		return $product_name;
	}

}

trait TRAIT__credit_card_validation {

	protected static $cards = [
		// Debit cards must come first, since they have more specific patterns than their credit-card equivalents.
		'visaelectron'       => [
			'type'             => 'visaelectron',
			'pattern'          => '/^4(026|17500|405|508|844|91[37])/',
			'length'           => [16],
			'cvcLength'        => [3],
			'luhn'             => TRUE,
		],
		'maestro'            => [
			'type'             => 'maestro',
			'pattern'          => '/^(5(018|0[23]|[68])|6(39|7))/',
			'length'           => [12, 13, 14, 15, 16, 17, 18, 19],
			'cvcLength'        => [3],
			'luhn'             => TRUE,
		],
		'forbrugsforeningen' => [
			'type'             => 'forbrugsforeningen',
			'pattern'          => '/^600/',
			'length'           => [16],
			'cvcLength'        => [3],
			'luhn'             => TRUE,
		],
		'dankort'            => [
			'type'             => 'dankort',
			'pattern'          => '/^5019/',
			'length'           => [16],
			'cvcLength'        => [3],
			'luhn'             => TRUE,
		],
		// Credit cards
		'visa'               => [
			'type'             => 'visa',
			'pattern'          => '/^4/',
			'length'           => [13, 16],
			'cvcLength'        => [3],
			'luhn'             => TRUE,
		],
		'mastercard'         => [
			'type'             => 'mastercard',
			'pattern'          => '/^(5[0-5]|2[2-7])/',
			'length'           => [16],
			'cvcLength'        => [3],
			'luhn'             => TRUE,
		],
		'amex'               => [
			'type'             => 'amex',
			'pattern'          => '/^3[47]/',
			'format'           => '/(\d{1,4})(\d{1,6})?(\d{1,5})?/',
			'length'           => [15],
			'cvcLength'        => [3, 4],
			'luhn'             => TRUE,
		],
		'dinersclub'         => [
			'type'             => 'dinersclub',
			'pattern'          => '/^3[0689]/',
			'length'           => [14],
			'cvcLength'        => [3],
			'luhn'             => TRUE,
		],
		'discover'           => [
			'type'             => 'discover',
			'pattern'          => '/^6([045]|22)/',
			'length'           => [16],
			'cvcLength'        => [3],
			'luhn'             => TRUE,
		],
		'unionpay'           => [
			'type'             => 'unionpay',
			'pattern'          => '/^(62|88)/',
			'length'           => [16, 17, 18, 19],
			'cvcLength'        => [3],
			'luhn'             => FALSE,
		],
		'jcb'                => [
			'type'             => 'jcb',
			'pattern'          => '/^35/',
			'length'           => [16],
			'cvcLength'        => [3],
			'luhn'             => TRUE,
		],
	];

	public static function valid_credit_card( $number, $type = NULL )
	{
		$ret       = [
			'valid'  => FALSE,
			'number' => '',
			'type'   => '',
		];

		// Strip non-numeric characters
		$number = preg_replace( '/[^0-9]/', '', $number );
		IF ( empty( $type ) ) :
			$type = self::credit_card_type( $number );
		ENDIF;

		IF ( array_key_exists( $type, self::$cards ) AND self::valid_card( $number, $type ) ) :
			return [
				'valid'  => TRUE,
				'number' => $number,
				'type'   => $type,
			];
		ENDIF;

		return $ret;
	}

	public static function valid_cvc( $cvc, $type )
	{
		return ( ctype_digit( $cvc ) AND array_key_exists( $type, self::$cards ) AND self::valid_cvc_length( $cvc, $type ) );
	}

	public static function valid_date( $year, $month )
	{
		$month = str_pad( $month, 2, '0', STR_PAD_LEFT );
		IF ( ! preg_match( '/^20\d\d$/', $year ) ) :
			return FALSE;
		ENDIF;
		IF ( ! preg_match( '/^(0[1-9]|1[0-2])$/', $month ) ) :
			return FALSE;
		ENDIF;
		// past date
		IF ( $year < date( 'Y' ) OR ( $year == date( 'Y' ) AND $month < date( 'm' ) ) ) :
			return FALSE;
		ENDIF;

		return TRUE;
	}

	protected static function credit_card_type( $number )
	{
		FOREACH ( self::$cards AS $type => $card ) :
			IF ( preg_match( $card['pattern'], $number ) ) :
				return $type;
			ENDIF;
		ENDFOREACH;

		return '';
	}

	protected static function valid_card( $number, $type )
	{
		return ( self::valid_pattern( $number, $type ) AND self::valid_length( $number, $type ) AND self::valid_luhn( $number, $type ) );
	}

	protected static function valid_pattern( $number, $type )
	{
		return preg_match( self::$cards[ $type ]['pattern'], $number );
	}

	protected static function valid_length( $number, $type )
	{
		FOREACH ( self::$cards[ $type ]['length'] AS $length ) :
			IF ( strlen( $number ) == $length ) :
				return TRUE;
			ENDIF;
		ENDFOREACH;

		return FALSE;
	}

	protected static function valid_cvc_length( $cvc, $type )
	{
		FOREACH ( self::$cards[ $type ]['cvcLength'] AS $length ) :
			IF ( strlen( $cvc ) == $length ) :
				return TRUE;
			ENDIF;
		ENDFOREACH;

		return FALSE;
	}

	protected static function valid_luhn( $number, $type )
	{
		IF ( ! self::$cards[ $type ]['luhn'] ) :
			return TRUE;
		ELSE:
			return self::luhn_check( $number );
		ENDIF;
	}

	protected static function luhn_check( $number )
	{
		$checksum = 0;

		FOR ( $i = ( 2 - ( strlen( $number ) % 2 ) ); $i <= strlen( $number ); $i += 2 ) :
			$checksum += (int) ( $number{$i-1} );
		ENDFOR;

		// Analyze odd digits in even length strings or even digits in odd length strings.
		FOR ( $i = ( strlen( $number ) % 2 ) + 1; $i < strlen( $number ); $i+=2 ) :
			$digit = (int) ( $number{$i-1} ) * 2;
			IF ( $digit < 10 ) :
				$checksum += $digit;
			ELSE:
				$checksum += ( $digit-9 );
			ENDIF;
		ENDFOR;

		return ( ( $checksum % 10 ) == 0 );
	}

}
$GLOBALS['mbrpayment'] = mbr_payment::getInstance();