<?php
final class mbr_payment_admin EXTENDS mbr_redux_base_class
{
	use TRAIT__getInstance;

	public $opt_name_payment_settings_parent = 'mbr_payment_settings_parent';
	public $opt_name_payment_settings        = 'mbr_payment_settings';
	public $page_slug_parent                 = 'mbr360-payment-parent';
	public $page_slug                        = 'mbr360-payment';
	public $opt_name_subscription_settings   = MBR_SUBSCRIPTION_SETTINGS;
	public $opt_name_product_settings        = MBR_PRODUCT_SETTINGS;
	public $opt_name_checkouttpl_settings    = MBR_CHECKOUTTPL_SETTINGS;

	public function plugins_loaded()
	{
		global $MBR_OPTIONS;
		$arr_license = apply_filters( 'mbr/license/status/get', NULL );
		IF ( $arr_license['status'] == 'inactive' OR empty( $MBR_OPTIONS['api_url'] ) OR empty( $MBR_OPTIONS['api_key'] ) ) :
			return;
		ENDIF;

		mbr_constant( 'MBR_MENU_PAYMENT_PAGE', 'mbr360-payment-parent' );

		add_action( 'plugins_loaded',                                                         [ $this, 'init_admin_cpt_panels' ],             99998 );
		add_action( 'plugins_loaded',                                                         [ $this, 'init_admin_payment_settings_panel' ], 99999 );
		add_action( 'plugins_loaded',                                                         [ $this, 'init_sections' ],                     99999 );
		add_filter( 'custom_menu_order',                                                      [ $this, 'submenu_order' ],                     10, 1 );
		add_filter( "redux/{$this->opt_name_payment_settings}/panel/template/header.tpl.php", [ $this, 'alternate_header' ]                         );
		add_filter( "redux/metaboxes/{$this->opt_name_subscription_settings}/boxes",          [ $this, 'subscription_settings_metabox' ],     10, 2 );
		add_filter( "redux/metaboxes/{$this->opt_name_product_settings}/boxes",               [ $this, 'product_settings_metabox' ],          10, 2 );
		add_filter( "redux/metaboxes/{$this->opt_name_checkouttpl_settings}/boxes",           [ $this, 'checkouttpl_settings_metabox'],       10, 2 );
	}

	public function submenu_order( $menu_order )
	{
		global $submenu;

		$main_item          = FALSE;
		$order              = [];

		FOREACH ( $submenu[ $this->page_slug_parent] AS $key => $submenu_item ) :
			IF ( $submenu_item[0] == __( 'Options Object', 'redux-framework' ) ) :
				unset( $submenu[ $this->page_slug_parent ][ $key ]);
			ELSEIF ( $submenu_item[0] == _x( 'Settings', 'wp_admin', 'activemember360' ) ):
				$main_item      = $submenu_item;
			ELSE:
				$order[]        = $submenu_item;
			ENDIF;
		ENDFOREACH;

		IF ( ! empty( $main_item ) ) :
			array_unshift( $order, $main_item );
		ENDIF;

		$submenu[ $this->page_slug_parent ] = $order;

		
		return $menu_order;
	}

	public function init_admin_cpt_panels()
	{
		$admin_title           = _x( 'Payments', 'wp_admin', 'activemember360' );

		$arr_config            =  [
			'opt_name'           => $this->opt_name_payment_settings_parent,
			'page_slug'          => $this->page_slug_parent,
			'admin_title'        => $admin_title,
			'menu_title'         => $admin_title,
			'menu_icon'          => apply_filters( 'mbr/plugin_favicon', NULL ),
		];

		$arr_config            = array_merge( $this->arr_default_main_admin_panel_settings, $arr_config );

		Redux::setArgs( $this->opt_name_payment_settings_parent, $arr_config );

		$arr_opt_name          = [ $this->opt_name_subscription_settings, $this->opt_name_product_settings, $this->opt_name_checkouttpl_settings ];
		FOREACH ( $arr_opt_name AS $key => $opt_name ) :
			$arr_config          =  [
				'opt_name'         => $opt_name,
				'page_parent'      => MBR_MENU_PAYMENT_PAGE,
			];

			$arr_config          = array_merge( $this->arr_default_hidden_admin_panel_settings, $arr_config );

			Redux::setArgs( $opt_name, $arr_config );
			Redux::setExtensions( $opt_name, MBR_REDUX_EXTENSIONS_DIR );
		ENDFOREACH;
	}

	public function init_admin_payment_settings_panel()
	{
		global $MBR_REDUX_HEADER_MODULE_NAMES;

		$MBR_REDUX_HEADER_MODULE_NAMES[ $this->opt_name_payment_settings ] = 'Payments Module';

		$admin_title = $page_title = _x( 'Settings', 'wp_admin', 'activemember360' );
		$page_title  = apply_filters( 'mbr/plugin_name', NULL ) . ' ' . $page_title;

		$arr_config         =  [
			'opt_name'        => $this->opt_name_payment_settings,
			'global_variable' => $this->opt_name_payment_settings,
			'page_slug'       => $this->page_slug,
			'page_parent'     => MBR_MENU_PAYMENT_PAGE,
			'page_title'      => $page_title,
			'display_name'    => $page_title,
			'admin_title'     => $admin_title,
			'menu_title'      => $admin_title,
		];

		$arr_config = array_merge( $this->arr_default_sub_admin_panel_settings, $arr_config );

		Redux::setArgs( $this->opt_name_payment_settings, $arr_config );
	}

	public function init_sections()
	{
		global $pagenow;

		IF ( $pagenow <> 'admin-ajax.php' AND $pagenow <> 'admin.php' OR ( $pagenow == 'admin.php' AND ( empty( $_GET['page'] ) OR $_GET['page'] <> $this->page_slug ) ) ) :
			return;
		ENDIF;

		Redux::setSection(
			$this->opt_name_payment_settings,
			[
				'id'                    => 'section_payment_general_settings',
				'title'                 => _x( 'Settings', 'wp_admin', 'activemember360' ),
				'desc'                  => _x( 'Payment Settings of ActiveMember360\'s integrated payment extension.', 'wp_admin', 'activemember360' ),
				'icon'                  => 'el el-cog',
				'fields'                => [
					[
						'id'                => 'currency',
						'type'              => 'select',
						'title'             => _x( 'Currency', 'wp_admin', 'activemember360' ),
						'subtitle'          => _x( 'Choose the currency to use.', 'wp_admin', 'activemember360' ),
						'desc'              => _x( 'Currency to use for ActiveMember360\'s payments.', 'wp_admin', 'activemember360' ),
						'options'           => [ 'custom_currency' => _x( 'Custom Currency', 'wp_admin', 'activemember360' ), 'usd' => 'USD', 'eur' => 'EUR', 'gbp' => 'GBP', 'aud' => 'AUD', 'cad' => 'CAD', 'zar' => 'ZAR', 'nzd' => 'NZD', 'chf' => 'CHF', 'ils' => 'ILS' ],
						'select2'           => [ 'allowClear' => FALSE ],
						'default'           => 'usd'
					],
					[
						'id'                => 'custom_currency',
						'type'              => 'text',
						'title'             => _x( 'Custom Currency', 'wp_admin', 'activemember360' ),
						'subtitle'          => _x( 'Type in the name of your currency', 'wp_admin', 'activemember360' ),
						'desc'              => _x( 'Your currency e.g. jpy.', 'wp_admin', 'activemember360' ),
						'required'          => [ 'currency', 'equals', 'custom_currency' ],
						'validate_callback' => 'mbr_validate_custom_currency',
					],
					[
						'id'                => 'currency_position',
						'type'              => 'select',
						'title'             => _x( 'Currency Position', 'wp_admin', 'activemember360' ),
						'subtitle'          => _x( 'Choose position.',  'wp_admin', 'activemember360' ),
						'desc'              => _x( 'Choose the currency\'s position.', 'wp_admin', 'activemember360' ),
						'options'           => [ 'left' => 'Left', 'right' => 'Right', 'left_with_space' => 'Left with space', 'right_with_space' => 'Right with space' ],
						'select2'           => [ 'allowClear' => FALSE ],
						'default'           => 'left'
					],
					[
						'id'                => 'currency_thousand_sep',
						'type'              => 'text',
						'title'             => _x( 'Thousand Separator', 'wp_admin', 'activemember360' ),
						'subtitle'          => _x( 'Set the thousand separator for displayed prices.', 'wp_admin', 'activemember360' ),
						'default'           => ','
					],
					[
						'id'                => 'currency_decimal_sep',
						'type'              => 'text',
						'title'             => _x( 'Decimal Separator', 'wp_admin', 'activemember360' ),
						'subtitle'          => _x( 'Set the decimal separator for displayed prices.', 'wp_admin', 'activemember360' ),
						'default'           => '.'
					],
					[
						'id'                => 'currency_decimals',
						'type'              => 'spinner',
						'min'               => '0',
						'max'               => '10',
						'title'             => _x( 'Number of Decimals', 'wp_admin', 'activemember360' ),
						'subtitle'          => _x( 'Number of decimal points shown in displayed prices.', 'wp_admin', 'activemember360' ),
						'default'           => '2'
					]
				]
			]
		);

		Redux::setSection(
			$this->opt_name_payment_settings,
			[
				'id'           => 'section_payment_company_settings',
				'title'        => _x( 'Company Settings', 'wp_admin', 'activemember360' ),
				'desc'         => _x( 'Company Settings of ActiveMember360\'s integrated payment extension.', 'wp_admin', 'activemember360' ),
				'icon'         => 'el el-group',
				'subsection'   => FALSE,
				'fields'       => [
					[
						'id'       => 'company_logo',
						'type'     => 'media',
						'title'    => _x( 'Company Logo', 'wp_admin', 'activemember360' ),
						'subtitle' => _x( 'Your company logo.', 'wp_admin', 'activemember360' ),
						'default'  => ''
					],
					[
						'id'       => 'company_name',
						'type'     => 'text',
						'title'    => _x( 'Company Name', 'wp_admin', 'activemember360' ),
						'subtitle' => _x( 'Your company name.', 'wp_admin', 'activemember360' ),
						'default'  => ''
					],
					[
						'id'       => 'company_email',
						'type'     => 'text',
						'title'    => _x( 'Company Email', 'wp_admin', 'activemember360' ),
						'subtitle' => _x( 'Your company email address.', 'wp_admin', 'activemember360' ),
						'default'  => ''
					],
					[
						'id'       => 'company_vat_id',
						'type'     => 'text',
						'title'    => _x( 'VAT ID', 'wp_admin', 'activemember360' ),
						'default'  => ''
					],
					[
						'id'       => 'company_street_address_1',
						'type'     => 'text',
						'title'    => _x( 'Street Address', 'wp_admin', 'activemember360' ),
						'default'  => ''
					],
					[
						'id'       => 'company_street_address_2',
						'type'     => 'text',
						'title'    => _x( 'Street Address 2', 'wp_admin', 'activemember360' ),
						'default'  => ''
					],
					[
						'id'       => 'company_postal_code',
						'type'     => 'text',
						'title'    => _x( 'Postal Code', 'wp_admin', 'activemember360' ),
						'default'  => ''
					],
					[
						'id'       => 'company_city',
						'type'     => 'text',
						'title'    => _x( 'City', 'wp_admin', 'activemember360' ),
						'default'  => ''
					],
					[
						'id'       => 'company_country',
						'type'     => 'select',
						'title'    => _x( 'Country', 'wp_admin', 'activemember360' ),
						'options'  => apply_filters( 'mbr/payments/countries/get', NULL ),
						'select2'  => [ 'allowClear' => FALSE ],
						'default'  => ''
					],
					[
						'id'       => 'company_state',
						'type'     => 'select',
						'title'    => _x( 'State', 'wp_admin', 'activemember360' ),
						'options'  => apply_filters( 'mbr/payments/states/us/get', NULL ),
						'select2'  => [ 'allowClear' => FALSE ],
						'required' => [ 'company_country', 'equals', 'US' ],
						'default'  => ''
					]
				]
			]
		);

		Redux::setSection(
			$this->opt_name_payment_settings,
			[
				'id'           => 'section_payment_tax_settings',
				'title'        => _x( 'Tax Settings', 'wp_admin', 'activemember360' ),
				'desc'         => _x( 'Tax Settings of ActiveMember360\'s integrated payment extension.', 'wp_admin', 'activemember360' ),
				'icon'         => 'el el-usd',
				'subsection'   => FALSE,
				'fields'       => [
					[
						'id'       => 'state_tax_percentage',
						'type'     => 'text',
						'validate' => 'numeric',
						'title'    => _x( 'State Tax percentage', 'wp_admin', 'activemember360' ),
						'subtitle' => _x( 'State Tax percentage which should be added to the product and/or subscription price.', 'wp_admin', 'activemember360' ),
						'default'  => '0'
					],
					[
						'id'       => 'country_tax_percentage',
						'type'     => 'text',
						'validate' => 'numeric',
						'title'    => _x( 'Country Tax percentage', 'wp_admin', 'activemember360' ),
						'subtitle' => _x( 'Country Tax percentage which should be added to the product and/or subscription price.', 'wp_admin', 'activemember360' ),
						'default'  => '0'
					]
				]
			]
		);

		$stripe_webhook_urls      = '';
		$arr_sec_codes            = apply_filters( 'mbr/secodes/get', NULL );
		IF ( ! empty( $arr_sec_codes ) ) :
			FOREACH ( $arr_sec_codes AS $sec_code_key => $sec_code ) :
				$stripe_webhook_urls .= '<li>- ' . home_url() . '/mbr-payment/stripe/v1/' . $sec_code . '</li>';
			ENDFOREACH;
		ELSE:
			$stripe_webhook_urls   .= '<li><strong>' . sprintf( _x( 'You need to create a security code %s, before we can show you the webhook urls.', 'wp_admin', 'activemember360' ), '<a href="' . admin_url( 'admin.php?page=mbr360-plugin-options' ) . '">' . _x( 'here', 'wp_admin', 'activemember360' ) . '</a>' ) . '</strong></li>';
		ENDIF;

		Redux::setSection(
			$this->opt_name_payment_settings,
			[
				'id'             => 'section_payment_stripe_settings',
				'title'          => _x( 'Stripe Settings', 'wp_admin', 'activemember360' ),
				'desc'           => _x( 'Stripe Settings of ActiveMember360\'s integrated payment extension.', 'wp_admin', 'activemember360' ),
				'icon'           => 'el el-credit-card',
				'subsection'     => FALSE,
				'fields'         => [
					[
						'id'         => 'stripe_status_start',
						'title'      => _x( 'Stripe Status', 'wp_admin', 'activemember360' ),
						'type'       => 'section',
						'indent'     => TRUE
					],
					[
						'id'         => 'stripe_status',
						'type'       => 'switch',
						'title'      => _x( 'Status', 'wp_admin', 'activemember360' ),
						'subtitle'   => _x( 'Activate/Deactivate Stripe', 'wp_admin', 'activemember360' ),
						'on'         => _x( 'Active', 'wp_admin', 'activemember360' ),
						'off'        => _x( 'Inactive', 'wp_admin', 'activemember360' ),
						'default'    => TRUE
					],
					[
						'id'         => 'stripe_settings_start',
						'title'      => _x( 'Stripe Keys', 'wp_admin', 'activemember360' ),
						'type'       => 'section',
						'required'   => ['stripe_status', 'equals', 1],
						'indent'     => TRUE
					],
					[
						'id'         => 'stripe_test_secret',
						'type'       => 'text',
						'title'      => _x( 'Test secret key', 'wp_admin', 'activemember360' ),
						'subtitle'   => _x( 'Enter your test secret key', 'wp_admin', 'activemember360' )
					],
					[
						'id'         => 'stripe_test_publishable',
						'type'       => 'text',
						'title'      => _x( 'Test publishable key', 'wp_admin', 'activemember360' ),
						'subtitle'   => _x( 'Enter your test publishable key', 'wp_admin', 'activemember360' )
					],
					[
						'id'         => 'stripe_live_secret',
						'type'       => 'text',
						'title'      => _x( 'Live secret key', 'wp_admin', 'activemember360' ),
						'subtitle'   => _x( 'Enter your live secret key', 'wp_admin', 'activemember360' )
					],
					[
						'id'         => 'stripe_live_publishable',
						'type'       => 'text',
						'title'      => _x( 'Live publishable key', 'wp_admin', 'activemember360' ),
						'subtitle'   => _x( 'Enter your live publishable key', 'wp_admin', 'activemember360' )
					],
					[
						'id'         => 'stripe_webhooks_start',
						'title'      => _x( 'Stripe Webhooks', 'wp_admin', 'activemember360' ),
						'type'       => 'section',
						'indent'     => TRUE
					],
					[
						'id'         => 'stripe_webhook_creation',
						'type'       => 'raw',
						'title'      => _x( 'Create a Stripe Webhook', 'wp_admin', 'activemember360' ),
						'markdown'   => FALSE,
						'full_width' => FALSE,
						'content'    => '
							<p>'  . _x( 'Create one webhook (ONLY ONE) for TEST MODE which points to one (ONLY ONE) of the following URL\'s:', 'wp_admin', 'activemember360' ) . '</p>
							<ul>' . $stripe_webhook_urls . '</ul>
							<p>'  . _x( 'Create one webhook (ONLY ONE) for LIVE MODE which points to one (ONLY ONE) of the following URL\'s:', 'wp_admin', 'activemember360' ) . '</p>
							<ul>' . $stripe_webhook_urls . '</ul>
							<p>'  . _x( 'Select following event types:', 'wp_admin', 'activemember360' ) . '</p>
							<ul>
								<li>- charge.refunded</li>
								<li>- customer.subscription.deleted</li>
								<li>- invoice.payment_failed</li>
								<li>- invoice.payment_succeeded</li>
							</ul>
						'
					],
					[
						'id'         => 'stripe_signing_secret',
						'type'       => 'text',
						'title'      => _x( 'Signing Secret Key (Live Mode)', 'wp_admin', 'activemember360' ),
						'subtitle'   => _x( 'This secret key is needed to verify incoming webhooks in live mode', 'wp_admin', 'activemember360' )
					],
					[
						'id'         => 'stripe_signing_secret_test',
						'type'       => 'text',
						'title'      => _x( 'Signing Secret Key (Test Mode)', 'wp_admin', 'activemember360' ),
						'subtitle'   => _x( 'This secret key is needed to verify incoming webhooks in test mode', 'wp_admin', 'activemember360' ),
					],
				]
			]
		);

		$invoice_default_subject        = sprintf( _x( 'New Invoice #%s', 'wp_admin', 'activemember360' ), '%%invoice_id%%' );
		$invoice_default_template       = $this->invoice_default_template();
		$supported_subject_placeholders = '<ul style="margin-left:20px;list-style-type:square;"><li>%%invoice_id%%</li></ul>';
		$supported_body_placeholders    = '<ul style="margin-left:20px;list-style-type:square;">'
			. '<li>%%invoice_id%%</li>'
			. '<li>%%company_address%%</li>'
			. '<li>%%company_logo%%</li>'
			. '<li>%%billing_address%%</li>'
			. '<li>%%order_id%%</li>'
			. '<li>%%order_date%%</li>'
			. '<li>%%company_vat_id%%</li>'
			. '<li>%%order_total%%</li>'
			. '<li>%%item_name%%</li>'
			. '<li>%%item_price%%</li>'
			. '<li>%%net_total%%</li>'
			. '<li>%%sales_tax%%</li>'
			. '<li>%%refund_total%%</li>'
			. '<li>%%order_refunds%%</li>'
			. '</ul>';

		IF ( ! empty( $_POST['mbr_payment_reset_invoice_ids'] ) AND isset( $_POST['mbr_payment_last_invoice_id'] ) ) :
			update_option( 'mbr_payment_invoice_id', $_POST['mbr_payment_last_invoice_id'] );
		ENDIF;

		Redux::setSection(
			$this->opt_name_payment_settings,
			[
				'id'             => 'section_payment_invoice_settings',
				'title'          => _x( 'Invoice Settings', 'wp_admin', 'activemember360' ),
				'desc'           => _x( 'Invoice template of ActiveMember360\'s integrated payment extension.', 'wp_admin', 'activemember360' ),
				'icon'           => 'el el-cog',
				'subsection'     => FALSE,
				'fields'         => [
					[
						'id'         => 'invoice_number_prefix',
						'type'       => 'text',
						'title'      => _x( 'Invoice # prefix', 'wp_admin', 'activemember360' ),
						'subtitle'   => _x( 'Set your Invoice # prefix.', 'wp_admin', 'activemember360' )
					],
					[
						'id'         => 'reset_invoice_number',
						'type'       => 'raw',
						'full_width' => FALSE,
						'desc'       => _x( 'This will reset your invoice ids. Unsaved data will be saved too!', 'wp_admin', 'activemember360' ),
						'title'      => _x( 'Rest Invoice ID', 'wp_admin', 'activemember360' ),
						'content'    => '<form method="post" action=""><label for="mbr_payment_last_invoice_id">' . _x( 'Last Invoice ID to be used to start', 'wp_admin', 'activemember360' ) . '</label> <input type="text" name="mbr_payment_last_invoice_id" id="mbr_payment_last_invoice_id" value="" /><br /><input type="submit" class="button button-primary" name="mbr_payment_reset_invoice_ids" value="' . _x( 'Reset Invoice ids', 'wp_admin', 'activemember360' ) . '" /></form>'
					]
				]
			]
		);

		Redux::setSection(
			$this->opt_name_payment_settings,
			[
				'id'          => 'section_payment_invoice_template',
				'title'       => _x( 'Invoice Template', 'wp_admin', 'activemember360' ),
				'desc'        => _x( 'Invoice template of ActiveMember360\'s integrated payment extension.', 'wp_admin', 'activemember360' ),
				'icon'        => 'el el-cogs',
				'subsection'  => TRUE,
				'fields'      => [
					[
						'id'      => 'invoice_email_subject',
						'title'   => _x( 'Subject', 'wp_admin', 'activemember360' ),
						'type'    => 'text',
						'desc'    => sprintf( _x( 'The subject used for the invoice sent by email. Supported placeholders: %s', 'wp_admin', 'activemember360' ), $supported_subject_placeholders ),
						'default' => $invoice_default_subject
					],
					[
						'id'      => 'invoice_template',
						'type'    => 'ace_editor',
						'mode'    => 'html',
						'theme'   => 'chrome',
						'title'   => _x( 'Invoice template', 'wp_admin', 'activemember360' ),
						'desc'    => sprintf( _x( 'Create your HTML invoice template. Supported placeholders: %s', 'wp_admin', 'activemember360' ), $supported_body_placeholders ),
						'default' => $invoice_default_template
					]
				]
			]
		);

		$refund_default_subject  = sprintf( _x( 'Refund for Invoice #%s', 'wp_admin', 'activemember360' ), '%%invoice_id%%' );
		$refund_default_template = $this->refund_default_template();

		Redux::setSection(
			$this->opt_name_payment_settings,
			[
				'id'          => 'section_payment_refund_settings',
				'title'       => _x( 'Refund Settings', 'wp_admin', 'activemember360' ),
				'desc'        => _x( 'Refund template of ActiveMember360\'s integrated payment extension.', 'wp_admin', 'activemember360' ),
				'icon'        => 'el el-lines',
				'subsection'  => FALSE,
				'fields'      => [
					[
						'id'      => 'refund_email_subject',
						'title'   => _x( 'Subject', 'wp_admin', 'activemember360' ),
						'type'    => 'text',
						'desc'    => sprintf( _x( 'The subject used for the refund sent by email. Supported placeholders: %s', 'wp_admin', 'activemember360' ), $supported_subject_placeholders ),
						'default' => $refund_default_subject
					],
					[
						'id'      => 'refund_template',
						'type'    => 'ace_editor',
						'mode'    => 'html',
						'theme'   => 'chrome',
						'title'   => _x( 'Refund template', 'wp_admin', 'activemember360' ),
						'desc'    => sprintf( _x( 'Create your HTML refund template. Supported placeholders: %s', 'wp_admin', 'activemember360' ), $supported_body_placeholders ),
						'default' => $refund_default_template
					]
				]
			]
		);

		$stored_invoice_template = trim( Redux::getOption( $this->opt_name_payment_settings, 'invoice_template' ) );
		IF ( empty( $stored_invoice_template ) ) :
			Redux::setOption( $this->opt_name_payment_settings, 'invoice_template', $invoice_default_template );
		ENDIF;

		$stored_refund_template = trim( Redux::getOption( $this->opt_name_payment_settings, 'refund_template' ) );
		IF ( empty( $stored_refund_template ) ) :
			Redux::setOption( $this->opt_name_payment_settings, 'refund_template', $refund_default_template );
		ENDIF;
	}

	public function refund_default_template()
	{
		return '
			<!doctype html>
			<html>
				<head>
				<meta charset="utf-8">
				<title>Refund for Invoice #%%invoice_id%%</title>
				' . $this->html_template_css() . '
				<link rel="license" href="http://www.opensource.org/licenses/mit-license/">
				</head>
				<body>
					<header>
					<h1>Refund for Invoice #%%invoice_id%%</h1>
					<address>%%company_address%%</address>
					<span><img src="%%company_logo%%" /></span>
					</header>
					<article>
						<address>%%billing_address%%</address>
						<table class="meta">
							<tr>
								<th>Refund for Invoice #</th>
								<td>%%invoice_id%%</td>
							</tr>
							<tr>
								<th>Order #</th>
								<td>%%order_id%%</td>
							</tr>
							<tr>
								<th>Date</th>
								<td>%%order_date%%</td>
							</tr>
							<tr>
								<th>VAT ID</th>
								<td>%%company_vat_id%%</td>
							</tr>
							<tr>
								<th><span>Amount Due</span></th>
								<td><span>%%order_total%%</span></td>
							</tr>
						</table>
						<table class="inventory">
							<thead>
								<tr>
									<th><span>Quantity</span></th>
									<th><span>Item</span></th>
									<th><span>Price</span></th>
								</tr>
							</thead>
							<tbody>
								<tr>
									<td><span>1</span></td>
									<td><span>%%item_name%%</span></td>
									<td><span>%%item_price%%</span></td>
								</tr>
							</tbody>
						</table>
						<table class="balance">
							<tr>
								<th><span>Net Total</span></th>
								<td><span>%%net_total%%</span></td>
							</tr>
							<tr>
								<th><span>Sales Tax</span></th>
								<td><span>%%sales_tax%%</span></td>
							</tr>
							<tr>
								<th><span>Total</span></th>
								<td><span>%%order_total%%</span></td>
							</tr>
						</table>
						<table class="balance">
							<tr>
								<th><span>Refund Total</span></th>
								<td><span>%%refund_total%%</span></td>
							</tr>
							<tr>
								<td class="inner-table" colspan="2">%%order_refunds%%</td>
							</tr>
						</table>
					</article>
					<aside>
						<h1><span>Additional Notes</span></h1>
						<div><p>Additional notes to this refund.</p></div>
					</aside>
				</body>
			</html>
		';
	}

	public function invoice_default_template()
	{
		return '
			<!doctype html>
			<html>
				<head>
					<meta charset="utf-8">
					<title>Invoice #%%invoice_id%%</title>
					' . $this->html_template_css() . '
					<link rel="license" href="http://www.opensource.org/licenses/mit-license/">
				</head>
				<body>
					<header>
						<h1>Invoice #%%invoice_id%%</h1>
						<address>
							%%company_address%%
						</address>
						<span><img src="%%company_logo%%" /></span>
					</header>
					<article>
						<address>
							%%billing_address%%
						</address>
						<table class="meta">
							<tr>
								<th>Invoice #</th>
								<td>%%invoice_id%%</td>
							</tr>
							<tr>
								<th>Order #</th>
								<td>%%order_id%%</td>
							</tr>
							<tr>
								<th>Date</th>
								<td>%%order_date%%</td>
							</tr>
							<tr>
								<th>VAT ID</th>
								<td>%%company_vat_id%%</td>
							</tr>
							<tr>
								<th><span>Amount Due</span></th>
								<td><span>%%order_total%%</span></td>
							</tr>
						</table>
						<table class="inventory">
							<thead>
								<tr>
									<th><span>Quantity</span></th>
									<th><span>Item</span></th>
									<th><span>Price</span></th>
								</tr>
							</thead>
							<tbody>
								<tr>
									<td><span>1</span></td>
									<td><span>%%item_name%%</span></td>
									<td><span>%%item_price%%</span></td>
								</tr>
							</tbody>
						</table>
						<table class="balance">
							<tr>
								<th><span>Net Total</span></th>
								<td><span>%%net_total%%</span></td>
							</tr>
							<tr>
								<th><span>Sales Tax</span></th>
								<td><span>%%sales_tax%%</span></td>
							</tr>
							<tr>
								<th><span>Total</span></th>
								<td><span>%%order_total%%</span></td>
							</tr>
						</table>
					</article>
					<aside>
						<h1><span>Additional Notes</span></h1>
						<div>
							<p>Additional notes to this invoice.</p>
						</div>
					</aside>
				</body>
			</html>
		';
	}

	public function html_template_css()
	{
		return '
			<style>
			
			* {
				border: 0;
				box-sizing: content-box;
				color: inherit;
				font-family: inherit;
				font-size: inherit;
				font-style: inherit;
				font-weight: inherit;
				line-height: inherit;
				list-style: none;
				margin: 0;
				padding: 0;
				text-decoration: none;
				vertical-align: top;
			}
			
			
			
			h1 { font: bold 100% sans-serif; letter-spacing: 0.5em; text-align: center; text-transform: uppercase; }
			
			
			
			table { font-size: 75%; table-layout: fixed; width: 100%; }
			table { border-collapse: separate; border-spacing: 2px; }
			th, td { border-width: 1px; padding: 0.5em; position: relative; text-align: left; }
			th, td { border-radius: 0.25em; border-style: solid; }
			th { background: #EEE; border-color: #BBB; }
			td { border-color: #DDD; }
			
			
			
			html { font: 16px/1 "Open Sans", sans-serif; overflow: auto; padding: 0.5in; }
			html { background: #999; cursor: default; }
			
			body { box-sizing: border-box; height: 11in; margin: 0 auto; overflow: hidden; padding: 0.5in; width: 8.5in; }
			body { background: #FFF; border-radius: 1px; box-shadow: 0 0 1in -0.25in rgba(0, 0, 0, 0.5); }
			
			
			
			header { margin: 0 0 3em; }
			header:after { clear: both; content: ""; display: table; }
			
			header h1 { background: #000; border-radius: 0.25em; color: #FFF; margin: 0 0 1em; padding: 0.5em 0; }
			header address { float: left; font-size: 75%; font-style: normal; line-height: 1.25; margin: 0 1em 1em 0; }
			header address p { margin: 0 0 0.25em; }
			header span, header img { display: block; float: right; }
			header span { margin: 0 0 1em 1em; max-height: 25%; max-width: 60%; position: relative; }
			header img { max-height: 100%; max-width: 100%; }
			
			
			
			article, article address, table.meta, table.inventory { margin: 0 0 3em; }
			article:after { clear: both; content: ""; display: table; }
			article h1 { clip: rect(0 0 0 0); position: absolute; }
			
			article address { float: left; font-size: 125%; }
			
			
			
			table.meta, table.balance { float: right; width: 36%; }
			table.meta:after, table.balance:after { clear: both; content: ""; display: table; }
			
			
			
			table.meta th, table.meta td { width: 50%; }
			table.meta td { text-align: right; }
			
			
			
			table.inventory { clear: both; width: 100%; }
			table.inventory th { font-weight: bold; text-align: center; }
			
			table.inventory th:nth-child(1) { text-align: left;  width: 12%; }
			table.inventory th:nth-child(2) { text-align: left;  width: 70%; }
			table.inventory th:nth-child(3) { text-align: right; width: 18%; }
			
			table.inventory td:nth-child(1) { text-align: left;}
			table.inventory td:nth-child(2) { text-align: left;}
			table.inventory td:nth-child(3) { text-align: right;}
			
			
			table.balance { clear: both; }
			table.balance th, table.balance td { width: 50%; }
			table.balance td { text-align: right; }
			table.balance td.inner-table { border:none;margin:0px;padding:0px;}
			table.balance td.inner-table table { margin:0px; padding:0px; font-size:100%; }
			
			
			
			aside h1 { border: none; border-width: 0 0 1px; margin: 0 0 1em; }
			aside h1 { border-color: #999; border-bottom-style: solid; }
			
			@media print {
				* { -webkit-print-color-adjust: exact; }
				html { background: none; padding: 0; }
				body { box-shadow: none; margin: 0; }
				span:empty { display: none; }
			}
			
			@page { margin: 0; }
			</style>
		';

	}

	public function subscription_settings_metabox( $metaboxes = [], $opt_name )
	{
		global $pagenow;

		$payment                   = apply_filters( 'mbr/payments/payment/settings/get', NULL );

		$live_secret_key           = ! empty( $payment['stripe_live_secret'] ) ? $payment['stripe_live_secret'] : NULL;
		$test_secret_key           = ! empty( $payment['stripe_test_secret'] ) ? $payment['stripe_test_secret'] : NULL;

		$stripe_live_plans         = [];
		$stripe_test_plans         = [];
		$stripe_setup_complete     = TRUE;
		$stripe_is_available       = TRUE;

		DO {

			IF ( empty( $live_secret_key ) AND empty( $test_secret_key ) ) :
				$stripe_setup_complete = FALSE;
				break;
			ENDIF;

			require_once MBR_REDUX_STRIPE_INIT;

			IF ( $live_secret_key ) :
				do_action( 'mbr/payments/stripe/api_key/set', $live_secret_key );
				
				$plans = apply_filters( 'mbr/payments/stripe/plans/all/get', NULL );
				FOREACH ( $plans['data'] AS $plan ) :
					$planname   = $plan['id'];
					IF ( ! empty( $plan['name'] ) ) :
						$planname = $plan['name'];
					ELSEIF ( ! empty( $plan['nickname'] ) ) :
						$planname = $plan['nickname'];
					ELSEIF ( ! empty( $plan['product'] ) ) :
						$planname = $plan['product'];
					ENDIF;
					$stripe_live_plans[ $plan['id'] ] = $planname;
				ENDFOREACH;
			ENDIF;

			IF ( $test_secret_key ) :
				do_action( 'mbr/payments/stripe/api_key/set', $test_secret_key );
				
				$plans = apply_filters( 'mbr/payments/stripe/plans/all/get', NULL );
				FOREACH ( $plans['data'] AS $plan ) :
					$planname   = $plan['id'];
					IF ( ! empty( $plan['name'] ) ) :
						$planname = $plan['name'];
					ELSEIF ( ! empty( $plan['nickname'] ) ) :
						$planname = $plan['nickname'];
					ELSEIF ( ! empty( $plan['product'] ) ) :
						$planname = $plan['product'];
					ENDIF;
					$stripe_test_plans[ $plan['id'] ] = $planname;
				ENDFOREACH;
			ENDIF;

		} WHILE ( FALSE );

		// If Stripe is not usable for any reason, only show a single section with the appropriate error message.
		IF ( empty( $stripe_setup_complete ) OR empty( $stripe_is_available ) ) :

			IF ( empty( $stripe_setup_complete ) ) :
				$desc               = sprintf( _x( 'This section is not available because neither "live" nor "test" Stripe API keys have been set. Please go to the %sPayment Settings%s section to continue.', 'wp_admin', 'activemember360' ), '<a href="' . admin_url( 'admin.php?page=mbr360-payment' ) . '"><strong>', '</strong></a>' );
			ELSEIF ( empty( $stripe_is_available ) ) :
				$desc               = sprintf( _x( 'This section is not available because Stripe could not be loaded. Please go to the %sPayment Settings%s section to ensure all settings are correct.',       'wp_admin', 'activemember360' ), '<a href="' . admin_url( 'admin.php?page=mbr360-payment' ) . '"><strong>', '</strong></a>' );
			ELSEIF ( empty( $stripe_live_plans ) AND empty( $stripe_test_plans ) ) :
				$desc               = sprintf( _x( 'This section is not available because there are currently neither "live" nor "test" plans defined in your Stripe Dashboard. Please go to your %sStripe Dashboard%s and create one.', 'wp_admin', 'activemember360' ), '<a href="https://dashboard.stripe.com/dashboard/"><strong>', '</strong></a>' );
			ENDIF;

			$boxSections[]        = [
				'title'             => _x( 'Subscriptions Settings', 'wp_admin', 'activemember360' ),
				'icon'              => 'el-icon-cog',
				'desc'              => $desc,
				'fields'            => [
					[
						'id'            => '_mbr_oitem_type',
						'type'          => 'text',
						'hidden'        => TRUE,
						'default'       => 'subscription'
					]
				]
			];

		ELSE :
			IF ( $pagenow == 'post.php' ) :
				do_action( 'mbr/site_tags/fetch' );
				do_action( 'mbr/site_lists/fetch' );
				do_action( 'mbr/site_automations/fetch' );
			ENDIF;

			$plans_not_found_text = _x( 'No plans were found in your Stripe configuration. Please create one in your %s.', 'wp_admin', 'activemember360' );
			$plans_found_text     = _x( 'Please select one of the available plans. To create an additional plan, please create one in your %s.', 'wp_admin', 'activemember360' );
			$live_stripe_desc     = empty( $stripe_live_plans ) ? sprintf( $plans_not_found_text, MBR_STRIPE_DASHBOARD_LIVE ) : sprintf( $plans_found_text, MBR_STRIPE_DASHBOARD_LIVE );
			$test_stripe_desc     = empty( $stripe_test_plans ) ? sprintf( $plans_not_found_text, MBR_STRIPE_DASHBOARD_TEST ) : sprintf( $plans_found_text, MBR_STRIPE_DASHBOARD_TEST );

			$boxSections[]        = [
				'title'             => _x( 'Subscriptions Settings', 'wp_admin', 'activemember360' ),
				'icon'              => 'el-icon-cog',
				'desc'              => _x( 'Please complete the items / settings below to enable the sale of this subscription to your visitors and customers.', 'wp_admin', 'activemember360' ),
				'fields'            => [
					[
						'id'            => '_mbr_oitem_type',
						'type'          => 'text',
						'hidden'        => TRUE,
						'default'       => 'subscription'
					],
					[
						'id'            => '_mbr_oitem_status',
						'type'          => 'button_set',
						'title'         => _x( 'Product Status', 'wp_admin', 'activemember360' ),
						'subtitle'      => _x( 'Activate/deactivate this product', 'wp_admin', 'activemember360' ),
						'options'       => [ 0 => _x( 'Inactive', 'wp_admin', 'activemember360' ), 1 => _x( 'Live Mode', 'wp_admin', 'activemember360' ), 2 => _x( 'Test Mode', 'wp_admin', 'activemember360' ) ],
						'default'       => 0
					],
				]
			];

			$boxSections[]        = [
				'title'             => _x( 'Stripe Plans', 'wp_admin', 'activemember360' ),
				'desc'              => _x( 'Select the Stripe plans to use for live and test purchase modes.', 'wp_admin', 'activemember360' ),
				'icon'              => 'el-icon-calendar',
				'fields'            => [
					[
						'id'            => 'info_critical_' . uniqid(),
						'type'          => 'info',
						'required'      => [ '_mbr_oitem_status', 'equals', '0' ],
						'style'         => 'critical',
						'icon'          => 'el el-warning-sign',
						'title'         => MBR_REDUX_INACTIVE_OITEM_HDR,
						'desc'          => MBR_REDUX_INACTIVE_OITEM_MSG,
					],
					[
						'id'            => '_mbr_oitem_live_regular_plan',
						'type'          => 'select',
						'title'         => _x( 'Regular subscription plan (LIVE mode)', 'wp_admin', 'activemember360' ),
						'desc'          => $live_stripe_desc,
						'options'       => empty( $stripe_live_plans ) ? [] : $stripe_live_plans,
						'select2'       => [ 'allowClear' => FALSE ],
						'required'      => [ '_mbr_oitem_status', 'equals', '1' ]
					],
					[
						'id'            => '_mbr_oitem_live_sale_plan',
						'type'          => 'select',
						'title'         => _x( 'Sale subscription plan (LIVE mode)', 'wp_admin', 'activemember360' ),
						'desc'          => $live_stripe_desc,
						'options'       => empty( $stripe_live_plans ) ? [] : $stripe_live_plans,
						'select2'       => [ 'allowClear' => TRUE ],
						'required'      => [ '_mbr_oitem_status', 'equals', '1' ],
					],
					[
						'id'            => '_mbr_oitem_test_regular_plan',
						'type'          => 'select',
						'title'         => _x( 'Regular subscription plan (TEST mode)', 'wp_admin', 'activemember360' ),
						'desc'          => $test_stripe_desc,
						'options'       => empty( $stripe_test_plans) ? [] : $stripe_test_plans,
						'select2'       => [ 'allowClear' => FALSE ],
						'required'      => [ '_mbr_oitem_status', 'equals', '2' ]
					],
					[
						'id'            => '_mbr_oitem_test_sale_plan',
						'type'          => 'select',
						'title'         => _x( 'Sale subscription plan (TEST mode)', 'wp_admin', 'activemember360' ),
						'desc'          => $test_stripe_desc,
						'options'       => empty( $stripe_test_plans ) ? [] : $stripe_test_plans,
						'select2'       => [ 'allowClear' => TRUE ],
						'required'      => [ '_mbr_oitem_status', 'equals', '2' ],
					],
				]
			];

			$boxSections[]        = apply_filters( 'mbr/payments/redux/section/box/actions', 'purchase' );
			$boxSections[]        = apply_filters( 'mbr/payments/redux/section/box/actions', 'trial' );
			$boxSections[]        = apply_filters( 'mbr/payments/redux/section/box/actions', 'reactivation' );
			$boxSections[]        = apply_filters( 'mbr/payments/redux/section/box/actions', 'failed' );
			$boxSections[]        = apply_filters( 'mbr/payments/redux/section/box/actions', 'cancel' );
			$boxSections[]        = apply_filters( 'mbr/payments/redux/section/box/actions', 'refund' );

			$boxSections[]        = [
				'title'             => _x( 'Checkout Template', 'wp_admin', 'activemember360' ),
				'desc'              => _x( 'Select the default checkout template.', 'wp_admin', 'activemember360' ),
				'icon'              => 'el-icon-lines',
				'fields'            => [
					[
						'id'            => '_mbr_oitem_checkout_template',
						'type'          => 'select',
						'title'         => _x( 'Default Checkout Template', 'wp_admin', 'activemember360' ),
						'data'          => 'posts',
						'args'          => [ 'post_type' => MBRCUSTCHECKOUTTPL ]
					]
				]
			];

		ENDIF;

		$metaboxes[]            = [
			'id'                  => 'subscription-settings',
			'title'               => _x( 'ActiveMember360 Subscription Settings', 'wp_admin', 'activemember360' ),
			'post_types'          => [ MBRCUSTSUBSCRIPTIONS ],
			'position'            => 'normal',
			'priority'            => 'high',
			'sidebar'             => FALSE,
			'sections'            => $boxSections
		];

		return $metaboxes;
	}

	public function product_settings_metabox( $metaboxes = [], $opt_name )
	{
		global $pagenow;

		IF ( $pagenow == 'post.php' ) :
			do_action( 'mbr/site_tags/fetch' );
			do_action( 'mbr/site_lists/fetch' );
			do_action( 'mbr/site_automations/fetch' );
		ENDIF;

		$payment                = apply_filters( 'mbr/payments/payment/settings/get', NULL );

		$live_secret_key        = ! empty( $payment['stripe_live_secret'] ) ? $payment['stripe_live_secret'] : NULL;
		$test_secret_key        = ! empty( $payment['stripe_test_secret'] ) ? $payment['stripe_test_secret'] : NULL;

		// If Stripe is not usable for any reason, only show a single section with the appropriate error message.
		IF ( empty( $live_secret_key ) AND empty( $test_secret_key ) ) :

			$desc                 = sprintf( _x( 'This section is not available because neither "live" nor "test" Stripe API keys have been set. Please go to the %sPayment Settings%s section to continue.', 'wp_admin', 'activemember360' ), '<a href="' . admin_url( 'admin.php?page=mbr360-payment' ) . '"><strong>', '</strong></a>' );

			$boxSections[]        = [
				'title'             => _x( 'Product Settings', 'wp_admin', 'activemember360' ),
				'icon'              => 'el-icon-cog',
				'desc'              => $desc,
				'fields'            => [
					[
						'id'            => '_mbr_oitem_type',
						'type'          => 'text',
						'hidden'        => TRUE,
						'default'       => 'product'
					],
				]
			];

		ELSE :

			$boxSections[]        = [
				'title'             => _x( 'Product Settings', 'wp_admin', 'activemember360' ),
				'icon'              => 'el-icon-cog',
				'desc'              => _x( 'Please complete the settings below to enable the sale of this product to your visitors and customers.', 'wp_admin', 'activemember360' ),
				'fields'            => [
					[
						'id'            => '_mbr_oitem_type',
						'type'          => 'text',
						'hidden'        => TRUE,
						'default'       => 'product'
					],
					[
						'id'            => '_mbr_oitem_status',
						'type'          => 'button_set',
						'title'         => _x( 'Product Status', 'wp_admin', 'activemember360' ),
						'subtitle'      => _x( 'Activate/deactivate this product', 'wp_admin', 'activemember360' ),
						'options'       => [ 0 => _x( 'Inactive', 'wp_admin', 'activemember360' ), 1 => _x( 'Live Mode', 'wp_admin', 'activemember360' ), 2 => _x( 'Test Mode', 'wp_admin', 'activemember360' ) ],
						'default'       => 0
					],
				]
			];

			$boxSections[]        = [
				'title'             => _x( 'Pricing', 'wp_admin', 'activemember360' ),
				'desc'              => _x( 'Set price points below for regular and sales prices', 'wp_admin', 'activemember360' ),
				'icon'              => 'el-icon-usd',
				'fields'            => [
					[
						'id'            => '_info_critical_pricing',
						'type'          => 'info',
						'style'         => 'critical',
						'icon'          => 'el el-warning-sign',
						'title'         => MBR_REDUX_INACTIVE_OITEM_HDR,
						'desc'          => MBR_REDUX_INACTIVE_OITEM_MSG,
						'required'      => [ '_mbr_oitem_status', 'equals', '0' ],
					],
					[
						'id'            => '_mbr_oitem_live_regular_price',
						'type'          => 'text',
						'title'         => _x( 'Regular price (LIVE mode)', 'wp_admin', 'activemember360' ),
						'desc'          => _x( 'The regular price of your product e.g. 10.99', 'wp_admin', 'activemember360' ),
						'placeholder'   => _x( 'e.g. 10.99', 'wp_admin', 'activemember360' ),
						'validate'      => 'numeric',
						'required'      => [ '_mbr_oitem_status', 'equals', '1' ]
					],
					[
						'id'            => '_mbr_oitem_live_sale_price',
						'type'          => 'text',
						'title'         => _x( 'Sale price (LIVE mode)', 'wp_admin', 'activemember360' ),
						'desc'          => _x( 'The sale price of your product e.g. 7.99. Leave empty to disable.', 'wp_admin', 'activemember360' ),
						'placeholder'   => _x( 'e.g. 7.99', 'wp_admin', 'activemember360' ),
						'validate'      => 'numeric',
						'required'      => [ '_mbr_oitem_status', 'equals', '1' ]
					],
					[
						'id'            => '_mbr_oitem_test_regular_price',
						'type'          => 'text',
						'title'         => _x( 'Regular price (TEST mode)', 'wp_admin', 'activemember360' ),
						'desc'          => _x( 'The regular price of your "test" product e.g. 10.99', 'wp_admin', 'activemember360' ),
						'validate'      => 'numeric',
						'placeholder'   => _x( 'e.g. 10.99', 'wp_admin', 'activemember360' ),
						'required'      => [ '_mbr_oitem_status', 'equals', '2' ]
					],
					[
						'id'            => '_mbr_oitem_test_sale_price',
						'type'          => 'text',
						'title'         => _x( 'Sale price (TEST mode)', 'wp_admin', 'activemember360' ),
						'desc'          => _x( 'The sale price of your "test" product e.g. 7.99. Leave empty to disable.', 'wp_admin', 'activemember360' ),
						'validate'      => 'numeric',
						'placeholder'   => _x( 'e.g. 7.99', 'wp_admin', 'activemember360' ),
						'required'      => [ '_mbr_oitem_status', 'equals', '2' ]
					],
				]
			];

			$boxSections[]        = apply_filters( 'mbr/payments/redux/section/box/actions', 'purchase' );
			$boxSections[]        = apply_filters( 'mbr/payments/redux/section/box/actions', 'refund' );

			$boxSections[]        = [
				'title'             => _x( 'Checkout Template', 'wp_admin', 'activemember360' ),
				'desc'              => _x( 'Select the default checkout template.', 'wp_admin', 'activemember360' ),
				'icon'              => 'el-icon-lines',
				'fields'            => [
					[
						'id'            => '_mbr_oitem_checkout_template',
						'type'          => 'select',
						'title'         => _x( 'Default Checkout Template', 'wp_admin', 'activemember360' ),
						'data'          => 'posts',
						'args'          => [ 'post_type' => MBRCUSTCHECKOUTTPL ]
					]
				]
			];

		ENDIF;

		$metaboxes[]            = [
			'id'                  => 'product-settings',
			'title'               => _x( 'ActiveMember360 Product Settings', 'wp_admin', 'activemember360' ),
			'post_types'          => [ MBRCUSTPRODUCTS ],
			'position'            => 'normal',
			'priority'            => 'high',
			'sidebar'             => FALSE,
			'sections'            => $boxSections
		];

		return $metaboxes;
	}

	public function checkouttpl_settings_metabox( $metaboxes = [], $opt_name ) {
		$boxSections[]       = [
			'title'            => _x( 'Layout Settings', 'wp_admin', 'activemember360' ),
			'icon'             => 'el-icon-website',
			'fields'           => [
				[
					'id'           => '_mbr_checkout_template',
					'title'        => _x( 'Select Template', 'wp_admin', 'activemember360' ),
					'type'         => 'image_select',
					'width'        => '150',
					'height'       => '230',
					'options'      => [
						'checkout-1' => [ 'img' => plugins_url( '/templates/assets/img/checkout-1.png', MBRNAME ), 'alt' => 'Checkout 1' ],
						'checkout-2' => [ 'img' => plugins_url( '/templates/assets/img/checkout-2.png', MBRNAME ), 'alt' => 'Checkout 2' ],
						'checkout-3' => [ 'img' => plugins_url( '/templates/assets/img/checkout-3.png', MBRNAME ), 'alt' => 'Checkout 3' ]
					],
					'default'      => 'checkout-1'
				],
				[
					'id'           => '_mbr_checkout_display_billing_fields',
					'title'        => _x( 'Display billings fields', 'wp_admin', 'activemember360' ),
					'type'         => 'switch',
					'on'           => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'          => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'      => 1
				],
				[
					'id'           => '_mbr_checkout_display_phone_number',
					'title'        => _x( 'Display phone number', 'wp_admin', 'activemember360' ),
					'type'         => 'switch',
					'on'           => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'          => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'      => 1
				],
				[
					'id'           => '_mbr_checkout_trust_bar',
					'type'         => 'media',
					'title'        => _x( 'Trustbar', 'wp_admin', 'activemember360' ),
					'default'      => ''
				],
				[
					'id'           => '_mbr_checkout_button_text',
					'title'        => _x( 'Checkout button text', 'wp_admin', 'activemember360' ),
					'type'         => 'text',
					'default'      => _x( 'Buy now!', 'wp_admin', 'activemember360' )
				],
				[
					'id'           => '_mbr_checkout_display_term_condition_checkbox',
					'title'        => _x( 'Display checkbox for terms and condition', 'wp_admin', 'activemember360' ),
					'type'         => 'switch',
					'on'           => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'          => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'      => 0
				],
				[
					'id'           => '_mbr_checkout_terms_and_condition_text',
					'title'        => _x( 'Terms and conditions text', 'wp_admin', 'activemember360' ),
					'type'         => 'text',
					'transparent'  => FALSE,
					'required'     => [ '_mbr_checkout_display_term_condition_checkbox', 'not', 0 ]
				]
			]
		];

		$boxSections[]       = [
			'title'            => _x( 'Color and Font Settings', 'wp_admin', 'activemember360' ),
			'icon'             => 'el-icon-brush',
			'fields'           => [
				[
					'id'           => '_mbr_checkout_background_color',
					'title'        => _x( 'Background color', 'wp_admin', 'activemember360' ),
					'type'         => 'color',
					'transparent'  => FALSE,
					'validate'     => 'color'
				],
				[
					'id'           => '_mbr_checkout_header_color',
					'title'        => _x( 'Header color', 'wp_admin', 'activemember360' ),
					'type'         => 'color',
					'transparent'  => FALSE,
					'validate'     => 'color',
					'required'     => [ '_mbr_checkout_template', 'not', 'checkout-3' ]
				],
				[
					'id'           => '_mbr_checkout_button_color',
					'title'        => _x( 'Button color', 'wp_admin', 'activemember360' ),
					'type'         => 'color',
					'transparent'  => FALSE,
					'validate'     => 'color'
				],
				[
					'id'           => '_mbr_checkout_footer_color',
					'title'        => _x( 'Footer color', 'wp_admin', 'activemember360' ),
					'type'         => 'color',
					'transparent'  => FALSE,
					'validate'     => 'color'
				],
				[
					'id'           => '_mbr_checkout_font_color',
					'title'        => _x( 'Font color', 'wp_admin', 'activemember360' ),
					'type'         => 'color',
					'transparent'  => FALSE,
					'validate'     => 'color'
				],
				[
					'id'           => '_mbr_checkout_font_family',
					'title'        => _x( 'Font family', 'wp_admin', 'activemember360' ),
					'type'         => 'text',
				],
				[
					'id'           => '_mbr_checkout_font_size',
					'title'        => _x( 'Font size', 'wp_admin', 'activemember360' ),
					'type'         => 'spinner',
					'min'          => 8,
					'max'          => 50,
					'default'      => 13
				]
			]
		];

		$metaboxes[]         = [
			'id'               => 'checkouttpl-settings',
			'title'            => _x( 'ActiveMember360 Checkout Template Settings', 'wp_admin', 'activemember360' ),
			'post_types'       => [ MBRCUSTCHECKOUTTPL ],
			'position'         => 'normal',
			'priority'         => 'high',
			'sidebar'          => FALSE,
			'sections'         => $boxSections
		];

		return $metaboxes;
	}

}
$GLOBALS['mbr_payment_admin'] = mbr_payment_admin::getInstance();
