<?php
final class mbr_plugin_options EXTENDS mbr_redux_base_class
{
	use TRAIT__getInstance;

	public $opt_name  = 'mbr_plugin_options';
	public $page_slug = 'mbr360-plugin-options';

	public function _load()
	{
		IF ( is_admin() ) :
			add_action( 'plugins_loaded',                                        [ $this, 'init_admin_panel' ], 99999 );
			add_action( 'plugins_loaded',                                        [ $this, 'init_sections' ],    99999 );
			add_filter( "redux/{$this->opt_name}/panel/template/header.tpl.php", [ $this, 'alternate_header' ] );
			add_action( "redux/options/{$this->opt_name}/validate",              [ $this, 'validate_save_settings' ] );
			add_action( "redux/options/{$this->opt_name}/saved",                 [ $this, 'backup_settings' ],  9999 );
		ENDIF;

		add_action( 'mbr/plugin_options/save',                                 [ $this, 'save_plugin_options' ],                10, 0 );
	}

	public function init_admin_panel()
	{
		global $MBR_REDUX_HEADER_MODULE_NAMES;

		$MBR_REDUX_HEADER_MODULE_NAMES[ $this->opt_name ] = $admin_title = _x( 'Plugin Options', 'wp_admin', 'activemember360' );

		Redux::setExtensions( $this->opt_name, MBR_REDUX_EXTENSIONS_DIR );

		$page_title         = apply_filters( 'mbr/plugin_name', '' ) . ' ' . $admin_title;

		$arr_config         =  [
			'opt_name'        => $this->opt_name,
			'global_variable' => $this->opt_name,
			'page_slug'       => $this->page_slug,
			'page_parent'     => MBR_MENU_PARENT_PAGE,
			'page_title'      => $page_title,
			'display_name'    => $page_title,
			'admin_title'     => $admin_title,
			'menu_title'      => $admin_title,
		];

		$arr_config         = array_merge( $this->arr_default_sub_admin_panel_settings, $arr_config );

		Redux::setArgs( $this->opt_name, $arr_config );
	}

	public function init_sections()
	{
		global $pagenow, $MBR_OPTIONS;

		IF ( $pagenow <> 'options.php' AND $pagenow <> 'admin.php' OR ($pagenow == 'admin.php' AND ( empty( $_GET['page'] ) OR $_GET['page'] <> $this->page_slug ) ) ) :
			return;
		ENDIF;

		do_action( 'mbr/site_tags/fetch' );
		$arr_tags             = apply_filters( 'mbr/site_tags/get', NULL );

		do_action( 'mbr/site_automations/fetch' );
		$arr_automations      = apply_filters( 'mbr/site_automations/get', NULL );

		$arr_date_cust_fields = [];
		$obj_cust_fields      = apply_filters( 'mbr/api', 'list_field_view?ids=all' );
		IF ( $obj_cust_fields AND is_object( $obj_cust_fields ) AND $obj_cust_fields->result_code == 1 ) :
			FOREACH ( $obj_cust_fields AS $key => $o_field ) :
				IF ( is_object( $o_field ) AND isset( $o_field->perstag ) AND trim( $o_field->perstag ) <> '' AND isset( $o_field->type ) AND $o_field->type == 'date' ) :
					$arr_date_cust_fields[ trim( $o_field->perstag ) ] = $o_field->title;
				ENDIF;
			ENDFOREACH;
		ENDIF;

		$arr_pages          = [];
		$obj_pages          = apply_filters( 'mbr/pages/get', NULL );
		FOREACH ( $obj_pages AS $page ) :
			$arr_pages[ $page->ID ] = ! empty( $page->post_title ) ? $page->post_title : _x( 'No page title', 'wp_admin', 'activemember360' );
		ENDFOREACH;

		$arr_section_login_settings =
		[
			'id'               => 'section_login_settings',
			'title'            => _x( 'Login Settings', 'wp_admin', 'activemember360' ),
			'icon'             => 'el el-lock',
			'fields'           => [
				[
					'id'           => 'mbr_po_require_login',
					'title'        => _x( 'Always require login', 'wp_admin', 'activemember360' ),
					'type'         => 'switch',
					'on'           => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'          => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'      => FALSE
				],
				[
					'id'           => 'mbr_po_siteban_tag',
					'title'        => _x( 'Tag used to ban a subscriber from site', 'wp_admin', 'activemember360' ),
					'type'         => 'select',
					'options'      => $arr_tags,
					'placeholder'  => _x( 'Do not use a siteban tag', 'wp_admin', 'activemember360' )
				],
				[
					'id'           => 'mbr_po_login_without_membership',
					'title'        => _x( 'Allow login without a membership tag', 'wp_admin', 'activemember360' ),
					'type'         => 'switch',
					'on'           => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'          => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'      => FALSE
				],
				[
					'id'           => 'mbr_po_last_login_field',
					'title'        => _x( 'Name of "last login" custom field', 'wp_admin', 'activemember360' ),
					'type'         => 'select',
					'options'      => $arr_date_cust_fields,
					'placeholder'  => _x( 'Do not track last login', 'wp_admin', 'activemember360' )
				],
				[
					'id'           => 'mbr_po_login_automation',
					'title'        => _x( 'Automation to run on login', 'wp_admin', 'activemember360' ),
					'type'         => 'select',
					'options'      => $arr_automations,
					'placeholder'  => _x( 'Do not run an automation on login', 'wp_admin', 'activemember360' )
				],
				[
					'id'           => 'mbr_po_logout_automation',
					'title'        => _x( 'Automation to run on logout', 'wp_admin', 'activemember360' ),
					'type'         => 'select',
					'options'      => $arr_automations,
					'placeholder'  => _x( 'Do not run an automation on logout', 'wp_admin', 'activemember360' )
				],
				[
					'id'           => 'mbr_po_first_login_redirect',
					'title'        => _x( 'Page redirect in first login', 'wp_admin', 'activemember360' ),
					'type'         => 'select',
					'options'      => $arr_pages,
					'placeholder'  => _x( 'No specific redirect on first login', 'wp_admin', 'activemember360' )
				],
				[
					'id'           => 'mbr_po_login_notes',
					'title'        => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
					'full_width'   => TRUE,
					'type'         => 'textarea'
				]
			]
		];

		$arr_section_passwords =
		[
			'id'               => 'section_passwords',
			'title'            => _x( 'Passwords', 'wp_admin', 'activemember360' ),
			'icon'             => 'el el-key'
		];
		IF ( empty( $MBR_OPTIONS['mbr_encrypted_passwords'] ) ) :
			$arr_section_passwords['fields'][] =
			[
				'id'               => 'mbr_po_passwords_encryption',
				'title'            => _x( 'Store encrypted passwords in ActiveCampaign', 'wp_admin', 'activemember360' ),
				'desc'             => _x( 'WARNING: Switching the password encryption mode to &#039;<strong>YES</strong>&#039; is irreversible.', 'wp_admin', 'activemember360' ),
				'type'             => 'switch',
				'on'               => _x( 'Yes', 'wp_admin', 'activemember360' ),
				'off'              => _x( 'No',  'wp_admin', 'activemember360' ),
				'default'          => FALSE,
			];
		ELSE:
			$arr_section_passwords['fields'][] =
			[
				'id'               => 'mbr_po_passwords_encryption',
				'title'            => _x( 'Store encrypted passwords in ActiveCampaign', 'wp_admin', 'activemember360' ),
				'type'             => 'raw',
				'full_width'       => FALSE,
				'content'          => _x( 'Active', 'wp_admin', 'activemember360' ),
			];
		ENDIF;
		$arr_section_passwords['fields'][] =
		[
			'id'           => 'mbr_po_passwords_min_character_length',
			'title'        => _x( 'Minimum password characters', 'wp_admin', 'activemember360' ),
			'type'         => 'text',
			'default'      => 8,
			'validate'     => 'not_empty'
		];
		$arr_section_passwords['fields'][] =
		[
			'id'           => 'mbr_po_passwords_max_character_length',
			'title'        => _x( 'Maximum password characters', 'wp_admin', 'activemember360' ),
			'type'         => 'text',
			'default'      => 16,
			'validate'     => 'not_empty'
		];
		$arr_section_passwords['fields'][] =
		[
			'id'           => 'mbr_po_passwords_allowed_characters',
			'title'        => _x( 'Allowed character set', 'wp_admin', 'activemember360' ),
			'type'         => 'text',
			'default'      => apply_filters( 'mbr/password/char_base', NULL ),
			'validate'     => 'not_empty'
		];

		$arr_section_passwords['fields'][] =
		[
			'id'           => 'mbr_po_passwords_notes',
			'title'        => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
			'full_width'   => TRUE,
			'type'         => 'textarea'
		];

		$arr_section_logging =
		[
			'id'               => 'section_logging',
			'title'            => _x( 'Logging', 'wp_admin', 'activemember360' ),
			'icon'             => 'el el-th-list',
			'fields'           => [
				[
					'id'           => 'mbr_po_api_log',
					'title'        => _x( 'Enable API log', 'wp_admin', 'activemember360' ),
					'type'         => 'switch',
					'on'           => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'          => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'      => FALSE
				],
				[
					'id'           => 'mbr_po_api_log_max_calls',
					'title'        => _x( 'API log max. number of "latest" calls', 'wp_admin', 'activemember360' ),
					'type'         => 'text',
					'required'     => [ 'mbr_po_api_log', 'equals', 1 ],
					'default'      => 0,
					'validate'     => 'numeric'
				],
				[
					'id'           => 'mbr_po_webhook_log',
					'title'        => _x( 'Enable Webhook log', 'wp_admin', 'activemember360' ),
					'type'         => 'switch',
					'on'           => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'          => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'      => FALSE
				],
				[
					'id'           => 'mbr_po_webhook_log_max_calls',
					'title'        => _x( 'Webhook max. number of "latest" calls', 'wp_admin', 'activemember360' ),
					'type'         => 'text',
					'required'     => [ 'mbr_po_webhook_log', 'equals', 1 ],
					'default'      => 0,
					'validate'     => 'numeric'
				],
				[
					'id'           => 'mbr_po_logging_notes',
					'title'        => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
					'full_width'   => TRUE,
					'type'         => 'textarea'
				]
			]
		];

		$arr_section_special_pages =
		[
			'id'               => 'section_special_pages',
			'title'            => _x( 'Special Pages', 'wp_admin', 'activemember360' ),
			'icon'             => 'el el-book',
			'fields'           => [
				[
					'id'           => 'mbr_po_login_page',
					'title'        => _x( 'Select your login page', 'wp_admin', 'activemember360' ),
					'type'         => 'select',
					'options'      => $arr_pages,
					'placeholder'  => _x( 'Default WP login page', 'wp_admin', 'activemember360' )
				],
				[
					'id'           => 'mbr_po_registration_page',
					'title'        => _x( 'Select your registration page', 'wp_admin', 'activemember360' ),
					'type'         => 'select',
					'options'      => $arr_pages,
					'placeholder'  => _x( 'Default WP registration page', 'wp_admin', 'activemember360' )
				],
				[
					'id'           => 'mbr_po_registration_external_url',
					'title'        => _x( 'OR URL to an external page', 'wp_admin', 'activemember360' ),
					'type'         => 'text',
					'validate'     => 'url',
				],
				[
					'id'           => 'mbr_po_excerpt_upsell_page',
					'title'        => _x( 'Select your "excerpt" upsell page', 'wp_admin', 'activemember360' ),
					'type'         => 'select',
					'options'      => $arr_pages,
					'placeholder'  => _x( 'No "Excerpt" upsell page', 'wp_admin', 'activemember360' )
				],
				[
					'id'           => 'mbr_po_alternate_404_page',
					'title'        => _x( 'Alternate "404 - Page not found" page', 'wp_admin', 'activemember360' ),
					'type'         => 'select',
					'options'      => $arr_pages,
					'placeholder'  => _x( 'No alternate 404 page', 'wp_admin', 'activemember360' )
				],
				[
					'id'           => 'mbr_po_special_pages_notes',
					'title'        => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
					'full_width'   => TRUE,
					'type'         => 'textarea'
				]
			]
		];

		$arr_section_security_options =
		[
			'id'               => 'section_security_options',
			'title'            => _x( 'Security Options', 'wp_admin', 'activemember360' ),
			'icon'             => 'el el-globe',
			'fields'           => [
				[
					'id'           => 'mbr_po_prevent_concurrent_login',
					'title'        => _x( 'Prevent concurrent logins', 'wp_admin', 'activemember360' ),
					'type'         => 'switch',
					'on'           => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'          => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'      => FALSE
				],
				[
					'id'           => 'mbr_po_email_detected_concurrent_logins',
					'title'        => _x( 'Email when concurrent login is detected', 'wp_admin', 'activemember360' ),
					'type'         => 'switch',
					'on'           => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'          => _x( 'No',  'wp_admin', 'activemember360' ),
					'required'     => [ 'mbr_po_prevent_concurrent_login', 'equals', 1 ],
					'default'      => FALSE
				],
				[
					'id'           => 'mbr_po_limit_failed_login_attempts',
					'title'        => _x( 'Limit failed login attempts', 'wp_admin', 'activemember360' ),
					'type'         => 'switch',
					'on'           => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'          => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'      => FALSE
				],
				[
					'id'           => 'mbr_po_limit_failed_login_attempts_max',
					'title'        => _x( 'Limit failed login attempts to', 'wp_admin', 'activemember360' ),
					'type'         => 'text',
					'required'     => [ 'mbr_po_limit_failed_login_attempts', 'equals', 1 ],
					'default'      => 0,
					'validate'     => 'numeric'
				],
				[
					'id'           => 'mbr_po_lock_logins_seconds',
					'title'        => _x( 'Set lockdown period after max. logins in seconds', 'wp_admin', 'activemember360' ),
					'type'         => 'text',
					'required'     => [ 'mbr_po_limit_failed_login_attempts', 'equals', 1 ],
					'default'      => 0,
					'validate'     => 'numeric'
				],
				[
					'id'           => 'mbr_po_auto_logout',
					'title'        => _x( 'Auto-logout after inactivity', 'wp_admin', 'activemember360' ),
					'type'         => 'switch',
					'on'           => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'          => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'      => FALSE

				],
				[
					'id'           => 'mbr_po_auto_logout_seconds',
					'title'        => _x( 'Auto-logout after inactivity in seconds', 'wp_admin', 'activemember360' ),
					'type'         => 'text',
					'required'     => [ 'mbr_po_auto_logout', 'equals', 1 ],
					'default'      => 0,
					'validate'     => 'numeric'
				],
				[
					'id'           => 'mbr_po_whitelist_ip_addresses',
					'title'        => _x( 'Whitelist IP addresses', 'wp_admin', 'activemember360' ),
					'type'         => 'repeater',
					'item_name'    => _x( 'IP Address', 'wp_admin', 'activemember360' ),
					'group_values' => TRUE,
					'sortable'     => FALSE,
					'limit'        => 9999,
					'fields'       => [
						[
							'id'       => 'ip_address',
							'title'    => _x( 'IP Address', 'wp_admin', 'activemember360'),
							'type'     => 'text',
						]
					]
				],
				[
					'id'           => 'mbr_po_security_codes',
					'title'        => _x( 'Security Codes', 'wp_admin', 'activemember360' ),
					'type'         => 'repeater',
					'item_name'    => _x( 'Security Code', 'wp_admin', 'activemember360' ),
					'group_values' => TRUE,
					'sortable'     => FALSE,
					'limit'        => 9999,
					'fields'       => [
						[
							'id'       => 'security_code',
							'title'    => _x( 'Security Code', 'wp_admin', 'activemember360'),
							'type'     => 'text',
						]
					]
				],
				[
					'id'           => 'mbr_po_autologin_module',
					'title'        => _x( 'Enable Autologin Module', 'wp_admin', 'activemember360' ),
					'type'         => 'switch',
					'on'           => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'          => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'      => FALSE
				],
				[
					'id'           => 'mbr_po_thrivecart_security_key',
					'title'        => _x( 'ThriveCart Secret Key', 'wp_admin', 'activemember360' ),
					'type'         => 'text',
				],
				[
					'id'           => 'mbr_po_jvzoo_security_key',
					'title'        => _x( 'JVZoo Secret Key', 'wp_admin', 'activemember360' ),
					'type'         => 'text',
				],
				[
					'id'           => 'mbr_po_clickbank_security_key',
					'title'        => _x( 'ClickBank Secret Key', 'wp_admin', 'activemember360' ),
					'type'         => 'text',
				],
				[
					'id'           => 'mbr_po_digistore24_security_key',
					'title'        => _x( 'DigiStore24 Secret Key', 'wp_admin', 'activemember360' ),
					'type'         => 'text',
				],
				[
					'id'           => 'mbr_po_security_options_notes',
					'title'        => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
					'full_width'   => TRUE,
					'type'         => 'textarea'
				]
			]
		];

		$arr_section_comments_options =
		[
			'id'                 => 'section_comments_options',
			'title'              => _x( 'Comments Options', 'wp_admin', 'activemember360' ),
			'icon'               => 'el el-comment-alt',
			'fields'             => [
				[
					'id'             => 'mbr_po_comments_options_enabled',
					'title'          => _x( 'Activate advanced comments options', 'wp_admin', 'activemember360' ),
					'type'           => 'switch',
					'on'             => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'            => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'        => FALSE
				],
				[
					'id'             => 'mbr_po_comments_display_reply_form',
					'title'          => _x( 'Display comments reply form', 'wp_admin', 'activemember360' ),
					'type'           => 'switch',
					'on'             => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'            => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'        => FALSE,
					'required'       => [ 'mbr_po_comments_options_enabled', 'equals', 1 ]
				],
				[
					'id'             => 'mbr_po_comments_display_in_protected_posts',
					'title'          => _x( 'Display comments in protected posts', 'wp_admin', 'activemember360' ),
					'type'           => 'select',
					'options'        => [ 0 => _x( 'Suppress comments altogether', 'wp_admin', 'activemember360' ), 1 => _x('Display comments', 'wp_admin', 'activemember360'), 2 => _x( 'Display comments excerpts only', 'wp_admin', 'activemember360' ) ],
					'default'        => 0,
					'select2'        => [ 'allowClear' => FALSE ],
					'required'       => [ 'mbr_po_comments_options_enabled', 'equals', 1 ]
				],
				[
					'id'             => 'mbr_po_comments_options_notes',
					'title'          => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
					'full_width'     => TRUE,
					'type'           => 'textarea'
				]
			]
		];

		$arr_section_content_access_options =
		[
			'id'                 => 'section_content_access_options',
			'title'              => _x( 'Content Access Options', 'wp_admin', 'activemember360' ),
			'icon'               => 'el el-arrow-right',
			'fields'             => [
				[
					'id'             => 'mbr_po_wp_admin_cap',
					'title'          => _x( 'Enable access to WP dashboard for capability', 'wp_admin', 'activemember360' ),
					'type'           => 'select',
					'data'           => 'capabilities',
					'default'        => 'activate_plugins',
					'select2'        => [ 'allowClear' => FALSE ]
				],
				[
					'id'             => 'mbr_po_suppress_wp_tool_bar',
					'title'          => _x( 'Suppress user "WP tool bar"', 'wp_admin', 'activemember360' ),
					'type'           => 'switch',
					'on'             => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'            => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'        => TRUE
				],
				[
					'id'             => 'mbr_po_access_options_notes',
					'title'          => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
					'full_width'     => TRUE,
					'type'           => 'textarea'
				]
			]
		];

		$arr_cache_refresh_frequency = [];
		FOR ( $x = 24; $x > 0; $x-- ) :
			$arr_cache_refresh_frequency[ $x ] = sprintf( _x( '%s hour(s)', 'wp_admin', 'activemember360' ), $x );
		ENDFOR;

		$arr_section_cache_options =
		[
			'id'                 => 'section_cache_options',
			'title'              => _x( 'Cache Options', 'wp_admin', 'activemember360' ),
			'icon'               => 'el el-refresh',
			'fields'             => [
				[
					'id'             => 'mbr_po_cache_refresh_frequency',
					'title'          => _x( 'Cache data refresh frequency', 'wp_admin', 'activemember360' ),
					'type'           => 'select',
					'options'        => $arr_cache_refresh_frequency,
					'default'        => 6
				],
				[
					'id'             => 'mbr_po_cache_options_notes',
					'title'          => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
					'full_width'     => TRUE,
					'type'           => 'textarea'
				]
			]
		];

		$arr_section_excerpts  = [
			'id'                 => 'section_excerpts',
			'title'              => _x( 'Excerpts', 'wp_admin', 'activemember360' ),
			'icon'               => 'el el-align-justify',
			'fields'             => [
				[
					'id'             => 'mbr_po_excerpts_force',
					'title'          => _x( 'Force excerpts at all times', 'wp_admin', 'activemember360' ),
					'type'           => 'switch',
					'on'             => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'            => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'        => FALSE
				],
				[
					'id'             => 'mbr_po_excerpts_blur_effect',
					'title'          => _x( 'Enable blur effect for excerpts', 'wp_admin', 'activemember360' ),
					'type'           => 'switch',
					'on'             => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'            => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'        => FALSE
				],
				[
					'id'             => 'mbr_po_excerpts_gradient_start_color',
					'title'          => _x( 'Excerpt gradient start color', 'wp_admin', 'activemember360' ),
					'type'           => 'color',
					'default'        => '#000000',
					'transparent'    => FALSE,
					'required'       => [ 'mbr_po_excerpts_blur_effect', 'equals', 1 ]
				],
				[
					'id'             => 'mbr_po_excerpts_gradient_end_color',
					'title'          => _x( 'Excerpt gradient end color', 'wp_admin', 'activemember360' ),
					'type'           => 'color',
					'default'        => '#ffffff',
					'transparent'    => FALSE,
					'required'       => [ 'mbr_po_excerpts_blur_effect', 'equals', 1 ]
				],
				[
					'id'             => 'mbr_po_excerpts_disable_links',
					'title'          => _x( 'Disable excerpt links', 'wp_admin', 'activemember360' ),
					'type'           => 'switch',
					'on'             => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'            => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'        => FALSE
				],
				[
					'id'             => 'mbr_po_excerpts_max_length',
					'title'          => _x( 'Max. length for WP-generated excerpts', 'wp_admin', 'activemember360' ),
					'type'           => 'text',
					'validate'       => 'numeric',
					'default'        => 55
				],
				[
					'id'             => 'mbr_po_excerpts_text',
					'title'          => _x( 'Excerpts text', 'wp_admin', 'activemember360' ),
					'type'           => 'editor',
					'args'           => [ 'media_buttons' => FALSE, 'wpautop' => FALSE ],
					'validate'       => 'not_empty',
					'default'        => _x( 'This is an excerpt. You do not have sufficient rights to view the full content.', 'wp_admin', 'activemember360' )
				],
				[
					'id'             => 'mbr_po_excerpts_notes',
					'title'          => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
					'full_width'     => TRUE,
					'type'           => 'textarea'
				]
			]
		];

		$arr_section_amazon_s3_expiring_link_options =
		[
			'id'                 => 'section_amazon_s3_expiring_link_options',
			'title'              => _x( 'AMAZON S3', 'wp_admin', 'activemember360' ),
			'heading'            => _x( 'AMAZON S3 expiring link options', 'wp_admin', 'activemember360' ),
			'icon'               => 'el el-link',
			'fields'             => [
				[
					'id'             => 'mbr_po_amazon_s3_access_key',
					'title'          => _x( 'S3 access key', 'wp_admin', 'activemember360' ),
					'type'           => 'text'
				],
				[
					'id'             => 'mbr_po_amazon_s3_secret_key',
					'title'          => _x( 'S3 secret key', 'wp_admin', 'activemember360' ),
					'type'           => 'text'
				],
				[
					'id'             => 'mbr_po_amazon_s3_default_expiration',
					'title'          => _x( 'Default expiration in seconds', 'wp_admin', 'activemember360' ),
					'type'           => 'text',
					'validate'       => 'numeric'
				],
				[
					'id'             => 'mbr_po_amazon_s3_default_host',
					'title'          => _x( 'Default S3 host', 'wp_admin', 'activemember360' ),
					'type'           => 'text'
				],
				[
					'id'             => 'mbr_po_amazon_s3_default_bucket',
					'title'          => _x( 'Default S3 bucket', 'wp_admin', 'activemember360' ),
					'type'           => 'text'
				],
				[
					'id'             => 'mbr_po_amazon_s3_notes',
					'title'          => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
					'full_width'     => TRUE,
					'type'           => 'textarea'
				]
			]
		];

		$arr_section_misc_options =
		[
			'id'                 => 'section_misc_options',
			'title'              => _x( 'Misc Options', 'wp_admin', 'activemember360' ),
			'heading'            => _x( 'Miscellaneous Options', 'wp_admin', 'activemember360' ),
			'icon'               => 'el el-plus-sign',
			'fields'             => [
				[
					'id'             => 'mbr_po_default_avatar_size',
					'title'          => _x( 'Default avatar size', 'wp_admin', 'activemember360' ),
					'type'           => 'text',
					'validate'       => 'numeric',
					'default'        => 48
				],
				[
					'id'             => 'mbr_po_suppress_empty_custom_menu_items',
					'title'          => _x( 'Suppress empty custom menu items', 'wp_admin', 'activemember360' ),
					'type'           => 'switch',
					'on'             => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'            => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'        => TRUE
				],
				[
					'id'             => 'mbr_po_hide_menu_to_admins',
					'title'          => _x( 'Apply "Hide menu" to admin users', 'wp_admin', 'activemember360' ),
					'type'           => 'switch',
					'on'             => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'            => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'        => FALSE
				],
				[
					'id'             => 'mbr_po_conditional_widgets',
					'title'          => _x( 'Enable conditional widget control', 'wp_admin', 'activemember360' ),
					'type'           => 'switch',
					'on'             => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'            => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'        => TRUE
				],
				[
					'id'             => 'mbr_po_failed_payment_text',
					'title'          => _x( 'Failed payment text', 'wp_admin', 'activemember360' ),
					'type'           => 'editor',
					'args'           => [ 'media_buttons' => FALSE, 'wpautop' => FALSE ],
					'validate'       => 'not_empty',
					'default'        => '<p>' . _x( 'There is a problem with your subscriber account. Your access is currently limited to public content. Please contact the operator of this site for more information.', 'wp_admin', 'activemember360' ) . '</p>'
				],
				[
					'id'             => 'mbr_po_misc_options_notes',
					'title'          => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
					'full_width'     => TRUE,
					'type'           => 'textarea'
				]
			]
		];

		$arr_section_mail_options =
		[
			'id'                 => 'section_mail_options',
			'title'              => _x( 'Mail Options', 'wp_admin', 'activemember360' ),
			'icon'               => 'el el-envelope',
			'fields'             => [
				[
					'id'             => 'mbr_po_mail_from_name',
					'title'          => _x( 'Default "Mail From" name', 'wp_admin', 'activemember360' ),
					'type'           => 'text',
					'default'        => 'Site Admin'
				],
				[
					'id'             => 'mbr_po_mail_from_email',
					'title'          => _x( 'Default "Mail From" address', 'wp_admin', 'activemember360' ),
					'desc'           => _x( 'Please ensure that your server is able to send emails using this email address, i.e. mail relay.', 'wp_admin', 'activemember360' ),
					'type'           => 'text',
					'default'        => get_bloginfo( 'admin_email' )
				],
				[
					'id'             => 'mbr_po_mail_options_notes',
					'title'          => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
					'full_width'     => TRUE,
					'type'           => 'textarea'
				]
			]
		];

		Redux::setSection( $this->opt_name, $arr_section_login_settings );
		Redux::setSection( $this->opt_name, $arr_section_special_pages );
		Redux::setSection( $this->opt_name, $arr_section_passwords );
		Redux::setSection( $this->opt_name, $arr_section_security_options);
		Redux::setSection( $this->opt_name, $arr_section_logging );
		Redux::setSection( $this->opt_name, $arr_section_comments_options );
		Redux::setSection( $this->opt_name, $arr_section_content_access_options );
		Redux::setSection( $this->opt_name, $arr_section_cache_options );
		Redux::setSection( $this->opt_name, $arr_section_excerpts );
		Redux::setSection( $this->opt_name, $arr_section_amazon_s3_expiring_link_options );
		Redux::setSection( $this->opt_name, $arr_section_misc_options );
		Redux::setSection( $this->opt_name, $arr_section_mail_options );
	}

	public function validate_save_settings( &$arr_settings )
	{
		
		IF ( ! empty( $arr_settings['mbr_po_registration_external_url'] ) ) :
			$arr_settings['mbr_po_registration_page'] = 0;
		ELSEIF ( ! empty( $arr_settings['mbr_po_registration_page'] ) ) :
			$arr_settings['mbr_po_registration_external_url'] = '';
		ENDIF;

		
		IF ( is_null( $arr_settings['mbr_po_passwords_encryption'] ) OR ! empty ( $arr_settings['mbr_po_passwords_encryption'] ) )  :
			$arr_settings['mbr_po_passwords_encryption'] = 1;
		ELSE:
			$arr_settings['mbr_po_passwords_encryption'] = 0;
		ENDIF;

		
		IF ( empty( $arr_settings['mbr_po_prevent_concurrent_login'] ) ) :
			$arr_settings['mbr_po_email_detected_concurrent_logins'] = 0;
		ENDIF;

		IF ( empty( $arr_settings['mbr_po_limit_failed_login_attempts'] ) ) :
			$arr_settings['mbr_po_limit_failed_login_attempts_max']  = 0;
			$arr_settings['mbr_po_lock_logins_seconds']              = 0;
		ELSE:
			IF ( empty( $arr_settings['mbr_po_lock_logins_seconds'] ) OR $arr_settings['mbr_po_lock_logins_seconds'] < 1 ) :
				$arr_settings['mbr_po_lock_logins_seconds']              = 3600;
			ENDIF;
			IF ( empty( $arr_settings['mbr_po_limit_failed_login_attempts_max'] ) OR $arr_settings['mbr_po_limit_failed_login_attempts_max'] < 1 ) :
				$arr_settings['mbr_po_limit_failed_login_attempts_max']  = 5;
			ENDIF;
		ENDIF;

		IF ( empty( $arr_settings['mbr_po_auto_logout'] ) ) :
			$arr_settings['mbr_po_auto_logout_seconds']              = 0;
		ELSEIF ( empty( $arr_settings['mbr_po_auto_logout_seconds'] ) OR $arr_settings['mbr_po_auto_logout_seconds'] < 1 ) :
			$arr_settings['mbr_po_auto_logout']                      = 0;
		ENDIF;

		IF ( ! empty( $arr_settings['mbr_po_whitelist_ip_addresses']['ip_address'] ) ) :
			$rebase       = FALSE;
			FOREACH ( $arr_settings['mbr_po_whitelist_ip_addresses']['ip_address'] AS $key => $ip_address ) :
				IF ( empty( $ip_address ) ) :
					unset( $arr_settings['mbr_po_whitelist_ip_addresses']['redux_repeater_data'][ $key ] );
					unset( $arr_settings['mbr_po_whitelist_ip_addresses']['ip_address'][ $key ] );
					$rebase = TRUE;
					continue;
				ENDIF;
			ENDFOREACH;
			IF ( $rebase ) :
				$arr_settings['mbr_po_whitelist_ip_addresses']['redux_repeater_data'] = array_values( $arr_settings['mbr_po_whitelist_ip_addresses']['redux_repeater_data'] );
				$arr_settings['mbr_po_whitelist_ip_addresses']['ip_address']          = array_values( $arr_settings['mbr_po_whitelist_ip_addresses']['ip_address'] );
			ENDIF;
		ENDIF;

		IF ( ! empty( $arr_settings['mbr_po_security_codes']['security_code'] ) ) :
			$rebase     = FALSE;
			FOREACH ( $arr_settings['mbr_po_security_codes']['security_code'] AS $key => $security_code ) :
				IF ( empty( $security_code ) ) :
					unset( $arr_settings['mbr_po_security_codes']['redux_repeater_data'][ $key ] );
					unset( $arr_settings['mbr_po_security_codes']['security_code'][ $key ] );
					$rebase = TRUE;
					continue;
				ENDIF;
			ENDFOREACH;
			IF ( $rebase ) :
				$arr_settings['mbr_po_security_codes']['redux_repeater_data'] = array_values( $arr_settings['mbr_po_security_codes']['redux_repeater_data'] );
				$arr_settings['mbr_po_security_codes']['security_code']       = array_values( $arr_settings['mbr_po_security_codes']['security_code'] );
			ENDIF;
		ENDIF;

		
		IF ( empty( $arr_settings['mbr_po_webhook_log'] ) ) :
			$arr_settings['mbr_po_webhook_log_max_calls'] = 0;
		ELSEIF ( empty( $arr_settings['mbr_po_webhook_log_max_calls'] ) OR $arr_settings['mbr_po_webhook_log_max_calls'] < 0 ) :
			$arr_settings['mbr_po_webhook_log']           = 0;
		ENDIF;
		IF ( empty( $arr_settings['mbr_po_api_log'] ) ) :
			$arr_settings['mbr_po_api_log_max_calls']     = 0;
		ELSEIF ( empty( $arr_settings['mbr_po_webhook_log_max_calls'] ) OR $arr_settings['mbr_po_webhook_log_max_calls'] < 0 ) :
			$arr_settings['mbr_po_api_log']               = 0;
		ENDIF;

		
		IF ( empty( $arr_settings['mbr_po_excerpts_max_length'] ) OR $arr_settings['mbr_po_excerpts_max_length'] < 5 ) :
			$arr_settings['mbr_po_excerpts_max_length']   = 55;
		ENDIF;

		
		IF ( empty( $arr_settings['mbr_po_comments_options_enabled'] ) ) :
			$arr_settings['mbr_po_comments_display_reply_form']         = 0;
			$arr_settings['mbr_po_comments_display_in_protected_posts'] = 0;
		ENDIF;
	}

	public function save_plugin_options()
	{
		global $MBR_OPTIONS;
		IF ( empty( $MBR_OPTIONS ) ) :
			return;
		ENDIF;

		$arr_settings                                                                     = get_option( 'mbr_plugin_options', [] );

		
		$arr_settings['mbr_po_require_login']                                             = $MBR_OPTIONS['force_login'];
		$arr_settings['mbr_po_login_without_membership']                                  = $MBR_OPTIONS['allow_login_wo_ms'];
		$arr_settings['mbr_po_siteban_tag']                                               = $MBR_OPTIONS['siteban_tag'];
		$arr_settings['mbr_po_last_login_field']                                          = $MBR_OPTIONS['custfld_lastlogin'];
		$arr_settings['mbr_po_login_automation']                                          = $MBR_OPTIONS['login_automation'];
		$arr_settings['mbr_po_logout_automation']                                         = $MBR_OPTIONS['logout_automation'];
		$arr_settings['mbr_po_first_login_redirect']                                      = $MBR_OPTIONS['first_login_page'];

		
		$arr_settings['mbr_po_login_page']                                                = $MBR_OPTIONS['login_page'];
		$arr_settings['mbr_po_registration_page']                                         = $MBR_OPTIONS['registration_page'];
		$arr_settings['mbr_po_registration_external_url']                                 = $MBR_OPTIONS['registration_static'];
		$arr_settings['mbr_po_excerpt_upsell_page']                                       = $MBR_OPTIONS['upsell_page'];
		$arr_settings['mbr_po_alternate_404_page']                                        = $MBR_OPTIONS['404_page'];

		
		$arr_settings['mbr_po_passwords_encryption']                                      = $MBR_OPTIONS['mbr_encrypted_passwords'];
		$arr_settings['mbr_po_passwords_min_character_length']                            = $MBR_OPTIONS['min_passlen'];
		$arr_settings['mbr_po_passwords_max_character_length']                            = $MBR_OPTIONS['max_passlen'];
		$arr_settings['mbr_po_passwords_allowed_characters']                              = $MBR_OPTIONS['pass_charset'];

		
		$arr_settings['mbr_po_api_log_max_calls']                                         = $MBR_OPTIONS['apilog_limit'];
		$arr_settings['mbr_po_webhook_log_max_calls']                                     = $MBR_OPTIONS['webhooklog_limit'];

		
		$arr_settings['mbr_po_email_detected_concurrent_logins']                          = $MBR_OPTIONS['enable_cclogin'];
		$arr_settings['mbr_po_email_detected_concurrent_logins']                          = $MBR_OPTIONS['enable_cclogin_mail'];
		$arr_settings['mbr_po_limit_failed_login_attempts_max']                           = $MBR_OPTIONS['max_logins'];
		$arr_settings['mbr_po_lock_logins_seconds']                                       = $MBR_OPTIONS['lock_logins'];
		$arr_settings['mbr_po_auto_logout_seconds']                                       = $MBR_OPTIONS['auto_logout'];

		$arr_whitelisted_ip_addresses = unserialize( $MBR_OPTIONS['excl_ip_logins'] );
		IF ( ! empty( $arr_whitelisted_ip_addresses ) ) :
			FOREACH ( $arr_whitelisted_ip_addresses AS $key => $ip_address ) :
				$arr_settings['mbr_po_whitelist_ip_addresses']['redux_repeater_data'][ $key ] = [ 'title' => '' ];
				$arr_settings['mbr_po_whitelist_ip_addresses']['ip_address'][ $key ]          = $ip_address;
			ENDFOREACH;
		ELSE:
			$arr_settings['mbr_po_whitelist_ip_addresses']['redux_repeater_data']           = [];
			$arr_settings['mbr_po_whitelist_ip_addresses']['ip_address']                    = [];
		ENDIF;

		$arr_security_codes = unserialize( $MBR_OPTIONS['security_codes'] );
		IF ( ! empty( $arr_security_codes ) ) :
			FOREACH ( $arr_security_codes AS $key => $security_code ) :
				$arr_settings['mbr_po_security_codes']['redux_repeater_data'][ $key ]         = [ 'title' => '' ];
				$arr_settings['mbr_po_security_codes']['security_code'][ $key ]               = $security_code;
			ENDFOREACH;
		ELSE:
			$arr_settings['mbr_po_security_codes']['redux_repeater_data']                   = [];
			$arr_settings['mbr_po_security_codes']['security_code']                         = [];
		ENDIF;

		$arr_settings['mbr_po_autologin_module']                                          = $MBR_OPTIONS['autologin_module'];
		$arr_settings['mbr_po_thrivecart_security_key']                                   = $MBR_OPTIONS['thrivecart_secret'];
		$arr_settings['mbr_po_jvzoo_security_key']                                        = $MBR_OPTIONS['jvzoo_secret'];
		$arr_settings['mbr_po_clickbank_security_key']                                    = $MBR_OPTIONS['clickbank_secret'];
		$arr_settings['mbr_po_digistore24_security_key']                                  = $MBR_OPTIONS['digistore_secret'];

		
		$arr_settings['mbr_po_comments_options_enabled']                                  = $MBR_OPTIONS['comments_options_activate'];
		$arr_settings['mbr_po_comments_display_reply_form']                               = $MBR_OPTIONS['comments_options_reply'];
		$arr_settings['mbr_po_comments_display_in_protected_posts']                       = $MBR_OPTIONS['comments_options_display'];

		
		$arr_settings['mbr_po_wp_admin_cap']                                              = $MBR_OPTIONS['enable_wpadmin_cap'];
		$arr_settings['mbr_po_suppress_wp_tool_bar']                                      = $MBR_OPTIONS['admin_bar'];

		
		$arr_settings['mbr_po_cache_refresh_frequency']                                   = $MBR_OPTIONS['cache_refresh_frequency'];

		
		$arr_settings['mbr_po_excerpts_force']                                            = $MBR_OPTIONS['enable_excerpt'];
		$arr_settings['mbr_po_excerpts_blur_effect']                                      = $MBR_OPTIONS['enable_excerpts_blur_effect'];
		$arr_settings['mbr_po_excerpts_gradient_start_color']                             = $MBR_OPTIONS['enable_excerpts_blur_gradient_start_color'];
		$arr_settings['mbr_po_excerpts_gradient_end_color']                               = $MBR_OPTIONS['enable_excerpts_blur_gradient_end_color'];
		$arr_settings['mbr_po_excerpts_disable_links']                                    = $MBR_OPTIONS['disable_excerpt_links'];
		$arr_settings['mbr_po_excerpts_max_length']                                       = $MBR_OPTIONS['excerpt_length'];
		$arr_settings['mbr_po_excerpts_text']                                             = $MBR_OPTIONS['site_excerpt'];

		
		$arr_settings['mbr_po_amazon_s3_access_key']                                      = $MBR_OPTIONS['mbr_s3_key'];
		$arr_settings['mbr_po_amazon_s3_secret_key']                                      = $MBR_OPTIONS['mbr_s3_secret'];
		$arr_settings['mbr_po_amazon_s3_default_expiration']                              = $MBR_OPTIONS['mbr_s3_expire'];
		$arr_settings['mbr_po_amazon_s3_default_host']                                    = $MBR_OPTIONS['mbr_s3_host'];
		$arr_settings['mbr_po_amazon_s3_default_bucket']                                  = $MBR_OPTIONS['mbr_s3_bucket'];

		
		$arr_settings['mbr_po_default_avatar_size']                                       = $MBR_OPTIONS['avatar_size'];
		$arr_settings['mbr_po_suppress_empty_custom_menu_items']                          = $MBR_OPTIONS['suppress_empty_menu'];
		$arr_settings['mbr_po_hide_menu_to_admins']                                       = $MBR_OPTIONS['admin_hide_menu'];
		$arr_settings['mbr_po_conditional_widgets']                                       = $MBR_OPTIONS['cond_widget_control_enabled'];
		$arr_settings['mbr_po_failed_payment_text']                                       = $MBR_OPTIONS['site_payf_text'];

		
		$arr_settings['mbr_po_mail_from_name']                                            = trim( $MBR_OPTIONS['mail_fromname'] );
		$arr_settings['mbr_po_mail_from_email']                                           = trim( $MBR_OPTIONS['mail_fromaddr'] );

		update_option( 'mbr_plugin_options', $arr_settings );
	}

	public function backup_settings( $arr_settings = [] )
	{
		global $wpdb;
		set_transient($this->opt_name . '_backup_' . time(), $arr_settings, 45 * 86400 );
		
		$obj_results = $wpdb->get_results( "SELECT * FROM " . $wpdb->prefix . "options WHERE option_name LIKE '_transient_" . $this->opt_name . "_backup_%'", OBJECT );
		FOREACH ($obj_results AS $key => $obj) :
			get_transient( substr( $obj->option_name, 11 ) );
		ENDFOREACH;
	}

}
$GLOBALS['mbr_inst_plugin_options'] = mbr_plugin_options::getInstance();