<?php









final class mbr_role_tags_mapping EXTENDS mbr_redux_base_class
{
	use TRAIT__getInstance;

	public $opt_name                 = 'mbr_role_tags_mapping';  
	public $section_name             = 'mbr_role_tags_map';
	public $page_slug                = 'mbr360-role-tags-mapping';

	public $enable_role_mapping_key  = 'enable_role_tags_mapping';

	public $role_mapping_key         = 'mbr_role_tag_map';
	public $site_historical_key      = 'mbr_role_tag_map_site_history';
	public $user_historical_key      = 'mbr_role_tag_map_user_history';

	function _load()
	{
		$this->protected_roles         = apply_filters( 'mbr/role_tags_mapping/protected_roles', [ 'administrator', 'editor', 'author', 'contributor', 'subscriber' ] );

		IF ( is_admin() ) :
			add_action( 'plugins_loaded',                                        [ $this, 'init_admin_panel' ],           99999 );
			add_action( 'plugins_loaded',                                        [ $this, 'init_sections' ],              99999 );
			add_filter( "redux/{$this->opt_name}/panel/template/header.tpl.php", [ $this, 'alternate_header' ] );
			add_action( "redux/options/{$this->opt_name}/validate",              [ $this, 'validate_save_settings' ] );
			add_filter( "redux/options/{$this->opt_name}/saved",                 [ $this, 'role_tags_mapping_saved' ],    9999, 2 );
		ELSE :
			add_action( 'mbr/role_tags_mapping/map_roles',                       [ $this, 'map_roles_to_tags' ],             0, 1 );
		ENDIF;
	}

	function init_admin_panel()
	{
		global $MBR_REDUX_HEADER_MODULE_NAMES;

		$MBR_REDUX_HEADER_MODULE_NAMES[ $this->opt_name ] = $admin_title = _x( 'Role to Tags Mapping', 'wp_admin', 'activemember360' );

		Redux::setExtensions( $this->opt_name, MBR_REDUX_EXTENSIONS_DIR );

		$page_title         = apply_filters( 'mbr/plugin_name', '' ) . ' ' . $admin_title;

		$arr_config         =  [
			'opt_name'        => $this->opt_name,
			'global_variable' => $this->opt_name,
			'page_slug'       => $this->page_slug,
			'page_parent'     => MBR_MENU_PARENT_PAGE,
			'page_title'      => $page_title,
			'display_name'    => $page_title,
			'admin_title'     => $admin_title,
			'menu_title'      => $admin_title,
		];

		$arr_config         = array_merge( $this->arr_default_sub_admin_panel_settings, $arr_config );

		Redux::setArgs( $this->opt_name, $arr_config );
	}

	public function init_sections()
	{
		global $pagenow;

		IF ( $pagenow <> 'admin-ajax.php' AND $pagenow <> 'admin.php' OR ( $pagenow == 'admin.php' AND ( empty( $_GET['page'] ) OR $_GET['page'] <> $this->page_slug ) ) ) :
			return;
		ENDIF;

		Redux::setSection(
			$this->opt_name,
			[
				'id'     => 'section_role_tags_mappings',
				'title'  => _x( 'Role Mappings', 'wp_admin', 'activemember360' ),
				'desc'   => _x( 'This extension allows mapping of WordPress user roles to ActiveCampaign tags.', 'wp_admin', 'activemember360' ),
				'icon'   => 'el el-cog',
				'fields' => [
					[
						'id'       => $this->section_name,
						'title'    => _x( 'Individual Role Mappings', 'wp_admin', 'activemember360' ),
						'subtitle' => _x( '<p>You can add as many role-tag mappings as you wish.</p><p>Click "Add" to create an additional mapping.</p><p>Each mapping must have at least one role and one tag selected.</p><p>However, multiple tags can be mapped to any given role.</p><p>To delete an individual mapping, click on "Delete" and that mapping will be deleted when the screen is saved.</p><p>When done with your modifications, click on "Save Changes" to save your work.</p>', 'wp_admin', 'activemember360' ),
						'type'     => 'repeater',
						'desc'     => '',
						'limit'    => 9999,
						'fields'   => [
							[
								'id'          => 'mbr_role',
								'title'       => _x( 'Select a role and one or more tags tag to create a mapping', 'wp_admin', 'activemember360' ),
								'type'        => 'select',
								'multi'       => FALSE,
								'options'     => $this->get_valid_roles(),
								'placeholder' => _x( 'Select a WordPress user role', 'wp_admin', 'activemember360' ),
							],
							[
								'id'          => 'mbr_tags',
								'type'        => 'select',
								'multi'       => TRUE,
								'options'     => apply_filters('mbr/site_tags/get', []),
								'placeholder' => _x( 'Select one or more tags', 'wp_admin', 'activemember360' ),
							]
						]
					],
					[
						'id'     => 'mbr_role_tags_map_notes',
						'title'  => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
						'type'   => 'textarea'
					]
				]
			]
		);
	}

	function map_roles_to_tags( $contact_tags = null )
	{
		global $current_user;

		


		SWITCH (TRUE) :
			CASE ( defined( 'DOING_AJAX' ) AND DOING_AJAX ) :
				return;
			CASE ( defined( 'DOING_CRON' ) AND DOING_CRON) :
				return;
			CASE ( isset( $_GET['doing_wp_cron'] ) ) :
				return;
			CASE ( !is_user_logged_in() ) :
				return;
			CASE ( !mbr_is_contact() ) :
				return;
			CASE ( empty( $contact_tags ) AND !$contact_tags = mbr_get_contact_tags() ) :
				return;
			CASE (!$role_mappings = get_option( $this->role_mapping_key, [] ) ) :
				return;
		ENDSWITCH;

		






		IF ( !$extension_roles = get_user_meta( $current_user->ID, $this->user_historical_key, TRUE ) ) :
			$extension_roles = [];
		ENDIF;

		


		FOREACH ($role_mappings AS $role => $tags) :

			


			IF ( !in_array( $role, $this->protected_roles ) ) :

				


				$role_found = in_array( $role, (array)$current_user->roles );

				



				IF ( !empty( $tags ) AND array_intersect_key( $contact_tags, $tags ) ) :
					



					IF ( !$role_found ) :
						$current_user->add_role( $role );
						


						$extension_roles[$role] = 1;
					ENDIF;
					continue;
				ENDIF;

				







				IF ( $role_found AND isset( $extension_roles[$role] ) ) :
					$current_user->remove_role( $role );
					


					unset( $extension_roles[$role] );
				ENDIF;

			ENDIF;

		ENDFOREACH;

		


		FOREACH (get_option( $this->site_historical_key, [] ) AS $role => $tags) :
			


			IF ( in_array( $role, (array)$current_user->roles ) ) :
				


				IF ( !in_array( $role, $this->protected_roles ) ) :
					


					IF ( isset( $extension_roles[$role] ) ) :
						$current_user->remove_role( $role );
						


						unset( $extension_roles[$role] );
					ENDIF;
				ENDIF;
			ENDIF;
		ENDFOREACH;

		


		update_user_meta( $current_user->ID, $this->user_historical_key, $extension_roles );
	}

	public function validate_save_settings( &$arr_settings )
	{
		IF ( ! empty( $arr_settings['mbr_role'] ) ) :
			FOREACH ( $arr_settings['mbr_role'] AS $key => $role ) :
				IF ( empty( $role ) OR empty( $arr_settings['mbr_tags'][ $key ] ) )  :
					unset( $arr_settings[ $this->section_name ]['redux_repeater_data'][ $key ] );
					unset( $arr_settings['mbr_role'][ $key ] );
					unset( $arr_settings['mbr_tags'][ $key ] );
					$rebase = TRUE;
					continue;
				ENDIF;
			ENDFOREACH;
		ENDIF;

		IF ( $rebase ) :
			$arr_settings[ $this->section_name ]['redux_repeater_data'] = array_values( $arr_settings[ $this->section_name ]['redux_repeater_data'] );
			$arr_settings['mbr_role']                                   = array_values( $arr_settings['mbr_role'] );
			$arr_settings['mbr_tags']                                   = array_values( $arr_settings['mbr_tags'] );
		ENDIF;
	}

	















	function role_tags_mapping_saved( $mappings = [], $prev_mappings = [] )
	{
		






		$mappings = $this->role_tags_mapping_normalize( $mappings );

		$role_mappings = [];

		IF ( !empty( $mappings['mbr_role'] ) ) :
			FOREACH ( $mappings['mbr_role'] AS $key => $role ) :
				IF ( !empty( $mappings['mbr_tags'][$key] ) ) :
					FOREACH ( $mappings['mbr_tags'][$key] AS $k => $tag ) :
						$role_mappings[$role][$tag] = $tag;
					ENDFOREACH;
				ENDIF;
			ENDFOREACH;
		ENDIF;

		


		update_option( $this->role_mapping_key, $role_mappings, 'yes' );

		




		$hist_mappings = get_option( $this->site_historical_key, []);

		FOREACH ( $hist_mappings AS $role => $tags ) :
			



			IF ( array_key_exists( $role, $role_mappings ) ) :
				unset($hist_mappings[$role]);
			ENDIF;
		ENDFOREACH;

		



		FOREACH ( $prev_mappings['mbr_role'] AS $key => $role ) :

			IF ( array_key_exists( $role, $role_mappings ) ) :
				



				IF ( isset( $hist_mappings[$role]) ) :
					unset( $hist_mappings[$role]);
				ENDIF;

			ELSE :
				




				FOREACH ( $prev_mappings['mbr_tags'][$key] AS $k => $tag ) :
					$hist_mappings[$role][$tag] = $tag;
				ENDFOREACH;

			ENDIF;
		ENDFOREACH;

		


		update_option( $this->site_historical_key, $hist_mappings, 'yes' );
	}

	function role_tags_mapping_normalize($mappings=[])
	{
		IF (empty($mappings)) :
			return;
		ENDIF;

		



		$tmp = [];
		FOREACH ($mappings['mbr_role'] AS $key => $role) :
			FOREACH ($mappings['mbr_tags'][$key] AS $key2 => $tag) :
				$tmp[$role][] = $tag;
			ENDFOREACH;
		ENDFOREACH;

		



		$mbr_data = $mbr_role = $mbr_tags = [];

		FOREACH ($tmp AS $role => $tags) :
			$mbr_data[] = ['title' => ''];
			$mbr_role[] = $role;
			$mbr_tags[] = $tags;
		ENDFOREACH;

		$mappings[$this->section_name]['redux_repeater_data'] = $mbr_data;
		$mappings['mbr_role']                                 = $mbr_role;
		$mappings['mbr_tags']                                 = $mbr_tags;

		


		update_option($this->opt_name, $mappings);

		


		return $mappings;
	}

	









	function get_valid_roles()
	{
		return array_filter( wp_roles()->get_names(), [$this, 'filter_valid_roles'], ARRAY_FILTER_USE_BOTH );
	}

	












	function filter_valid_roles( $name, $role )
	{
		return !in_array( $role, $this->protected_roles );
	}

}
$GLOBALS['mbr_inst_role_tags_mapping'] = mbr_role_tags_mapping::getInstance();
