<?php
final class mbr_templates EXTENDS mbr_redux_base_class
{
	use TRAIT__getInstance;

	public $opt_name  = 'mbr_templates';
	public $page_slug = 'mbr360-templates';

	public function _load()
	{
		IF ( is_admin() ) :
			add_action( 'plugins_loaded',                                        [ $this, 'init_admin_panel' ], 99999 );
			add_action( 'plugins_loaded',                                        [ $this, 'init_sections' ],    99999 );
			add_filter( "redux/{$this->opt_name}/panel/template/header.tpl.php", [ $this, 'alternate_header' ] );
			add_action( "redux/options/{$this->opt_name}/saved",                 [ $this, 'backup_settings' ],  9999 );
		ENDIF;
	}

	public function init_admin_panel()
	{
		global $MBR_REDUX_HEADER_MODULE_NAMES;

		$MBR_REDUX_HEADER_MODULE_NAMES[ $this->opt_name ] = $admin_title = _x( 'Templates', 'wp_admin', 'activemember360' );

		$page_title         = apply_filters( 'mbr/plugin_name', '' ) . ' ' . $admin_title;

		$arr_config         =  [
			'opt_name'        => $this->opt_name,
			'global_variable' => $this->opt_name,
			'page_slug'       => $this->page_slug,
			'page_parent'     => MBR_MENU_PARENT_PAGE,
			'page_title'      => $page_title,
			'display_name'    => $page_title,
			'admin_title'     => $admin_title,
			'menu_title'      => $admin_title,
		];

		$arr_config         = array_merge( $this->arr_default_sub_admin_panel_settings, $arr_config );

		Redux::setArgs( $this->opt_name, $arr_config );
	}

	public function init_sections()
	{
		global $pagenow;

		IF ( $pagenow <> 'options.php' AND $pagenow <> 'admin.php' OR ( $pagenow == 'admin.php' AND ( empty( $_GET['page'] ) OR $_GET['page'] <> $this->page_slug ) ) ) :
			return;
		ENDIF;

		$arr_default_template_new_user = apply_filters( 'mbr/templates/email/new_user_reg/default', NULL );

		$arr_section_new_user_registration_email =
		[
			'id'                      => 'section_template_new_user_registration_email',
			'title'                   => _x( 'New User Email', 'wp_admin', 'activemember360' ),
			'heading'                 => _x( 'New User Registration Email', 'wp_admin', 'activemember360' ),
			'icon'                    => 'el el-user',
			'fields'                  => [
				[
					'id'                  => 'mbr_tpl_new_user_registration_from_name',
					'title'               => _x( 'From Name', 'wp_admin', 'activemember360' ),
					'type'                => 'text',
					'validate'            => 'not_empty',
					'default'             => $arr_default_template_new_user['mbr_tpl_new_user_registration_from_name']
				],
				[
					'id'                  => 'mbr_tpl_new_user_registration_from_email',
					'title'               => _x( 'From Mail', 'wp_admin', 'activemember360' ),
					'type'                => 'text',
					'validate'            => 'not_empty',
					'default'             => $arr_default_template_new_user['mbr_tpl_new_user_registration_from_email']
				],
				[
					'id'                  => 'mbr_tpl_new_user_registration_subject',
					'title'               => _x( 'Subject', 'wp_admin', 'activemember360' ),
					'type'                => 'text',
					'validate'            => 'not_empty',
					'default'             => $arr_default_template_new_user['mbr_tpl_new_user_registration_subject']
				],
				[
					'id'                  => 'mbr_tpl_new_user_registration_message',
					'title'               => _x( 'Message', 'wp_admin', 'activemember360' ),
					'desc'                => sprintf( _x( 'You can use the following placeholder in your email template:%s', 'wp_admin', 'activemember360' ), '<br /><strong>%login_url%</strong><br /><strong>%user_login%</strong><br /><strong>%user_password%</strong>' ),
					'type'                => 'editor',
					'args'                => [ 'media_buttons' => FALSE ],
					'validate'            => 'not_empty',
					'default'             => $arr_default_template_new_user['mbr_tpl_new_user_registration_message']
				],
				[
					'id'                  => 'mbr_tpl_new_user_registration_notes',
					'title'               => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
					'type'                => 'textarea'
				]
			]
		];

		$arr_default_template_password_reminder = apply_filters( 'mbr/templates/email/password_reminder/default', NULL );

		$arr_section_password_reminder_email =
		[
			'id'                      => 'section_template_password_reminder_email',
			'title'                   => _x( 'Password Reminder Email', 'wp_admin', 'activemember360' ),
			'icon'                    => 'el el-user',
			'fields'                  => [
				[
					'id'                  => 'mbr_tpl_password_reminder_from_name',
					'title'               => _x( 'From Name', 'wp_admin', 'activemember360' ),
					'type'                => 'text',
					'validate'            => 'not_empty',
					'default'             => $arr_default_template_password_reminder['mbr_tpl_password_reminder_from_name']
				],
				[
					'id'                  => 'mbr_tpl_password_reminder_from_email',
					'title'               => _x( 'From Mail', 'wp_admin', 'activemember360' ),
					'type'                => 'text',
					'validate'            => 'not_empty',
					'default'             => $arr_default_template_password_reminder['mbr_tpl_password_reminder_from_email']
				],
				[
					'id'                  => 'mbr_tpl_password_reminder_subject',
					'title'               => _x( 'Subject', 'wp_admin', 'activemember360' ),
					'type'                => 'text',
					'validate'            => 'not_empty',
					'default'             => $arr_default_template_password_reminder['mbr_tpl_password_reminder_subject']
				],
				[
					'id'                  => 'mbr_tpl_password_reminder_message',
					'title'               => _x( 'Message', 'wp_admin', 'activemember360' ),
					'desc'                => sprintf( _x( 'You can use the following placeholder in your email template:%s', 'wp_admin', 'activemember360' ), '<br /><strong>%login_url%</strong><br /><strong>%user_login%</strong><br /><strong>%user_password%</strong>' ),
					'type'                => 'editor',
					'args'                => [ 'media_buttons' => FALSE ],
					'validate'            => 'not_empty',
					'default'             => $arr_default_template_password_reminder['mbr_tpl_password_reminder_message']
				],
				[
					'id'                  => 'mbr_tpl_password_reminder_notes',
					'title'               => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
					'type'                => 'textarea'
				]
			]
		];

		$arr_default_template_login_credentials = apply_filters( 'mbr/templates/email/login_credentials/default', NULL );

		$arr_section_login_credentials_email =
			[
				'id'                      => 'section_template_login_credentials_email',
				'title'                   => _x( 'Login Credentials Email', 'wp_admin', 'activemember360' ),
				'icon'                    => 'el el-user',
				'fields'                  => [
					[
						'id'                  => 'mbr_tpl_login_credentials_from_name',
						'title'               => _x( 'From Name', 'wp_admin', 'activemember360' ),
						'type'                => 'text',
						'validate'            => 'not_empty',
						'default'             => $arr_default_template_login_credentials['mbr_tpl_login_credentials_from_name']
					],
					[
						'id'                  => 'mbr_tpl_login_credentials_from_email',
						'title'               => _x( 'From Mail', 'wp_admin', 'activemember360' ),
						'type'                => 'text',
						'validate'            => 'not_empty',
						'default'             => $arr_default_template_login_credentials['mbr_tpl_login_credentials_from_email']
					],
					[
						'id'                  => 'mbr_tpl_login_credentials_subject',
						'title'               => _x( 'Subject', 'wp_admin', 'activemember360' ),
						'type'                => 'text',
						'validate'            => 'not_empty',
						'default'             => $arr_default_template_login_credentials['mbr_tpl_login_credentials_subject']
					],
					[
						'id'                  => 'mbr_tpl_login_credentials_message',
						'title'               => _x( 'Message', 'wp_admin', 'activemember360' ),
						'desc'                => sprintf( _x( 'You can use the following placeholder in your email template:%s', 'wp_admin', 'activemember360' ), '<br /><strong>%login_url%</strong><br /><strong>%user_login%</strong><br /><strong>%user_password%</strong>' ),
						'type'                => 'editor',
						'args'                => [ 'media_buttons' => FALSE ],
						'validate'            => 'not_empty',
						'default'             => $arr_default_template_login_credentials['mbr_tpl_login_credentials_message']
					],
					[
						'id'                  => 'mbr_tpl_login_credentials_notes',
						'title'               => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
						'type'                => 'textarea'
					]
				]
			];


		
		Redux::setSection( $this->opt_name, $arr_section_new_user_registration_email );
		Redux::setSection( $this->opt_name, $arr_section_password_reminder_email );
		Redux::setSection( $this->opt_name, $arr_section_login_credentials_email );
	}

	public function backup_settings( $arr_settings = [] )
	{
		global $wpdb;
		set_transient($this->opt_name . '_backup_' . time(), $arr_settings, 45 * 86400 );
		
		$obj_results = $wpdb->get_results( "SELECT * FROM " . $wpdb->prefix . "options WHERE option_name LIKE '_transient_" . $this->opt_name . "_backup_%'", OBJECT );
		FOREACH ($obj_results AS $key => $obj) :
			get_transient( substr( $obj->option_name, 11 ) );
		ENDFOREACH;
	}

}

$GLOBALS['mbr_inst_templates'] = mbr_templates::getInstance();
