<?php
class mbr_tracking {

  use TRAIT__getInstance;

  function _load()
  {
  	global $MBR_OPTIONS;
		$this->arr_extension_settings  = $MBR_OPTIONS['extensions'];

		$this->site_tracking_enabled   = ! empty( $MBR_OPTIONS['extensions']['mbr_ext_ac_site_tracking_enabled']  ) ? TRUE : FALSE;
		$this->event_tracking_enabled  = ! empty( $MBR_OPTIONS['extensions']['mbr_ext_ac_event_tracking_enabled'] ) ? TRUE : FALSE;
		$this->ext_woocommerce_enabled = ! empty( $MBR_OPTIONS['extensions']['mbr_ext_woocommerce_enabled']       ) ? TRUE : FALSE;

		IF ( $this->event_tracking_enabled ) :
			add_action( 'mbr_authenticated_remote_login', [ $this, 'login_event'], 10, 2);
			add_action( 'wp_logout',                      [ $this, 'logout_event'], 0); 

			
			add_action( 'register_new_user',              [ $this, 'register_new_user_event'], 12, 1);

			
			add_action( 'user_register_login',            [ $this, 'user_register_login_event'], 10, 1);
			add_action( 'woocommerce_payment_complete',   [ $this, 'payment_completed_event'], 10, 1);
			add_action( 'order_status_event',             [ $this, 'order_status_event'], 10, 3);

			add_action( 'wp_footer',                      [ $this, 'onclick_event_listener']);
			add_action( 'wp_ajax_mbr_ajax_eventlog',      [ $this, 'ajax_event_log']);
		ENDIF;

		add_action( 'wp_enqueue_scripts',               [ $this, 'add_ac_tracking_code']);

		add_filter( 'mbr/shortcodes/extend',            [ $this, 'extend_tracking_shortcodes'], 0, 1);
  }

  public function extend_tracking_shortcodes( $SHORT = [] )
  {
		$SHORT['log_event'] = [ [ $this, 'sc_log_event' ], 0 ];

		return $SHORT;
  }

  public function sc_log_event( $parm = '', $content = NULL, $scname = '' )
  {
		$contact_id = apply_filters( 'mbr/contact_id', NULL );
		IF ( empty( $parm ) OR empty( $contact_id ) OR empty( $this->event_tracking_enabled ) ) :
			return NULL;
		ENDIF;

		$parm = apply_filters( 'mbr/shortcodes/atts',  [ 'event' => '', 'value' => get_permalink() ], $parm, $scname );
		IF ( ! $parm['event'] ) :
			return NULL;
		ENDIF;
		$contact_session = apply_filters( 'mbr/session/get', NULL );
		IF ( ! empty( $contact_session['email'] ) ) :
			$this->log_event( trim ( preg_replace('/[^0-9a-zA-Z ]/', '', $parm['event'] ) ), $parm['value'], $contact_session['email'] );
		ENDIF;
  }

  public function onclick_event_listener()
  {
		IF ( ! is_user_logged_in() ) :
			return;
		ENDIF;

		?>
		<script>
			jQuery( document ).ready( function(){
				jQuery( '*[data-mbr-eventname]' ).click( function(){
					if ( jQuery( this ).data( 'mbr-eventname' ) && jQuery( this ).data( 'mbr-eventvalue' ) ) {
						jQuery.post( ajaxurl, {
							'action'     : 'mbr_ajax_eventlog',
							'eventname'  : jQuery( this ).data( 'mbr-eventname' ),
							'eventvalue' : jQuery( this ).data( 'mbr-eventvalue' )
						});
					}
				});

				jQuery( '*[data-mbr-form-eventname]' ).submit( function(){
					if ( jQuery( this ).data( 'mbr-form-eventname' ) && jQuery( this ).data( 'mbr-form-eventvalue' ) ) {
						jQuery.post( ajaxurl, {
							'action'     : 'mbr_ajax_eventlog',
							'eventname'  : jQuery( this ).data( 'mbr-form-eventname' ),
							'eventvalue' : jQuery( this ).data( 'mbr-form-eventvalue' )
						});
					}
				});
			});
		</script>
		<?php
  }

  public function ajax_event_log()
  {
		$contact_session = apply_filters( 'mbr/session/get', NULL );
		IF ( ! empty( $contact_session['email'] ) AND ! empty( $_POST['eventname'] ) AND ! empty( $_POST['eventvalue'] ) ) :
			$this->log_event( trim( preg_replace('/[^0-9a-zA-Z ]/', '', $_POST['eventname'] ) ), $_POST['eventvalue'], $contact_session['email'] );
		ENDIF;
		exit;
  }

  public function login_event( $user, $contact )
  {
  	global $MBR_OPTIONS;

		$login_event = $MBR_OPTIONS['extensions']['mbr_ext_ac_event_login'];
		IF ( ! empty( $login_event ) ) :
			$this->log_event( $login_event, MBRWPURL, $contact['email'] );
		ENDIF;
  }

  public function logout_event()
  {
  	global $MBR_OPTIONS;
		$logout_event = $MBR_OPTIONS['extensions']['mbr_ext_ac_event_logout'];
		IF ( ! empty( $logout_event ) ) :
			$contact_session = apply_filters( 'mbr/session/get', NULL );
			IF ( ! empty( $contact_session['email'] ) ) :
			$this->log_event( $logout_event, MBRWPURL, $contact_session['email'] );
			ENDIF;
		ENDIF;
  }

  public function register_new_user_event( $user_id )
  {
  	global $MBR_OPTIONS;
		$user_registered_event = $MBR_OPTIONS['extensions']['mbr_ext_ac_event_user_registration'];
		IF ( ! empty( $user_registered_event ) ) :
			IF ( $user = get_userdata( $user_id ) AND ! empty( $user->user_email ) ) :
			$this->log_event( $user_registered_event, MBRWPURL, $user->user_email );
			ENDIF;
		ENDIF;
  }

  public function user_register_login_event( $user_email )
  {
  	global $MBR_OPTIONS;
		$user_registered_event = $MBR_OPTIONS['extensions']['mbr_ext_ac_event_user_registration'];
		IF ( ! empty( $user_registered_event ) ) :
			$this->log_event( $user_registered_event, MBRWPURL, $user_email );
		ENDIF;
  }

  public function payment_completed_event( $order_id )
  {
  	global $MBR_OPTIONS;

		$payment_completed_event = $MBR_OPTIONS['extensions']['mbr_ext_ac_event_woocommerce_payment_completed'];
		$purchased_product_event = $MBR_OPTIONS['extensions']['mbr_ext_ac_event_woocommerce_purchased_product'];

		IF ( ! empty( $payment_completed_event ) OR !empty( $purchased_product ) ) :
			IF ( ! $order = new WC_Order( $order_id ) ) :
				return;
			ENDIF;

			IF ( ! empty( $payment_completed_event ) ) :
				$this->log_event( $payment_completed_event , sprintf( __( 'Order Id %s', 'activemember360' ), $order_id ), $order->get_billing_email() );
			ENDIF;

			IF ( ! empty( $purchased_product_event ) ) :
				$order_items  = $order->get_items( [ 'line_item' ] );
				FOREACH ( $order_items AS $k => $order_item ) :
					$product_id = empty( $order_item['variation_id'] ) ? $order_item['product_id'] : $order_item['variation_id'];
					IF ( ! empty( $product_id ) ) :
						$this->log_event( $purchased_product_event, $product_id,         $order->get_billing_email() );
						$this->log_event( $purchased_product_event, $order_item['name'], $order->get_billing_email() );
					ENDIF;
				ENDFOREACH;
			ENDIF;
		ENDIF;
  }

  public function order_status_event( $user_email, $order_status, $order_id )
  {
  	global $MBR_OPTIONS;

		$event_order_status = $MBR_OPTIONS['extensions']['mbr_ext_ac_event_woocommerce_' . str_replace( '-', '_', $order_status ) ];
		IF ( ! empty( $event_order_status ) ) :
			$this->log_event( $event_order_status, sprintf( __( 'Order Id %s', 'activemember360' ), $order_id ), $user_email );
		ENDIF;
  }

  



  public function log_event( $event = FALSE, $event_data = FALSE, $email = FALSE)
  {
		$arr_account_info = apply_filters( 'mbr/ac_account/get', NULL );

		$post_data              = [];
		$post_data['actid']     = $arr_account_info['track_id'];
		$post_data['key']       = $arr_account_info['event_key'];
		$post_data['event']     = $event;
		$post_data['eventdata'] = $event_data;
		IF ( $email ) :
			$post_data['visit']   = json_encode( [ 'email' => $email ] );
		ENDIF;
		$curl                   = curl_init();
		curl_setopt( $curl, CURLOPT_URL, 'https://trackcmp.net/event' );
		curl_setopt( $curl, CURLOPT_RETURNTRANSFER, TRUE );
		curl_setopt( $curl, CURLOPT_POST, TRUE );
		curl_setopt( $curl, CURLOPT_POSTFIELDS, $post_data );
		curl_exec( $curl );
  }

  





  public function create_event( $event = FALSE )
  {
		IF ( ! empty( $event) ) :
			$this->log_event( $event, FALSE, FALSE );
		ENDIF;
  }

  public function add_ac_tracking_code()
  {
		IF (is_admin()) :
			return;
		ENDIF;

		$arr_account_info = apply_filters( 'mbr/ac_account/get', NULL );

		IF ( ! empty( $this->site_tracking_enabled ) AND ! empty( $arr_account_info['track_id'] ) ) :
			wp_enqueue_script( 'mbr_ac_tracking_code', $arr_account_info['script_url'], [], FALSE, TRUE );
			$email   = '';
			$contact_session = apply_filters( 'mbr/session/get', NULL );
			IF ( ! empty( $contact_session['email'] ) ) :
			$email = $contact_session['email'];
			ENDIF;
			$arr_ac_tracking = [ 'actid' => $arr_account_info['track_id'], 'email' => $email ];
			wp_localize_script( 'mbr_ac_tracking_code', 'mbr_activecampaign_tracking', $arr_ac_tracking );
		ENDIF;
  }

}
$GLOBALS['mbr_tracking'] = mbr_tracking::getInstance();
