<?php
add_action( 'init', 'mbr_init_update_plugin' );

function mbr_init_update_plugin()
{
	new activemember360_plugin_updater();
}

class activemember360_plugin_updater
{
	private $update_plugin_cache_expiration                    = 7200;
	private $plugin_name                                       = 'ActiveMember360';
	private $update_path                                       = 'https://digitalfactorycloud.com/update/check_update_v3.php';
	private $arr_plugin_row_support_links                      = [ 'support'  => [ 'url' => 'https://activemember360.com/support/', 'label' => 'Support' ], 'facebook' => [ 'url' => 'https://www.facebook.com/groups/activemembership/', 'label' => 'Forum' ] ];
	private $arr_plugin_row_action_links                       = [ 'download' => [ 'url' => 'http://download.activemember360.com/updates/activemember360.zip', 'label' => 'Download' ] ];
	private $settings_page                                     = 'activemember360';

	public function __construct()
	{
		$this->plugin_slug                                       = $this->get_parent_plugin_basename();
		$this->slug_to_compare                                   = strtolower( str_ireplace( '.php', '', basename( $this->plugin_slug ) ) );
		$this->plugin_dir                                        = dirname( $this->plugin_slug );
		$this->update_plugin_cache_option_key                    = $this->plugin_dir . '_plugin_update_cache';

		add_action( 'admin_notices',                             [ $this, 'plugin_update_available' ] );
		IF ( is_multisite() ) :
			add_action( 'network_admin_notices',                   [ $this, 'plugin_update_available' ] ) ;
		ENDIF;

		$event                                                   = 'update_plugins';
		add_filter( 'site_transient_' . $event,                  [ $this, 'check_update' ],        10, 2 );
		add_filter( 'site_transient_' . $event,                  [ $this, 'force_update' ],        10, 2 );

		add_filter( 'plugins_api',                               [ $this, 'check_info' ],          10, 3 );
		add_filter( 'plugin_action_links_' . $this->plugin_slug, [ $this, 'plugin_action_links' ], 10, 4 );
		add_filter( 'plugin_row_meta',                           [ $this, 'plugin_row_meta' ],     10, 4 );

		add_action( 'upgrader_process_complete',                 [ $this, 'after_update' ],        10, 2 );
	}

	public function check_update( $transient_value, $transient_type )
	{
		IF ( empty( $transient_value->checked ) ) :
			return $transient_value;
		ENDIF;

		IF ( $transient_type <> 'update_plugins' ) :
			return $transient_value;
		ENDIF;

		IF ( FALSE == $remote_version = get_transient( $this->update_plugin_cache_option_key ) ) :
			$remote_version    = $this->get_remote( [ 'slug' => $this->slug_to_compare, 'version' => MBRVERSION ] );
			set_transient( $this->update_plugin_cache_option_key, $remote_version, $this->update_plugin_cache_expiration );
		ENDIF;

		IF ( $remote_version ) :
			IF ( version_compare( MBRVERSION, $remote_version->new_version, '<' ) ) :
				$transient_value->response[ $this->plugin_slug ] = $remote_version;
				update_option( $this->plugin_dir . '_update_notice', $remote_version->new_version );
			ENDIF;
		ENDIF;

		return $transient_value;
	}

	public function force_update( $transient_value, $transient_type )
	{
		IF ( isset( $_GET['action'], $_GET['plugin'] ) AND $transient_type == 'update_plugins' ) :
			IF ( $_REQUEST['plugin'] == $this->slug_to_compare OR $_REQUEST['plugin'] == $this->plugin_slug ) :
				IF ( 'upgrade-plugin' == $_GET['action'] OR 'install-plugin' == $_GET['action'] ) :
					IF ( 'install-plugin'  == $_GET['action'] ) :
						$_GET['action']   = $_REQUEST['action']   = 'upgrade-plugin';
						$_GET['_wpnonce'] = $_REQUEST['_wpnonce'] = wp_create_nonce( 'upgrade-plugin_' . $this->plugin_slug );
						$_GET['plugin']   = $_REQUEST['plugin']   = $this->plugin_slug;
					ENDIF;
					$response           = $this->get_remote( [ 'slug' => $this->slug_to_compare, 'version' => '0.0.0.0' ] );
					$me                 = new stdClass();
					$me->id             = 4;
					$me->slug           = $this->slug_to_compare;
					$me->plugin         = $this->plugin_slug;
					$me->new_version    = $response->new_version;
					$me->url            = $response->homepage;
					$me->package        = $response->download_link;
					$transient_value->response[ $this->plugin_slug ] = $me;
				ENDIF;
			ENDIF;
		ENDIF;

		return $transient_value;
	}

	public function check_info( $obj, $action, $arg )
	{
		IF ( ( $action == 'query_plugins' OR $action == 'plugin_information') AND isset( $arg->slug ) AND $arg->slug === $this->slug_to_compare ) :
			return $this->get_remote( [ 'slug' => $this->slug_to_compare, 'version' => MBRVERSION ] );
		ENDIF;

		return $obj;
	}

	public function plugin_row_meta( $plugin_meta, $plugin_file, $plugin_data, $status )
	{
		IF ( $plugin_file == $this->plugin_slug ) :
			FOREACH ( $plugin_meta AS $key => $value ) :
				IF ( strpos( $value, __( 'View details' ) ) !== FALSE ) :
					GOTO SKIP_INSERT_META_ROW;
				ENDIF;
			ENDFOREACH;

			array_push( $plugin_meta, sprintf( '<a href="%s" class="thickbox open-plugin-details-modal" aria-label="%s" data-title="%s">%s</a>', esc_url( network_admin_url( 'plugin-install.php?tab=plugin-information&plugin=' . $this->slug_to_compare . '&TB_iframe=true' ) ), esc_attr( sprintf( __( 'More information about %s' ), 'activemember360' ) ), esc_attr( $this->plugin_name ), __( 'View details' ) ) );

			SKIP_INSERT_META_ROW:

			FOREACH ( $this->arr_plugin_row_support_links AS $type => $arr ) :
				IF ( ! empty( $arr['url'] ) AND ! empty( $arr['label'] ) ) :
					array_push( $plugin_meta, sprintf( '<a href="%s" target="_blank">%s</a>', $arr['url'], $arr['label'] ) );
				ENDIF;
			ENDFOREACH;

		ENDIF;

		return $plugin_meta;
	}

	public function plugin_action_links( $links, $file, $plugin_data, $context )
	{
		IF ( ! empty( $this->settings_page ) ) :
			array_unshift( $links, '<a href="' . admin_url( 'admin.php?page=' . $this->settings_page ) . '">' . _x( 'Settings', 'wp_admin', 'activemember360' ) . '</a>' );
		ENDIF;

		FOREACH ( $this->arr_plugin_row_action_links AS $type => $arr ) :
			IF ( ! empty( $arr['url'] ) AND ! empty( $arr['label'] ) ) :
				array_push( $links, sprintf( '<a href="%s" target="_blank">%s</a>', $arr['url'], $arr['label'] ) );
			ENDIF;
		ENDFOREACH;

		array_push( $links, sprintf( '<a href="%1$s">' . _x( 'Update Now', 'wp_admin', 'activemember360' ) . '</a>', wp_nonce_url( self_admin_url( 'update.php?action=upgrade-plugin&plugin=' ) . $file, 'upgrade-plugin_' . $file ) ) );

		return $links;
	}

	public function get_remote( $args )
	{
		$request = wp_remote_post( $this->update_path, [ 'body' => $args ] );

		IF ( ! is_wp_error( $request ) OR wp_remote_retrieve_response_code( $request ) === 200 ) :
			return unserialize( wp_remote_retrieve_body( $request ) );
		ENDIF;

		return FALSE;
	}

	public function after_update( $upgrader_object, $options )
	{
		IF ( $options['action'] == 'update' AND $options['type'] === 'plugin' ) :
			delete_transient( $this->update_plugin_cache_option_key );
			update_option( $this->plugin_dir . '_update_notice', 0 );
		ENDIF;
	}

	public function plugin_update_available()
	{
		IF ( $new_version = get_option( $this->plugin_dir . '_update_notice', FALSE ) ) :

			IF ( version_compare( MBRVERSION, $new_version, '<' ) ) :
				echo '<div class="notice notice-warning is-dismissible"><p>' . sprintf( _x( 'Version %s for %s is available.', 'wp_admin', 'activemember360' ), $new_version, $this->plugin_name ) . ' ' . sprintf( '<a href="%1$s">' . _x( 'Update Now', 'wp_admin', 'activemember360' ) . '.</a>', wp_nonce_url( self_admin_url( 'update.php?action=upgrade-plugin&plugin=' ) . $this->plugin_slug, 'upgrade-plugin_' . $this->plugin_slug ) ) . '</p></div>';
			ELSE:
				delete_transient( $this->update_plugin_cache_option_key );
				update_option( $this->plugin_dir . '_update_notice', 0 );
			ENDIF;

		ENDIF;
	}

	public function get_parent_plugin_basename()
	{
		$path               = basename( dirname( __FILE__ ) );
		$active_plugins     = wp_get_active_and_valid_plugins();

		IF ( is_multisite() ) :
			$active_plugins_2 = wp_get_active_network_plugins();
			$active_plugins   = array_merge( $active_plugins, $active_plugins_2 );
		ENDIF;

		FOREACH ( $active_plugins AS $plugin ) :
			IF ( basename( dirname( $plugin ) ) == $path ) :
				return plugin_basename( $plugin );
			ENDIF;
		ENDFOREACH;

		return FALSE;
	}

}