<?php
final class mbr_utilities EXTENDS mbr_redux_base_class
{
	use TRAIT__getInstance;

	public $opt_name  = 'mbr_utilities';
	public $page_slug = 'mbr360-utilities';

	public function _load()
	{
		IF ( is_admin() ) :
			add_action( 'plugins_loaded',                                           [ $this, 'init_admin_panel' ], 99999 );
			add_action( 'plugins_loaded',                                           [ $this, 'init_sections' ],    99999 );
			add_filter( "redux/{$this->opt_name}/panel/template/header.tpl.php",    [ $this, 'alternate_header' ] );
			add_filter( "redux/{$this->opt_name}/panel/template/container.tpl.php", [ $this, 'alternate_container' ] );
			add_action( 'wp_ajax_mbr_utl_get_automations',                          [ $this, 'ajax_get_automations' ] );
			add_action( 'wp_ajax_mbr_utl_get_custom_fields',                        [ $this, 'ajax_get_custom_fields' ] );
			add_action( 'wp_ajax_mbr_utl_get_forms',                                [ $this, 'ajax_get_forms' ] );
			add_action( 'wp_ajax_mbr_utl_get_lists',                                [ $this, 'ajax_get_lists' ] );
			add_action( 'wp_ajax_mbr_utl_get_mail_templates',                       [ $this, 'ajax_get_mail_templates' ] );
			add_action( 'wp_ajax_mbr_utl_get_tags',                                 [ $this, 'ajax_get_tags' ] );
			add_action( 'wp_ajax_mbr_utl_get_webhooks',                             [ $this, 'ajax_get_webhooks' ] );
			
			add_action( 'wp_ajax_mbr_utl_get_api_log',                              [ $this, 'ajax_get_api_log' ] );
			
			add_action( 'wp_ajax_get_api_log',                                      [ $this, 'ajax_get_api_log_results' ] );
			add_action( 'wp_ajax_mbr_utl_get_database',                             [ $this, 'ajax_get_database' ] );
			add_action( 'wp_ajax_mbr_utl_get_environment',                          [ $this, 'ajax_get_environment' ] );
			add_action( 'wp_ajax_mbr_utl_get_php_info',                             [ $this, 'ajax_get_php_info' ] );
			add_action( 'wp_ajax_mbr_utl_get_cache_refresh',                        [ $this, 'ajax_get_cache_refresh' ] );
			add_action( 'wp_ajax_mbr_utl_refresh_local_cache',                      [ $this, 'ajax_refresh_local_cache' ] );
		ENDIF;
	}

	public function alternate_container()
	{
		return MBR_DIR . '/templates/container_utilities.tpl.php';
	}

	public function init_admin_panel()
	{
		global $MBR_REDUX_HEADER_MODULE_NAMES;

		$MBR_REDUX_HEADER_MODULE_NAMES[ $this->opt_name ] = $admin_title = _x( 'Utilities', 'wp_admin', 'activemember360' );

		$page_title         = apply_filters( 'mbr/plugin_name', '' ) . ' ' . $admin_title;

		$arr_config         =  [
			'opt_name'        => $this->opt_name,
			'global_variable' => $this->opt_name,
			'page_slug'       => $this->page_slug,
			'page_parent'     => MBR_MENU_PARENT_PAGE,
			'page_title'      => $page_title,
			'display_name'    => $page_title,
			'admin_title'     => $admin_title,
			'menu_title'      => $admin_title,
			'hide_save'       => TRUE,
			'settings_api'    => FALSE
		];

		$arr_config         = array_merge( $this->arr_default_sub_admin_panel_settings, $arr_config );

		Redux::setArgs( $this->opt_name, $arr_config );
	}

	public function init_sections()
	{
		global $pagenow;

		IF ( $pagenow <> 'options.php' AND $pagenow <> 'admin.php' OR ( $pagenow == 'admin.php' AND ( empty( $_GET['page'] ) OR $_GET['page'] <> $this->page_slug ) ) ) :
			return;
		ENDIF;

		


		$arr_section_activecampaign =
		[
			'id'                      => 'section_activecampaign',
			'title'                   => _x( 'ActiveCampaign', 'wp_admin', 'activemember360' ),
			'icon'                    => 'el el-network',
			'class'                   => 'mbr_utl_section_api_log'
		];

		


		$arr_section_api_log        =
		[
			'id'                      => 'section_api_log',
			'title'                   => _x( 'API Log', 'wp_admin', 'activemember360' ),
			'class'                   => 'mbr_utl_section_api_log',
			'subsection'              => TRUE,
			'icon'                    => 'el el-arrow-right',
			'fields'                  => [
				[
					'id'                  => 'mbr_utl_api_log',
					'full_width'          => TRUE,
					'type'                => 'raw',
					'content'             => '<div id="mbr_utl_api_log_container"></div>'
				]
			]
		];

		


		$arr_section_automations    =
		[
			'id'                      => 'section_automations',
			'title'                   => _x( 'Automations', 'wp_admin', 'activemember360' ),
			'desc'                    => _x( 'All your automations, defined in ActiveCampaign are listed on this page.', 'wp_admin', 'activemember360' ),
			'class'                   => 'mbr_utl_section_automations',
			'subsection'              => TRUE,
			'icon'                    => 'el el-refresh',
			'fields'                  => [
				[
					'id'                  => 'mbr_utl_automations',
					'full_width'          => TRUE,
					'type'                => 'raw',
					'content'             => '<div id="mbr_utl_automations_container"></div>'
				]
			]
		];

		


		$arr_section_custom_fields  =
		[
			'id'                      => 'section_custom_fields',
			'title'                   => _x( 'Custom Fields', 'wp_admin', 'activemember360' ),
			'desc'                    => _x( 'All your custom fields, defined in ActiveCampaign are listed on this page.', 'wp_admin', 'activemember360' ),
			'class'                   => 'mbr_utl_section_custom_fields',
			'subsection'              => TRUE,
			'icon'                    => 'el el-th-list',
			'fields'                  => [
				[
					'id'                  => 'mbr_utl_custom_fields',
					'full_width'          => TRUE,
					'type'                => 'raw',
					'content'             => '<div id="mbr_utl_custom_fields_container"></div>'
				]
			]
		];

		


		$arr_section_forms          =
		[
			'id'                      => 'section_forms',
			'title'                   => _x( 'Forms', 'wp_admin', 'activemember360' ),
			'desc'                    => _x( 'All your forms, defined in ActiveCampaign are listed on this page.', 'wp_admin', 'activemember360' ),
			'class'                   => 'mbr_utl_section_forms',
			'subsection'              => TRUE,
			'icon'                    => 'el el-pencil',
			'fields'                  => [
				[
					'id'                  => 'mbr_utl_forms',
					'full_width'          => TRUE,
					'type'                => 'raw',
					'content'             => '<div id="mbr_utl_forms_container"></div>'
				]
			]
		];

		


		$arr_section_lists          =
		[
			'id'                      => 'section_lists',
			'title'                   => _x( 'Lists', 'wp_admin', 'activemember360' ),
			'desc'                    => _x( 'All your lists, defined in ActiveCampaign are listed on this page.', 'wp_admin', 'activemember360' ),
			'class'                   => 'mbr_utl_section_lists',
			'subsection'              => TRUE,
			'icon'                    => 'el el-envelope',
			'fields'                  => [
				[
					'id'                  => 'mbr_utl_lists',
					'full_width'          => TRUE,
					'type'                => 'raw',
					'content'             => '<div id="mbr_utl_lists_container"></div>'
				]
			]
		];

		


		$arr_section_templates      =
		[
			'id'                      => 'section_mail_templates',
			'title'                   => _x( 'Mail Templates', 'wp_admin', 'activemember360' ),
			'desc'                    => _x( 'All your templates, defined in ActiveCampaign are listed on this page.', 'wp_admin', 'activemember360' ),
			'class'                   => 'mbr_utl_section_mail_templates',
			'subsection'              => TRUE,
			'icon'                    => 'el el-envelope',
			'fields'                  => [
				[
					'id'                  => 'mbr_utl_templates',
					'full_width'          => TRUE,
					'type'                => 'raw',
					'content'             => '<div id="mbr_utl_mail_templates_container"></div>'
				]
			]
		];

		


		$arr_section_tags           =
		[
			'id'                      => 'section_tags',
			'title'                   => _x( 'Tags', 'wp_admin', 'activemember360' ),
			'desc'                    => _x( 'All your tags, defined in ActiveCampaign are listed on this page as well as your tags used for membership levels.', 'wp_admin', 'activemember360' ),
			'class'                   => 'mbr_utl_section_tags',
			'subsection'              => TRUE,
			'icon'                    => 'el el-tags',
			'fields'                  => [
				[
					'id'                  => 'mbr_utl_tags',
					'full_width'          => TRUE,
					'type'                => 'raw',
					'content'             => '<div id="mbr_utl_tags_container"></div>'
				]
			]
		];

		


		$arr_section_webhooks       =
		[
			'id'                      => 'section_webhooks',
			'title'                   => _x( 'Webhooks', 'wp_admin', 'activemember360' ),
			'desc'                    => _x( 'All your webhooks, defined in ActiveCampaign are listed on this page.', 'wp_admin', 'activemember360' ),
			'class'                   => 'mbr_utl_section_webhooks',
			'subsection'              => TRUE,
			'icon'                    => 'el el-arrow-right',
			'fields'                  => [
				[
					'id'                  => 'mbr_utl_webhooks',
					'full_width'          => TRUE,
					'type'                => 'raw',
					'content'             => '<div id="mbr_utl_webhooks_container"></div>'
				]
			]
		];

		


		$arr_section_server_environment =
		[
			'id'                     => 'section_server_environment',
			'title'                  => _x( 'Server Environment', 'wp_admin', 'activemember360' ),
			'icon'                   => 'el el-network',
			'class'                  => 'mbr_utl_section_environment'
		];

		


		$show_environment          = FALSE;
		IF ( ! defined( 'MBR_SHOW_ENVIRONMENT' ) OR MBR_SHOW_ENVIRONMENT ) :
			$show_environment        = TRUE;
			$arr_section_environment =
			[
				'id'                   => 'section_environment',
				'title'                => _x( 'Environment', 'wp_admin', 'activemember360' ),
				'class'                => 'mbr_utl_section_environment',
				'subsection'           => TRUE,
				'icon'                 => 'el el-globe',
				'fields'               => [
					[
						'id'               => 'mbr_utl_environment',
						'full_width'       => TRUE,
						'type'             => 'raw',
						'content'          => '<div id="mbr_utl_environment_container"></div>'
					]
				]
			];
		ENDIF;

		


		$show_database             = FALSE;
		IF ( ! defined( 'MBR_SHOW_DATABASE' ) OR MBR_SHOW_DATABASE ) :
			$show_database           = TRUE;
			$arr_section_database    =
			[
				'id'                   => 'section_database',
				'title'                => _x( 'Database', 'wp_admin', 'activemember360' ),
				'icon'                 => 'el el-file',
				'subsection'           => TRUE,
				'class'                => 'mbr_utl_section_database',
				'fields'               => [
					[
						'id'               => 'mbr_utl_database',
						'full_width'       => TRUE,
						'type'             => 'raw',
						'content'          => '<div id="mbr_utl_database_container"></div>'
					]
				]
			];
		ENDIF;

		


		$show_php_info             = FALSE;
		IF ( ! defined( 'MBR_SHOW_PHPINFO' ) OR MBR_SHOW_PHPINFO ) :
			$show_php_info           = TRUE;
			$arr_section_php_info    =
			[
				'id'                   => 'section_php_info',
				'title'                => _x( 'PHP Info', 'wp_admin', 'activemember360' ),
				'class'                => 'mbr_utl_section_php_info',
				'subsection'           => TRUE,
				'icon'                 => 'el el-adjust-alt',
				'fields'               => [
					[
						'id'               => 'mbr_utl_php_info',
						'full_width'       => TRUE,
						'type'             => 'raw',
						'content'          => '<div id="mbr_utl_php_info_container"></div>'
					]
				]
			];
		ENDIF;

		$arr_section_cache_refresh =
		[
			'id'                     => 'section_cache_refresh',
			'title'                  => _x( 'Cache Refresh', 'wp_admin', 'activemember360' ),
			'class'                  => 'mbr_utl_section_cache_refresh',
			'icon'                   => 'el el-refresh',
			'fields'                 => [
				[
					'id'                 => 'mbr_utl_cache_refresh',
					'full_width'         => TRUE,
					'type'               => 'raw',
					'content'            => '<div id="mbr_utl_cache_refresh_container"></div>'
				]
			]
		];

		
		Redux::setSection( $this->opt_name, $arr_section_activecampaign );
		Redux::setSection( $this->opt_name, $arr_section_api_log );
		Redux::setSection( $this->opt_name, $arr_section_automations );
		Redux::setSection( $this->opt_name, $arr_section_custom_fields );
		Redux::setSection( $this->opt_name, $arr_section_forms );
		Redux::setSection( $this->opt_name, $arr_section_lists );
		Redux::setSection( $this->opt_name, $arr_section_templates );
		Redux::setSection( $this->opt_name, $arr_section_tags );
		Redux::setSection( $this->opt_name, $arr_section_webhooks );
		IF ( $show_environment OR $show_database OR $show_php_info ) :
			Redux::setSection( $this->opt_name, $arr_section_server_environment );
			IF ( $show_environment ) :
				Redux::setSection( $this->opt_name, $arr_section_environment );
			ENDIF;
			IF ( $show_database ) :
				Redux::setSection( $this->opt_name, $arr_section_database );
			ENDIF;
			IF ( $show_php_info ) :
				Redux::setSection( $this->opt_name, $arr_section_php_info );
			ENDIF;
		ENDIF;
		Redux::setSection( $this->opt_name, $arr_section_cache_refresh );
	}

	public function ajax_get_automations()
	{
		do_action( 'mbr/site_automations/fetch' );

		$arr_automations   = apply_filters( 'mbr/site_automations/get', NULL );
		$count_automations = count( $arr_automations );

		$out               = '<h2>' . sprintf( _x( 'Total: %s automation(s)', 'wp_admin', 'activemember360' ), $count_automations ) . '</h2>';
		$out              .= '<table class="wp-list-table widefat striped mbr-redux-listing  mbr-col-75">';
		$out              .= '<thead><tr><td>' . _x( 'ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Automation Name', 'wp_admin', 'activemember360' ) . '</td></tr></thead>';
		$out              .= '<tfoot><tr><td>' . _x( 'ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Automation Name', 'wp_admin', 'activemember360' ) . '</td></tr></tfoot>';
		$out              .= '<tbody id="the-list">';
		IF ( ! empty( $arr_automations ) AND is_array( $arr_automations ) ) :
			FOREACH ( $arr_automations AS $automation_id => $automation_name ) :
				$out          .= '<tr><td>' . $automation_id . '</td><td>' . $automation_name . '</td></tr>';
			ENDFOREACH;
		ELSE:
			$out            .= '<tr><td colspan="2">' . _x( 'You do not have defined any automations in ActiveCampaign.', 'wp_admin', 'activemember360' ) . '</td></tr>';
		ENDIF;
		$out              .= '</tbody>';
		$out              .= '</table>';

		echo $out;
		exit;
	}

	public function ajax_get_custom_fields()
	{
		do_action( 'mbr/site_custom_fields/fetch' );

		$arr_custom_fields      = apply_filters( 'mbr/site_custom_fields/get', NULL );
		$count_custom_fields    = count( $arr_custom_fields );

		$arr_cust_fld           = [];
		FOREACH ( $arr_custom_fields AS $k => $cust_field ) :
			$arr_cust_fld[ $cust_field['perstag'] ] = $cust_field;
		ENDFOREACH;

		ksort( $arr_cust_fld, SORT_NATURAL );

		$out      = '<h2>' . sprintf( _x( 'Total: %s custom field(s)', 'wp_admin', 'activemember360' ), $count_custom_fields ) . '</h2>';
		$out     .= '<table class="wp-list-table widefat striped mbr-redux-listing">';
		$out     .= '<thead><tr><td>' . _x( 'Title', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Perstag', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Data Type', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Default Value', 'wp_admin', 'activemember360' ) . '</td></tr></thead>';
		$out     .= '<tfoot><tr><td>' . _x( 'Title', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Perstag', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Data Type', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Default Value', 'wp_admin', 'activemember360' ) . '</td></tr></tfoot>';
		$out     .= '<tbody id="the-list">';
		IF ( ! empty( $arr_cust_fld ) AND is_array( $arr_cust_fld ) ) :
			FOREACH ( $arr_cust_fld AS $perstag => $cust_field ) :
				$out .= '<tr>
																<td>' . strtolower( $cust_field['title'] ) . '</td>
																<td>' . $cust_field['id']                  . '</td>
																<td>' . $cust_field['perstag']             . '</td>
																<td>' . $cust_field['type']                . '</td>
																<td>' . $cust_field['default']             . '</td>
															</tr>';
			ENDFOREACH;
		ELSE:
			$out   .= '<tr><td colspan="5">' . _x( 'You do not have defined any custom fields in ActiveCampaign.', 'wp_admin', 'activemember360' ) . '</td></tr>';
		ENDIF;
		$out     .= '</tbody>';
		$out     .= '</table>';

		echo $out;
		exit;
	}

	public function ajax_get_forms()
	{
		$arr_forms        = apply_filters( 'mbr/site_forms/fetch', NULL );
		$count_forms      = count( $arr_forms );

		$out              = '<h2>' . sprintf( _x( 'Total: %s form(s)', 'wp_admin', 'activemember360' ), $count_forms ) . '</h2>';
		$out             .= '<table class="wp-list-table widefat striped mbr-redux-listing mbr-col-75">';
		$out             .= '<thead><tr><td>' . _x( 'ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Form Name', 'wp_admin', 'activemember360' ) . '</td></tr></thead>';
		$out             .= '<tfoot><tr><td>' . _x( 'ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Form Name', 'wp_admin', 'activemember360' ) . '</td></tr></tfoot>';
		$out             .= '<tbody id="the-list">';
		IF ( ! empty( $arr_forms ) AND is_array( $arr_forms ) ) :
			FOREACH ( $arr_forms AS $form_id => $form_name ) :
				$out         .= '<tr><td>' . $form_id . '</td><td>' . $form_name . '</td></tr>';
			ENDFOREACH;
		ELSE:
			$out           .= '<tr><td colspan="2">' . _x( 'You do not have defined any forms in ActiveCampaign.', 'wp_admin', 'activemember360' ) . '</td></tr>';
		ENDIF;
		$out             .= '</tbody>';
		$out             .= '</table>';

		echo $out;
		exit;
	}

	public function ajax_get_lists()
	{
		do_action( 'mbr/site_lists/fetch' );

		$arr_lists        = apply_filters( 'mbr/site_lists/get', NULL );
		$count_lists      = count( $arr_lists );

		$out              = '<h2>' . sprintf( _x( 'Total: %s list(s)', 'wp_admin', 'activemember360' ), $count_lists ) . '</h2>';
		$out             .= '<table class="wp-list-table widefat striped mbr-redux-listing mbr-col-75">';
		$out             .= '<thead><tr><td>' . _x( 'ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'List Name', 'wp_admin', 'activemember360' ) . '</td></tr></thead>';
		$out             .= '<tfoot><tr><td>' . _x( 'ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'List Name', 'wp_admin', 'activemember360' ) . '</td></tr></tfoot>';
		$out             .= '<tbody id="the-list">';
		IF ( ! empty( $arr_lists ) AND is_array( $arr_lists ) ) :
			FOREACH ( $arr_lists AS $list_id => $list_name ) :
				$out         .= '<tr><td>' . $list_id . '</td><td>' . $list_name . '</td></tr>';
			ENDFOREACH;
		ELSE:
			$out           .= '<tr><td colspan="2">' . _x( 'You do not have defined any lists in ActiveCampaign.', 'wp_admin', 'activemember360' ) . '</td></tr>';
		ENDIF;
		$out             .= '</tbody>';
		$out             .= '</table>';

		echo $out;
		exit;
	}

	public function ajax_get_mail_templates()
	{
		$arr_templates   = apply_filters( 'mbr/site_messages/templates/fetch', NULL );
		$count_templates = count( $arr_templates );

		$out             = '<h2>' . sprintf( _x( 'Total: %s mail template(s)', 'wp_admin', 'activemember360' ), $count_templates ) . '</h2>';
		$out            .= '<table class="wp-list-table widefat striped mbr-redux-listing mbr-col-75">';
		$out            .= '<thead><tr><td>' . _x( 'ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Template Name', 'wp_admin', 'activemember360' ) . '</td></tr></thead>';
		$out            .= '<tfoot><tr><td>' . _x( 'ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Template Name', 'wp_admin', 'activemember360' ) . '</td></tr></tfoot>';
		$out            .= '<tbody id="the-list">';
		IF ( ! empty( $arr_templates ) AND is_array( $arr_templates ) ) :
			FOREACH ( $arr_templates AS $key => $obj_template ) :
				$out        .= '<tr><td>' . $obj_template->id . '</td><td>' . $obj_template->name . '</td></tr>';
			ENDFOREACH;
		ELSE:
			$out          .= '<tr><td colspan="2">' . _x( 'You do not have defined any templates in ActiveCampaign.', 'wp_admin', 'activemember360' ) . '</td></tr>';
		ENDIF;
		$out            .= '</tbody>';
		$out            .= '</table>';

		echo $out;
		exit;
	}

	public function ajax_get_tags()
	{
		do_action( 'mbr/site_tags/fetch' );

		$arr_tags              = apply_filters( 'mbr/site_tags/get', NULL );
		$count_tags            = count( $arr_tags );
		$arr_memberships       = apply_filters( 'mbr/site_membership_levels/get', NULL );
		$count_membership_tags = count( $arr_memberships );

		$out                   = '<h2>' . sprintf( _x( 'Membership Tags [ %s tag(s) ]', 'wp_admin', 'activemember360' ), $count_membership_tags ) . '</h2>';
		$out                  .= '<table class="wp-list-table widefat striped mbr-redux-listing mbr-col-75">';
		$out                  .= '<thead><tr><td>' . _x( 'ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Tag Name', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Membership Name', 'wp_admin', 'activemember360' ) . '</td></tr></thead>';
		$out                  .= '<tfoot><tr><td>' . _x( 'ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Tag Name', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Membership Name', 'wp_admin', 'activemember360' ) . '</td></tr></tfoot>';
		$out                  .= '<tbody id="the-list">';
		IF ( ! empty( $arr_memberships ) AND is_array( $arr_memberships ) ) :
			FOREACH ( $arr_memberships AS $tag_id => $level ) :
				$out              .= '<tr><td>' . $tag_id . '</td><td>' . $arr_tags[ $tag_id ] . '</td><td>' . $level['name'] . '</td></tr>';
			ENDFOREACH;
		ELSE:
			$out                .= '<tr><td colspan="3">' . _x( 'You do not have defined any membership levels yet.', 'wp_admin', 'activemember360' ) . '</td></tr>';
		ENDIF;
		$out                  .= '</tbody>';
		$out                  .= '</table>';

		$out                  .= '<h2>' . sprintf( _x( 'All Tags [ %s tag(s) ]', 'wp_admin', 'activemember360' ), $count_tags ) . '</h2>';
		$out                  .= '<table class="wp-list-table widefat striped mbr-redux-listing mbr-col-75">';
		$out                  .= '<thead><tr><td>' . _x( 'Tag ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Tag Name', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Membership Name', 'wp_admin', 'activemember360' ) . '</td></tr></thead>';
		$out                  .= '<tfoot><tr><td>' . _x( 'Tag ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Tag Name', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Membership Name', 'wp_admin', 'activemember360' ) . '</td></tr></tfoot>';
		$out                  .= '<tbody id="the-list">';
		IF ( ! empty( $arr_tags ) AND is_array( $arr_tags ) ) :
			FOREACH ( $arr_tags AS $tag_id => $tag_name ) :
				IF ( empty( $arr_memberships[ $tag_id ]['name'] ) ) :
					$level_name      = '';
				ELSE :
					$level_name      = $arr_memberships[ $tag_id ]['name'];
				ENDIF;
				$out              .= '<tr><td>' . $tag_id . '</td><td>' . $tag_name . '</td><td>' . $level_name . '</td></tr>';
			ENDFOREACH;
		ELSE:
			$out                .= '<tr><td colspan="3">' . _x( 'You do not have defined any tags in ActiveCampaign.', 'wp_admin', 'activemember360' ) . '</td></tr>';
		ENDIF;
		$out                  .= '</tbody>';
		$out                  .= '</table>';

		echo $out;
		exit;
	}

	public function ajax_get_webhooks()
	{
		$arr_tmp_webhooks        = apply_filters( 'mbr/webhooks/fetch', NULL );
		$arr_webhooks            = [];
		FOREACH ( $arr_tmp_webhooks AS $arr_webhook_cat ) :
			FOREACH ( $arr_webhook_cat AS $key => $webhook ) :
				$arr_webhooks[]      = $webhook;
			ENDFOREACH;
		ENDFOREACH;

		$count_webhooks          = count( $arr_webhooks );

		$out                     = '<h2>' . sprintf( _x( 'Total: %s webhook(s)', 'wp_admin', 'activemember360' ), $count_webhooks ) . '</h2>';
		$out                    .= '<table class="wp-list-table widefat striped mbr-redux-listing mbr-col-75">';
		$out                    .= '<thead><tr><td>' . _x( 'ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Webhook Name', 'wp_admin', 'activemember360' ) . '</td><td colspan="2">' . _x( 'Webhook URL', 'wp_admin', 'activemember360' ) . '</td></tr></thead>';
		$out                    .= '<tfoot><tr><td>' . _x( 'ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Webhook Name', 'wp_admin', 'activemember360' ) . '</td><td colspan="2">' . _x( 'Webhook URL', 'wp_admin', 'activemember360' ) . '</td></tr></tfoot>';
		$out                    .= '<tbody id="the-list">';
		IF ( ! empty( $arr_webhooks ) AND is_array( $arr_webhooks ) ) :
			$webhook_events        = apply_filters( 'mbr/webhooks/events/get',   NULL );
			$webhook_triggers      = apply_filters( 'mbr/webhooks/triggers/get', NULL );

			FOREACH ( $arr_webhooks AS $key => $obj_webhook ) :
				$evt                 = '';
				$trg                 = '';

				$a_obj               = array_filter( (array) $obj_webhook, function ( $var ) { return ( $var == 1 ); } );

				FOREACH ( $a_obj AS $event => $value ) :
					IF ( in_array( $event, $webhook_events ) ) :
						$evt            .= ' ' . $event .  ',';
					ELSEIF ( in_array( $event, $webhook_triggers ) ) :
						$trg            .= ' ' . substr( $event, 5 ) .  ',';
					ENDIF;
				ENDFOREACH;

				$events_triggers     = '<strong>' . _x( 'Events:',   'wp_admin', 'activemember360' ) . '</strong>' . trim( $evt, ',' ) . '<br />';
				$events_triggers    .= '<strong>' . _x( 'Triggers:', 'wp_admin', 'activemember360' ) . '</strong>' . trim( $trg, ',' );
				$out                .= '<tr><td>' . $obj_webhook->id . '</td><td>' . $obj_webhook->name . '</td><td>' . $obj_webhook->url . '</td><td>' . $events_triggers . '</td></tr>';
			ENDFOREACH;

		ELSE:
			$out                  .= '<tr><td colspan="4">' . _x( 'You do not have defined any webhooks in ActiveCampaign.', 'wp_admin', 'activemember360' ) . '</td></tr>';
		ENDIF;
		$out                    .= '</tbody>';
		$out                    .= '</table>';

		echo $out;
		exit;
	}

	public function ajax_get_api_log()
	{
		$template = '<table id="mbr_api_log_tbl" class="wp-list-table widefat striped mbr-redux-listing" style="width:100%;"><thead>%1$s</thead><tfoot>%1$s</tfoot></table>';
		$str      = '<tr><td>' . _x( 'ID', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Method', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Response Time', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Params', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Post Data', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Response', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'IP', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'User', 'wp_admin', 'activemember360' ) . '</td><td>' . _x( 'Timestamp', 'wp_admin', 'activemember360' ) . '</td></tr>';

		echo '<script src="' . plugins_url('js/datatables_mbr.js', __FILE__ ) . '"></script>';
		echo sprintf( $template, $str );
		exit;
	}

	public function ajax_get_api_log_results()
	{
		global $MBR_OPTIONS, $wpdb;

		$arr_order_by = [];
		FOR ( $x = 0; $x < 7; $x++ ) :
			IF ( isset( $_GET['order'][ $x ]['column'] ) ) :
				IF ( $_GET['order'][ $x ]['column'] == 0 ) :
					$arr_order_by[] = "`id` {$_GET['order'][ $x ]['dir']}";
				ELSEIF ( $_GET['order'][ $x ]['column'] == 1 ) :
					$arr_order_by[] = "`method` {$_GET['order'][ $x ]['dir']}";
				ELSEIF ( $_GET['order'][ $x ]['column'] == 2 ) :
					$arr_order_by[] = "`resptime` {$_GET['order'][ $x ]['dir']}";
				ELSEIF ( $_GET['order'][ $x ]['column'] == 6 ) :
					$arr_order_by[] = "`ipnr` {$_GET['order'][ $x ]['dir']}";
				ELSEIF ( $_GET['order'][$x]['column'] == 7 ) :
					$arr_order_by[] = "`user` {$_GET['order'][ $x ]['dir']}";
				ELSEIF ( $_GET['order'][ $x ]['column'] == 8 ) :
					$arr_order_by[] = "`timestamp` {$_GET['order'][ $x ]['dir']}";
				ENDIF;
			ELSE:
				break;
			ENDIF;
		ENDFOR;

		$order_by   = '';
		IF ( ! empty( $arr_order_by ) ) :
			$order_by = 'ORDER BY ' . implode( ',', $arr_order_by );
		ENDIF;

		$latest_logs = $MBR_OPTIONS['apilog_limit'] + $MBR_OPTIONS['webhooklog_limit'];

		$apilog_display_key = $wpdb->get_var( "SELECT id FROM {$wpdb->tbl_apilog} ORDER BY id DESC LIMIT {$latest_logs}, 1" );
		IF ( $apilog_display_key == '' ) :
			$apilog_display_key = 0;
		ENDIF;

		$where_clause    = "WHERE id > {$apilog_display_key}";
		IF ( isset( $_GET['search']['value'] ) AND trim( $_GET['search']['value']) <> '' ) :
			$search_value  = $_GET['search']['value'];
			$where_clause .=  " AND `response` LIKE '%{$search_value}%' OR `params` LIKE '%{$search_value}%' OR `post_data` LIKE '%{$search_value}%' OR `id` LIKE '%{$search_value}%' OR `timestamp` LIKE '%{$search_value}%' OR `user` LIKE '%{$search_value}%' OR `resptime` LIKE '%{$search_value}%' OR `ipnr` LIKE '%{$search_value}%' OR `method` LIKE '%{$search_value}%'";
		ENDIF;

		$results         = $wpdb->get_results( "SELECT * FROM {$wpdb->tbl_apilog} {$where_clause} {$order_by} LIMIT {$_GET['start']}, {$_GET['length']}", OBJECT );
		$filtered_rows   = $wpdb->get_var( "SELECT COUNT(id) AS `count` FROM {$wpdb->tbl_apilog} {$where_clause} LIMIT 1" );
		$total_rows      = $wpdb->get_var( "SELECT COUNT(id) AS `count` FROM {$wpdb->tbl_apilog}  WHERE id > {$apilog_display_key} LIMIT 1" );

		$arr_apilogs     = [];
		FOREACH ( $results AS $key => $obj ) :
			$params        = $this->apilog_popup( 'patbpop_apilog' . $obj->id, print_r( gz_decode( $obj->params ),    TRUE ) );
			$postdata      = $this->apilog_popup( 'pdbpop_apilog'  . $obj->id, print_r( gz_decode( $obj->post_data ), TRUE ) );
			$response      = $this->apilog_popup( 'rebpop_apilog'  . $obj->id, print_r( gz_decode( $obj->response ),  TRUE ) );
			$arr_apilogs[] = [ $obj->id, $obj->method, $obj->resptime, $params, $postdata, $response, $obj->ipnr, $obj->user, $obj->timestamp ];
		ENDFOREACH;

		echo json_encode( [ 'recordsTotal' => $total_rows, 'recordsFiltered' => $filtered_rows, 'data' => $arr_apilogs ] );
		exit;
	}

	public function apilog_popup( $div, $data )
	{
		IF ( trim( $data ) ) :

			$out     = '';

			static $has_run = FALSE;

			IF ( ! $has_run ) :
				$has_run = TRUE;

				$out  .= "
					<script>
						jQuery( document ).ready( function() {
							jQuery( '.mbr-thickbox-popup' ).click( function() {
								inline_id       = jQuery( this ).attr( 'data-mbrapilogid' );
								old_tb_position = tb_position;

								tb_position     = function() {
									jQuery( '#TB_window' ).css( { marginLeft: '-' + parseInt( ( TB_WIDTH / 2 ), 10 ) + 'px', width: TB_WIDTH + 'px', height: TB_HEIGHT + 2 + 'px', marginTop: '-' + parseInt( ( TB_HEIGHT / 2 ), 10 ) + 'px' } );
								}

								tb_show( '" . _x( 'Display', 'wp_admin', 'activemember360' ) . "', '/#TB_inline?width=960&height=680&inlineId=' + inline_id, '' );
								tb_position     = old_tb_position;
							} );
						} );
					</script>";
			ENDIF;
			$out   .= '<div id="' . $div . '"    style="display:none;"><pre style="white-space: pre-wrap">' . htmlspecialchars( print_r( $data, TRUE ) ) . '</pre></div>';
			$out   .= '<a href="#" class="mbr-thickbox-popup" data-mbrapilogid="' . $div . '">' . _x( 'Display', 'wp_admin', 'activemember360' ) . '</a>';
			return $out;
		ENDIF;

		return NULL;
	}

	public function ajax_get_database()
	{
		global $wpdb;
		$out                = '';
		IF ( $arrTABLES = $wpdb->get_results( "SHOW FULL TABLES", ARRAY_N ) ) :
			FOREACH ( $arrTABLES AS $k => $table ) :
				IF ( $table[1] <> 'BASE TABLE' ) :
					continue;
				ENDIF;

				IF ( $arrCREATE = $wpdb->get_results( "SHOW CREATE TABLE " . $table[0], ARRAY_N ) ) :
					$txt          = $arrCREATE[0][1];
				ELSE :
					$txt          = '&nbsp;';
				ENDIF;

				$records        = $wpdb->get_var( "SELECT COUNT(*) FROM " . $table[0] );
				$out           .= '<h2>' . $table[0] . ' [ ' . $records . ' ' . _x( 'Record(s)', 'wp_admin', 'activemember360' ) . ' ]</h2>';
				$out           .= '<textarea style="min-height:200px;max-height:400px;min-width:50%;max-width:100%;">' . $txt . '</textarea>';
			ENDFOREACH;
		ENDIF;

		echo $out;
		exit;
	}

	public function ajax_get_environment() {
		global $wpdb, $MBR_OPTIONS, $blog_id;
		
		$arr_license = apply_filters( 'mbr/license/status/get', NULL );
		$out         = '<h2>' . _x( 'License', 'wp_admin', 'activemember360' ) . '</h2>';
		$out        .= '<table class="wp-list-table widefat striped mbr-redux-listing mbr-col-300">';
		$out        .= '<tbody id="the-list">';
		$out        .= '<tr><td><strong>' . _x( 'Status', 'wp_admin', 'activemember360' ) . '</strong></td><td>' . $arr_license['status' ] . '</td></tr>';
		$out        .= '<tr><td><strong>' . _x( 'Type',   'wp_admin', 'activemember360' ) . '</strong></td><td>' . $arr_license['level' ]  . '</td></tr>';
		$out        .= '</tbody>';
		$out        .= '</table>';

		
		$out        .= '<h2>' . _x( 'Versions', 'wp_admin', 'activemember360' ) . '</h2>';
		$out        .= '<table class="wp-list-table widefat striped mbr-redux-listing mbr-col-300">';
		$out        .= '<tbody id="the-list">';
		$out        .= '<tr><td><strong>' . _x( 'PHP',             'wp_admin', 'activemember360' ) . '</strong></td><td>' . phpversion() . '</td></tr>';
		$out        .= '<tr><td><strong>' . _x( 'WordPress',       'wp_admin', 'activemember360' ) . '</strong></td><td>' . strtolower( get_bloginfo( 'version' ) )  . '</td></tr>';
		$out        .= '<tr><td><strong>' . _x( 'ActiveMember360', 'wp_admin', 'activemember360' ) . '</strong></td><td>' . MBRVERSION   . '</td></tr>';
		IF ( defined( 'MBRBUILD' ) ) :
			$out      .= '<tr><td><strong>' . _x( 'Build',           'wp_admin', 'activemember360' ) . '</strong></td><td>' . MBRBUILD     . '</td></tr>';
		ENDIF;
		$out        .= '</tbody>';
		$out        .= '</table>';

		
		$out        .= '<h2>' . _x( 'Site', 'wp_admin', 'activemember360' ) . '</h2>';
		$out        .= '<table class="wp-list-table widefat striped mbr-redux-listing mbr-col-300">';
		$out        .= '<tbody id="the-list">';
		$out        .= '<tr><td><strong>' . _x( 'Site Name',              'wp_admin', 'activemember360' ) . '</strong></td><td>' . strtolower( get_bloginfo( 'blog' ) )         . '</td></tr>';
		$out        .= '<tr><td><strong>' . _x( 'Site Description',       'wp_admin', 'activemember360' ) . '</strong></td><td>' . strtolower( get_bloginfo( 'description' ) )  . '</td></tr>';
		$out        .= '<tr><td><strong>' . _x( 'Site URL',               'wp_admin', 'activemember360' ) . '</strong></td><td>' . strtolower( site_url() )      . '</td></tr>';
		$out        .= '<tr><td><strong>' . _x( 'WP URL',                 'wp_admin', 'activemember360' ) . '</strong></td><td>' . strtolower( home_url() )      . '</td></tr>';
		$out        .= '<tr><td><strong>' . _x( 'Admin URL',              'wp_admin', 'activemember360' ) . '</strong></td><td>' . strtolower( admin_url() )     . '</td></tr>';
		$out        .= '<tr><td><strong>' . _x( 'Plugins URL',            'wp_admin', 'activemember360' ) . '</strong></td><td>' . strtolower( plugins_url() )   . '</td></tr>';
		$out        .= '<tr><td><strong>' . _x( 'Admin email',            'wp_admin', 'activemember360' ) . '</strong></td><td>' . $MBR_OPTIONS['mail_fromaddr'] . '</td></tr>';
		$out        .= '<tr><td><strong>' . _x( 'Multisite installation', 'wp_admin', 'activemember360' ) . '</strong></td><td>' . ( MBR_MULTISITE ? _x( 'Yes', 'wp_admin', 'activemember360' ) : _x( 'No', 'wp_admin', 'activemember360' ) ) . '</td></tr>';
		$out        .= '</tbody>';
		$out        .= '</table>';

		
		$out        .= '<h2>' . _x( 'Server', 'wp_admin', 'activemember360' ) . '</h2>';
		$out        .= '<table class="wp-list-table widefat striped mbr-redux-listing mbr-col-300">';
		$out        .= '<tbody id="the-list">';
		$out        .= '<tr><td><strong>' . _x( 'Plugin location',       'wp_admin', 'activemember360' ) . '</strong></td><td>' . dirname( __FILE__ ) . '</td></tr>';
		$out        .= '<tr><td><strong>' . _x( 'WP Memory Limit',       'wp_admin', 'activemember360' ) . '</strong></td><td>' . apply_filters( 'mbr/memory/limit/get', NULL )       . '</td></tr>';
		$out        .= '<tr><td><strong>' . _x( 'WP-Admin Memory Limit', 'wp_admin', 'activemember360' ) . '</strong></td><td>' . apply_filters( 'mbr/memory/limit/admin/get', NULL ) . '</td></tr>';
		$out        .= '<tr><td><strong>' . _x( 'Memory used',           'wp_admin', 'activemember360' ) . '</strong></td><td>' . memory_get_usage( TRUE ) / 1048576 . 'M</td></tr>';
		$out        .= '<tr><td><strong>' . _x( 'Server IP',             'wp_admin', 'activemember360' ) . '</strong></td><td>' . $_SERVER['SERVER_ADDR']   . '</td></tr>';
		$out        .= '<tr><td><strong>' . _x( 'Host name',             'wp_admin', 'activemember360' ) . '</strong></td><td>' . MBR_DOMAIN . '</td></tr>';
		$out        .= '</tbody>';
		$out        .= '</table>';

		
		IF ( ! defined( 'MBR_SHOW_WPCONFIG' ) OR MBR_SHOW_WPCONFIG ) :
			$file                  = 'wp-config.php';
			$content_wp_config     = '';
			IF ( file_exists( ABSPATH . $file ) AND $hdl = fopen( ABSPATH . $file, 'r' ) ) :
				$content_wp_config   = fread( $hdl, filesize( ABSPATH . $file ) );
				fclose( $hdl );
			ENDIF;
			$out      .= '<h2>' . _x( 'wp-config.php', 'wp_admin', 'activemember360' ) . '</h2>';
			$out      .= '<textarea style="height:400px;width:100%;">' . $content_wp_config . '</textarea>';
		ENDIF;

		
		IF ( ! defined( 'MBR_SHOW_HTACCESS' ) OR MBR_SHOW_HTACCESS ) :
			$file                  = '.htaccess';
			$content_htaccess      = '';
			IF ( file_exists( ABSPATH . $file ) AND $hdl = fopen( ABSPATH . $file, 'r' ) ) :
				$content_htaccess    = fread( $hdl, filesize( ABSPATH . $file ) );
				fclose( $hdl );
			ENDIF;
			$out      .= '<h2>' . _x( '.htaccess', 'wp_admin', 'activemember360' ) . '</h2>';
			$out      .= '<textarea style="height:400px;width:100%;">' . $content_htaccess . '</textarea>';
		ENDIF;

		
		IF ( ! defined( 'MBR_SHOW_CONSTANTS' ) OR MBR_SHOW_CONSTANTS ) :
			$tmpCONSTANTS          = get_defined_constants( TRUE );
			$tmpCONSTANTS          = $tmpCONSTANTS['user'];
			ksort( $tmpCONSTANTS );
			$out      .= '<h2>' . _x( 'ActiveMember360 Constants', 'wp_admin', 'activemember360' ) . '</h2>';
			$out      .= '<table class="wp-list-table widefat striped mbr-redux-listing mbr-col-300">';
			$out      .= '<tbody id="the-list">';
			FOREACH ( $tmpCONSTANTS AS $k => $v ) :
				IF ( substr( $k, 0, 3 ) == 'MBR' OR substr( $k, 0, 6 ) == 'MS360_' ) :
					IF ( $v === FALSE ) :
						$v               = 'FALSE';
					ELSEIF ( $v === TRUE ) :
						$v               = 'TRUE';
					ENDIF;
					$out  .= '<tr><td><strong>' . str_pad( $k, 20 ) . '</strong></td><td>' . htmlspecialchars( (string) $v ) . '</td></tr>';
				ENDIF;
			ENDFOREACH;
			$out      .= '</tbody>';
			$out      .= '</table>';
		ENDIF;

		
		$aplugins                = blog_get_option( 'active_plugins', FALSE );
		$iplugins                = get_plugins();
		$arr_active_plugins      = [];
		$arr_inactive_plugins    = [];
		FOREACH ( $iplugins AS $k => $row ) :
			$plugin                = ['name' => $row['Name'], 'version' => ( ! empty( $row['Version'] ) ? $row['Version'] : _x( 'Unknown', 'wp_admin', 'activemember360' ) ) ];
			IF ( in_array( $k, $aplugins ) ) :
				$arr_active_plugins[]   = $plugin;
			ELSE :
				$arr_inactive_plugins[] = $plugin;
			ENDIF;
		ENDFOREACH;
		natcasesort( $arr_active_plugins );
		natcasesort( $arr_inactive_plugins );

		$out        .= '<h2>' . _x( 'Active Plugins', 'wp_admin', 'activemember360' ) . '</h2>';
		$out        .= '<table class="wp-list-table widefat striped mbr-redux-listing mbr-col-300">';
		$out        .= '<thead><tr><td>' . _x( 'Plugin Name', 'wp_admin', 'activemember360' ). '</td><td>' . _x( 'Plugin Version', 'wp_admin', 'activemember360' ) . '</td></tr></thead>';
		$out        .= '<tfoot><tr><td>' . _x( 'Plugin Name', 'wp_admin', 'activemember360' ). '</td><td>' . _x( 'Plugin Version', 'wp_admin', 'activemember360' ) . '</td></tr></tfoot>';
		$out        .= '<tbody id="the-list">';
		IF ( ! empty ( $arr_active_plugins ) AND is_array( $arr_active_plugins ) ) :
			FOREACH ( $arr_active_plugins AS $key => $plugin ) :
				$out    .= '<tr><td>' . $plugin['name'] . '</td><td>' . $plugin['version'] . '</td></tr>';
			ENDFOREACH;
		ELSE:
			$out      .= '<tr><td colspan="2">' . _x( 'You do not have any active plugins', 'wp_admin', 'activemember360' ) . '</td></tr>';
		ENDIF;
		$out        .= '</tbody>';
		$out        .= '</table>';

		$out        .= '<h2>' . _x( 'Inactive Plugins', 'wp_admin', 'activemember360' ) . '</h2>';
		$out        .= '<table class="wp-list-table widefat striped mbr-redux-listing mbr-col-300">';
		$out        .= '<thead><tr><td>' . _x( 'Plugin Name', 'wp_admin', 'activemember360' ). '</td><td>' . _x( 'Plugin Version', 'wp_admin', 'activemember360' ) . '</td></tr></thead>';
		$out        .= '<tfoot><tr><td>' . _x( 'Plugin Name', 'wp_admin', 'activemember360' ). '</td><td>' . _x( 'Plugin Version', 'wp_admin', 'activemember360' ) . '</td></tr></tfoot>';
		$out        .= '<tbody id="the-list">';
		IF ( ! empty ( $arr_inactive_plugins ) AND is_array( $arr_inactive_plugins ) ) :
			FOREACH ( $arr_inactive_plugins AS $key => $plugin ) :
				$out    .= '<tr><td>' . $plugin['name'] . '</td><td>' . $plugin['version'] . '</td></tr>';
			ENDFOREACH;
		ELSE:
			$out      .= '<tr><td colspan="2">' . _x( 'You do not have any inactive plugins', 'wp_admin', 'activemember360' ) . '</td></tr>';
		ENDIF;
		$out        .= '</tbody>';
		$out        .= '</table>';

		
		IF ( MBR_MULTISITE ) :
			IF ( $rows = $wpdb->get_results( "SELECT * FROM {$wpdb->base_prefix}blogs" ) ) :
				$arr_multisites      = [];
				FOREACH ( $rows AS $k => $row ) :
					$arr_multisites[]  = $row->site_id . " - " . $row->blog_id . " - " . $row->domain . " - " . $row->path;
				ENDFOREACH;
			ENDIF;

			$out      .= '<h2>' . _x( 'Multisite', 'wp_admin', 'activemember360' ) . '</h2>';
			$out      .= '<table class="wp-list-table widefat striped mbr-redux-listing mbr-col-300">';
			$out      .= '<tbody id="the-list">';
			$out      .= '<tr><td><strong>' . _x( 'Multisite installation', 'wp_admin', 'activemember360' ) . '</strong></td><td>' . _x( 'Yes', 'wp_admin', 'activemember360' ) . '</td></tr>';
			$out      .= '<tr><td><strong>' . _x( 'Blog ID',                'wp_admin', 'activemember360' ) . '</strong></td><td>' . $blog_id . '</td></tr>';
			$out      .= '<tr><td style="vertical-align:top;"><strong>' . _x( 'Blogs',                  'wp_admin', 'activemember360' ) . '</strong></td><td>' . implode( '<br />', $arr_multisites ) . '</td></tr>';
			$out      .= '</tbody>';
			$out      .= '</table>';
		ENDIF;

		echo $out;
		exit;
	}

	public function ajax_get_php_info()
	{
		ob_start();
		phpinfo();
		$php_info = ob_get_clean();
		echo "<iframe style='width:100%;background-color:#fff;height:100vh;' srcdoc='$php_info'></iframe>";
		exit;
	}

	public function ajax_get_cache_refresh()
	{
		$out  = '';

		
		$out .= "
		<script>
			jQuery( '#mbr_utl_refresh_local_cache' ).click( function () {
				jQuery( '#mbr_utl_refresh_local_cache_message' ).html('');
				mbr_send_utl_ajax( 'mbr_utl_refresh_local_cache', '#mbr_utl_refresh_local_cache_message' );
				return false;
			} );
		</script>";

		$out .= '<div id="mbr_utl_refresh_local_cache_message"></div>';
		$out .= '<p>' . _x( 'Your local cache is automatically refreshed at certain intervals as well when the need is auto-detected.', 'wp_admin', 'activemember360' ) . '</p>';
		$out .= '<p>' . _x( 'This module enables you to force an immediate cache refresh.', 'wp_admin', 'activemember360' ) . '</p>';
		$out .= '<p><span id="mbr_utl_refresh_local_cache" class="button button-primary">' . _x( 'Refresh Now', 'wp_admin', 'activemember360' ) . '</span></p>';
		echo $out;
		exit;
	}

	public function ajax_refresh_local_cache()
	{
		do_action( 'mbr/cache/refresh' );
		$out = mbr_admin_msg( _x( 'Your local cache has been updated.', 'wp_admin', 'activemember360' ), FALSE, TRUE );
		echo $out;
		exit;
	}

}
$GLOBALS['mbr_inst_utilities'] = mbr_utilities::getInstance();