<?php
final class mbr_webhooks EXTENDS mbr_redux_base_class
{
	use TRAIT__getInstance;

	public $opt_name                = 'mbr_webhooks';
	public $page_slug               = 'mbr360-webhooks';

	public $site_webhook_fields_key = 'site_contact_webhook_fields';
	public $contact_webhook_fields  = [ 'id', 'first_name', 'last_name', 'email', 'phone', 'ip', 'tags', 'last_updated' ];

	public $webhooks_events         = [ 'subscribe', 'unsubscribe', 'update', 'bounce', 'sent', 'open', 'click', 'forward', 'share', 'reply', 'subscriber_note', 'contact_tag_added', 'contact_tag_removed', 'list_add', 'sms_sent', 'sms_reply', 'sms_unsub', 'deal_add', 'deal_update', 'deal_note_add', 'deal_pipeline_add', 'deal_stage_add', 'deal_task_add', 'deal_tasktype_add', 'deal_task_complete' ];
	public $webhooks_triggers       = [ 'init_public', 'init_admin', 'init_system', 'init_api' ];

	public $webhook_default_inits   = [ 'public', 'admin', 'system', 'api' ];
	public $webhook_default_actions = [ 'subscribe', 'update' ];

	public function _load()
	{
		$this->webhook_name           = 'MBR.WH.' . md5('4711mbr' . MBRWPURL . 'version0001');

		$list_home_url                = explode('://', MBRWPURL );
		$list_home_url                = $list_home_url[1];
		$this->webhook_list_name      = 'Member List for ' . $list_home_url;

		add_action( 'parse_request',                                           [ $this, 'webhook_parser' ], 7 );

		add_action( 'init',                                                    [ $this, '_init' ] );
		add_action( 'init',                                                    [ $this, 'setup_schedules_webhooks' ], 0 );
		add_action( 'mbr/webhooks/event/fields/fetch',                         [ $this, 'fetch_ac_webhook_fields' ] );
		add_action( 'mbr/webhooks/event/check',                                [ $this, 'check_webhooks' ] );

		IF ( is_admin() ) :
			add_action( 'plugins_loaded',                                        [ $this, 'init_admin_panel' ], 99999 );
			add_action( 'plugins_loaded',                                        [ $this, 'init_sections' ],    99999 );
			add_filter( "redux/{$this->opt_name}/panel/template/header.tpl.php", [ $this, 'alternate_header' ] );
			add_action( "redux/options/{$this->opt_name}/saved",                 [ $this, 'check_webhooks' ] );
		ENDIF;
	}

	public function _init()
	{
		
		$this->ac_account_info     = apply_filters( 'mbr/ac_account/get', NULL );
		$this->webhook_url         = MBRWPURL . '/?mbr=' . $this->webhook_name . '&appname=' . $this->ac_account_info['account'];
		$this->mbr_wh_params       = [
			'name'   => $this->webhook_name,
			'url'    => $this->webhook_url . '&master=' . date( 'YmdHis' ),
			'lists'  => [ 0 ],
			'action' => $this->webhook_default_actions,
			'init'   => $this->webhook_default_inits
		];

		$remote_wh_fields               = json_decode( get_option( $this->site_webhook_fields_key ) );
		IF ( is_array( $remote_wh_fields ) ) :
			$this->contact_webhook_fields = $remote_wh_fields;
		ENDIF;

	}

	public function setup_schedules_webhooks()
	{
		IF ( ! wp_next_scheduled( 'mbr/webhooks/event/check' ) ) :
			wp_schedule_event( time(), 'hourly', 'mbr/webhooks/event/check' );
		ENDIF;
		IF ( ! wp_next_scheduled( 'mbr/webhooks/event/fields/fetch' ) ) :
			wp_schedule_event( time(), 'mbr_remote_ac_fields_schedule', 'mbr/webhooks/event/fields/fetch' );
		ENDIF;
	}

	public function fetch_ac_webhook_fields()
	{
		IF ( $wh_fields = get_file_contents( MBR()->ac_webhook_fields_url, TRUE ) ) :
			update_option( $this->site_webhook_fields_key, $wh_fields );
		ENDIF;
	}

	public function fetch_webhooks()
	{
		$page         = 1;
		$max_per_page = 20;

		$arr_webhooks = [];

		DO {

			$obj_wh = apply_filters( 'mbr/api', 'webhook_list?page=' . $page++ );
			IF ( $obj_wh->result_code == 0 ) :
				break;
			ENDIF;

			$fetch_count = 0;
			FOREACH ( (array) $obj_wh AS $k => $webhook ) :
				
				IF ( is_object( $webhook ) ) :
					$fetch_count++;
					
					IF ( $webhook->name == $this->webhook_name ) :
						$arr_webhooks['local'][]       = $webhook;
					ELSEIF ( 'MBR.WH.' == substr( $webhook->name, 0, 7 ) ) :
						$arr_webhooks['remote'][]      = $webhook;
					ELSEIF ( 'MBR.WHM.' == substr( $webhook->name, 0, 8 ) ) :
						$arr_webhooks['old_masters'][] = $webhook;
					ENDIF;
				ENDIF;
			ENDFOREACH;

			IF ( $fetch_count < $max_per_page ) :
				break;
			ENDIF;

		} WHILE ( TRUE );

		return $arr_webhooks;
	}

	public function is_master_webhook( $webhook = FALSE )
	{
		IF ( empty( $webhook ) OR ! is_object( $webhook ) ) :
			return FALSE;
		ENDIF;

		IF ( $webhook->listid > 0 ) :
			return FALSE;
		ENDIF;

		$tmpURL = parse_url( $webhook->url );
		parse_str( $tmpURL['query'], $query );

		IF ( ! empty( $query['master'] ) ) :
			return TRUE;
		ENDIF;

		return FALSE;
	}

	public function add_webhook( $list_id = 0 )
	{
		$params          = $this->mbr_wh_params;
		$params['lists'] = [ $list_id ];
		$response        = apply_filters( 'mbr/api', 'webhook_add', $params );

		return $response;
	}

	public function make_master_webhook( $make_to_master )
	{
		IF ( ! empty( $make_to_master ) AND is_object( $make_to_master ) ) :
			$arr_master_webhook = [
				'id'     => $make_to_master->id,
				'name'   => $make_to_master->name,
				'url'    => $make_to_master->url . '&master=' . date( 'YmdHis' ),
				'lists'  => $make_to_master->listid,
				'action' => $this->webhook_default_actions,
				'init'   => $this->webhook_default_inits
			];
			apply_filters( 'mbr/api', 'webhook_edit', $arr_master_webhook );
		ENDIF;
	}

	public function make_slave_webhooks( $arr_webhooks )
	{
		IF ( ! empty( $arr_webhooks ) AND is_array( $arr_webhooks ) ) :
			FOREACH ( $arr_webhooks AS $key => $webhook ) :
				IF ( $this->is_master_webhook( $webhook ) ) :
					$params           = [];
					$params['name']   = $webhook->name;
					$params['url']    = preg_replace( '/(&master=[[:digit:]]{14})/', '', $webhook->url );
					$params['id']     = $webhook->id;
					$params['lists']  = [ 0 ];
					$params['action'] = $this->webhook_default_actions;
					$params['init']   = $this->webhook_default_inits;
					apply_filters( 'mbr/api', 'webhook_edit', $params );
				ENDIF;
			ENDFOREACH;
		ENDIF;
	}

	public function delete_all_site_webhooks()
	{
		$arr_webhooks = $this->fetch_webhooks();
		IF ( ! empty( $arr_webhooks['local'] ) ) :
			$this->delete_webhooks( $arr_webhooks['local'] );
		ENDIF;

		IF ( ! empty( $arr_webhooks['remote'] ) ) :
			$arr_master     = $this->check_for_master_webhook( $arr_webhooks['remote'] );
			$master_exists  = $arr_master['master_exists'];
			$make_to_master = $arr_master['make_to_master'];

			IF ( ! $master_exists ) :
				$this->make_master_webhook( $make_to_master );
			ENDIF;
		ENDIF;
	}

	public function delete_webhooks( $arr_webhooks )
	{
		IF ( ! empty( $arr_webhooks ) AND is_array( $arr_webhooks ) ) :
			FOREACH ( $arr_webhooks AS $key => $webhook ) :
				apply_filters( 'mbr/api', 'webhook_delete?id=' . $webhook->id );
			ENDFOREACH;
		ENDIF;
	}

	public function check_for_master_webhook( $arr_webhooks = [] )
	{
		$master_exists  = FALSE;
		$make_to_master = [];
		IF ( ! empty( $arr_webhooks ) ) :
			FOREACH ( (array) $arr_webhooks AS $key => $webhook ) :
				IF ( empty( $make_to_master ) ) :
					$make_to_master = $webhook;
				ENDIF;
				IF ( $this->is_master_webhook( $webhook ) ) :
					$master_exists = TRUE;
					break;
				ENDIF;
			ENDFOREACH;
		ENDIF;

		return [ 'master_exists' => $master_exists, 'make_to_master' => $make_to_master ];
	}

	public function get_auto_generated_list_id()
	{
		do_action( 'mbr/site_lists/fetch', NULL );
		$arr_lists            = apply_filters( 'mbr/site_lists/get', NULL );
		$site_list_id         = FALSE;
		FOREACH ( $arr_lists AS $list_id => $list_name ) :
			IF ( $list_name == $this->webhook_list_name ) :
				$site_list_id     = $list_id;
				break;
			ENDIF;
		ENDFOREACH;

		return $site_list_id;
	}

	public function check_webhooks( $settings = FALSE )
	{
		IF ( ! mbr_get_lock( __FUNCTION__ ) ) :
			return;
		ENDIF;

		IF ( "redux/options/{$this->opt_name}/saved" == current_filter() ) : 
			$arr_settings = $settings;
		ELSE:
			$arr_settings = get_option( $this->opt_name, FALSE ); 
		ENDIF;

		IF ( empty( $arr_settings ) ) :
			
			$arr_settings = [ 'mbr_wh_webhooks_enabled' => 1, 'mbr_wh_webhooks_type' => 'default' ];
		ENDIF;

		$arr_webhooks = $this->fetch_webhooks();

		IF ( ! empty( $arr_webhooks['old_masters'] ) ) :
			$this->delete_webhooks( $arr_webhooks['old_masters'] );
		ENDIF;

		
		IF ( empty( $arr_settings['mbr_wh_webhooks_enabled'] ) OR ! $arr_settings['mbr_wh_webhooks_enabled'] OR $arr_settings['mbr_wh_webhooks_enabled'] == '0' ) :
			IF ( ! empty( $arr_webhooks['local'] ) ) :
				$this->delete_webhooks( $arr_webhooks['local'] );
			ENDIF;

			IF ( ! empty( $arr_webhooks['remote'] ) ) :
				$arr_master     = $this->check_for_master_webhook( $arr_webhooks['remote'] );

				$master_exists  = $arr_master['master_exists'];
				$make_to_master = $arr_master['make_to_master'];

				IF ( ! $master_exists ) :
					
					$this->make_master_webhook( $make_to_master );
				ENDIF;

			ENDIF;

			return;
		ENDIF;

		IF ( ! empty( $arr_settings['mbr_wh_webhooks_enabled'] ) AND $arr_settings['mbr_wh_webhooks_enabled'] == '1' ) :
			IF ( $arr_settings['mbr_wh_webhooks_type'] == 'default' ) :
				
				IF ( ! empty( $arr_webhooks['remote'] ) ) :
					$this->make_slave_webhooks( $arr_webhooks['remote'] );
				ENDIF;

				$wh_found = FALSE;
				$update   = FALSE;

				IF ( ! empty( $arr_webhooks['local'] ) ) :
					$ignore_keys     = [ 'id', 'cdate', 'listid', 'integration', 'name', 'url' ];

					$new_actions     = $this->webhook_default_actions;
					FOREACH ( $this->webhook_default_inits AS $k => $key ) :
						$new_actions[] = 'init_' . $key;
					ENDFOREACH;

					$valid_webhooks  = [];

					FOREACH ( (array) $arr_webhooks['local'] AS $k => $webhook ) :

						IF ( in_array( [ $webhook->name, $webhook->listid ], $valid_webhooks ) ) :
							apply_filters( 'mbr/api', 'webhook_delete?id=' . $webhook->id );
							continue;
						ELSEIF ( in_array( [ $webhook->name, 0 ], $valid_webhooks ) ) :
							apply_filters( 'mbr/api', 'webhook_delete?id=' . $webhook->id );
							continue;
						ELSEIF ( $webhook->listid == 0 ) :
							IF ( ! empty( $valid_webhooks ) ) :
								FOREACH ( $valid_webhooks AS $id => $wh ) :
									IF ( $wh[0] == $webhook->name ) :
										apply_filters( 'mbr/api', 'webhook_delete?id=' . $id );
										unset( $valid_webhooks[ $id ] );
									ENDIF;
								ENDFOREACH;
							ENDIF;
						ENDIF;

						$valid_webhooks[ $webhook->id ] = [ $webhook->name, $webhook->listid ];

						$wh_found = TRUE;

						
						$tmpURL = parse_url( $webhook->url );
						parse_str( $tmpURL['query'], $query );

						$webhook_appname = empty( $query['appname'] ) ? FALSE  : $query['appname'];
						$webhook_master  = empty( $query['master'] )  ? FALSE  : $query['master'];

						
						IF ( $webhook_appname <> $this->ac_account_info['account'] ) :
							$update = TRUE;
						ENDIF;

						IF ( empty( $webhook_master ) ) :
							$update = TRUE;
						ENDIF;

						IF ( ! $update ) :
							
							FOREACH ( (array) $webhook AS $key => $val ) :
								IF ( in_array( $key, $ignore_keys) ) :
									continue;
								ENDIF;
								IF ( $val == 1 AND ! in_array( $key, $new_actions ) ) :
									$update = TRUE;
									break;
								ELSEIF ( $val == 0 AND in_array( $key, $new_actions ) ) :
									$update = TRUE;
									break;
								ENDIF;
							ENDFOREACH;
						ENDIF;

						IF ( $update ) :
							$max_attempts      = 9;
							$params            = $this->mbr_wh_params;
							$params['id']      = $webhook->id;

							$site_list_id      = $this->get_auto_generated_list_id();
							
							IF ( $webhook->listid <> $site_list_id ) :
								$params['lists'] = $webhook->listid;
							ENDIF;
							DO {
								$response = apply_filters( 'mbr/api', 'webhook_edit', $params );
							} WHILE ( $max_attempts-- AND $response->result_code <> 1 );
						ENDIF;

					ENDFOREACH;
				ENDIF;

				
				IF ( $wh_found === FALSE ) :
					
					$max_attempts = 9;
					DO {
						$response = $this->add_webhook();
					} WHILE ( $max_attempts-- AND $response->result_code <> 1 );
				ENDIF;

				return;
			ENDIF;

			IF ( $arr_settings['mbr_wh_webhooks_type'] == 'mbr_list' ) :
				
				IF ( ! empty( $arr_webhooks['local'] ) ) :
					$this->delete_webhooks( $arr_webhooks['local'] );
				ENDIF;

				
				$site_list_id         = $this->get_auto_generated_list_id();

				IF ( empty( $site_list_id ) ) :
					$response = apply_filters( 'mbr/api', 'list_add', [
						'name'            => $this->webhook_list_name,
						'sender_name'     => MBRWPURL,
						'sender_addr1'    => MBRWPURL,
						'sender_zip'      => MBRWPURL,
						'sender_city'     => MBRWPURL,
						'sender_country'  => MBRWPURL,
						'sender_url'      => MBRWPURL,
						'sender_reminder' => sprintf( _x( 'Automatically created list for webhooks by ActiveMember360 for %s', 'wp_admin', 'activemember360' ), MBRWPURL )
					] );
					IF ( $response->result_code ) :
						$site_list_id = $response->id;
					ENDIF;
				ENDIF;

				IF ( empty( $site_list_id ) ) :
					return;
				ENDIF;

				IF ( ! empty( $arr_webhooks['remote'] ) ) :

					$arr_master     = $this->check_for_master_webhook( $arr_webhooks['remote'] );

					$master_exists  = $arr_master['master_exists'];
					$make_to_master = $arr_master['make_to_master'];

					IF ( ! $master_exists ) :
						
						$this->make_master_webhook( $make_to_master );
					ENDIF;

				ENDIF;

				$this->add_webhook( $site_list_id );

				return;
			ENDIF;

			return;
		ENDIF;

		mbr_release_lock( __FUNCTION__ );
	}

	public function webhook_parser()
	{
		global $wpdb, $MBR_OPTIONS;
		
		IF ( ! isset( $_REQUEST['mbr'] ) OR $_REQUEST['mbr'] <> $this->webhook_name ) :
			return;
		ENDIF;

		IF ( ! isset($_REQUEST['appname'] ) OR $_REQUEST['appname'] <> $this->ac_account_info['account'] ) :
			return;
		ENDIF;

		IF ( ! in_array( $_POST['type'], [ 'subscribe', 'update' ]) OR ! isset( $_POST['contact']['id'] ) ) :
			return;
		ENDIF;

		$is_master_webhook   = FALSE;
		IF ( ! empty( $_GET['master'] ) OR ! empty( $_GET['update'] ) ) :
			$is_master_webhook = TRUE;
		ENDIF;

		$time_begin = microtime( TRUE );

		IF ( WEBHOOKLOG_LIMIT > 0 ) :
			$wpdb->insert( $wpdb->tbl_apilog, [ 'user' => 'AC Webhook', 'resptime' => microtime( TRUE ) - $time_begin, 'ipnr' => MBR_IPADDR, 'method' => 'incoming_webhook' . ( $is_master_webhook  ? ' [master]' : '' ), 'params' => gz_encode( $_GET ), 'post_data' => gz_encode( $_POST ), 'response' => '' ] );
		ENDIF;

		do_action( 'mbr/site_custom_fields/sync', $_POST['contact']['fields'] );
		$passfield_id = apply_filters( 'mbr/passfield/id', NULL );

		IF ( $is_master_webhook ) :
			
			IF ( empty( $_POST['contact']['fields'][ $passfield_id ] ) ) :
				$lock_name = md5( $_POST['contact']['email'] );
				IF ( mbr_get_lock( $lock_name ) ) :

					$tmpCONTACT = apply_filters( 'mbr/fetch_contact', [ 'id' => $_POST['contact']['id'], 'local' => FALSE, 'sync' => FALSE ], FALSE );
					IF ( empty( $tmpCONTACT[ apply_filters( 'mbr/passfield', NULL ) ] ) ) :
						
						$arrPWUPD                          = [];
						$arrPWUPD['email']                 = $_POST['contact']['email'];
						
						$user             = get_user_by( 'email', $_POST['contact']['email'] );
						$clear_password   = FALSE;
						IF ( ! empty( $user->user_pass ) ) :
							IF ( $MBR_OPTIONS['mbr_encrypted_passwords'] ) :
								$password     = $user->user_pass;
							ELSE:
								$clear_password = $password = apply_filters( 'mbr/password/generate', NULL );
							ENDIF;
						ELSE:
							$clear_password = apply_filters( 'mbr/password/generate', NULL );
							$password       = apply_filters( 'mbr/password/hash', $clear_password );
						ENDIF;

						$arrPWUPD[ apply_filters( 'mbr/password_perstag', NULL ) ] = $password;
						IF ( apply_filters( 'mbr/api', 'contact_sync', $arrPWUPD ) ) :
							IF ( $MBR_OPTIONS['mbr_encrypted_passwords'] ) :
								do_action( 'mbr/templates/email/login_credentials/send', $_POST['contact']['email'], $clear_password );
							ENDIF;
						ENDIF;
						mbr_release_lock( $lock_name );
					ENDIF;
				ENDIF;
			ENDIF;
		ENDIF;

		
		
		IF ( $contact = apply_filters( 'mbr/fetch_contact/local', [ 'contact_id', $_POST['contact']['id'], FALSE ] ) OR $contact = apply_filters( 'mbr/fetch_contact/local', [ 'email', $_POST['contact']['email'], FALSE ] ) ) :

			IF ( ! empty( $_POST['contact']['fields'][ $passfield_id ] ) ) :
				IF ( empty( $GLOBALS['mbr_passwords'] ) ) :
					$GLOBALS['mbr_passwords'] = mbr_passwords::getInstance();
				ENDIF;
				$GLOBALS['mbr_passwords']->site_password_encryption( $_POST['contact']['fields'][ $passfield_id ] );
			ENDIF;

			$site_cust_fields = apply_filters( 'mbr/site_custom_fields/get', NULL );

			
			FOREACH ( $_POST['contact'] AS $field => $value ) :

				IF ( $field == 'id' ) :
					continue;
				ENDIF;

				IF ( $field == 'fields' ) :

					
					FOREACH ( $site_cust_fields AS $k => $cust_field ) :
						FOREACH ( $_POST['contact']['fields'] AS $cust_field_id => $value ) :
							IF ( $cust_field['id'] == $cust_field_id ) :
								$contact[ lower_trim( $cust_field['perstag'] ) ] = $value;
							ENDIF;
						ENDFOREACH;
					ENDFOREACH;

				ELSEIF ( $field == 'tags' ) :

					
					$updated_tags = FALSE;
					$arr_tags     = [];
					$site_tags    = apply_filters( 'mbr/site_tags/get', NULL );

					IF ( isset( $_POST['contact']['tags'] ) ) :
						$arr_post_tags = array_map( 'trim', explode( ',', $_POST['contact']['tags'] ) );
						IF ( ! empty( $arr_post_tags ) AND is_array( $arr_post_tags ) ) :
							FOREACH ( $arr_post_tags AS $k => $tag_name ) :
								$tag_id = array_search( $tag_name, $site_tags );
								IF ( ! $tag_id ) :
									IF ( ! $updated_tags ) :
										
										do_action( 'mbr/site_tags/fetch' );
										$site_tags    = apply_filters( 'mbr/site_tags/get', NULL );

										$updated_tags = TRUE;
										$tag_id       = array_search( $tag_name, $site_tags );
									ENDIF;
								ENDIF;
								IF ( $tag_id ) :
									$arr_tags[]  = $tag_id;
								ENDIF;
							ENDFOREACH;

							
							$contact['tags']    = implode( ',', $arr_post_tags );
							$contact['tag_ids'] = implode( ',', $arr_tags );
						ELSE:
							$contact['tags']    = '';
							$contact['tag_ids'] = '';
						ENDIF;
					ENDIF;

				ELSE :
					
					$contact[ $field ] = $value;
				ENDIF;

			ENDFOREACH;
			$contact['hash'] = md5( $_POST['contact']['id'] . $_POST['contact']['email'] );

			IF ( $_POST['type'] == 'subscribe' ) :
				$contact['lists'][ $_POST['list'] ]['status'] = 1;
			ENDIF;

			$contact['name']    = $contact['first_name'] . ' ' . $contact['last_name'];

			$wpdb->query( $wpdb->prepare( "REPLACE INTO {$wpdb->tbl_contact} (contact_id, hash, email, data, last_active, last_updated) VALUES(%s,%s,%s,%s,UTC_TIMESTAMP(),UTC_TIMESTAMP())", $_POST['contact']['id'], $contact['hash'], $_POST['contact']['email'], gz_encode( $contact ) ) );
		ENDIF;

		exit;
	}

	public function init_admin_panel()
	{
		global $MBR_REDUX_HEADER_MODULE_NAMES;

		$MBR_REDUX_HEADER_MODULE_NAMES[ $this->opt_name ] = $admin_title = _x( 'Webhooks', 'wp_admin', 'activemember360' );

		$page_title         = apply_filters( 'mbr/plugin_name', '' ) . ' ' . $admin_title;

		$arr_config         =  [
			'opt_name'        => $this->opt_name,
			'global_variable' => $this->opt_name,
			'page_slug'       => $this->page_slug,
			'page_parent'     => MBR_MENU_PARENT_PAGE,
			'page_title'      => $page_title,
			'display_name'    => $page_title,
			'admin_title'     => $admin_title,
			'menu_title'      => $admin_title,
		];

		$arr_config         = array_merge( $this->arr_default_sub_admin_panel_settings, $arr_config );

		Redux::setArgs( $this->opt_name, $arr_config );
	}

	public function init_sections()
	{
		global $pagenow;

		IF ( $pagenow <> 'options.php' AND $pagenow <> 'admin.php' OR ( $pagenow == 'admin.php' AND ( empty($_GET['page'] ) OR $_GET['page'] <> $this->page_slug ) ) ) :
			return;
		ENDIF;

		do_action( 'mbr/site_lists/fetch', NULL );
		$arr_lists = apply_filters( 'mbr/site_lists/get', NULL );

		$arr_lists_options = [];
		FOREACH ( $arr_lists AS $list_id => $list_name ) :
			$arr_lists_options[ $list_id ]             = $list_name;
			$arr_lists_options[ 'master_' . $list_id ] = $list_name . ' (MASTER)';
		ENDFOREACH;

		$arr_section_webhooks =
		[
			'id'             => 'section_webhooks',
			'title'          => _x( 'Webhooks', 'wp_admin', 'activemember360' ),
			'desc'           => _x( 'This option activates ActiveMember360\'s automatic webhooks.', 'wp_admin', 'activemember360' ),
			'icon'           => 'el el-th-list',
			'fields'         => [
				[
					'id'         => 'mbr_wh_webhooks_enabled',
					'title'      => _x( 'Enable Automatic Webhooks', 'wp_admin', 'activemember360' ),
					'type'       => 'switch',
					'on'         => _x( 'Yes', 'wp_admin', 'activemember360' ),
					'off'        => _x( 'No',  'wp_admin', 'activemember360' ),
					'default'    => 1
				],
				[
					'id'         => 'mbr_wh_webhooks_type',
					'title'      => _x( 'Choose your webhook type', 'wp_admin', 'activemember360' ),
					'type'       => 'select',
					'options'    => [ 'default' => _x( 'Default (A Webhook triggered by all lists)', 'wp_admin', 'activemember360' ), 'mbr_list' => _x( 'MBR List (A Webhook triggered only for contacts by an auto-generated list of ActiveMember360)', 'wp_admin', 'activemember360' ) ],
					'default'    => 'default',
					'select2'    => [ 'allowClear' => FALSE ],
					'required'   => [ 'mbr_wh_webhooks_enabled', 'equals', '1' ]
				],
				[
					'id'         => 'mbr_wh_webhooks_site_list',
					'title'      => '<p style="text-align:center;">' . _x( 'Add all applicable contacts to the following list, which was created (or will be created when you save this settings panel) for you by ActiveMember360 in your ActiveCampaign account. <br />Please note: <strong>DO NOT DELETE</strong> this list under any circumstance as it may result in contact records being deleted from your ActiveCampaign account.', 'wp_admin', 'activemember360' ) . '</p>',
					'full_width' => TRUE,
					'type'       => 'raw',
					'content'    => '<p style="font-size:1.5em;text-align:center;">==> <strong>' . $this->webhook_list_name . '</strong> <==</p>',
					'required'   => [ [ 'mbr_wh_webhooks_enabled', 'equals', '1' ], [ 'mbr_wh_webhooks_type', 'equals', 'mbr_list' ] ]
				],









[
					'id'         => 'mbr_wh_webhooks_notes',
					'title'      => _x( 'Your notes related to this settings panel', 'wp_admin', 'activemember360' ),
					'type'       => 'textarea'
				]
			]
		];

		
		Redux::setSection( $this->opt_name, $arr_section_webhooks );
	}

}
$GLOBALS['mbr_inst_webhooks'] = mbr_webhooks::getInstance();