<?php
class mbr_widget_control
{

	use TRAIT__getInstance;

	


	private $arr_widget_conditions = [];

	


	private $arr_system_tags       = [];

	


	private $sidebars_widgets      = [];

	


	private $arr_conditionals      = [];

	public function _load()
	{
		add_action( 'wp',                       [ $this, 'set_widget_contexts_frontend' ], 10, 1 );
		add_action( 'admin_init',               [ $this, 'admin_init' ],                   10, 1 );
		add_action( 'in_widget_form',           [ $this, 'add_controls_to_widget_form' ],  10, 3 );
		add_action( 'sidebar_admin_setup',      [ $this, 'save_widget_conditions' ],       10, 0 );
		add_action( 'admin_enqueue_scripts',    [ $this, 'admin_enqueue_scripts' ],        10, 1 );
	}

	












	public function set_widget_contexts_frontend( &$wp ) {
		IF ( ! ( ! is_admin() OR defined( 'DOING_AJAX' ) ) AND ! defined( 'DOING_CRON' ) ) :
			return NULL;
		ENDIF;

		add_filter( 'sidebars_widgets',      [ $this, 'maybe_unset_widgets_by_conditions' ], -9999 );
	}

	










	public function admin_init()
	{
		global $pagenow;

		// Get tags only on widgets and customize view.
		IF ( ! in_array( $pagenow, [ 'widgets.php', 'customize.php', 'admin-ajax.php' ], TRUE ) ) :
			return NULL;
		ENDIF;

		$this->arr_widget_conditions = get_option( 'mbr_widget_conditions', [] );
		$this->arr_system_tags       = apply_filters( 'mbr/site_tags/get',  NULL );
		$this->arr_conditionals      = apply_filters( 'mbr/conditionals/conditionals/get', [] );
	}

	














	public function admin_enqueue_scripts( $hook_suffix = '')
	{
		global $pagenow;

		
		IF ( ! in_array( $pagenow, [ 'widgets.php' ], TRUE ) ) :
			return NULL;
		ENDIF;

		$locale       = str_replace( '_', '-', get_locale() );
		$locale_short = substr( $locale, 0, 2 );
		$locale       = file_exists( RWMB_DIR . "js/select2/i18n/$locale.js" ) ? $locale : $locale_short;

		wp_register_script( 'mbr-widget-control', plugins_url( 'js/widget-control.js', __FILE__ ), [ 'jquery' ], MBRVERSION, TRUE );
		wp_enqueue_script( 'mbr-widget-control' );

		wp_register_style( 'mbr-widget-control', plugins_url( 'css/widget-control.css', __FILE__ ), [], MBRVERSION, 'all' );
		wp_enqueue_style( 'mbr-widget-control' );

		wp_register_style( 'mbr-widget-control-select2', plugins_url( 'libs/meta-box/framework/css/select2/select2.css', __FILE__ ), [], MBRVERSION, 'all' );
		wp_enqueue_style( 'mbr-widget-control-select2' );

		wp_register_script( 'mbr-widget-control-select2', plugins_url( 'libs/meta-box/framework/js/select2/select2.min.js', __FILE__ ), [ 'jquery' ], MBRVERSION );
		wp_enqueue_script( 'mbr-widget-control-select2' );

		wp_register_script( 'mbr-widget-control-select2-i18n', plugins_url( 'libs/meta-box/framework/js/select2/i18n/' . $locale . '.js', __FILE__ ), [ 'jquery' ], MBRVERSION );
		wp_enqueue_script( 'mbr-widget-control-select2-i18n' );
	}

	














	public function add_controls_to_widget_form( $widget, $return, $instance )
	{
		$widget_control_settings                       = $this->get_widget_condition( $widget->id );

		IF ( empty( $widget_control_settings ) AND '__i__' <> $widget->number ) :
			$widget_control_settings                     = $this->convert_old_widget_settings( $widget->id );
		ENDIF;

		IF ( ! isset( $widget_control_settings['mode'] ) ) :
			$widget_control_settings['mode']             = 'individual';
		ENDIF;

		IF ( ! isset( $widget_control_settings['display_if'] ) ) :
			$widget_control_settings['display_if']       = 'always';
		ENDIF;

		IF ( ! isset( $widget_control_settings['pos_tags'] ) ) :
			$widget_control_settings['pos_tags']         = [];
		ENDIF;

		IF ( ! isset( $widget_control_settings['neg_tags'] ) ) :
			$widget_control_settings['neg_tags']         = [];
		ENDIF;

		IF ( ! isset( $widget_control_settings['php_statement'] ) ) :
			$widget_control_settings['php_statement']    = '';
		ENDIF;

		IF ( ! isset( $widget_control_settings['conditional_eval'] ) ) :
			$widget_control_settings['conditional_eval'] = 'true';
		ENDIF;

		IF ( ! isset( $widget_control_settings['conditional_id'] ) ) :
			$widget_control_settings['conditional_id']   = 0;
		ENDIF;

		IF ( ! isset( $widget_control_settings['internal_notes'] ) ) :
			$widget_control_settings['internal_notes']   = '';
		ENDIF;

		$conditional_mode_style   = '';
		$individual_mode_style    = '';
		IF ( 'individual' == $widget_control_settings['mode'] ) :
			$conditional_mode_style = "style='display:none;'";
		ELSE:
			$individual_mode_style  = "style='display:none;'";
		ENDIF;
		?>
		<div class="mbr-widget-control-conditions-container">
			<h4><?php echo sprintf( _x( '%s Widget Conditions:', 'widget-control', 'mbr' ), apply_filters( 'mbr/plugin_name', NULL ) ); ?></h4>
			<?php
				$status    = 'mbr-widget-active';
				
				IF ( '__i__' == $widget->number ) :
					$status  = 'mbr-widget-inactive';
				ENDIF;
			?>
			<div class="<?php echo $status; ?>">

				<p><?php echo _x( 'Control Mode:' , 'widget-control', 'mbr' ); ?><br />
					<select name="mbr_widget_control[<?php echo $widget->id; ?>][mode]" class="mbr-widget-control-mode" data-mbr-widget-control-mode-id="<?php echo $widget->id; ?>" onchange="mbr_control_mode_switcher(this)">
						<?php
						FOREACH ( [ 'individual' => _x( 'Individual', 'wp_admin', 'activemember360' ), 'conditional' => _x( 'Conditional', 'wp_admin', 'activemember360' ),  ] AS $value => $label ) :
							IF ( ! empty( $widget_control_settings['mode'] ) ) :
								$selected = selected( $widget_control_settings['mode'], $value, FALSE );
							ELSE:
								$selected = '';
							ENDIF;
							echo '<option value="' . $value .'" ' . $selected . '>' . $label . '</option>';
						ENDFOREACH;
						?>
					</select>
				</p>

				<div id="mbr-widget-control-individual-<?php echo $widget->id; ?>" <?php echo $individual_mode_style; ?>>

					<p><?php echo _x( 'Display when:' , 'widget-control', 'mbr' ); ?><br />
						<select name="mbr_widget_control[<?php echo $widget->id; ?>][display_if]">
							<?php
							FOREACH ( [ 'always' => _x( 'Always', 'widget-control', 'mbr' ), 'is_logged_in' => _x( 'Is logged in', 'widget-control', 'mbr' ), 'not_logged_in' => _x ( 'Not logged in', 'widget-control', 'mbr' ), 'never' => _x( 'Never', 'widget-control', 'mbr' ) ] AS $value => $label ) :
								IF ( ! empty( $widget_control_settings['display_if'] ) ) :
									$selected = selected( $widget_control_settings['display_if'], $value, FALSE );
								ELSE:
									$selected = '';
								ENDIF;
								echo '<option value="' . $value .'" ' . $selected . '>' . $label . '</option>';
							ENDFOREACH;
							?>
						</select>
					</p>

					<p><?php echo _x( 'AND if  the visitor has ANY of the following tags:' , 'widget-control', 'mbr' ); ?><br />
						<select name="mbr_widget_control[<?php echo $widget->id; ?>][pos_tags][]" multiple="multiple">
							<?php
							IF ( ! empty( $this->arr_system_tags ) AND is_array( $this->arr_system_tags ) ) :
								FOREACH ( $this->arr_system_tags AS $tag_id => $tag_name ) :
									IF ( ! empty( $widget_control_settings['pos_tags'] ) AND is_array( $widget_control_settings['pos_tags'] ) ) :
										$selected = in_array( $tag_id, $widget_control_settings['pos_tags'] ) ? 'selected="selected"' : '';
									ELSE:
										$selected = '';
									ENDIF;
									echo '<option value="' . $tag_id .'" ' . $selected . '>#' . $tag_id . ' - ' . $tag_name . '</option>';
								ENDFOREACH;
							ENDIF;
							?>
						</select>
					</p>

					<p><?php echo _x( 'AND if the visitor has NONE of the following tags:' , 'widget-control', 'mbr' ); ?><br />
						<select name="mbr_widget_control[<?php echo $widget->id; ?>][neg_tags][]" multiple="multiple">
							<?php
							IF ( ! empty( $this->arr_system_tags ) AND is_array( $this->arr_system_tags ) ) :
								FOREACH ( $this->arr_system_tags AS $tag_id => $tag_name ) :
									IF ( ! empty( $widget_control_settings['neg_tags'] ) AND is_array( $widget_control_settings['neg_tags'] ) ) :
										$selected = in_array( $tag_id, $widget_control_settings['neg_tags'] ) ? 'selected="selected"' : '';
									ELSE:
										$selected = '';
									ENDIF;
									echo '<option value="' . $tag_id .'" ' . $selected . '>#' . $tag_id . ' - ' . $tag_name . '</option>';
								ENDFOREACH;
							ENDIF;
							?>
						</select>
					</p>

					<p><?php echo _x( 'AND the following PHP condition is true:' , 'widget-control', 'mbr' ); ?><br />
						<?php
						$php_statement    = '';
						IF ( ! empty( $widget_control_settings['php_statement'] ) ) :
							$php_statement  = stripcslashes( esc_textarea( $widget_control_settings['php_statement'] ) );
						ENDIF;
						?>
						<textarea class="widefat" name="mbr_widget_control[<?php echo $widget->id; ?>][php_statement]" ><?php echo $php_statement; ?></textarea>
					</p>

				</div>

				<div id="mbr-widget-control-conditional-<?php echo $widget->id; ?>" <?php echo $conditional_mode_style; ?>>
					<p><?php echo _x( 'Display if conditional evaluates:' , 'wp_admin', 'activemember360' ); ?><br />
						<select name="mbr_widget_control[<?php echo $widget->id; ?>][conditional_eval]">
							<?php
							FOREACH ( [ 'true' => _x( 'True', 'wp_admin', 'activemember360' ), 'false' => _x( 'False', 'wp_admin', 'activemember360' ),  ] AS $value => $label ) :
								echo '<option value="' . $value .'" ' . selected( $widget_control_settings['conditional_eval'], $value, FALSE ) . '>' . $label . '</option>';
							ENDFOREACH;
							?>
						</select>
					</p>

					<p><?php echo _x( 'Conditional:' , 'widget-control', 'mbr' ); ?><br />
						<select name="mbr_widget_control[<?php echo $widget->id; ?>][conditional_id]" multiple="multiple">
							<?php
							IF ( ! empty( $this->arr_conditionals ) AND is_array( $this->arr_conditionals ) ) :
								FOREACH ( $this->arr_conditionals AS $conditional_id => $conditional_name ) :
									echo '<option value="' . $conditional_id . '" ' . selected( $widget_control_settings['conditional_id'], $conditional_id, false ) . '>' . $conditional_name . '</option>';
								ENDFOREACH;
							ELSE:
								echo '<strong>' . _x( 'No conditionals defined yet.', 'wp_admin', 'activemember360' ) .'</strong>';
							ENDIF;
							?>
						</select>
					</p>

				</div>

				<p><?php echo _x( 'Internal notes:' , 'widget-control', 'mbr' ); ?><br />
					<?php
					$internal_notes   = '';
					IF ( ! empty( $widget_control_settings['internal_notes'] ) ) :
						$internal_notes = stripcslashes( esc_textarea( $widget_control_settings['internal_notes'] ) );
					ENDIF;
					?>
					<textarea class="widefat" name="mbr_widget_control[<?php echo $widget->id; ?>][internal_notes]" ><?php echo $internal_notes; ?></textarea>
				</p>
			</div>
		</div>
	<?php
	}

	











	public function save_widget_conditions()
	{
		IF ( ! current_user_can( 'edit_theme_options' ) OR empty( $_POST ) OR ! isset( $_POST['mbr_widget_control'] ) ) :
			return NULL;
		ENDIF;

		
		IF ( isset( $_POST['delete_widget'] ) AND isset( $_POST['widget-id'] ) ) :
			unset( $_POST['mbr_widget_control'][ $_POST['widget-id'] ] );
			unset( $this->arr_widget_conditions[ $_POST['widget-id'] ] );
		ENDIF;

		
		$this->arr_widget_conditions = array_merge( $this->arr_widget_conditions, $_POST['mbr_widget_control'] );

		$sidebars_widgets            = wp_get_sidebars_widgets();
		$all_widget_ids              = [];

		
		IF ( ! empty( $sidebars_widgets ) AND is_array( $sidebars_widgets ) ) :
			FOREACH ( $sidebars_widgets AS $widget_area => $widgets ) :
				FOREACH ( $widgets AS $widget_order => $widget_id ) :
					$all_widget_ids[]      = $widget_id;
				ENDFOREACH;
			ENDFOREACH;
		ENDIF;

		
		IF ( ! empty( $this->arr_widget_conditions ) AND is_array( $this->arr_widget_conditions ) ) :
			FOREACH ( $this->arr_widget_conditions AS $widget_id => $widget_condition ) :
				IF ( ! in_array( $widget_id, $all_widget_ids ) ) :
					unset( $this->arr_widget_conditions[ $widget_id ] );
				ENDIF;
			ENDFOREACH;
		ENDIF;

		update_option( 'mbr_widget_conditions', $this->arr_widget_conditions );
	}

	










	private function get_widget_condition( $widget_id = FALSE )
	{
		IF ( isset( $this->arr_widget_conditions[ $widget_id ] ) ) :
			return $this->arr_widget_conditions[ $widget_id ];
		ENDIF;

		return [];
	}

	










	public function maybe_unset_widgets_by_conditions( $sidebars_widgets )
	{

		
		IF ( is_admin() OR MBRADMIN ) :
			return $sidebars_widgets;
		ENDIF;

		
		IF ( ! empty( $this->sidebars_widgets ) ) :
			return $this->sidebars_widgets;
		ENDIF;

		$this->arr_widget_conditions = get_option( 'mbr_widget_conditions', [] );
		$this->arr_system_tags       = apply_filters( 'mbr/site_tags/get',  NULL );

		IF ( ! empty( $sidebars_widgets ) AND is_array( $sidebars_widgets ) ) :
			FOREACH ( $sidebars_widgets as $widget_area => $widget_list ) :

				IF ( $widget_area == 'wp_inactive_widgets' OR empty( $widget_list ) ) :
					continue;
				ENDIF;

				FOREACH ( $widget_list AS $pos => $widget_id ) :
					IF ( ! $this->check_widget_conditions( $widget_id ) ) :
						unset( $sidebars_widgets[ $widget_area ][ $pos ] );
					ENDIF;
				ENDFOREACH;

			ENDFOREACH;
		ENDIF;

		
		$this->sidebars_widgets = $sidebars_widgets;

		return $sidebars_widgets;
	}

	








	private function check_widget_conditions( $widget_id )
	{
		IF ( empty( $this->arr_widget_conditions[ $widget_id ] ) OR ! is_array( $this->arr_widget_conditions[ $widget_id ] ) ) :
			$this->arr_widget_conditions[ $widget_id ] = $this->convert_old_widget_settings( $widget_id );
		ENDIF;

		IF ( ! empty( $this->arr_widget_conditions[ $widget_id ] ) AND is_array( $this->arr_widget_conditions[ $widget_id ] ) ) :

			IF ( 'conditional' == $this->arr_widget_conditions[ $widget_id ]['mode'] ) :
				$conditional_eval     = $this->arr_widget_conditions[ $widget_id ]['conditional_eval'];
				$conditional_id       = $this->arr_widget_conditions[ $widget_id ]['conditional_id'];
				IF ( ! empty( $conditional_id ) AND ! empty( $conditional_eval ) ) :
					$match = apply_filters( 'mbr/conditionals/conditional/check', NULL, $conditional_id );
					IF ( 'true' == $conditional_eval AND FALSE === $match ) :
						return FALSE;
					ELSEIF ( 'false' == $conditional_eval AND FALSE !== $match ) :
						return FALSE;
					ENDIF;
				ENDIF;
			ELSE:
				IF ( 'always' <> $this->arr_widget_conditions[ $widget_id ]['display_if'] ) :
					IF ( 'never' == $this->arr_widget_conditions[ $widget_id ]['display_if'] ) :
						return FALSE;
					ENDIF;
					IF ( 'is_logged_in' == $this->arr_widget_conditions[ $widget_id ]['display_if'] AND ! is_user_logged_in() ) :
						return FALSE;
					ENDIF;
					IF ( 'not_logged_in' == $this->arr_widget_conditions[ $widget_id ]['display_if'] AND is_user_logged_in() ) :
						return FALSE;
					ENDIF;
				ENDIF;

				$php_statement   = '';

				$pos_tags        = array_filter( $this->arr_widget_conditions[ $widget_id ]['pos_tags'] );
				$neg_tags        = array_filter( $this->arr_widget_conditions[ $widget_id ]['neg_tags'] );

				IF ( isset( $this->arr_widget_conditions[ $widget_id ]['php_statement'] ) ) :
					$php_statement = trim( $this->arr_widget_conditions[ $widget_id ]['php_statement'] );
				ENDIF;
				IF ( empty( $pos_tags ) AND empty( $neg_tags ) AND empty( $php_statement ) ) :
					return TRUE;
				ENDIF;

				IF ( ! empty( $pos_tags ) AND is_array( $pos_tags ) ) :
					IF ( ! mbr_has_tags( $pos_tags ) ) :
						return FALSE;
					ENDIF;
				ENDIF;

				IF ( ! empty( $neg_tags ) AND is_array( $neg_tags ) ) :
					IF ( mbr_has_tags( $neg_tags ) ) :
						return FALSE;
					ENDIF;
				ENDIF;

				IF ( empty( $php_statement ) ) :
					$php_statement = "return true;";
				ELSE:
					$php_statement = ( stristr( $php_statement,  'return' ) ) ? $php_statement : 'return (' . $php_statement . ');';
					$php_statement = stripcslashes( $php_statement );
				ENDIF;

				IF ( eval( $php_statement ) ) :
					return TRUE;
				ENDIF;

				return FALSE;
			ENDIF;

		ENDIF;

		return TRUE;
	}

	








	private function convert_old_widget_settings( $widget_id )
	{
		$widgetcontrol_options                                 = $this->get_widget_options();
		$widget_control_settings                               = [];
		IF ( ! empty( $widgetcontrol_options[ $widget_id ] ) ) :
			FOREACH ($widgetcontrol_options[ $widget_id ] AS $k => $str) :
				$widgetcontrol_options[ $widget_id ][ $k ]         = stripcslashes( $widgetcontrol_options[ $widget_id ][ $k ] );
				$widgetcontrol_options[ $widget_id ][ $k ]         = addslashes( $widgetcontrol_options[ $widget_id ][ $k ] );
			ENDFOREACH;
			IF ( ! empty( $widgetcontrol_options[ $widget_id ] ) AND is_array( $widgetcontrol_options[ $widget_id ] ) ) :
				$widget_control_settings_tmp                       = [];
				IF ( isset( $widgetcontrol_options[ $widget_id ]['mode'] ) ) :
					$widget_control_settings_tmp['mode']             = $widgetcontrol_options[ $widget_id ]['mode'];
				ENDIF;

				IF ( isset( $widgetcontrol_options[ $widget_id ]['loggedin'] ) ) :
					IF ( empty( $widgetcontrol_options[ $widget_id ]['loggedin'] ) OR 0 == (int) $widgetcontrol_options[ $widget_id ]['loggedin'] ):
						$widget_control_settings_tmp['display_if']     = 'always';
					ELSEIF ( 1 == (int) $widgetcontrol_options[ $widget_id ]['loggedin'] ) :
						$widget_control_settings_tmp['display_if']     = 'is_logged_in';
					ELSEIF ( 2 == (int) $widgetcontrol_options[ $widget_id ]['loggedin'] ) :
						$widget_control_settings_tmp['display_if']     = 'not_logged_in';
					ELSEIF ( 3 == (int) $widgetcontrol_options[ $widget_id ]['loggedin'] ) :
						$widget_control_settings_tmp['display_if']     = 'never';
					ENDIF;
				ENDIF;

				IF ( isset( $widgetcontrol_options[ $widget_id ]['incltags'] ) ) :
					$widget_control_settings_tmp['pos_tags']         = explode( ',', $widgetcontrol_options[ $widget_id ]['incltags'] );
				ENDIF;

				IF ( isset( $widgetcontrol_options[ $widget_id ]['excltags'] ) ) :
					$widget_control_settings_tmp['neg_tags']         = explode( ',', $widgetcontrol_options[ $widget_id ]['excltags'] );
				ENDIF;

				IF ( isset( $widgetcontrol_options[ $widget_id ]['condtrue'] ) ) :
					$widget_control_settings_tmp['php_statement']    = $widgetcontrol_options[ $widget_id ]['condtrue'];
				ENDIF;

				IF ( isset( $widgetcontrol_options[ $widget_id ]['conditional_eval'] ) ) :
					$widget_control_settings_tmp['conditional_eval'] = $widgetcontrol_options[ $widget_id ]['conditional_eval'];
				ENDIF;

				IF ( isset( $widgetcontrol_options[ $widget_id ]['conditional_id'] ) ) :
					$widget_control_settings_tmp['conditional_id']   = $widgetcontrol_options[ $widget_id ]['conditional_id'];
				ENDIF;

				$widget_control_settings                           = $widget_control_settings_tmp;

				
				unset( $widgetcontrol_options[ $widget_id ] );
				update_option( 'mbr_widget_control', $widgetcontrol_options );

				
				$this->arr_widget_conditions[ $widget_id ]         = $widget_control_settings;
				update_option( 'mbr_widget_conditions', $this->arr_widget_conditions );
			ENDIF;
		ENDIF;

		return $widget_control_settings;
	}

	






	private function get_widget_options()
	{
		IF ( $widgetcontrol_options = get_option( 'mbr_widget_control' ) AND is_array( $widgetcontrol_options ) ) :
			return $widgetcontrol_options;
		ELSE :
			return [];
		ENDIF;
	}

}

$GLOBALS['mbrwidgetcontrol'] = mbr_widget_control::getInstance();
