<?php
class mbr_woocommerce
{
	use TRAIT__getInstance;

	public $woocommerce_billing_fields  = [
		'billing_email',
		'billing_first_name',
		'billing_last_name',
		'billing_company',
		'billing_address_1',
		'billing_address_2',
		'billing_city',
		'billing_state',
		'billing_postcode',
		'billing_country',
		'billing_phone'
	];

	public function _load()
	{
		global $pagenow;

		add_action( 'init',                      [ $this, '_init' ], 11 );
		add_filter( 'mbr/cust_fields/required',  [ $this, 'required_fields_billing' ], 10, 1 );

		
		IF ( is_admin() ) :
			IF ( in_array( $pagenow, [ 'post.php', 'post-new.php' ] ) ) :
				add_action( "load-{$pagenow}",         [ $this, 'init_metaboxes' ], 0 );
			ENDIF;
			add_action( 'mbr/quick_edit/protection/box', [ $this, 'init_quick_edit' ], 0 );
		ENDIF;
	}

	public function plugins_loaded()
	{
		IF ( ! class_exists( 'WooCommerce' ) ) :
			return;
		ENDIF;

		add_filter( 'mbr/protectable_post_types/built_in', [ $this, 'add_protectable_post_types' ], 10, 1 );

		$this->wc_custom_price_key = 'wc_mbr_custom_price';
		update_option( 'woocommerce_registration_generate_password', 'yes' );

		
		$this->woo_product_category_count = [];

		











		












		











		
		$this->wc_ext_options_key  = 'woocommerce_mbr_wc_admin_settings';
		$this->wc_ext_options      = get_option( $this->wc_ext_options_key, NULL );

		$update_options = FALSE;
		
		IF ( empty( $this->wc_ext_options['add_update_on_processing'] ) ) :
			unset( $this->wc_ext_options['add_update_on_processing'] );
			$update_options = TRUE;
		ENDIF;

		
		IF ( empty( $this->wc_ext_options['update_on_order_status_changed'] ) ) :
			$this->wc_ext_options['update_on_order_status_changed'] = 'yes';
			$update_options                                         = TRUE;
		ENDIF;
		IF ( empty( $this->wc_ext_options['enable_tags_in_panels'] ) ) :
			$this->wc_ext_options['enable_tags_in_panels'] = 'yes';
			$update_options                                = TRUE;
		ENDIF;
		IF ( empty( $this->wc_ext_options['enable_automations_in_panels'] ) ) :
			$this->wc_ext_options['enable_automations_in_panels'] = 'yes';
			$update_options                                       = TRUE;
		ENDIF;
		IF ( empty( $this->wc_ext_options['enable_lists_in_panels'] ) ) :
			$this->wc_ext_options['enable_lists_in_panels'] = 'yes';
			$update_options                                 = TRUE;
		ENDIF;
		IF ( $update_options ) :
			update_option( $this->wc_ext_options_key, $this->wc_ext_options );
		ENDIF;

		$this->wc_ext_valid_status = wc_get_order_statuses();

		$this->product_data_key = '_mbr_woocommerce_product_data';
		$this->coupon_data_key  = '_mbr_woocommerce_coupon_data';

		
		$this->product_settings_index = [
			'sku_tag_name'                            => '',
			'product_tag_names'                       => '',
			'product_cat_names'                       => '',
			'abandoned_cart_tags'                     => [],

			'assign_tags'                             => [],
			'remove_tags'                             => [],
			'add_to_lists'                            => [],
			'remove_from_lists'                       => [],
			'add_to_automations'                      => [],
			'remove_from_automations'                 => [],

			'assign_tags_refunded'                    => [],
			'remove_tags_refunded'                    => [],
			'add_to_lists_refunded'                   => [],
			'remove_from_lists_refunded'              => [],
			'add_to_automations_refunded'             => [],
			'remove_from_automations_refunded'        => [],

			'add_tags_sub_active'                     => [],
			'remove_tags_sub_active'                  => [],
			'add_to_lists_sub_active'                 => [],
			'remove_from_lists_sub_active'            => [],
			'add_to_automations_sub_active'           => [],
			'remove_from_automations_sub_active'      => [],

			'add_tags_sub_cancelled'                  => [],
			'remove_tags_sub_cancelled'               => [],
			'add_to_lists_sub_cancelled'              => [],
			'remove_from_lists_sub_cancelled'         => [],
			'add_to_automations_sub_cancelled'        => [],
			'remove_from_automations_sub_cancelled'   => [],

			'add_tags_sub_onhold'                     => [],
			'remove_tags_sub_onhold'                  => [],
			'add_to_lists_sub_onhold'                 => [],
			'remove_from_lists_sub_onhold'            => [],
			'add_to_automations_sub_onhold'           => [],
			'remove_from_automations_sub_onhold'      => [],

			'add_tags_sub_reactivated'                => [],
			'remove_tags_sub_reactivated'             => [],
			'add_to_lists_sub_reactivated'            => [],
			'remove_from_lists_sub_reactivated'       => [],
			'add_to_automations_sub_reactivated'      => [],
			'remove_from_automations_sub_reactivated' => [],

			'add_tags_sub_expired'                    => [],
			'remove_tags_sub_expired'                 => [],
			'add_to_lists_sub_expired'                => [],
			'remove_from_lists_sub_expired'           => [],
			'add_to_automations_sub_expired'          => [],
			'remove_from_automations_sub_expired'     => [],

			'add_tags_sub_refunded'                   => [],
			'remove_tags_sub_refunded'                => [],
			'add_to_lists_sub_refunded'               => [],
			'remove_from_lists_sub_refunded'          => [],
			'add_to_automations_sub_refunded'         => [],
			'remove_from_automations_sub_refunded'    => []
		];

		IF ( ! empty( $this->wc_ext_options['number_alt_prices'] ) ) :
			FOR ( $x = 1; $x <= $this->wc_ext_options['number_alt_prices']; $x++ ) :
				$this->product_settings_index['alt_price'][ $x ]  = '';
				$this->product_settings_index['alt_tags'][ $x ]   = [];
			ENDFOR;
		ENDIF;

		
		$this->coupon_settings_index = [
			'coupon_tag_name'                       => '',
			'assign_tags'                           => [],
			'remove_tags'                           => [],
			'add_to_lists'                          => [],
			'remove_from_lists'                     => [],
			'add_to_automations'                    => [],
			'remove_from_automations'               => []
		];

		IF ( is_admin() AND class_exists( 'WC_Integration' ) ) :
			require_once( __DIR__ . '/class.woocommerceintegration.php' );
			add_filter( 'woocommerce_integrations',  [ $this, 'woocommerce_integrations' ] );
		ENDIF;
	}

	public function _init()
	{
		IF ( ! class_exists( 'WooCommerce' ) ) :
			return;
		ENDIF;

		IF ( is_admin() ) :
			
			add_filter( 'woocommerce_product_data_tabs',                  [ $this, 'woocommerce_product_data_tabs' ]);
			add_action( 'woocommerce_product_data_panels',                [ $this, 'woocommerce_product_data_panels' ]);
			
			add_filter( 'woocommerce_coupon_data_tabs',                   [ $this, 'woocommerce_coupon_data_tabs' ]);
			add_action( 'woocommerce_coupon_data_panels',                 [ $this, 'woocommerce_coupon_data_panels' ]);
			
			add_action( 'woocommerce_process_product_meta',               [ $this, 'woocommerce_process_product_meta' ], 999);
			
			add_action( 'woocommerce_coupon_options_save',                [ $this, 'woocommerce_coupon_options_save' ]);
			
			add_action( 'woocommerce_product_options_pricing',            [ $this, 'woocommerce_product_options_pricing' ]);

			
			add_action( 'woocommerce_product_after_variable_attributes',  [ $this, 'woocommerce_product_after_variable_attributes' ], 10, 3);
			add_action( 'woocommerce_update_product_variation',           [ $this, 'woocommerce_update_product_variation' ], 10, 1);
		ENDIF;

		IF ( ! is_admin() ) :
			add_filter( 'woocommerce_add_cart_item',                      [ $this, 'add_custom_price' ] );
			add_filter( 'woocommerce_get_cart_item_from_session',         [ $this, 'set_session_prices' ], 20 , 3 );
			add_filter( 'woocommerce_get_price_html',                     [ $this, 'woocommerce_get_price_html' ], 100, 2 );

			
			add_action( 'woocommerce_add_to_cart',                        [ $this, 'woocommerce_add_to_cart' ],       10, 6 );
			add_action( 'woocommerce_cart_item_removed',                  [ $this, 'woocommerce_cart_item_removed' ], 10, 2 );
			
			add_action('woocommerce_before_cart_item_quantity_zero',      [ $this, 'woocommerce_before_cart_item_quantity_zero' ], 10, 1 );
			
			add_action( 'woocommerce_before_checkout_form',               [ $this, 'add_abandoned_cart_tags_before_checkout' ],10, 1 );
			
			add_action( 'woocommerce_payment_complete',                   [ $this, 'remove_abandoned_cart_tags_after_checkout' ], 10, 1 );
			add_action( 'woocommerce_order_status_processing',            [ $this, 'remove_abandoned_cart_tags_after_checkout' ], 10, 1 );
			add_action( 'woocommerce_order_status_completed',             [ $this, 'remove_abandoned_cart_tags_after_checkout' ], 10, 1 );
			
			
			add_action( 'woocommerce_order_status_on-hold',               [ $this, 'remove_abandoned_cart_tags_after_checkout' ], 10, 1 );
			add_action( 'woocommerce_order_status_failed',                [ $this, 'remove_abandoned_cart_tags_after_checkout' ], 10, 1 );

			add_filter( 'woocommerce_checkout_get_value',                 [ $this, 'woocommerce_checkout_get_value' ], 10,2 );
			add_filter( 'woocommerce_registration_auth_new_customer',     [ $this, 'woocommerce_registration_auth_new_customer' ], 10, 2 );
			add_filter( 'woocommerce_lost_password_message',              [ $this, 'woocommerce_lost_password_message' ] );
			add_filter( 'woocommerce_add_message',                        [ $this, 'woocommerce_add_message' ] );
			add_filter( 'attribute_escape',                               [ $this, 'change_reset_password_button' ] );

			
			add_filter( 'get_terms',                                       [ $this, 'filter_empty_product_categories' ], 10, 4 );
		ENDIF;

		
		add_action( 'woocommerce_order_status_changed',                 [ $this, 'woocommerce_order_status_changed' ], 10, 3 );

		
		add_action( 'woocommerce_subscription_status_updated',          [ $this, 'woocommerce_subscription_status_updated' ], 10, 3 );
		
		

		
		

		
		add_filter( 'woocommerce_checkout_order_processed',             [ $this, 'woocommerce_checkout_order_processed'], 99, 2 );

		add_filter( 'woocommerce_created_customer',                     [ $this, 'woocommerce_created_customer'],         10, 3 );

		add_filter( 'woocommerce_registration_errors',                  [ $this, 'woocommerce_registration_errors_validate_password_base'], 10, 3 );
		add_action( 'woocommerce_after_checkout_validation',            [ $this, 'woocommerce_after_checkout_validation_validate_password_base'], 10, 4 );

		
		add_filter( 'woocommerce_new_customer_data',                    [ $this, 'woocommerce_new_customer_data'], 99, 1 );

		
		add_action( 'woocommerce_save_account_details',                 [ $this, 'woocommerce_save_account_details'], 10, 1 );
		add_action( 'woocommerce_save_account_details_errors',          [ $this, 'woocommerce_save_account_details_errors'], 10, 2 );
		add_filter( 'check_password',                                   [ $this, 'check_password'], 10, 2 );

		
		add_action( 'retrieve_password',                                [ $this, 'lostpassword_post_form_process' ], 1, 2 );

		add_filter( 'woocommerce_add_to_cart_product_id',               [ $this, 'woocommerce_add_to_cart_product_id' ], 10, 1 );
		add_filter( 'woocommerce_subcategory_count_html',               [ $this, 'woocommerce_subcategory_count_html' ], 0, 2 );

		add_action( 'wp_loaded',                                        [ $this, 'multi_add_to_cart_action'], 20 );

		







		add_filter( 'mbr/shortcodes/extend',                            [ $this, 'extend_woocommerce_shortcodes' ], 0, 1 );
		add_filter( 'woocommerce_mail_content',                         [ $this, 'woocommerce_mail_content' ], 9999, 1 );
	}

	public function add_protectable_post_types( $arr_protectable_post_types = [] )
	{
		array_push( $arr_protectable_post_types, 'product' );
		return $arr_protectable_post_types;
	}

	public function woocommerce_mail_content( $message )
	{
		IF ( strpos( $message, '$P$B' ) !== FALSE ) :
			global $wp_hasher;

			IF ( empty( $wp_hasher->itoa64 ) ) :
				require_once( ABSPATH . WPINC . '/class-phpass.php' );
				$wp_hasher         = new PasswordHash( 8, TRUE );
			ENDIF;

			$passhash_string     = $wp_hasher->itoa64;

			IF ( empty( $wp_hasher->itoa64 ) ) :
				$passhash_string   = '\.\/0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
			ELSE:
				$passhash_string   = str_replace( [ '.', '/' ], [ '\.', '\/' ], $passhash_string );
			ENDIF;

			IF ( preg_match( '/(\$P\$B[' . $passhash_string . ']*)/', $message, $matches ) ) :
				IF ( ! empty( $matches[0] ) ) :
					$message         = str_replace( $matches[0], '', $message );
					$text_to_replace = sprintf( __( 'Your password has been automatically generated: %s', 'woocommerce'), '<strong></strong>' );
					$regex_start     = addslashes( substr( $text_to_replace,0,8 ) );
					$regex_end       = addslashes( substr( $text_to_replace, -4 ) );
					$pattern         = $regex_start . '.*' . $regex_end;
					IF ( preg_match( '/' . $pattern . '/s', $message, $matches1 ) ) :
						IF ( ! empty( $matches1[0] ) ) :
							$message     = str_replace( $matches1[0], __( 'Please use your current password to login.', 'activemember360' ), $message );
						ENDIF;
					ENDIF;
				ENDIF;
			ENDIF;

		ENDIF;

		return $message;
	}

	public function extend_woocommerce_shortcodes( $SHORT = [] )
	{
		$SHORT['quick_checkout'] = [ [ $this, 'sc_quick_checkout' ],  0  ];

		return $SHORT;
	}

	function get_product_ids()
	{
		$products_ids = new WP_Query( [ 'post_type' => 'product', 'post_status' => 'publish', 'fields' => 'ids' ] );
		return $products_ids->posts;
	}

	
	public function multi_add_to_cart_action()
	{
		IF ( defined( 'DOING_AJAX' ) AND DOING_AJAX ) :
			return;
		ENDIF;

		IF ( empty( $_REQUEST['madd-to-cart'] ) ) :
			return;
		ELSEIF ( ! $products = explode( ',', $_REQUEST['madd-to-cart'] ) ) :
			wp_safe_redirect( wc_get_cart_url() );
			exit;
		ELSEIF ( ! is_array( $products ) ) :
			wp_safe_redirect( wc_get_cart_url() );
			exit;
		ENDIF;

		$added_to_cart_count = 0;

		FOREACH ( $products AS $k => $product ) :
			DO {
				IF ( empty( $product ) ) :
					break;
				ENDIF;

				IF ( stripos($product, ':') !== FALSE ) :
					list( $product_id, $product_qty ) = explode( ':', $product );
					IF ( empty( $product_id ) ) :
						break;
					ENDIF;
					$product_id = absint( $product );
					IF ( empty( $product_qty ) OR absint( $product_qty ) < 1 ) :
						$product_qty = 1;
					ENDIF;
				ELSE :
					$product_id  = absint( $product );
					IF ( empty( $product_id ) ) :
						break;
					ENDIF;
					$product_qty = 1;
				ENDIF;

				IF ( ! apply_filters( 'mbr/match_user_content', $product_id ) OR ! WC()->cart->add_to_cart( $product_id, $product_qty ) ) :
					wc_add_notice( sprintf( __( 'Product ID %s could not be added to the cart', 'activemember360'), $product_id ), 'error' );
					break;
				ENDIF;

				$added_to_cart_count++;
			} WHILE ( FALSE );
		ENDFOREACH;

		IF ( $added_to_cart_count > 0 ) :
			wc_add_notice( sprintf( __( 'Added %d item(s) to the cart', 'activemember360' ), $added_to_cart_count ), 'notice' );
			wp_safe_redirect( wc_get_checkout_url() );
			exit;
		ENDIF;

		wc_add_notice( sprintf( __( 'No items could be added to the cart: %s', 'activemember360' ), $added_to_cart_count ), 'error' );
		wp_safe_redirect( wc_get_cart_url() );

		exit;
	}

	public function change_reset_password_button( $safe_text )
	{
		IF ( basename( apply_filters( 'mbr/url', NULL ) ) == get_option( 'woocommerce_myaccount_lost_password_endpoint', 'lost-password' ) ) :
			IF ( $safe_text == __( 'Reset Password', 'woocommerce' ) ) :
				return __( 'Send Password' , 'activemember360' );
			ENDIF;
		ENDIF;

		return $safe_text;
	}

	public function woocommerce_lost_password_message( $org_message )
	{
		global $MBR_OPTIONS;

		IF ( $MBR_OPTIONS['mbr_encrypted_passwords'] ) :
			return $org_message;
		ENDIF;

		IF ( empty( $_GET['reset-link-sent'] ) ) :
			return __( 'Lost your password? Enter your email address below to receive your password via email.', 'activemember360' );
		ENDIF;

		return __( 'Your login credentials have been sent to the email address on file for your account, but may take several minutes to appear in your inbox. Please wait at least 10 minutes before attempting another reset.', 'activemember360' );
	}

	public function woocommerce_add_message( $message )
	{
		global $MBR_OPTIONS;

		IF ( $MBR_OPTIONS['mbr_encrypted_passwords'] ) :
			return $message;
		ENDIF;

		IF ( $message == __('Password reset email has been sent.', 'woocommerce') ) :
			return  __( 'An email with your login credentials has been sent to you. Please check your email.', 'activemember360' );
		ENDIF;

		return $message;
	}

	function retrieve_password($user_login, $key)
	{
		do_action( 'mbr/password/lostpassword_post_form_process', $user_login );
	}

	public function required_fields_billing( $cust_fields = [] )
	{
		FOREACH ( $this->woocommerce_billing_fields AS $key => $field_name ) :
			$cust_fields[] = $field_name;
		ENDFOREACH;

		return $cust_fields;
	}

	
	public function woocommerce_product_data_tabs( $data_tabs )
	{
		$data_tabs['activemember360'] =
		[
			'label'  => 'ActiveMember360',
			'target' => 'activemember360_settings'
		];

		return $data_tabs;
	}

	public function woocommerce_product_data_panels()
	{
		global $thepostid, $post;
		$post_id = empty( $thepostid ) ? $post->ID : $thepostid;

		do_action( 'mbr/site_tags/fetch' );
		do_action( 'mbr/site_lists/fetch' );
		do_action( 'mbr/site_automations/fetch' );

		$arr_site_tags        = apply_filters( 'mbr/site_tags/get',               NULL );
		$arr_site_lists       = apply_filters( 'mbr/site_lists/get',              NULL );
		$arr_site_automations = apply_filters( 'mbr/site_automations/get/active', NULL );

		IF ( $data = get_post_meta( $post_id, $this->product_data_key, TRUE ) ) :
			$data = json_decode( $data, TRUE );
		ENDIF;

		FOREACH ( $this->product_settings_index AS $settings => $default ) :
			IF ( empty( $data[ $settings ] ) ) :
				$data[ $settings ] = $default;
			ENDIF;
		ENDFOREACH;

		echo '<div id="activemember360_settings" class="panel woocommerce_options_panel">';

		echo "<div class='options_group_product'>";

		echo '<p style="font-size:1.1em;font-weight:bold;">' . _x( 'Select the tags to assign when this product is in an abandoned cart.', 'wp_admin', 'activemember360' ) . '</p>';

		$this->woocommerce_wp_multiselect(
			[
				'id'                => 'mbr_woocommerce_product_data[abandoned_cart_tags][]',
				'label'             => _x( 'Abandoned cart tags', 'wp_admin', 'activemember360' ),
				'class'             => 'mbr-wc-panel-select2',
				'desc_tip'          => 'true',
				'options'           => $arr_site_tags,
				'value'             => $data['abandoned_cart_tags'],
				'description'       => _x( 'Select the ActiveCampaign tags that should be added to contacts when they abandon a cart.', 'wp_admin', 'activemember360' )
			]
		);

		echo '<p style="font-size:1.1em;font-weight:bold;">' . _x( 'Select the actions to take when a purchase takes place.', 'wp_admin', 'activemember360' ) . '</p>';

		woocommerce_wp_checkbox(
			[
				'id'                => 'mbr_woocommerce_product_data[sku_tag_name]',
				'label'             => _x( 'Add "SKU" tag', 'wp_admin', 'activemember360' ),
				'description'       => _x( 'When this box is checked, a tag will be created and added to contacts using this product\'s SKU.', 'wp_admin', 'activemember360' ),
				'value'             => $data['sku_tag_name']
			]
		);

		woocommerce_wp_checkbox(
			[
				'id'                => 'mbr_woocommerce_product_data[product_tag_names]',
				'label'             => _x( 'Add "product tags"', 'wp_admin', 'activemember360' ),
				'description'       => _x( 'When this box is checked, tag(s) will be created and added to contacts using this product\'s tag names.', 'wp_admin', 'activemember360' ),
				'value'             => $data['product_tag_names']
			]
		);

		woocommerce_wp_checkbox(
			[
				'id'                => 'mbr_woocommerce_product_data[product_cat_names]',
				'label'             => _x( 'Add "product categories"', 'wp_admin', 'activemember360' ),
				'description'       => _x( 'When this box is checked, tag(s) will be created and added to contacts using this product\'s category names.', 'wp_admin', 'activemember360' ),
				'value'             => $data['product_cat_names']
			]
		);

		IF ( $this->wc_ext_options['enable_automations_in_panels'] == 'yes' ) :
			$this->woocommerce_wp_multiselect(
				[
					'id'              => 'mbr_woocommerce_product_data[add_to_automations][]',
					'label'           => _x( 'Add to automations', 'wp_admin', 'activemember360' ),
					'class'           => 'mbr-wc-panel-select2',
					'desc_tip'        => 'true',
					'options'         => $arr_site_automations,
					'value'           => $data['add_to_automations'],
					'description'     => _x( 'Select the ActiveCampaign automations to which you would like contacts added when they purchase this product.', 'wp_admin', 'activemember360' )
				]
			);

			$this->woocommerce_wp_multiselect(
				[
					'id'              => 'mbr_woocommerce_product_data[remove_from_automations][]',
					'label'           => _x( 'Remove from automations', 'wp_admin', 'activemember360' ),
					'class'           => 'mbr-wc-panel-select2',
					'desc_tip'        => 'true',
					'options'         => $arr_site_automations,
					'value'           => $data['remove_from_automations'],
					'description'     => _x( 'Select the ActiveCampaign automations from which you would like contacts removed when they purchase this product.', 'wp_admin', 'activemember360' )
				]
			);
		ENDIF;

		IF ( $this->wc_ext_options['enable_tags_in_panels'] == 'yes' ) :
			$this->woocommerce_wp_multiselect(
				[
					'id'              => 'mbr_woocommerce_product_data[assign_tags][]',
					'label'           => _x( 'Assign tags', 'wp_admin', 'activemember360' ),
					'class'           => 'mbr-wc-panel-select2',
					'desc_tip'        => 'true',
					'options'         => $arr_site_tags,
					'value'           => $data['assign_tags'],
					'description'     => _x( 'Select the ActiveCampaign tags that should be added to contacts when they purchase this product.', 'wp_admin', 'activemember360' )
				]
			);

			$this->woocommerce_wp_multiselect([
				'id'                => 'mbr_woocommerce_product_data[remove_tags][]',
				'label'             => _x( 'Remove tags', 'wp_admin', 'activemember360' ),
				'class'             => 'mbr-wc-panel-select2',
				'desc_tip'          => 'true',
				'options'           => $arr_site_tags,
				'value'             => $data['remove_tags'],
				'description'       => _x( 'Select the ActiveCampaign tags that should be removed from contacts when they purchase this product.', 'wp_admin', 'activemember360' )
			]);
		ENDIF;

		IF ( $this->wc_ext_options['enable_lists_in_panels'] == 'yes' ) :
			$this->woocommerce_wp_multiselect(
				[
					'id'              => 'mbr_woocommerce_product_data[add_to_lists][]',
					'label'           => _x( 'Subscribe', 'wp_admin', 'activemember360' ),
					'class'           => 'mbr-wc-panel-select2',
					'desc_tip'        => 'true',
					'options'         => $arr_site_lists,
					'value'           => $data['add_to_lists'],
					'description'     => _x( 'Select the ActiveCampaign lists to which you would like contacts subscribed when they purchase this product.', 'wp_admin', 'activemember360' )
				]
			);

			$this->woocommerce_wp_multiselect(
				[
					'id'              => 'mbr_woocommerce_product_data[remove_from_lists][]',
					'label'           => _x( 'Unsubscribe', 'wp_admin', 'activemember360' ),
					'class'           => 'mbr-wc-panel-select2',
					'desc_tip'        => 'true',
					'options'         => $arr_site_lists,
					'value'           => $data['remove_from_lists'],
					'description'     => _x( 'Select the ActiveCampaign lists from which you would like contacts unsubscribed when they purchase this product.', 'wp_admin', 'activemember360' )
				]
			);
		ENDIF;

		echo '</div>';

		echo "<div class='options_group_product'>";
		echo '<p style="font-size:1.1em;font-weight:bold;">' . _x( 'Select the actions to take when a refund takes place.', 'wp_admin', 'activemember360' ) . '</p>';

		IF ( $this->wc_ext_options['enable_automations_in_panels'] == 'yes' ) :
			$this->woocommerce_wp_multiselect(
				[
					'id'              => 'mbr_woocommerce_product_data[add_to_automations_refunded][]',
					'label'           => _x( 'Add to automations', 'wp_admin', 'activemember360' ),
					'class'           => 'mbr-wc-panel-select2',
					'desc_tip'        => 'true',
					'options'         => $arr_site_automations,
					'value'           => $data['add_to_automations_refunded'],
					'description'     => _x( 'Select the ActiveCampaign automations to which you would like contacts added when a refund takes place.', 'wp_admin', 'activemember360' )
				]
			);

			$this->woocommerce_wp_multiselect(
				[
					'id'              => 'mbr_woocommerce_product_data[remove_from_automations_refunded][]',
					'label'           => _x( 'Remove from automations', 'wp_admin', 'activemember360' ),
					'class'           => 'mbr-wc-panel-select2',
					'desc_tip'        => 'true',
					'options'         => $arr_site_automations,
					'value'           => $data['remove_from_automations_refunded'],
					'description'     => _x( 'Select the ActiveCampaign automations from which you would like contacts removed when a refund takes place.', 'wp_admin', 'activemember360' )
				]
			);
		ENDIF;

		IF ( $this->wc_ext_options['enable_tags_in_panels'] == 'yes' ) :
			$this->woocommerce_wp_multiselect(
				[
					'id'              => 'mbr_woocommerce_product_data[assign_tags_refunded][]',
					'label'           => _x( 'Assign tags', 'wp_admin', 'activemember360' ),
					'class'           => 'mbr-wc-panel-select2',
					'desc_tip'        => 'true',
					'options'         => $arr_site_tags,
					'value'           => $data['assign_tags_refunded'],
					'description'     => _x( 'Select the ActiveCampaign tags that should be added to contacts when a refund takes place.', 'wp_admin', 'activemember360' )
				]
			);

			$this->woocommerce_wp_multiselect(
				[
					'id'              => 'mbr_woocommerce_product_data[remove_tags_refunded][]',
					'label'           => _x( 'Remove tags', 'wp_admin', 'activemember360' ),
					'class'           => 'mbr-wc-panel-select2',
					'desc_tip'        => 'true',
					'options'         => $arr_site_tags,
					'value'           => $data['remove_tags_refunded'],
					'description'     => _x( 'Select the ActiveCampaign tags that should be removed from contacts when  a refund takes place', 'wp_admin', 'activemember360' )
				]
			);
		ENDIF;

		IF ( $this->wc_ext_options['enable_lists_in_panels'] == 'yes' ) :
			$this->woocommerce_wp_multiselect(
				[
					'id'              => 'mbr_woocommerce_product_data[add_to_lists_refunded][]',
					'label'           => _x( 'Subscribe', 'wp_admin', 'activemember360' ),
					'class'           => 'mbr-wc-panel-select2',
					'desc_tip'        => 'true',
					'options'         => $arr_site_lists,
					'value'           => $data['add_to_lists_refunded'],
					'description'     => _x( 'Select the ActiveCampaign lists to which you would like contacts subscribed when a refund takes place.', 'wp_admin', 'activemember360' )
				]
			);

			$this->woocommerce_wp_multiselect(
				[
					'id'              => 'mbr_woocommerce_product_data[remove_from_lists_refunded][]',
					'label'           => _x( 'Unsubscribe', 'wp_admin', 'activemember360' ),
					'class'           => 'mbr-wc-panel-select2',
					'desc_tip'        => 'true',
					'options'         => $arr_site_lists,
					'value'           => $data['remove_from_lists_refunded'],
					'description'     => _x( 'Select the ActiveCampaign lists from which you would like contacts unsubscribed when a refund takes place.', 'wp_admin', 'activemember360' )
				]
			);
		ENDIF;
		echo "</div>";

		IF ( class_exists( 'WC_Subscription' ) ) :

			$wrapper_class = 'show_if_subscription show_if_variable-subscription';

			echo "<div class='options_group_product {$wrapper_class}'>";

			echo '<hr style="height:1px;" />';
			echo '<div class="show_if_subscription clear"></div>';

			echo '<p style="font-size:1.1em;font-weight:bold;">' . __('Select the actions to take when this subscription is activated, cancelled, refunded, expires, put on hold or reactivated.', 'activemember360') . '</p>';


			IF ( $this->wc_ext_options['enable_tags_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_tags_sub_active][]',
						'label'             => _x( 'Activated: Add tags', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_tags,
						'value'             => $data['add_tags_sub_active'],
						'description'       => _x( 'Select the ActiveCampaign tags that should be assigned to contacts when this subscription is activated.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_tags_sub_active][]',
						'label'             => _x('Activated: Remove tags', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_tags,
						'value'             => $data['remove_tags_sub_active'],
						'description'       => _x( 'Select the ActiveCampaign tags that should be removed from contacts when this subscription is activated.', 'wp_admin', 'activemember360' )
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_automations_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_to_automations_sub_active][]',
						'label'             => _x( 'Activated: Add to automations', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_automations,
						'value'             => $data['add_to_automations_sub_active'],
						'description'       => _x( 'Select the ActiveCampaign automations to which you would like contacts added when this subscription is activated.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_from_automations_sub_active][]',
						'label'             => _x( 'Activated: Remove from automations', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_automations,
						'value'             => $data['remove_from_automations_sub_active'],
						'description'       => _x( 'Select the ActiveCampaign automations from which you would like contacts removed when this subscription is activated.', 'wp_admin', 'activemember360' )
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_lists_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_to_lists_sub_active][]',
						'label'             => _x( 'Activated: Subscribe', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_lists,
						'value'             => $data['add_to_lists_sub_active'],
						'description'       => _x( 'Select the ActiveCampaign lists to which you would like contacts subscribed when this subscription is activated.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_from_lists_sub_active][]',
						'label'             => _x( 'Activated: Unsubscribe', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_lists,
						'value'             => $data['remove_from_lists_sub_active'],
						'description'       => _x( 'Select the ActiveCampaign lists from which you would like contacts unsubscribed when this subscription is activated.', 'wp_admin', 'activemember360' )
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_tags_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_tags_sub_cancelled][]',
						'label'             => _x( 'Cancelled: Add tags', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_tags,
						'value'             => $data['add_tags_sub_cancelled'],
						'description'       => _x( 'Select the ActiveCampaign tags that should be assigned from contacts when this subscription is cancelled.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_tags_sub_cancelled][]',
						'label'             => _x( 'Cancelled: Remove tags', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_tags,
						'value'             => $data['remove_tags_sub_cancelled'],
						'description'       => _x( 'Select the ActiveCampaign tags that should be removed from contacts when this subscription is cancelled.', 'wp_admin', 'activemember360' )
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_automations_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_to_automations_sub_cancelled][]',
						'label'             => _x( 'Cancelled: Add to automations', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_automations,
						'value'             => $data['add_to_automations_sub_cancelled'],
						'description'       => _x( 'Select the ActiveCampaign automations to which you would like contacts added when this subscription is cancelled.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_from_automations_sub_cancelled][]',
						'label'             => _x( 'Cancelled: Remove from automations', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_automations,
						'value'             => $data['remove_from_automations_sub_cancelled'],
						'description'       => _x( 'Select the ActiveCampaign automations from which you would like contacts removed when this subscription is cancelled.', 'wp_admin', 'activemember360' )
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_lists_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_to_lists_sub_cancelled][]',
						'label'             => _x( 'Cancelled: Subscribe', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_lists,
						'value'             => $data['add_to_lists_sub_cancelled'],
						'description'       => _x( 'Select the ActiveCampaign lists to which you would like contacts subscribed when this subscription is cancelled.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_from_lists_sub_cancelled][]',
						'label'             => _x( 'Cancelled: Unsubscribe', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_lists,
						'value'             => $data['remove_from_lists_sub_cancelled'],
						'description'       => _x( 'Select the ActiveCampaign lists from which you would like contacts unsubscribed when this subscription is cancelled.', 'wp_admin', 'activemember360' )
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_tags_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_tags_sub_onhold][]',
						'label'             => _x( 'On Hold: Add tags', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_tags,
						'value'             => $data['add_tags_sub_onhold'],
						'description'       => _x( 'Select the ActiveCampaign tags that should be assigned to contacts when this subscription is placed on hold.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_tags_sub_onhold][]',
						'label'             => _x( 'On Hold: Remove tags', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_tags,
						'value'             => $data['remove_tags_sub_onhold'],
						'description'       => _x( 'Select the ActiveCampaign tags that should be removed from contacts when this subscription is placed on hold.', 'wp_admin', 'activemember360' )
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_automations_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_to_automations_sub_onhold][]',
						'label'             => _x( 'On hold: Add to automations', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_automations,
						'value'             => $data['add_to_automations_sub_onhold'],
						'description'       => _x( 'Select the ActiveCampaign automations to which you would like contacts added when this subscription is on hold.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_from_automations_sub_onhold][]',
						'label'             => _x( 'On hold: Remove from automations', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_automations,
						'value'             => $data['remove_from_automations_sub_onhold'],
						'description'       => _x( 'Select the ActiveCampaign automations from which you would like contacts removed when this subscription is on hold.', 'wp_admin', 'activemember360' )
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_lists_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_to_lists_sub_onhold][]',
						'label'             => _x( 'On hold: Subscribe', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_lists,
						'value'             => $data['add_to_lists_sub_onhold'],
						'description'       => _x( 'Select the ActiveCampaign lists to which you would like contacts subscribed when this subscription is on hold.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_from_lists_sub_onhold][]',
						'label'             => __('On hold: Unsubscribe', 'activemember360'),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_lists,
						'value'             => $data['remove_from_lists_sub_onhold'],
						'description'       => __('Select the ActiveCampaign lists from which you would like contacts unsubscribed when this subscription is on hold.', 'activemember360' )
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_tags_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_tags_sub_reactivated][]',
						'label'             => __('Reactivated: Add tags', 'activemember360'),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_tags,
						'value'             => $data['add_tags_sub_reactivated'],
						'description'       => __('Select the ActiveCampaign tags that should be assigned to contacts when this subscription is reactivated.', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_tags_sub_reactivated][]',
						'label'             => _x( 'Reactivated: Remove tags', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_tags,
						'value'             => $data['remove_tags_sub_reactivated'],
						'description'       => _x( 'Select the ActiveCampaign tags that should be removed from contacts when this subscription is reactivated.', 'wp_admin', 'activemember360' )
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_automations_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_to_automations_sub_reactivated][]',
						'label'             => _x( 'Reactivated: Add to automations', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_automations,
						'value'             => $data['add_to_automations_sub_reactivated'],
						'description'       => _x( 'Select the ActiveCampaign automations to which you would like contacts added when this subscription is reactivated.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_from_automations_sub_reactivated][]',
						'label'             => _x( 'Reactivated: Remove from automations', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_automations,
						'value'             => $data['remove_from_automations_sub_reactivated'],
						'description'       => _x( 'Select the ActiveCampaign automations from which you would like contacts removed when this subscription is reactivated.', 'wp_admin', 'activemember360' )
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_lists_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_to_lists_sub_reactivated][]',
						'label'             => _x( 'Reactivated: Subscribe', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_lists,
						'value'             => $data['add_to_lists_sub_reactivated'],
						'description'       => _x( 'Select the ActiveCampaign lists to which you would like contacts subscribed when this subscription is reactivated.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_from_lists_sub_reactivated][]',
						'label'             => _x( 'Reactivated: Unsubscribe', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_lists,
						'value'             => $data['remove_from_lists_sub_reactivated'],
						'description'       => _x( 'Select the ActiveCampaign lists from which you would like contacts unsubscribed when this subscription is reactivated.', 'wp_admin', 'activemember360' )
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_tags_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_tags_sub_expired][]',
						'label'             => _x( 'Expired: Add tags', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_tags,
						'value'             => $data['add_tags_sub_expired'],
						'description'       => _x( 'Select the ActiveCampaign tags that should be assigned from contacts when this subscription expires.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_tags_sub_expired][]',
						'label'             => _x( 'Expired: Remove tags', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_tags,
						'value'             => $data['remove_tags_sub_expired'],
						'description'       => _x( 'Select the ActiveCampaign tags that should be removed from contacts when this subscription expires.', 'wp_admin', 'activemember360' )
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_automations_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_to_automations_sub_expired][]',
						'label'             => _x( 'Expired: Add to automations', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_automations,
						'value'             => $data['add_to_automations_sub_expired'],
						'description'       => _x( 'Select the ActiveCampaign automations to which you would like contacts added when this subscription is expired.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_from_automations_sub_expired][]',
						'label'             => _x( 'Expired: Remove from automations', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_automations,
						'value'             => $data['remove_from_automations_sub_expired'],
						'description'       => _x( 'Select the ActiveCampaign automations from which you would like contacts removed when this subscription is expired.', 'wp_admin', 'activemember360' )
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_lists_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_to_lists_sub_expired][]',
						'label'             => _x( 'Expired: Subscribe', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_lists,
						'value'             => $data['add_to_lists_sub_expired'],
						'description'       => _x( 'Select the ActiveCampaign lists to which you would like contacts subscribed when this subscription is expired.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_from_lists_sub_expired][]',
						'label'             => _x( 'Expired: Unsubscribe', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_lists,
						'value'             => $data['remove_from_lists_sub_expired'],
						'description'       => _x( 'Select the ActiveCampaign lists from which you would like contacts unsubscribed when this subscription is expired.', 'wp_admin', 'activemember360' )
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_tags_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_tags_sub_refunded][]',
						'label'             => _x( 'Refunded: Add tags', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_tags,
						'value'             => $data['add_tags_sub_refunded'],
						'description'       => _x( 'Select the ActiveCampaign tags that should be assigned from contacts when this subscription is refunded.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_tags_sub_refunded][]',
						'label'             => _x( 'Refunded: Remove tags', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_tags,
						'value'             => $data['remove_tags_sub_refunded'],
						'description'       => _x( 'Select the ActiveCampaign tags that should be removed from contacts when this subscription is refunded.', 'wp_admin', 'activemember360' ),
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_automations_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_to_automations_sub_refunded][]',
						'label'             => _x( 'Refunded: Add to automations', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_automations,
						'value'             => $data['add_to_automations_sub_refunded'],
						'description'       => _x( 'Select the ActiveCampaign automations to which you would like contacts added when this subscription is refunded.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_from_automations_sub_refunded][]',
						'label'             => _x( 'Refunded: Remove from automations', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_automations,
						'value'             => $data['remove_from_automations_sub_refunded'],
						'description'       => _x( 'Select the ActiveCampaign automations from which you would like contacts removed when this subscription is refunded.', 'wp_admin', 'activemember360' )
					]
				);
			ENDIF;

			IF ( $this->wc_ext_options['enable_lists_in_panels'] == 'yes' ) :
				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[add_to_lists_sub_refunded][]',
						'label'             => _x( 'Refunded: Subscribe', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_lists,
						'value'             => $data['add_to_lists_sub_refunded'],
						'description'       => _x( 'Select the ActiveCampaign lists to which you would like contacts subscribed when this subscription is refunded.', 'wp_admin', 'activemember360' )
					]
				);

				$this->woocommerce_wp_multiselect(
					[
						'id'                => 'mbr_woocommerce_product_data[remove_from_lists_sub_refunded][]',
						'label'             => _x( 'Refunded: Unsubscribe', 'wp_admin', 'activemember360' ),
						'class'             => 'mbr-wc-panel-select2',
						'wrapper_class'     => $wrapper_class,
						'desc_tip'          => 'true',
						'options'           => $arr_site_lists,
						'value'             => $data['remove_from_lists_sub_refunded'],
						'description'       => _x( 'Select the ActiveCampaign lists from which you would like contacts unsubscribed when this subscription is refunded.', 'wp_admin', 'activemember360' )
					]
				);
			ENDIF;
			echo '</div>';
		ENDIF;

		echo '</div>';
	}

	public function woocommerce_product_options_pricing()
	{
		IF ( empty( $this->wc_ext_options['number_alt_prices'] ) ) :
			return;
		ENDIF;

		global $thepostid, $post;
		$post_id = empty( $thepostid ) ? $post->ID : $thepostid;

		IF ( $data = get_post_meta( $post_id, $this->product_data_key, TRUE ) ) :
			$data  = json_decode( $data, TRUE );
		ENDIF;

		FOREACH ( $this->product_settings_index AS $settings => $default ) :
			IF ( empty( $data[ $settings ] ) ) :
				$data[ $settings ] = $default;
			ENDIF;
		ENDFOREACH;

		$arr_site_tags = apply_filters( 'mbr/site_tags/get', NULL );

		echo '<p style="font-size:1.1em;font-weight:bold;">' . _x( 'Specify tag-based alternate price points.', 'wp_admin', 'activemember360' ) . '</p>';
		FOR ( $x = 1; $x <= $this->wc_ext_options['number_alt_prices']; $x++ ) :
			$this->woocommerce_wp_multiselect(
				[
					'id'                => 'mbr_woocommerce_product_data[alt_price_tags][' . $x . '][]',
					'class'             => 'mbr-wc-panel-select2',
					'label'             => sprintf( _x( 'Tags for #%s', 'activemember360' ), $x ),
					'options'           => $arr_site_tags,
					'value'             => $data['alt_price_tags'][ $x ],
				], TRUE
			);
			woocommerce_wp_text_input(
				[
					'id'                => 'mbr_woocommerce_product_data[alt_price][' . $x . ']',
					'label'             => sprintf( _x( 'Alternate price #%s', 'wp_admin', 'activemember360' ) . ' (' . get_woocommerce_currency_symbol() . ')', $x ),
					'class'             => 'short wc-input-price',
					'value'             => $data['alt_price'][ $x ],
					'data_type'         => 'price'
				]
			);
		ENDFOR;
	}

	public function get_lowest_price_by_product_id( $product_id = FALSE )
	{
		IF ( empty( $product_id ) ) :
			return FALSE;
		ENDIF;

		$low_price = $this->get_lowest_price( FALSE, $product_id );

		return $low_price;
	}

	public function get_lowest_price_by_data( $data = [] )
	{
		IF ( empty( $data ) ) :
			return FALSE;
		ENDIF;

		$low_price = $this->get_lowest_price( $data, FALSE );

		return $low_price;
	}

	public function get_lowest_price( $data = [], $product_id = FALSE )
	{
		IF ( ! is_user_logged_in() OR ! apply_filters( 'mbr/contact_id', NULL ) ) :
			return FALSE;
		ENDIF;

		IF ( empty( $data ) AND empty( $product_id ) ) :
			return FALSE;
		ENDIF;

		IF ( empty( $data ) AND ! empty( $product_id ) ) :
			IF ( $data = get_post_meta( $product_id, $this->product_data_key, TRUE ) ) :
				$data = json_decode( $data, TRUE );
			ELSE:
				return FALSE;
			ENDIF;
		ENDIF;

		IF ( empty( $data['alt_price_tags'] ) OR ! is_array( $data['alt_price_tags'] ) OR empty( $data['alt_price'] ) OR ! is_array( $data['alt_price'] ) ) :
			return FALSE;
		ENDIF;

		$arr_contact_tags = apply_filters( 'mbr/contact/tags', NULL );
		$low_price = FALSE;
		FOREACH ( $data['alt_price_tags'] AS $x => $arr_tags ) :
			IF ( empty( $data['alt_price'][ $x ] ) OR empty( $arr_tags ) ) :
				continue;
			ENDIF;

			FOREACH ( $arr_tags AS $filler => $tag_id ) :
				IF ( ! array_key_exists( $tag_id, $arr_contact_tags ) ) :
					continue;
				ELSEIF ( empty( $low_price ) ) :
					$low_price = $data['alt_price'][ $x ];                    
				ELSE :
					$low_price = min( $data['alt_price'][ $x ], $low_price ); 
				ENDIF;
			ENDFOREACH;
		ENDFOREACH;

		return $low_price;
	}

	public function woocommerce_get_price_html( $price_html, $product )
	{
		IF ( ! $data = get_post_meta( $product->get_id(), $this->product_data_key, TRUE ) ) :
			return $price_html;
		ENDIF;

		$data = json_decode( $data, TRUE );
		IF ( empty( $data['alt_price_tags'] ) OR ! is_array( $data['alt_price_tags'] ) ) :
			return $price_html;
		ENDIF;

		$low_price = $this->get_lowest_price_by_data( $data );

		IF ( empty( $low_price ) ) :
			return $price_html;
		ENDIF;

		IF ( $product->is_on_sale() ) :
			$sale_price = wc_get_price_to_display( $product );
			IF ( $sale_price < $low_price ) :  
				return $price_html;
			ENDIF;
		ENDIF;

		return wc_format_sale_price( wc_get_price_to_display( $product, [ 'price' => $product->get_regular_price() ] ), wc_get_price_to_display( $product, [ 'price' => $low_price ] ) ) . $product->get_price_suffix();
	}

	public function add_custom_price( $cart_item )
	{
		IF ( ! is_user_logged_in() OR ! apply_filters( 'mbr/contact_id', NULL ) ) :
			return $cart_item;
		ENDIF;

		$low_price = $this->get_lowest_price_by_product_id( $cart_item['product_id'] );

		IF ( ! empty( $low_price ) ) :
			$cart_item['data']->set_price( $low_price );
			$cart_item[ $this->wc_custom_price_key ] = $low_price;
		ENDIF;

		return $cart_item;
	}

	public function set_session_prices( $cart_item, $values, $key )
	{
		IF ( ! is_user_logged_in() OR ! apply_filters( 'mbr/contact_id', NULL ) ) :
			return $cart_item;
		ENDIF;
		IF ( ! empty( $cart_item[ $this->wc_custom_price_key ] ) ) :
			$cart_item['data']->set_price( $cart_item[ $this->wc_custom_price_key ] );
		ENDIF;

		return $cart_item;
	}

	
	public function woocommerce_subscriptions_product_price( $price, $product )
	{
		IF ( ! is_user_logged_in() OR ! apply_filters( 'mbr/contact_id', NULL ) ) :
			return $price;
		ENDIF;

		IF ( ! $product->is_type( 'subscription' ) AND ! $product->is_type( 'variable-subscription' ) ) :
			return $price;
		ENDIF;

		$cfilter = current_filter();

		IF ( $data = get_post_meta( $product->get_id(), $this->product_data_key, TRUE ) ) :
			$data = json_decode( $data, TRUE );
		ENDIF;

		IF ( ! empty( $data['alt_price_tags'] ) ) :
			$arr_contact_tags = apply_filters( 'mbr/contact/tags', NULL );
			FOREACH ( $data['alt_price_tags'] AS $x => $arr_tags ) :
				FOREACH ( $arr_tags AS $filler => $tag_id ) :
					IF ( array_key_exists( $tag_id, $arr_contact_tags ) ) :
						IF ( ! empty( $data['alt_price'][ $x ] ) ) :
							$price2  = $data['alt_price'][ $x ];
							
							IF ( $price2 < $price ) :
								$price = $price2;
							ENDIF;
							break 2;
						ENDIF;
					ENDIF;
				ENDFOREACH;
			ENDFOREACH;
		ENDIF;

		return $price;
	}

	
	public function woocommerce_subscriptions_sign_up_fee( $sign_up_fee, $order, $product_id )
	{
		return $sign_up_fee;
	}

	
	public function woocommerce_subscriptions_product_sale_price( $price, $product )
	{
		IF ( ! is_user_logged_in() OR ! apply_filters( 'mbr/contact_id', NULL ) ) :
			return $price;
		ENDIF;

		IF ( ! $product->is_type( 'subscription' ) AND ! $product->is_type( 'variable-subscription' ) ) :
			return $price;
		ENDIF;

		$cfilter = current_filter();

		IF ( $data = get_post_meta($product->get_id(), $this->product_data_key, TRUE ) ) :
			$data = json_decode( $data, TRUE );
		ENDIF;

		IF ( ! empty( $data['alt_price_tags'] ) ) :
			$arr_contact_tags = apply_filters( 'mbr/contact/tags', NULL );
			FOREACH ( $data['alt_price_tags'] AS $x => $arr_tags ) :
				FOREACH ( $arr_tags AS $filler => $tag_id ) :
					IF ( array_key_exists( $tag_id, $arr_contact_tags ) ) :
						IF ( ! empty( $data['alt_price'][ $x ] ) ) :
							$price2  = $data['alt_price'][ $x ];
							
							IF ( $price2 < $price ) :
								$price = $price2;
							ENDIF;
							break 2;
						ENDIF;
					ENDIF;
				ENDFOREACH;
			ENDFOREACH;
		ENDIF;

		return $price;
	}

	
	public function woocommerce_product_get_price( $price, $product )
	{
		IF ( ! is_user_logged_in() OR ! apply_filters( 'mbr/contact_id', NULL ) ) :
			return $price;
		ENDIF;

		IF ( ! $product->is_type( 'simple' ) AND ! $product->is_type( 'subscription' ) ) :
			return $price;
		ENDIF;

		$cfilter = current_filter();
		






















		return $price;
	}

	
	public function woocommerce_product_get_sale_price( $price, $product )
	{
		IF ( ! is_user_logged_in() OR ! apply_filters( 'mbr/contact_id', NULL ) ) :
			return $price;
		ENDIF;

		IF ( ! $product->is_type( 'simple' ) AND ! $product->is_type( 'subscription' ) ) :
			return $price;
		ENDIF;

		$cfilter = current_filter();

		IF ( $data = get_post_meta( $product->get_id(), $this->product_data_key, TRUE ) ) :
			$data = json_decode( $data, TRUE );
		ENDIF;

		IF ( ! empty( $data['alt_price_tags'] ) ) :
			$arr_contact_tags = apply_filters( 'mbr/contact/tags', NULL );
			FOREACH ( $data['alt_price_tags'] AS $x => $arr_tags ) :
				FOREACH ( $arr_tags AS $filler => $tag_id ) :
					IF ( array_key_exists( $tag_id, $arr_contact_tags ) ) :
						IF ( ! empty( $data['alt_price'][ $x ] ) ) :
							$price2  = $data['alt_price'][ $x ];
							
							IF ( $price2 < $price ) :
								$price = $price2;
							ENDIF;
							break 2;
						ENDIF;
					ENDIF;
				ENDFOREACH;
			ENDFOREACH;
		ENDIF;

		return $price;
	}

	public function woocommerce_product_after_variable_attributes( $loop, $variation_data, $variation )
	{
		echo '<p class="form-field"><label for="mbr-variation-tags"><strong>' . _x( 'ActiveMember360 - Select the actions to take when this product variation has been purchased.', 'wp_admin', 'activemember360' ) . '</strong></label>';
		IF ( $this->wc_ext_options['enable_tags_in_panels'] == 'yes' ) :

			$arr_site_tags         = apply_filters( 'mbr/site_tags/get', NULL );

			IF ( $data = get_post_meta( $variation->ID, $this->product_data_key, TRUE ) ) :
				$data = json_decode( $data, TRUE );
			ENDIF;

			FOREACH ( $this->product_settings_index AS $settings => $default ) :
				IF ( empty( $data[ $settings ] ) ) :
					$data[ $settings ] = $default;
				ENDIF;
			ENDFOREACH;

			$this->woocommerce_wp_multiselect(
				[
					'id'                => 'mbr_woocommerce_product_data[variations][' . $variation->ID. '][assign_tags][]',
					'class'             => 'mbr-wc-panel-select2',
					'label'             => _x('Assign tags', 'wp_admin', 'activemember360' ),
					'desc_tip'          => 'true',
					'options'           => $arr_site_tags,
					'value'             => $data['assign_tags'],
					'description'       => _x( 'Select the ActiveCampaign tags that should be added to contacts when they purchase this product variation.', 'wp_admin', 'activemember360' ),
				], TRUE
			);

			$this->woocommerce_wp_multiselect(
				[
					'id'                => 'mbr_woocommerce_product_data[variations][' . $variation->ID. '][remove_tags][]',
					'class'             => 'mbr-wc-panel-select2',
					'label'             => _x( 'Remove tags', 'wp_admin', 'activemember360' ),
					'desc_tip'          => 'true',
					'options'           => $arr_site_tags,
					'value'             => $data['remove_tags'],
					'description'       => _x( 'Select the ActiveCampaign tags that should be removed from contacts when they purchase this product variation.', 'wp_admin', 'activemember360' ),
				], TRUE
			);
		ELSE:
			echo '<p>' . __( 'Add/remove tags in panels is disabled.', 'wp_admin', 'activemember360' ) . '<br />' . _x( 'This can be enabled in WooCommerce -> Settings -> Integrations -> ActiveMember360.', 'wp_admin', 'activemember360' ) . '</p>';
		ENDIF;
		echo '</p>';
	}

	public function woocommerce_update_product_variation( $variation_id )
	{
		$data = isset( $_POST['mbr_woocommerce_product_data']['variations'][ $variation_id ]) ? $_POST['mbr_woocommerce_product_data']['variations'][ $variation_id ] : [];
		FOREACH ( $this->product_settings_index AS $settings => $default ) :
			IF ( empty( $data[ $settings ] ) ) :
				$data[ $settings ] = $default;
			ENDIF;
		ENDFOREACH;

		update_post_meta( $variation_id, $this->product_data_key, json_encode( $data ) );
	}

	
	public function woocommerce_coupon_data_tabs( $data_tabs )
	{
		$data_tabs['activemember360'] = [ 'label'  => 'ActiveMember360', 'target' => 'activemember360_coupon_settings' ];

		return $data_tabs;
	}

	public function woocommerce_coupon_data_panels()
	{
		global $thepostid, $post;
		$post_id = empty( $thepostid ) ? $post->ID : $thepostid;

		do_action( 'mbr/site_tags/fetch' );
		do_action( 'mbr/site_lists/fetch' );
		do_action( 'mbr/site_automations/fetch' );

		IF ( $data = get_post_meta( $post_id, $this->coupon_data_key, TRUE ) ) :
			$data = json_decode( $data, TRUE );
		ENDIF;

		FOREACH ( $this->coupon_settings_index AS $settings => $default ) :
			IF ( empty( $data[ $settings ] ) ) :
				$data[ $settings ] = $default;
			ENDIF;
		ENDFOREACH;

		echo '<div id="activemember360_coupon_settings" class="panel woocommerce_options_panel">';
		echo '<div class="options_group">';
		echo '<p style="font-size:1.1em;font-weight:bold;">' . _x('Select the actions to take when this coupon is used.', 'wp_admin', 'activemember360' ) . '</p>';

		woocommerce_wp_checkbox(
			[
				'id'                  => 'mbr_woocommerce_coupon_data[coupon_tag_name]',
				'label'               => _x( 'Add coupon name as tag', 'wp_admin', 'activemember360' ),
				'description'         => sprintf( _x( 'When this box is checked, a tag will be created and added to the contacts when they use this coupon.', 'wp_admin', 'activemember360' ) ),
				'value'               => $data['coupon_tag_name']
			]
		);

		IF ( $this->wc_ext_options['enable_automations_in_panels'] == 'yes' ) :
			$arr_site_automations   = apply_filters( 'mbr/site_automations/get/active', NULL );
			$this->woocommerce_wp_multiselect(
				[
					'id'                => 'mbr_woocommerce_coupon_data[add_to_automations][]',
					'label'             => _x( 'Add contacts<br />to automation(s)', 'wp_admin', 'activemember360' ),
					'class'             => 'mbr-wc-panel-select2',
					'desc_tip'          => 'true',
					'options'           => $arr_site_automations,
					'value'             => $data['add_to_automations'],
					'description'       => _x( 'Select the ActiveCampaign lists to which you would like contacts added when they use this coupon.', 'wp_admin', 'activemember360' ),
				]
			);

			$this->woocommerce_wp_multiselect(
				[
					'id'                => 'mbr_woocommerce_coupon_data[remove_from_automations][]',
					'label'             => _x( 'Remove contacts<br />from automation(s)', 'wp_admin', 'activemember360' ),
					'class'             => 'mbr-wc-panel-select2',
					'desc_tip'          => 'true',
					'options'           => $arr_site_automations,
					'value'             => $data['remove_from_automations'],
					'description'       => _x( 'Select the ActiveCampaign lists from which you would like contacts removed when they use this coupon.', 'wp_admin', 'activemember360' ),
				]
			);
		ENDIF;

		IF ( $this->wc_ext_options['enable_tags_in_panels'] == 'yes' ) :
			$arr_site_tags          = apply_filters( 'mbr/site_tags/get', NULL );
			$this->woocommerce_wp_multiselect(
				[
					'id'                => 'mbr_woocommerce_coupon_data[assign_tags][]',
					'label'             => _x( 'Assign tag(s)<br />to contacts', 'wp_admin', 'activemember360' ),
					'class'             => 'mbr-wc-panel-select2',
					'desc_tip'          => 'true',
					'options'           => $arr_site_tags,
					'value'             => $data['assign_tags'],
					'description'       => _x( 'Select the ActiveCampaign tags that should be added to contacts when they use this coupon.', 'wp_admin', 'activemember360' ),
				]
			);

			$this->woocommerce_wp_multiselect(
				[
					'id'                => 'mbr_woocommerce_coupon_data[remove_tags][]',
					'label'             => _x( 'Remove tag(s)<br />from contacts', 'wp_admin', 'activemember360' ),
					'class'             => 'mbr-wc-panel-select2',
					'desc_tip'          => 'true',
					'options'           => $arr_site_tags,
					'value'             => $data['remove_tags'],
					'description'       => _x( 'Select the ActiveCampaign tags that should be removed from contacts when they use this coupon.', 'wp_admin', 'activemember360' ),
				]
			);
		ENDIF;

		IF ( $this->wc_ext_options['enable_lists_in_panels'] == 'yes' ) :
			$arr_site_lists          = apply_filters( 'mbr/site_tags/get', NULL );
			$this->woocommerce_wp_multiselect(
				[
					'id'                => 'mbr_woocommerce_coupon_data[add_to_lists][]',
					'label'             => _x( 'Subscribe contacts<br />to list(s)', 'wp_admin', 'activemember360' ),
					'class'             => 'mbr-wc-panel-select2',
					'desc_tip'          => 'true',
					'options'           => $arr_site_lists,
					'value'             => $data['add_to_lists'],
					'description'       => _x( 'Select the ActiveCampaign lists to which you would like contacts subscribed when they purchase this product.', 'wp_admin', 'activemember360' ),
				]
			);

			$this->woocommerce_wp_multiselect(
			[
					'id'                => 'mbr_woocommerce_coupon_data[remove_from_lists][]',
					'label'             => _x( 'Unsubscribe contacts<br />from list(s)', 'wp_admin', 'activemember360' ),
					'class'             => 'mbr-wc-panel-select2',
					'desc_tip'          => 'true',
					'options'           => $arr_site_lists,
					'value'             => $data['remove_from_lists'],
					'description'       => _x( 'Select the ActiveCampaign lists from which you would like contacts unsubscribed when they use this coupon.', 'wp_admin', 'activemember360' ),
				]
			);
		ENDIF;
		echo '</div>';
		echo '</div>';
	}

	public function woocommerce_wp_multiselect( $field, $is_variation = FALSE )
	{
		global $thepostid, $post;

		$thepostid              = empty( $thepostid )              ? $post->ID               : $thepostid;
		$field['class']         = isset( $field['class'] )         ? $field['class']         : 'select short';
		$field['style']         = isset( $field['style'] )         ? $field['style']         : '';
		$field['wrapper_class'] = isset( $field['wrapper_class'] ) ? $field['wrapper_class'] : '';
		$field['value']         = isset( $field['value'] )         ? $field['value']         : get_post_meta( $thepostid, $field['id'], TRUE );
		$field['name']          = isset( $field['name'] )          ? $field['name']          : $field['id'];

		
		$field['token']         = isset( $field['token'] )         ? $field['token']         : uniqid( 'mbr_token_' );

		
		$custom_attributes = [];
		IF ( ! empty( $field['custom_attributes'] ) AND is_array( $field['custom_attributes'] ) ) :
			FOREACH ( $field['custom_attributes'] AS $attribute => $value ) :
				$custom_attributes[] = esc_attr( $attribute ) . '="' . esc_attr( $value ) . '"';
			ENDFOREACH;
		ENDIF;

		$field_id = esc_attr( $field['id'] );
		echo '<p class="form-field ' . $field_id . '_field ' . esc_attr( $field['wrapper_class'] ) . '">';
		echo '<label for="' . $field['token'] . '">' . wp_kses_post( $field['label'] ) . '</label>';
		IF ( $is_variation AND ! empty( $field['description'] ) ) :
			IF ( isset( $field['desc_tip'] ) AND FALSE !== $field['desc_tip'] ) :
				echo '<span class="mbr_wc_help_tip">' . wc_help_tip( $field['description'] )  . '</span>';
			ELSE :
				echo '<span class="description">'     . wp_kses_post( $field['description'] ) . '</span>';
			ENDIF;
		ENDIF;

		echo '<select multiple="multiple" id="' . $field['token'] . '" name="' . esc_attr( $field['name'] ) . '" class="' . esc_attr( $field['class'] ) . '" style="' . esc_attr( $field['style'] ) . '" ' . implode( ' ', $custom_attributes ) . '>';

		FOREACH ( $field['options'] AS $key => $value ) :
			$selected = in_array( $key, $field['value'] ) ? ' selected="selected"' : '';
			echo '<option value="' . esc_attr( $key ) . '"' . $selected . '>' . esc_html( $key ) . ' - ' . esc_html( $value ) . '</option>';
		ENDFOREACH;
		echo '</select> ';

		IF ( $is_variation ) :
			echo "<script type='text/javascript'>jQuery( '.mbr-wc-panel-select2' ).select2( { width: '100%' } );</script>";
			static $has_run_css = FALSE;
			IF ( ! $has_run_css ) :
				$has_run_css = TRUE;
				echo '<style>.mbr_wc_help_tip .woocommerce-help-tip{float:right;}</style>';
			ENDIF;
		ENDIF;

		IF ( ! $is_variation AND ! empty( $field['description'] ) ) :
			IF ( isset( $field['desc_tip'] ) AND FALSE !== $field['desc_tip'] ) :
				echo wc_help_tip( $field['description'] );
			ELSE :
				echo '<span class="description">'     . wp_kses_post( $field['description'] ) . '</span>';
			ENDIF;
		ENDIF;
		echo '</p>';
	}

	public function woocommerce_coupon_options_save( $post_id )
	{
		
		$data = isset( $_POST['mbr_woocommerce_coupon_data'] ) ? $_POST['mbr_woocommerce_coupon_data'] : [];

		
		FOREACH ( $this->coupon_settings_index AS $settings => $default ) :
			IF ( empty( $data[ $settings ] ) ) :
				$data[ $settings ] = $default;
			ENDIF;
		ENDFOREACH;

		
		update_post_meta( $post_id, $this->coupon_data_key, json_encode( $data ) );
	}

	public function woocommerce_process_product_meta( $post_id )
	{
		



		
		$data = isset( $_POST['mbr_woocommerce_product_data'] ) ? $_POST['mbr_woocommerce_product_data'] : [];

		FOREACH ( $this->product_settings_index AS $settings => $default ) :
			IF ( empty( $data[ $settings ] ) ) :
				$data[ $settings ] = $default;
			ENDIF;
		ENDFOREACH;

		
		IF ( ! in_array( $_POST['product-type'], ['subscription', 'variable-subscription'] ) ) :
			$data['add_tags_sub_active']                     = [];
			$data['remove_tags_sub_active']                  = [];
			$data['add_to_lists_sub_active']                 = [];
			$data['remove_from_lists_sub_active']            = [];
			$data['add_to_automations_sub_active']           = [];
			$data['remove_from_automations_sub_active']      = [];

			$data['add_tags_sub_cancelled']                  = [];
			$data['remove_tags_sub_cancelled']               = [];
			$data['add_to_lists_sub_cancelled']              = [];
			$data['remove_from_lists_sub_cancelled']         = [];
			$data['add_to_automations_sub_cancelled']        = [];
			$data['remove_from_automations_sub_cancelled']   = [];

			$data['add_tags_sub_onhold']                     = [];
			$data['remove_tags_sub_onhold']                  = [];
			$data['add_to_lists_sub_onhold']                 = [];
			$data['remove_from_lists_sub_onhold']            = [];
			$data['add_to_automations_sub_onhold']           = [];
			$data['remove_from_automations_sub_onhold']      = [];

			$data['add_tags_sub_reactivated']                = [];
			$data['remove_tags_sub_reactivated']             = [];
			$data['add_to_lists_sub_reactivated']            = [];
			$data['remove_from_lists_sub_reactivated']       = [];
			$data['add_to_automations_sub_reactivated']      = [];
			$data['remove_from_automations_sub_reactivated'] = [];

			$data['add_tags_sub_expired']                    = [];
			$data['remove_tags_sub_expired']                 = [];
			$data['add_to_lists_sub_expired']                = [];
			$data['remove_from_lists_sub_expired']           = [];
			$data['add_to_automations_sub_expired']          = [];
			$data['remove_from_automations_sub_expired']     = [];

			$data['add_tags_sub_refunded']                   = [];
			$data['remove_tags_sub_refunded']                = [];
			$data['add_to_lists_sub_refunded']               = [];
			$data['remove_from_lists_sub_refunded']          = [];
			$data['add_to_automations_sub_refunded']         = [];
			$data['remove_from_automations_sub_refunded']    = [];
		ENDIF;

		$new_data_tags  = [];
		$new_data_price = [];

		IF ( ! empty( $this->wc_ext_options['number_alt_prices'] ) ) :
			$z = 1; 
			FOREACH ( $data['alt_price_tags'] AS $x => $filler ) :
				IF ( ! empty( $data['alt_price_tags'][ $x ] ) ) :
					IF ( ! empty($data['alt_price'][ $x ] ) ) :

						$new_data_tags[ $z ]  = $data['alt_price_tags'][ $x ];
						$new_data_price[ $z ] = $data['alt_price'][ $x ];
						$z++;
					ENDIF;
				ENDIF;
			ENDFOREACH;
		ENDIF;

		$data['alt_price_tags'] = $new_data_tags;
		$data['alt_price']      = $new_data_price;

		
		update_post_meta( $post_id, $this->product_data_key, json_encode( $data ) );
	}

	public function woocommerce_integrations( $integrations )
	{
		array_unshift( $integrations, 'mbr_wc_integration' );
		return $integrations;
	}

	public function woocommerce_subscription_status_updated( $subscription, $new_status = NULL, $old_status = NULL )
	{
		$add_tags = $remove_tags = $add_to_automations = $remove_from_automations = $list_contact = [];

		$order_items = $subscription->get_items( [ 'line_item' ] );

		
		$arr_subscription_relevant_statuses = [ 'wc-on-hold' => 'onhold', 'wc-cancelled' => 'cancelled', 'wc-expired' => 'expired', 'wc-active' => 'active' ];
		$arr_subscription_types             = [ 'subscription', 'subscription_variation' ];

		IF ( array_key_exists( 'wc-' . $subscription->get_status(), $arr_subscription_relevant_statuses ) ) :
			$action_suffix   = $arr_subscription_relevant_statuses[ 'wc-' . $subscription->get_status() ];
			IF ( 'wc-' . $subscription->get_status() == 'wc-active' ) :
				IF ( $old_status == 'on-hold' ) :
					$action_suffix = 'reactivated';
				ENDIF;
			ENDIF;

			FOREACH ( $order_items AS $k => $order_item ) :

				IF ( $product_data = get_post_meta( $order_item['product_id'], $this->product_data_key, TRUE ) ) :
					$product_data = json_decode( $product_data, TRUE );
				ENDIF;

				$product = wc_get_product( empty( $order_item['variation_id'] ) ? $order_item['product_id'] : $order_item['variation_id'] );

				IF ( $product_data ) :

					IF ( ! empty( $product ) AND in_array( $product->get_type(), $arr_subscription_types ) ) :

						IF ( $this->wc_ext_options['enable_tags_in_panels'] == 'yes' ) :
							IF ( ! empty( $product_data['remove_tags_sub_' . $action_suffix] ) ) :
								$remove_tags = array_merge( $remove_tags, explode( ',', apply_filters( 'mbr/tags/normalize/names', $product_data['remove_tags_sub_' . $action_suffix] ) ) );
							ENDIF;
							IF ( ! empty( $product_data['add_tags_sub_' . $action_suffix] ) ) :
								$add_tags    = array_merge( $add_tags, explode( ',', apply_filters( 'mbr/tags/normalize/names', $product_data['add_tags_sub_' . $action_suffix] ) ) );
							ENDIF;
						ENDIF;

						IF ( $this->wc_ext_options['enable_lists_in_panels'] == 'yes' ) :
							IF ( ! empty( $product_data['add_to_lists_sub_' . $action_suffix] ) ) :
								FOREACH ( $product_data['add_to_lists_sub_' . $action_suffix] AS $k => $list ) :
									$list_contact["p[{$list}]"]      = $list;
									$list_contact["status[{$list}]"] = 1;
								ENDFOREACH;
							ENDIF;
							IF ( ! empty( $product_data['remove_from_lists_sub_' . $action_suffix] ) ) :
								FOREACH ( $product_data['remove_from_lists_sub_' . $action_suffix] AS $k => $list ) :
									$list_contact["p[{$list}]"]      = $list;
									$list_contact["status[{$list}]"] = 2;
								ENDFOREACH;
							ENDIF;
						ENDIF;

						IF ( $this->wc_ext_options['enable_automations_in_panels'] == 'yes' ) :
							IF ( ! empty( $product_data['remove_from_automations_sub_' . $action_suffix] ) ) :
								$remove_from_automations = array_merge( $remove_from_automations, $product_data['remove_from_automations_sub_' . $action_suffix] );
							ENDIF;
							IF ( ! empty( $product_data['add_to_automations_sub_' . $action_suffix] ) ) :
								$add_to_automations = array_merge( $add_to_automations, $product_data['add_to_automations_sub_' . $action_suffix] );
							ENDIF;
						ENDIF;

					ENDIF;

				ENDIF;

			ENDFOREACH;
		ENDIF;

		$parms['full_sync']               = TRUE;
		$parms['add_tags']                = $add_tags;
		$parms['remove_tags']             = $remove_tags;
		$parms['add_to_automations']      = $add_to_automations;
		$parms['remove_from_automations'] = $remove_from_automations;
		$parms['list_contact']            = $list_contact;
		$parms['add_note']                = FALSE;

		$this->woocommerce_order_contact_sync( $subscription, $parms );
	}

	
	public function woocommerce_order_status_changed( $order_id, $old_status = NULL, $new_status = NULL )
	{
		IF ( ! $order = new WC_Order( $order_id ) ) :
			return;
		ENDIF;

		do_action( 'order_status_event', $order->get_billing_email(), 'wc-' . $order->get_status(), $order_id );

		IF ( $this->wc_ext_options['update_on_order_status_changed'] == 'yes' ) :
			
		ELSEIF ( in_array( 'wc-' . $order->get_status(), [ 'wc-completed', 'wc-processing', 'wc-refunded' ] ) ) :
			
		ELSE:
			return;
		ENDIF;

		$add_tags = $remove_tags  = [];

		$order_status_tag         = apply_filters( 'mbr/tags/normalize/names', $this->wc_ext_options['order_status_tag'] );
		IF ( ! empty( $order_status_tag ) ) :
			FOREACH ( $this->wc_ext_valid_status AS $loop_status => $filler ) :
				IF ( $loop_status == 'wc-' . $order->get_status() ) :
					$add_tags[]         = $order_status_tag . '_' . $order->get_status();
				ELSE :
					$remove_tags[]      = $order_status_tag . '_' . substr( $loop_status,3 );
				ENDIF;
			ENDFOREACH;
		ENDIF;

		IF ( ! in_array( 'wc-' . $order->get_status(), [ 'wc-completed', 'wc-processing', 'wc-refunded' ] ) OR ( $new_status == 'completed' AND $old_status == 'processing' ) ) :
			$parms                  = [];
			$parms['full_sync']     = FALSE;
			$parms['add_tags']      = $add_tags;
			$parms['remove_tags']   = $remove_tags;
		ELSEIF ( 'wc-' . $order->get_status() == 'wc-refunded' ) :
			$parms                  = $this->woocommerce_order_refunded($order);
			$parms['full_sync']     = TRUE;
			$parms['add_tags']      = array_merge( $add_tags, $parms['add_tags'] );
			$parms['remove_tags']   = array_merge( $remove_tags, $parms['remove_tags'] );
		ELSE :
			$parms                  = $this->woocommerce_payment_complete( $order );
			$parms['full_sync']     = TRUE;
			$parms['add_tags']      = array_merge( $add_tags, $parms['add_tags'] );
			$parms['remove_tags']   = array_merge( $remove_tags, $parms['remove_tags'] );
		ENDIF;

		$this->woocommerce_order_contact_sync( $order, $parms );
	}

	public function woocommerce_order_refunded( $order )
	{
		$add_tags                = [];
		$remove_tags             = [];
		$add_to_automations      = [];
		$remove_from_automations = [];
		$list_contact            = [];

		$order_items             = $order->get_items(['line_item']);

		FOREACH ( $order_items AS $k => $order_item ) :

			IF ( $order_item['type'] <> 'line_item' ) :
				continue;
			ENDIF;

			IF ( $product_data = get_post_meta($order_item['product_id'], $this->product_data_key, TRUE ) ) :
				$product_data = json_decode( $product_data, TRUE );
			ENDIF;

			IF ( $product_data ) :
				IF ( $this->wc_ext_options['enable_tags_in_panels'] == 'yes' ) :
					IF ( ! empty( $product_data['remove_tags_refunded'] ) ) :
						$remove_tags = array_merge( $remove_tags, explode( ',', apply_filters( 'mbr/tags/normalize/names', $product_data['remove_tags_refunded'] ) ) );
					ENDIF;
					IF ( ! empty($product_data['assign_tags_refunded'] ) ) :
						$add_tags = array_merge( $add_tags, explode( ',', apply_filters( 'mbr/tags/normalize/names', $product_data['assign_tags_refunded'] ) ) );
					ENDIF;
				ENDIF;

				IF ( $this->wc_ext_options['enable_lists_in_panels'] == 'yes' ) :
					IF ( ! empty($product_data['add_to_lists_refunded'] ) ) :
						FOREACH ( $product_data['add_to_lists_refunded'] AS $k => $list ) :
							$list_contact["p[{$list}]"]      = $list;
							$list_contact["status[{$list}]"] = 1;
						ENDFOREACH;
					ENDIF;
					IF ( ! empty($product_data['remove_from_lists_refunded'] ) ) :
						FOREACH ($product_data['remove_from_lists_refunded'] AS $k => $list) :
							$list_contact["p[{$list}]"]      = $list;
							$list_contact["status[{$list}]"] = 2;
						ENDFOREACH;
					ENDIF;
				ENDIF;

				IF ( $this->wc_ext_options['enable_automations_in_panels'] == 'yes' ) :
					IF ( ! empty( $product_data['remove_from_automations_refunded'] ) ) :
						$remove_from_automations = array_merge( $remove_from_automations, $product_data['remove_from_automations_refunded'] );
					ENDIF;
					IF ( ! empty( $product_data['add_to_automations_refunded'] ) ) :
						$add_to_automations = array_merge( $add_to_automations, $product_data['add_to_automations_refunded'] );
					ENDIF;
				ENDIF;

				$product = wc_get_product($order_item['product_id']);
				IF ( in_array( $product->get_type(), [ 'subscription', 'variable-subscription' ] ) ) :
					IF ( $this->wc_ext_options['enable_tags_in_panels'] == 'yes' ) :
						IF ( ! empty( $product_data['remove_tags_sub_refunded'] ) ) :
							$remove_tags = array_merge( $remove_tags, explode( ',', apply_filters( 'mbr/tags/normalize/names', $product_data['remove_tags_sub_refunded'] ) ) );
						ENDIF;
						IF ( ! empty( $product_data['add_tags_sub_refunded'] ) ) :
							$add_tags = array_merge( $add_tags, explode( ',', apply_filters( 'mbr/tags/normalize/names', $product_data['add_tags_sub_refunded'] ) ) );
						ENDIF;
					ENDIF;

					IF ( $this->wc_ext_options['enable_lists_in_panels'] == 'yes' ) :
						IF ( ! empty( $product_data['add_to_lists_sub_refunded'] ) ) :
							FOREACH ( $product_data['add_to_lists_sub_refunded'] AS $k => $list ) :
								$list_contact["p[{$list}]"]      = $list;
								$list_contact["status[{$list}]"] = 1;
							ENDFOREACH;
						ENDIF;
						IF ( ! empty( $product_data['remove_from_lists_sub_refunded'] ) ) :
							FOREACH ( $product_data['remove_from_lists_sub_refunded'] AS $k => $list ) :
								$list_contact["p[{$list}]"]      = $list;
								$list_contact["status[{$list}]"] = 2;
							ENDFOREACH;
						ENDIF;
					ENDIF;

					IF ( $this->wc_ext_options['enable_automations_in_panels'] == 'yes' ) :
						IF ( ! empty( $product_data['remove_from_automations_sub_refunded'] ) ) :
							$remove_from_automations = array_merge( $remove_from_automations, $product_data['remove_from_automations_sub_refunded'] );
						ENDIF;
						IF ( ! empty( $product_data['add_to_automations_sub_refunded'] ) ) :
							$add_to_automations      = array_merge( $add_to_automations, $product_data['add_to_automations_sub_refunded'] );
						ENDIF;
					ENDIF;
				ENDIF;

			ENDIF;

		ENDFOREACH;

		return [ 'list_contact' => $list_contact, 'remove_tags' =>$remove_tags, 'add_tags' => $add_tags, 'add_to_automations' => $add_to_automations, 'remove_from_automations' => $remove_from_automations ];
	}

	public function woocommerce_payment_complete( $order )
	{
		$on_order_add_tags                = apply_filters( 'mbr/tags/normalize/names', $this->wc_ext_options['on_order_add_tags'] );
		$on_order_remove_tags             = apply_filters( 'mbr/tags/normalize/names', $this->wc_ext_options['on_order_remove_tags'] );

		$on_order_add_to_lists            = $this->wc_ext_options['on_order_add_to_lists'];
		$on_order_remove_from_lists       = $this->wc_ext_options['on_order_remove_from_lists'];
		$on_order_add_to_automations      = $this->wc_ext_options['on_order_add_to_automations'];
		$on_order_remove_from_automations = $this->wc_ext_options['on_order_remove_from_automations'];

		$add_tags                = [];
		$remove_tags             = [];
		$add_to_automations      = [];
		$remove_from_automations = [];
		$list_contact            = [];

		IF ( ! empty( $on_order_add_tags ) ) :
			$add_tags    = array_merge( $add_tags, explode( ',', $on_order_add_tags ) );
		ENDIF;

		IF ( ! empty( $on_order_remove_tags ) ) :
			$remove_tags = array_merge( $remove_tags, explode( ',', $on_order_remove_tags ) );
		ENDIF;

		IF ( ! empty( $on_order_remove_from_lists ) ) :
			FOREACH ( $on_order_remove_from_lists AS $k => $list ) :
				$list_contact["p[{$list}]"]      = $list;
				$list_contact["status[{$list}]"] = 2;
			ENDFOREACH;
		ENDIF;

		IF ( ! empty( $on_order_add_to_lists ) ) :
			FOREACH ( $on_order_add_to_lists AS $k => $list ) :
				$list_contact["p[{$list}]"]      = $list;
				$list_contact["status[{$list}]"] = 1;
			ENDFOREACH;
		ENDIF;

		IF ( $on_order_add_to_automations ) :
			$add_to_automations      = $on_order_add_to_automations;
		ENDIF;
		IF ( $on_order_remove_from_automations ) :
			$remove_from_automations = $on_order_remove_from_automations;
		ENDIF;

		$order_items = $order->get_items( [ 'line_item', 'coupon' ] );

		FOREACH ($order_items AS $k => $order_item) :

			IF ($order_item['type'] <> 'line_item') :
				continue;
			ENDIF;

			IF ($product_data = get_post_meta( $order_item['product_id'], $this->product_data_key, TRUE ) ) :
				$product_data = json_decode( $product_data, TRUE );
			ENDIF;

			IF ($product_data) :

				$product = wc_get_product( empty( $order_item['variation_id']) ? $order_item['product_id'] : $order_item['variation_id'] );

				IF ( ! empty( $product_data['sku_tag_name'] ) AND $product_data['sku_tag_name'] == 'yes' ) :
					IF ( $sku = $product->get_sku() ) :
						$add_tags[] = $sku;
					ENDIF;
				ENDIF;

				IF ( ! empty( $product_data['product_tag_names'] ) AND $product_data['product_tag_names'] == 'yes' ) :
					$product_tags = get_the_terms( $order_item['product_id'], 'product_tag' );
					IF ( ! empty($product_tags ) ) :
						FOREACH ( $product_tags AS $filler => $obj_tag ) :
							$add_tags[] = $obj_tag->name;
						ENDFOREACH;
					ENDIF;
				ENDIF;

				IF ( ! empty( $product_data['product_cat_names'] ) AND $product_data['product_cat_names'] == 'yes' ) :
					$product_cats = get_the_terms( $order_item['product_id'], 'product_cat' );
					IF ( ! empty( $product_cats ) ) :
						FOREACH ( $product_cats AS $filler => $obj_tag ) :
							$add_tags[] = $obj_tag->name;
						ENDFOREACH;
					ENDIF;
				ENDIF;

				IF ( $this->wc_ext_options['enable_tags_in_panels'] == 'yes' ) :
					IF ( ! empty( $product_data['remove_tags'] ) ) :
						$remove_tags = array_merge( $remove_tags, explode( ',', apply_filters( 'mbr/tags/normalize/names', $product_data['remove_tags'] ) ) );
					ENDIF;

					IF ( ! empty( $product_data['assign_tags'] ) ) :
						$add_tags = array_merge( $add_tags, explode( ',', apply_filters( 'mbr/tags/normalize/names', $product_data['assign_tags'] ) ) );
					ENDIF;

					IF ( ! empty( $order_item['variation_id'] ) ) :
						IF ( $product_data_variation = get_post_meta( $order_item['variation_id'], $this->product_data_key, TRUE ) ) :
							$product_data_variation = json_decode( $product_data_variation, TRUE );
						ENDIF;

						IF ( $product_data_variation ) :
							IF ( ! empty( $product_data_variation['remove_tags'] ) ) :
								$remove_tags = array_merge( $remove_tags, explode( ',', apply_filters( 'mbr/tags/normalize/names', $product_data_variation['remove_tags'] ) ) );
							ENDIF;
							IF ( ! empty( $product_data_variation['assign_tags'] ) ) :
								$add_tags    = array_merge( $add_tags, explode( ',', apply_filters( 'mbr/tags/normalize/names', $product_data_variation['assign_tags'] ) ) );
							ENDIF;
						ENDIF;
					ENDIF;
				ENDIF;

				IF ( $this->wc_ext_options['enable_lists_in_panels'] == 'yes' ) :
					IF ( ! empty( $product_data['add_to_lists'] ) ) :
						FOREACH ( $product_data['add_to_lists'] AS $k => $list ) :
							$list_contact["p[{$list}]"]      = $list;
							$list_contact["status[{$list}]"] = 1;
						ENDFOREACH;
					ENDIF;
					IF ( ! empty($product_data['remove_from_lists'] ) ) :
						FOREACH ( $product_data['remove_from_lists'] AS $k => $list ) :
							$list_contact["p[{$list}]"]      = $list;
							$list_contact["status[{$list}]"] = 2;
						ENDFOREACH;
					ENDIF;
				ENDIF;

				IF ( $this->wc_ext_options['enable_automations_in_panels'] == 'yes' ) :
					IF ( ! empty( $product_data['remove_from_automations'] ) ) :
						$remove_from_automations = array_merge( $remove_from_automations, $product_data['remove_from_automations'] );
					ENDIF;
					IF ( ! empty( $product_data['add_to_automations'] ) ) :
						$add_to_automations      = array_merge( $add_to_automations, $product_data['add_to_automations'] );
					ENDIF;
				ENDIF;
			ENDIF;

		ENDFOREACH;

		
		$order_coupons = $order->get_used_coupons();

		FOREACH ( $order_coupons AS $k => $coupon_code ) :

			IF ( ! $coupon = new WC_Coupon( $coupon_code ) ) :
				continue;
			ENDIF;

			IF ($coupon_data = get_post_meta( $coupon->get_id(), $this->coupon_data_key, TRUE ) ) :
				$coupon_data = json_decode( $coupon_data, TRUE );
			ENDIF;
			IF (!$coupon_data) :
				continue;
			ENDIF;

			IF ( ! empty( $coupon_data['coupon_tag_name'] ) AND $coupon_data['coupon_tag_name'] == 'yes' ) :
				$add_tags[] = $coupon->get_code();
			ENDIF;

			IF ( $this->wc_ext_options['enable_tags_in_panels'] == 'yes' ) :
				IF ( ! empty($coupon_data['remove_tags'] ) ) :
					$remove_tags = array_merge( $remove_tags, explode( ',', apply_filters( 'mbr/tags/normalize/names', $coupon_data['remove_tags'] ) ) );
				ENDIF;
				IF (!empty($coupon_data['assign_tags'])) :
					$add_tags    = array_merge( $add_tags, explode( ',', apply_filters( 'mbr/tags/normalize/names', $coupon_data['assign_tags'] ) ) );
				ENDIF;
			ENDIF;

			IF ( $this->wc_ext_options['enable_lists_in_panels'] == 'yes' ) :
				IF ( ! empty( $coupon_data['add_to_lists'] ) ) :
					FOREACH ( $coupon_data['add_to_lists'] AS $k => $list ) :
						$list_contact["p[{$list}]"]      = $list;
						$list_contact["status[{$list}]"] = 1;
					ENDFOREACH;
				ENDIF;
				IF ( ! empty( $coupon_data['remove_from_lists'] ) ) :
					FOREACH ( $coupon_data['remove_from_lists'] AS $k => $list ) :
						$list_contact["p[{$list}]"]      = $list;
						$list_contact["status[{$list}]"] = 2;
					ENDFOREACH;
				ENDIF;
			ENDIF;

			IF ( $this->wc_ext_options['enable_automations_in_panels'] == 'yes' ) :
				IF ( ! empty( $coupon_data['remove_from_automations'] ) ) :
					$remove_from_automations = array_merge( $remove_from_automations, $coupon_data['remove_from_automations'] );
				ENDIF;
				IF ( ! empty( $coupon_data['add_to_automations'] ) ) :
					$add_to_automations      = array_merge( $add_to_automations, $coupon_data['add_to_automations'] );
				ENDIF;
			ENDIF;

		ENDFOREACH;

		return [ 'list_contact' => $list_contact, 'remove_tags' => $remove_tags, 'add_tags' => $add_tags, 'add_to_automations' => $add_to_automations, 'remove_from_automations' => $remove_from_automations ];
	}

	
	public function woocommerce_order_contact_sync( $order, $parms )
	{
		$parms = shortcode_atts( [ 'full_sync' => FALSE, 'add_note' => TRUE, 'list_contact' => [], 'remove_tags' => [], 'add_tags' => [], 'add_to_automations' => [], 'remove_from_automations' => [] ], $parms );
		extract($parms);

		$logging_enabled = $this->wc_ext_options['debug_log'];
		IF ( $logging_enabled <> 'yes' ) :
			$logging_enabled = FALSE;
		ELSEIF ( $log = new WC_Logger() ) :
			$logging_enabled = TRUE;
		ELSE :
			$logging_enabled = FALSE;
		ENDIF;

		IF ( ! is_email( $order->get_billing_email() ) ) :
			$order->set_billing_email( 'unknown_' . $order->get_id() . '@' . MBR_DOMAIN );
			$order->save();
			IF ( $logging_enabled ) :
				$log->add( 'activemember360', sprintf( __( 'Error: Invalid customer (billing) email address = %s in order # %s', 'activemember360' ), $order->get_billing_email(), $order->get_id() ) );
			ENDIF;
		ENDIF;

		$contact = [
			'email'              => $order->get_billing_email(),
			'phone'              => $order->get_billing_phone(),
			'first_name'         => $order->get_billing_first_name(),
			'last_name'          => $order->get_billing_last_name(),
			'field'              =>  [
				'%billing_first_name%' => $order->get_billing_first_name(),
				'%billing_last_name%'  => $order->get_billing_last_name(),
				'%billing_company%'    => $order->get_billing_company(),
				'%billing_email%'      => $order->get_billing_email(),
				'%billing_address_1%'  => $order->get_billing_address_1(),
				'%billing_address_2%'  => $order->get_billing_address_2(),
				'%billing_city%'       => $order->get_billing_city(),
				'%billing_state%'      => $order->get_billing_state(),
				'%billing_postcode%'   => $order->get_billing_postcode(),
				'%billing_country%'    => $order->get_billing_country(),
				'%billing_phone%'      => $order->get_billing_phone()
			]
		];

		IF ( ! empty( $list_contact ) AND is_array( $list_contact ) ) :
			$contact = $contact + $list_contact;
		ENDIF;

		$contact_id     = apply_filters( 'mbr/contact_id', NULL );
		$session        = apply_filters( 'mbr/session/get', NULL );

		IF ( $contact_id AND $session['email'] == $order->get_billing_email() ) : 
			
			$contact['email']        = $session['email'];
			IF ( ! empty( $session['first_name'] ) ) :
				$contact['first_name'] = $session['first_name'];
			ENDIF;
			IF ( ! empty( $session['last_name'] ) ) :
				$contact['last_name']  = $session['last_name'];
			ENDIF;
			IF ( ! empty( $session['phone'] ) ) :
				$contact['phone']      = $session['phone'];
			ENDIF;
		ELSE :
			



			$local_contact = apply_filters( 'mbr/fetch_contact', [ 'email' => $order->get_billing_email(), 'local' => TRUE ], FALSE );
			IF ( ! empty( $local_contact['id'] ) ) :
				IF ( ! empty( $local_contact['email'] ) ) :
					$contact['email']      = $local_contact['email'];
				ENDIF;
				IF ( ! empty( $local_contact['first_name'] ) ) :
					$contact['first_name'] = $local_contact['first_name'];
				ENDIF;
				IF ( ! empty($local_contact['last_name'] ) ) :
					$contact['last_name']  = $local_contact['last_name'];
				ENDIF;
				IF ( ! empty( $local_contact['phone'] ) ) :
					$contact['phone']      = $local_contact['phone'];
				ENDIF;
			ENDIF;
		ENDIF;

		$contact_sync = apply_filters( 'mbr/api', 'contact_sync', $contact );

		IF ( (int) $contact_sync->result_code == 1 ) :

			$contact_id = (int) $contact_sync->subscriber_id;
			
			IF (!empty($remove_tags)) :
				apply_filters( 'mbr/api', 'contact_tag_remove', [ 'id' => $contact_id, 'tags' => $parms['remove_tags'] ] );
			ENDIF;
			
			IF ( ! empty( $add_tags ) ) :
				apply_filters( 'mbr/api', 'contact_tag_add', [ 'id' => $contact_id, 'tags' => $parms['add_tags'] ] );
			ENDIF;

			IF ( $parms['full_sync'] ) :
				
				$remove_automations = implode( ',', array_unique( array_filter( $parms['remove_from_automations'] ) ) );
				IF ( $remove_automations ) :
					apply_filters( 'mbr/api', 'automation_contact_remove', [ 'contact_id' => $contact_id, 'automation' => $remove_automations ] );
				ENDIF;

				
				$add_automations    = implode( ',', array_unique( array_filter( $parms['add_to_automations'] ) ) );
				IF ( $add_automations ) :
					apply_filters( 'mbr/api', 'automation_contact_add', [ 'contact_id' => $contact_id, 'automation' => $add_automations ] );
				ENDIF;

				
				IF ( $parms['add_note'] ) :
					$current_order_id = $order->get_id();
					apply_filters( 'mbr/api', 'contact_note_add',
						[
							'id'   => $contact_id,
							'note' => sprintf(
								"Order # %s \nStatus: %s \nTotal: %s \n%s",
								$current_order_id,
								$order->get_status(),
								strip_tags( html_entity_decode( $order->get_formatted_order_total( '', FALSE ) ) ),
								admin_url( "post.php?post={$current_order_id}&action=edit" )
							)
						]
					);
					$account_info = apply_filters( 'mbr/ac_account/get', NULL );
					$order->add_order_note( __('ActiveCampaign Sync: ', 'activemember360' ) . "<a href='https://{$account_info['account']}/contact/{$contact_id}' target='_blank'>" . __( 'Click here', 'activemember360' ) . '</a>' );
				ENDIF;
			ELSE :
				IF ( 'wc-' . $order->get_status() == 'wc-completed' AND $parms['add_note'] ) :
					$current_order_id = $order->get_id();
					
					apply_filters( 'mbr/api', 'contact_note_add',
						[
							'id'   => $contact_id,
							'note' => sprintf(
								"Order # %s \nStatus: %s \nTotal: %s \n%s" ,
								$current_order_id,
								$order->get_status(),
								strip_tags( html_entity_decode( $order->get_formatted_order_total( '', FALSE ) ) ),
								admin_url("post.php?post={$current_order_id}&action=edit")
							)
						]
					);
				ENDIF;
			ENDIF;
			$log_message = sprintf( __( 'Contact was successfully added/updated (ID = %s)', 'activemember360' ), $contact_id );
		ELSE :
			$log_message = sprintf( __( 'Contact could not be added/updated. Error: %s)', 'activemember360' ), $contact_sync->error );
		ENDIF;

		IF ( $logging_enabled ) :
			$log->add( 'activemember360', $log_message );
		ENDIF;
	}

	public function woocommerce_checkout_order_processed( $order_id, $data )
	{
		global $current_user;

		$user_email = is_user_logged_in() ? $current_user->user_email : $data['billing_email'];

		$contact    = [
			'email'                  => $user_email,
			'phone'                  => $data['billing_phone'],
			'first_name'             => $data['billing_first_name'],
			'last_name'              => $data['billing_last_name'],
			'field'                  =>  [
				'%billing_first_name%' => $data['billing_first_name'],
				'%billing_last_name%'  => $data['billing_last_name'],
				'%billing_company%'    => $data['billing_company'],
				'%billing_email%'      => $data['billing_email'],
				'%billing_address_1%'  => $data['billing_address_1'],
				'%billing_address_2%'  => $data['billing_address_2'],
				'%billing_city%'       => $data['billing_city'],
				'%billing_state%'      => $data['billing_state'],
				'%billing_postcode%'   => $data['billing_postcode'],
				'%billing_country%'    => $data['billing_country'],
				'%billing_phone%'      => $data['billing_phone']
			]
		];

		apply_filters( 'mbr/api', 'contact_sync', $contact );

		


		IF (is_user_logged_in()) :
			IF ( $contact = apply_filters( 'mbr/fetch_contact', [ 'email' => $current_user->user_email, 'local' => FALSE ], TRUE ) ) :
				do_action( 'mbr/create_user_cookie', $contact['hash'] );
			ENDIF;
		ENDIF;
	}

	public function woocommerce_registration_auth_new_customer( $true, $user_id )
	{
		


		IF ( $contact = apply_filters( 'mbr/fetch_contact', [ 'email' =>  $_POST['email'], 'local' => FALSE ], TRUE ) ) :
			do_action( 'mbr/create_user_cookie', $contact['hash'] );
		ENDIF;

		return $true;
	}

	public function woocommerce_checkout_get_value( $null, $input )
	{
		global $current_user;

		
		IF ( is_user_logged_in() ) :
			
			$fm = [
				0 => [ $input => $input, 'billing_first_name' => 'first_name', 'billing_last_name' => 'last_name', 'billing_email' => 'email' ],
				1 => [ $input => $input, 'billing_phone'      => 'phone',      'billing_company'   => 'orgname' ]
			];

			$session = apply_filters( 'mbr/session/get', NULL );

			FOREACH ( $fm AS $k => $fld ) :
				IF ( ! empty( $session[ $fld[ $input ] ] ) ) :
					return $session[ $fld[ $input ] ];
				ENDIF;
			ENDFOREACH;
			FOREACH ( $fm AS $k => $fld ) :
				IF ( ! empty( $current_user->$fld[ $input ] ) ) :
					return $current_user->$fld[ $input ];
				ENDIF;
			ENDFOREACH;
		ENDIF;

		
		return $null;
	}

	public function woocommerce_created_customer( $customer_id, $user_data, $password_generated )
	{
		$contact['email']             = $user_data['user_email'];

		$password_perstag             = apply_filters( 'mbr/password_perstag', NULL );

		$contact[ $password_perstag ] = apply_filters( 'mbr/password/hash', $user_data['user_pass'] );
		IF ( substr( $contact[ $password_perstag ],0,4) <> '$P$B' ) :
			wp_set_password( $contact[ $password_perstag ], $customer_id );
		ENDIF;

		apply_filters( 'mbr/api', 'contact_sync', $contact );

		do_action( 'user_register_login', $user_data['user_email'] );
	}

	
	public function woocommerce_new_customer_data( $user_data = NULL )
	{
		$contact = apply_filters( 'mbr/fetch_contact', [ 'email' => $user_data['user_email'], 'local' => FALSE ], FALSE );

		IF ( $user = get_user_by( 'email', $user_data['user_email'] ) OR $user = get_user_by( 'login', $user_data['user_login'] ) ) :
			// do nothing, he must log in manually!!!!!
		ELSE:
			$user_data['user_login']    = $user_data['user_email'];
			IF (!empty($contact['id'])) :
				$password_field           = apply_filters( 'mbr/passfield', NULL );
				IF ( ! empty( $contact[ $password_field ] ) ) :
					$user_data['user_pass'] = $contact[ $password_field ];
				ENDIF;
				$user_id                  = wp_insert_user( $user_data );
				$user_data['ID']          = $user_id;
			ENDIF;
		ENDIF;

		return $user_data;
	}

	public function check_password( $check, $password )
	{
		global $MBR_OPTIONS;

		IF ( $MBR_OPTIONS['mbr_encrypted_passwords'] ) :
			return $check;
		ENDIF;

		$contact_id       = apply_filters( 'mbr/contact_id', NULL );
		IF ( $contact_id ) :
			$password_field = apply_filters( 'mbr/passfield', NULL );
			$contact_record = apply_filters( 'mbr/fetch_remote_contact/by_id', $contact_id );
			return ( $password == $contact_record[ $password_field ] );
		ENDIF;

		return $check;
	}

	public function woocommerce_save_account_details_errors( &$errors, $user )
	{
		global $MBR_OPTIONS;

		$session = apply_filters( 'mbr/session/get', NULL );

		$_POST['account_email'] = lower_trim( $_POST['account_email'] );
		IF ( ! empty( $_POST['account_email'] ) AND $_POST['account_email'] <> lower_trim( $session['email'] ) ) :
			IF ( $obj_contact = apply_filters( 'mbr/api', "contact_view_email?email={$_POST['account_email']}") AND (int) $obj_contact->result_code ) :
				$errors->add( 'error', __( 'The email address entered already exists. Please use a different one.', 'activemember360' ) );
			ENDIF;
		ENDIF;

		IF ( ! $MBR_OPTIONS['mbr_encrypted_passwords'] ) :
			IF ( ! empty( $_POST['password_1'] ) ) :
				$pass_len = strlen( $_POST['password_1'] );
				IF ( $pass_len < $MBR_OPTIONS['min_passlen'] ) :
					$errors->add( 'error', sprintf( __( 'Your password is too short. It must be at least %s characters long.','activemember360' ), $MBR_OPTIONS['min_passlen'] ) );
				ELSEIF ( $pass_len > $MBR_OPTIONS['max_passlen'] ) :
					$errors->add('error', sprintf( __( 'Your password is too long. It may not be more than %s characters long.','activemember360' ), $MBR_OPTIONS['max_passlen'] ) );
				ENDIF;

				$password_base = apply_filters( 'mbr/password/char_base', NULL );
				IF ( trim( $password_base) > '' ) :
					FOR ( $i = 0; $i < $pass_len; $i++ ) :
						IF ( FALSE === stripos( $password_base, $_POST['password_1'][ $i ] ) ) :
							$errors->add( 'error', __( 'Your password includes characters that are not allowed.<br />Only the following characters may be used:<br />', 'activemember360' ) . $password_base );
							break;
						ENDIF;
					ENDFOR;
				ENDIF;
			ENDIF;
		ENDIF;
	}

	public function woocommerce_save_account_details( $user_id )
	{
		global $MBR_OPTIONS, $wpdb;

		IF ( $contact_id = apply_filters( 'mbr/contact_id', NULL ) ) :

			$session = apply_filters( 'mbr/session/get', NULL );
			$_POST['account_first_name'] = trim( $_POST['account_first_name'] );
			IF ( ! empty( $_POST['account_first_name'] ) AND $_POST['account_first_name'] <> trim( $session['first_name'] ) ) :
				$contact['first_name'] = $session['first_name'] = $_POST['account_first_name'];
			ENDIF;

			$_POST['account_last_name'] = trim( $_POST['account_last_name'] );
			IF ( ! empty( $_POST['account_last_name'] ) AND $_POST['account_last_name'] <> trim( $session['last_name'] ) ) :
				$contact['last_name']  = $session['last_name']  = $_POST['account_last_name'];
			ENDIF;

			$_POST['password_1'] = trim( $_POST['password_1'] );
			$password_field           = apply_filters( 'mbr/passfield', NULL );
			$password_perstag         = apply_filters( 'mbr/password_perstag', NULL );
			IF ( ! empty( $_POST['password_1'] ) AND $_POST['password_1'] <> trim( $session[ $password_field] ) ) :
				
				IF ( ! $MBR_OPTIONS['mbr_encrypted_passwords'] ) :
					$contact[ $password_perstag ] = $session[ $password_field ]  = $_POST['password_1'];
				ENDIF;
			ENDIF;

			IF ( ! empty( $contact ) ) :
				$contact['id'] = $contact_id;
				apply_filters( 'mbr/save_contact_fields', $contact );
			ENDIF;

			$_POST['account_email'] = lower_trim( $_POST['account_email'] );
			IF ( ! empty($_POST['account_email'] ) AND $_POST['account_email'] <> lower_trim( $session['email'] ) ) :
				$user = get_user_by( 'ID', $user_id );

				IF ( apply_filters( 'mbr/save_contact_fields', [ 'id' => $contact_id, 'email' => $_POST['account_email'] ] ) ) :
					apply_filters( 'mbr/fetch_contact', [ 'id' => $contact_id, 'local' => FALSE, 'sync' => FALSE ] );

					do_action( 'mbr/update_cookie',      $user_id, $_POST['account_email'] . '|' . date( 'YmdHis' ) );
					do_action( 'mbr/delete_cookie',      MBR_COOKIE_NAME );
					do_action( 'mbr/create_user_cookie', $session['hash'] );

					IF ( is_email( $user->user_login ) ) :
						$wpdb->update($wpdb->users, [ 'user_login' => $_POST['account_email'] ], [ 'ID' => $user_id ] );
						$GLOBALS['current_user']->user_login = $_POST['account_email'];

						wp_cache_delete( $user_id, 'users' );
						wp_clear_auth_cookie();
						wp_set_current_user( $user_id );
						wp_set_auth_cookie( $user_id, 'forever', is_ssl() );
					ENDIF;
				ENDIF;

			ENDIF;
		ENDIF;
	}

	public function woocommerce_add_to_cart_product_id( $product_id )
	{
		IF ( ! apply_filters( 'mbr/match_user_content', $product_id ) ) :
			wc_add_notice( __('Sorry, this item is not available for purchase', 'activemember360' ) . " (" . sprintf( __( 'Product ID: %s', 'activemember360' ), $product_id ) . ').', 'notice' );
			return 0;
		ENDIF;

		return $product_id;
	}

	
	public function filter_empty_product_categories( $terms, $taxonomies, $args, $term_query = '' )
	{
		global $wpdb;

		static $count  = [];
		static $_posts = [];
		static $_perms = [];

		IF ( ! is_array( $terms ) ) :
			return $terms;
		ENDIF;

		FOREACH ( $terms AS $key => $term ) :
			IF ( empty( $term ) OR ! is_object( $term ) ) :
				continue;
			ENDIF;
			IF ( ! in_array( $term->taxonomy, [ 'product_cat','product_tag' ] ) ) :
				continue;
			ENDIF;

			IF ( empty( $count[ $term->term_id ] ) ) :
				$count[ $term->term_id ] = 0;
				IF ( empty( $term->count ) ) :
					continue;
				ENDIF;
				$rows = $wpdb->get_results( "SELECT object_id FROM {$wpdb->prefix}term_relationships WHERE term_taxonomy_id={$term->term_taxonomy_id}" );
				FOREACH ( $rows AS $key => $row ) :
					IF ( empty( $_posts[ $row->object_id ] ) ) :
						$_posts[ $row->object_id ] = get_post( $row->object_id );
					ENDIF;
					$_post = $_posts[ $row->object_id ];
					IF ( $_post->post_type == 'product' ) :
						IF ( empty( $_perms[ $row->object_id ] ) ) :
							$_perms[ $row->object_id ] = apply_filters( 'mbr/match_user_content', $_post );
						ENDIF;
						IF ( $_perms[$row->object_id ] ) :
							$count[ $term->term_id ]++;
						ENDIF;
					ENDIF;
				ENDFOREACH;
			ENDIF;
		ENDFOREACH;

		FOREACH ( $terms AS $key => $term ) :
			IF ( empty( $term ) OR ! is_object( $term ) ) :
				continue;
			ENDIF;
			IF ( ! in_array( $term->taxonomy, [ 'product_cat','product_tag' ] ) ) :
				continue;
			ENDIF;
			$terms[ $key ]->count = $count[ $term->term_id ];
			IF ( empty( $terms[ $key ]->count ) ) :
				unset( $terms[ $key ] );
			ENDIF;
		ENDFOREACH;

		return $terms;
	}

	



	public function product_cat_register_meta()
	{
		register_meta( 'term', 'product_category_protection', [ $this, 'sanitize_protection' ] );
	}
	




	public function sanitize_protection( $protection )
	{
		return $protection;
	}

	




	public function product_cat_add_protection_meta()
	{
		wp_nonce_field( basename( __FILE__ ), 'mbr_product_cat_protection_nonce' );
		echo '<div class="form-field">';
		echo '<label for="mbr-product-cat-protection">' . __( 'Category Protection', 'activemember360' ) . '</label>';
		echo '<input name="mbr-product-cat-protection" id="mbr-product-cat-protection" value="" />';
		echo '<p class="description">' . __( 'Select tags above in order to limit who may view and purchase products in this category','activemember360' ) . '</p>';
		echo '</div>';
	}

	





	public function product_cat_edit_protection_meta( $term )
	{
		$product_cat_protection   = get_term_meta( $term->term_id, 'product_category_protection', TRUE );
		IF ( ! $product_cat_protection ) :
			$product_cat_protection = '';
		ENDIF;
		wp_nonce_field( basename( __FILE__ ), 'mbr_product_cat_protection_nonce' );
		echo '<tr class="form-field term-protection-wrap">';
		echo '<th scope="row"><label for="protection">' . __( 'Category Protection', 'activemember360' ) . '</label></th>';
		echo '<td>';
		echo "<input name='mbr-product-cat-protection' value='{$product_cat_protection}'>";
		echo '<p class="description">' . __( 'Select tags above in order to limit who may view and purchase products in this category', 'activemember360' ) . '</p>';
		echo '</td>';
		echo '</tr>';
	}

	


	public function product_cat_display_protection_meta()
	{
		IF ( ! is_tax( 'product_cat' ) ) :
			return;
		ENDIF;

		$term_id = get_queried_object()->term_id;
		IF ( $protection = get_term_meta( $term_id, 'product_category_protection', TRUE ) ) :
			echo '<div class="product-cat-protection">' . $protection . '</div>';
		ENDIF;
	}

	





	public function product_cat_protection_meta_save( $term_id )
	{
		IF ( empty( $_POST['mbr_product_cat_protection_nonce'] ) OR ! wp_verify_nonce( $_POST['mbr_product_cat_protection_nonce'], basename( __FILE__ ) ) ) :
			return;
		ENDIF;

		$old_protection = get_term_meta( $term_id, 'product_category_protection', TRUE );
		$new_protection = trim( $_POST['mbr-product-cat-protection'] ) ? trim( $_POST['mbr-product-cat-protection'] ) : '';

		IF ( $old_protection AND empty( $new_protection ) ) :
			delete_term_meta( $term_id, 'product_category_protection' );
		ELSEIF ( $old_protection !== $new_protection ) :
			update_term_meta( $term_id, 'product_category_protection', $this->sanitize_protection( $new_protection ) );
		ENDIF;
	}

	public function sc_quick_checkout( $parms = '', $content = NULL, $scname = '' )
	{
		$defaults = [
			'autorun'      => NULL,
			'link_text'    => __( 'Add to cart', 'activemember360' ),
			'link_style'   => '',
			'button_text'  => '',
			'button_image' => '',
			'button_style' => '',
			'scname'       => $scname
		];

		$parms = apply_filters( 'mbr/shortcodes/atts', $defaults, $parms, $scname );
		IF  ($parms['autorun'] == 'yes' ) :
			$this->process_quick_checkout( $parms );
		ELSE :
			return $this->click_quick_checkout( $parms );
		ENDIF;
	}

	public function click_quick_checkout( $parms )
	{
		IF ( isset( $_POST['mbr_hash'] ) AND $_POST['mbr_hash'] == $parms['mbr_hash'] ) :
			$this->process_quick_checkout( $parms );
		ENDIF;

		$add_products = $this->validate_quick_checkout_products( $parms );
		IF ( empty( $add_products ) ) :
			return NULL;
		ENDIF;

		$randkey      = uniqid();
		$retCODE      = '';
		$retCODE     .= "<form name='{$randkey}' action='' method='post'>";
		$retCODE     .= "<input type='hidden' name='mbr_quick_checkout' value='{$randkey}' />";
		$retCODE     .= "<input type='hidden' name='mbr_hash' value='{$parms['mbr_hash']}' />";
		$retCODE     .= '</form>';
		$retCODE     .= "<span class='{$randkey}'>";

		IF ( $parms['button_text'] OR $parms['button_image'] ) :
			$button_css = empty( $parms['button_style'] ) ? '' : "style='{$parms['button_style']}'";
			IF ( $parms['button_image'] ) :
				$retCODE .= "<input onclick=\"document.forms['{$randkey}'].submit();\" type='image'  class='{$parms['scname']}_btn' name='{$parms['scname']}[btn]' src='{$parms['button_image']}' alt='{$parms['button_text']}' {$button_css} />";
			ELSE :
				$button_text = empty( $parms['button_text'] ) ? __( 'Add to cart', 'activemember360' ) : $parms['button_text'];
				$retCODE .= "<input onclick=\"document.forms['{$randkey}'].submit();\" type='submit' class='{$parms['scname']}_btn' name='{$parms['scname']}[btn]' value='{$button_text}' {$button_css} />";
			ENDIF;
		ELSE :
			$link_text  = empty( $parms['link_text'] ) ? __( 'Add to cart', 'activemember360' ) : $parms['link_text'];
			$link_css   = $parms['link_style'] ? "style='{$parms['link_style']}'" : '';
			$retCODE   .= "<a href='#' onclick=\"document.forms['{$randkey}'].submit();\" class='{$parms['scname']}_link' {$link_css}>{$link_text}</a>";
		ENDIF;
		$retCODE     .= '</span>';

		return $retCODE;
	}

	public function validate_quick_checkout_products( $parms = [] )
	{
		$add_products = [];
		FOREACH ( $parms AS $k => $v ) :

			DO {
				list( $prod_id, $prod_qty ) = is_numeric( $k ) ? [ trim( $v ), 1 ] : [ trim( $k ), intval( trim( $v ) ) ];
				
				IF ( empty( $prod_qty ) ) :
					break;
				ENDIF;
				
				IF ( 'p' <> substr( $prod_id, 0, 1 ) ) :
					break;
				ENDIF;
				
				$product_id = substr( $prod_id, 1 );
				
				IF ( ! apply_filters( 'mbr/match_user_content', $product_id ) ) :
					break;
				ENDIF;
				
				IF ( ! wc_get_product( $product_id ) ) :
					break;
				ENDIF;
				$add_products[ $product_id ] = empty( $add_products[ $product_id ] ) ? $prod_qty : $add_products[ $product_id ] + $prod_qty;
			} WHILE (FALSE);

		ENDFOREACH;

		return $add_products;
	}

	public function process_quick_checkout( $parms = [] )
	{
		$add_products        = $this->validate_quick_checkout_products( $parms );
		$added_to_cart_count = 0;

		IF ( ! empty( $add_products ) ) :
			FOREACH ( $add_products AS $product_id => $product_qty ) :

				DO {
					IF ( ! apply_filters( 'mbr/match_user_content', $product_id ) OR ! WC()->cart->add_to_cart( $product_id, $product_qty ) ) :
						wc_add_notice( sprintf( __('Product ID %s could not be added to the cart', 'activemember360' ), $product_id ), 'error' );
						break;
					ENDIF;
					$added_to_cart_count++;
				} WHILE (FALSE);

			ENDFOREACH;
		ENDIF;

		IF ( $added_to_cart_count > 0 ) :
			wc_add_notice( sprintf( __( 'Added %d item(s) to the cart', 'activemember360' ), $added_to_cart_count ), 'notice' );
			wp_safe_redirect( wc_get_checkout_url() );
			exit;
		ENDIF;

		wc_add_notice( sprintf( __( 'No items could be added to the cart: %s', 'activemember360' ), $added_to_cart_count ), 'error' );
		wp_safe_redirect( wc_get_cart_url() );

		exit;
	}

	public function init_metaboxes( $_post_type = FALSE, $_post = FALSE )
	{
		global $pagenow, $typenow;

		IF ( 'product' == $typenow ) :
			IF ( "load-{$pagenow}" == current_filter() ) :
				add_action( 'add_meta_boxes',   [ $this, __FUNCTION__ ], 10, 2 );
			ELSEIF ( 'add_meta_boxes' == current_filter() ) :
				
				do_action( 'mbr/add_meta_box', $_post, [ $this, 'render_meta_boxes_protection_product' ], NULL );
			ENDIF;
		ENDIF;
	}

	public function init_quick_edit( $_post )
	{
		IF ( 'product' == $_post->post_type ) :
			$this->render_meta_boxes_protection_product( $_post, 'quick-protection' );
		ENDIF;
	}

	public function render_meta_boxes_protection_product( $_post, $box )
	{
		$settings = apply_filters( 'mbr/get_meta_box_protection_settings', $_post );

		do_action( 'mbr/render_metabox_css_helper', $box);

		IF ( empty( $settings['membership_levels'] ) ) :
			echo "<p style='color:" . apply_filters( 'mbr/plugin_color', NULL ) . ";font-size:10px;'>" . sprintf( _x( 'You have not yet defined which memberships you want to use on this site. Please do so in <a href="' . admin_url( "admin.php?page=ActiveMember360&mbrtab=memberships" ) . '">%s.</a>', 'wp_admin', 'activemember360' ), "'Memberships'" ) . "</p>";
			return;
		ENDIF;

		do_action( 'mbr/render_meta_boxes_protection_generic',            $_post, $settings );
		do_action( 'mbr/render_meta_boxes_protection_memberships',        $_post, $settings );
		do_action( 'mbr/render_meta_boxes_protection_filters',            $_post, $settings );
		do_action( 'mbr/render_meta_boxes_protection_addtl_tags',         $_post, $settings );
		do_action( 'mbr/render_meta_boxes_protection_comment_automation', $_post, $settings );
		do_action( 'mbr/render_meta_boxes_protection_redirect',           $_post, $settings );
	}

	public function woocommerce_registration_errors_validate_password_base( $errors, $sanitized_user_login, $user_email )
	{
		global $MBR_OPTIONS;

		IF ( empty( $_POST['password'] ) ) :
			return $errors;
		ENDIF;

		$pass_len = strlen( $_POST['password'] );

		IF ( $pass_len < $MBR_OPTIONS['min_passlen'] ) :
			$errors->add( 'invalid_password_length', sprintf( __( 'Your password is too short. It must be at least %s characters long.', 'activemember360' ), $MBR_OPTIONS['min_passlen'] ) );
		ELSEIF ($pass_len > $MBR_OPTIONS['max_passlen']) :
			$errors->add( 'invalid_password_length', sprintf( __( 'Your password is too long. It may not be more than %s characters long.', 'activemember360' ), $MBR_OPTIONS['max_passlen'] ) );
		ENDIF;

		$password_base = apply_filters( 'mbr/password/char_base', NULL );
		IF ( trim( $password_base ) > '' ) :
			FOR ( $i = 0; $i < $pass_len; $i++ ) :
				IF ( FALSE === stripos( $password_base, $_POST['password'][ $i ] ) ) :
					$errors->add( 'invalid_characters', __( 'Your password includes characters that are not allowed.<br />Only the following characters may be used:<br />', 'activemember360' ) . $password_base );
					break;
				ENDIF;
			ENDFOR;
		ENDIF;

		return $errors;
	}

	public function woocommerce_after_checkout_validation_validate_password_base( $posted )
	{
		global $MBR_OPTIONS;

		IF ( empty( $posted['account_password'] ) ) :
			return;
		ENDIF;

		$pass_len = strlen( $posted['account_password'] );

		IF ( $pass_len < $MBR_OPTIONS['min_passlen'] ) :
			wc_add_notice( sprintf( __( 'Your password is too short. It must be at least %s characters long.','activemember360' ), $MBR_OPTIONS['min_passlen'] ), 'error' );
		ENDIF;

		IF ( $pass_len > $MBR_OPTIONS['max_passlen'] ) :
			wc_add_notice( sprintf( __( 'Your password is too long. It may not be more than %s characters long.','activemember360' ), $MBR_OPTIONS['max_passlen'] ), 'error' );
		ENDIF;

		$password_base = apply_filters( 'mbr/password/char_base', NULL );
		IF ( trim( $password_base) > '' ) :
			FOR ( $i = 0; $i < $pass_len; $i++ ) :
				IF ( FALSE === stripos($password_base, $posted['account_password'][ $i ] ) ) :
					wc_add_notice( __( 'Your password includes characters that are not allowed.<br />Only the following characters may be used:<br />', 'activemember360' ) . $password_base, 'error' );
					return;
				ENDIF;
			ENDFOR;
		ENDIF;

		wc_add_notice( $posted['email'], 'error' );
	}

	


	public function woocommerce_add_to_cart( $cart_item_key, $product_id, $quantity, $variation_id, $variation, $cart_item_data )
	{
		$arr_abandoned_cart_tags   = [];

		IF ($product_data = get_post_meta( $product_id, $this->product_data_key, TRUE ) ) :
			$product_data            = json_decode( $product_data, TRUE );
			$arr_abandoned_cart_tags = $product_data['abandoned_cart_tags'];
		ENDIF;

		$this->add_abandoned_cart_tags( $arr_abandoned_cart_tags );

		return;
	}

	public function woocommerce_cart_item_removed( $cart_item_key, $_this )
	{
		$remove_general_tags       = FALSE;
		$arr_abandoned_cart_tags   = [];
		$product_id                = $_this->removed_cart_contents[ $cart_item_key ]['product_id'];

		IF ($product_data = get_post_meta( $product_id, $this->product_data_key, TRUE ) ) :
			$product_data            = json_decode( $product_data, TRUE );
			$arr_abandoned_cart_tags = $product_data['abandoned_cart_tags'];
		ENDIF;

		IF ( WC()->cart->get_cart_contents_count() == 0 ) :
			$remove_general_tags     = TRUE;
		ENDIF;

		$this->remove_abandoned_cart_tags( $arr_abandoned_cart_tags, $remove_general_tags );

		return;
	}

	public function woocommerce_before_cart_item_quantity_zero( $cart_item_key )
	{
		$remove_general_tags       = FALSE;
		$arr_abandoned_cart_tags   = [];

		$cart                      = WC()->cart->get_cart();
		$product_id                = $cart[ $cart_item_key ]['product_id'];

		IF ( $product_data = get_post_meta( $product_id, $this->product_data_key, TRUE ) ) :
			$product_data            = json_decode( $product_data, TRUE );
			$arr_abandoned_cart_tags = $product_data['abandoned_cart_tags'];
		ENDIF;

		




		IF ( WC()->cart->get_cart_contents_count() == 1 ) :
			$remove_general_tags = true;
		ENDIF;

		$this->remove_abandoned_cart_tags( $arr_abandoned_cart_tags, $remove_general_tags );

		return;
	}

	public function add_abandoned_cart_tags_before_checkout( $checkout )
	{
		global $woocommerce;

		$products                    = $woocommerce->cart->get_cart();
		$arr_abandoned_cart_tags     = [];

		FOREACH ( $products AS $product ) :
			IF ($product_data          = get_post_meta( $product['product_id'], $this->product_data_key, TRUE ) ) :
				$product_data            = json_decode( $product_data, TRUE );
				$tmp_abandoned_cart_tags = $product_data['abandoned_cart_tags'];
				$arr_abandoned_cart_tags = array_merge( $arr_abandoned_cart_tags, $tmp_abandoned_cart_tags );
			ENDIF;
		ENDFOREACH;

		$this->add_abandoned_cart_tags( $arr_abandoned_cart_tags );

		return;
	}

	public function remove_abandoned_cart_tags_after_checkout( $order_id )
	{
		$arr_abandoned_cart_tags     = [];
		$order                       = new WC_Order( $order_id );
		$products                    = $order->get_items();

		FOREACH ( $products AS $product ) :
			IF ( $product_data = get_post_meta( $product['product_id'], $this->product_data_key, TRUE ) ) :
				$product_data            = json_decode( $product_data, TRUE );
				$tmp_abandoned_cart_tags = $product_data['abandoned_cart_tags'];
				$arr_abandoned_cart_tags = array_merge( $arr_abandoned_cart_tags, $tmp_abandoned_cart_tags );
			ENDIF;
		ENDFOREACH;

		$this->remove_abandoned_cart_tags( $arr_abandoned_cart_tags,TRUE );

		return;
	}

	public function add_abandoned_cart_tags( $additional_tags = [] )
	{
		IF ( ! apply_filters( 'mbr/contact_id', NULL ) ) :
			return;
		ENDIF;
		IF ( empty( $this->wc_ext_options['abandoned_cart_tags'] ) AND empty( $additional_tags ) ) :
			return;
		ENDIF;
		IF ( empty( $additional_tags ) ) :
			$additional_tags = [];
		ENDIF;
		IF ( empty( $this->wc_ext_options['abandoned_cart_tags'] ) ) :
			$this->wc_ext_options['abandoned_cart_tags'] = [];
		ENDIF;

		$assign_tags                    = array_merge( $this->wc_ext_options['abandoned_cart_tags'], $additional_tags );

		$session                        = apply_filters( 'mbr/session/get', NULL );

		$array['contact_data']['email'] = $session['email'];
		$array['assign_tags']           = $assign_tags;
		apply_filters( 'mbr/api/sync_ctla', $array );

		return;
	}

	public function remove_abandoned_cart_tags( $additional_tags = [], $remove_general_tags = FALSE)
	{
		IF ( ! apply_filters( 'mbr/contact_id', NULL ) ) :
			return;
		ENDIF;
		IF ( empty( $this->wc_ext_options['abandoned_cart_tags'] ) AND empty( $additional_tags ) ) :
			return;
		ENDIF;

		IF ( empty( $additional_tags ) ) :
			$additional_tags = [];
		ENDIF;
		IF ( empty( $this->wc_ext_options['abandoned_cart_tags'] ) ) :
			$this->wc_ext_options['abandoned_cart_tags'] = [];
		ENDIF;

		IF ( $remove_general_tags ) :
			$remove_tags = array_merge( $this->wc_ext_options['abandoned_cart_tags'], $additional_tags );
		ELSE:
			$remove_tags = $additional_tags;
		ENDIF;

		$session                        = apply_filters( 'mbr/session/get', NULL );

		$array['contact_data']['email'] = $session['email'];
		$array['remove_tags']           = $remove_tags;
		apply_filters( 'mbr/api/sync_ctla', $array );

		return;
	}

}

$GLOBALS['mbrwoocommerce'] = mbr_woocommerce::getInstance();