<?php
IF ( ! defined( 'ABSPATH' ) ) : exit(); ENDIF;


IF ( ! class_exists( 'mbr_wc_integration' ) ) :

	class mbr_wc_integration extends WC_Integration
	{

		public function __construct()
		{
			




			$this->id           = 'mbr_wc_admin';
			$this->method_title = 'ActiveMember360';

			IF ( ( ! empty( $_GET['page'] ) AND $_GET['page'] == 'wc-settings' ) AND ( ! empty( $_GET['tab'] ) AND $_GET['tab'] == 'integration' ) AND ( empty($_GET['section'] ) OR $_GET['section'] == $this->id ) ) :
				$this->method_description  = '<p>' . _x( 'Please complete the items below to specify preferences and actions to carry out each time an order takes place.', 'wp_admin', 'activemember360' ) . '</p>';
				$this->method_description .= '<p>' . _x( 'Product specific options are also available in the ActiveMember360 tab under each product data section.',         'wp_admin', 'activemember360' ) . '</p>';

				
				add_action( 'mbr/woocommerce/scripts/mbr', [ $this, 'init_form_fields' ] );
				do_action( 'mbr/woocommerce/scripts/mbr' );

				
				add_action( 'woocommerce_update_options_integration_' . $this->id, [ $this, 'process_admin_options' ] );
			ENDIF;
		}

		public function init_form_fields()
		{
			do_action( 'mbr/site_tags/fetch' );
			do_action( 'mbr/site_lists/fetch' );
			do_action( 'mbr/site_automations/fetch' );

			$arr_tags                      = apply_filters( 'mbr/site_tags/get',               NULL );
			$arr_lists                     = apply_filters( 'mbr/site_lists/get',              NULL );
			$arr_automations               = apply_filters( 'mbr/site_automations/get/active', NULL );

			$this->form_fields             = [];

			
			$this->form_fields['update_on_order_status_changed'] = [
				'title'                      => _x( 'Update order status', 'wp_admin', 'activemember360' ),
				'type'                       => 'checkbox',
				'label'                      => _x( 'Do not wait for completion of a successfull payment to post order status changes to ActiveCampaign', 'wp_admin', 'activemember360' ),
				'description'                => _x( 'If not checked, ActiveMember360 will wait for a successful payment before updating ActiveCampaign.', 'wp_admin', 'activemember360' ),
				'default'                    => 'yes'
			];

			$this->form_fields['enable_tags_in_panels'] = [
				'title'                      => _x(  'Enable Tags in Panels', 'wp_admin', 'activemember360' ),
				'type'                       => 'checkbox',
				'label'                      => _x( 'Enable add and remove tags options in panels', 'wp_admin', 'activemember360' ),
				'description'                => _x( 'If not checked, add and remove tags will be disabled in panels.', 'wp_admin', 'activemember360' ),
				'default'                    => 'yes'
			];

			$this->form_fields['enable_automations_in_panels'] = [
				'title'                      => _x( 'Enable Automations in Panels', 'wp_admin', 'activemember360'),
				'type'                       => 'checkbox',
				'label'                      => _x( 'Enable add and remove to/from automations options in panels', 'wp_admin', 'activemember360' ),
				'description'                => _x( 'If not checked, add and remove to/from automations will be disabled in panels.', 'wp_admin', 'activemember360' ),
				'default'                    => 'yes',
			];

			$this->form_fields['enable_lists_in_panels'] = [
				'title'                      => _x( 'Enable Lists in Panels', 'wp_admin', 'activemember360'),
				'type'                       => 'checkbox',
				'label'                      => _x( 'Enable subscribe and unsubscribe to/from lists options in panels', 'wp_admin', 'activemember360' ),
				'description'                => _x( 'If not checked, subscribe and unsubscribe to/from lists will be disabled in panels.', 'wp_admin', 'activemember360' ),
				'default'                    => 'yes'
			];

			$arr_order_status_tags         = [ '' => _x( 'Do not use a tag to track order status', 'wp_admin', 'activemember360' ) ];
			$arr_order_status_tags         = $arr_order_status_tags + $arr_tags;

			$this->form_fields['order_status_tag'] = [
				'title'                      => _x( 'Track Order Status Tag', 'wp_admin', 'activemember360' ),
				'type'                       => 'select',
				'label'                      => _x( 'Add WooCommerce order status to tags', 'wp_admin', 'activemember360' ),
				'description'                => _x( 'Depending on order status, this tag will be used with a suffix appended to reflect the current order status.<br />Suffixes: _processing, _completed, _pending, _failed, _on-hold, or _cancelled', 'wp_admin', 'activemember360' ),
				'options'                    => $arr_order_status_tags
			];

			$this->form_fields['on_order_add_to_automations'] = [
				'title'                      => _x( 'Add contacts<br />to automations', 'wp_admin', 'activemember360' ),
				'type'                       => 'multiselect',
				'class'                      => 'mbr-wc-integration-panel-select2',
				'description'                => _x( 'Select the ActiveCampaign automations to which contacts should be added.', 'wp_admin', 'activemember360' ),
				'options'                    => $arr_automations
			];

			$this->form_fields['on_order_remove_from_automations'] = [
				'title'                      => _x( 'Remove contacts<br />from automations', 'wp_admin', 'activemember360' ),
				'type'                       => 'multiselect',
				'class'                      => 'mbr-wc-integration-panel-select2',
				'description'                => _x( 'Select the ActiveCampaign automations from which contacts should be removed.', 'wp_admin', 'activemember360' ),
				'options'                    => $arr_automations
			];

			$this->form_fields['on_order_add_tags'] = [
				'title'                      => _x( 'Assign tags<br />to contacts', 'wp_admin', 'activemember360' ),
				'type'                       => 'multiselect',
				'class'                      => 'mbr-wc-integration-panel-select2',
				'description'                => _x( 'Select the ActiveCampaign tags that should be added to contacts.', 'wp_admin', 'activemember360' ),
				'options'                    => $arr_tags
			];

			$this->form_fields['on_order_remove_tags'] = [
				'title'                      => _x( 'Remove tags<br />from contacts', 'wp_admin', 'activemember360' ),
				'type'                       => 'multiselect',
				'class'                      => 'mbr-wc-integration-panel-select2',
				'description'                => _x( 'Select the ActiveCampaign tags that should be removed from contacts.', 'wp_admin', 'activemember360' ),
				'options'                    => $arr_tags
			];

			$this->form_fields['on_order_add_to_lists'] = [
				'title'                      => _x( 'Subscribe contacts<br />to lists', 'wp_admin', 'activemember360' ),
				'type'                       => 'multiselect',
				'class'                      => 'mbr-wc-integration-panel-select2',
				'description'                => _x( 'Select the ActiveCampaign lists to which you would like contacts subscribed.', 'wp_admin', 'activemember360' ),
				'options'                    => $arr_lists
			];

			$this->form_fields['on_order_remove_from_lists'] = [
				'title'                      => _x( 'Unsubscribe contacts<br />from lists', 'wp_admin', 'activemember360' ),
				'type'                       => 'multiselect',
				'class'                      => 'mbr-wc-integration-panel-select2',
				'description'                => _x( 'Select the ActiveCampaign lists from which you would like contacts unsubscribed.', 'wp_admin', 'activemember360' ),
				'options'                    => $arr_lists
			];

			$this->form_fields['abandoned_cart_tags'] = [
				'title'                      => _x( 'Abandoned Cart Tags', 'wp_admin', 'activemember360' ),
				'type'                       => 'multiselect',
				'class'                      => 'mbr-wc-integration-panel-select2',
				'label'                      => _x( 'Add abandoned cart tags', 'wp_admin', 'activemember360' ),
				'description'                => _x( 'Add tags when a logged in contact abandons a cart without checkout.', 'wp_admin', 'activemember360' ),
				'options'                    => $arr_tags
			];

			$arr_number_alt_prices         = [];
			$arr_number_alt_prices[0]      = _x( 'No alternate tag-based prices', 'wp_admin', 'activemember360' );
			FOR ( $x = 1; $x <= 9; $x++ ) :
				$arr_number_alt_prices[ $x ] = $x;
			ENDFOR;

			$this->form_fields['number_alt_prices'] = [
				'title'                      => _x( 'Number of alternate tag-based prices', 'wp_admin', 'activemember360' ),
				'type'                       => 'select',
				'label'                      => _x( 'Number of alternate tag-based prices', 'wp_admin', 'activemember360' ),
				'description'                => _x( 'This option defines how many alternate tag-based prices will be allowed for each product.', 'wp_admin', 'activemember360' ),
				'options'                    => $arr_number_alt_prices
			];

			$this->form_fields['debug_log'] = [
				'title'                      => _x( 'Debug Log', 'wp_admin', 'activemember360' ),
				'type'                       => 'checkbox',
				'label'                      => _x( 'Enable Activemember360 to update your WooCommerce error log',    'wp_admin', 'activemember360' ),
				'description'                => _x( 'Add ActiveMember360 update errors to your WooCommerce log file', 'wp_admin', 'activemember360' ),
				'default'                    => 'no'
			];

		}
	}
ENDIF;
