<?php
IF ( ! defined( 'ABSPATH' ) ) :
	die( 'This file cannot be accessed directly.' );
ENDIF;

class mbr_crm
{
	use common;

	public $arr_settings = [];

	public function __construct()
	{
		$this->arr_settings = get_option( 'mbr_tickets_settings', FALSE );

		IF ( is_admin() ) :
			add_filter( 'mbr_tickets/metabox/contact/crm/table',      [ $this, 'meta_box_contact_crm_details'], 10, 2 );
			add_action( 'mbr_tickets/crm/reply/staff/tag/add',        [ $this, 'add_staff_replied_tag'] );
			add_action( 'mbr_tickets/crm/reply/staff/automation/add', [ $this, 'add_staff_replied_automation'] );
		ENDIF;
		add_action( 'mbr_tickets/crm/note/add',                     [ $this, 'add_ticket_note'],              10, 3 );
		add_action( 'mbr_tickets/crm/note/edit',                    [ $this, 'edit_ticket_note'],             10, 4 );
	}

	public function meta_box_contact_crm_details( $arr_table_entries, $wp_contact = [] )
	{
		IF ( empty( $wp_contact ) OR ! is_object( $wp_contact ) OR empty( $wp_contact->ID ) ) :
			return $arr_table_entries;
		ENDIF;

		$ac_contact_id      = get_user_meta( $wp_contact->ID, 'mbr_user_id', TRUE );

		$ac_contact_url     = _x( 'Unknown', 'wp_admin', 'mbr-tickets' );
		IF ( ! empty( $ac_contact_id ) ) :
			$ac_account       = apply_filters( 'mbr/ac_account/get', NULL );
			IF ( ! empty( $ac_account['account'] ) ) :
				$ac_contact_url = '<a href="https://' . $ac_account['account'] . '/app/contacts/' . $ac_contact_id . '" target="_blank">' . $ac_contact_id . '</a>';
			ENDIF;
		ENDIF;
		$arr_table_entries[ _x( 'ActiveC. ID:',  'wp_admin', 'mbr-ticket' ) ] = $ac_contact_url;

		return $arr_table_entries;
	}

	public function add_staff_replied_tag( $wp_contact_id = FALSE )
	{
		IF ( empty( $this->arr_settings['crm_assign_tag_on_staff_reply'] ) ) :
			return;
		ENDIF;

		IF ( empty( $wp_contact_id ) ) :
			return;
		ENDIF;

		$ac_contact_id      = get_user_meta( $wp_contact_id, 'mbr_user_id', TRUE );
		IF ( empty( $ac_contact_id ) ) :
			return;
		ENDIF;

		apply_filters( 'mbr/api', 'contact_tag_add', [ 'id' => $ac_contact_id, 'tags' => apply_filters( 'mbr/tags/normalize/names', $this->arr_settings['crm_assign_tag_on_staff_reply'] ) ] );
	}

	public function add_staff_replied_automation( $wp_contact_id = FALSE )
	{
		IF ( empty( $this->arr_settings['crm_assign_automation_on_staff_reply'] ) ) :
			return;
		ENDIF;

		IF ( empty( $wp_contact_id ) ) :
			return;
		ENDIF;

		$ac_contact_id      = get_user_meta( $wp_contact_id, 'mbr_user_id', TRUE );
		IF ( empty( $ac_contact_id ) ) :
			return;
		ENDIF;

		apply_filters( 'mbr/api', 'automation_contact_add', [ 'contact_id' => $ac_contact_id, 'automation' => $this->arr_settings['crm_assign_automation_on_staff_reply'] ] );
	}

	public function add_ticket_note( $wp_contact_id, $inserted_id, $arr_ticket )
	{
		IF ( empty( $this->arr_settings['crm_ticket_as_note_enabled'] ) ) :
			return;
		ENDIF;

		IF ( empty( $wp_contact_id ) ) :
			return;
		ENDIF;

		$ac_contact_id      = get_user_meta( $wp_contact_id, 'mbr_user_id', TRUE );
		IF ( empty( $ac_contact_id ) ) :
			return;
		ENDIF;

		$note  = sprintf( __( 'Ticket ID #%s (%s)', 'mbr-tickets' ), $inserted_id, $this->convert_datetime( time(), TRUE ) ) . "\n\n";
		$note .= $arr_ticket['post_title']           . "\n";
		$note .= $arr_ticket['post_content']         . "\n\n";
		$note .= "--------------------------------------";

		$note_id = $this->note_create( $ac_contact_id, $note );
		IF ( ! empty( $note_id ) ) :
			update_post_meta( $inserted_id, 'mbr_crm_note_id', $note_id );
		ENDIF;
	}

	public function edit_ticket_note( $wp_contact_id, $crm_note_id, $arr_reply, $user_name )
	{
		IF ( empty( $this->arr_settings['crm_ticket_as_note_enabled'] ) ) :
			return;
		ENDIF;

		IF ( empty( $wp_contact_id ) AND ! empty( $crm_note_id ) ) :
			return;
		ENDIF;

		$ac_contact_id    = get_user_meta( $wp_contact_id, 'mbr_user_id', TRUE );
		IF ( empty( $ac_contact_id ) ) :
			return;
		ENDIF;

		IF ( empty( $GLOBALS['MBR_OPTIONS']['api_url'] ) OR empty( $GLOBALS['MBR_OPTIONS']['api_key'] ) ) :
			return;
		ENDIF;

		$response = $this->note_retrieve( $crm_note_id );

		$note     = $response['note']['note'];
		$note    .= "\n\n" . sprintf( '%s (%s)', $user_name, $this->convert_datetime( time(), TRUE ) ) . "\n\n";
		$note    .= $arr_reply['post_title']            . "\n";
		$note    .= $arr_reply['post_content']          . "\n\n";
		$note    .= "--------------------------------------";

		$this->note_update( $crm_note_id, $note );
	}

	public function note_create( $contact_id, $note )
	{
		$body = [
			'note'      => [
				'note'    => $note,
				'reltype' => 'Subscriber',
				'relid'   => $contact_id
			]
		];

		$response = $this->do_api( 'POST', 'notes', json_encode( $body ) );

		return ! empty( $response['note']['id'] ) ? $response['note']['id'] : FALSE;
	}

	public function note_retrieve( $note_id )
	{
		$response = $this->do_api( 'GET', 'notes/' . $note_id );

		return $response;
	}

	public function note_update( $note_id, $note )
	{
		$body = [
			'note'      => [
				'note'    => $note
			]
		];

		$response = $this->do_api( 'PUT', 'notes/' . $note_id, json_encode( $body ) );

		return ! empty( $response['note']['id'] ) ? $response['note']['id'] : FALSE;

	}

	public function do_api( $http_method, $path, $body = [] )
	{
		IF ( empty( $http_method ) OR empty( $path ) OR empty( $GLOBALS['MBR_OPTIONS']['api_url'] ) OR empty( $GLOBALS['MBR_OPTIONS']['api_key'] ) ) :
			return FALSE;
		ENDIF;

		$response         = wp_remote_post( $GLOBALS['MBR_OPTIONS']['api_url'] . '/api/3/' . $path,
			[ 'method'      => strtoupper( $http_method ),
				'timeout'     => 45,
				'redirection' => 5,
				'httpversion' => '1.0',
				'blocking'    => TRUE,
				'headers'     => [ 'Api-Token' => $GLOBALS['MBR_OPTIONS']['api_key'] ],
				'body'        => $body,
				'cookies'     => []
			]
		);

		IF ( is_wp_error( $response ) ) :
			$response       =  FALSE;
		ELSE:
			$response       = json_decode( $response['body'], TRUE );
		ENDIF;

		return $response;
	}

}

mbr_crm::getinstance();