<?php
IF ( ! defined( 'ABSPATH' ) ) :
	die( 'This file cannot be accessed directly.' );
ENDIF;

class mbr_tickets_admin EXTENDS mbr_redux_base_class
{
	use common;

	public $admin_notice       = [];
	public $page_parent        = 'mbr-tickets';
	public $page_child         = 'mbr-tickets-settings';

	public $capability         = 'manage_options';
	public $opt_name           = 'mbr_tickets_settings';

	public $arr_settings       = [];

	public function __construct()
	{
		TRY {
			IF ( ! ( include_once( MBR_TICKETS_ABSPATH . 'includes/class.mbr-list-tables.php' ) ) ) :
				THROW NEW Exception( sprintf( _x( 'Error: Could not include required file %s. The MBR Tickets plugin is not functional.', 'error', 'mbr-tickets' ), '"class.mbr-list-tables.php"' ) );
			ENDIF;
		} CATCH ( Exception $e ) {
			$this->admin_notice = [ 'type' => 'error', 'message' => $e->getMessage() ];
			add_action( 'admin_notices', [ $this, 'admin_notice' ] );

			return;
		}

		$this->arr_settings = get_option( 'mbr_tickets_settings', FALSE );

		add_action( 'plugins_loaded',                                        [ $this, 'init_admin_panel' ],                 99998 );
		add_action( 'plugins_loaded',                                        [ $this, 'init_sections' ],                    99999 );

		add_filter( "redux/{$this->opt_name}/panel/template/header.tpl.php", [ $this, 'alternate_header' ]                        );
		add_action( "redux/options/{$this->opt_name}/validate",              [ $this, 'validate_save_settings' ],           10, 1 );

		add_action( 'wp_dashboard_setup',                                    [ $this, 'dashboard_widget_ticket_overview' ], 99999 );

		add_action( 'add_meta_boxes_mbr-ticket',                             [ $this, 'add_meta_boxes' ],                   10, 2 );
		add_action( 'admin_menu',                                            [ $this, 'admin_menu' ],                       10, 0 );
		add_action( 'admin_enqueue_scripts',                                 [ $this, 'admin_enqueue_scripts' ],            10, 0 );

		add_action( 'wp_ajax_mbr_ajax_table_tickets_display_history',        [ $this, 'ajax_table_tickets_history' ],       10, 0 );
		add_action( 'wp_ajax_mbr_ajax_table_tickets_display',                [ $this, 'ajax_table_tickets_display' ],       10, 0 );
		add_action( 'wp_ajax_mbr_ajax_ticket_status_change',                 [ $this, 'ajax_ticket_status_change' ],        10, 0 );
		add_action( 'wp_ajax_mbr_ajax_ticket_set_tags',                      [ $this, 'ajax_ticket_set_tags' ],             10, 0 );
		add_action( 'wp_ajax_mbr_ajax_ticket_reply',                         [ $this, 'ajax_ticket_reply' ],                10, 0 );

		/* frontend ajax calls */
		add_action( 'wp_ajax_mbr_ajax_frontend_ticket_refresh_replies',      [ $this, 'ajax_frontend_refresh_replies' ],    10, 0 );
		add_action( 'wp_ajax_mbr_ajax_frontend_ticket_refresh_overview',     [ $this, 'ajax_frontend_refresh_overview' ],   10, 0 );

		add_action( 'mbr_tickets/loading/animation',                         [ $this, 'loading_animation' ],                10, 0 );
		add_filter( 'mbr_tickets/ticket/details/html/get',                   [ $this, 'get_ticket_details_html' ],          10, 1 );
		add_filter( 'mbr_tickets/ticket/details/get',                        [ $this, 'get_ticket_details' ],               10, 1 );
		add_filter( 'mbr_tickets/ticket/replies/html/get',                   [ $this, 'get_ticket_replies_html' ],          10, 1 );
		add_filter( 'mbr_tickets/ticket/replies/get',                        [ $this, 'get_ticket_replies' ],               10, 1 );

		add_action( 'mbr_tickets/notifications/staff/replied/email/send',    [ $this, 'staff_send_reply_notification' ],          10, 3 );
		add_action( 'mbr_list_tables/extra_tablenav/top',                    [ $this, 'extra_tablenav_top' ],               10, 0 );

		add_filter( 'heartbeat_received',                                    [ $this, 'heartbeat_received' ],               10, 3 );
	}

	public function dashboard_widget_ticket_overview()
	{
		global $wp_meta_boxes;

		wp_add_dashboard_widget( 'mbr_ticket_overview_dashboard_widget', _x( 'Tickets', 'wp_admin', 'mbr-tickets' ), [ $this, 'dashboard_widget_ticket_overview_callback' ] );

		$normal_dashboard                             = $wp_meta_boxes['dashboard']['normal']['core'];

		$widget_backup                                = [ 'mbr_ticket_overview_dashboard_widget' => $normal_dashboard['mbr_ticket_overview_dashboard_widget'] ];
		unset( $normal_dashboard['mbr_ticket_overview_dashboard_widget'] );

		$sorted_dashboard                             = array_merge( $widget_backup, $normal_dashboard );
		$wp_meta_boxes['dashboard']['normal']['core'] = $sorted_dashboard;
	}

	public function dashboard_widget_ticket_overview_callback()
	{
		global $wpdb;

		$open     = $wpdb->get_var( "SELECT COUNT(*) FROM {$wpdb->posts} WHERE post_type = 'mbr-ticket' AND post_status = 'mbr_ticket_open'" );
		$answered = $wpdb->get_var( "SELECT COUNT(*) FROM {$wpdb->posts} WHERE post_type = 'mbr-ticket' AND post_status = 'mbr_ticket_answered'" );
		$closed   = $wpdb->get_var( "SELECT COUNT(*) FROM {$wpdb->posts} WHERE post_type = 'mbr-ticket' AND post_status = 'mbr_ticket_closed'" );
		$total    = $open + $answered + $closed;

		echo '<div class="mbr-tickets-badge-open mbr-tickets-status-widget">'     . _x( 'Open:',     'wp_admin', 'mbr-tickets') . ' ' . $open     . '</div>';
		echo '<div class="mbr-tickets-badge-answered mbr-tickets-status-widget">' . _x( 'Answered:', 'wp_admin', 'mbr-tickets') . ' ' . $answered . '</div>';
		echo '<div class="mbr-tickets-badge-closed mbr-tickets-status-widget">'   . _x( 'Closed:',   'wp_admin', 'mbr-tickets') . ' ' . $closed   . '</div>';
		echo '<div class="mbr-tickets-badge-total mbr-tickets-status-widget">'    . _x( 'Total:',    'wp_admin', 'mbr-tickets') . ' ' . $total    . '</div>';
		echo '<hr />';
		echo '<a href="' . admin_url( 'admin.php?page=' . $this->page_parent ) . '" class="button button-primary">' . _x( 'View Tickets', 'wp_admin', 'mbr-tickets' ) . '</a>';
	}

	public function init_admin_panel()
	{
		global $MBR_REDUX_HEADER_MODULE_NAMES;

		$MBR_REDUX_HEADER_MODULE_NAMES[ $this->opt_name ] = _x( 'Tickets Module', 'wp_admin', 'mbr-tickets' );

		Redux::setExtensions( $this->opt_name, MBR_REDUX_EXTENSIONS_DIR );

		$admin_title        = $page_title = _x( 'Settings', 'wp_admin', 'mbr-tickets' );
		$page_title         = apply_filters( 'mbr/plugin_name', NULL ) . ' ' . $page_title;

		$arr_config         =  [
			'opt_name'        => $this->opt_name,
			'global_variable' => $this->opt_name,
			'page_parent'     => $this->page_parent,
			'page_slug'       => $this->page_child,
			'page_title'      => $page_title,
			'display_name'    => $page_title,
			'admin_title'     => $admin_title,
			'menu_title'      => $admin_title,
			'save_defaults'   => TRUE
		];

		$arr_config         = array_merge( $this->arr_default_sub_admin_panel_settings, $arr_config );

		Redux::setArgs( $this->opt_name, $arr_config );
	}

	public function init_sections()
	{
		global $pagenow;

		IF ( $pagenow <> 'options.php' AND $pagenow <> 'admin.php' OR ( $pagenow == 'admin.php' AND ( empty( $_GET['page'] ) OR $_GET['page'] <> $this->page_child ) ) ) :
			//return;
		ENDIF;

		$arr_section_settings =
		[
			'id'               => 'section_tickets_settings',
			'title'            => _x( 'Settings', 'wp_admin', 'mbr-tickets' ),
			'icon'             => 'el el-cog',
			'fields'           => [
				[
					'id'           => 'tickets_per_page',
					'title'        => _x( 'Tickets per page', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'text',
					'validate'     => 'numeric_not_empty',
					'default'      => 25
				],
				[
					'id'           => 'frontend_ticket_replies_refresh_interval',
					'title'        => _x( 'Replies refresh interval (Customer)', 'wp_admin', 'mbr-tickets' ),
					'desc'         => _x( 'Enter the interval in seconds to check for new replies on a single ticket view.', 'wp_admin', 'mbr-tickets'),
					'type'         => 'text',
					'validate'     => 'numeric_not_empty',
					'default'      => 10
				],
				[
					'id'           => 'frontend_ticket_overview_refresh_interval',
					'title'        => _x( 'Ticket overview refresh interval (Customer)', 'wp_admin', 'mbr-tickets' ),
					'desc'         => _x( 'Enter the interval in seconds to check for ticket modifications on the ticket overview.', 'wp_admin', 'mbr-tickets'),
					'type'         => 'text',
					'validate'     => 'numeric_not_empty',
					'default'      => 10
				]
			]
		];

		$arr_section_notifications =
		[
			'id'               => 'section_tickets_notifications',
			'title'            => _x( 'Notifications', 'wp_admin', 'mbr-tickets' ),
			'icon'             => 'el el-envelope',
			'fields'           => [
				[
					'id'           => 'push_notifications_enabled',
					'title'        => _x( 'Enable Push notifications', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'switch',
					'default'      => 0
				],
				[
					'id'           => 'push_notifications_icon',
					'title'        => _x( 'Push Notification Icon', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'media',
					'default'      => '',
					'required'     => [ 'push_notifications_enabled', 'equals', 1 ]
				],
				[
					'id'           => 'audio_reply_notifications_enabled',
					'title'        => _x( 'Enable Audio Reply notifications', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'switch',
					'default'      => 0
				],
				[
					'id'           => 'send_wp_email_on_staff_reply_enabled',
					'title'        => _x( 'Send an email to the customer when support has replied to the ticket', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'switch',
					'default'      => 0
				]
			]
		];

		$woocommerce_fields  = [];
		IF ( class_exists( 'WooCommerce' ) ) :
			$woocommerce_fields[] = [
				'id'             => 'woocommerce_my_account_integration_enabled',
				'title'          => _x( 'Integrate in "WooCommerce MyAccount"', 'wp_admin', 'mbr-tickets' ),
				'type'           => 'switch',
				'default'        => 0
			];
		ELSE:
			$woocommerce_fields[] = [
				'id'             => 'woocommerce_my_account_integration_placeholder',
				'title'          => _x( 'Integrate in "WooCommerce MyAccount"', 'wp_admin', 'mbr-tickets' ),
				'type'           => 'raw',
				'content'        => _x( 'This option is available once WooCommerce is installed and activated.', 'wp_admin', 'mbr-tickets' )
			];
		ENDIF;

		$arr_section_woocommerce =
		[
			'id'               => 'section_tickets_woocommerce',
			'title'            => _x( 'WooCommerce', 'wp_admin', 'mbr-tickets' ),
			'icon'             => 'el el-shopping-cart',
			'fields'           => $woocommerce_fields
		];

		$arr_section_frontend_template =
		[
			'id'               => 'section_tickets_frontend',
			'title'            => _x( 'Appearance', 'wp_admin', 'mbr-tickets' ),
			'icon'             => 'el el-website'
		];

		$arr_section_frontend_template_endpoints =
		[
			'id'               => 'section_tickets_frontend_endpoints',
			'title'            => _x( 'Endpoints', 'wp_admin', 'mbr-tickets' ),
			'icon'             => 'el el-network',
			'subsection'       => TRUE,
			'fields'           => [
				[
					'id'           => 'frontend_endpoint_main',
					'title'        => _x( 'Main Endpoint', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'text',
					'validate'     => 'not_empty',
					'default'      => 'mbr-tickets'
				],
				[
					'id'           => 'frontend_endpoint_create',
					'title'        => _x( 'Create Endpoint', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'text',
					'validate'     => 'not_empty',
					'default'      => 'create'
				],
				[
					'id'           => 'frontend_endpoint_edit',
					'title'        => _x( 'Edit Endpoint', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'text',
					'validate'     => 'not_empty',
					'default'      => 'edit'
				]
			]
		];

		$arr_section_frontend_template_header =
		[
			'id'               => 'section_tickets_frontend_header',
			'title'            => _x( 'Header', 'wp_admin', 'mbr-tickets' ),
			'icon'             => 'el el-lines',
			'subsection'       => TRUE,
			'fields'           => [
				[
					'id'           => 'frontend_template_favicon',
					'title'        => _x( 'Favicon', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'media',
					'placeholder'  => _x( 'No Favicon selected', 'wp_admin', 'mbr-tickets' )
				],
				[
					'id'           => 'frontend_template_logo',
					'title'        => _x( 'Logo', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'media',
					'placeholder'  => _x( 'No Logo selected', 'wp_admin', 'mbr-tickets' )
				],
				[
					'id'           => 'frontend_template_header_color_scheme',
					'title'        => _x( 'Header Color Scheme', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'select',
					'options'      => [
						'is-primary' => _x( 'Light green', 'wp_admin', 'mbr-tickets' ),
						'is-link'    => _x( 'Dark Blue',   'wp_admin', 'mbr-tickets' ),
						'is-info'    => _x( 'Blue',        'wp_admin', 'mbr-tickets' ),
						'is-success' => _x( 'Green',       'wp_admin', 'mbr-tickets' ),
						'is-warning' => _x( 'Yellow',      'wp_admin', 'mbr-tickets' ),
						'is-danger'  => _x( 'Red',         'wp_admin', 'mbr-tickets' ),
						'is-black'   => _x( 'Black',       'wp_admin', 'mbr-tickets' ),
						'is-dark'    => _x( 'Dark',        'wp_admin', 'mbr-tickets' ),
						'is-light'   => _x( 'Light grey',  'wp_admin', 'mbr-tickets' ),
						'is-white'   => _x( 'White',       'wp_admin', 'mbr-tickets' ),
					],
					'placeholder'  => _x( 'Default',     'wp_admin', 'mbr-tickets' ),
					'default'      => 'is-info',
				]
			]
		];

		$arr_section_frontend_template_footer =
		[
			'id'               => 'section_tickets_frontend_footer',
			'title'            => _x( 'Footer', 'wp_admin', 'mbr-tickets' ),
			'icon'             => 'el el-lines',
			'subsection'       => TRUE,
			'fields'           => [
				[
					'id'           => 'frontend_template_footer_enabled',
					'title'        => _x( 'Enable Footer', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'switch',
					'default'      => 0
				],
				[
					'id'           => 'frontend_template_footer_color_scheme',
					'title'        => _x( 'Footer Color Scheme', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'select',
					'options'      => [
						'is-primary' => _x( 'Light green', 'wp_admin', 'mbr-tickets' ),
						'is-link'    => _x( 'Dark Blue',   'wp_admin', 'mbr-tickets' ),
						'is-info'    => _x( 'Blue',        'wp_admin', 'mbr-tickets' ),
						'is-success' => _x( 'Green',       'wp_admin', 'mbr-tickets' ),
						'is-warning' => _x( 'Yellow',      'wp_admin', 'mbr-tickets' ),
						'is-danger'  => _x( 'Red',         'wp_admin', 'mbr-tickets' ),
						'is-black'   => _x( 'Black',       'wp_admin', 'mbr-tickets' ),
						'is-dark'    => _x( 'Dark',        'wp_admin', 'mbr-tickets' ),
						'is-light'   => _x( 'Light grey',  'wp_admin', 'mbr-tickets' ),
						'is-white'   => _x( 'White',       'wp_admin', 'mbr-tickets' ),
					],
					'placeholder'  => _x( 'Default',     'wp_admin', 'mbr-tickets' ),
					'default'      => 'is-info',
					'required'     => [ 'frontend_template_footer_enabled', 'equals', 1 ]
				],
				[
					'id'           => 'frontend_template_footer_text',
					'title'        => _x( 'Footer Text', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'text',
					'required'     => [ 'frontend_template_footer_enabled', 'equals', 1 ]
				]
			]
		];

		$arr_section_frontend_template_create_ticket =
		[
			'id'               => 'section_tickets_frontend_create_ticket',
			'title'            => _x( 'Create Ticket', 'wp_admin', 'mbr-tickets' ),
			'icon'             => 'el el-comment',
			'subsection'       => TRUE,
			'fields'           => [
				[
					'id'           => 'frontend_template_create_ticket_user_info_enabled',
					'title'        => _x( 'Enable User Info', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'switch',
					'default'      => 0
				],
				[
					'id'           => 'frontend_template_create_ticket_user_info_color_scheme',
					'title'        => _x( 'User Info Color Scheme', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'select',
					'options'      => [
						'is-primary' => _x( 'Light green', 'wp_admin', 'mbr-tickets' ),
						'is-link'    => _x( 'Dark Blue',   'wp_admin', 'mbr-tickets' ),
						'is-info'    => _x( 'Blue',        'wp_admin', 'mbr-tickets' ),
						'is-success' => _x( 'Green',       'wp_admin', 'mbr-tickets' ),
						'is-warning' => _x( 'Yellow',      'wp_admin', 'mbr-tickets' ),
						'is-danger'  => _x( 'Red',         'wp_admin', 'mbr-tickets' ),
						'is-black'   => _x( 'Black',       'wp_admin', 'mbr-tickets' ),
						'is-dark'    => _x( 'Dark',        'wp_admin', 'mbr-tickets' ),
						'is-light'   => _x( 'Light grey',  'wp_admin', 'mbr-tickets' ),
						'is-white'   => _x( 'White',       'wp_admin', 'mbr-tickets' ),
					],
					'placeholder'  => _x( 'Default',     'wp_admin', 'mbr-tickets' ),
					'default'      => 'is-light',
					'required'     => [ 'frontend_template_create_ticket_user_info_enabled', 'equals', 1 ]
				],
				[
					'id'           => 'frontend_template_create_ticket_user_info_text',
					'title'        => _x( 'User Info Text', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'editor',
					'args'         => [ 'wpautop' => TRUE ],
					'required'     => [ 'frontend_template_create_ticket_user_info_enabled', 'equals', 1 ]
				]
			]
		];

		$arr_section_frontend_template_ticket_replies =
		[
			'id'               => 'section_tickets_frontend_ticket_replies',
			'title'            => _x( 'Replies', 'wp_admin', 'mbr-tickets' ),
			'icon'             => 'el el-comment-alt',
			'subsection'       => TRUE,
			'fields'           => [
				[
					'id'           => 'frontend_template_ticket_replies_staff_color_scheme',
					'title'        => _x( 'Staff Color Scheme', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'select',
					'options'      => [
						'is-primary' => _x( 'Light green', 'wp_admin', 'mbr-tickets' ),
						'is-link'    => _x( 'Dark Blue',   'wp_admin', 'mbr-tickets' ),
						'is-info'    => _x( 'Blue',        'wp_admin', 'mbr-tickets' ),
						'is-success' => _x( 'Green',       'wp_admin', 'mbr-tickets' ),
						'is-warning' => _x( 'Yellow',      'wp_admin', 'mbr-tickets' ),
						'is-danger'  => _x( 'Red',         'wp_admin', 'mbr-tickets' ),
						'is-black'   => _x( 'Black',       'wp_admin', 'mbr-tickets' ),
						'is-dark'    => _x( 'Dark',        'wp_admin', 'mbr-tickets' ),
						'is-light'   => _x( 'Light grey',  'wp_admin', 'mbr-tickets' ),
						'is-white'   => _x( 'White',       'wp_admin', 'mbr-tickets' ),
					],
					'placeholder'  => _x( 'Default',     'wp_admin', 'mbr-tickets' ),
					'default'      => 'is-primary',
				],
				[
					'id'           => 'frontend_template_ticket_replies_contact_color_scheme',
					'title'        => _x( 'Contact Color Scheme', 'wp_admin', 'mbr-tickets' ),
					'type'         => 'select',
					'options'      => [
						'is-primary' => _x( 'Light green', 'wp_admin', 'mbr-tickets' ),
						'is-link'    => _x( 'Dark Blue',   'wp_admin', 'mbr-tickets' ),
						'is-info'    => _x( 'Blue',        'wp_admin', 'mbr-tickets' ),
						'is-success' => _x( 'Green',       'wp_admin', 'mbr-tickets' ),
						'is-warning' => _x( 'Yellow',      'wp_admin', 'mbr-tickets' ),
						'is-danger'  => _x( 'Red',         'wp_admin', 'mbr-tickets' ),
						'is-black'   => _x( 'Black',       'wp_admin', 'mbr-tickets' ),
						'is-dark'    => _x( 'Dark',        'wp_admin', 'mbr-tickets' ),
						'is-light'   => _x( 'Light grey',  'wp_admin', 'mbr-tickets' ),
						'is-white'   => _x( 'White',       'wp_admin', 'mbr-tickets' ),
					],
					'placeholder'  => _x( 'Default',     'wp_admin', 'mbr-tickets' ),
					'default'      => 'is-light',
				]
			]
		];

		$crm_fields          = [];
		IF ( defined( 'MBR_TICKETS_CRM' ) AND strtolower( MBR_TICKETS_CRM ) == 'activecampaign' ) :
			$arr_tags          = apply_filters( 'mbr/site_tags/get', NULL );
			$arr_automations   = apply_filters( 'mbr/site_automations/get', NULL );
			$crm_fields[]      = [
				'id'             => 'crm_ticket_as_note_enabled',
				'title'          => _x( 'Save tickets and replies as notes in ActiveCampaign', 'wp_admin', 'mbr-tickets' ),
				'type'           => 'switch',
				'default'        => 0
			];
			$crm_fields[]      = [
				'id'             => 'crm_assign_tag_on_staff_reply',
				'title'          => _x( 'Assign tag in ActiveCampaign when support replies a ticket', 'wp_admin', 'mbr-tickets' ),
				'placeholder'    => _x( 'Do not assign a tag when support replies to customer', 'wp_admin', 'mbr-tickets' ),
				'type'           => 'select',
				'options'        => $arr_tags
			];
			$crm_fields[]      = [
				'id'             => 'crm_assign_automation_on_staff_reply',
				'title'          => _x( 'Assign automation in ActiveCampaign when support replies a ticket', 'wp_admin', 'mbr-tickets' ),
				'placeholder'    => _x( 'Do not assign an automation when support replies to customer', 'wp_admin', 'mbr-tickets' ),
				'type'           => 'select',
				'options'        => $arr_automations
			];
		ELSEIF ( defined( 'MBR_TICKETS_CRM' ) AND strtolower( MBR_TICKETS_CRM ) == 'infusionsoft') :
			$crm_fields[]      = [
				'id'             => 'crm_ticket_as_note_enabled',
				'title'          => _x( 'Save tickets and replies as notes in Infusionsoft', 'wp_admin', 'mbr-tickets' ),
				'type'           => 'switch',
				'default'        => 0
			];
		ELSE:
			$crm_fields[]      = [
				'id'             => 'crm_ticket_as_note_enabled_placeholder',
				'title'          => _x( 'CRM Integration', 'wp_admin', 'mbr-tickets' ),
				'type'           => 'raw',
				'content'        => _x( 'This option is available once ActiveMember360 or iMember360 is installed and activated.', 'wp_admin', 'mbr-tickets' )
			];
		ENDIF;

		$arr_section_crm     =
		[
			'id'               => 'section_tickets_crm',
			'title'            => MBR_TICKETS_CRM,
			'icon'             => 'el el-group',
			'fields'           => $crm_fields
		];

		Redux::setSection( $this->opt_name, $arr_section_settings );
		Redux::setSection( $this->opt_name, $arr_section_notifications );
		Redux::setSection( $this->opt_name, $arr_section_woocommerce );
		Redux::setSection( $this->opt_name, $arr_section_frontend_template );
		Redux::setSection( $this->opt_name, $arr_section_frontend_template_header );
		Redux::setSection( $this->opt_name, $arr_section_frontend_template_footer );
		Redux::setSection( $this->opt_name, $arr_section_frontend_template_create_ticket );
		Redux::setSection( $this->opt_name, $arr_section_frontend_template_ticket_replies );
		Redux::setSection( $this->opt_name, $arr_section_frontend_template_endpoints );
		Redux::setSection( $this->opt_name, $arr_section_crm );
	}

	public function validate_save_settings( &$arr_settings )
	{
		$arr_settings['frontend_endpoint_main']   = trim( $arr_settings['frontend_endpoint_main'],   '/\\' );
		$arr_settings['frontend_endpoint_create'] = trim( $arr_settings['frontend_endpoint_create'], '/\\' );
		$arr_settings['frontend_endpoint_edit']   = trim( $arr_settings['frontend_endpoint_edit'],   '/\\' );
	}

	public function admin_notice( $print = TRUE )
	{
		IF ( empty( $this->admin_notice ) OR empty( $this->admin_notice['message'] ) ) :
			return;
		ENDIF;

		$type    = ! empty( $this->admin_notice['type'] )  ? $this->admin_notice['type']  : 'info';
		$style   = ! empty( $this->admin_notice['style'] ) ? $this->admin_notice['style'] : '';

		$output  = '<div id="mbr-tickets-notice" class="notice notice-' . $type . ' is-dismissible" ' . $style . '>';
		$output .= '<p>' . $this->admin_notice['message'] . '</p>';
		$output .= '</div>';

		IF ( $print ) :
			echo $output;
		ELSE:
			return $output;
		ENDIF;
	}

	public function loading_animation()
	{
		return '<div class="mbr-tickets-loading-bar"><img src="' . includes_url( 'js/thickbox/loadingAnimation.gif' ) . '" /></div>';
	}

	public function add_meta_boxes()
	{
		remove_meta_box( 'tag_list_search_', 'mbr-ticket', 'side' );
		remove_meta_box( 'submitdiv',        'mbr-ticket', 'side' );
		remove_meta_box( 'slugdiv',          'mbr-ticket', 'normal' );
		add_meta_box( 'mbr-ticket-details-box',     _x( 'Ticket Details', 'wp_admin', 'mbr-tickets'), [ $this, 'meta_box_ticket_details' ],     'mbr-ticket', 'side',   'high' );
		add_meta_box( 'mbr-ticket-actions-box',     _x( 'Actions',        'wp_admin', 'mbr-tickets'), [ $this, 'meta_box_ticket_actions' ],     'mbr-ticket', 'side',   'high' );
		add_meta_box( 'mbr-ticket-contact-box',     _x( 'Contact',        'wp_admin', 'mbr-tickets'), [ $this, 'meta_box_ticket_contact' ],     'mbr-ticket', 'side',   'high' );
		add_meta_box( 'mbr-ticket-description-box', _x( 'Description',    'wp_admin', 'mbr-tickets'), [ $this, 'meta_box_ticket_description' ], 'mbr-ticket', 'normal', 'high' );
		add_meta_box( 'mbr-ticket-reply-box',       _x( 'Reply',          'wp_admin', 'mbr-tickets'), [ $this, 'meta_box_ticket_reply' ],       'mbr-ticket', 'normal', 'high' );
		add_meta_box( 'mbr-ticket-replies-box',     _x( 'Replies',        'wp_admin', 'mbr-tickets'), [ $this, 'meta_box_ticket_replies' ],     'mbr-ticket', 'normal', 'high' );
	}

	public function meta_box_ticket_details()
	{
		global $post;

		echo apply_filters( 'mbr_tickets/loading/animation', NULL );
		$display = apply_filters( 'mbr_tickets/ticket/details/html/get', $post );

		echo '<div id="mbr-ticket-details-container">';
		echo $display;
		echo '</div>';
	}

	public function meta_box_ticket_actions()
	{
		global $post;

		echo '<span id="mbr-ticket-id" style="display:none;">' . $post->ID . '</span>';
		echo '<span id="mbr-ticket-mark-as-answered-button" class="button button-primary button-large">' . _x( 'Mark as answered', 'wp_admin', 'mbr-tickets' ) . '</span> <span id="mbr-ticket-mark-as-closed-button" class="button button-primary button-large">' . _x( 'Mark as closed', 'wp_admin', 'mbr-ticket' ) . '</span>';
	}

	public function meta_box_ticket_contact()
	{
		global $post;

		$contact        = get_userdata( $post->post_author );

		echo '<table>';
		echo '<tr><td>' . _x( 'Email:',        'wp_admin', 'mbr-ticket' ) . '</td><td>' . $contact->user_email               . '</td></tr>';
		echo '<tr><td>' . _x( 'First Name:',   'wp_admin', 'mbr-ticket' ) . '</td><td>' . $contact->first_name               . '</td></tr>';
		echo '<tr><td>' . _x( 'Last Name:',    'wp_admin', 'mbr-ticket' ) . '</td><td>' . $contact->last_name                . '</td></tr>';
		echo '<tr><td>' . _x( 'WordPress ID:', 'wp_admin', 'mbr-ticket' ) . '</td><td><a href="' . get_edit_user_link( $contact->ID ) . '" target="_blank">' . $contact->ID . '</a></td></tr>';

		IF ( has_filter( 'mbr_tickets/metabox/contact/crm/table' ) ) :
			$arr_crm_links = apply_filters( 'mbr_tickets/metabox/contact/crm/table', [], $contact );
			IF ( ! empty( $arr_crm_links ) ) :
				FOREACH ( $arr_crm_links AS $label => $value ) :
					echo '<tr><td>' . $label . '</td><td>' . $value . '</td></tr>';
				ENDFOREACH;
			ENDIF;
		ENDIF;

		echo '</table>';
	}

	public function meta_box_ticket_description()
	{
		global $post;

		echo '<script>jQuery( ".wrap h1" ).html( "#' . $post->ID . ' - ' . $post->post_title . '" );</script>';
		echo nl2br( $post->post_content );
	}

	public function meta_box_ticket_reply()
	{
		echo '<textarea id="mbr-ticket-reply-textarea" class="mbr-ticket-reply-textarea"></textarea>';
		echo '<span id="mbr-ticket-reply-button" class="button button-primary button-large">' . _x( 'Reply', 'wp_admin', 'mbr-tickets' ) . '</span>';
	}

	public function meta_box_ticket_replies()
	{
		global $post;

		echo apply_filters( 'mbr_tickets/loading/animation', NULL );
		$display = apply_filters( 'mbr_tickets/ticket/replies/html/get', $post );

		echo '<div id="mbr-ticket-replies-container">';
		echo $display;
		echo '</div>';
	}

	public function get_ticket_details_html( $ticket = FALSE )
	{
		$arr_ticket_details = apply_filters( 'mbr_tickets/ticket/details/get', $ticket );

		IF ( ! is_array( $arr_ticket_details ) ) :
			return _x( 'Unknown error. Ticket details could not be retrieved', 'wp_admin', 'mbr-tickets' );
		ENDIF;

		IF ( $arr_ticket_details['status'] == 'mbr_ticket_closed' ) :
			$status = '<div class="mbr-tickets-badge-closed mbr-tickets-status-widget">'   . _x( 'Closed',   'wp_admin', 'mbr-tickets' ) . '</div>';
		ELSEIF ( $arr_ticket_details['status'] == 'mbr_ticket_answered' ) :
			$status = '<div class="mbr-tickets-badge-answered mbr-tickets-status-widget">' . _x( 'Answered', 'wp_admin', 'mbr-tickets' ) . '</div>';
		ELSE:
			$status = '<div class="mbr-tickets-badge-open mbr-tickets-status-widget">'     . _x( 'Open',     'wp_admin', 'mbr-tickets' ) . '</div>';
		ENDIF;

		$display  = $status;
		$display .= '<table>';
		$display .= '<tr><td>' . _x( 'Created:', 'wp_admin', 'mbr-ticket' ) . '</td><td>' . $arr_ticket_details['created'] . '</td></tr>';
		$display .= '<tr><td>' . _x( 'Updated:', 'wp_admin', 'mbr-ticket' ) . '</td><td>' . $arr_ticket_details['updated'] . '</td></tr>';
		$display .= '</table>';

		return $display;
	}

	public function get_ticket_details( $ticket = FALSE )
	{
		IF ( empty( $ticket ) OR ( ! is_object( $ticket ) AND ! is_numeric( $ticket ) ) ) :
			return;
		ENDIF;

		IF ( is_numeric( $ticket ) ) :
			$ticket = get_post( $ticket );
			IF ( 'mbr-ticket' <> $ticket->post_type ) :
				return;
			ENDIF;
		ENDIF;

		return [
			'status'  => $ticket->post_status,
			'created' => $this->convert_datetime( $ticket->post_date_gmt, TRUE ),
			'updated' => $this->convert_datetime( $ticket->post_modified_gmt, TRUE )
		];
	}

	public function get_ticket_replies( $ticket = FALSE )
	{
		IF ( empty( $ticket ) OR ( ! is_object( $ticket ) AND ! is_numeric( $ticket ) ) ) :
			return;
		ENDIF;

		IF ( is_numeric( $ticket ) ) :
			$ticket = get_post( $ticket );
			IF ( 'mbr-ticket' <> $ticket->post_type ) :
				return;
			ENDIF;
		ENDIF;

		$arr_replies = get_posts( [ 'post_parent' => $ticket->ID, 'post_type' => 'mbr-ticket-reply', 'orderby' => 'date', 'order' => 'desc', 'numberposts' => -1 ] );

		return $arr_replies;
	}

	public function get_ticket_replies_html( $ticket = FALSE )
	{
		$arr_ticket_replies  = apply_filters( 'mbr_tickets/ticket/replies/get', $ticket );

		IF ( ! is_array( $arr_ticket_replies ) ) :
			return _x( 'Unknown error. Ticket details could not be retrieved', 'wp_admin', 'mbr-tickets' );
		ENDIF;

		$display             = '';
		IF ( ! empty( $arr_ticket_replies ) ) :
			IF ( is_numeric( $ticket ) ) :
				$ticket          = get_post( $ticket );
			ENDIF;

			$customer          = get_userdata( $ticket->post_author );
			$arr_customer      = [];
			IF ( ! empty( $customer->first_name ) ) :
				$arr_customer[]  = $customer->first_name;
			ENDIF;
			IF ( ! empty( $customer->last_name ) ) :
				$arr_customer[]  = $customer->last_name;
			ENDIF;

			$customer_name     = implode( ' ', $arr_customer );

			$cnt               = 0;
			FOREACH ( $arr_ticket_replies AS $reply ) :
				$add_class       = '';
				IF ( $cnt == 0 ) :
					$add_class    .= ' mbr-tickets-reply-container-first';
				ENDIF;

			  IF ( $reply->post_author <> $ticket->post_author ) :
					$staff         = get_userdata( $reply->post_author );
					$arr_staff     = [];
					IF ( ! empty( $staff->first_name ) ) :
						$arr_staff[] = $staff->first_name;
					ENDIF;
					IF ( ! empty( $staff->last_name ) ) :
						$arr_staff[] = $staff->last_name;
					ENDIF;
					$name          = '<strong>(' . __( 'Staff', 'mbr-tickets' ) . ')</strong> ' . implode( ' ', $arr_staff );
					$add_class    .= ' mbr-tickets-reply-container-staff';
				ELSE:
					$name          = $customer_name;
					$add_class    .= ' mbr-tickets-reply-container-customer';
				ENDIF;

				$display        .= '<div class="mbr-tickets-reply-container' . $add_class . '">';

				$display        .= '<div class="mbr-tickets-reply-meta-data">' . sprintf( _x( '%s wrote on %s', 'wp_admin', 'mbr-tickets' ), $name, $this->convert_datetime( $reply->post_modified_gmt, TRUE ) ) . '</div>';
				$display        .= nl2br( $reply->post_content );

				$display        .= '</div>';

				$cnt++;
			ENDFOREACH;
		ELSE:
			$display          .= _x( 'No replies yet.', 'wp_admin', 'mbr-tickets' );
		ENDIF;

		return $display;
	}

	/*
	 * Submenus are added through redux
	 */
	public function admin_menu() {
		add_menu_page( _x( 'Tickets', 'wp_admin', 'mbr-tickets' ), _x( 'Tickets', 'wp_admin', 'mbr-tickets' ), $this->capability, $this->page_parent, [ $this, 'support_tickets' ], apply_filters( 'mbr/plugin_favicon', NULL ), 4 );
	}

	public function admin_enqueue_scripts()
	{
		global $pagenow;

		IF ( 'mbr-ticket' == get_post_type() ) :
			wp_dequeue_script( 'autosave' );
		ENDIF;

		wp_enqueue_script( 'heartbeat' );

		wp_register_style( 'mbr-tickets-admin', plugins_url( 'assets/css/mbr-tickets-admin.css', dirname( __FILE__ ) ), FALSE, MBR_TICKETS_VERSION );
		wp_enqueue_style( 'mbr-tickets-admin' );
		wp_register_script( 'mbr-tickets', plugins_url( 'assets/js/mbr-tickets.js', dirname( __FILE__ ) ), [ 'jquery' ], MBR_TICKETS_VERSION, TRUE );
		wp_enqueue_script( 'mbr-tickets' );
		wp_register_script( 'mbr-tickets-admin', plugins_url( 'assets/js/mbr-tickets-admin.js', dirname( __FILE__ ) ), [ 'jquery' ], MBR_TICKETS_VERSION, TRUE );
		wp_enqueue_script( 'mbr-tickets-admin' );
		wp_localize_script( 'mbr-tickets-admin', 'mbr_tickets', [ 'push_notifications' => $this->push_notifications(), 'audio_notifications' => $this->arr_settings['audio_reply_notifications_enabled'], 'audio_dir' => plugins_url( 'assets/audio/', dirname( __FILE__ ) ), 'save_ticket_tags_label' => _x( 'Save Tags', 'wp_admin', 'mbr-tickets' ), 'last_modified' => $this->ticket_last_modified() ] );

		IF ( $pagenow == 'admin.php' AND ( ! empty( $_GET['page'] ) AND $_GET['page'] == $this->page_parent ) ) :
			wp_register_style( 'mbr-list-tables', plugins_url( 'assets/css/mbr-list-tables.css', dirname( __FILE__ ) ), FALSE, MBR_TICKETS_VERSION );
			wp_enqueue_style( 'mbr-list-tables' );
			wp_register_script( 'mbr-list-tables', plugins_url( 'assets/js/mbr-list-tables.js', dirname( __FILE__ ) ), [ 'jquery' ], MBR_TICKETS_VERSION, TRUE );
			wp_enqueue_script( 'mbr-list-tables' );
		ENDIF;
	}

	public function support_tickets()
	{
		echo '<div class="wrap">';
		echo '<h1>' . _x( 'Tickets', 'wp_admin', 'mbr-tickets' ) . '</h1>';
		echo MBR_LIST_TABLE::create_table( 'mbr-tickets-table', 'mbr-tickets-history-table', 'mbr_ajax_table_tickets_display', 'mbr_ajax_table_tickets_display_history', 'desc', 'post_modified', 'mbr_ticket_open', _x( 'Search Results for: ', 'wp_admin', 'mbr-tickets' ), 'mbr-list-tables' );
		echo '</div>';
	}

	public function get_tickets()
	{
		global $wpdb;

		$search                          = " WHERE";

		IF ( empty( $_REQUEST['post_status'] ) ) :
			$search                       .= " post_status = 'mbr_ticket_open'";
		ELSE:
			$search                       .= " post_status = '{$_REQUEST['post_status']}'";
		ENDIF;
		IF ( ! empty( $_REQUEST['s'] ) ) :
			$search                       .= " AND (";
			$search                       .= " post_title LIKE '%{$_REQUEST['s']}%'";
			$search                       .= " OR ID LIKE '%{$_REQUEST['s']}%'";
			$search                       .= " OR post_content LIKE '%{$_REQUEST['s']}%'";
			$search                       .= " OR (post_author IN (SELECT ID FROM {$wpdb->users} WHERE user_email LIKE '%{$_REQUEST['s']}%' OR user_login LIKE '%{$_REQUEST['s']}%' OR user_nicename LIKE '%{$_REQUEST['s']}%' OR display_name LIKE '%{$_REQUEST['s']}%'))";
			$search                       .= " OR (post_author IN (SELECT user_id FROM {$wpdb->usermeta} WHERE (meta_key = 'first_name' AND meta_value LIKE '%{$_REQUEST['s']}%') OR (meta_key = 'last_name' AND meta_value LIKE '%{$_REQUEST['s']}%')))";
			$search                       .= ")";
		ENDIF;
		IF ( ! empty( $_REQUEST['filter'] ) ) :
			$search                       .= " AND (SELECT DISTINCT(object_id) FROM {$wpdb->term_relationships} WHERE object_id = {$wpdb->posts}.ID AND term_taxonomy_id = {$_REQUEST['filter']})";
		ENDIF;

		$arr_total                       = $wpdb->get_results( "SELECT SQL_CALC_FOUND_ROWS DISTINCT ({$wpdb->posts}.ID) FROM {$wpdb->posts}{$search}", ARRAY_A );
		$total                           = count( $arr_total );

		$open                            = $wpdb->get_var( "SELECT COUNT(*) FROM {$wpdb->posts} WHERE post_type = 'mbr-ticket' AND post_status = 'mbr_ticket_open'" );
		$answered                        = $wpdb->get_var( "SELECT COUNT(*) FROM {$wpdb->posts} WHERE post_type = 'mbr-ticket' AND post_status = 'mbr_ticket_answered'" );
		$closed                          = $wpdb->get_var( "SELECT COUNT(*) FROM {$wpdb->posts} WHERE post_type = 'mbr-ticket' AND post_status = 'mbr_ticket_closed'" );

		$orderby                         = " ORDER BY post_modified desc";
		IF ( ! empty( $_REQUEST['order'] ) AND ! empty( $_REQUEST['orderby'] ) ) :
			IF ( ! in_array( $_REQUEST['orderby'], [ 'name', 'email' ] ) ) :
				$orderby                     = " ORDER BY {$_REQUEST['orderby']} {$_REQUEST['order']}";
			ELSEIF ( $_REQUEST['orderby'] == 'email' ) :
				$orderby                     = " ORDER BY (SELECT user_email FROM {$wpdb->users} WHERE {$wpdb->posts}.post_author = ID) {$_REQUEST['order']}";
			ELSEIF ( $_REQUEST['orderby'] == 'name' ) :
				$orderby                     = " ORDER BY (SELECT meta_value FROM {$wpdb->usermeta} WHERE {$wpdb->posts}.post_author = user_id AND meta_key = 'last_name') {$_REQUEST['order']}";
			ENDIF;
		ENDIF;

		$limit                           = " LIMIT {$this->arr_settings['tickets_per_page']}";
		IF ( isset( $_REQUEST['paged'] ) ) :
			$limit                         = " LIMIT " . ( $this->arr_settings['tickets_per_page'] * ( $_REQUEST['paged'] - 1 ) ) . ",{$this->arr_settings['tickets_per_page']}";
		ENDIF;

		$sql                             = "SELECT * FROM {$wpdb->posts}{$search}{$orderby}{$limit}";

		$arr_tickets                     = $wpdb->get_results( $sql, ARRAY_A );

		$arr_data                        = [];
		IF ( ! empty( $arr_tickets ) AND is_array( $arr_tickets ) ) :
			FOREACH ( $arr_tickets AS $key => $arr_ticket ) :
				IF ( $arr_ticket['post_status'] == 'mbr_ticket_closed' ) :
					$status                    = '<div class="mbr-tickets-badge-closed">'   . _x( 'Closed',   'wp_admin', 'mbr-tickets' ) . '</div>';
				ELSEIF ( $arr_ticket['post_status'] == 'mbr_ticket_answered' ) :
					$status                    = '<div class="mbr-tickets-badge-answered">' . _x( 'Answered', 'wp_admin', 'mbr-tickets' ) . '</div>';
				ELSE:
					$status                    = '<div class="mbr-tickets-badge-open">'     . _x( 'Open',     'wp_admin', 'mbr-tickets' ) . '</div>';
				ENDIF;

				$contact                     = get_userdata( $arr_ticket['post_author'] );

				$arr_data[]                  = [
					'ID'                       => $arr_ticket['ID'],
					'post_status'              => $status,
					'post_title'               => $arr_ticket['post_title'],
					'email'                    => $contact->user_email,
					'name'                     => $contact->user_lastname . ', ' . $contact->user_firstname,
					'post_date'                => $arr_ticket['post_date'],
					'post_modified'            => $arr_ticket['post_modified'],
					'view'                     => '<a href="' . admin_url( 'post.php?post=' . $arr_ticket['ID'] . '&action=edit' ) . '">' . _x( 'View', 'wp_admin', 'mbr-tickets') . '</a>',
				];
			ENDFOREACH;
		ENDIF;

		$arr_columns                     = [
			'ID'                           => _x( 'Ticket ID',    'wp_admin', 'mbr-tickets' ),
			'post_status'                  => _x( 'Status',       'wp_admin', 'mbr-tickets' ),
			'post_title'                   => _x( 'Subject',      'wp_admin', 'mbr-tickets' ),
			'email'                        => _x( 'Email',        'wp_admin', 'mbr-tickets' ),
			'name'                         => _x( 'Name',         'wp_admin', 'mbr-tickets' ),
			'post_date'                    => _x( 'Date Created', 'wp_admin', 'mbr-tickets' ),
			'post_modified'                => _x( 'Last Updated', 'wp_admin', 'mbr-tickets' ),
			'view'                         => _x( 'View',         'wp_admin', 'mbr-tickets')
		];

		$arr_sortable_columns            = [
			'ID'                           => [ 'ID',            TRUE ],
			'post_status'                  => [ 'post_status',   TRUE ],
			'post_title'                   => [ 'post_title',    TRUE ],
			'email'                        => [ 'email',         TRUE ],
			'name'                         => [ 'name',          TRUE ],
			'post_date'                    => [ 'post_date',     TRUE ],
			'post_modified'                => [ 'post_modified', TRUE ]
		];

		$arr_status_filter               = [
			'mbr_ticket_open'              => sprintf( _x( 'Open (%s)',     'wp_admin', 'mbr-tickets' ), $open ),
			'mbr_ticket_answered'          => sprintf( _x( 'Answered (%s)', 'wp_admin', 'mbr-tickets' ), $answered ),
			'mbr_ticket_closed'            => sprintf( _x( 'Closed (%s)',   'wp_admin', 'mbr-tickets' ), $closed ),
		];

		return [ 'data' => $arr_data, 'columns' => $arr_columns, 'per_page' => $this->arr_settings['tickets_per_page'], 'sortable_columns' => $arr_sortable_columns, 'total' => $total, 'search_label' => _x( 'Search Tickets', 'wp-admin', 'mbr-tickets' ), 'status_filter' => $arr_status_filter ];
	}

	public function staff_send_reply_notification( $wp_customer_id = FALSE, $arr_reply = [], $staff_name = '' )
	{
		IF ( empty( $this->arr_settings['send_wp_email_on_staff_reply_enabled'] ) ) :
			return;
		ENDIF;

		IF ( empty( $wp_customer_id ) ) :
			return;
		ENDIF;

		IF ( empty( $arr_reply ) OR ! is_array( $arr_reply ) ) :
			return;
		ENDIF;

		$customer = get_userdata( $wp_customer_id );
		IF ( empty( $customer ) ) :
			return;
		ENDIF;

		IF ( ! empty( $customer->user_email ) AND is_email( $customer->user_email ) ) :

			$support_member = ! empty( $staff_name ) ? $staff_name : __( 'Support Member', 'mbr-tickets' );
			$message        = __( 'DO NOT REPLY! PLEASE LOG INTO OUR SUPPORT SYSTEM TO REPLY!', 'mbr-tickets' ) . "\r\n\r\n";
			$message       .= sprintf( __( '%s wrote on %s', 'mbr-tickets' ), $staff_name, $this->convert_datetime( time(), TRUE ) ) . "\r\n\r\n";
			$message       .= $arr_reply['post_content'];

			wp_mail( $customer->user_email, sprintf( __( 'Request updated #%s', 'mbr-tickets' ), $arr_reply['post_parent'] ), $message );
		ENDIF;
	}

	public function ajax_table_tickets_history()
	{
		$array         = $this->get_tickets();
		MBR_LIST_TABLE::ajax_history_callback( $array );
	}

	public function ajax_table_tickets_display()
	{
		$array         = $this->get_tickets();
		MBR_LIST_TABLE::ajax_display_callback( $array );
	}

	public function ajax_ticket_status_change()
	{
		$this->admin_notice               = [];
		$this->admin_notice['type']       = 'error';

		IF ( empty( $_POST['ticket_id'] ) ) :
			$this->admin_notice['message']  = _x( 'Error. Empty Ticket ID. Ticket status not changed.', 'wp_admin', 'mbr-tickets' );
			$notification                   = $this->admin_notice( FALSE );
			die( json_encode( [ 'status' => 0, 'notification' => $notification, 'details' => '' ] ) );
		ENDIF;

		IF ( ! array_key_exists( $_POST['status'], MBR_TICKETS_STATUSES ) ) :
			$this->admin_notice['message']  = _x( 'Error. Wrong Ticket status. Ticket status not changed.', 'wp_admin', 'mbr-tickets' );
			$notification                   = $this->admin_notice( FALSE );
			die( json_encode( [ 'status' => 0, 'notification' => $notification, 'details' => '' ] ) );
		ENDIF;

		IF ( 'mbr-ticket' <> get_post_type( $_POST['ticket_id'] ) ) :
			$this->admin_notice['message']  = _x( 'Error. Ticket ID has wrong post type. Ticket status not changed.', 'wp_admin', 'mbr-tickets' );
			$notification                   = $this->admin_notice( FALSE );
			die( json_encode( [ 'status' => 0, 'notification' => $notification, 'details' => '' ] ) );
		ENDIF;

		wp_update_post( [ 'ID' => $_POST['ticket_id'], 'post_status' => $_POST['status'] ] );

		$details                          = apply_filters( 'mbr_tickets/ticket/details/html/get', $_POST['ticket_id'] );

		$this->admin_notice['type']       = 'success';
		$this->admin_notice['message']    = _x( 'Ticket status changed successfully.', 'wp_admin', 'mbr-tickets' );
		$notification                     = $this->admin_notice( FALSE );

		die( json_encode( [ 'status' => 1, 'notification' => $notification, 'details' => $details ] ) );
	}

	public function ajax_ticket_reply()
	{
		$this->admin_notice               = [];
		$this->admin_notice['type']       = 'error';

		IF ( empty( $_POST['ticket_id'] ) ) :
			$this->admin_notice['message']  = _x( 'Error. Empty Ticket ID.', 'wp_admin', 'mbr-tickets' );
			$notification                   = $this->admin_notice( FALSE );
			die( json_encode( [ 'status' => 0, 'notification' => $notification ] ) );
		ENDIF;

		IF ( 'mbr-ticket' <> get_post_type( $_POST['ticket_id'] ) ) :
			$this->admin_notice['message']  = _x( 'Error. Ticket ID has wrong post type.', 'wp_admin', 'mbr-tickets' );
			$notification                   = $this->admin_notice( FALSE );
			die( json_encode( [ 'status' => 0, 'notification' => $notification ] ) );
		ENDIF;

		$message = trim( $_POST['message'] );
		IF ( empty( $message ) ) :
			$this->admin_notice['message']  = _x( 'Error. Reply message is empty.', 'wp_admin', 'mbr-tickets' );
			$notification                   = $this->admin_notice( FALSE );
			die( json_encode( [ 'status' => 0, 'notification' => $notification ] ) );
		ENDIF;

		wp_update_post( [ 'ID' => $_POST['ticket_id'], 'post_status' => 'mbr_ticket_answered' ] );

		global $current_user;
		$arr_reply = [
			'ID'           => 0,
			'post_author'  => $current_user->ID,
			'post_content' => $message,
			'post_type'    => 'mbr-ticket-reply',
			'post_status'  => 'publish',
			'post_parent'  => $_POST['ticket_id']
		];

		IF ( wp_insert_post( $arr_reply ) ) :
			global $wpdb;
			$ticket_owner_id = $wpdb->get_var( "SELECT post_author FROM {$wpdb->posts} WHERE ID = {$_POST['ticket_id']} LIMIT 1" );
			$crm_note_id     = get_post_meta( $_POST['ticket_id'], 'mbr_crm_note_id', TRUE );
			$staff_name      = implode( ' ', $this->get_current_user_name() );
			IF ( ! empty( $crm_note_id ) ) :
				do_action( 'mbr_tickets/crm/note/edit', $ticket_owner_id, $crm_note_id, $arr_reply, '(' . __( 'Staff', 'mbr-tickets' ) . ') ' . $staff_name );
			ENDIF;
			do_action( 'mbr_tickets/crm/reply/staff/tag/add',                $ticket_owner_id );
			do_action( 'mbr_tickets/crm/reply/staff/automation/add',         $ticket_owner_id );
			do_action( 'mbr_tickets/notifications/staff/replied/email/send', $ticket_owner_id, $arr_reply, $staff_name );
		ENDIF;

		$details           = apply_filters( 'mbr_tickets/ticket/details/html/get', $_POST['ticket_id'] );
		$replies           = apply_filters( 'mbr_tickets/ticket/replies/html/get', $_POST['ticket_id'] );

		$this->admin_notice['type']     = 'success';
		$this->admin_notice['message']  = _x( 'Successfully replied.', 'wp_admin', 'mbr-tickets' );
		$notification                   = $this->admin_notice( FALSE );

		die( json_encode( [ 'status' => 1, 'notification' => $notification, 'replies' => $replies, 'details' => $details ] ) );
	}

	public function ajax_ticket_set_tags()
	{
		IF ( empty( $_POST['ticket_id'] ) ) :
			$this->admin_notice['message']  = _x( 'Error. Empty Ticket ID.', 'wp_admin', 'mbr-tickets' );
			$notification                   = $this->admin_notice( FALSE );
			die( json_encode( [ 'status' => 0, 'notification' => $notification ] ) );
		ENDIF;

		IF ( 'mbr-ticket' <> get_post_type( $_POST['ticket_id'] ) ) :
			$this->admin_notice['message']  = _x( 'Error. Ticket ID has wrong post type.', 'wp_admin', 'mbr-tickets' );
			$notification                   = $this->admin_notice( FALSE );
			die( json_encode( [ 'status' => 0, 'notification' => $notification ] ) );
		ENDIF;

		$tags_to_set                      = ! empty( $_POST['tags'] ) ? $_POST['tags'] : '';
		IF ( wp_set_post_terms( $_POST['ticket_id'], $tags_to_set, 'mbr_ticket_tag', FALSE ) ) :
			$this->admin_notice['type']     = 'success';
			$this->admin_notice['message']  = _x( 'Tags successfully set.', 'wp_admin', 'mbr-tickets' );
		ELSE:
			$this->admin_notice['type']     = 'error';
			$this->admin_notice['message']  = _x( 'Error. Could not set tags. Please try again.', 'wp_admin', 'mbr-tickets' );
		ENDIF;

		$notification                     = $this->admin_notice( FALSE );

		die( json_encode( [ 'status' => 1, 'notification' => $notification ] ) );
	}

	public function ajax_frontend_refresh_replies()
	{
		IF ( ! is_user_logged_in() OR  empty( $_POST['mbr_ticket_id'] ) ) :
			exit;
		ENDIF;

		IF ( ! isset( $_POST['mbr_last_reply_id'] ) ) :
			exit;
		ENDIF;

		global $wpdb, $current_user;

		IF ( ! $current_ticket = get_post( $_POST['mbr_ticket_id'] ) AND $current_ticket->post_author == $current_user->ID AND $current_ticket->post_type = 'mbr-ticket' ) :
			exit;
		ENDIF;

		$arr_latest_replies  = $wpdb->get_results( "SELECT * FROM {$wpdb->posts} WHERE post_parent = '{$_POST['mbr_ticket_id']}' AND ID > '{$_POST['mbr_last_reply_id']}' ORDER BY ID desc" );
		$count_new_replies   = count( $arr_latest_replies );

		$newest_replies      = '';
		$ticket_details      = '';
		$last_modified       = strtotime( $current_ticket->post_modified_gmt );
		$arr_customer_name   = $this->get_current_user_name();

		IF ( ! empty( $count_new_replies ) ) :

			FOREACH ( $arr_latest_replies AS $reply ) :
				IF ( empty( $last_reply_id ) ) :
					$last_reply_id = $reply->ID;
				ENDIF;
				$newest_replies .= $this->frontend_single_reply_html( $arr_customer_name, $reply, $current_ticket );
			ENDFOREACH;

		ELSE:
			$last_reply_id     = $_POST['mbr_last_reply_id'];
		ENDIF;

		IF ( $last_modified <> $_POST['mbr_last_modified'] ) :
			$ticket_details   .= $this->frontend_single_ticket_details( $arr_customer_name, $current_ticket );
		ENDIF;

		die( json_encode( [ 'last_reply_id' => $last_reply_id, 'newest_replies' => $newest_replies, 'last_modified' => $last_modified, 'ticket_details' => $ticket_details ] ) );
	}

	public function ajax_frontend_refresh_overview()
	{
		IF ( ! is_user_logged_in() OR  empty( $_POST['mbr_tickets_overview_last_modified'] ) ) :
			exit;
		ENDIF;

		global $wpdb, $current_user;

		$last_modified = $wpdb->get_var( "SELECT post_modified_gmt FROM {$wpdb->posts} WHERE post_author = {$current_user->ID} AND post_type = 'mbr-ticket' ORDER BY `post_modified_gmt` DESC LIMIT 1" );

		die( json_encode( [ 'last_modified' => strtotime( $last_modified ) ] ) );
	}

	public function heartbeat_received( $response, $data, $screen_id )
	{
		$response['mbr_tickets_refresh']['tickets']['refresh']       = FALSE;
		$response['mbr_tickets_refresh']['single_ticket']['refresh'] = FALSE;

		IF ( $screen_id == 'mbr-ticket' ) :
			IF ( $data['mbr_tickets_reply_check'] == 'true' ) :

				$ticket_id = ! empty( $data['wp-refresh-post-lock']['post_id'] ) ? $data['wp-refresh-post-lock']['post_id'] : FALSE;

				IF ( $ticket_id ) :
					$response['mbr_tickets_refresh']['single_ticket']['refresh'] = TRUE;
					$response['mbr_tickets_refresh']['single_ticket']['details']['display'] = $this->get_ticket_details_html( $ticket_id );
					$response['mbr_tickets_refresh']['single_ticket']['replies']['display'] = $this->get_ticket_replies_html( $ticket_id );
				ENDIF;
			ENDIF;
		ELSE:
			IF ( $data['mbr_tickets_check'] == 'true' ) :
				$response['mbr_tickets_refresh']['tickets'] = $this->heartbeat_tickets_refresh( $data['mbr_tickets_last_modified'] );
			ENDIF;
		ENDIF;

		return $response;
	}

	public function heartbeat_tickets_refresh( $user_last_modified = FALSE )
	{
		$return                               = [];
		$return['refresh']                    = FALSE;
		$return['display']                    = '';

		IF ( is_user_logged_in() AND ! empty( $user_last_modified )) :

			$last_modified = $this->ticket_last_modified();

			IF ( $last_modified <> $user_last_modified ) :
				$return['refresh']                = TRUE;

				$this->admin_notice['message']    = sprintf( _x( 'There was some activity in your ticket system. %s', 'wp_admin', 'mbr-tickets' ), '<a href="' . admin_url( 'admin.php?page=' . $this->page_parent ) . '">' . _x( 'Check your tickets now.', 'wp_admin', 'mbr-ticket' ) . '</a>' );
				$this->admin_notice['message']   .= $this->push_notification_send( get_bloginfo( 'name' ), _x( 'There was some activity in your ticket system.', 'wp_admin', 'mbr-tickets' ) );
				$this->admin_notice['type']       = 'info';

				$return['display']                = $this->admin_notice( FALSE );
			ENDIF;

			$return['last_modified']            = $last_modified;
		ENDIF;

		return $return;
	}

	public function ticket_last_modified()
	{
		global $wpdb;

		$last_modified                      = $wpdb->get_var( "SELECT post_modified_gmt FROM {$wpdb->posts} WHERE post_type = 'mbr-ticket' AND FIND_IN_SET(post_status, ('mbr_ticket_open') ) ORDER BY `post_modified_gmt` DESC LIMIT 1" );
		$last_modified                      = strtotime( $last_modified );

		return $last_modified;
	}

	public function extra_tablenav_top()
	{
		$arr_ticket_tags = get_terms( [ 'taxonomy' => 'mbr_ticket_tag', 'hide_empty' => TRUE ] );

		IF ( ! empty( $arr_ticket_tags ) ) :
			echo '<div id="mbr-wp-table-filter-box" class="alignleft actions">';
			echo '<label for="mbr-tickets-tag-filter" class="screen-reader-text">' . _x( 'Filter by tag', 'wp_admin', 'mbr-tickets' ) . '</label>';
			echo '<select id="mbr-tickets-tag-filter" name="filter">';
			echo '<option value="">' . _x( 'All', 'wp_admin', 'mbr-tickets' ) . '</option>';
			FOREACH ( $arr_ticket_tags AS $key => $obj_tag ) :
				echo '<option value="' . $obj_tag->term_id. '">' . $obj_tag->name . '</option>';
			ENDFOREACH;
			echo '</select>';
			echo '<span id="mbr-wp-table-filter" class="button">' . _x( 'Filter', 'wp_admin', 'mbr-tickets' ) . '</span>';
			echo '</div>';
		ENDIF;
	}

}

mbr_tickets_admin::getinstance();