<?php
IF ( ! defined( 'ABSPATH' ) ) :
	die( 'This file cannot be accessed directly.' );
ENDIF;

class mbr_tickets_frontend
{
	use common;

	public $arr_settings = [];

	public function __construct()
	{
		$this->arr_settings = get_option( 'mbr_tickets_settings', FALSE );

		add_filter( 'mbr/endpoints/rewrite',                               [ $this, 'rewrite_endpoints' ], 10, 1 );
		add_action( 'template_redirect',                                   [ $this, 'template_redirect' ] );

		IF ( is_user_logged_in() ) :
			add_action( 'wp_enqueue_scripts',                                [ $this, 'enqueue_scripts' ] );
			add_action( 'init',                                              [ $this, 'form_submission_ticket_create' ] );
			add_action( 'init',                                              [ $this, 'form_submission_ticket_reply' ] );

			IF ( class_exists( 'WooCommerce' ) ) :
				IF ( ! empty( $this->arr_settings['woocommerce_my_account_integration_enabled'] ) ) :
					add_filter( 'woocommerce_account_menu_items',                [ $this, 'woocommerce_add_my_account_support_link'], 9999, 1 );
				ENDIF;
			ENDIF;

		ENDIF;
	}

	public function enqueue_scripts()
	{
		wp_register_script( 'mbr-tickets-frontend', plugins_url( 'assets/js/mbr-tickets-frontend.js', dirname( __FILE__ ) ), [ 'jquery' ], MBR_TICKETS_VERSION, TRUE );
		wp_enqueue_script( 'mbr-tickets-frontend' );
		wp_localize_script( 'mbr-tickets-frontend', 'mbr_tickets', [ 'support_endpoint_main' =>  home_url( $this->arr_settings['frontend_endpoint_main'] ) ] );
	}

	public function rewrite_endpoints( $rewrite_endpoints = [] )
	{
		$rewrite_endpoints[ $this->arr_settings['frontend_endpoint_main'] ] = EP_ROOT;

		return $rewrite_endpoints;
	}

	/*
	 * triggers only if user is logged in
	 */
	public function template_redirect()
	{
		global $wp_query, $wp_rewrite, $current_user;

		$ticket_endpoints  = $this->rewrite_endpoints();

		$endpoint_confirmed = FALSE;
		FOREACH ( $wp_rewrite->endpoints AS $key => $arr_endpoint ) :
			IF ( array_key_exists( $arr_endpoint[1], $ticket_endpoints ) ) :
				$endpoint_confirmed = TRUE;
				break;
			ENDIF;
		ENDFOREACH;

		IF ( ! $endpoint_confirmed ) :
			return;
		ENDIF;

		IF ( ! isset( $wp_query->query_vars[ $this->arr_settings['frontend_endpoint_main'] ] ) ) :
			return;
		ENDIF;

		$is_user_logged_in   = FALSE;
		IF ( is_user_logged_in() ) :
			$is_user_logged_in = TRUE;
		ENDIF;

		$tmp_action                = explode('/', $wp_query->query_vars[ $this->arr_settings['frontend_endpoint_main'] ] );
		$action                    = '';
		$current_ticket_id         = 0;
		$redirect                  = FALSE;

		IF ( $is_user_logged_in ) :
			$meta_title              = sprintf( __( 'Dashboard | %s', 'mbr-tickets' ), get_bloginfo( 'name' ) );
		ELSE:
			$meta_title              = sprintf( __( 'Sign In | %s', 'mbr-tickets' ), get_bloginfo( 'name' ) );
		ENDIF;

		IF ( ! empty( $tmp_action[0] ) AND $tmp_action[0] == $this->arr_settings['frontend_endpoint_create'] ) :
			$action                  = 'create';
		ELSEIF ( ! empty( $tmp_action[0] ) AND $tmp_action[0] == $this->arr_settings['frontend_endpoint_edit'] ) :
			$action                  = 'edit';
		ELSEIF ( ! empty( $tmp_action[0] ) ):
			$action                  = 'redirect_to_main';
		ENDIF;

		IF ( $is_user_logged_in ) :
			IF ( $action == 'create' ) :
				$redirect              = ! empty( $tmp_action[1] ) ? TRUE : FALSE;
				$meta_title            = sprintf( __( 'Create a request | %s', 'mbr-tickets' ), get_bloginfo( 'name' ) );
			ELSEIF ( $action == 'edit' ) :
				IF ( ! empty( $tmp_action[1] ) AND is_numeric( $tmp_action[1] ) ) :
					IF ( $current_ticket = get_post( $tmp_action[1] ) AND $current_ticket->post_author == $current_user->ID AND $current_ticket->post_type = 'mbr-ticket' ) :
						$current_ticket_id = $tmp_action[1];
						$meta_title        = sprintf( __( 'Edit a request | %s', 'mbr-tickets' ), get_bloginfo( 'name' ) );
					ENDIF;
				ENDIF;
			ENDIF;
		ENDIF;

		IF ( $action == 'redirect_to_main' ) :
			wp_redirect( home_url( $this->arr_settings['frontend_endpoint_main'] ) );
			exit;
		ENDIF;

		IF ( $action == 'edit' ) :
			IF ( ! $is_user_logged_in OR empty( $current_ticket_id ) ) :
				wp_redirect( home_url( $this->arr_settings['frontend_endpoint_main'] ) );
				exit;
			ENDIF;
		ENDIF;

		IF ( $action == 'create' ) :
			IF ( ! $is_user_logged_in ) :
				wp_redirect( home_url( $this->arr_settings['frontend_endpoint_main'] ) );
				exit;
			ENDIF;

			IF ( $redirect ) :
				wp_redirect( home_url( $this->arr_settings['frontend_endpoint_main'] . '/' . $this->arr_settings['frontend_endpoint_create'] ) );
				exit;
			ENDIF;
		ENDIF;

		$replies_refresh                = 0;
		$current_ticket_last_reply_id   = 0;
		$current_ticket_last_modified   = 0;
		$tickets_overview_refresh       = 0;
		$tickets_overview_last_modified = 0;
		?>
		<!DOCTYPE html>
		<html <?php language_attributes(); ?> class="has-navbar-fixed-top has-navbar-fixed-bottom">
			<head>
				<meta charset="<?php bloginfo( 'charset' ); ?>" />
				<meta name="viewport" content="width=device-width, initial-scale=1">
				<title><?php echo $meta_title; ?></title>
				<?php IF ( ! empty( $this->arr_settings['frontend_template_favicon']['url'] ) ) : ?>
				<link rel="shortcut icon" href="<?php echo $this->arr_settings['frontend_template_favicon']['url']; ?>" />
				<?php ENDIF; ?>
				<link rel="stylesheet" href="<?php echo plugins_url( '/assets/css/mbr-tickets-frontend.css', dirname(__FILE__ ) ); ?>" type="text/css" media="all" />
				<link rel="stylesheet" href="<?php echo plugins_url( '/libs/bulma/bulma.css',                MBRNAME ); ?>" type="text/css" media="all" />
				<link rel="stylesheet" href="<?php echo plugins_url( '/libs/fontawesome/css/fontawesome-all.min.css', MBRNAME ); ?>" type="text/css" media="all" />
				<script type="text/javascript" src="<?php echo includes_url( '/js/jquery/jquery.js' ); ?>"></script>
			</head>
			<body>

				<nav class="navbar has-shadow is-fixed-top <?php echo $this->arr_settings['frontend_template_header_color_scheme']; ?>">
					<div class="navbar-brand">
						<?php IF ( ! empty( $this->arr_settings['frontend_template_logo']['url'] ) ) : ?>
						<a class="navbar-item" href="<?php echo esc_url( home_url( '/' ) ) ;?> ">
							<img src="<?php echo $this->arr_settings['frontend_template_logo']['url']; ?>" alt="" />
						</a>
						<?php ENDIF; ?>
						<div id="navbar-burger-id" class="navbar-burger">
							<span></span>
							<span></span>
							<span></span>
						</div>
					</div>

					<div id="navbar-menu-id" class="navbar-menu">

						<div class="navbar-start">
							<a class="navbar-item" href="<?php echo esc_url( home_url( '/' ) );?>"><?php _e( 'Home', 'mbr-tickets' ); ?></a>
							<?php IF ( $is_user_logged_in ) : ?>
							<a class="navbar-item" href="<?php echo esc_url( home_url( $this->arr_settings['frontend_endpoint_main'] ) ); ?>"><?php _e( 'Dashboard', 'mbr-tickets' ); ?></a>
							<?php ENDIF; ?>
						</div>

						<div class="navbar-end">
							<div class="navbar-item">
								<div class="field is-grouped">
									<p class="control">
										<?php IF ( $is_user_logged_in ) : ?>
										<a class="button is-primary" href="<?php echo esc_url( home_url( $this->arr_settings['frontend_endpoint_main'] . '/' . $this->arr_settings['frontend_endpoint_create'] ) ); ?>"><i class="fas fa-forward"></i>&nbsp;<span><?php _e( 'Submit a Request', 'mbr-tickets' ); ?></span></a>
										&nbsp;<a class="button is-danger" href="<?php echo wp_logout_url( esc_url( home_url( $this->arr_settings['frontend_endpoint_main'] ) ) ); ?>"><i class="fas fa-sign-out-alt"></i>&nbsp;<span><?php _e( 'Logout', 'mbr-tickets' ); ?></span></a>
										<?php ENDIF; ?>
									</p>
								</div>
							</div>
						</div>

					</div>

				</nav>

				<section class="section">
					<div class="container">

						<?php IF ( empty ( $is_user_logged_in ) ) : ?>
							<div class="box container-centered login">
								<h1 class="title"><?php _e( 'Sign In', 'mbr-tickets' ); ?></h1>

								<?php
								IF ( defined('WPE_APIKEY') ) :
									$action_target = '';
								ELSE :
									$action_target = site_url( 'wp-login.php', 'login' );
								ENDIF;
								?>
								<form method="post" id="mbr-tickets-login-form" action="<?php echo $action_target; ?>">
									<input type="hidden" name="redirect_to" value="<?php echo esc_url( home_url( $this->arr_settings['frontend_endpoint_main'] ) ); ?>" />
									<div class="field">
										<label class="label"><?php _e( 'Email', 'mbr-tickets' ); ?></label>
										<div class="control has-icons-left has-icons-right">
											<input class="input" type="email" name="log" placeholder="<?php _e( 'Enter your email address.', 'mbr-tickets' ); ?>" value="" />
											<span class="icon is-small is-left">
												<i class="fas fa-envelope"></i>
											</span>
											<span class="mbr-tickets-login-log-help icon is-small is-right is-hidden">
												<i class="fas fa-exclamation-triangle"></i>
											</span>
										</div>
										<p class="mbr-tickets-login-log-help help is-danger is-hidden"><?php _e( 'This field is required.', 'mbr-tickets' ); ?></p>
									</div>

									<div class="field">
										<label class="label"><?php _e( 'Password', 'mbr-tickets' ); ?></label>
										<div class="control has-icons-left has-icons-right">
											<input class="input " type="password" name="pwd" placeholder="<?php _e( 'Enter your password.', 'mbr-tickets' ); ?>" value="" />
											<span class="icon is-small is-left">
												<i class="fas fa-lock"></i>
											</span>
											<span class="mbr-tickets-login-pwd-help icon is-small is-right is-hidden">
												<i class="fas fa-exclamation-triangle"></i>
											</span>
										</div>
										<p class="mbr-tickets-login-pwd-help help is-danger is-hidden"><?php _e( 'This field is required.', 'mbr-tickets' ); ?></p>
									</div>
										<div class="control is-pulled-left">
											<label class="checkbox">
												<input type="checkbox" name="rememberme" value="forever" /> <?php _e( 'Remember Me', 'mbr-tickets' ); ?>
											</label>
										</div>
										<div class="control is-pulled-right">
											<input type="submit" name="wp-submit" class="button is-link" value="<?php _e( ' Sign In', 'mbr-tickets' ); ?>" />
										</div>
									<div class="is-clearfix"></div>
									<a href="<?php echo wp_lostpassword_url(); ?>"><?php _e( 'Lost your password?', 'mbr-tickets' ); ?></a>
								</form>
							</div>
						<?php ELSEIF ( empty( $action ) ) : ?>

						<?php
						IF ( ! empty( $_GET['status'] ) AND ! empty( $_GET['message'] ) ) :
							?>
							<div class="notification <?php echo $_GET['status']; ?>">
								<button class="delete"></button>
								<?php echo urldecode( $_GET['message'] ); ?>
							</div>
							<?php
						ENDIF;
						?>

						<h1 class="title"><?php _e( 'Dashboard', 'mbr-tickets' ); ?></h1>

						<div class="content">
							<table class="table is-responsive">

								<thead><tr><th><?php _e( 'ID', 'mbr-tickets' ); ?></th><th><?php _e( 'Subject', 'mbr-tickets' ); ?></th><th><?php _e( 'Created', 'mbr-tickets' ); ?></th><th><?php _e( 'Last activity', 'mbr-tickets' ); ?></th><th class="has-text-right"><?php _e( 'Status', 'mbr-tickets' ); ?></th></tr></thead>
								<tbody>
									<?php
									$tickets_overview_refresh = '1';
									$arr_tickets              = [];
									IF ( ! empty( $current_user->ID ) ) :
										$arr_tickets            = get_posts( [ 'post_type' => 'mbr-ticket', 'post_status' => 'any', 'author' => $current_user->ID, 'orderby' => 'modified', 'order' => 'DESC', 'posts_per_page' => -1 ] );
									ENDIF;
									IF ( ! empty( $arr_tickets ) ) :
										FOREACH ( $arr_tickets AS $key => $obj_ticket ) :
											IF ( empty( $tickets_overview_last_modified ) ) :
												$tickets_overview_last_modified = strtotime( $obj_ticket->post_modified_gmt );
											ENDIF;

											$status_class   = 'is-dark';
											IF ( $obj_ticket->post_status == 'mbr_ticket_answered' ) :
												$status_class = 'is-success';
											ELSEIF ( $obj_ticket->post_status == 'mbr_ticket_open' ) :
												$status_class = 'is-info';
											ELSEIF ( $obj_ticket->post_status == 'mbr_ticket_closed' ) :
												$status_class = 'is-danger';
											ENDIF;

											echo "<tr>
															<td>{$obj_ticket->ID}</td>
															<td><a href='" . esc_url( home_url( $this->arr_settings['frontend_endpoint_main'] . '/' . $this->arr_settings['frontend_endpoint_edit'] . '/' . $obj_ticket->ID ) ) . "'>{$obj_ticket->post_title}</a></td>
															<td>" . $this->convert_datetime( $obj_ticket->post_date_gmt, TRUE ) . "</td>
															<td>" . $this->convert_datetime( $obj_ticket->post_modified_gmt, TRUE ) . "</td>
															<td class='has-text-right'><span style='width:100%;' class='tag " . $status_class . "'>" . MBR_TICKETS_STATUSES[ $obj_ticket->post_status]['label_overview'] . "</span></td>
														</tr>";
										ENDFOREACH;
									ELSE:
										?>
										<tr><td colspan="5"><?php _e( 'You have not made any requests so far.', 'mbr-tickets' ); ?></td></tr>
										<?php
									ENDIF;
									?>
								</tbody>

							</table>
						</div>
						<?php ELSEIF ( $action == 'create') : ?>

						<?php
						IF ( ! empty( $_GET['status'] ) AND ! empty( $_GET['message'] ) ) :
							?>
							<div class="notification <?php echo $_GET['status']; ?>">
								<button class="delete"></button>
								<?php echo urldecode( $_GET['message'] ); ?>
							</div>
						<?php
						ENDIF;
						?>

						<h1 class="title"><?php _e( 'Submit a Ticket', 'mbr-tickets' ); ?></h1>
						<div class="container">
							<div class="columns">

								<div class="column is-half">
									<form id="mbr-tickets-create-form" method="post" action="">
										<?php echo wp_nonce_field('mbr_tickets_handle_ticket_create', 'mbr_tickets_nonce_ticket_create' ); ?>

										<div class="field">
											<label class="label"><?php _e( 'Subject *', 'mbr-tickets' ); ?></label>
											<div class="control has-icons-left has-icons-right">
												<input name="mbr_tickets_subject" class="input" type="text" placeholder="<?php _e( 'Enter your ticket subject.', 'mbr-tickets' ); ?>" value="" />
												<span class="icon is-small is-left">
													<i class="fas fa-comment"></i>
												</span>
												<span class="mbr-tickets-subject-help icon is-small is-right is-hidden">
													<i class="fas fa-exclamation-triangle"></i>
												</span>
											</div>
											<p class="mbr-tickets-subject-help help is-danger is-hidden"><?php _e( 'This field is required.', 'mbr-tickets' ); ?></p>
										</div>

										<div class="field">
											<label class="label"><?php _e( 'Description *', 'mbr-tickets' ); ?></label>
											<div class="control">
												<textarea name="mbr_tickets_description" class="textarea" rows="10" placeholder="<?php _e( 'Enter your ticket description.', 'mbr-tickets' ); ?>"></textarea>
											</div>
											<p class="mbr-tickets-description-help help is-danger is-hidden"><?php _e( 'This field is required.', 'mbr-tickets' ); ?></p>
										</div>

										<div class="field is-grouped">
											<div class="control">
												<button class="button is-link"><?php _e( 'Submit Request', 'mbr-tickets' ); ?></button>
											</div>
											<div class="control">
												<a href="<?php echo esc_url( home_url( $this->arr_settings['frontend_endpoint_main'] ) ); ?>" class="button is-text"><?php _e( 'Cancel', 'mbr-tickets' ); ?></a>
											</div>
										</div>
									</form>
								</div>
								<div class="column">
									<?php IF ( ! empty( $this->arr_settings['frontend_template_create_ticket_user_info_enabled'] ) AND ! empty( $this->arr_settings['frontend_template_create_ticket_user_info_text'] )  ) : ?>
									<div class="notification <?php echo $this->arr_settings['frontend_template_create_ticket_user_info_color_scheme']; ?>"><?php echo nl2br( $this->arr_settings['frontend_template_create_ticket_user_info_text'] ); ?></div>
									<?php ENDIF; ?>
								</div>

							</div>
						</div>
						<?php ELSEIF ( $action == 'edit') : ?>

						<?php
							IF ( ! empty( $_GET['status'] ) AND ! empty( $_GET['message'] ) ) :
							?>
							<div class="notification <?php echo $_GET['status']; ?>">
								<button class="delete"></button>
								<?php echo urldecode( $_GET['message'] ); ?>
							</div>
						<?php
						ENDIF;
						?>

						<h1 class="title"><?php echo sprintf( __( 'Request #%s', 'mbr-tickets' ), $current_ticket_id ); ?></h1>
						<div class="container">
							<div class="columns">

								<div class="column is-three-quarters">
									<div class="notification is-light">
										<h2 class="title is-4"><?php echo $current_ticket->post_title; ?></h2>
										<p><?php echo nl2br( $current_ticket->post_content ); ?></p>
									</div>
									<hr />

									<button id="mbr-tickets-reply-form-toggle" class="button is-link"><span id="mbr-tickets-reply-form-toggle-icon"><?php _e( 'Reply', 'mbr-tickets' ); ?></span></button>

									<form id="mbr-tickets-reply-form" class="is-hidden" method="post" action="">
										<br />
										<?php echo wp_nonce_field('mbr_tickets_handle_ticket_reply', 'mbr_tickets_nonce_ticket_reply' ); ?>
										<input name="mbr_tickets_reply_ticket_id" type="hidden" value="<?php echo $current_ticket_id; ?>" />
										<div class="field">
											<div class="control">
												<textarea name="mbr_tickets_reply" class="textarea" rows="10" placeholder="<?php _e( 'Enter your ticket reply.', 'mbr-tickets' ); ?>"></textarea>
											</div>
											<p id="mbr-tickets-reply-help" class="help is-danger is-hidden"><?php _e( 'This field is required.', 'mbr-tickets' ); ?></p>
										</div>

										<div class="field is-grouped">
											<div class="control">
												<button id="mbr-tickets-submit-reply" class="button is-link"><?php _e( 'Submit', 'mbr-tickets' ); ?></button>
											</div>
										</div>
									</form>
									<hr id="mbr-tickets-replies" />
									<?php
									$replies_refresh            = '1';
									$arr_customer_name          = $this->get_current_user_name();

									$arr_replies                = get_posts( [ 'post_parent' => $current_ticket_id, 'post_type' => 'mbr-ticket-reply', 'orderby' => 'date', 'order' => 'desc', 'numberposts' => -1 ] );
									IF ( ! empty( $arr_replies ) ) :
										FOREACH ( $arr_replies AS $reply ) :
											IF ( empty( $current_ticket_last_reply_id ) ) :
												$current_ticket_last_reply_id = $reply->ID;
											ENDIF;
											echo $this->frontend_single_reply_html( $arr_customer_name, $reply, $current_ticket );
										ENDFOREACH;
									ELSE:
										?>
										<div id="mbr-tickets-no-replies-yet" class="box notification <?php echo $this->arr_settings['frontend_template_ticket_replies_staff_color_scheme']; ?>">
											<?php _e( 'No replies yet.', 'mbr-tickets' ); ?>
										</div>
									<?php
									ENDIF;
									?>
								</div>

								<div class="column">
									<?php
									$current_ticket_last_modified  = strtotime( $current_ticket->post_modified_gmt );
									?>
									<div id="mbr-tickets-ticket-details" class="notification is-light">
										<?php echo $this->frontend_single_ticket_details( $arr_customer_name, $current_ticket ); ?>
									</div>
								</div>
							</div>
						</div>
						<?php ENDIF; ?>

					</div>
				</section>

				<?php	IF ( ! empty( $this->arr_settings['frontend_template_footer_enabled'] AND ! empty( $this->arr_settings['frontend_template_footer_text'] ) ) ) : ?>
				<footer class="navbar is-fixed-bottom has-shadow <?php echo $this->arr_settings['frontend_template_footer_color_scheme']; ?>">
					<div class="container content has-text-centered container-centered">
						<p><small><?php echo $this->arr_settings['frontend_template_footer_text']; ?></small></p>
					</div>
				</footer>
				<?php ENDIF; ?>

				<script type="text/javascript">
					/* <![CDATA[ */
					var mbr_tickets = {
						"is_endpoint"                           : 1,
						"ajax_url"                              : "<?php echo admin_url( 'admin-ajax.php' ); ?>",
						"replies_refresh"                       : <?php echo $replies_refresh; ?>,
						"replies_refresh_interval"              : <?php echo ( $this->arr_settings['frontend_ticket_replies_refresh_interval'] * 1000 ); ?>,
						"current_ticket_id"                     : <?php echo $current_ticket_id; ?>,
						"current_ticket_last_reply_id"          : <?php echo $current_ticket_last_reply_id; ?>,
						"current_ticket_last_modified"          : <?php echo $current_ticket_last_modified; ?>,
						"tickets_overview_refresh"              : <?php echo $tickets_overview_refresh; ?>,
						"tickets_overview_refresh_interval"     : <?php echo ( $this->arr_settings['frontend_ticket_overview_refresh_interval'] * 1000 ); ?>,
						"tickets_overview_last_modified"        : <?php echo $tickets_overview_last_modified; ?>,
						"tickets_overview_refresh_notification" : "<?php echo '<a href=\"' . esc_url( home_url( $this->arr_settings['frontend_endpoint_main'] ) ) . '\" style=\"text-decoration:none;\"><i class=\"fas fa-sync\"></i>&nbsp;&nbsp;' . __( 'Request updates available.', 'mbr-tickets' ) . '</a>'; ?>",
						"audio_notification_enabled"            : <?php echo $this->arr_settings['audio_reply_notifications_enabled']; ?>,
						"audio_files_directory"                 : "<?php echo plugins_url( 'assets/audio/', dirname( __FILE__ ) ); ?>",
						"push_notifications_enabled"            : <?php echo $this->push_notifications(); ?>,
						"push_icon_url"                         : "<?php echo ( ! empty( $this->arr_settings['push_notifications_icon']['url'] ) ? $this->arr_settings['push_notifications_icon']['url'] : '' ); ?>",
						"push_reply_title"                      : "<?php _e( 'Reply posted for ticket #%s', 'mbr-tickets' ); ?>",
						"push_reply_body"                       : "<?php _e( 'Our support team has posted a reply. Please check your request.', 'mbr-tickets' ); ?>",
						"push_overview_title"                   : "<?php _e( 'Requests have been changed. ', 'mbr-tickets' ); ?>",
						"push_overview_body"                    : "<?php _e( 'Your requests have been changed. Please refresh your dashboard.', 'mbr-tickets' ); ?>"
					};
					/* ]]> */
				</script>
				<script type="text/javascript" src="<?php echo plugins_url( '/assets/js/mbr-tickets.js',          dirname(__FILE__ ) ); ?>"></script>
				<script type="text/javascript" src="<?php echo plugins_url( '/assets/js/mbr-tickets-frontend.js', dirname(__FILE__ ) ); ?>"></script>
			</body>
		</html>
		<?php
		exit;
	}

	public function woocommerce_add_my_account_support_link( $items )
	{
		$logout                   = $items['customer-logout'];
		unset( $items['customer-logout'] );

		$items['#mbr-support']    = __( 'Support', 'mbr-tickets' );
		$items['customer-logout'] = $logout;

		return $items;
	}

	public function form_submission_ticket_create()
	{
		IF ( ! empty( $_POST['mbr_tickets_nonce_ticket_create'] ) ) :
			IF ( ! wp_verify_nonce( $_POST['mbr_tickets_nonce_ticket_create'], 'mbr_tickets_handle_ticket_create' ) ) :
				wp_die( __( 'You are not authorized to perform this action.', 'mbr-tickets' ) );
			ELSE:

				IF ( empty( $_POST['mbr_tickets_subject'] ) OR empty( $_POST['mbr_tickets_description'] ) ) :
					wp_redirect( add_query_arg( [ 'status' => 'is-danger', 'message' => urlencode( __( 'Please complete your inputs.', 'mbr-tickets' ) ) ], home_url( $this->arr_settings['frontend_endpoint_main'] . '/' .  $this->arr_settings['frontend_endpoint_create'] ) ) );
					exit;
				ENDIF;

				global $current_user;
				$arr_ticket = [
					'ID'           => 0,
					'post_author'  => $current_user->ID,
					'post_title'   => $_POST['mbr_tickets_subject'],
					'post_content' => $_POST['mbr_tickets_description'],
					'post_type'    => 'mbr-ticket',
					'post_status'  => 'mbr_ticket_open',
				];

				IF ( $inserted_id = wp_insert_post( $arr_ticket ) ) :
					do_action( 'mbr_tickets/crm/note/add', $current_user->ID, $inserted_id, $arr_ticket );
					wp_redirect( add_query_arg( [ 'status' => 'is-success', 'message' => urlencode( __( 'Ticket created successfully.', 'mbr-tickets' ) ) ], home_url( $this->arr_settings['frontend_endpoint_main'] ) ) );
					exit;
				ELSE:
					wp_redirect( add_query_arg( [ 'status' => 'is-danger',  'message' => urlencode( __( 'An error occured. Could not create your ticket. Please try again.', 'mbr-tickets' ) ) ], home_url( $this->arr_settings['frontend_endpoint_main'] ) ) );
					exit;
				ENDIF;

			ENDIF;
		ENDIF;
	}

	public function form_submission_ticket_reply()
	{
		IF ( ! empty( $_POST['mbr_tickets_nonce_ticket_reply'] ) ) :
			IF ( ! wp_verify_nonce( $_POST['mbr_tickets_nonce_ticket_reply'], 'mbr_tickets_handle_ticket_reply' ) ) :
				wp_die( __( 'You are not authorized to perform this action.', 'mbr-tickets' ) );
			ELSE:

				IF ( empty( $_POST['mbr_tickets_reply_ticket_id'] ) ) :
					wp_redirect( add_query_arg( [ 'status' => 'is-danger', 'message' => urlencode( __( 'An error occured. No request id provided.', 'mbr-tickets' ) ) ], home_url( $this->arr_settings['frontend_endpoint_main'] ) ) );
					exit;
				ENDIF;

				$ticket_id           = $_POST['mbr_tickets_reply_ticket_id'];
				$current_ticket_page = $this->arr_settings['frontend_endpoint_main'] . '/' . $this->arr_settings['frontend_endpoint_edit'] . '/' . $ticket_id;

				IF ( empty( $_POST['mbr_tickets_reply'] ) ) :
					wp_redirect( add_query_arg( [ 'status' => 'is-danger', 'message' => urlencode( __( 'Please complete your inputs.', 'mbr-tickets' ) ) ], home_url( $current_ticket_page ) ) );
					exit;
				ENDIF;

				wp_update_post( [ 'ID' => $ticket_id, 'post_status' => 'mbr_ticket_open' ] );

				global $current_user;
				$arr_reply = [
					'ID'           => 0,
					'post_author'  => $current_user->ID,
					'post_content' => $_POST['mbr_tickets_reply'],
					'post_type'    => 'mbr-ticket-reply',
					'post_status'  => 'publish',
					'post_parent'  => $ticket_id
				];

				IF ( $inserted_id = wp_insert_post( $arr_reply ) ) :
					$crm_note_id = get_post_meta( $ticket_id, 'mbr_crm_note_id', TRUE );
					IF ( ! empty( $crm_note_id ) ) :
						$customer_name = implode( ' ', $this->get_current_user_name() );
						do_action( 'mbr_tickets/crm/note/edit', $current_user->ID, $crm_note_id, $arr_reply, '(' . __( 'Contact', 'mbr-tickets' ) . ') ' . $customer_name );
					ENDIF;
					wp_redirect( add_query_arg( [ 'status' => 'is-success', 'message' => urlencode( __( 'Successfully replied.', 'mbr-tickets' ) ) ], home_url( $current_ticket_page ) ) );
					exit;
				ELSE:
					wp_redirect( add_query_arg( [ 'status' => 'is-danger',  'message' => urlencode( __( 'An error occured. Could not create reply. Please try again.', 'mbr-tickets' ) ) ], home_url( $current_ticket_page ) ) );
					exit;
				ENDIF;

			ENDIF;
		ENDIF;
	}

}

mbr_tickets_frontend::getinstance();