<?php
IF ( ! defined( 'ABSPATH' ) ) :
	die( 'This file cannot be accessed directly.' );
ENDIF;

IF ( ! class_exists( 'WP_List_Table' ) ) :
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
ENDIF;

IF ( !class_exists( 'MBR_LIST_TABLE' ) ) :

	class MBR_LIST_TABLE extends WP_LIST_TABLE {

		public $mbr_data;
		public $mbr_columns;
		public $mbr_sortable_columns;
		public $mbr_per_page;
		public $mbr_total_entries;
		public $mbr_search_label;
		public $mbr_post_statuses;

		public function __construct( $mbr_data, $mbr_columns, $mbr_per_page = FALSE, $mbr_sortable_columns = [], $mbr_total_entries = 0, $mbr_search_label = '', $mbr_post_statuses = [] )
		{
			$this->mbr_data             = $mbr_data;
			$this->mbr_columns          = $mbr_columns;
			$this->mbr_per_page         = $mbr_per_page;
			$this->mbr_sortable_columns = $mbr_sortable_columns;
			$this->mbr_total_entries    = $mbr_total_entries;
			$this->mbr_search_label     = ! empty( $mbr_search_label ) ? $mbr_search_label : __( 'Search' );
			$this->mbr_post_statuses    = $mbr_post_statuses;

			parent::__construct( [ 'ajax' => FALSE ] );
		}

		public static function create_table( $form_id, $history_id, $display_action, $history_action, $default_order, $default_orderby, $default_post_status, $search_results_label, $localize_handle )
		{
			wp_localize_script( $localize_handle, 'mbr_table', [ 'form_id' => $form_id, 'history_id' => $history_id, 'display_action' => $display_action, 'history_action' => $history_action, 'order' => $default_order, 'orderby' => $default_orderby, 'post_status' => $default_post_status, 'search_results_label' => $search_results_label ] );
			ob_start();
			?>
			<form id="<?php echo $form_id; ?>" method="get">

				<div id="<?php echo $history_id; ?>">
					<input type="hidden" name="order"       value="<?php echo $default_order; ?>" />
					<input type="hidden" name="orderby"     value="<?php echo $default_orderby; ?>" />
					<input type="hidden" name="post_status" value="<?php echo $default_post_status; ?>" />
					<!-- placeholder for initial call -->
					<table class="wp-list-table widefat fixed striped mbr-wp-list-table-placeholder">
						<thead><tr><th scope="col">&nbsp;</th></tr></thead>
						<tbody id="the-list"><tr><td class="mbr-wp-list-table-loading-bg">&nbsp;</td></tr></tbody>
						<tfoot><tr><th scope="col">&nbsp;</th></tr></tfoot>
					</table>
				</div>

			</form>
			<?php
			$html = ob_get_contents();
			ob_end_clean();

			return $html;
		}

		public function get_views()
		{
			$status_links                   = [];
			IF ( ! empty( $this->mbr_post_statuses ) ) :
				FOREACH ( $this->mbr_post_statuses AS $post_status => $post_link ) :
					$status_links[ $post_status ] = '<a class="mbr-list-post-status" id="' . $post_status . '" href="#">' . $post_link . '</a>';
				ENDFOREACH;
			ENDIF;

			return $status_links;
		}

		public function get_columns()
		{
			return $this->mbr_columns;
		}

		public function prepare_items()
		{
			$columns               = $this->get_columns();
			$hidden                = [];
			$sortable              = ! empty( $this->mbr_sortable_columns ) ? $this->get_sortable_columns() : [];

			IF ( empty( $this->mbr_per_page ) ) :
				$this->mbr_per_page  = 999999;
			ENDIF;

			function usort_reorder( $a, $b ) {
				$orderby = ( ! empty( $_REQUEST['orderby'] ) ) ? $_REQUEST['orderby'] : '';
				$order   = ( ! empty( $_REQUEST['order'] ) )   ? $_REQUEST['order']   : 'desc';
				$result  = strnatcasecmp( $a[ $orderby ], $b[ $orderby ] );
				return ( 'asc' === $order ) ? $result : -$result;
			}
			usort( $this->mbr_data , 'usort_reorder' );

			$total_items           = $this->mbr_total_entries;

			$this->_column_headers = [ $columns, $hidden, $sortable ];
			$this->items           = $this->mbr_data;

			$this->set_pagination_args(
				[
					'total_items' => $total_items,
					'per_page'    => $this->mbr_per_page,
					'total_pages' => ceil( $total_items / $this->mbr_per_page ),
					'orderby'     => ! empty( $_REQUEST['orderby'] )     AND '' <> $_REQUEST['orderby']     ? $_REQUEST['orderby']     : '',
					'order'       => ! empty( $_REQUEST['order'] )       AND '' <> $_REQUEST['order']       ? $_REQUEST['order']       : 'desc',
					'post_status' => ! empty( $_REQUEST['post_status'] ) AND '' <> $_REQUEST['post_status'] ? $_REQUEST['post_status'] : ''
				]
			);
		}

		public function get_sortable_columns()
		{
			return $this->mbr_sortable_columns;
		}

		public function column_default( $item, $column_name )
		{
			switch( $column_name ) {
				default:
					return $item[ $column_name ];
			}
		}

		public function display() {
			wp_nonce_field( 'mbr_wp_list_table', 'mbr_wp_list_table_nonce' );

			echo '<input type="hidden" id="order"       name="order"       value="' . $this->_pagination_args['order']       . '" />';
			echo '<input type="hidden" id="orderby"     name="orderby"     value="' . $this->_pagination_args['orderby']     . '" />';
			echo '<input type="hidden" id="post_status" name="post_status" value="' . $this->_pagination_args['post_status'] . '" />';
			echo '<span id="mbr-wp-list-table-status-filter">';
			$this->status_filter();
			echo '</span>';

			echo '<span id="mbr-wp-list-table-search-box">';
			$this->search_box( $this->mbr_search_label, 'mbr-table' );
			echo '</span>';

			parent::display();
		}

		public function extra_tablenav( $which )
		{
			IF ( $which == 'top' ) :
				do_action( 'mbr_list_tables/extra_tablenav/top' );
			ENDIF;
		}

		public function ajax_response() {
			check_ajax_referer( 'mbr_wp_list_table', 'mbr_wp_list_table_nonce' );

			$this->prepare_items();

			extract( $this->_args );
			extract( $this->_pagination_args, EXTR_SKIP );

			ob_start();
			$this->search_box( $this->mbr_search_label, 'mbr-table' );
			$search_box = ob_get_clean();

			ob_start();
			IF ( ! empty( $_REQUEST['no_placeholder'] ) ) :
				$this->display_rows();
			ELSE:
				$this->display_rows_or_placeholder();
			ENDIF;
			$rows              = ob_get_clean();

			ob_start();
			$this->status_filter();
			$filters           = ob_get_clean();

			ob_start();
			$this->print_column_headers();
			$headers           = ob_get_clean();

			ob_start();
			$this->pagination('top');
			$pagination_top    = ob_get_clean();

			ob_start();
			$this->pagination('bottom');
			$pagination_bottom = ob_get_clean();

			$response                         = [ 'rows' => $rows ];
			$response['status_filter']        = $filters;
			$response['search_box']           = $search_box;
			$response['pagination']['top']    = $pagination_top;
			$response['pagination']['bottom'] = $pagination_bottom;
			$response['column_headers']       = $headers;

			IF ( isset( $total_items ) ) :
				$response['total_items_i18n']   = sprintf( _n( '1 item', '%s items', $total_items ), number_format_i18n( $total_items ) );
			ENDIF;

			IF ( isset( $total_pages ) ) :
				$response['total_pages']        = $total_pages;
				$response['total_pages_i18n']   = number_format_i18n( $total_pages );
			ENDIF;

			die( json_encode( $response ) );
		}

		public function status_filter()
		{
			$views = $this->get_views();
			IF ( empty( $views ) ) :
				return;
			ENDIF;

			$this->views();
		}

		public static function ajax_history_callback( $array )
		{
			$wp_list_table = new MBR_LIST_TABLE( $array['data'], $array['columns'], $array['per_page'], $array['sortable_columns'], $array['total'], $array['search_label'], $array['status_filter'] );
			$wp_list_table->ajax_response();
		}

		public static function ajax_display_callback( $array )
		{
			$wp_list_table = new MBR_LIST_TABLE( $array['data'], $array['columns'], $array['per_page'], $array['sortable_columns'], $array['total'], $array['search_label'], $array['status_filter'] );
			$wp_list_table->prepare_items();

			ob_start();
			$wp_list_table->display();
			$display       = ob_get_clean();

			die( json_encode( [ 'display' => $display ] ) );
		}

	}

ENDIF;