<?php
trait common {

	public function __destruct()  { }
	public function __clone()     { _doing_it_wrong( __FUNCTION__ , __( 'Cheatin&#8217; huh?', 'mbr-tickets' ), '1.0' ); }
	public function __wakeup()    { _doing_it_wrong( __FUNCTION__ , __( 'Cheatin&#8217; huh?', 'mbr-tickets' ), '1.0' ); }

	public static function getinstance() {
		static $instance = NULL;

		IF ( NULL === $instance ) :
			$instance = new self();
		ENDIF;

		return $instance;
	}

	/**
	 * Defines a constant if not already defined.
	 *
	 * @param string $name The name of the constant to be defined.
	 * @param bool|string|array|integer $value The value of the constant to be defined.
	 *
	 * @return void
	 */
	public function define( $name, $value )
	{
		IF ( ! defined( $name ) ) :
			define( $name, $value );
		ENDIF;
	}

	public function convert_datetime( $datetime, $time = FALSE )
	{
		IF ( ! is_numeric( $datetime ) OR (int) $datetime !== $datetime ) :
			$datetime = strtotime( $datetime )  + ( get_option( 'gmt_offset' ) * 3600 );
		ELSE:
			$datetime = $datetime + ( get_option( 'gmt_offset' ) * 3600 );
		ENDIF;

		$date       = date_i18n( get_option( 'date_format' ), $datetime );
		IF ( $time ) :
			$date    .= ' ' . date_i18n( get_option( 'time_format' ), $datetime );
		ENDIF;

		return $date;
	}

	public function push_notifications()
	{
		static $has_run                   = NULL;
		static $push_notification_enabled = 0;

		IF ( $has_run ) :
			return $push_notification_enabled;
		ENDIF;

		$has_run                          = TRUE;

		$push_notification_enabled        = 0;
		IF ( ! empty( $this->arr_settings['push_notifications_enabled'] ) ) :
			$push_notification_enabled      = 1;
		ENDIF;

		IF ( $push_notification_enabled ) :
			IF ( ! is_ssl() ) :
				$push_notification_enabled    = 0;
			ENDIF;
		ENDIF;

		return $push_notification_enabled;
	}

	public function push_notification_send( $headline, $body )
	{
		IF ( ! $this->push_notifications() ) :
			return '';
		ENDIF;

		IF ( empty( $headline ) OR empty( $body ) ) :
			return '';
		ENDIF;

		$icon   = '';
		IF ( ! empty( $this->arr_settings['push_notifications_icon']['url'] ) ) :
			$icon = $this->arr_settings['push_notifications_icon']['url'];
		ENDIF;

		IF ( empty( $icon ) ) :
			return '<script>mbr_push_send( "' . $headline . '", "' . $body . '" )</script>';
		ENDIF;

		return '<script>mbr_push_send( "' . $headline . '", "' . $body . '", "' . $icon . '" )</script>';
	}

	public function get_current_user_name()
	{
		global $current_user;

		IF ( ! is_user_logged_in() ) :
			return [];
		ENDIF;

		$arr_current_user_name      = [];
		IF ( ! empty( $current_user->first_name ) ) :
			$arr_current_user_name[]  = $current_user->first_name;
		ENDIF;
		IF ( ! empty( $current_user->last_name ) ) :
			$arr_current_user_name[]  = $current_user->last_name;
		ENDIF;

		return $arr_current_user_name;
	}

	public function frontend_single_reply_html($arr_customer, $reply, $current_ticket )
	{
		IF ( empty( $arr_customer) OR empty( $reply ) OR empty( $current_ticket) ) :
			return '';
		ENDIF;

		IF ( $reply->post_author <> $current_ticket->post_author ) :
			$status_class  = $this->arr_settings['frontend_template_ticket_replies_staff_color_scheme'];
			$staff         = get_userdata( $reply->post_author );
			$arr_staff     = [];
			IF ( ! empty( $staff->first_name ) ) :
				$arr_staff[] = $staff->first_name;
			ENDIF;
			IF ( ! empty( $staff->last_name ) ) :
				$arr_staff[] = $staff->last_name;
			ENDIF;
			$name          = '<strong>(' . __( 'Staff', 'mbr-tickets' ) . ')</strong> ' . implode( ' ', $arr_staff );
		ELSE:
			$status_class  = $this->arr_settings['frontend_template_ticket_replies_contact_color_scheme'];
			$name          = '<strong>(' . __( 'Me', 'mbr-tickets' ) . ')</strong> ' . implode( ' ', $arr_customer );
		ENDIF;

		/*
		$html            = '<div class="notification ' . $status_class . '">';
		$html           .= '<div class="is-pulled-left"><small>' . $name . '</small></div><div class="is-pulled-right"><small>' . $this->convert_datetime( $reply->post_date_gmt, TRUE ) . '</small></div>';
		$html           .= '<div class="is-clearfix"></div>';
		$html           .= nl2br( $reply->post_content );
		$html           .= '</div>';
		*/

		$html  = '<div class="box notification ' . $status_class . '">';
		$html .= '<article class="media">';
		$html .= '<div class="media-left">';
		$html .= '<figure class="image is-64x64">';
		$html .= get_avatar(  $reply->post_author, 64 );
		$html .= '</figure>';
		$html .= '</div>';
		$html .= '<div class="media-content">';
		$html .= '<div class="content">';
		$html .= '<p>';
		$html .= '<strong>' . $name . '</strong> <small>' . $this->convert_datetime( $reply->post_date_gmt, TRUE ) . '</small><br />';
		$html .= nl2br( $reply->post_content );
		$html .= '</p>';
		$html .= '</div>';
		$html .= '</div>';
		$html .= '</article>';
		$html .= '</div>';

		return $html;
	}

	public function frontend_single_ticket_details( $arr_customer_name, $current_ticket )
	{
		$status_class      = 'is-dark';
		IF ( $current_ticket->post_status == 'mbr_ticket_answered' ) :
			$status_class    = 'is-success';
		ELSEIF ( $current_ticket->post_status == 'mbr_ticket_open' ) :
			$status_class    = 'is-info';
		ELSEIF ( $current_ticket->post_status == 'mbr_ticket_closed' ) :
			$status_class    = 'is-danger';
		ENDIF;

		$ticket_details    = '<p><small>' . sprintf( __( '%s submitted this request.', 'mbr-tickets'), implode( ' ', $arr_customer_name) ) . '</small></p>';
		$ticket_details   .= '<br />';
		$ticket_details   .= '<div class="is-pulled-left"><strong>' . __( 'Status',  'mbr-tickets' ) . ':</strong></div><div class="is-pulled-right tag ' . $status_class . '">' . MBR_TICKETS_STATUSES[ $current_ticket->post_status]['label_overview'] . '</div>';
		$ticket_details   .= '<div class="is-clearfix"></div>';
		$ticket_details   .= '<br />';
		$ticket_details   .= '<div class="is-pulled-left"><strong>' . __( 'Request', 'mbr-tickets' ) . ':</strong></div><div class="is-pulled-right"><small>#' . $current_ticket->ID . '</small></div>';
		$ticket_details   .= '<div class="is-clearfix"></div>';
		$ticket_details   .= '<div class="is-pulled-left"><strong>' . __( 'Created', 'mbr-tickets' ) . ':</strong></div><div class="is-pulled-right"><small>'  . $this->convert_datetime( $current_ticket->post_date_gmt, TRUE ) . '</small></div>';
		$ticket_details   .= '<div class="is-clearfix"></div>';
		$ticket_details   .= '<div class="is-pulled-left"><strong>' . __( 'Updated', 'mbr-tickets' ) . ':</strong></div><div class="is-pulled-right"><small>'  . $this->convert_datetime( $current_ticket->post_modified_gmt, TRUE ) . '</small></div>';
		$ticket_details   .= '<div class="is-clearfix"></div>';

		return $ticket_details;
	}

	public function get_heartbeat_interval( $settings )
	{
		$this->heartbeat_interval = $settings['interval'];

		return $settings;
	}

}