<?php
/*
 * Plugin Name: MBR Tickets
 */

IF ( ! defined( 'ABSPATH' ) ) :
	die( 'This file cannot be accessed directly.' );
ENDIF;

define( 'MBR_TICKETS_ABSPATH', dirname( __FILE__ ) . '/' );

TRY {
	IF ( ! ( include_once( MBR_TICKETS_ABSPATH . 'includes/trait.common.php' ) ) ) :
		THROW NEW Exception( sprintf( _x( 'Error: Could not include required file %s.', 'error', 'mbr-tickets' ), '"trait.common.php"' ) );
	ENDIF;
} CATCH ( Exception $e ) {
	wp_die( $e->getMessage() );
}

class mbr_tickets {
	use common;

	public $ticket_post_statuses;

	public function __construct()
	{
		IF ( defined( 'MBRVERSION' ) ) :
			$this->define( 'MBR_TICKETS_CRM' , 'ActiveCampaign' );
			require_once( MBR_TICKETS_ABSPATH . 'includes/class.activecampaign.php' );
		ELSEIF ( defined( 'I4WVERSION' ) ) :
			$this->define( 'MBR_TICKETS_CRM' , 'Infusionsoft' );
			require_once( MBR_TICKETS_ABSPATH . 'includes/class.infusionsoft.php' );
		ELSE:
			return;
		ENDIF;

		$this->ticket_post_statuses = [
			'mbr_ticket_open'     => [ 'label_overview' => __( 'Open',     'mbr-tickets' ), 'label_regular' => __( 'Open',     'mbr-tickets' ) ],
			'mbr_ticket_answered' => [ 'label_overview' => __( 'Answered', 'mbr-tickets' ), 'label_regular' => __( 'Answered', 'mbr-tickets' ) ],
			'mbr_ticket_closed'   => [ 'label_overview' => __( 'Closed',   'mbr-tickets' ), 'label_regular' => __( 'Closed',   'mbr-tickets' ) ],
		];

		add_action( 'plugins_loaded', [ $this, 'define_constants' ],               1, 0 );
		add_action( 'init',           [ $this, 'register_ticket_post_type' ],      1, 0 );
		add_action( 'init',           [ $this, 'register_ticket_tag_taxonomy' ],   1, 0 );
		add_action( 'init',           [ $this, 'register_ticket_post_statuses' ],  1, 0 );

		IF ( is_admin() ) :
			require_once( MBR_TICKETS_ABSPATH . 'includes/class.admin.php' );
		ELSE:
			require_once( MBR_TICKETS_ABSPATH . 'includes/class.frontend.php' );
		ENDIF;
	}

	public function define_constants()
	{
		$this->define( 'MBR_TICKETS_VERSION', '0.0.1' );
		$this->define( 'MBR_TICKETS_STATUSES', $this->ticket_post_statuses );
	}

	public function register_ticket_post_type()
	{
		$args = [
			'labels'                 => [
				'name'                 => __( 'Tickets', 'mbr-tickets' )
				,'singular_name'       => __( 'Ticket', 'mbr-tickets' )
				,'add_new_item'        => __( 'Add new ticket', 'mbr-tickets' )
				,'edit_item'           => __( 'Edit ticket', 'mbr-tickets' )
				,'new_item'            => __( 'New ticket', 'mbr-tickets' )
				,'view'                => FALSE
				,'view_item'           => __( 'View ticket', 'mbr-tickets' )
				,'search_items'        => __( 'Search tickets', 'mbr-tickets' )
				,'not_found'           => __( 'No matching tickets found', 'mbr-tickets' )
				,'not_found_in_trash'  => __( 'No tickets found in trash', 'mbr-tickets' )
			]
			,'description'           => __( 'Tickets', 'mbr-tickets' )
			,'public'                => FALSE
			,'exclude_from_search'   => TRUE
			,'publicly_queryable'    => FALSE
			,'show_ui'               => TRUE
			,'show_in_nav_menus'     => FALSE
			,'show_in_menu'          => FALSE
			,'show_in_admin_bar'     => FALSE
			,'menu_icon'             => ''
			,'capability_type'       => 'post'
			,'capabilities'          => [ 'create_posts' => 'do_not_allow' ]
			,'map_meta_cap'          => TRUE
			,'can_export'            => TRUE
			,'supports'              => ['']
		];
		register_post_type( 'mbr-ticket', $args );
	}

	public function register_ticket_tag_taxonomy()
	{
		$labels = [
			'name'                       => _x( 'Tags', 'taxonomy general name' ),
			'singular_name'              => _x( 'Tag', 'taxonomy singular name' ),
			'search_items'               => __( 'Search Tags' ),
			'popular_items'              => __( 'Popular Tags' ),
			'all_items'                  => __( 'All Tags' ),
			'parent_item'                => NULL,
			'parent_item_colon'          => NULL,
			'edit_item'                  => __( 'Edit Tag' ),
			'update_item'                => __( 'Update Tag' ),
			'add_new_item'               => __( 'Add New Tag' ),
			'new_item_name'              => __( 'New Tag Name' ),
			'separate_items_with_commas' => __( 'Separate tags with commas' ),
			'add_or_remove_items'        => __( 'Add or remove tags' ),
			'choose_from_most_used'      => __( 'Choose from the most used tags' ),
			'menu_name'                  => __( 'Tags' ),
		];

		register_taxonomy( 'mbr_ticket_tag', 'mbr-ticket', [
			'hierarchical'          => FALSE,
			'labels'                => $labels,
			'show_ui'               => TRUE,
			'update_count_callback' => '_update_generic_term_count',
			'query_var'             => TRUE,
			'rewrite'               => [ 'slug' => 'mbr-ticket-tag' ],
		] );
	}

	public function register_ticket_post_statuses()
	{
		FOREACH ( MBR_TICKETS_STATUSES AS $post_status => $arr_post_status ) :
			register_post_status( $post_status,
				[
					'label'                     => $arr_post_status['label_overview'],
					'public'                    => TRUE,
					'exclude_from_search'       => FALSE,
					'show_in_admin_status_list' => TRUE,
					'label_count'               => _n_noop( $arr_post_status['label_overview'] . ' <span class="count">(%s)</span>', $arr_post_status['label_overview'] . ' <span class="count">(%s)</span>' ),
				]
			);
		ENDFOREACH;
	}

}

add_action( 'plugins_loaded', 'mbr_tickets::getinstance', 0 );
