<?php
trait TRAIT__activesdk
{
	public $ac_url_base;
	public $ac_api_url;
	public $ac_api_key;
	public $ac_api_debug    = false;
	public $ac_api_output   = "json";
	public $api_success_msg = 'Success: Something is returned';
	public $api_failure_msg = 'Failed: Nothing is returned';

	function init_hooks_TRAIT__activesdk()
	{
		$this->api_init();
	}

	function api_init()
	{
		global $MBR_OPTIONS;

		IF ($MBR_OPTIONS['api_key'] AND $MBR_OPTIONS['api_url']) :
			
			$this->ac_api_init = true;
		 
			$this->ac_api_url = rtrim($MBR_OPTIONS['api_url'], '/');
			$this->ac_api_url = "{$MBR_OPTIONS['api_url']}/admin/api.php?api_key={$MBR_OPTIONS['api_key']}&api_output={$this->ac_api_output}";

			$this->version2_methods = [
				'tracking/site/list'        => ['track/site',  'GET',    'tracking_site_list']
			 ,'tracking/site/status'      => ['track/site',  'POST',   'tracking_site_status']
			 ,'tracking/whitelist'        => ['track/site',  'PUT',    'tracking_whitelist']
			 ,'tracking/whitelist/remove' => ['track/site',  'DELETE', 'tracking_whitelist']
			 ,'tracking/event/list'       => ['track/event', 'GET',    'tracking_event_list']
			 ,'tracking/event/status'     => ['track/event', 'POST',   'tracking_event_status']
			 ,'tracking/event/remove'     => ['track/event', 'DELETE', 'tracking_event_remove']
			];

			return true;
		ENDIF;
		return false;
	}

	function api_check_account()
	{
		$this->api_init();
		return $this->api("account_view");
	}

	function api_credentials_test()
	{
		$r = $this->api("user_me");
		return (is_object($r) AND (int)$r->result_code);
	}

	









	function sync_automations_tags_lists($array)
	{
		extract($array);

		$sync       = !empty($contact_data);
		$contact_id = false;

		IF (empty($contact_data['email']) OR !is_email($contact_data['email'])) :
			IF (empty($this->CONTACTID) OR empty($this->SESSION['email'])) :
				return FALSE;
			ENDIF;
			$contact_data['email'] = $this->SESSION['email'];
		ENDIF;

		IF (isset($subscribe_lists) AND is_array($subscribe_lists)) :
			FOREACH ($subscribe_lists AS $k => $list) :
				IF (is_numeric($list)) :
					$sync = true;
					$contact_data["p[{$list}]"]      = $list;
					$contact_data["status[{$list}]"] = 1;
				ENDIF;
			ENDFOREACH;
		ENDIF;

		IF (!empty($unsubscribe_lists) AND is_array($unsubscribe_lists)) :
			FOREACH ($unsubscribe_lists AS $k => $list) :
				IF (is_numeric($list)) :
					$sync = true;
					$contact_data["p[{$list}]"]      = $list;
					$contact_data["status[{$list}]"] = 2;
				ENDIF;
			ENDFOREACH;
		ENDIF;

		IF ($sync) :
			$contact_sync = $this->api('contact_sync', $contact_data);
			IF (!empty($contact_sync->subscriber_id)) :
				$contact_id = $contact_sync->subscriber_id;
			ENDIF;
		ENDIF;

		IF (empty($assign_tags) AND empty($remove_tags) AND empty($assign_automations) AND empty($remove_automations)) :
			return $contact_id;
		ENDIF;

		IF (empty($contact_id)) :
			IF (!empty($this->CONTACTID)) :
				$contact_id = $this->CONTACTID;
			ELSE :
				return FALSE;
			ENDIF;
		ENDIF;

		IF (empty($contact_id)) :
			return FALSE;
		ENDIF;

		IF (!empty($remove_tags)) :
			$this->api('contact_tag_remove',  ['id' => $contact_id, 'tags' => explode(',', $this->normalize_tag_names($remove_tags))]);
		ENDIF;

		IF (!empty($assign_tags)) :
			$this->api('contact_tag_add',     ['id' => $contact_id, 'tags' => explode(',', $this->normalize_tag_names($assign_tags))]);
		ENDIF;

		IF (!empty($remove_automations) AND is_array($remove_automations)) :
			FOREACH ($remove_automations AS $key => $automation_id) :
				IF (is_numeric($automation_id)) :
					$this->api("automation_contact_remove", ['contact_id' => $contact_id, 'automation' => $automation_id]);
				ENDIF;
			ENDFOREACH;
		ENDIF;

		IF (!empty($assign_automations)) :
			IF (is_array($assign_automations)) :
				$assign_automations = implode(',', $assign_automations);
			ENDIF;
			$this->api("automation_contact_add",      ['contact_id' => $contact_id, 'automation' => $assign_automations]);
		ENDIF;

		return $contact_id;
	}

	function api($path, $post_data = [])
	{
		global $MBR_OPTIONS;

		
		IF (!isset($this->ac_api_init) AND !$this->api_init()) :
			return;
		ENDIF;

		
		 $path_info = explode('?', $path);
		$method = !empty($path_info[0]) ? $path_info[0] : false;
		$params = !empty($path_info[1]) ? $path_info[1] : false;

		IF (array_key_exists($method, $this->version2_methods)) :
			list($method, $http_method, $params) = $this->version2_methods[$method];
			$url = $MBR_OPTIONS['api_url'] . '/api/2/' . $method . "?api_key={$MBR_OPTIONS['api_key']}&api_output={$this->ac_api_output}";
		ELSE :
			list($method, $http_method, $params) = [str_ireplace('/', '_', $method), 'POST', $params];
			$url = $this->ac_api_url . "&api_action={$method}" . (!empty($params) ? "&{$params}" : '');
		ENDIF;

		$api_time = microtime(true);
		$response = wp_remote_post($url, ['method' => $http_method, 'timeout' => 45, 'redirection' => 5, 'httpversion' => '1.0', 'blocking' => true, 'headers' => [], 'body' => $post_data, 'cookies' => [] ]);
		$api_time = microtime(true) - $api_time;

		IF (is_wp_error($response)) :
			$response = $response->errors;
		ELSE:
			$response = json_decode($response['body']);
			IF (empty($response) OR (!is_object($response) AND !is_array($response))) :
				
				$response                 = new stdClass;
				$response->result_code    =  $response->success  = 0;
				$response->result_output  = $this->ac_api_output;
				$response->http_code      = 412;
				$response->result_message =  $response->error    = "Failed: Nothing is returned ({$method})";
			ENDIF;
			IF (!empty($this->SESSION['id'])) :
				IF (in_array($method, ['contact_tag_add', 'contact_tag_remove', 'contact_edit', 'contact_sync'])) :
					IF ($response->result_code == 1) :
						IF ( (isset($post_data['id']) AND $this->SESSION['id'] == $post_data['id']) OR (isset($post_data['email']) AND $this->SESSION['email'] == $post_data['email']) ) :
							$this->fetch_contact_remote("contact_view?id={$this->SESSION['id']}", false, true);
						ENDIF;
					ENDIF;
				ENDIF;
			ENDIF;
		ENDIF;

		IF (APILOG_LIMIT > 0) :
			$post_data   = (!empty($post_data) ? gz_encode($post_data) : '');
			IF (!empty($params)) :
				parse_str($params, $params);
			ELSE:
				$params    = '';
			ENDIF;
			$params      = (!empty($params)    ? gz_encode($params)    : '');
			$response_db = (!empty($response)  ? gz_encode($response)  : '');
			IF (defined('DOING_AJAX') AND DOING_AJAX) :
				$method .= ' [ajax]';
			ELSEIF ((defined('DOING_CRON') AND DOING_CRON) OR isset($_GET['doing_wp_cron'])) :
				$method .= ' [cron]';
			ELSEIF (is_admin()) :
				$method .= ' [admin]';
			ENDIF;
			$log_email = empty($this->SESSION['email']) ? '' : $this->SESSION['email'];
			$this->wpdb->insert($this->wpdb->tbl_apilog, ['user' => $log_email, 'resptime' => $api_time, 'ipnr' => MBR_IPADDR, 'method' => $method, 'params' => $params, 'post_data' => $post_data, 'response' => $response_db]);
		ENDIF;
		return $response;
	}
}
