<?php
trait TRAIT__database
{
	public $arrTABLES                  = ['tbl_contact' => 'mbrContact', 'tbl_apilog' => 'mbrApiLog', 'tbl_errorlog' => 'mbrErrorLog'];
	public $site_check_tables_key      = 'site_check_tables';
	public $site_check_tables_expires  = 3840;

	function init_hooks_TRAIT__database()
	{
		add_action('mbr/database/init', [$this, 'init_database'], 10, 1);
	}

	function setup_wpdb_table_names()
	{
		global $wpdb;
		FOREACH ($this->arrTABLES AS $table => $table_name) :
			$wpdb->$table = $wpdb->prefix . $table_name;
		ENDFOREACH;
	}

	function init_database($force = false)
	{
		
		IF (is_admin()) :
			IF (!empty($_GET['mbr_updated']) OR version_compare(get_option('site_installed_version', '0.0.0'), MBRVERSION) <> 0) :
				$this->setup_db_tables($force);
				$this->dbDelta();
			ENDIF;
		ENDIF;
	}

	function setup_db_tables($force=false)
	{
		IF ((defined('DOING_AJAX') AND DOING_AJAX) OR (defined('DOING_CRON') AND DOING_CRON) OR isset($_GET['doing_wp_cron'])) :
			return;
		ENDIF;

		IF (!$force) :
			IF ($check_tables = get_transient($this->site_check_tables_key)) :
				return;
			ENDIF;
		ENDIF;

		IF (mbr_get_lock(__FUNCTION__)) :

			IF (!$tmpTABLES = $this->wpdb->get_results("SHOW TABLES LIKE '{$this->wpdb->prefix}mbr%'", ARRAY_N)) :
				$tmpTABLES = [];
			ENDIF;

			$arrTABLES = [];
			IF ($tmpTABLES) :
				FOREACH ($tmpTABLES as $k => $row) :
					$arrTABLES[$row[0]] = 1;
				ENDFOREACH;
			ENDIF;

			require_once(ABSPATH . '/wp-admin/includes/upgrade.php');

			$arrSQL = $this->get_internal_table_structures();

			FOREACH ($arrSQL as $nameTBL => $sqlTBL) :
				IF ($force OR !$arrTABLES OR !array_key_exists($nameTBL, $arrTABLES)) :
					dbDelta($sqlTBL);
				ENDIF;
			ENDFOREACH;

			set_transient($this->site_check_tables_key, '1', $this->site_check_tables_expires);
			mbr_release_lock(__FUNCTION__);

		ENDIF;
	}

	function get_internal_table_structures()
	{
		$arrSQL = [];

		$arrSQL[$this->wpdb->tbl_contact] = "CREATE TABLE {$this->wpdb->tbl_contact} (
	contact_id int(20) DEFAULT NULL,
	email varchar(255) COLLATE utf8_unicode_ci DEFAULT NULL,
	hash varchar(32) COLLATE utf8_unicode_ci DEFAULT NULL,
	data longtext COLLATE utf8_unicode_ci,
	last_active timestamp NULL DEFAULT NULL,
	last_updated timestamp NULL DEFAULT NULL,
	UNIQUE KEY UniqId (contact_id),
	UNIQUE KEY UniqEmail (email)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci";

		$arrSQL[$this->wpdb->tbl_errorlog] = "CREATE TABLE {$this->wpdb->tbl_errorlog} (
	id int(20) NOT NULL AUTO_INCREMENT,
	timestamp timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
	ipnr varchar(255) DEFAULT NULL,
	user varchar(255) DEFAULT NULL,
	log longtext,
	PRIMARY KEY (id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci";

		$arrSQL[$this->wpdb->tbl_apilog] = "CREATE TABLE {$this->wpdb->tbl_apilog} (
	id INT(20) NOT NULL AUTO_INCREMENT,
	timestamp TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
	resptime FLOAT(16,12) DEFAULT NULL,
	ipnr VARCHAR(255) DEFAULT NULL,
	user VARCHAR(255) DEFAULT NULL,
	method VARCHAR(255) DEFAULT NULL,
	params LONGTEXT,
	post_data LONGTEXT,
	response LONGTEXT,
	PRIMARY KEY (id)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8";

		return $arrSQL;
	}

	function get_WP_table_structures()
	{
		$arrSQL = [];

		$arrSQL[$this->wpdb->users] = "CREATE TABLE {$this->wpdb->users} (
	ID bigint(20) unsigned NOT NULL AUTO_INCREMENT,
	user_login varchar(255) NOT NULL DEFAULT '',
	user_pass varchar(255) NOT NULL DEFAULT '',
	user_nicename varchar(255) NOT NULL DEFAULT '',
	user_email varchar(255) NOT NULL DEFAULT '',
	user_url varchar(255) NOT NULL DEFAULT '',
	user_registered datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	user_activation_key varchar(255) NOT NULL DEFAULT '',
	user_status int(11) NOT NULL DEFAULT '0',
	display_name varchar(255) NOT NULL DEFAULT '',
	PRIMARY KEY  (ID),
	KEY user_login_key (user_login),
	KEY user_nicename (user_nicename)
	KEY user_email (user_email)
) DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci";

		return $arrSQL;
	}


	function dbDelta($force=false)
	{
		IF ((defined('DOING_AJAX') AND DOING_AJAX) OR (defined('DOING_CRON') AND DOING_CRON) OR isset($_GET['doing_wp_cron'])) :
			return;
		ENDIF;

		IF (!$force AND file_exists(dirname(__FILE__) . '/.noupdate')) :
			return;
		ENDIF;

		IF (mbr_get_lock(__FUNCTION__)) :

			
			@$this->maintenance_mode(true);
			ignore_user_abort(true);

			IF (get_option('site_database_version',0) !== MBRVERSIONDB) : 

				require_once(ABSPATH . '/wp-admin/includes/upgrade.php');

				IF ($arrSQL = $this->get_WP_table_structures()) :
					FOREACH ($arrSQL as $nameTBL => $sqlTBL) :
						@dbDelta($sqlTBL);
					ENDFOREACH;
				ENDIF;

				IF ($arrSQL = $this->get_internal_table_structures()) :
					FOREACH ($arrSQL as $nameTBL => $sqlTBL) :
						@dbDelta($sqlTBL);
					ENDFOREACH;
				ENDIF;

			ENDIF;

			update_option('site_installed_version', MBRVERSION);
			update_option('site_database_version',  MBRVERSIONDB);

			
			@$this->maintenance_mode(false);
			mbr_release_lock(__FUNCTION__);
		ELSE :
			mbr_wait_lock(__FUNCTION__, 500000, 60);
		ENDIF;
	}

}
